
#ifndef _INLINEFUNC_H_
#define _INLINEFUNC_H_

#include <math.h>
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifndef PI 
#define PI 3.14159265
#endif

#ifndef MAX
# define MAX(a,b) (((a) > (b)) ? (a) : (b))
#endif
#ifndef MIN
# define MIN(a,b) (((a) < (b)) ? (a) : (b))
#endif

#ifndef SK_ALMOSTZERO
#define SK_ALMOSTZERO 1e-6
#endif

#ifndef SK_INFINITE
#define SK_INFINITE 1000000
#endif

#ifndef ROUND_UNIT
#define ROUND_UNIT 0.1
#endif

void TakeABreath();

/////////////////////////////////////////////////////////////////////
// swap()
// swap two value
template<class T>
inline void swap(T &t1, T &t2)
{
	T t;
	t = t1;
	t1 = t2;
	t2 = t;
};


template<class T>
int dataIndexOfDecreasingArray(T *data, int *pIndex, int n)
{
	int i, j;
	for (i = 0; i < n; i++)
	{
		pIndex[i] = i;
	}
	for (i = 0; i < n-1; i++)
	{
		for (j = i+1; j < n; j++)
		{
			if (data[i] < data[j])
			{
				swap(data[i], data[j]);
				swap(pIndex[i], pIndex[j]);
			}
		}
	}
	return 0;
}


template<class T>
T sum(T *data, int n)
{
	T s = 0;
	for (int i = 0; i < n; i++)
		s += data[i];
	return s;
}

template<class T>
int sign(T x)
{
	if (x > 0)
		return 1;
	else if (x == 0)
		return 0;
	else
		return -1;
}

/////////////////////////////////////////////////////////////////////
// round()
// round a value to multiple of the unit
inline float round( float x, float unit)
{
	return (int)(x / unit+0.5) * unit;
}

///////////////////
// sort()
template<class T>
int sort( int nNum, T *p, int nFlag )
{// nFlag 0-increasing, 1-decreasing
	if( nNum <= 1 )
		return 1;
	if( nFlag == 0 )
	{// increasing
		for( int i = 0; i < nNum - 1; i++ )
		{
			for( int j = i + 1; j < nNum; j++ )
			{
				if( p[i] > p[j] )
				{
					T temp = p[i];
					p[i] = p[j];
					p[j] = temp;
				}
			}
		}
	}
	else if( nFlag == 1 )
	{// decreasing
		for( int i = 0; i < nNum - 1; i++ )
		{
			for( int j = i + 1; j < nNum; j++ )
			{
				if( p[i] < p[j] )
				{
					T temp = p[i];
					p[i] = p[j];
					p[j] = temp;
				}
			}
		}
	}
	return 0;
}//int sort( int nNum, int *p, int nFlag )


/////////////////////////////////////////////////////////////////////
// merging sort

///////////////////
// merge()
// Known r[l...m] and r[m+1...n] are sorted
// merge them into a sorted list r2[l...n]
// acending
template <class T>
inline void merge(T *r, int l, int m, int n, T *r2)
{
	int i = l;
	int j = m+1;
	int k = l-1;
	while (i <= m && j <= n)
	{
		k++;
		if (r[i] <= r[j])
		{
			r2[k] = r[i];
			i++;
		}
		else
		{
			r2[k] = r[j];
			j++;
		}
	}
	// copy left-over in r[l...m] or r[m+1...n] into r2
	if (i <= m)
	{
		for (int ii = i+1; ii <= m; ii++)
			r2[k++] = r[ii];
	}
	if (j <= n)
	{
		for (int ii = j+1; ii <= n; ii++)
			r2[k++] = r[ii];
	}
}
///////////////////
// mergesort()
// sort the list r[s...t]
template <class T>
inline void mergesort(T *r, T *r1, T *r2, int s, int t)
{
	if (s == t)
		r1[s] = r[s];
	else
	{
		mergesort(r, r2, s, (s+t)/2,);
		mergesort(r, r2, (s+t)/2+1, t,);
		merge(r2, s, (s+t)/2, t, r1);
	}
}


/////////////////////////////////////////////////////////////////////
// quick sort, acending

///////////////////
// qkpass()
// one pass for the list r[s...t],
// after this passing, get i (s<=i<=t)
// if s < i then all elements in r[s...i-1] are less or equal to r[i] (acending)
// if i < t then all elements in r[i+1...t] are larger or equal r[i]
template <class T>
inline void qkpass(T *r, int s, int t, int &i)
{
	i = s;
	int j =t;
	T x = r[i];
	while (i < j)
	{
		while (i < j && r[j] >= x)
			j--;
		swap(r[i], r[j]);
		while (i < j && r[i] <= x)
			i++;
		swap(r[i], r[j]);
	}
}
///////////////////
// qksort() - acending
template <class T>
inline void qksort(T *r, int s, int t)
{
	int k;
	if (s < t)
	{
		qkpass(r, s, t, k);
		qksort(r, s, k-1);
		qksort(r, k+1, t);
	}
}

/////////////////////////////////////////////////////////////////////
// combination

///////////////////
// combination()
// return the combination C(n, k)
inline int combination(int n, int k)
{
	if (n <= 0)
		return 0;
	if (n < k)
//		return 0;
		return 1;

	int s = 1;
	int s1 = 1;
	int i;
	int mm = MIN(n, k);

	for (i = 0; i < mm; i++)
	{
		s *= (n-i);
		s1 *= (i+1);
	}
	return s/s1;
};

///////////////////
// recursiveCOrder()
// recursively retrieve one order with index m from the combination C(n, k)
inline int recursiveCOrder(int n, int k, int &s, int m, int *pnOrder)
{
	if ( s-1 == m)
		return s;
	if (k <= 0)
	{
		s++;
		return s;
	}
	if (n <= k)
	{
		for (int i = 0; i < k; i++)
			pnOrder[i] = i;
		s++;
		return s;
	}
	else
	{
		recursiveCOrder(n-1, k, s, m, pnOrder);
		if ( s-1 == m)
			return s;
		recursiveCOrder(n-1, k-1, s, m, pnOrder);
		pnOrder[k-1] = n-1;
		return s;
	}
}

///////////////////
// combinationOrder()
// compute the index m from the combination C(n, k)
// retrieve the order
inline int combinationOrder(int n, int k, int m, int *pnOrder)
{
	if (n < k)
	{
		assert(m < 1);
		for (int i = 0; i < n; i++)
			pnOrder[i] = i;
//		return 0;
		return 1;
	}
	int s = 0;
	recursiveCOrder(n, k, s, m, pnOrder);
	return k;
};

///////////////////
// recursiveCOrder()
// recursively retrieve all of the order from the combination C(n, k)
inline int recursiveCOrder(int n, int k, int &s, int **ppnOrder)
{
	if (k <= 0)
	{
		s++;
		return s;
	}
	if (n <= k)
	{
		for (int i = 0; i < k; i++)
			ppnOrder[s][i] = i;
		s++;
		return s;
	}
	else
	{
		recursiveCOrder(n-1, k, s, ppnOrder);
		int s0 = s;
		recursiveCOrder(n-1, k-1, s, ppnOrder);
		for (int j = s0; j < s; j++)
			ppnOrder[j][k-1] = n-1;
		return s;
	}
}
///////////////////
// combinationOrder()
// retrieve all of the order from the combination C(n, k)
inline int combinationOrder(int n, int k, int **ppnOrder)
{
	if (n < k)
	{
		for (int i = 0; i < n; i++)
			ppnOrder[0][i] = i;
		return 1;
	}
	int s = 0;
	recursiveCOrder(n, k, s, ppnOrder);
	return s;
};


/////////////////////////////////////////////////////////////////////
// permutation()

///////////////////
// permutation()
// return the number in permutation P(n, k)
inline int permutation(int n, int k)
{
	int s = 1;
	int i;
	int mm = MIN(n, k);

	for (i = 0; i < mm; i++)
		s *= (n-i);
	return s;
};

///////////////////
// permutationOrder()
// compute the index m from the permutation P(n, k)
// retrieve the order
inline int permutationOrder(int n, int k, int m, int *pnOrder)
{
	int r;
	int i, j;
	int min = MIN(n, k);
	int s;

	int pnTmp[10000];
	for (i = 0; i < n; i++)
		pnTmp[i] = i;

	for (i = 0; i < min; i++)
	{
		s = 1;
		for (j = i+1; j < min; j++)
			s *= (n-j);
		r = m / s;
		pnOrder[i] = pnTmp[r];
		m -= r * s;
		//pnTmp[r] is used, remove it from possible chices
		for (j = r; j < n; j++)
			pnTmp[j] = pnTmp[j+1];
	}
	return k;
};

///////////////////
// insertOneElement2Array()
// insert "nValue" to "pnArray" at "nPos"
// nDim - orginal dimension
// return nDim+1, new dimension
inline int insertOneElement2Array(int *pnArray, int nDim, int nValue, int nPos)
{
	for (int i = nDim; i > nPos; i--)
		pnArray[i] = pnArray[i-1];
	pnArray[nPos] = nValue;
	return nDim+1;
}
///////////////////
// permutationOrder()
// retrieve the order for all possible permutation P(n, k)
// ppnOrder should be allocated before hand, P(n,k)*k
// return P(n,k)
// idea:
//	P(n,k)=P(n-1,k)+k*P(n-1,k-1)
#define MAX_N 256
#define MAX_PRMT_NUM 1024
inline int permutationOrder(int n, int k, int ppnOrder[MAX_PRMT_NUM][MAX_N])
{
	if (n < k)//no permuation
		return 0;

	if ( k == 1)
	{// k=1, then n choices
		for (int j = 0; j < n; j++)
			ppnOrder[j][0] =  j;
		return n;
	}

	int p1 = permutationOrder(n-1, k, ppnOrder);//P(n-1,k)
	int p2 = permutationOrder(n-1, k-1, ppnOrder+p1);//P(n-1,k-1)
	// multiple p2 k times
	for (int i = k-1; i >=0; i--)
	{//for each group (totally k)
		for (int p = 0; p < p2; p++)
		{//for each possible order (totally p2)
			for (int j = 0; j < k-1; j++)
			{//for each element (totally k-1)
				ppnOrder[i*p2+p+p1][j] = ppnOrder[p+p1][j];
			}
			insertOneElement2Array(ppnOrder[i*p2+p+p1], k-1, n-1, i);//(int *pnArray, int nDim, int nValue, int nPos)
		}
	}
	return p1+p2*k;
};


/////////////////////////////////////////////////////////////////////
// save data array to file

///////////////////
// save array
// destroy orginal file
template<class T>
inline int saveArray(T* pData, int nDim, const char *name)
{
	FILE  *f=fopen(name, "w");
	if (f == 0)
		return 1;
	for(int n = 0; n < nDim; n++)
		fprintf(f, "%10.5f  ", (float)(pData[n]));
	fclose(f);
	return 0;
}
///////////////////
// appendArray()
// keep orginal file and append the data to the end of the file
template<class T>
inline int appendArray(T* pData, int nDim, char *name)
{
	FILE  *f=fopen(name, "a+");
	if (f == 0)
		return 1;
	fprintf(f, "\n");
	for(int n = 0; n < nDim; n++)
		fprintf(f, "%.6f  ",(float)(pData[n]));
	fclose(f);
	return 0;
}


/////////////////////////////////////////////////////////////////////
// findLargestValue()
// find largest value in an array
template<class T>
inline T findLargestValue(T *pArray, int nDim, int &nMax)
{
	if (nDim <= 0)
		return 0;
	T tMax = pArray[0];
	nMax = 0;
	for (int i = 1; i < nDim; i++)
	{
		if (tMax < pArray[i])
		{
			tMax = pArray[i];
			nMax = i;
		}
	}
	return tMax;
};

///////////////////
// findSmallestValue()
// find smallest value in an array
template<class T>
inline T findSmallestValue(T *pArray, int nDim, int &nMin)
{
	if (nDim <= 0)
		return 0;
	T tMin = pArray[0];
	nMin = 0;
	for (int i = 1; i < nDim; i++)
	{
		if (tMin > pArray[i])
		{
			tMin = pArray[i];
			nMin = i;
		}
	}
	return tMin;
};

/////////////////////////////////////////////////////////////////////
// insideImage()
// check if one point is inside the image
// if not, change the point due to the condition
inline int insideImage(int &x, int nWidth, int cond=2)
// cond: 0-repeat image, 1-mirror image on boundary, 2-hard boundary
{
	if (x >=0 && x < nWidth)
		return 1;//change nothing, already inside
	if (x < 0)
	{
		switch (cond)
		{
		case 0://repeat image
			x = nWidth + x;
			break;
		case 1://mirror image on boundary
			x = -x;
			break;
		case 2://hard boundary
			x = 0;
			break;
		default:
			x = nWidth + x;
			break;
		}
	}
	else if (x >= nWidth)
	{
		switch (cond)
		{
		case 0://repeat image
			x = x%nWidth;
			break;
		case 1://mirror image on boundary
			x = nWidth*2 - x - 1;
			break;
		case 2://hard boundary
			x = nWidth - 1;
			break;
		default:
			x = x - nWidth;
			break;
		}
	}
	return x;
};
/*
inline 	double distanceBetweenPointsInAImage( int row1, int col1, int row2, int col2, int nWidth, int nHeight )
{//inline to save time
	int nDRow = min( abs(row1-row2), nHeight - abs(row1-row2) );
	int nDCol = min( abs(col1-col2), nWidth - abs(col1-col2) );
	return sqrt(nDRow*nDRow + nDCol*nDCol);
}
*/

/////////////////////////////////////////////////////////////////////
// distanceBetweenPoints()
// Euler Distance
inline 	double distanceBetweenPoints( int row1, int col1, int row2, int col2)
{//inline to save time
	int nDRow = row1 - row2;
	int nDCol = col1 - col2;
	return sqrt(double(nDRow*nDRow + nDCol*nDCol));
}

/////////////////////////////////////////////////////////////////////
// directionOfTwoPointsInAImage()
// p1->p2
// return 0~2PI
inline double directionOfTwoPoints(int row1, int col1, int row2, int col2)
{//0 ~2*PI
	int nDRow = row2 - row1;
	int nDCol = col2 - col1;
	double dAngle = atan2( double(nDRow), double(nDCol) );//-PI~PI
	if (dAngle < 0)
		dAngle += 2*PI;//0~2*PI
	return dAngle;
}


/////////////////////////////////////////////////////////////////////
// relativeDirection()
// dDir: 0~2*PI
// dOrt: 0~PI
// return 0~2PI
inline double relativeDirection(double dDir, double dOrt)
{//dDir 0~2*PI, dOrt 0~PI
	double d = dDir - dOrt;//-PI~2PI
	if (d < 0)
		d += 2*PI;
	return d;//0~2PI
}

/////////////////////////////////////////////////////////////////////
// rotate()
// dAngle: radian, not degree
// ^ y
// |    /
// |  /
// |/  a
// ----------> x
// a>0 means CCW
inline void rotate(double &x, double &y, double dAngle)
{
	double dCos = cos(dAngle);
	double dSin = sin(dAngle);
	double dNewX, dNewY;
	dNewX = x * dCos - y * dSin;
	dNewY = x * dSin + y * dCos;
	x = dNewX;
	y = dNewY;
}
/////////////////////////////////////////////////////////////////////
// rotate4LeftTopOrigin()
// dAngle: radian, not degree
// ----------> x
// | \ a  
// |   \
// |     \
// v y
// a>0 means CW
inline void rotate4LeftTopOrigin(double &x, double &y, double dAngle)
{//take exactly the formulate as rotate
	double dCos = cos(dAngle);
	double dSin = sin(dAngle);
	double dNewX, dNewY;
	dNewX = x * dCos - y * dSin;
	dNewY = x * dSin + y * dCos;
	x = dNewX;
	y = dNewY;
}

/////////////////////////////////////////////////////////////////////
// rotateInteger()
//	to speed-up, we use static variables to save call results
// dAngle: radian, not degree
// ^ y
// |    /
// |  /
// |/  a
// ----------> x
// a>0 means CCW
inline void rotateInteger(double &x, double &y, double dAngle)
{
#define max_xy 256
#define max_ang 72
	static double two_pi = 2*PI;
	static double ang_unit = (two_pi/max_ang);
	static int half_xy = 128;
	static double pppdX[max_xy][max_xy][max_ang];
	static double pppdY[max_xy][max_xy][max_ang];
	static bool computed[max_xy][max_xy][max_ang];
	static bool visited = false;

	int i, j, k;
	if (!visited)
	{
		for (i = 0; i < max_xy; i++)
		{
			for (j = 0; j < max_xy; j++)
			{
				for (k = 0; k < max_ang; k++)
					computed[i][j][k] = false;
			}
		}
		visited = true;
	}

	if(x > 0) 
		i = (int)(x+0.5) + half_xy;
	else
		i = (int)(x-0.5) + half_xy;

	if(y > 0) 
		j = (int)(y+0.5) + half_xy;
	else
		j = (int)(y-0.5) + half_xy;

	while (dAngle >= two_pi)
		dAngle -= two_pi;
	while (dAngle < 0)
		dAngle += two_pi;

	k = (int)(dAngle / ang_unit +0.5);

	assert(i >= 0 && j >= 0 && k >= 0 && i < max_xy && j < max_xy && k < max_ang);

	if (computed[i][j][k])
	{
		x = pppdX[i][j][k];
		y = pppdY[i][j][k];
	}
	else
	{
		double dCos = cos(dAngle);
		double dSin = sin(dAngle);
		double dNewX, dNewY;
		dNewX = x * dCos - y * dSin;
		dNewY = x * dSin + y * dCos;
		x = dNewX;
		y = dNewY;
		pppdX[i][j][k] = x;
		pppdY[i][j][k] = y;
		computed[i][j][k] = true;
	}
}

/////////////////////////////////////////////////////////////////////
// rasterize_line_segment()
// rasterize line segment of (x0,y0)->(x1,y1) to pnX, pnY
// return the num of points 
inline int rasterize_line_segment(const int x0, const int y0, const int x1, const int y1, int *pnX, int *pnY)
{// close inteval [x0, x1], [y0, y1]
  int x, y;
  int n = 0;

  int xStart, xEnd, yStart, yEnd;
  int dx = abs(x0 - x1);
  int dy = abs(y0 - y1);
  if (x0 < x1)
  {
	  xStart = x0;
	  yStart = y0;
	  xEnd = x1;
	  yEnd = y1;
  }
  else
  {
	  xStart = x1;
	  yStart = y1;
	  xEnd = x0;
	  yEnd = y0;
  }
  int mSign = (x1 - x0) * (y1 - y0 );//slope
  // if vertical lines
  if (dx == 0)
  {
	  if (yStart < yEnd)
		  for (y = yStart; y <= yEnd; y++)//close interval
		  {
			  pnX[n] = x0;
			  pnY[n] = y;
			  n++;
		  }
	  else
		  for (y = yStart; y >= yEnd; y--)//close interval
		  {
			  pnX[n] = x0;
			  pnY[n] = y;
			  n++;
		  }
	  return n;
  }

  // if horizontal lines
  if (dy == 0)
  {
	  for (x = xStart; x <= xEnd; x++)//close interval
	  {
		  pnX[n] = x;
		  pnY[n] = y0;
		  n++;
	  }
	  return n;
  }
  // if diagonal lines
  if (dx == dy)
  {
	  if (mSign > 0)// positive slope
	  {
		  for ( x = xStart, y = yStart; x <= xEnd; x++, y++)//close interval
		  {
			  pnX[n] = x;
			  pnY[n] = y;
			  n++;
		  }
	  }
	  else// negtive slope
	  {
		  for ( x = xStart, y = yStart; x <= xEnd; x++, y--)//close interval
		  {
			  pnX[n] = x;
			  pnY[n] = y;
			  n++;
		  }
	  }
	  return n;
  }


  if (dx > dy )
  {// 0 < abs(m) < 1
	  int p = 2 * dy - dx;
	  int twoDy = 2 * dy;
	  int twoDyDx = 2 * (dy - dx);

	  x = xStart;
	  y = yStart;
	  pnX[n] = x;
	  pnY[n] = y;
	  n++;
	  while (x < xEnd)
	  {
		  x++;
		  if (p < 0)
		  	p += twoDy;
		  else
		  {
			  if (mSign > 0)
			  	y++;
			  else
			  	y--;
			  p += twoDyDx;
		  }
		  pnX[n] = x;
		  pnY[n] = y;
		  n++;
	  }
   }
   else
   {// 1 < abs(m)
	  //swap dx, dy
	  swap(dx, dy);
	  // re-calculate start and end positions
	  if (y0 < y1)
	  {
		  xStart = y0;
		  yStart = x0;
		  xEnd = y1;
		  yEnd = x1;
	  }
	  else
	  {
		  xStart = y1;
		  yStart = x1;
		  xEnd = y0;
		  yEnd = x0;
	  }

	  int p = 2 * dy - dx;
	  int twoDy = 2 * dy;
	  int twoDyDx = 2 * (dy - dx);

	  x = xStart;
	  y = yStart;
	  pnX[n] = y;
	  pnY[n] = x;//x, y are swapped
	  n++;
	  while (x < xEnd)
	  {
		  x++;
		  if (p < 0)
		  	p += twoDy;
		  else
		  {
			  if (mSign > 0)
			  	y++;
			  else
			  	y--;
			  p += twoDyDx;
		  }
		  pnX[n] = y;//x, y are swapped
		  pnY[n] = x;
		  n++;
	  }
	}
	return n;
};


template<class T>
int inverseArray(T *pData, int n)
{
	for (int i = 0; i < n/2; i++)
		swap(pData[i], pData[n-i-1]);
	return n;
}
/////////////////////////////////////////////////////////////////////
// rasterize_line_segment()
// rasterize line segment of (x0,y0)->(x1,y1) to pnX, pnY
// return the num of points 
inline int rasterize_line_segment_keeping_order(const int x0, const int y0, const int x1, const int y1, int *pnX, int *pnY)
{// close inteval [x0, x1], [y0, y1]
	int n = rasterize_line_segment(x0, y0, x1, y1, pnX, pnY);
	if (pnX[0] != x0 || pnY[0] != y0)
	{//inverse the points
		inverseArray(pnX, n);
		inverseArray(pnY, n);
	}
	return n;
}
//axis = 0: x, 1~y; prfl_sign = +1, -1
inline void raster_line_axis_profile_sign(const int x0, const int y0, const int x1, const int y1, 
										  int &axis, int &prfl_sign)
{
	int dx = x1 - x0;
	int dy = y1 - y0;
	if (abs(dx) >= abs(dy))
	{
		axis = 0;//along x
		if (dx >= 0)
		{
			prfl_sign = 1;
		}
		else
		{
			prfl_sign = -1;
		}
	}
	else 
	{
		axis = 1;//along y
		if (dy >= 0)
		{
			prfl_sign = -1;
		}
		else
		{
			prfl_sign = 1;
		}
	}
}

/////////////////////////////////////////////////////////////////////
// rasterize_line_segment()
// rasterize line segment of (x0,y0)->(x1,y1) to pnX, pnY
// return the num of points 
inline int rasterize_line_segment(const int x0, const int y0, const int x1, const int y1, int *pnX, int *pnY, int line_wd)
{// close inteval [x0, x1], [y0, y1]
  int x, y;
  int n = 0;

  int nHalfWd = line_wd / 2;
  int nWdStart = -nHalfWd;
  int nWdEnd;
  if (nHalfWd * 2 == line_wd)
	  nWdEnd = nHalfWd -1;
  else
	  nWdEnd = nHalfWd;
  int w;

  int xStart, xEnd, yStart, yEnd;
  int dx = abs(x0 - x1);
  int dy = abs(y0 - y1);
  if (x0 < x1)
  {
	  xStart = x0;
	  yStart = y0;
	  xEnd = x1;
	  yEnd = y1;
  }
  else
  {
	  xStart = x1;
	  yStart = y1;
	  xEnd = x0;
	  yEnd = y0;
  }
  int mSign = (x1 - x0) * (y1 - y0 );//slope
  // if vertical lines
  if (dx == 0)
  {
	  if (yStart < yEnd)
		  for (y = yStart; y <= yEnd; y++)//close interval
		  {
			  for (w = nWdStart; w <= nWdEnd; w++)
			  {
				  pnX[n] = x0+w;
				  pnY[n] = y;
				  n++;
			  }
		  }
	  else
		  for (y = yStart; y >= yEnd; y--)//close interval
		  {
			  for (w = nWdStart; w <= nWdEnd; w++)
			  {
				  pnX[n] = x0+w;
				  pnY[n] = y;
				  n++;
			  }
		  }
	  return n;
  }

  // if horizontal lines
  if (dy == 0)
  {
	  for (x = xStart; x <= xEnd; x++)//close interval
	  {
		  for (w = nWdStart; w <= nWdEnd; w++)
		  {
			  pnX[n] = x;
			  pnY[n] = y0+w;
			  n++;
		  }
	  }
	  return n;
  }
  // if diagonal lines
  if (dx == dy)
  {
	  if (mSign > 0)// positive slope
	  {
		  for ( x = xStart, y = yStart; x <= xEnd; x++, y++)//close interval
		  {
			  for (w = nWdStart; w <= nWdEnd; w++)
			  {
				  pnX[n] = x;
				  pnY[n] = y+w;
				  n++;
			  }
		  }
	  }
	  else// negtive slope
	  {
		  for ( x = xStart, y = yStart; x <= xEnd; x++, y--)//close interval
		  {
			  for (w = nWdStart; w <= nWdEnd; w++)
			  {
				  pnX[n] = x;
				  pnY[n] = y+w;
				  n++;
			  }
		  }
	  }
	  return n;
  }


  if (dx > dy )
  {// 0 < abs(m) < 1
	  int p = 2 * dy - dx;
	  int twoDy = 2 * dy;
	  int twoDyDx = 2 * (dy - dx);

	  x = xStart;
	  y = yStart;
	  for (w = nWdStart; w <= nWdEnd; w++)
	  {
		  pnX[n] = x;
		  pnY[n] = y+w;
		  n++;
	  }
	  while (x < xEnd)
	  {
		  x++;
		  if (p < 0)
		  	p += twoDy;
		  else
		  {
			  if (mSign > 0)
			  	y++;
			  else
			  	y--;
			  p += twoDyDx;
		  }
		  for (w = nWdStart; w <= nWdEnd; w++)
		  {
			  pnX[n] = x;
			  pnY[n] = y+w;
			  n++;
		  }
	  }
   }
   else
   {// 1 < abs(m)
	  //swap dx, dy
	  swap(dx, dy);
	  // re-calculate start and end positions
	  if (y0 < y1)
	  {
		  xStart = y0;
		  yStart = x0;
		  xEnd = y1;
		  yEnd = x1;
	  }
	  else
	  {
		  xStart = y1;
		  yStart = x1;
		  xEnd = y0;
		  yEnd = x0;
	  }

	  int p = 2 * dy - dx;
	  int twoDy = 2 * dy;
	  int twoDyDx = 2 * (dy - dx);

	  x = xStart;
	  y = yStart;
	  for (w = nWdStart; w <= nWdEnd; w++)
	  {
		  pnX[n] = y+w;
		  pnY[n] = x;//x, y are swapped
		  n++;
	  }
	  while (x < xEnd)
	  {
		  x++;
		  if (p < 0)
		  	p += twoDy;
		  else
		  {
			  if (mSign > 0)
			  	y++;
			  else
			  	y--;
			  p += twoDyDx;
		  }
		  for (w = nWdStart; w <= nWdEnd; w++)
		  {
			  pnX[n] = y+w;//x, y are swapped
			  pnY[n] = x;
			  n++;
		  }
	  }
	}
	return n;
};


/////////////////////////////////////////////////////////////////////
// rasterize_line_segment()
// rasterize line segment of (x0,y0)->(x1,y1) to pnX, pnY
// return the num of points 
inline int rasterize_line_segment_floating_point(const int x0, const int y0, const int x1, const int y1, double *pdX, double *pdY)
{// close inteval [x0, x1], [y0, y1]
  int n = 0;

  int x, y;
  double xx, yy;

  int xStart, xEnd, yStart, yEnd;
  int dx = abs(x0 - x1);
  int dy = abs(y0 - y1);
  if (x0 < x1)
  {
	  xStart = x0;
	  yStart = y0;
	  xEnd = x1;
	  yEnd = y1;
  }
  else
  {
	  xStart = x1;
	  yStart = y1;
	  xEnd = x0;
	  yEnd = y0;
  }
  int mSign = (x1 - x0) * (y1 - y0 );//slope
  // if vertical lines
  if (dx == 0)
  {
	  if (yStart < yEnd)
	  {
		  for (y = yStart; y <= yEnd; y++)//close interval
		  {
			  pdX[n] = x0;
			  pdY[n] = y;
			  n++;
		  }
	  }
	  else
	  {
		  for (y = yStart; y >= yEnd; y--)//close interval
		  {
			  pdX[n] = x0;
			  pdY[n] = y;
			  n++;
		  }
	  }
	  return n;
  }

  // if horizontal lines
  if (dy == 0)
  {
	  for (x = xStart; x <= xEnd; x++)//close interval
	  {
		  pdX[n] = x;
		  pdY[n] = y0;
		  n++;
	  }
	  return n;
  }
  // if diagonal lines
  if (dx == dy)
  {
	  if (mSign > 0)// positive slope
	  {
		  for ( x = xStart, y = yStart; x <= xEnd; x++, y++)//close interval
		  {
			  pdX[n] = x;
			  pdY[n] = y;
			  n++;
		  }
	  }
	  else// negtive slope
	  {
		  for ( x = xStart, y = yStart; x <= xEnd; x++, y--)//close interval
		  {
			  pdX[n] = x;
			  pdY[n] = y;
			  n++;
		  }
	  }
	  return n;
  }


	if (dx > dy )
	{// 0 < abs(m) < 1
	  for ( x = xStart; x <= xEnd; x++)//close interval
	  {
	  }
	}
	else
	{// 1 < abs(m)

	}
	assert(false);//not finish yet
	return n;
};

/////////////////////////////////////////////////////////////////////
// rasterize_line_segment()
// rasterize line segment of (x0,y0)->(x1,y1) to pnX, pnY
// return the num of points 
// last_dir: the width is along x(0) or y(1)
inline int rasterize_line_segment_consider_last_width_direction(const int x0, const int y0, const int x1, const int y1, int *pnX, int *pnY, int line_wd, int &last_dir)
{// close inteval [x0, x1], [y0, y1]
  int x, y;
  int n = 0;

  int nHalfWd = line_wd / 2;
  int nWdStart = -nHalfWd;
  int nWdEnd;
  if (nHalfWd * 2 == line_wd)
	  nWdEnd = nHalfWd -1;
  else
	  nWdEnd = nHalfWd;
  int w;

  int xStart, xEnd, yStart, yEnd;
  int dx = abs(x0 - x1);
  int dy = abs(y0 - y1);
  if (x0 < x1)
  {
	  xStart = x0;
	  yStart = y0;
	  xEnd = x1;
	  yEnd = y1;
  }
  else
  {
	  xStart = x1;
	  yStart = y1;
	  xEnd = x0;
	  yEnd = y0;
  }
  int mSign = (x1 - x0) * (y1 - y0 );//slope
  // if vertical lines
  if (dx == 0)
  {
	  if (yStart < yEnd)
		  for (y = yStart; y <= yEnd; y++)//close interval
		  {
			  for (w = nWdStart; w <= nWdEnd; w++)
			  {
				  pnX[n] = x0+w;
				  pnY[n] = y;
				  n++;
			  }
		  }
	  else
		  for (y = yStart; y >= yEnd; y--)//close interval
		  {
			  for (w = nWdStart; w <= nWdEnd; w++)
			  {
				  pnX[n] = x0+w;
				  pnY[n] = y;
				  n++;
			  }
		  }
	  last_dir = 0;//x, data is along x axis
	  return n;
  }

  // if horizontal lines
  if (dy == 0)
  {
	  for (x = xStart; x <= xEnd; x++)//close interval
	  {
		  for (w = nWdStart; w <= nWdEnd; w++)
		  {
			  pnX[n] = x;
			  pnY[n] = y0+w;
			  n++;
		  }
	  }
	  last_dir = 1;//y, data is along y axis
	  return n;
  }
  // if diagonal lines
  if (dx == dy)
  {
	  if (mSign > 0)// positive slope
	  {
		  for ( x = xStart, y = yStart; x <= xEnd; x++, y++)//close interval
		  {
			  for (w = nWdStart; w <= nWdEnd; w++)
			  {
				  if (last_dir == 1)//y axis data
				  {
					  pnX[n] = x;
					  pnY[n] = y+w;
				  }
				  else
				  {
					  pnX[n] = x+w;
					  pnY[n] = y;
				  }
				  n++;
			  }
		  }
	  }
	  else// negtive slope
	  {
		  for ( x = xStart, y = yStart; x <= xEnd; x++, y--)//close interval
		  {
			  for (w = nWdStart; w <= nWdEnd; w++)
			  {
				  if (last_dir == 1)//y axis data
				  {
					  pnX[n] = x;
					  pnY[n] = y+w;
				  }
				  else
				  {
					  pnX[n] = x+w;
					  pnY[n] = y;
				  }
				  n++;
			  }
		  }
	  }
	  return n;
  }


  if (dx > dy )
  {// 0 < abs(m) < 1

	  last_dir = 1;//data is along y

	  int p = 2 * dy - dx;
	  int twoDy = 2 * dy;
	  int twoDyDx = 2 * (dy - dx);

	  x = xStart;
	  y = yStart;
	  for (w = nWdStart; w <= nWdEnd; w++)
	  {
		  pnX[n] = x;
		  pnY[n] = y+w;
		  n++;
	  }
	  while (x < xEnd)
	  {
		  x++;
		  if (p < 0)
		  	p += twoDy;
		  else
		  {
			  if (mSign > 0)
			  	y++;
			  else
			  	y--;
			  p += twoDyDx;
		  }
		  for (w = nWdStart; w <= nWdEnd; w++)
		  {
			  pnX[n] = x;
			  pnY[n] = y+w;
			  n++;
		  }
	  }
   }
   else
   {// 1 < abs(m)
	  last_dir = 0;//data is along x

	  //swap dx, dy
	  swap(dx, dy);
	  // re-calculate start and end positions
	  if (y0 < y1)
	  {
		  xStart = y0;
		  yStart = x0;
		  xEnd = y1;
		  yEnd = x1;
	  }
	  else
	  {
		  xStart = y1;
		  yStart = x1;
		  xEnd = y0;
		  yEnd = x0;
	  }

	  int p = 2 * dy - dx;
	  int twoDy = 2 * dy;
	  int twoDyDx = 2 * (dy - dx);

	  x = xStart;
	  y = yStart;
	  for (w = nWdStart; w <= nWdEnd; w++)
	  {
		  pnX[n] = y+w;
		  pnY[n] = x;//x, y are swapped
		  n++;
	  }
	  while (x < xEnd)
	  {
		  x++;
		  if (p < 0)
		  	p += twoDy;
		  else
		  {
			  if (mSign > 0)
			  	y++;
			  else
			  	y--;
			  p += twoDyDx;
		  }
		  for (w = nWdStart; w <= nWdEnd; w++)
		  {
			  pnX[n] = y+w;//x, y are swapped
			  pnY[n] = x;
			  n++;
		  }
	  }
	}
	return n;
};
/////////////////////////////////////////////////////////////////////
// rasterize_pie
// rasterize a pie (not only a circle)
inline int rasterize_pie(const int cx, const int cy, const int radius, int *pnX, int *pnY)
{
  int x = 0;
  int y = radius;
  int p = 1 - radius;

  int n = 0;
  pnX[n] = cx + x;
  pnY[n] = cy + y;
  n++;
  if (radius <= 0)
	  return n;
  pnX[n] = cx + x;
  pnY[n] = cy - y;
  n++;
  for (int i = -radius; i <= radius; i++)
  {
	  pnX[n] = cx + i;
	  pnY[n] = cy;
	  n++;
  }
  while (x < y)
  {
	  x++;
	  if (p < 0)
	  	p += 2 * x + 1;
	  else
	  {
		  y--;
		  p += 2 * (x - y) + 1;
	  }
	  for (int i = -x; i <= x; i++)
	  {// first part
		  for (int j = -1; j <= 1; j+=2)
		  {//mirror
			  pnX[n] = cx + i;
			  pnY[n] = cy + y*j;
			  n++;
		  }
	  }
	  int xx = y;
	  int yy = x;//swap x, y
	  for (int i = -xx; i <= xx; i++)
	  {// second part
		  for (int j = -1; j <= 1; j+=2)
		  {//mirror
			  pnX[n] = cx + i;
			  pnY[n] = cy + yy*j;
			  n++;
		  }
	  }
  }
  return n;
};
/////////////////////////////////////////////////////////////////////
// rasterize_circle
inline int rasterize_circle(const int cx, const int cy, const int radius, int *pnX, int *pnY)
{
  int x = 0;
  int y = radius;
  int p = 1 - radius;

  int n = 0;
  pnX[n] = cx + x;
  pnY[n] = cy + y;
  n++;
  if (radius <= 0)
	  return n;
  pnX[n] = cx + x;
  pnY[n] = cy - y;
  n++;

  int xx = y;
  int yy = x;
  pnX[n] = cx + xx;
  pnY[n] = cy + yy;
  n++;
  pnX[n] = cx - xx;
  pnY[n] = cy + yy;
  n++;

  while (x < y)
  {
	  x++;
	  if (p < 0)
	  	p += 2 * x + 1;
	  else
	  {
		  y--;
		  p += 2 * (x - y) + 1;
	  }
	  for (int j = -1; j <= 1; j+=2)
	  {//mirror
		  for (int k = -1; k <= 1; k+=2)
		  {//mirror
			  pnX[n] = cx + x*j;
			  pnY[n] = cy + y*k;
			  n++;
		  }
	  }
	  int xx = y;
	  int yy = x;//swap x, y
	  for (int j = -1; j <= 1; j+=2)
	  {//mirror
		  for (int k = -1; k <= 1; k+=2)
		  {//mirror
			  pnX[n] = cx + xx*j;
			  pnY[n] = cy + yy*k;
			  n++;
		  }
	  }
  }
  return n;
};
/////////////////////////////////////////////////////////////////////
// rasterize_circle
inline int rasterize_circle_part(const int cx, const int cy, const int radius, int *pnX, int *pnY, double dOrtStart, double dOrtEnd)
{
	int fx, fy;//final x, y
	double dOrt;
  int x = 0;
  int y = radius;
  int p = 1 - radius;

  int n = 0;
  fx = cx + x;
  fy = cy + y;
  dOrt = atan2(double(fy), double(fx));
  if (dOrt >= dOrtStart && dOrt <= dOrtEnd)
  {
	  pnX[n] = fx;
	  pnY[n] = fy;
	  n++;
  }
  if (radius <= 0)
	  return n;
  fx = cx + x;
  fy = cy - y;
  dOrt = atan2(double(fy), double(fx));
  if (dOrt >= dOrtStart && dOrt <= dOrtEnd)
  {
	  pnX[n] = fx;
	  pnY[n] = fy;
	  n++;
  }
  while (x < y)
  {
	  x++;
	  if (p < 0)
	  	p += 2 * x + 1;
	  else
	  {
		  y--;
		  p += 2 * (x - y) + 1;
	  }
	  for (int j = -1; j <= 1; j+=2)
	  {//mirror
		  fx = cx + x;
		  fy = cy + y*j;
		  dOrt = atan2(double(fy), double(fx));
		  if (dOrt >= dOrtStart && dOrt <= dOrtEnd)
		  {
			  pnX[n] = fx;
			  pnY[n] = fy;
			  n++;
		  }
	  }
	  int xx = y;
	  int yy = x;//swap x, y
	  for (int j = -1; j <= 1; j+=2)
	  {//mirror
		  fx = cx + xx;
		  fy = cy + yy*j;
		  dOrt = atan2(double(fy), double(fx));
		  if (dOrt >= dOrtStart && dOrt <= dOrtEnd)
		  {
			  pnX[n] = fx;
			  pnY[n] = fy;
			  n++;
		  }
	  }
  }
  return n;
};

///////////////////
// changeSign()
//	change the sign of the data
//	it is needed when probabilty is proportional to energy, not inversely 
inline int changeSign(double *pd, int n)
{
	int i;
	for (i = 0; i < n; i++)
		pd[i] = -pd[i];
	return n;
}
template<class T>
inline int indexOfMin(T *data, int n)
{
	T dMin = 1e38;
	int nMin = -1;
	for (int i = 0; i < n; i++)
		if (dMin > data[i])
		{
			dMin = data[i];
			nMin = i;
		}
	assert(nMin >= 0 && nMin < n);
	return nMin;
}
inline double Min(double *data, int n)
{
	double dMin = 1e38;
	int nMin = -1;
	for (int i = 0; i < n; i++)
		if (dMin > data[i])
		{
			dMin = data[i];
			nMin = i;
		}
	assert(nMin >= 0 && nMin < n);
	return dMin;
}
inline double Max(double *data, int n)
{
	double dMax = -1e38;
	int nMax = -1;
	for (int i = 0; i < n; i++)
		if (dMax < data[i])
		{
			dMax = data[i];
			nMax = i;
		}
	assert(nMax >= 0 && nMax < n);
	return dMax;
}
inline double skMagicTempratureByWhichBiggestProbIsAsTwiceAsSecondBiggest(double *pdEnergy, int nTotalNum)
{
	double dMin = 1e38;
	int nMin = -1;
	for (int n = 0; n < nTotalNum; n++)
	{
		if (dMin > pdEnergy[n])
		{
			dMin = pdEnergy[n];
			nMin = n;
		}
	}

	double dMin2 = 1e38;
	for (int n = 0; n < nTotalNum; n++)
	{
		if (nMin == n)
			continue;
		if (dMin2 > pdEnergy[n])
			dMin2 = pdEnergy[n];
	}

	double dDelta = dMin2 - dMin;

	double dTemp = dDelta / log (2.0f);
	
	return dTemp;

}

/////////////////////////////////////////////////////////////////////
// changeEnergy2Probability()
// pdEnergy - pointer to the energies
// pdProb - probability after converting
// nTotalNum - total number
// dTemp - temperature
inline int	changeEnergy2Probability(double *pdEnergy, double *pdProb, int nTotalNum, double dTemp=1.0)
{
	//if only one data, obsolute energy and prob is considered
	if (nTotalNum == 1)
	{
		*pdProb = exp(-(*pdEnergy));
		return 0;
	}

	// more than one data, relative energy and prob is assumed
//find min
	double dMin = 1e38;
	for (int n = 0; n < nTotalNum; n++)
		if (dMin > pdEnergy[n])
			dMin = pdEnergy[n];
//sum     
	double sum=0;
	for (int n = 0; n < nTotalNum; n++)
	{
		pdProb[n] = exp(-(pdEnergy[n]-dMin)/dTemp);
		sum += pdProb[n];
	}
//normalize
	for (int n = 0; n < nTotalNum; n++)
	{
		if( sum != 0 )
			pdProb[n] /= sum;
		else
			pdProb[n] = 1.0/(nTotalNum);
	}
	return 0;
}

///////////////////
// samplingFromProbability()
// general sampling from a probability
// pdProb - pointer to the probability
// num - number, or called dimension
// *pdProbDrawn - the probability by which the sampling happens
// return sampled index
inline int samplingFromProbability( double *pdProb, int num, double *pdProbDrawn=0)
{
	double dRandom = rand()/(RAND_MAX+1.0);
	double dSum = 0;
	int i=0;
	for( i = 0; i < num; i++ )
		dSum += pdProb[i];
	if( dSum == 0 )
		return (int)(dRandom*num);
	double dAccumlate = 0;
	for( i = 0; i < num; i++ )
	{
		dAccumlate += pdProb[i] / dSum;
		if( dAccumlate >= dRandom )
			break;
	}
	if( i >= num )
	{
		i = num - 1;
		if (pdProbDrawn != 0)
			*pdProbDrawn = 1.0;
	}
	else
		if (pdProbDrawn != 0)
			*pdProbDrawn = pdProb[i] / dSum;
	return i;
}
///////////////////
// samplingFromCdf()
// general sampling from a comulative density function
// num - number, or called dimension
// return sampled index
inline int samplingFromCdf( double *pdCdf, int num)
{
	double dRandom = rand()/(RAND_MAX+1.0);
	int i;
	for( i = 0; i < num; i++ )
	{
		if( pdCdf[i] >= dRandom )
			break;
	}
	assert(i >= 0 && i < num);
	return i;
}

///////////////////
// oneDEntropy()
// pdProb - pointer to the 1D probability
// num - number, or called dimension
// return entropy
inline double oneDEntropy( double *pdProb, int num)
{
	double dEntropy = 0;
	for (int i = 0; i < num; i++)
	{
		if (pdProb[i] != 0)
			dEntropy += -pdProb[i] * log(pdProb[i]);
	}
	return dEntropy;
}


///////////////////
// metropolisSampling()
//	from given proposal and probability of old state and new state, 
//	decide move (return 1) or not (return 0) to new state by metropolis sampling
inline int metropolisSampling(double dProbOld, double dPrpsOld, double dProbNew, double dPrpsNew)
{
	if (dProbNew == 0)
	{
		return 0;
	}
	if (dProbOld == 0 || dPrpsNew == 0)
	{
		assert(false);//no zero prob or prps is allowed
		return 0;
	}
	double alpha = dProbNew*dPrpsOld/(dProbOld*dPrpsNew);
	if (alpha > 1.0)
		alpha = 1.0;
	double dRand = rand()/(RAND_MAX+1.0);
	if (alpha > dRand)
		return 1;//move to new state
	else
		return 0;//keep old state
}
///////////////////
// randomSelectMNumbersOutOfN()
//	pbSelected[i] is true is i is selected
inline int randomSelectMNumbersOutOfN(bool *pbSelected, int n, int m)
{
	bool bInverse = false;
	if (m > n/2)
	{
		m = n - m;
		bInverse = true;
	}
	int i, j;
	for (i = 0; i < n; i++)
		pbSelected[i] = false;
	int left = n;
	int nChoose, nNum;
	for (i = 0; i < m; i++)
	{
		nChoose = (int)(rand()/(RAND_MAX+1.0)*left);
		nNum = 0;
		for (j = 0; j < n; j++)
		{
			if (!pbSelected[j])
			{
				if (nChoose == nNum)
				{
					pbSelected[j] = true;
					left--;
					break;
				}//if reach
				nNum++;
			}//if still available
		}//search from the beginning
	}//for number needed
	if (bInverse)
		for (i = 0; i < n; i++)
			pbSelected[i] = !pbSelected[i];
	return m;
}

inline double angleDifference(double dAngle1, double dAngle2)
{
	double dDeltaAngle = dAngle1 - dAngle2;

	while (dDeltaAngle >= 2*PI)
		dDeltaAngle -= 2*PI;
	while (dDeltaAngle < 0)
		dDeltaAngle += 2*PI;
	if (dDeltaAngle >= PI)
		dDeltaAngle = 2*PI - dDeltaAngle;
	assert(dDeltaAngle >= 0 && dDeltaAngle <= PI);

	return dDeltaAngle;

}
// return the length of the vertical line
inline int verticalLine(int x0, int y0, int x1, int y1, //given line
						int &v_x0, int &v_y0, int &v_x1, int &v_y1, int l)//vertical line with length l)
{
	int dx = x1-x0;
	int dy = y1-y0;

	double dScale = (double)l / sqrt(double((abs(dx)+1)*(abs(dx)+1) + (abs(dy)+1)*(abs(dy)+1)));
	double v_dx = dy * dScale;
	double v_dy = -dx * dScale;

	if (v_dx > 0)
		v_x0 = (int)(v_dx/2+0.5);
	else
		v_x0 = (int)(v_dx/2-0.5);
	if (v_dy > 0)
		v_y0 = (int)(v_dy/2+0.5);
	else
		v_y0 = (int)(v_dy/2-0.5);

	v_x1 = -v_x0;
	v_y1 = -v_y0;
	
	return MAX(abs(v_x1)*2+1, abs(v_y1)*2+1);
}
// return the length of the vertical line
inline int verticalLine(double dOrt, //give line orientation
						int &v_x0, int &v_y0, int &v_x1, int &v_y1, int l)//vertical line with length l)
{
	double v_dx = -l/2;
	double v_dy = 0;

	::rotate(v_dx, v_dy, dOrt+PI/2);

	if (v_dx > 0)
		v_x0 = (int)(v_dx+0.5);
	else
		v_x0 = (int)(v_dx-0.5);
	if (v_dy > 0)
		v_y0 = (int)(v_dy+0.5);
	else
		v_y0 = (int)(v_dy-0.5);

	v_x1 = -v_x0;
	v_y1 = -v_y0;
	
	return MAX(abs(v_x1)*2+1, abs(v_y1)*2+1);
}

// return the length of the vertical line
inline int verticalLineApproximately(int x0, int y0, int x1, int y1, //given line
						int &v_x0, int &v_y0, int &v_x1, int &v_y1, int l)//vertical line with length l)
{
	int dx = x1 - x0;
	int dy = y1 - y0;
	if (abs(dx) >= abs(dy))
	{
		if (dx >= 0)
		{
			v_x0 = 0;
			v_y0 = -l/2;
		}
		else
		{
			v_x0 = 0;
			v_y0 = l/2;
		}
	}
	else 
	{
		if (dy >= 0)
		{
			v_x0 = l/2;
			v_y0 = 0;
		}
		else
		{
			v_x0 = -l/2;
			v_y0 = 0;
		}
	}
	v_x1 = -v_x0;
	v_y1 = -v_y0;

	return (abs(v_x0)+abs(v_y0))*2+1;
}
// return the length of the vertical line
inline int verticalLineApproximately(double dOrt, //orientation
						int &v_x0, int &v_y0, int &v_x1, int &v_y1, int l)//vertical line with length l)
{
	while (dOrt >= 2*PI)
		dOrt -= 2*PI;
	while (dOrt < 0)
		dOrt += 2*PI;

	//[0,2*PI) now
	if (dOrt >= 7*PI/4 || dOrt < PI/4)
	{
		v_x0 = 0;
		v_y0 = -l/2;
	}
	else if (dOrt >= PI/4 && dOrt < PI*3/4)
	{
		v_x0 = l/2;
		v_y0 = 0;
	}
	else if (dOrt >= PI*3/4 && dOrt < PI*5/4)
	{
		v_x0 = 0;
		v_y0 = l/2;
	}
	else
	{
		v_x0 = -l/2;
		v_y0 = 0;
	}

	v_x1 = -v_x0;
	v_y1 = -v_y0;

	return (abs(v_x0)+abs(v_y0))*2+1;
}

// order[0] will be the index of the largest value
template<class T>
inline int sortDecreaselyWithTrackingOrder(T *data, int nDim, int *order)
{
	int i, j;
	for (i = 0; i < nDim; i++)
	{
		order[i] = i;
	}

	for (i = 0; i < nDim-1; i++)
	{
		for (j = i+1; j < nDim; j++)
		{
			if (data[i] < data[j])
			{
				swap(data[i], data[j]);
				swap(order[i], order[j]);
			}
		}
	}
	return 0;
}

// order[0] will be the index of the smallest value
template<class T>
inline int sortIncreaselyWithTrackingOrder(T *data, int nDim, int *order)
{
	int i, j;
	for (i = 0; i < nDim; i++)
	{
		order[i] = i;
	}

	for (i = 0; i < nDim-1; i++)
	{
		for (j = i+1; j < nDim; j++)
		{
			if (data[i] > data[j])
			{
				swap(data[i], data[j]);
				swap(order[i], order[j]);
			}
		}
	}
	return 0;
}
//compute the average coding length with gaussian
inline double uniformCodingLength(double m)
{
	return log(m);
}

//compute the average coding length with gaussian
inline double gaussianCodingLength(double mu, double std)
{
	static double sqrt_2pi = sqrt(2*PI);
	return 0.5 + log(sqrt_2pi * std);
}
//compute the average coding length of gaussian mu1, std1 by another gaussian mu2, std2
inline double gaussianCodingLength(double mu1, double std1, double mu2, double std2)
{
	static double sqrt_2pi = sqrt(2*PI);
	return log(sqrt_2pi * std2) + (std1*std1 + (mu1-mu2)*(mu1-mu2))/(2*std2*std2);
}

inline int changeNumTo2Char(char * ch, int n)
{
	if (n < 10)
		sprintf(ch, "0%d", n);
	else
		sprintf(ch, "%d", n);
	return 0;
}
inline int changeNumTo3Char(char * ch, int n)
{
	if (n < 10)
		sprintf(ch, "00%d", n);
	else if (n < 100)
		sprintf(ch, "0%d", n);
	else
		sprintf(ch, "%d", n);
	return 0;
}
inline int changeNumTo4Char(char * ch, int n)
{
	if (n < 10)
		sprintf(ch, "000%d", n);
	else if (n < 100)
		sprintf(ch, "00%d", n);
	else if (n < 1000)
		sprintf(ch, "0%d", n);
	else
		sprintf(ch, "%d", n);
	return 0;
}

///////////////////
// findMeanValue()
// find mean value in an array
template<class T>
inline T findMeanValue(T *pArray, int nDim)
{
	if (nDim <= 0)
		return 0;
	T sum = 0;
	for (int i = 0; i < nDim; i++)
	{
		sum += pArray[i];
	}
	return sum/nDim;
};


///////////////////
// changeArray2ZeroMean()
// change the array to zero mean
//	return original mean
template<class T>
inline T changeArray2ZeroMean(T *pArray, int nDim)
{
	T mean = findMeanValue(pArray, nDim);
	for (int i = 0; i < nDim; i++)
	{
		pArray[i] -= mean;
	}
	return mean;
};

///////////////////
// compute linearity from a given set of x, y
//	return linearity 0~1.0
template<class T>
inline double computeLinearity(T *pnX, T *pnY, //x and y
							   int n, //number of point
							   double &dOrt,//orientation
							   double &a, double &b)//Y=a*X+b
{
	a = 0;
	b = 0;
	double dLinearity = 0.0;
	if (n <= 1)
	{
		dOrt = 0;
		dLinearity = 0.0;
		return dLinearity;
	}

	int dx, dy;
	if (n == 2)
	{
		dLinearity = 1.0;
		dx = pnX[1] - pnX[0];
		dy = pnY[1] - pnY[0];
		dOrt = atan2(double(dy), dx);
		if (dx != 0)
			a = dy/dx;
		else
			a = 1000000;
		b = pnY[0] - a*pnX[0];
		return dLinearity;
	}

	// expectation, (mean)
	double EX=0, EY=0;
	int i;
	for (i = 0; i < n; i++)
	{
		EX += pnX[i];
		EY += pnY[i];
	}
	EX /= n;
	EY /= n;
	// variance
	double DX=0, DY=0;
	for (i = 0; i < n; i++)
	{
		DX += (pnX[i] - EX) * (pnX[i] - EX);
		DY += (pnY[i] - EY) * (pnY[i] - EY);
	}
	DX /= n;
	DY /= n;
	// co-variance
	double CXY = 0;
	for (i = 0; i < n; i++)
	{
		CXY += (pnX[i] - EX) * (pnY[i] - EY);
	}
	CXY /= n;

	if (DX ==0 || DY == 0)
		dLinearity = 1.0;
	else
		dLinearity = CXY/sqrt(DX*DY);

	//Y= aX + b;
	if (DX != 0)
		a = CXY/DX;
	else
		a = 1000000;

	b = EY - a*EX;

	dx = pnX[n-1] - pnX[0];
	dy = pnY[n-1] - pnY[0];
	dOrt = atan2(double(dy), dx);

	return fabs(dLinearity);
		
}

template<class T>
int getMeanStd(T *data, int n, double &dMean, double &dStd)
{
	if (n < 1)
	{
		assert(false);
		return 0;
	}
	dMean = 0;
	for (int i = 0; i < n; i++)
	{
		dMean += data[i];
	}
	dMean /= n;
	dStd = 0;
	for (i = 0; i < n; i++)
	{
		dStd += (data[i]-dMean) * (data[i]-dMean);
	}
	dStd = sqrt(dStd/n);
	return 0;
}

inline void removeFileTypeInName(char *name)
{
	char *temp;
	temp = strrchr(name, '.');//find '.', return the pointer to it
	*temp = '\0';
}


inline double sqrtDistance(double x1, double y1, double x2, double y2)
{
	return sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
}
inline double lineLength(double x1, double y1, double x2, double y2)
{
	return sqrt( (fabs(x1-x2)+1.0)*(fabs(x1-x2)+1.0) 
		       + (fabs(y1-y2)+1.0)*(fabs(y1-y2)+1.0) );
}
///////////////////
// polygonFilling()
//	For given polygon vertices, filling the polygon region
//	The boundary is included
inline int polygonFilling(int m, //number of vertices
						  int *vx, int *vy, //vertex position
						  int *pnX, int *pnY )//the points in the region
{
	if( m <= 2 )// less than 2 edges can't form a polygon
		return 1;

	int nMinY = 1000000, nMaxY = -1;
	int nMinX = 1000000, nMaxX = -1;
	for (int i = 0; i < m; i++)
	{
		if (nMinY > vy[i])
			nMinY = vy[i];
		if (nMaxY < vy[i])
			nMaxY = vy[i];
		if (nMinX > vx[i])
			nMinX = vx[i];
		if (nMaxX < vx[i])
			nMaxX = vx[i];

	}
	int *pIntersectionPointsX = new int[m];
	int nIntersectionNum;
	int x;
	int p0_x, p0_y, p1_x, p1_y;
	int s = 0;//total points
	for( int y = nMinY; y <= nMaxY; y++ )
	{
		nIntersectionNum = 0;
		int n = 0;
		p0_x = vx[n];
		p0_y = vy[n];
		n++;
		for( int l = 0; l < m; l++ )
		{//get all intersections of X
			if (n == m)
				n = 0;
			p1_x = vx[n];
			p1_y = vy[n];
			n++;
			if( ( p0_y >= y && p1_y < y ) || ( p0_y < y && p1_y >= y ) )
			{// intersect
				x = (int)( (float)(p1_x-p0_x)*(y-p0_y)/(p1_y-p0_y) +p0_x);
				pIntersectionPointsX[nIntersectionNum] = x;
				nIntersectionNum++;
			}
			p0_x = p1_x;
			p0_y = p1_y;
		}
		sort( nIntersectionNum, pIntersectionPointsX, 0 );// 0-increasing, 1-decreasing
		for( int i = 0; i < nIntersectionNum; i = i+2 )
		{
			int nStart = MAX( 0, pIntersectionPointsX[i] );
			int nEnd = MIN( pIntersectionPointsX[i+1], nMaxX );
			for( int xx = nStart; xx <= nEnd; xx++ )
			{
				pnX[s] = xx;
				pnY[s] = y;
				s++;
			}
		}
	}
	delete[] pIntersectionPointsX;
	return s;

}

inline int skGetRotatedRectangleRegion(int x0, int y0, //rotate center
									double dOrt0,//the orientation
									double dLeft, double dRight, //left and right extention
									double dTop, double dBottom, //top and bottom extenstion
									int *pnX, int *pnY)//the points
{
	double x[4], y[4];//compute the four rotated corners
	x[0] = dLeft;	y[0] = dTop;
	x[1] = dRight;	y[1] = dTop;
	x[2] = dRight;	y[2] = dBottom;
	x[3] = dLeft;	y[3] = dBottom;

	int ix[4], iy[4];//integer
	int min_x = 1000000, min_y = 1000000;
	int max_x = -1000000, max_y = -1000000;

	for (int i = 0; i < 4; i++)
	{
		//rotate
		::rotate(x[i], y[i], dOrt0);
		ix[i] = (int)(x0 + x[i] + 0.5);
		iy[i] = (int)(y0 + y[i] + 0.5);
	}
	int n = polygonFilling(4, ix, iy, pnX, pnY);
	return n;

}

// right is as double high as left
// return the total number
inline int skGetRotatedTrapezoidleRegion(int x0, int y0, //rotate center
									double dOrt0,//the orientation
									double dLeft, double dRight, //left and right extention
									double dTop, double dBottom, //top and bottom extenstion
									int *pnX, int *pnY)//the points
{
	double x[4], y[4];//compute the four rotated corners
	x[0] = dLeft;	y[0] = dTop;
	x[1] = dRight;	y[1] = dTop*2;
	x[2] = dRight;	y[2] = dBottom*2;
	x[3] = dLeft;	y[3] = dBottom;

	int ix[4], iy[4];//integer
	int min_x = 1000000, min_y = 1000000;
	int max_x = -1000000, max_y = -1000000;

	for (int i = 0; i < 4; i++)
	{
		//rotate
		::rotate(x[i], y[i], dOrt0);
		ix[i] = (int)(x0 + x[i] + 0.5);
		iy[i] = (int)(y0 + y[i] + 0.5);
	}
	int n = polygonFilling(4, ix, iy, pnX, pnY);
	return n;

}


template<class T>
double skMinPt2CurveDistance(double x0, double y0, T *pX, T *pY, int n, int &nMinDisPt)
{
	nMinDisPt = -1;

	double dMinDis = 10000000;
	double dDis;
	for (int i = 0; i < n; i++)
	{
		dDis = sqrtDistance(x0, y0, pX[i], pY[i]);
		if (dDis < dMinDis)
		{
			dMinDis = dDis;
			nMinDisPt = i;
		}
	}
	assert(nMinDisPt >= 0);
	return dMinDis;
}
template<class T>
double skMinCurve2CurveDistance(T* pdX0, T* pdY0, int n0, 
								T* pdX1, T* pdY1, int n1, 
								int &nMinDisPt0, int &nMinDisPt1)

{
	nMinDisPt0 = -1;
	nMinDisPt1 = -1;
	double dDis, dMinDis = 1000000;
	for (int i = 0; i < n0; i++)
	{
		for (int j = 0; j < n1; j++)
		{
			dDis = sqrtDistance(pdX0[i], pdY0[i], pdX1[j], pdY1[j]);
			if (dDis < dMinDis)
			{
				dMinDis = dDis;
				nMinDisPt0 = i;
				nMinDisPt1 = j;
			}
		}
	}
	assert(nMinDisPt0 != -1);

	return dMinDis;
}
inline double skAbsoluteAngleDifferenceOfTwoLines(double dOrt0, double dOrt1)
{//[0,pi/2)
	double dDeltaOrt = dOrt0 - dOrt1;

	double pi = PI;
	while (dDeltaOrt >= pi)
			dDeltaOrt -= 2*pi;
	while (dDeltaOrt < -pi)
		dDeltaOrt += 2*pi; //[-pi, pi)
	if (dDeltaOrt >= pi/2)
		dDeltaOrt = dDeltaOrt - pi;
	if (dDeltaOrt < -pi/2)
		dDeltaOrt = dDeltaOrt + pi;//[-pi/2,pi/2)

	return fabs(dDeltaOrt);

}
inline double skAbsoluteAngleDifferenceOfTwoVectors(double dOrt0, double dOrt1)
{//[0,pi)
	double dDeltaOrt = dOrt0 - dOrt1;

	double pi = PI;
	while (dDeltaOrt >= pi)
			dDeltaOrt -= 2*pi;
	while (dDeltaOrt < -pi)
		dDeltaOrt += 2*pi; //[-pi, pi)

	return fabs(dDeltaOrt);

}
template<class T>
double skGetOrientation(T x0, T y0, T x1, T y1)
{
	double dx1 = x1 - x0;
	double dy1 = y1 - y0;
	double dOrt1 = ::atan2(dx1, dy1);
	return dOrt1;
}

template<class T>
double skGetCornerAngle(T x0, T y0, T x1, T y1, T x2, T y2)
{//[0,PI)
	double dOrt1 = skGetOrientation(x0, y0, x1, y1);
	double dOrt2 = skGetOrientation(x0, y0, x2, y2);

	return skGetCornerAngle(dOrt1, dOrt2);

}
//[0,PI)
inline double skGetCornerAngle(double dOrt1, double dOrt2)
{
	double dDeltaOrt = dOrt2 - dOrt1;

	double pi = PI;
	while (dDeltaOrt >= pi)
			dDeltaOrt -= 2*pi;
	while (dDeltaOrt < -pi)
		dDeltaOrt += 2*pi; //[-pi, pi)

	return fabs(dDeltaOrt);//return the absolute value
}
inline double getSearchOrt(double dOrt0, //previous direction
					double dOrt, //current orientation
					int e)//end type
{

	//get delta ort
	double dError = 0.00001;
	double dDeltaOrt = dOrt0 - dOrt;

	// set delta ort [pi,pi)
	double pi = PI;
	while (dDeltaOrt >= pi)
			dDeltaOrt -= 2*pi;
	while (dDeltaOrt < -pi)
		dDeltaOrt += 2*pi; //[-pi, pi)
	// set delta ort [-pi/2, pi/2)
	if (dDeltaOrt >= pi/2)
		dDeltaOrt = dDeltaOrt - pi;
	if (dDeltaOrt < -pi/2)
		dDeltaOrt = dDeltaOrt + pi;

	//get the new ort which has the difference to dOrt0 less than pi/2
	double dOrt_new = dOrt0 - dDeltaOrt;

	if (e == 0)
	{//if search left, inverse the direction
		dOrt_new += pi;
	}
	return dOrt_new;
}
inline int skComputeSortedAngleFromOrt(double *pdOrt0, double *pdAngle, int n)
{
	double *pdOrt = new double[n];
	int i;
	for (i = 0; i < n; i++)
		pdOrt[i] = pdOrt0[i];

	for (i = 0; i < n; i++)
	{
		while (pdOrt[i] >= 2*PI)
			pdOrt[i] -= 2*PI;
		while (pdOrt[i] < 0)
			pdOrt[i] += 2*PI;
	}
	//[0,2*PI)
	::sort(n, pdOrt, 0);//0-increasing, 1-decreasing
	for (i = 0; i < n-1; i++)
	{
		pdAngle[i] = pdOrt[i+1] - pdOrt[i];
	}
	pdAngle[n-1] = pdOrt[0] + 2*PI - pdOrt[n-1];
	::sort(n, pdAngle, 0);//increasing
	delete []pdOrt;
	return 0;
}

inline int diagonalType(int x1, int y1, int x2, int y2)
{
	int dx = x2 - x1;
	int dy = y1 - y1;
	if (dx == dy)
		return 1;
	if (dx == -dy)
		return -1;
	return 0;
}

inline bool diagonalCross(int x1_s, int y1_s, int x1_e, int y1_e,
						  int x2_s, int y2_s, int x2_e, int y2_e)
{
	int d1 = diagonalType(x1_s, y1_s, x1_e, y1_e);
	if (d1 != 0)
	{
		int d2 = diagonalType(x2_s, y2_s, x2_e, y2_e);
		if (d2 != 0)
		{
			if (d1 == -d2)
			{
				int min_x1 = MIN(x1_s, x1_e);
				int max_x1 = MAX(x1_s, x1_e);
				int min_y1 = MIN(y1_s, y1_e);
				int max_y1 = MAX(y1_s, y1_e);

				int min_x2 = MIN(x2_s, x2_e);
				int max_x2 = MAX(x2_s, x2_e);
				int min_y2 = MIN(y2_s, y2_e);
				int max_y2 = MAX(y2_s, y2_e);

				if (min_x1 <= max_x2 && min_x2 <= max_x1 
					&& min_y1 <= max_y2 && min_y2 <= max_y1)
					return true;
			}
		}
	}
	return false;
}
#endif //_INLINEFUNC_H_