#ifndef _ADAPTHIS_H_
#define _ADAPTHIS_H_

#include <stdio.h>

#define  OVAL_INFINITE 1e6
// INIFINITE is defined for properties of token neighbors.
// If there is no neighbor, all the properties are infinite.
// If the connection to the neighbor is crossed boundary, all the properties are infinite too if hard boundary is applied.
// The infinite properties are treated as the largest bin in that property.
// So we set it 1e6 

#ifndef ADD
#define ADD 0
#endif

#ifndef REMOVE 
#define REMOVE 1
#endif


class CAdaptHis
{// very stupid design, needs to be improved

#define DEFAULT_EACH_DIM_BIN_NUM 1
#define DEFAULT_UPPER_LIMIT 100
#define DEFAULT_LOWER_LIMIT 0
#define DEFAULT_EACH_DIM_MIN_STEP 1
#define DEFAULT_MAX_SAMPLE_NUM 1
#define MAX_DIM_NUM 2

private:
	int		m_nDim;//dimension of joint distribution
	int		m_nBinNum;//total number of joint bins
	bool	m_bSaveSample;//save the sample in this class or not
	bool	m_bContinous;//true - consider all the cells between upper and lower limit as bins, false-only consider cells have samples 
	int		m_nSampleNum;//sample number
	int		m_nMaxSampleNum;//maximun sample number, allocated memory for samples
	int		m_nTotalCellNum;//total cell number to sum the samples
	double	*m_pdMinStep;//min step for each dimension
	double  *m_pdBinStep;//real step for each bin
	int		*m_pnMinBinNum;//min bin num for each dimension
	int		*m_pnMaxBinNum;//max bin num for each dimension
	int		*m_pnBinNum;//real bin num for each dimension
	double	*m_pdUpperLimits;//max value of each dim
	double	*m_pdLowerLimits;//min value of each dim
	int		*m_pnBinNoOfEachCell;//matrix shows bin index for each cell
	int		*m_pnPointSumOfEachCell;//matrix shows sample num at each cell
	double	*m_pdSamples;//pointer to samples
	double	*m_pdEachBinCenter;//the center value of each bin
	double	*m_pdHis;//histogram of all valid bins
	double  m_dAbsWeightCenter;
	double  m_dKurtosis;
	bool	m_bValid;
	double  m_dErrorCut;// percentage of cut at each side
public:
	bool	m_bZeroMustBeOneBin;

public:
	CAdaptHis();
	~CAdaptHis();
		// applied when as a matrix 
	CAdaptHis(double v){};
	CAdaptHis & operator = (CAdaptHis &his);
	CAdaptHis &operator = (double v){return *this;};
	bool operator > (CAdaptHis &h){return false;}
	bool operator < (CAdaptHis &h){return false;}
	bool operator != (CAdaptHis &h){return false;}
	bool operator == (CAdaptHis &h);//{return false;}
	CAdaptHis operator + (CAdaptHis &h){ return *this;}
	CAdaptHis &operator += (CAdaptHis &h){ return *this;}
	CAdaptHis operator - (CAdaptHis &h){ return *this;}
	CAdaptHis operator * (CAdaptHis &h){ return *this;}
	CAdaptHis operator * (double v){return *this;}
	CAdaptHis operator / (const CAdaptHis h){ return *this;}
	CAdaptHis operator / (double v){ return *this;};

	void setErrorCut(double c) { m_dErrorCut = c; }
	double getErrorCut() { return m_dErrorCut; }
	void setValid(bool b){m_bValid = b;}
	bool valid() {return m_bValid;}
	int buildByLimitsAndBinNum(double* pdLowerLimits, double* pdUpperLimits, int* pnBinNum);
	int buildFromSamplesAndMinStepAndMaxBinNum(double* pdMinStep, int* pnMaxBinNum);//must add samples first
	int	AddOneSample( double *pdSample );
	void SetMaxSampleNum( int nMaxSampleNum );
	void setSaveSample(bool b) {m_bSaveSample = b;}
	int	 addSamples(CAdaptHis &his);
	int	 SetDimension( int nDim );
	void SetZeroMustBeOneBin(bool b) { m_bZeroMustBeOneBin=b;}
	int  changePointOfCell(double *pdValue, int flag);
	int  changePointOfCell(int bin, int flag);
	int	 removeAllSamples();
	int	 removeSamples(CAdaptHis &his);
	void  clearSamplesAndHistogram();
	int		CalculateHistogram();
	double histogramDifference(CAdaptHis *pHis);
	double histogramProduct(CAdaptHis *pHis);

	int getBinStep(double *pdBinStep) { for (int i=0; i<m_nDim; i++) pdBinStep[i]=m_pdBinStep[i]; return m_nDim; }
	int getLowerLimits(double *pdLowerLimits) {for (int i=0; i<m_nDim; i++) pdLowerLimits[i]=m_pdLowerLimits[i]; return m_nDim; }
	int getUpperLimits(double *pdUpperLimits) {for (int i=0; i<m_nDim; i++) pdUpperLimits[i]=m_pdUpperLimits[i]; return m_nDim; }

	int		GetDim() { return m_nDim; }
	int		Finished();

	double	*getHis() { return m_pdHis; }
	int		GetSampleNum() { return m_nSampleNum; }
	int		GetBinNum()	{ return m_nBinNum; }
	void	SetBinNum(int nBinNum) { m_nBinNum = nBinNum; }
	int		Real2Bin( double *pdReal );
	int		Bin2Real( int &nBin, double *pdReal );
	int		Bin2RealRandom( int &nBin, double *pdReal );//not center, but randomly in that bin
	void	SetMinStep( double *pdMinStep );
	int		SetMaxBinNum( int *pnMaxBinNum );
	int		SetMinBinNum( int *pnMinBinNum );
	double  GetMinStep(int i) { return m_pdMinStep[i]; }
	void	Reset() { DefaultSetting(); }
	int		getSampleNum(int nBin) { if (nBin<0 || nBin >= m_nBinNum) return 0; else return (int)(m_pdHis[nBin]*m_nSampleNum+0.5); }
	int		getSampleNum() { return m_nSampleNum; }
	double	*GetPointer2Samples() { return m_pdSamples; }


	double	randomSample(double *pdValue);
	int		saveHistogram(FILE *pFile);
	int		saveHistogram(const char *name);
	double	computeAbsWeightCenter();
	double	computeKurtosis();

private:
	void	Zero();
	void	DefaultSetting();
	void	ReleaseMemory();
	bool	SetContinous( bool c );
	int		AllocateMemory4Matrices();
	int		OneDimReal2Bin( double dReal, int nDimNo );
	double	OneDimBin2Real( int nBin, int nDimNo );
	int		GetCellPos( int *pnBin );
	void	GetBinNoFromCellPos( int* pnBinNo, int nCellPos );
	void	CalculateLimitsFromSamples();
	void	ErrorCut();
	void	CalculatePointSum();
	int		SetBinNoAtEachCell();
	int		CalculateEachBinNumAndLimits();
};
#endif// _ADAPTHIS_H_
