#include <assert.h>
#include <memory.h>
#include <math.h>

#include "skAdaptHis.h"
#include "inlinefunc.h"

// constructor
CAdaptHis::CAdaptHis()
{// default setting
	m_bZeroMustBeOneBin = false;
	m_bValid = true;
	m_dErrorCut = 0;
	Zero();
	DefaultSetting();
}
// deconstructor
CAdaptHis::~CAdaptHis()
{
	ReleaseMemory();
}

///////////////////
// operator = 
CAdaptHis & CAdaptHis::operator = (CAdaptHis &his)
{//only copy the histogram (sample num in each bin), parameters, but no samples backup
	m_bZeroMustBeOneBin = his.m_bZeroMustBeOneBin;
	int nDim = his.GetDim();
	if (m_nDim != nDim)
		SetDimension(nDim);

	m_bContinous = his.m_bContinous;
	m_bSaveSample = his.m_bSaveSample;

	// each dim min step
	memcpy(m_pdMinStep, his.m_pdMinStep, sizeof(double)*nDim);
	memcpy(m_pdBinStep, his.m_pdBinStep, sizeof(double)*nDim);
	// each dim min bin num
	memcpy(m_pnMinBinNum, his.m_pnMinBinNum, sizeof(int)*nDim);
	// each dim max bin num
	memcpy(m_pnMaxBinNum, his.m_pnMaxBinNum, sizeof(int)*nDim);
	// each dim bin num
	memcpy(m_pnBinNum, his.m_pnBinNum, sizeof(int)*nDim);
	// each dim upper limits
	memcpy(m_pdUpperLimits, his.m_pdUpperLimits, sizeof(double)*nDim);
	// each dim lower limits
	memcpy(m_pdLowerLimits, his.m_pdLowerLimits, sizeof(double)*nDim);


	m_nSampleNum = his.getSampleNum();
	m_nMaxSampleNum = 0;

	// histogram
	if (m_nBinNum != his.m_nBinNum)
	{
		if (m_pdHis != 0)
			delete[] m_pdHis;
		m_nBinNum = his.m_nBinNum;
		m_pdHis = new double[m_nBinNum];
	}
	memcpy(m_pdHis, his.m_pdHis, sizeof(double)*m_nBinNum);

	// memory for matrices
	m_nTotalCellNum = his.m_nTotalCellNum;
	if( m_pnPointSumOfEachCell != 0 )
		delete[] m_pnPointSumOfEachCell;
	m_pnPointSumOfEachCell = new int[m_nTotalCellNum];
	memcpy( m_pnPointSumOfEachCell, his.m_pnPointSumOfEachCell, sizeof(int)*m_nTotalCellNum );
	if( m_pnBinNoOfEachCell != 0 )
		delete[] m_pnBinNoOfEachCell;
	m_pnBinNoOfEachCell = new int[m_nTotalCellNum];
	memcpy( m_pnBinNoOfEachCell, his.m_pnBinNoOfEachCell, sizeof(int)*m_nTotalCellNum );

	if( m_pdEachBinCenter != 0 )
		delete[] m_pdEachBinCenter;
	m_pdEachBinCenter = new double[m_nDim*m_nBinNum];//one more room for empty bins
	memcpy(m_pdEachBinCenter, his.m_pdEachBinCenter, sizeof(double)*m_nDim*m_nBinNum);


	return *this;

}

bool CAdaptHis::operator == (CAdaptHis &his)
{
	if (m_bZeroMustBeOneBin != his.m_bZeroMustBeOneBin)
		return false;

	int nDim = his.GetDim();
	if (m_nDim != nDim)
		return false;

	if (m_bContinous != his.m_bContinous)
		return false;

	if (m_nSampleNum != his.getSampleNum())
		return false;

	if (m_nTotalCellNum != his.m_nTotalCellNum)
		return false;

	for (int i = 0; i < m_nTotalCellNum; i++)
	{
		if (m_pnPointSumOfEachCell[i] != his.m_pnPointSumOfEachCell[i])
			return false;
	}

	return true;
}
///////////////////
// Zero all the pointers
void CAdaptHis::Zero()
{
	m_nDim = 0;// one dimension
	m_nBinNum = 0;
	m_nSampleNum = 0;
	m_nMaxSampleNum = DEFAULT_MAX_SAMPLE_NUM;
	m_nTotalCellNum = 0;

	m_bSaveSample = true;
	m_bContinous = true;

	m_pdMinStep = 0;
	m_pdBinStep = 0;
	m_pnMinBinNum = 0;
	m_pnMaxBinNum = 0;
	m_pnBinNum = 0;
	m_pdUpperLimits = 0;
	m_pdLowerLimits = 0;
	m_pnBinNoOfEachCell = 0;
	m_pnPointSumOfEachCell = 0;
	m_pdSamples = 0;
	m_pdEachBinCenter = 0;
	m_pdHis = 0;
}
///////////////////
// clear all the memory
void CAdaptHis::ReleaseMemory()
{
	if( m_pdMinStep != 0 )
		delete[] m_pdMinStep;
	if( m_pdBinStep != 0 )
		delete[] m_pdBinStep;
	if( m_pnMinBinNum != 0 )
		delete[] m_pnMinBinNum;
	if( m_pnMaxBinNum != 0 )
		delete[] m_pnMaxBinNum;
	if( m_pnBinNum != 0 )
		delete[] m_pnBinNum;
	if( m_pdUpperLimits != 0 )
		delete[] m_pdLowerLimits;
	if( m_pdUpperLimits != 0 )
		delete[] m_pdUpperLimits;
	if( m_pnBinNoOfEachCell != 0 )
		delete[] m_pnBinNoOfEachCell;
	if( m_pnPointSumOfEachCell != 0 )
		delete[] m_pnPointSumOfEachCell;
	if( m_pdSamples != 0 )
		delete[] m_pdSamples;
	if( m_pdEachBinCenter != 0 )
		delete[] m_pdEachBinCenter;
	if (m_pdHis != 0)
		delete[] m_pdHis;
	Zero();
}
void CAdaptHis::DefaultSetting()
{
	SetDimension(1);//one dimension
}
// set the dimension, all other related memories are reset
int CAdaptHis::SetDimension( int nDim )
{
	int i;
	//dimension
	m_nDim = nDim;
	m_nSampleNum = 0;

	// each dim min step
	if( m_pdMinStep != 0 )
		delete[] m_pdMinStep;
	m_pdMinStep = new double[m_nDim];
	if( m_pdBinStep != 0 )
		delete[] m_pdBinStep;
	m_pdBinStep = new double[m_nDim];
	for (i = 0; i < m_nDim; i++)
	{
		m_pdMinStep[i] = DEFAULT_EACH_DIM_MIN_STEP;
		m_pdBinStep[i] = DEFAULT_EACH_DIM_MIN_STEP;
	}
	// each dim min bin num
	if( m_pnMinBinNum != 0 )
		delete[] m_pnMinBinNum;
	m_pnMinBinNum = new int[m_nDim];
	for( i = 0; i < m_nDim; i++ )
		m_pnMinBinNum[i] = 1;
	// each dim max bin num
	if( m_pnMaxBinNum != 0 )
		delete[] m_pnMaxBinNum;
	m_pnMaxBinNum = new int[m_nDim];
	for( i = 0; i < m_nDim; i++ )
		m_pnMaxBinNum[i] = DEFAULT_EACH_DIM_BIN_NUM;
	// each dim bin num
	if( m_pnBinNum != 0 )
		delete[] m_pnBinNum;
	m_pnBinNum = new int[m_nDim];
	for( i = 0; i < m_nDim; i++ )
		m_pnBinNum[i] = DEFAULT_EACH_DIM_BIN_NUM;
	// each dim upper limits
	if( m_pdUpperLimits != 0 )
		delete[] m_pdUpperLimits;
	m_pdUpperLimits = new double[m_nDim];
	for( i = 0; i < m_nDim; i++ )
		m_pdUpperLimits[i] = DEFAULT_UPPER_LIMIT;
	// each dim lower limits
	if( m_pdLowerLimits != 0 )
		delete[] m_pdLowerLimits;
	m_pdLowerLimits = new double[m_nDim];
	for( i = 0; i < m_nDim; i++ )
		m_pdLowerLimits[i] = DEFAULT_LOWER_LIMIT;
	// max sample num
	if (m_pdSamples != 0)
		delete[] m_pdSamples;
	m_nMaxSampleNum = DEFAULT_MAX_SAMPLE_NUM;
	m_pdSamples = new double[m_nMaxSampleNum*m_nDim];
	// histogram
	if (m_pdHis != 0)
		delete[] m_pdHis;
	m_pdHis = new double[DEFAULT_EACH_DIM_BIN_NUM*m_nDim];
	return 0;
}
// set the max bin num for each dim
int CAdaptHis::SetMaxBinNum( int *pnMaxBinNum )
{
	assert( m_nDim > 0 );
	for( int i = 0; i < m_nDim; i++ )
		m_pnMaxBinNum[i] = pnMaxBinNum[i];
	return 0;
}

// set the min bin num for each dim
int CAdaptHis::SetMinBinNum( int *pnMinBinNum )
{
	assert( m_nDim > 0 );
	for( int i = 0; i < m_nDim; i++ )
		m_pnMinBinNum[i] = pnMinBinNum[i];
	return 0;
}
// set continous, if continous=true, all cells are considered, otherwise, empty cells will be discarded
bool CAdaptHis::SetContinous( bool c )
{
	m_bContinous = c;
	return c;
}
// allcoate memory for cells
int CAdaptHis::AllocateMemory4Matrices()
{
	assert( m_nDim > 0 );
	m_nTotalCellNum = 1;
	for( int i = 0; i < m_nDim; i++ )
		m_nTotalCellNum *= m_pnBinNum[i];
	if( m_pnPointSumOfEachCell != 0 )
		delete[] m_pnPointSumOfEachCell;
	m_pnPointSumOfEachCell = new int[m_nTotalCellNum];
	memset( m_pnPointSumOfEachCell, 0, sizeof(int)*m_nTotalCellNum );
	if( m_pnBinNoOfEachCell != 0 )
		delete[] m_pnBinNoOfEachCell;
	m_pnBinNoOfEachCell = new int[m_nTotalCellNum];
	memset( m_pnBinNoOfEachCell, 0, sizeof(int)*m_nTotalCellNum );
	return 0;
}
///////////////////
// clearHistogram()
//	set the histogram to zero, also the m_pnPointSumOfEachCell
void CAdaptHis::clearSamplesAndHistogram()
{
	removeAllSamples();
	memset( m_pnPointSumOfEachCell, 0, sizeof(int)*m_nTotalCellNum );
	memset( m_pdHis, 0, sizeof(double)*m_nTotalCellNum );

}
// min step for each dim is set
void CAdaptHis::SetMinStep( double *pdMinStep )
{
	assert( m_nDim > 0 );
	for( int i = 0; i < m_nDim; i++ )
		m_pdMinStep[i] = pdMinStep[i];
}
// give real values, the bin no is returned
int CAdaptHis::Real2Bin( double *pdReal )
{
	if (m_nDim == 1)//to speedup
		return OneDimReal2Bin( pdReal[0], 0 );
	else
	{
		int pnBin[MAX_DIM_NUM];
		for( int i = 0; i < m_nDim; i++ )
		{
			pnBin[i] = OneDimReal2Bin( pdReal[i], i );
		}
		int nCellPos = GetCellPos( pnBin );
		return m_pnBinNoOfEachCell[nCellPos];
	}
}
// give a bin no, the real values are returned
int CAdaptHis::Bin2Real( int &nBin, double *pdReal )
{
	int nOffset = nBin*m_nDim;
	for( int i = 0; i < m_nDim; i++ )
	{
		pdReal[i] = m_pdEachBinCenter[nOffset+i];
	}
	return 0;
}
//not center, but randomly in that bi
int	CAdaptHis::Bin2RealRandom( int &nBin, double *pdReal )
{
	int nOffset = nBin*m_nDim;
	for( int i = 0; i < m_nDim; i++ )
	{
		pdReal[i] = m_pdEachBinCenter[nOffset+i];
		pdReal[i] += (rand()/(RAND_MAX+1.0)-0.5) * m_pdBinStep[i];
	}
	return 0;
}

// to reserve the space
void CAdaptHis::SetMaxSampleNum( int nMaxSampleNum )
{
	m_nMaxSampleNum = nMaxSampleNum;
	if( m_nDim > 0 && m_bSaveSample )
	{
		if( m_pdSamples != 0 )
			delete[]m_pdSamples;
		m_pdSamples = new double[m_nMaxSampleNum*m_nDim];
	}
}
// add one sample
int CAdaptHis::AddOneSample( double *pdSample )
{
	if( m_bSaveSample )
	{//save the samples
		if (m_nSampleNum == m_nMaxSampleNum )
		{// not enough memory any more
			double *pdSamples = new double[(m_nMaxSampleNum+8)*m_nDim];
			memcpy(pdSamples, m_pdSamples, sizeof(double)*m_nMaxSampleNum*m_nDim);
			m_nMaxSampleNum += 8;
			delete []m_pdSamples;
			m_pdSamples = pdSamples;
		}
		int nOffset = m_nSampleNum*m_nDim;
		for( int i = 0; i < m_nDim; i++ )
			m_pdSamples[nOffset+i] = pdSample[i];
		m_nSampleNum++;
	}
	else
	{//not save the samples, change the bin directly
		int nBin = Real2Bin(pdSample);
		m_pnPointSumOfEachCell[nBin]++;
		m_nSampleNum++;
	}
	return 0;
}
// give the real value at one dim, the bin at that dim will be returned
int CAdaptHis::OneDimReal2Bin( double dReal, int nDimNo )
{
//	assert( m_pnBinNum[nDimNo] > 0 );
	if( m_pdLowerLimits[nDimNo] == m_pdUpperLimits[nDimNo] )
		return 0;
	int nBin = (int)( (dReal-m_pdLowerLimits[nDimNo]) / (m_pdUpperLimits[nDimNo]-m_pdLowerLimits[nDimNo] ) * m_pnBinNum[nDimNo] );
//	int nBin = 0;
	if( nBin < 0 )
		nBin = 0;
	else if( nBin >= m_pnBinNum[nDimNo] )
		nBin = m_pnBinNum[nDimNo] - 1;
	return nBin;
}
// give the bin at one dim, the real value is returned
double CAdaptHis::OneDimBin2Real( int nBin, int d )
{
	assert( m_pnBinNum[d] > 0 );
	if( m_pdLowerLimits[d] == m_pdUpperLimits[d] )
		return m_pdLowerLimits[d];
	double dReal = m_pdLowerLimits[d] + (nBin+0.5) * (m_pdUpperLimits[d] - m_pdLowerLimits[d] ) / m_pnBinNum[d];
	assert( dReal >= m_pdLowerLimits[d] );
	assert( dReal < m_pdUpperLimits[d] );
	return dReal;
}
// give the bin no in each dim, the cell pos will be returned
int CAdaptHis::GetCellPos( int *pnBin )
{
	int nCellPos = 0;
	int nD = 1;
	for( int i = 0; i < m_nDim; i++ )
	{
		nD = 1;
		for( int j = 0; j < i; j++ )
		{
			nD *= m_pnBinNum[j];
		}
		nCellPos += nD * pnBin[i];
	}
	return nCellPos;
}
// give cell pos, the bin no for each dim is returned
void CAdaptHis::GetBinNoFromCellPos( int* pnBinNo, int nCellPos )
{// change cell pos to bin no in each dimension
	int nPos = nCellPos;
	for( int i = m_nDim-1; i >= 0; i-- )
	{
		int nD = 1;
		for( int j = 0; j < i; j++ )
		{
			nD *= m_pnBinNum[j];
		}
		pnBinNo[i] = nPos / nD;
		nPos -= pnBinNo[i]*nD;
	}
}
// calculate limits for each dim
void CAdaptHis::CalculateLimitsFromSamples()
{// here is a trick, only consider those samples smaller than OVAL_INFINITE/2 as limits
	if( m_nSampleNum <= 0 )
		return;
	for( int i = 0; i < m_nDim; i++ )
	{
		m_pdLowerLimits[i] = OVAL_INFINITE;//m_pdSamples[i];
		m_pdUpperLimits[i] = -OVAL_INFINITE;//m_pdSamples[i];
	}
	int nOffset = 0;
	for( int i = 0; i < m_nSampleNum; i++ )
	{
		for( int j = 0; j < m_nDim; j++ )
		{
			if( fabs(m_pdSamples[nOffset+j]) > OVAL_INFINITE/2 )
				continue;// here is the trick
			if( m_pdLowerLimits[j] > m_pdSamples[nOffset+j] )
				m_pdLowerLimits[j] = m_pdSamples[nOffset+j];
			if(	m_pdUpperLimits[j] < m_pdSamples[nOffset+j] )
				m_pdUpperLimits[j] = m_pdSamples[nOffset+j];
		}
		nOffset += m_nDim;
	}
	for( int i = 0; i < m_nDim; i++ )
	{
		if( m_pdLowerLimits[i] > m_pdUpperLimits[i] )
			m_pdLowerLimits[i] = m_pdUpperLimits[i] = 0;
	}
	ErrorCut();
}
// use some error percent to cut the boundary of each dim
void CAdaptHis::ErrorCut()
{
	if( m_nDim != 1 )
		return;
	if( m_pdLowerLimits[0] - m_pdUpperLimits[0] == 0 )
		return;
#define HisNum 10001
	int pnSum[HisNum];
	memset( pnSum, 0, sizeof(int)*HisNum );
	int nBin;
	double dSegUnit = (m_pdUpperLimits[0] - m_pdLowerLimits[0])/(HisNum-1);
	int nTotalSum = 0;
	int i=0;
	for( i = 0; i < m_nSampleNum; i++ )
	{
		if( m_pdSamples[i] < -OVAL_INFINITE/2 )
				continue;
		nBin = (int)((m_pdSamples[i] - m_pdLowerLimits[0])/dSegUnit);
		pnSum[nBin]++;
		nTotalSum++;
	}
	int nSum = 0;
	for( i = 0; i < HisNum; i++ )
	{
		nSum += pnSum[i];
		if( nSum >= nTotalSum * m_dErrorCut )
			break;
	}
	m_pdLowerLimits[0] += dSegUnit * i;
	nSum = 0;
	for( i = HisNum - 1; i >=0 ; i-- )
	{
		nSum += pnSum[i];
		if( nSum >= nTotalSum * m_dErrorCut )
			break;
	}
	m_pdUpperLimits[0] -= dSegUnit * (HisNum-i-1);
}
// calculate points num for each cell
void CAdaptHis::CalculatePointSum()
{
//	int *pnBin = new int[m_nDim];
	int pnBin[MAX_DIM_NUM];
	int nOffset = 0;
	for( int i = 0; i < m_nSampleNum; i++ )
	{
		int nCellPos;
		for( int j = 0; j < m_nDim; j++ ) 
			pnBin[j] = OneDimReal2Bin( m_pdSamples[nOffset+j], j );
		nCellPos = GetCellPos( pnBin );
		m_pnPointSumOfEachCell[nCellPos]++;
		nOffset += m_nDim;
	}
//	delete[] pnBin;
}
// calculate bin num and limits of each dimension
int CAdaptHis::CalculateEachBinNumAndLimits()
{
	for( int i = 0; i < m_nDim; i++ )
	{
		if( m_pdUpperLimits[i] == m_pdLowerLimits[i] )
		{
			// even though in the samples, upper and lower limits are the same
			// we may also want more bins for the models (only one bin does not always satisfy the application)
			// So MinBinNum may be set for each dim
			m_pnBinNum[i] = MAX( 1, m_pnMinBinNum[i] ) / 2 * 2 + 1;//keep odd
			m_pdUpperLimits[i] += m_pdMinStep[i] * (m_pnBinNum[i]/2+0.5);
			m_pdLowerLimits[i] -= m_pdMinStep[i] * (m_pnBinNum[i]/2+0.5);
				// changed on 01-09-2002, without understanding why set Num = 3, not 1.
		}
		else
		{
			if( m_pdUpperLimits[i] - m_pdLowerLimits[i] < m_pdMinStep[i] )
			{
				m_pdUpperLimits[i] = (m_pdUpperLimits[i] + m_pdLowerLimits[i])/2 + m_pdMinStep[i]/2;
				m_pdLowerLimits[i] = (m_pdUpperLimits[i] + m_pdLowerLimits[i])/2 - m_pdMinStep[i]/2;
			}
			m_pnBinNum[i] = (int)( (m_pdUpperLimits[i] - m_pdLowerLimits[i]) / m_pdMinStep[i]) + 1;
			if( m_pnBinNum[i] > m_pnMaxBinNum[i] )
				m_pnBinNum[i] = m_pnMaxBinNum[i];
			m_pdMinStep[i] = (m_pdUpperLimits[i] - m_pdLowerLimits[i]) / (m_pnBinNum[i]-1);
//			m_pnBinNum[i] += 2;
//			m_pdUpperLimits[i] += m_pdMinStep[i] * 3 / 2;
//			m_pdLowerLimits[i] -= m_pdMinStep[i] * 3 / 2;
			m_pnBinNum[i] += 0;
			m_pdUpperLimits[i] += m_pdMinStep[i] / 2;
			m_pdLowerLimits[i] -= m_pdMinStep[i] / 2;
			if( m_pnBinNum[i] < m_pnMinBinNum[i] )
			{
				int nDeltaNum = m_pnMinBinNum[i] - m_pnBinNum[i];
				nDeltaNum = (nDeltaNum+1)/2;
				m_pdUpperLimits[i] += m_pdMinStep[i] * nDeltaNum;
				m_pdLowerLimits[i] -= m_pdMinStep[i] * nDeltaNum;
				m_pnBinNum[i] += 2*nDeltaNum;
			}
			// copy min step to bin step
			m_pdBinStep[i] = m_pdMinStep[i];
			//add one more bin for the illegal values, April 15, 2001
			m_pnBinNum[i] += 1;
			m_pdUpperLimits[i] += m_pdMinStep[i];
			//add one more bin for the less range values, April 15, 2001
			m_pnBinNum[i] += 1;
			m_pdLowerLimits[i] -= m_pdMinStep[i];

		}//else
	}// for i, each dimension

	if (m_bZeroMustBeOneBin)
	{

		for( int i = 0; i < m_nDim; i++ )
		{//for each dimension
			if (m_pdLowerLimits[i] < 0 && m_pdUpperLimits[i] > 0)
			{
				int k = (fabs(m_pdLowerLimits[i]/m_pdMinStep[i]));
				m_pdLowerLimits[i] = -(k + 0.5)* m_pdMinStep[i];
				m_pdUpperLimits[i] = (m_pnBinNum[i] - k - 0.5)* m_pdMinStep[i];
			}
		}
	}
	return 0;
}
// after all sampled are added, finish all the needed calculation
int CAdaptHis::Finished()
{
	CalculateLimitsFromSamples();
	CalculateEachBinNumAndLimits();
	AllocateMemory4Matrices();
	CalculatePointSum();
	SetBinNoAtEachCell();
	CalculateHistogram();
	return 0;
}
// set bin no for each cell
int CAdaptHis::SetBinNoAtEachCell()
{
	assert( m_nTotalCellNum > 0 );
	//set bin No at bin matrix
	bool bEmpty = false;
//	if( m_nDim > 1 || !m_bContinous)
	if( !m_bContinous)
	{//NOT continueous, to reduce the dimension
		m_nBinNum = 0;
		for( int i = 0; i < m_nTotalCellNum; i++ )
		{
			if( m_pnPointSumOfEachCell[i] > 0 )
			{
				m_pnBinNoOfEachCell[i] = m_nBinNum;
				m_nBinNum++;
			}
		}
		for( int i = 0; i < m_nTotalCellNum; i++ )
			if( m_pnPointSumOfEachCell[i] <= 0 )//for empty cells, bin = binNum
			{
				m_pnBinNoOfEachCell[i] = m_nBinNum+1;
				bEmpty = true;
			}
		if (bEmpty)
			m_nBinNum++;
	}
	else
	{// continuous, keep all cells
		for( int i = 0; i < m_nTotalCellNum; i++ )
			m_pnBinNoOfEachCell[i] = i;
		m_nBinNum = m_nTotalCellNum;
	}
	//set center for each bin, and bin num for each dimension
	if( m_pdEachBinCenter != 0 )
		delete[] m_pdEachBinCenter;
	m_pdEachBinCenter = new double[m_nDim*m_nBinNum];//one more room for empty bins
	int *pnBinNo = new int[m_nDim];
	if (bEmpty)
	{
		int nOffset = (m_nBinNum-1)*m_nDim;
		for( int i = 0; i < m_nTotalCellNum; i++ )
		{
			if(m_pnPointSumOfEachCell[i] <= 0 )// bins which are kicked out
			{
				GetBinNoFromCellPos( pnBinNo, i );
				// set the center of the first blank cell as all the centers for the bins which are kicked out
				for( int j = 0; j < m_nDim; j++ )
					m_pdEachBinCenter[nOffset+j] = OneDimBin2Real( pnBinNo[j], j );
				break;
			}
		}
	}
	// set bin centers
	for( int i = 0; i < m_nTotalCellNum; i++ )
	{
		int nOffset = m_pnBinNoOfEachCell[i]*m_nDim;
//		if( m_pnPointSumOfEachCell[i] > 0 )// bins which are kept, not necessary, all could be calculated
		{
			GetBinNoFromCellPos( pnBinNo, i );
			for( int j = 0; j < m_nDim; j++ )
				m_pdEachBinCenter[nOffset+j] = OneDimBin2Real( pnBinNo[j], j );
		}
	}
	delete[] pnBinNo;
	return 0;
}
// calculate histogram
//	the total sample num known, also m_pnPointSumOfEachCell known
int CAdaptHis::CalculateHistogram()
{
	if (m_pdHis != 0)
		delete []m_pdHis;
	int nBinNum = GetBinNum();
	int nSampleNum = GetSampleNum();
//	if (m_pdHis == 0)
		m_pdHis = new double[nBinNum];
	if (nSampleNum <= 0)
	{
		memset(m_pdHis, 0, sizeof(double)*nBinNum);
		return 1;
	}
	int nBin;
	for (int i = 0; i < m_nTotalCellNum; i++)
	{
		nBin = m_pnBinNoOfEachCell[i];
		m_pdHis[nBin] = (double)(m_pnPointSumOfEachCell[i])/nSampleNum;
	}
	return 0;
}
// build bins directly from given limits and bin num
int CAdaptHis::buildByLimitsAndBinNum(double* pdLowerLimits, double* pdUpperLimits, int* pnBinNum)
{
	for( int i = 0; i < m_nDim; i++ )
	{
//		assert(pdLowerLimits[i] < pdUpperLimits[i]);
		if ( pdLowerLimits[i] > pdUpperLimits[i] )
		{
			 pdLowerLimits[i] = 0;
			 pdUpperLimits[i] = 1;
		}

		m_pdLowerLimits[i] = pdLowerLimits[i];
		m_pdUpperLimits[i] = pdUpperLimits[i];
		m_pnBinNum[i] = pnBinNum[i];
		m_pdBinStep[i] = (pdUpperLimits[i]-pdLowerLimits[i])/pnBinNum[i];
	}
	AllocateMemory4Matrices();
	SetContinous(true);//not miss any bins
	SetBinNoAtEachCell();
	return 0;
}
// build bins from samples and given min step and max steps
// must add samples first
int CAdaptHis::buildFromSamplesAndMinStepAndMaxBinNum(double* pdMinStep, int* pnMaxBinNum)
{
	SetMinStep(pdMinStep);
	SetMaxBinNum(pnMaxBinNum);
	Finished();
	return 0;
}

///////////////////
// addSamples()
// add all of the samples in "his" to THIS his
// nothing else (not recalculate other properties)
// return samples added
int	CAdaptHis::addSamples(CAdaptHis &his)
{
	int nDim = his.GetDim();
	assert(nDim == GetDim());

	int nAddSampleNum = his.GetSampleNum();
	int nSampleNum = GetSampleNum();
	if (nSampleNum + nAddSampleNum > m_nMaxSampleNum)
	{// memory need to be realocated
		double *pdSamples = new double[(nSampleNum+nAddSampleNum)*m_nDim];
		memcpy(pdSamples, m_pdSamples, sizeof(double)*nSampleNum*m_nDim);
		m_nMaxSampleNum = nSampleNum+nAddSampleNum;
		delete []m_pdSamples;
		m_pdSamples = pdSamples;
	}
	memcpy(m_pdSamples + nSampleNum*nDim, his.GetPointer2Samples(), sizeof(double)*nAddSampleNum*nDim);
	m_nSampleNum = nSampleNum+nAddSampleNum;
	return nAddSampleNum;
}
///////////////////
// removeSamples()
// remove all of the samples in "his" to THIS his
// nothing else (not recalculate other properties)
// return new samples num after removed
int	CAdaptHis::removeSamples(CAdaptHis &his)
{
	int nDim = his.GetDim();
	assert(nDim == GetDim());

	int nOrgSampleNum = GetSampleNum();
	int nRemoveSampleNum = his.GetSampleNum();
	assert(nOrgSampleNum >= nRemoveSampleNum);
	double *pdRSamples = his.GetPointer2Samples();
	double *pdSamples = GetPointer2Samples();
	for (int i = 0; i < nRemoveSampleNum; i++)
	{//for each sample
		pdSamples = GetPointer2Samples();
		int nSampleNum = GetSampleNum();
		bool bFind = false;
		for (int j = 0; j < nSampleNum; j++)
		{
			if (fabs(*pdSamples - *pdRSamples) < SK_ALMOSTZERO)
			{ 
				bool bEqual = true;
				for (int d = 1; d < nDim; d++)
				{
					if (fabs(*(pdSamples+d) - *(pdRSamples+d)) > SK_ALMOSTZERO)
					{
						bEqual = false;
						break;
					}
				}
				if (bEqual)
				{
					for (int k = j; k < nSampleNum-1; k++)
						for (int d = 0; d < nDim; d++)
							*(pdSamples+(k-j)*nDim+d) = *(pdSamples+(k-j+1)*nDim+d);
					m_nSampleNum--;	
					bFind = true;
					break;
				}//if find
			}//if the first element is equal
			pdSamples += nDim;
		}// for j, all the samples
		assert(bFind);
		pdRSamples += nDim;
	}// for i, all the samples needed to be removed
	return nOrgSampleNum - nRemoveSampleNum;
}
///////////////////
// removeAllSamples()
// set m_nSampleNum = 0
// return previous sample num
int	CAdaptHis::removeAllSamples()
{
	int nSampleNum = GetSampleNum();
	m_nSampleNum = 0;
	return nSampleNum;
}

///////////////////
// randomSample()
// get one random sample from this adaptive histogram
// return the possiblity
double	CAdaptHis::randomSample(double *pdValue)
{
	double dProb = 1.0;
	int nBin = ::samplingFromProbability(m_pdHis, m_nBinNum, &dProb);
	Bin2Real(nBin, pdValue);
	return dProb;
}

///////////////////
// saveHistogram()
//	save the histogram to file
int CAdaptHis::saveHistogram(const char *name)
{
	FILE *pFile;
	pFile = fopen(name, "w");
	if (!pFile)
		return -1;
	saveHistogram(pFile);
	fclose(pFile);
	return 0;
}
///////////////////
// saveHistogram()
//	save the histogram to file
//	for 1D histogram, show all the hist
//	for 2D or more, only show those with samples
int CAdaptHis::saveHistogram(FILE *pFile)
{
	int nBinNum = GetBinNum();
	int nDim = GetDim();
	double dValue;

	// center value
	int nOffset;
	int d, i;
	int ii = 0;
	for (i = 0; i < nBinNum; i++)
	{//print index
		if (m_pnPointSumOfEachCell[i] > 0 || nDim == 1)
		{
			fprintf(pFile, "%d ", ii);
			ii++;
		}
	}
	fprintf(pFile, "\n");
	for (d = 0; d < nDim; d++)
	{//print center
		for (i = 0; i < nBinNum; i++)
		{
			if (m_pnPointSumOfEachCell[i] > 0 || nDim == 1)
			{
				nOffset = i*nDim + d;
				dValue = m_pdEachBinCenter[nOffset];
				fprintf(pFile, "%.5f ", dValue);
			}
		}
		fprintf(pFile, "\n");
	}
	int n;
	for (i = 0; i < nBinNum; i++)
	{//print sample number in each bin
		if (m_pnPointSumOfEachCell[i] > 0 || nDim == 1)
		{
			n = m_pnPointSumOfEachCell[i];
			fprintf(pFile, "%d ", n);
		}
	}
	fprintf(pFile, "\n");
	for (i = 0; i < nBinNum; i++)
	{//print percentage
		if (m_pnPointSumOfEachCell[i] > 0 || nDim == 1)
		{
//			n = m_pnPointSumOfEachCell[i];
//			fprintf(pFile, "%.4f ", (double)n/m_nSampleNum);
			fprintf(pFile, "%.7f ", m_pdHis[i]);
		}
	}
	fprintf(pFile, "\n");
	return 0;
}

///////////////////
// changePointOfCell()
//	not add the value as sample, only counting by change the points in cells
int  CAdaptHis::changePointOfCell(int nBin, int flag)
{
	assert(nBin == m_pnBinNoOfEachCell[nBin]);//not sparse histogram considered
	if (flag == ADD)
	{
		m_nSampleNum++;
		m_pnPointSumOfEachCell[nBin]++;
	}
	else if (flag == REMOVE)
	{
		m_nSampleNum--;
		m_pnPointSumOfEachCell[nBin]--;
	}
	else
		assert(false);
	assert(m_nSampleNum >= 0);
	assert(m_pnPointSumOfEachCell[nBin] >= 0);
	return m_pnPointSumOfEachCell[nBin];
}

///////////////////
// changePointOfCell()
//	not add the value as sample, only counting by change the points in cells
int  CAdaptHis::changePointOfCell(double *pdValue, int flag)
{
	int nBin = Real2Bin(pdValue);
	return changePointOfCell(nBin, flag);
}
///////////////////
//	histogramDifference()
double CAdaptHis::histogramDifference(CAdaptHis *pHis)
{
	double dDiff = 0;
	assert(m_nDim == pHis->GetDim());
	assert(m_nBinNum == pHis->GetBinNum());
	double sum;
	double sub;
	for (int i = 0; i < m_nBinNum; i++)
	{
		sum = m_pdHis[i] + *(pHis->getHis() + i);
		if (sum != 0)
		{
			sub = m_pdHis[i] - *(pHis->getHis() + i);
			dDiff += sub*sub/sum;
		}
	}
	return dDiff / 2;
}	
///////////////////
//	histogramProduct()
double CAdaptHis::histogramProduct(CAdaptHis *pHis)
{
//	assert(m_nDim == pHis->GetDim());
//	assert(m_nBinNum == pHis->GetBinNum());
	double sum = 0;
	double *h = pHis->getHis();

		double pdTemp1[512], pdTemp2[512];

	for (int i = 0; i < m_nBinNum; i++)
	{
			pdTemp1 [i] = m_pdHis[i];
			pdTemp2 [i] = h[i];

		sum += m_pdHis[i] * h[i];
	}
	return sum;
}	
///////////////////
//	computeAbsWeightCenter()
double CAdaptHis::computeAbsWeightCenter()
{
	double sum = 0;
	int nBinNum = GetBinNum();

	for (int i = 0; i < nBinNum; i++)
	{
		sum += fabs(i-nBinNum/2.f)*m_pdHis[i];
	}
	sum /= nBinNum;
	m_dAbsWeightCenter = sum;
	return m_dAbsWeightCenter;
}

///////////////////
//	computeKurtosis()
double CAdaptHis::computeKurtosis()
{
	double sum2 = 0;
	double sum4 = 0;
	double kur;
	int nBinNum = GetBinNum();

	for (int i = 0; i < nBinNum; i++)
	{
		sum2 += (i-nBinNum/2)*(i-nBinNum/2)*m_pdHis[i];
		sum4 += pow((i-nBinNum/2.f),4.f) * m_pdHis[i];
	}
	if (sum2 != 0)
		kur = sum4/sum2*sum2 - 3;
	else
		kur = 1;
	m_dKurtosis = kur;
	return m_dKurtosis;
}
