#include <direct.h>
#include "skBasePursuit.h"
#include "skMatrixIplOp.h"
#include "skOutput.h"
#include "skNbrManager.h"
#include "skSketchGraph.h"
#include "skPyramid.h"
#include "skSketchRefine.h"

#include "skCurve.h"
#include "skIOImage.h"

/////////////////////////////////////////////////////////////////////
// class SkCompEntropyParam
SkCompEntropyParam::SkCompEntropyParam()
{
	m_nScale = 1;
	m_dNoBasePrior = 0.99;
}
SkCompEntropyParam::~SkCompEntropyParam()
{
}
int SkCompEntropyParam::read(const char *name)
{
	FILE *pFile;
	pFile = fopen(name, "r");
	if (pFile == 0)
		return 1;
	
	fscanf(pFile, "Scale (1,2,3):      %d\n", &m_nScale);// the scale of filters
	fscanf(pFile, "NoBasePrior:        %lf\n", &m_dNoBasePrior);// the prior for no bases
	
	fclose(pFile);
	return 0;
}
int SkCompEntropyParam::save(const char *name)
{
	FILE *pFile;
	pFile = fopen(name, "w");
	if (pFile == 0)
		return 1;
	
	fprintf(pFile, "Scale (1,2,3):      %d\n", m_nScale);// the scale of filters
	fprintf(pFile, "NoBasePrior:        %.6f\n", m_dNoBasePrior);// the prior for no bases

	fclose(pFile);
	return 0;
}

/////////////////////////////////////////////////////////////////////
// class SkBasePursuitParam
SkBasePursuitParam::SkBasePursuitParam()
{
	initialize(0);
}
int SkBasePursuitParam::initialize(int nSelect)
{	// sketching pursuit
	
	m_nBuildPyramid = 0;// whether build the Lapalacian paramid, and operate on the whitened image
	m_dPyramidKernalSize = 3;//the Gaussian Kernal size to build the Lapalacian Pyramid
	m_nMaxBaseNum = 20000;//the max base num
	m_dMinBaseCoefHi = 5.0;//the minimum base coef
	m_dMinBaseCoefLow = 3.0;//the minimum base coef
	m_nUpdateRes = 0;//whether to update the residual bases
	m_nUseGestaltPrior = 1;//whether use Gestalt prior, 0-no, general matching pursuit, 1-yes, sketching pursuit
	m_nShowProcess = 0;//show the process
	m_nMaxProfileWd = 7;
	m_dInhibitionDis = 3;
	m_bUseRes4Prfl = 1;
	m_nConsiderEntropy = 0;
	m_nSaveMaxCoef = 0;
	m_nSaveBasePatches = 0;

	switch (nSelect)
	{
	case 0://original image
		m_nBuildPyramid = 0;// whether build the Lapalacian paramid, and operate on the whitened image
		m_dMinBaseCoefHi = 3.0;//the minimum base coef - hi
		m_dMinBaseCoefLow = 2.0;//the minimum base coef -low
		m_nMaxProfileWd = 7;
		m_dInhibitionDis = 7;
		m_nUseGestaltPrior = 0;//whether use Gestalt prior, 0-no, general matching pursuit, 1-yes, sketching pursuit
		break;
	case 1://whitened image
		m_nBuildPyramid = 1;// whether build the Lapalacian paramid, and operate on the whitened image
		m_dMinBaseCoefHi = 3;//the minimum base coef - hi
		m_dMinBaseCoefLow = 1;//the minimum base coef - low
		m_nMaxProfileWd = 7;
		m_dInhibitionDis = 7;
		m_nUseGestaltPrior = 1;//whether use Gestalt prior, 0-no, general matching pursuit, 1-yes, sketching pursuit
		m_dPyramidKernalSize = 3;//the Gaussian Kernal size to build the Lapalacian Pyramid
		m_nShowProcess = 0;//show the process
		m_nSaveMaxCoef = 1;

		m_nSaveBasePatches = 1;
		break;
	case 2://edge strength
		m_dMinBaseCoefHi = 4.0;//the minimum base coef
		m_dMinBaseCoefLow = 3.0;//the minimum base coef
		m_nMaxProfileWd = 13;
		m_dInhibitionDis = 13;
		break;
	case 3://entropy image
		m_dMinBaseCoefHi = 3.0;//the minimum base coef
		m_dMinBaseCoefLow = 3.0;//the minimum base coef
		break;
	case 4://matching pursuit
		m_nBuildPyramid = 0;// whether build the Lapalacian paramid, and operate on the whitened image
		m_nUseGestaltPrior = 0;//whether use Gestalt prior, 0-no, general matching pursuit, 1-yes, sketching pursuit
		m_dMinBaseCoefHi = 1.0;//the minimum base coef
		m_dMinBaseCoefLow =1.0;//the minimum base coef
		m_nUpdateRes = 1;//whether to update the residual bases
		m_nMaxProfileWd = 13;
		m_dInhibitionDis = 0;
		m_nShowProcess = 0;//show the process
		m_nSaveBasePatches = 0;
		break;
	case 5://stroke pursuit, use bars as bases
		m_nBuildPyramid = 1;// whether build the Lapalacian paramid, and operate on the whitened image
		m_nUseGestaltPrior = 0;//whether use Gestalt prior, 0-no, general matching pursuit, 1-yes, sketching pursuit
		m_dMinBaseCoefHi = 1.0;//the minimum base coef
		m_dMinBaseCoefLow = 1.0;//the minimum base coef
		m_nUpdateRes = 1;//whether to update the residual bases
		m_nMaxProfileWd = 8;
		m_dInhibitionDis = 0;
		break;
	default:
		assert(false);
		break;
	}

	return 0;
}

SkBasePursuitParam::~SkBasePursuitParam()
{
}

int SkBasePursuitParam::read(const char *name)
{
	FILE *pFile;
	pFile = fopen(name, "r");
	if (pFile == 0)
		return 1;
	fscanf(pFile, "BuildPyramid:      %d\n", &m_nBuildPyramid);// whether build the Lapalacian paramid, and operate on the whitened image
	fscanf(pFile, "PyramidKernalSize: %lf\n", &m_dPyramidKernalSize);//the Gaussian Kernal size to build the Lapalacian Pyramid
	fscanf(pFile, "MaxBaseNum:        %d\n", &m_nMaxBaseNum);//the max base num
	fscanf(pFile, "MinBaseCoefHi:     %lf\n", &m_dMinBaseCoefHi);//the minimum base coef, hi thresh
	fscanf(pFile, "MinBaseCoefLow:    %lf\n", &m_dMinBaseCoefLow);//the minimum base coef, low thresh
	fscanf(pFile, "UpdateResImg:      %d\n", &m_nUpdateRes);//whether to update the residual bases
	fscanf(pFile, "UseGestaltPrior:   %d\n", &m_nUseGestaltPrior);//whether use Gestalt prior, 0-no, general matching pursuit, 1-yes, sketching pursuit
	fscanf(pFile, "ShowProcess:       %d\n", &m_nShowProcess);//show the process
	fscanf(pFile, "nMaxProfileWd:     %d\n", &m_nMaxProfileWd);//the max profile width
	fscanf(pFile, "InhibitionDis:     %lf\n", &m_dInhibitionDis);//the distance of inhibition
	fclose(pFile);
	return 0;
}
int SkBasePursuitParam::save(const char *name)
{
	FILE *pFile;
	pFile = fopen(name, "w");
	if (pFile == 0)
		return 1;
	fprintf(pFile, "BuildPyramid:      %d\n", m_nBuildPyramid);// whether build the Lapalacian paramid, and operate on the whitened image
	fprintf(pFile, "PyramidKernalSize: %.2lf\n", m_dPyramidKernalSize);//the Gaussian Kernal size to build the Lapalacian Pyramid
	fprintf(pFile, "MaxBaseNum:        %d\n", m_nMaxBaseNum);//the max base num
	fprintf(pFile, "MinBaseCoefHi:     %lf\n", m_dMinBaseCoefHi);//the minimum base coef, hi thresh
	fprintf(pFile, "MinBaseCoefLow:    %lf\n", m_dMinBaseCoefLow);//the minimum base coef, low thresh
	fprintf(pFile, "UpdateResImg:      %d\n", m_nUpdateRes);//whether to update the residual bases
	fprintf(pFile, "UseGestaltPrior:   %d\n", m_nUseGestaltPrior);//whether use Gestalt prior, 0-no, general matching pursuit, 1-yes, sketching pursuit
	fprintf(pFile, "ShowProcess:       %d\n", m_nShowProcess);//show the process
	fprintf(pFile, "nMaxProfileWd:     %d\n", m_nMaxProfileWd);//the max profile width
	fprintf(pFile, "InhibitionDis:     %.2f\n", m_dInhibitionDis);//the distance of inhibition
	fclose(pFile);
	return 0;
}

/////////////////////////////////////////////////////////////////////
// 
///////////////////
// skComputeEntropy()
int skComputeEntropy(const char *pchImgName)
{
	char pchProjName[512], name[512], pchDir[512], msg[512];

	sprintf(msg, "\n\nComputing Entropy on image %s ..............................", pchImgName);
	skOutput(msg);

	DWORD Time0 = ::GetTickCount();

	char pchDir0[512];
	_getcwd(pchDir0, 512);


	strcpy(pchProjName, pchImgName);
	char *temp;
	temp = strrchr(pchProjName, '.');//find '.', return the pointer to it
	*temp = '\0';


	Matrix<float> imgOrg;
	int nReturn;
	/*
	nReturn = skReadBitmap(pchImgName, &imgOrg);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", pchImgName);
		skOutput(msg);
		return 1;
	}
*/
	Matrix<int> pImgChn[3];
	nReturn = ::skLoadImage2YCrCb(pchImgName, pImgChn);

	int ht, wd;
	pImgChn[0].GetDimension(ht, wd);
	if (ht <= 0 || wd <= 0)
		return 1;

	::skCopyMatrix(pImgChn[0], imgOrg);

	_mkdir(pchProjName);
	_chdir(pchProjName);


//	skComputeDerivativeEntropy(pImgChn[0], pchProjName);
//	return 0;


//	nReturn = skSaveMatrixAsBitmap(pchImgName, imgOrg, 0, 1);
	::skSaveImageFromYCrCb(pchImgName, pImgChn, 1);

	sprintf(pchDir, "%s\\%s", pchDir0, pchProjName);

	double dImgMean0 = 0, dImgStd0 = -1;
	skNormalizeMatrix2ZeroMean1Std(&imgOrg, dImgMean0, dImgStd0);//-1 std will lead to find the mean and std

	skComputeEntropyImage(imgOrg, dImgMean0, dImgStd0, pchDir, pchProjName);

	_chdir(pchDir0);

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "\n Total Computing Entropy time for image %s:   %d min %.3fs \n ", pchImgName, nMin, dSec);
	::skOutput(msg);
	return 0;
}

int skComputeEntropyImage(Matrix<float> &imgOrg,//original image
								   double dImgMean0, double dImgStd0,//original image mean and std
								   const char* pchDir, // working directory
								   const char* pchProjName)
{
	char name[512];
	SkCompEntropyParam param;
	sprintf(name, "%s_entropy_param.txt", pchProjName);
	param.save(name);

	double dImgMean, dImgStd;
	char skps_dir[32];
	sprintf(skps_dir, "skps");
	_mkdir(skps_dir);

	Matrix<float> *psImg;//the image which entropy works on
	SkPyramid pyrOrg;
	double m[2], s[2];
	pyrOrg.setLevels(2);
	pyrOrg.setGaussianKernalSize(3);
	pyrOrg.importImage(imgOrg);
	for (int i = 0; i < 2; i++)
	{
		sprintf(name, "%s\\%s_l%d.bmp", skps_dir, pchProjName, i);
		s[i] = dImgStd0 * pyrOrg.m_pStd[i];
		m[i] = dImgMean0 + pyrOrg.m_pMean[i]* dImgStd0;
		skSaveMatrixAsBitmap(name, pyrOrg.m_pImScale[i], m[i], s[i]);
	}
	dImgMean = m[1];
	dImgStd = s[1];
	psImg = &(pyrOrg.m_pImScale[1]);
//	psImg = &pyrOrg;

	sprintf(name, "%s\\%s_org.bmp", skps_dir, pchProjName);
	skSaveMatrixAsBitmap(name, pyrOrg, dImgMean0, dImgStd0);//test the correctness of pyramid

	
	char pchProjName1[512];
	sprintf(pchProjName1, "%s", pchProjName);
	SkFilterCoef fc;
	fc.setProjName(pchProjName1);

//	fc.setConsiderEntropy(true); //just for compute direction field
	fc.setConsiderEntropy(false); //must be true for compute entroy, just for compute direction field, set if false
	fc.setSaveMaxCoef(true);
	
	fc.setNoBasePrior(param.m_dNoBasePrior);

//	fc.setMatrixImage(&imgOrg); 
	fc.setMatrixImage(psImg); 

	SkFilterBank filterBank, *pFB = &filterBank;
	char pchType[32];
	for (int i = 1; i <= 1; i++)
	{
		sprintf(name, "%s_entropy_filter_param_s%d.txt", pchProjName, i);
//	if (pFB->readParameters(name) != 0)
		pFB->m_param.initilaize2ComputeEntropy(i);

//		pFB->saveParameters(name);//no such parameter file, save the default
		pFB->buildFilters();
		sprintf(pchType, "comp_ent_s%d", i);
		pFB->saveFiltersAsBmp(pchType);

		fc.setFilterBank(pFB);
		fc.setEntropyScale(i);
		fc.initializeCoefficencies();
	}
//	fc.saveLowHighEntropyPatches(dImgMean0, dImgStd0);

//	fc.saveLowHighEntropyPatches(dImgMean, dImgStd);
	
	return 0;
}

double skLocalEntropy(Matrix<int> &M, int r0, int c0, int nBinNum, int nHalfWnd)
{
	int ht, wd;
	M.GetDimension(ht, wd);

	double pdHist[1024];
	assert(nBinNum <= 1024);
	int i;
	for (i = 0; i < nBinNum; i++)
		pdHist[i] = 0;

	int r, c, dr, dc;
	int v;
	int nPt = 0;
	for (dr = -nHalfWnd; dr <= nHalfWnd; dr++)
	{
		r = r0 + dr;
		if (r < 0 || r >= ht)
			continue;
		for (dc = -nHalfWnd; dc <= nHalfWnd; dc++)
		{
			c = c0 + dc;
			if (c < 0 || c >= wd)
				continue;
			v = M(r,c);
			if (v < 0) 
				v = 0;
			else if (v >= nBinNum -1)
				v = nBinNum -1;

			pdHist[v]++;
			nPt++;
		}
	}
	assert(nPt > 0);
	for (i = 0; i < nBinNum; i++)
		pdHist[i] /= nPt;
	double dEntropy = skComputeEntropy(pdHist, nBinNum);
	return dEntropy;
}
int skComputeDerivativeEntropy(Matrix<int>& img, const char *proj)
{
	Matrix<int> DX, DY;
	Matrix<int> sum_DXY;
	Matrix<float> ent;

	int ht, wd;
	img.GetDimension(ht, wd);

	ent.SetDimension(ht, wd);

	int nBinNum = 16;
	int nHalfWnd = 3;

	//compute DX and DY
	DX =  skDirectiveX(img);
	DY =  skDirectiveY(img);

	//DY.Set(0);

	sum_DXY = skSqureRootOfSqureSum(DX, DY);

	double max_DXY = sum_DXY.GetMax();
	int r, c;

	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			sum_DXY(r,c) = (int)(sum_DXY(r,c) * nBinNum/ (max_DXY+1));
		}
	}

	max_DXY = sum_DXY.GetMax();
	// compute local histogram

	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			ent(r,c) =  skLocalEntropy(sum_DXY, r, c, nBinNum, nHalfWnd);
		}
	}
	// save results
	char dir[512], name[512];
	sprintf(dir, "%s\\edge", proj);
	_mkdir(dir);

	sprintf(name, "%s\\%s_deriv_ent.bmp", dir, proj);
	::skSaveMatrixAsBitmap(name, ent, 0, -1);

	sprintf(name, "%s_deriv_ent.bmp", proj);
	::skSaveMatrixAsBitmap(name, ent, 0, -1);
	return 0;
}


int skSaveEntropyPatchWithScale(FILE *pFile)
{
	DWORD Time0 = ::GetTickCount();

	char pchDir0[512], msg[512], name[512], dir[512], pchX[32], pchY[32], patch_dir[64];
	_getcwd(pchDir0, 512);

	sprintf(patch_dir, "patches");
	_mkdir(patch_dir);
	char scale_en[512], scale_en_avg[512];
	sprintf(scale_en, "%s\\scale_entropy.txt", patch_dir);
	sprintf(scale_en_avg, "%s\\scale_entropy_avg.txt", patch_dir);

	sprintf(msg, "\n\nSave Entropy Patch With Scale ..............................");
	skOutput(msg);

	char pchImgName[512];
	Matrix<double> entropy;
	Matrix<float> img;
#define POS_NUM 4
#define SCALE_NUM 7
	int f = 7;
	Matrix<float> M(2*f+1, 2*f+1);

	int x0[POS_NUM], y0[POS_NUM], x, y;

	double ppdEn[SCALE_NUM][POS_NUM];

	Matrix<float> minEnFlag, minEnValue;
	int nNumLocalMinEn = 0;

	int ht, wd, ht0, wd0;
	int i, j;//index of scale and position
	int r, c, dr, dc;
	int r0, c0, rr, cc;

	for (i = 0; i < SCALE_NUM; i++)
	{
		sprintf(dir, "leaves001_01_s%d", i);
		sprintf(name, "%s\\leaves001_01_s%d_entropy_s1.txt", dir, i);
		skReadMatrix(name, entropy);
		sprintf(name, "%s\\leaves001_01_s%d.bmp", dir, i);
		skReadBitmap(name, &img);
		entropy.GetDimension(ht, wd);
		if (i == 0)
		{//select 4 points
			x0[0]=wd/4 + 100;
			y0[0]=ht/4 - 100;
			x0[1]=wd/4 + 100; 
			y0[1]=ht*3/4 - 100;
			x0[2]=wd*3/4 + 100; 
			y0[2]=ht/4 - 100;
			x0[3]=wd*3/4 + 100; 
			y0[3]=ht*3/4 - 100;

			ht0 = ht;
			wd0 = wd;

			double min_en;
			int min_dr, min_dc;

			int r, c, dr, dc;
			int r0, c0, rr, cc;
			for (j = 0; j < POS_NUM; j++)
			{
				rr = y0[j];
				cc = x0[j];
				while (true)
				{
					min_dr = 0;
					min_dc = 0;
					min_en = 2.0;
					for (dr = -f; dr <= f; dr++)
					{
						r0 = rr + dr;
						if (r0 < 0 || r0 >= ht)
							continue;
						for (dc = -f; dc <= f; dc++)
						{
							c0 = cc + dc;
							if (c0 < 0 || c0 >= wd)
								continue;
							if (min_en > entropy(r0, c0))
							{
								min_en = entropy(r0, c0);
								min_dr = dr;
								min_dc = dc;
							}
						}
					}
					assert(min_dr >= -f && min_dc >= -f); 
					if (min_dr == 0 && min_dc == 0)
						break;
					rr = rr + min_dr;
					cc = cc + min_dc;
				}
				if (rr < 2*f || rr >= ht-2*f || cc < 2*f || cc >= wd - 2*f)
				{//cross the boundares
					assert(false);
					x0[j] = -1; 
					y0[j] = -1;
				}
				else
				{
					x0[j] = cc;
					y0[j] = rr;
				}
			}//for j

			// consider all local minima
			minEnFlag.SetDimension(ht, wd);
			minEnFlag.Set(1);

			for (r = f; r < ht; r += f)
			{
				for (c = f; c < wd; c += f)
				{
					rr = r;
					cc = c;
					while (true)
					{
						min_dr = 0;
						min_dc = 0;
						min_en = 2.0;
						for (dr = -f; dr <= f; dr++)
						{
							r0 = rr + dr;
							if (r0 < 0 || r0 >= ht)
								continue;
							for (dc = -f; dc <= f; dc++)
							{
								c0 = cc + dc;
								if (c0 < 0 || c0 >= wd)
									continue;
								if (min_en > entropy(r0, c0))
								{
									min_en = entropy(r0, c0);
									min_dr = dr;
									min_dc = dc;
								}
							}
						}
						assert(min_dr >= -f && min_dc >= -f); 
						if (min_dr == 0 && min_dc == 0)
							break;
						rr = rr + min_dr;
						cc = cc + min_dc;
					}
					if (rr < 2*f || rr >= ht-2*f || cc < 2*f || cc >= wd - 2*f)
						continue;//cross the boundares

					if (min_en > 0.02)
						continue;

					minEnFlag(rr,cc) = 0;
					nNumLocalMinEn++;
				}
			}

			minEnValue.SetDimension(nNumLocalMinEn, SCALE_NUM);
			sprintf(msg, "\n      Total counted local entropy min: %d", nNumLocalMinEn);
			skOutput(msg);

		}//if i == 0

		int nn = 0;
		double dSumEn = 0;
		for (r0 = 0; r0 < ht0; r0++)
		{
			for (c0 = 0; c0 < wd0; c0++)
			{
				if (minEnFlag(r0, c0) != 0)
					continue;
				r = r0/pow(2.f, i);
				c = c0/pow(2.f, i);
				minEnValue(nn, i) = entropy(r, c);
				nn++;
				dSumEn += entropy(r, c);
			}
		}
		assert(nn > 0);
		double avg_en = dSumEn / nn;

		if (i == 0)
			::saveArray(&avg_en, 1, scale_en_avg);
		else
			::appendArray(&avg_en, 1, scale_en_avg);


		for (j = 0; j < POS_NUM; j++)
		{
			x = x0[j]/pow(2.f, i);
			y = y0[j]/pow(2.f, i);
			ppdEn[i][j] = entropy(y, x);
	
			::skCopyImage(&img, &M, x-f, y-f);
			::changeNumTo3Char(pchX, x);
			::changeNumTo3Char(pchY, y);
//			sprintf(name, "leaves001_01_s%d_pos%d_x%s_y%s.bmp", i, j, pchX, pchY);
			sprintf(name, "%s\\leaves001_01_s%d_pos%d.bmp", patch_dir, i, j);
			::skSaveMatrixAsBitmap(name, M, 0, 1);
		}
		if (i == 0)
			::saveArray(ppdEn[i], POS_NUM, scale_en);
		else
			::appendArray(ppdEn[i], POS_NUM, scale_en);



	}//for i;

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "\n Total time for Saving Entropy Patch With Scale:   %d min %.3fs \n ", nMin, dSec);
	::skOutput(msg);

	_chdir(pchDir0);
	return 0;
}

///////////////////
// skSketchPursuit()
int skSketchPursuit(const char *pchImgName, int nSelect)
{
	char pchProjName[512], name[512], pchDir[512], msg[512];

	sprintf(msg, "\n\nSketching pursuit on image %s ..............................", pchImgName);
	skOutput(msg);

	DWORD Time0 = ::GetTickCount();

	char pchDir0[512];
	_getcwd(pchDir0, 512);


	strcpy(pchProjName, pchImgName);
	char *temp;
	temp = strrchr(pchProjName, '.');//find '.', return the pointer to it
	*temp = '\0';

	//load image
	//read original image
	Matrix<int> pImgChn[3];
	::skLoadImage2YCrCb(pchImgName, pImgChn);

	Matrix<float> imgOrg;
	int nReturn;
/*
	nReturn = skReadBitmap(pchImgName, &imgOrg);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", pchImgName);
		skOutput(msg);
		return 1;
	}
*/
	::skCopyMatrix(pImgChn[0], imgOrg);

	_mkdir(pchProjName);
	_chdir(pchProjName);

	nReturn = skSaveMatrixAsBitmap(pchImgName, imgOrg, 0, 1);

	sprintf(pchDir, "%s\\%s", pchDir0, pchProjName);
	// save the results
	char skps_dir[32];
	sprintf(skps_dir, "skps");
	_mkdir(skps_dir);

	int wd, ht;
	imgOrg.GetDimension(ht, wd);
	// normalize the image to 0 mean 1 variance
	double dImgMean0 = 0, dImgStd0 = -1;
	skNormalizeMatrix2ZeroMean1Std(&imgOrg, dImgMean0, dImgStd0);//-1 std will lead to find the mean and std


	skSketchPursuitOnSelectedImage(imgOrg, dImgMean0, dImgStd0, skps_dir, pchDir, pchProjName, nSelect);
	
	_chdir(pchDir0);

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "\n Total sketching pursuit time for image %s:   %d min %.3fs \n ", pchImgName, nMin, dSec);
	::skOutput(msg);
	return 0;
}

int skSketchPursuitOnSelectedImage(Matrix<float> &imgOrg,//original image
								   double dImgMean0, double dImgStd0,//original image mean and std
								   const char* skps_dir,//sketching pursuit working directory
								   const char* pchDir, // working directory
								   const char* pchProjName,//project name
								   int nSelect)//0-original, 1-whitened, 2-edge strength, 3-entropy image
{
	//-------------------------------
	// sketching pursuit on original image

	int ht, wd;
	imgOrg.GetDimension(ht, wd);
	// write or read paramters
	SkBasePursuitParam param;
	char select_name[32], name[512], msg[512];
	Matrix<float> *pImg, img; 
	double dImgMean=dImgMean0, dImgStd=dImgStd0;
	//0-org, 1-whitened image, 2-edge strength, 3-entropy, 4-matching pursuit, 5-stroke pursuit
	switch (nSelect)
	{
	case 0://org
		sprintf(select_name, "skps_org");
//		sprintf(select_name, "0");
		pImg = &imgOrg;
		break;
	case 1://whitened image
		sprintf(select_name, "skps_wht");
//		sprintf(select_name, "1");
		pImg = &imgOrg;
		break;
	case 2://edge strength
		sprintf(select_name, "skps_sth");
//		sprintf(select_name, "2");
		sprintf(name, "%s\\%s_sth_edge_s2.bmp", pchDir, pchProjName);
		if (skReadBitmap(name, &img)  == 0)
		{
			dImgMean = 0;
			dImgStd = -1;
			skNormalizeMatrix2ZeroMean1Std(&img, dImgMean, dImgStd);//-1 std will lead to find the mean and std
			double dMin = img.GetMin();
			img = img - dMin;
			pImg = &img;
		}
		else
		{
			skOutput("\n No edge strength image found !!!!!\n");
			return -1;
		}

		break;
	case 3://entropy
		sprintf(select_name, "skps_ent");
//		sprintf(select_name, "3");
		sprintf(name, "%s\\%s_entropy.bmp", pchDir, pchProjName);
		if (skReadBitmap(name, &img)  == 0)
		{
			dImgMean = 0;
			dImgStd = -1;
			skNormalizeMatrix2ZeroMean1Std(&img, dImgMean, dImgStd);//-1 std will lead to find the mean and std
			pImg = &img;
		}
		else
		{
			skOutput("\n No entropy image found !!!!!\n");
			return -1;
		}
		break;
	case 4://base pursuit, = matching pursuit, no spatial prior
		sprintf(select_name, "bsps_org");
//		sprintf(select_name, "4");
		pImg = &imgOrg;
		break;
	case 5://stroke pursuit, matching pursuit with bar bases, no spatial prior
		sprintf(select_name, "stps_img");
//		sprintf(select_name, "5");
		pImg = &imgOrg;
		break;
	default:
		assert(false);
		break;
	}

	sprintf(msg, "\n Sketching pursuit on %s ........................ \n", select_name);
	skOutput(msg);

	char pchProjName1[512];
	sprintf(pchProjName1, "%s_%s", pchProjName, select_name);
	sprintf(name, "%s\\%s_param.txt", pchDir, pchProjName1);
	param.initialize(nSelect);
	if (param.read(name) != 0)
	{
		param.m_nMaxBaseNum = ht*wd;
		param.save(name);//if read fails, then save the default
	}
	sprintf(param.m_pchFilterParamName, "%s\\%s_filter_param.txt", pchDir, pchProjName1);

	// construct filters and compute the filter responses
	DWORD Time0 = ::GetTickCount();
	SkFilterBank filterBank, *pFB = &filterBank;
	pFB->m_param.initilaize4BasePursuit(nSelect);
	if (pFB->readParameters(param.m_pchFilterParamName) != 0)
		pFB->saveParameters(param.m_pchFilterParamName);

		SkFilterBank::m_dLPNorm = 2;//L2 norm
	pFB->saveParameters(param.m_pchFilterParamName);//no such parameter file, save the default
	pFB->buildFilters();
	pFB->saveFiltersAsBmp(select_name);

//	return 0;

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "\n Build and save filter bank Time: %d min %.3fs \n ", nMin, dSec);
	::skOutput(msg);

	skSketchPursuit(*pImg, param, dImgMean, dImgStd, pFB, pchDir, skps_dir, pchProjName1);

	sprintf(msg, "\n Sketching pursuit on %s ........................ done \n", select_name);
	skOutput(msg);

	return 0;
}


///////////////////
// skSketchPursuit()
int skSketchPursuit(Matrix<float> &imgOrg,//orignal image
					SkBasePursuitParam &param, //sketching pursuit parameters
					double dImgMean0, double dImgStd0, //image mean and std
					SkFilterBank* pFB, //filter bank
					const char *pchDir, //working dir
					const char *skps_dir, //sketching pursuit result dir
					const char *pchProjName)//project name
{
	char name[512], msg[512];

	int wd, ht;
	imgOrg.GetDimension(ht, wd);

	double dImgMean = dImgMean0;
	double dImgStd = dImgStd0;

	Matrix<float> *psImg;//the image which the basepursuit work on
	SkPyramid pyrOrg;
	double m[2], s[2];
	if (param.m_nBuildPyramid)
	{
		pyrOrg.setLevels(2);
		pyrOrg.setGaussianKernalSize(param.m_dPyramidKernalSize);
		pyrOrg.importImage(imgOrg);
		for (int i = 0; i < 2; i++)
		{
			sprintf(name, "%s\\%s_l%d.bmp", skps_dir, pchProjName, i);
			s[i] = dImgStd0 * pyrOrg.m_pStd[i];
			m[i] = dImgMean0 + pyrOrg.m_pMean[i]* dImgStd0;
			skSaveMatrixAsBitmap(name, pyrOrg.m_pImScale[i], m[i], s[i]);
		}
		dImgMean = m[1];
		dImgStd = s[1];
		psImg = &(pyrOrg.m_pImScale[1]);

		sprintf(name, "%s\\%s_org.bmp", skps_dir, pchProjName);
		skSaveMatrixAsBitmap(name, pyrOrg, dImgMean0, dImgStd0);//test the correctness of pyramid
	}
	else
	{
		psImg = &imgOrg;
		sprintf(name, "%s\\%s_org.bmp", skps_dir, pchProjName);
		skSaveMatrixAsBitmap(name, imgOrg, dImgMean0, dImgStd0);
	}

	DWORD Time0 = ::GetTickCount();

	SkFilterCoef orgFC;
	orgFC.setProjName(pchProjName);
	orgFC.setConsiderEntropy(param.m_nConsiderEntropy);
	orgFC.setSaveMaxCoef(param.m_nSaveMaxCoef);//save the max coef
//	orgFC.setImageSigma(1.0);
	orgFC.setFilterBank(pFB);
	orgFC.setMatrixImage(psImg); 
	orgFC.setBlockDivideByBlockSize(100);
	orgFC.initializeCoefficencies();
//	orgFC.saveLowHighEntropyPatches(dImgMean0, dImgStd0);

	//orgFC.saveLowHighEntropyPatches(dImgMean, dImgStd);//not save


	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "\n Convolution Time: %d min %.3fs \n ", nMin, dSec);
	::skOutput(msg);

	//	return 0;
	//---------------------------------------
	// bottom up bases
	SkBaseMap baseMap;
	baseMap.setDimension(wd, ht);

	Matrix<float> imgRes, imgRec;
	imgRes = *psImg;


//	sprintf(name, "%s\\%s\\%s_skps.bsm", pchDir, skps_dir, pchProjName);
//	if (baseMap.read(name) != 0)
//	{//bug!! - neighborhood was not saved

		skBottomUpBases(&orgFC, &baseMap, *psImg, imgRes, dImgMean, dImgStd, param, pchProjName);
//	}
//	else
//	{
//		baseMap.setPointer2FilterBank(pFB);
//	}

	skSynthesizeImgFromBases(&baseMap, &imgRec, pFB, KER);
	sprintf(name, "%s\\%s_skps_rec.bmp",skps_dir, pchProjName);
	skSaveMatrixAsBitmap(name, imgRec, dImgMean, dImgStd);

//	sprintf(name, "%s_syn1.bmp", pchProjName);
//	skSaveMatrixAsBitmap(name, imgRec, dImgMean, dImgStd);

	imgRes = imgOrg - imgRec;
	sprintf(name, "%s\\%s_skps_res.bmp", skps_dir, pchProjName);
	skSaveMatrixAsBitmap(name, imgRes, dImgMean, dImgStd);

//	sprintf(name, "%s_res1.bmp", pchProjName);
//	skSaveMatrixAsBitmap(name, imgRes, dImgMean, dImgStd);

	skSynthesizeImgFromBases(&baseMap, &imgRec, pFB, SMB);
	sprintf(name, "%s\\%s_skps_smb.bmp", skps_dir, pchProjName);
	skSaveMatrixAsBitmap(name, imgRec, 255, 255);

	sprintf(name, "%s\\%s_skps.bsm", skps_dir, pchProjName);
	baseMap.save(name);


//	::skSaveMatrixAsBitmap("org.bmp", imgOrg, 0, -1);
//	::skSaveMatrixAsBitmap("wht.bmp", *psImg, 0, -1);


	// to learn pathches
	if (param.m_nSaveBasePatches)
	{
		int patch_ht = 13;
		int patch_wd = 13;
		skSavePatchesFromBaseMap(&baseMap, pchProjName, patch_ht, patch_wd, imgOrg, *psImg);
	}

	// save the errors of patch synthesizing
	sprintf(name, "%s\\%s_sp_err.txt", skps_dir, pchProjName);
	skSaveResidualErrorPatchByPatch(name, &baseMap, psImg, pFB, dImgMean,dImgStd, pchProjName);

	return 0; //no build graph


	//save the constraints and profiles
	SkSketchGraph sketchGraph;

	sketchGraph.setImgDim(wd, ht);
	sketchGraph.setImgMean(dImgMean);
	sketchGraph.setImgStd(dImgStd);
	sketchGraph.setBaseMap(&baseMap);
	sketchGraph.setImgOrg(&imgOrg);
//	sketchGraph.setImgOrg(psImg);
	sketchGraph.setInhibitionDis(param.m_dInhibitionDis);
	sketchGraph.setMaxProfileWd(param.m_nMaxProfileWd);
	sketchGraph.setProjName(pchProjName);
	sketchGraph.setShowProcess(param.m_nShowProcess);
	sketchGraph.setProcessSteps(baseMap.getSize());
	int nScaleImgOrg = 1;
	sketchGraph.setScaleImgOrg(nScaleImgOrg);
	sketchGraph.setUseRes4Prfl(param.m_bUseRes4Prfl);

	
	sprintf(name, "%s\\%s_sp_err.txt", skps_dir, pchProjName);
	sketchGraph.buildSketchGraphFromNbredBaseMap(&baseMap, name);// also save the residual image error for each sketch
//	sketchGraph.computeReconstructedAndResidualImage();

	int line_wd;

	line_wd= 1;
	skSaveSketchGraph(sketchGraph, "w1_1", pchProjName, skps_dir, wd, ht, line_wd);
	line_wd= 3;
	skSaveSketchGraph(sketchGraph, "1", pchProjName, skps_dir, wd, ht, line_wd);
	
	sprintf(name, "%s\\%s\\%s_1_res.bmp", pchDir, skps_dir, pchProjName);
	skSaveMatrixAsBitmap(name, sketchGraph.m_imgRes, dImgMean0, dImgStd0);
	
//	skSaveProfilesAsBases(sketchGraph, pchProjName);

	const char *temp;
	temp = strrchr(pchDir, '\\');//find '.', return the pointer to it
	sprintf(name, "%s_1.tkn", temp+1);
	skSaveSketchGraph2CurveMapFile(name, sketchGraph);

	sprintf(name, "%s.tkn", temp+1);
	skSaveSketchGraph2CurveMapFile(name, sketchGraph);

	if (param.m_nUseGestaltPrior == 0)
		return 0;//no refinement needed for matching pursuit

//	return 0;//no refinement

	SkSketchRefineParam skrf_param;
	sprintf(name, "%s\\%s_skrf_param.txt", pchDir, pchProjName);
	skrf_param.read(name);
	skrf_param.save(name);//if read fails, then save the default
	
	skSketchRefine(sketchGraph, skrf_param, imgOrg, dImgMean0, dImgStd0, pchProjName, pchDir, skps_dir);

	temp = strrchr(pchDir, '\\');//find '.', return the pointer to it
	sprintf(name, "%s.tkn", temp+1);
	skSaveSketchGraph2CurveMapFile(name, sketchGraph);



	return 0;

}

///////////////////
// skBottomUpBases()
//	This is a matching pursuit algorithm with the choice of pursuing with prior (sketching pursuit)
//	For matching pursuit: the order of bases which is pursued is dependent only on its coef, repeat below (1)
//	For skeching pursuit: the order of bases which is pursued is dependent only on its coef, 
//							but also on the prior of its nbrs, execute below (1)-(4)
// 1). bottom-up the first base( with biggest coef)
// 2). search possible addressed nbrs, if its coef greater than dMinBaseCoef, then add this base
// 3). for the new bases, search its possible addressed nbrs, too. 
//	   Repeat this process, until no existing bases has possible addressed nbrs
// 4). repeat 1-2-3 until no possible base has ceof greater than dMinBaseCoef
int skBottomUpBases(SkFilterCoef *pFC, //the FilterCoefs
					SkBaseMap *pBaseMap,//the base map to save all bases
					Matrix<float> &imgOrg, //orignal image, after noramlization, zero mean and unit variance
					Matrix<float> &imgRes, //residual image
					double dImgMean, double dImgStd,//original image mean and std
					SkBasePursuitParam &param,//parameters for the base pursuit
					const char *pchProjName)//the project name
{
	char dir[512], name[512];
	sprintf(dir, "skps");
	_mkdir(dir);

	int wd, ht;//image width, height
	imgOrg.GetDimension(ht, wd);
	imgRes = imgOrg;

	pBaseMap->setDimension(wd, ht);
	pBaseMap->removeAll();
	
	double dMinBaseCoefHi = param.m_dMinBaseCoefHi;
	double dMinBaseCoefLow = param.m_dMinBaseCoefLow;
	int nUseGestaltPrior = param.m_nUseGestaltPrior;
	int nUpdateRes = param.m_nUpdateRes;
	int nShowProcess = param.m_nShowProcess;
	int nMaxBaseNum = param.m_nMaxBaseNum;
	double dInhibitionDis = param.m_dInhibitionDis;

	Matrix<oval_IntArray> *pOcpMat = pBaseMap->getOccupyMatrix();
	SkFilterBank *pFB = pFC->getFilterBank();


//	SaveResults::saveGestaltPrior(this);
	DWORD Time0 = ::GetTickCount();

	SkBase base(pFB), base1(pFB);
	int id, idNbr;
	int nbrType = ADR_NBR;
	int n = 0, m;
	SkBaseNode *pNode;
	float fWeightedCoef = -1;
	int a = 0;//nCurSearAddNbr

	int nGraphNum = 0;

	if (!param.m_nUseGestaltPrior)
		dMinBaseCoefHi = dMinBaseCoefLow;//high thresh

	int nShowSketchStep = 1;

	double dDeltaCoef = dMinBaseCoefHi - dMinBaseCoefLow;
	double dMinBaseCoefHi_bak = dMinBaseCoefHi;
	double dMinBaseCoefLow_bak = dMinBaseCoefLow;

//	for (dMinBaseCoefHi = 6.0; dMinBaseCoefHi >= dMinBaseCoefHi_bak; dMinBaseCoefHi -= 1.0)
	{
//		dMinBaseCoefLow = dMinBaseCoefHi - dDeltaCoef;

		pFC->setAllBlocksUnvisited();

	SkIntegerStack idStack;
	while (true)
	{
		// find largest coef base
		a = 0;
		fWeightedCoef = pFC->findLargestCoefBase(base, pOcpMat, pBaseMap, dMinBaseCoefHi);
		if (fabs(base.a) < dMinBaseCoefHi || fWeightedCoef < 0.001)
			break;
		if (n >= nMaxBaseNum)
			break;
		nGraphNum++;
		// add this base to base map
		pFC->updateOneBase(base, ADD);
		id = pBaseMap->add(base);
		pNode = pBaseMap->getNodeAt(id);
		pBaseMap->updateOccupyMatrix(pNode, ADD, dInhibitionDis);
		if (nUpdateRes == 1)
			skUpdateResidualImage(base, imgRes, pFB, ADD);
		n++;
		if (nShowProcess)
			skSaveBaseMap2ShowBottomUpProcess(pBaseMap, "0", pchProjName, pFB, dImgMean, dImgStd);
//		if (!nUseGestaltPrior)
		{
			if (n/nShowSketchStep * nShowSketchStep ==n )
			{//save intermediate results
				Matrix<float> imgRec;
				char num[32];
				::changeNumTo4Char(num, n);
				skSynthesizeImgFromBases(pBaseMap, &imgRec, pFB, KER);
				sprintf(name, "%s\\%s_rec_%s.bmp", dir, pchProjName, num);
				skSaveMatrixAsBitmap(name, imgRec, dImgMean, dImgStd);

				imgRes = imgOrg - imgRec;
				sprintf(name, "%s\\%s_res_%s.bmp", dir, pchProjName, num);
				skSaveMatrixAsBitmap(name, imgRes, dImgMean, dImgStd);

				skSynthesizeImgFromBases(pBaseMap, &imgRec, pFB, SMB);
				sprintf(name, "%s\\%s_smb_%s.bmp", dir, pchProjName, num);
//				skSaveMatrixAsBitmap(name, imgRec, 255, 255);

				sprintf(name, "%s\\%s_%s.bsm", dir, pchProjName, num);
//				pBaseMap->save(name);
			}
		}
		if (!nUseGestaltPrior)
			continue;

		if (pFB->getType(base.k) == DOG)
			continue;//no nbr for DOG bases
		// add this base to base stack
		idStack.push(id);
		while (true)
		{
			// check base id stack
			id = skGetBaseIDNeedAdrNbr(idStack, a, pBaseMap);
			if (id < 0)
			{
				a++;
				if (a == MAX_ADR_NUM)
					break;
				continue;
			}

			pNode = pBaseMap->getNodeAt(id);
			idNbr = skBottomUpOneAdrNbrBaseWithPrior(a, id, pFC, pBaseMap, imgRes, pFB, param, idStack, dImgMean, dImgStd, pchProjName, dMinBaseCoefLow);
			if (idNbr >= 0)
			{
				a = 0;//re-search from first adr nbr
				n++;
				if (n/nShowSketchStep * nShowSketchStep ==n )
				{//save intermediate results
					Matrix<float> imgRec;
					char num[32];
					::changeNumTo4Char(num, n);
					skSynthesizeImgFromBases(pBaseMap, &imgRec, pFB, KER);
					sprintf(name, "%s\\%s_rec_%s.bmp", dir, pchProjName, num);
					skSaveMatrixAsBitmap(name, imgRec, dImgMean, dImgStd);

					imgRes = imgOrg - imgRec;
					sprintf(name, "%s\\%s_res_%s.bmp", dir, pchProjName, num);
//					skSaveMatrixAsBitmap(name, imgRes, dImgMean, dImgStd);

					skSynthesizeImgFromBases(pBaseMap, &imgRec, pFB, SMB);
					sprintf(name, "%s\\%s_smb_%s.bmp", dir, pchProjName, num);
//					skSaveMatrixAsBitmap(name, imgRec, 255, 255);

					sprintf(name, "%s\\%s_%s.bsm", dir, pchProjName, num);
//					pBaseMap->save(name);
				}
			}
//			::TakeABreath();
		}
	}//while true

	}// for high coef

	char msg[512];
	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "\n Base Pursuit Time: %d min %.3fs              Curve Num %d: \n ", nMin, dSec, nGraphNum);
	::skOutput(msg);

	// save the residual image error for each sketch
	sprintf(name, "%s\\%s_mp_err.txt", dir, pchProjName);
	skSaveResidualErrorBaseByBase(name, pBaseMap, &imgOrg, pFB, dImgMean, dImgStd, pchProjName);

	return 0;
}

///////////////////
// skGetBaseIDNeedAdrNbr()
//	from the base stack, search the first base which doesn't has the adr nbr 'a' yet
//	return the base id which has the adr nbr 'a' not checked
//	return -1 if no such base
int skGetBaseIDNeedAdrNbr(SkIntegerStack & stack, int a, SkBaseMap *pBaseMap)
{
	int n = stack.getSize();
	int id, i;
	int nbrType = ADR_NBR;
	SkBaseNode *pNode;
	for (i = n-1; i >= 0; i--)
	{
		id = stack.getAt(i);
		pNode = pBaseMap->getNodeAt(id);
		if (!pNode->getNbrChecked(a, nbrType))
			return id;
	}
	return -1;
}

///////////////////
// updateResidualImage()
//	return residual image square  change
double skUpdateResidualImage(SkBase &base,//the base
						  Matrix<float> &imgRes, //the residual image
						  SkFilterBank *pFilterBank,//filter bank
						  int flag,//ADD or REMOVE the base
						  int nBound)
{
	double dResErrorChg = 0;//residual error change

	float a;
	if (flag == ADD)
		a = base.a;
	else
		a = -base.a;

	int h = pFilterBank->m_pFilters[base.k].getHalf();
//	int nBond = pFilterBank->getImgBond();
	int ht = imgRes.rows();
	int wd = imgRes.cols();// r, c: image rows and cols
	cv::Ptr<cv::Mat> f = pFilterBank->m_pFilters[base.k].m_pIplKer;
	int mx = base.x;
	int my = base.y;
	int fx, fy, x, y;
	int nYOffset;
	for (fy = -h; fy <= h; fy++)
	{
		y = my + fy;
		if (y < nBound || y >= ht-nBound)
			continue;
//		if (y < 0 ) y += ht;
//		if (y >= ht) y -= ht;
		nYOffset = (fy+h)*(2*h+1);
		for (fx = -h; fx <= h; fx++)
		{
			x = mx + fx;
			if (x < nBound || x >= wd-nBound)
				continue;
//			if (x < 0 )	x += wd;
//			if (x >= wd) x -= wd;

			dResErrorChg -= imgRes(y, x)*imgRes(y, x);

			imgRes(y, x) -= a * f->at<float>(fy+h, fx+h);//[nYOffset+fx+h];
			
			dResErrorChg += imgRes(y, x)*imgRes(y, x);
		}
	}
	return dResErrorChg;
}

///////////////////
// updateResidualImageByPatch()
//	return residual image square  change
//  the pixel in the boundary range is not considerred
double skUpdateResidualImageByPatch(SkBase &base,//the base
						  Matrix<float> &imgRes, //the residual image
						  SkFilterBank *pFilterBank,//filter bank
						  int nBound, double &dCoef)
{
	double dResErrorChg = 0;//residual error change


	int h = pFilterBank->m_pFilters[base.k].getHalf();
	int ht = imgRes.rows();
	int wd = imgRes.cols();// r, c: image rows and cols

	// compute residual
//	int patch_wd = pFilterBank->getHalfValidLn(base.k)*2+1;//2*h+1;//
//	int patch_ht = pFilterBank->getHalfValidWd(base.k)*2+1;//2*h+1;//
	int patch_wd = 2*h+1;//
//	int patch_ht = h/2*2+1;//
	int patch_ht = h*2+1;//
	Matrix<float> patch(patch_ht, patch_wd);

	double dOrt = base.getOrt();
	int r0 = base.y;
	int c0 = base.x;
	float invalid_value = 0;

	::skGetRotatedPatch(imgRes, dOrt, r0, c0, patch, invalid_value);

//	::skComputeBestPatchSize4AverageModeling(patch);
//	patch.GetDimension(patch_ht, patch_wd);


	Matrix<float> rowAvg;
	::skGetMatrixRowAverage(patch, rowAvg);

	dCoef = skMultipleRowAverage(patch, rowAvg);

	::setSameValue4EachRow(patch, rowAvg);

	int max_patch_size = MAX(patch_wd, patch_ht);
	int rot_patch_wd = ((int)(max_patch_size * 1.5))/2 * 2 + 1;
	int rot_patch_ht = rot_patch_wd;

	Matrix<float> rot_patch(rot_patch_ht, rot_patch_wd);
	::skGetRotatedPatch(patch, -dOrt, patch_ht/2, patch_wd/2, rot_patch, invalid_value);

//	::skSaveMatrixAsBitmap("patch.bmp", patch, 0, -1);
//	::skSaveMatrixAsBitmap("patch_rot.bmp", rot_patch, 0, -1);

	int dx = c0 - rot_patch_wd/2;
	int dy = r0 - rot_patch_ht/2;
//	dResErrorChg = ::skSubtractImageAndReturnSquareSumChangeWithInhibition(&imgRes, &rot_patch, dx , dy, nBound);
	dResErrorChg = ::skSubtractImageAndReturnSquareSumChange(&imgRes, &rot_patch, dx , dy, nBound);

	
	return dResErrorChg;
}

///////////////////
// updateResidualImageByPatchWithDictionary()
//	return residual image square  change
//  the pixel in the boundary range is not considerred
double skUpdateResidualImageByPatchWithDictionary(SkBase &base,//the base
						  Matrix<float> &imgRes, //the residual image
						  SkFilterBank *pFilterBank,//filter bank
						  int nBound,
						  Matrix<float>& dict)
{
	double dResErrorChg = 0;//residual error change


	int h = pFilterBank->m_pFilters[base.k].getHalf();
	int ht = imgRes.rows();
	int wd = imgRes.cols();// r, c: image rows and cols

	int dict_size, dict_wd;
	dict.GetDimension(dict_size, dict_wd);

	int patch_wd = 2*h+1;
//	int patch_ht = 2*h+1;
	int patch_ht = dict_wd;
	Matrix<float> patch(patch_ht, patch_wd);

	double dOrt = base.getOrt();
	int r0 = base.y;
	int c0 = base.x;
	float invalid_value = 0;

	::skGetRotatedPatch(imgRes, dOrt, r0, c0, patch, invalid_value);

	Matrix<float> rowAvg;
	::skGetMatrixRowAverage(patch, rowAvg);

	//rowAvg = dict*a + b;
	double a, b;
	int match_no;
	match_no = ::skBestMatch2Dictionary(dict, rowAvg.T(), a, b);

	Matrix<float> match(patch_wd, 1);
	for (int r = 0; r < patch_wd; r++)
		match(r, 0) = dict(match_no, r) * a + b;

	::setSameValue4EachRow(patch, match);

	int max_patch_size = MAX(patch_wd, patch_ht);
	int rot_patch_wd = ((int)(max_patch_size * 1.5))/2 * 2 + 1;
	int rot_patch_ht = rot_patch_wd;

	Matrix<float> rot_patch(rot_patch_ht, rot_patch_wd);
	::skGetRotatedPatch(patch, -dOrt, patch_ht/2, patch_wd/2, rot_patch, invalid_value);

//	::skSaveMatrixAsBitmap("patch.bmp", patch, 0, -1);
//	::skSaveMatrixAsBitmap("patch_rot.bmp", rot_patch, 0, -1);

	int dx = c0 - rot_patch_wd/2;
	int dy = r0 - rot_patch_ht/2;
	dResErrorChg = ::skSubtractImageAndReturnSquareSumChange(&imgRes, &rot_patch, dx , dy, nBound);

	
	return dResErrorChg;
}


///////////////////
// skSynthesizeImgFromBases()
int skSynthesizeImgFromBases(SkBaseMap *pBaseMap,//the base map
							 Matrix<float> *M, //the image
							 SkFilterBank *pFilterBank,//the filter bank
							 int flag)//KER or SMB symbolic
{
	int n = 0;
	assert(M != 0);
	int wd, ht;
	wd = pBaseMap->getWidth();
	ht = pBaseMap->getHeight();
	int h, w;
	M->GetDimension(h, w);
	if (wd != w || ht != h)
		M->SetDimension(ht, wd);

	M->Zero();

	SkBaseNode *pBaseNode;
	int nPos = pBaseMap->getFirstPos();
	while (nPos >= 0)
	{
		pBaseNode = pBaseMap->getNextNode(nPos);
		if (!pBaseNode->valid())
			continue;
		skAddBase2Img(pBaseNode, M, pFilterBank, flag);
		n++;
	}
	return n;
}

///////////////////
// skAddBase2Img()
//	add the base to image M
//	flag: SMB - symbolic, SYN - synthesis
//	nConsiderAtomOrt : 
//	x0,y0: the Atom center
int skAddBase2Img(SkBaseNode *pBaseNode, //the base node will be added
				  Matrix<float> *M, //the image
				  SkFilterBank *pFilterBank, //the filter bank
				  int flag, //SMB - symbolic, SYN - synthesis
				  int nConsiderAtomOrt, //1-Yes, 0-Not, default-0, we rotate the base as its atomort before add it
				  int x0, int y0)//the atom center, default (0,0)
{
	int wd = M->cols();
	int ht = M->rows();
	int nBond = pFilterBank->getImgBond();
	int ind, x, y, h, s, fx, fy, mx, my, xs, ys;
	float coef;
	cv::Ptr<cv::Mat> f;
	SkFilter *F;

	ind = pBaseNode->m_data.k;
	x = pBaseNode->m_data.x;
	y = pBaseNode->m_data.y;
	coef = pBaseNode->m_data.a;
	if (nConsiderAtomOrt )
	{
		// update k because of the atom ort
		double dAtomOrt = pBaseNode->getAtomOrt();
		if (dAtomOrt != 0)
		{
			double dSign;
			double ort = pBaseNode->getRealOrt(&dSign);//ort
			coef *= dSign;
			double fscale = pBaseNode->m_data.getfScale();//fscale
			int type = (int)(pBaseNode->m_data.getType());//type

			if (pBaseNode->m_data.getType() != DOG)
			{
				ind = pFilterBank->getFilterIndex(type, fscale, ort);
				assert(type == pFilterBank->getType(ind));
				assert(fabs(fscale-pFilterBank->getfScale(ind)) < 0.5 );
			}
			double dx = x - x0;
			double dy = y - y0;
			::rotate(dx, dy, -dAtomOrt);//because the X-Y system are different
			x = (int)(x0 + dx);
			y = (int)(y0 + dy);
		}

	}
	
	SkBase base;
	base.setFilterBank(pFilterBank);
	base.x = x;
	base.y = y;
	base.k = ind;
	base.a = coef;
	
//	if (flag == KER)//use the same code as updateResidualImage(), why?? Dec. 19, 2002
//		return updateResidualImage(base, *M, pFilterBank, REMOVE);

	F = &(pFilterBank->m_pFilters[ind]);
	h = F->getHalf(); //half of the filter size
	s = F->getSub(); // subsample of the filter
	xs = x*s;
	ys = y*s;
	if (flag == SMB)
		f = F->m_pIplSym;//filter response 
	else//KER
		f = F->m_pIplKer;
	int nFilterSize = 2*h + 1;
	for (fy = 0; fy < nFilterSize; fy++)
	{
		my = ys + fy - h;
		if (my < nBond || my >= ht - nBond)
			continue;
//		if (my < 0) 
//			my += ht;
//		if (my >= ht) 
//			my -= ht;

		for (fx = 0; fx < nFilterSize; fx++)//for all pixels covered by this filter
		{
			mx = xs + fx-h;
			if (mx < nBond || mx >= wd - nBond)
				continue;
//			if (mx < 0) 
//				mx += wd;
//			if (mx >= wd) 
//				mx -= wd;
			if (flag == SMB)
				(*M)(my, mx) += fabs(coef) * f->at<float>(fy, fx); //[fy*nFilterSize + fx];
			else
				(*M)(my, mx) += coef * f->at<float>(fy, fx); //[fy*nFilterSize + fx];
		}
	}
	return 0;
}

///////////////////
// skBottomUpOneAdrNbrBaseWithPrior()
//	bottom up the no 'a' nbr bases of pNode with prior
//	find not pursued or checked addressed nbr bases of pNode, and add them one by one
//	return new nbr base id, -1, if no nbr base found
int skBottomUpOneAdrNbrBaseWithPrior(int a, //the nbr #
									 int nNodeID, //current node ID
									 SkFilterCoef *pFC,//filter coef
									 SkBaseMap *pBaseMap, //the base map
									 Matrix<float> &imgRes,//residual image
									 SkFilterBank *pFB, //the filter bank
									 SkBasePursuitParam &param,//the parameters
									 SkIntegerStack &idStack,//the stack for ids
									 double dImgMean, double dImgStd,//image mean and std
 									 const char *pchProjName,//the project name
									 double dMinBaseCoef)

{
//	double dMinBaseCoef = param.m_dMinBaseCoefLow;//min base coef 
	int nUpdateRes = param.m_nUpdateRes;
	int nShowProcess = param.m_nShowProcess; //update residual image or not
	double dInhibitionDis = param.m_dInhibitionDis;

#define MAX_POSS_NBR_BASE 4000
	SkBase pBaseCand[MAX_POSS_NBR_BASE];
	double pdProb[MAX_POSS_NBR_BASE];
	Matrix<oval_IntArray> *pOcpMat = pBaseMap->getOccupyMatrix();

	int n = skPossibleAdrNbrBase(nNodeID, a, pFC, pBaseMap, pFB, dMinBaseCoef, pBaseCand, pdProb);
	int i = skChooseOneAdrNbrBase(pBaseCand, pdProb, n, pOcpMat, pBaseMap, pFB);

	int nbrType = ADR_NBR;
	SkBaseNode *pNode = pBaseMap->getNodeAt(nNodeID);
	pNode->setNbrChecked(a, true, nbrType);

	if ( i < 0)
		return -1;


	SkBase base(pFB);
	base = pBaseCand[i];
	base.setFilterBank(pFB);


	pFC->updateOneBase(base, ADD);
	int id = pBaseMap->add(base);
	SkBaseNode *pNodeNew = pBaseMap->getNodeAt(id);
	pBaseMap->updateOccupyMatrix(pNodeNew, ADD, dInhibitionDis); 
	if (nUpdateRes == 1)
		skUpdateResidualImage(base, imgRes, pFB, ADD);

	if (nShowProcess)
		skSaveBaseMap2ShowBottomUpProcess(pBaseMap, "0", pchProjName, pFB, dImgMean, dImgStd);

	int a_i = SkNodeNbrs::adrNbrInverse(a);
	assert(a_i >= 0);

	pNode = pBaseMap->getNodeAt(nNodeID);//after adding one base, the pointer pNode may be changed
	SkNbrManager::addNbr(pNode, pNodeNew, nbrType, a);
	SkNbrManager::addNbr(pNodeNew, pNode, nbrType, a_i);
//	pNodeNew->setNbrChecked(false, nbrType);
//	pNodeNew->setNbrChecked(a_i, true, nbrType);//not necessary, addNbr already set checked

	idStack.push(id);

	return id;
}

///////////////////
// skPossibleAdrNbrBase()
//	try to find all possible AdrNbr bases of node "pNode"
//	save bases to "pBaseCand", with probabilities "pdProb"
//	return the # of possible bases
//	find not pursued or checked addressed nbr bases of pNode, and add them one by one
int skPossibleAdrNbrBase(int nNodeID, //the base node id
						 int a, //nbr id
						 SkFilterCoef *pFC, //filter coef
						 SkBaseMap *pBaseMap, //base map
						 SkFilterBank *pFB, //filter bank
						 double dMinBaseCoef, //min base coef
						 SkBase* pBaseCand,//candidates
						 double *pdProb)//prob for each candidates
{
	SkBaseNode *pNode = pBaseMap->getNodeAt(nNodeID);

	int l = pNode->m_data.getValidLength();
	int w = pNode->m_data.getValidWidth();
	double dOrtUnit = pNode->m_data.getOrtUnit();

	int sn = skPossibleAdrNbrBase(a, pBaseCand, pdProb, l, w, dOrtUnit);//without consider base type, ort, and coef

	int wd = pBaseMap->getWidth();
	int ht = pBaseMap->getHeight();

	int nType = pNode->m_data.getType();
	double dOrt = pNode->m_data.getOrt();//node ort
	double fscale = pNode->m_data.getfScale();
	int nScale = pNode->m_data.getScale();

	int x0, y0, x, y;//node center and nbr center, and center relative position
	int k, kk;
	SkBase base(pFB);
	double xx, yy;
	x0 = pNode->getCol();
	y0 = pNode->getRow();

	double dOrtNbr;
	int n = 0;
	int nFilterNum = pFB->getTotalFilterNum();

	int nBound = 7;

	for (int i = 0; i < sn; i++)
	{
		xx = pBaseCand[i].x;
		yy = pBaseCand[i].y;
//		::rotateInteger(xx, yy, dOrt);
		::rotate(xx, yy, dOrt);
		x = (int)(x0 + xx);
		y = (int)(y0 + yy);

		if (x < nBound || x >= wd-nBound || y < nBound || y >= ht-nBound)
			continue;

		dOrtNbr = dOrt + pBaseCand[i].a;
		while (dOrtNbr >= PI)
		{
			dOrtNbr -= PI;
		}
		while (dOrtNbr < 0)
		{
			dOrtNbr += PI;
		}

		if (a == R_CNTN || a == L_CNTN || a == R_BL_COCV || a == R_AB_COCV || a == L_BL_COCV || a == L_AB_COCV)// right below co-curve
		{
//			k = pFB->getFilterIndex(nType, fscale, dOrtNbr);
			k = pFB->getFilterIndex(nType, nScale, dOrtNbr);
//			assert(kk == k);
//			for (k = 0; k < nFilterNum; k++)
			{
//				if (fabs(pFB->getOrt(k) - dOrtNbr) > PI/180 * 15)
//					continue;
				base.x = x;
				base.y = y;
				base.k = k;
				base.a = (*pFC->m_ppRemainCoef[k])(y,x);
				// Added 11-25-2003
				float fMaxCoef = pFC->maxAbsCoef(y, x);
//				if (fabs(base.a) >= dMinBaseCoef && fabs(fabs(base.a) - fMaxCoef) < 1.0 )
				if (fabs(base.a) >= dMinBaseCoef)
				{
					pBaseCand[n] = base;
					pdProb[n] = pdProb[i];
	//				if (nType == GCOS && base.a * pNode->m_data.a < 0)//odd base, coef should has the same sign, otherwise reduce the prob half
	//					pdProb[n] = pdProb[i]/2;
					n++;
				}//if
			}
		}
		else if (a == BL_PRPD || a == AB_PRPD || a == BL_PRLL || a == AB_PRLL) //// below parallel // above parallel above perpendicular // below perpendicular
		{
			assert(false);//not consider yet
			/*
			int e1, e2;
			if (g_nUsedSin)	e1 = 0;	else e1 = 1;
			if (g_nUsedCos)	e2 = 1;	else e2 = 0;

			for (int e = e1; e <= e2; e++)
			{
				k = pFB->getFilterIndex(e, nScale, dOrtNbr);
				base.x = x;
				base.y = y;
				base.k = k;
				base.a = (*pFC->m_ppRemainCoef[k])(y,x);
				if (fabs(base.a) >= dMinBaseCoef)
				{
					pBaseCand[n] = base;
					pdProb[n] = pdProb[i];
//					if (nType == GCOS && base.a * pNode->m_data.a < 0)//odd base, coef should has the same sign, otherwise reduce the prob half
//						pdProb[n] = pdProb[i]/2;
					n++;
				}//if
			}
			*/
		}
		else
			assert(false);
	}
	return n;
}
///////////////////
// skPossibleAdrNbrBase()
//	try to find all possible AdrNbr bases of a horizontal node
//	save bases to "pBaseCand", with probabilities "pdProb"
//	return the # of possible bases
//	find not pursued or checked addressed nbr bases of pNode, and add them one by one
int skPossibleAdrNbrBase(int a, //nbr #
						SkBase* pBaseCand, //candidates
						double *pdProb, //probability
						int l, int w, //length and width of the base
						double dOrtUnit)//the orientaion unit
{
	// static to save time
	//	for each a and l, save them in a_s and l_s, with the # a_num, l_num
	static int visited = false;
	static int a_num = 0, max_a_num = 2;
	static int l_num = 0, max_l_num = 5;
	static SkBase pppCand[2][5][10000];
	static double pppdProb[2][5][10000];
	static int a_s[2];
	static int l_s[5];
	static int cand_num[2][5];

	int i, j;
	if (!visited)
	{
		for (i = 0; i < max_a_num; i++)
			a_s[i] = -1;
		for (j = 0; j < max_l_num; j++)
			l_s[j] = -1;
		visited = true;
	}

	int a_ind = -1, l_ind = -1;//index

	for (i = 0; i < a_num; i++)
	{
		if (a == a_s[i])
		{
			a_ind = i;
			break;
		}
	}
	for (j = 0; j < l_num; j++)
	{
		if (l == l_s[j])
		{
			l_ind = j;
			break;
		}
	}

	int x, y;//node center and nbr center, and center relative position
	double dx, dy;

	double dOrtZ = 0.2, dPosZ = 1.0;
	double dOrtNbr;
	int n = 0;
	if (a == R_CNTN || a == L_CNTN)
	{
		if (a_ind >= 0 && l_ind >= 0)
		{
			memcpy(pBaseCand, pppCand[a_ind][l_ind], sizeof(SkBase)*cand_num[a_ind][l_ind]);
			memcpy(pdProb, pppdProb[a_ind][l_ind], sizeof(double)*cand_num[a_ind][l_ind]);
			return cand_num[a_ind][l_ind];
		}

		if (a_ind < 0)
		{
			a_s[a_num] = a;
			a_ind = a_num;
			a_num++;
			assert(a_num <= max_a_num);
		}
		if (l_ind < 0)
		{
			l_s[l_num] = l;
			l_ind = l_num;
			l_num++;
			assert(l_num <= max_l_num);
		}

		int dxdy_sign = 1;
		int ort_sign = 1;
		if (a == L_CNTN)
		{
			dxdy_sign = -1;
			ort_sign = -1;
		}

		int dx_s = -2;//dx start
//		int dx_e = 6;//dx end
		int dx_e = 3;//dx end
		int dy_s = -2;
		int dy_e = 2;

		int nOrtNum = (int)(PI/dOrtUnit + 0.5);

		int ort_s, ort_e;
		
		if (nOrtNum <= 2)
		{
			ort_s = 0;
			ort_e = 0;
		}
		else if (nOrtNum <= 6)
		{
			ort_s = -1;
			ort_e = 1;
		}
		else
		{
			ort_s = -2;
			ort_e = 2;
		}


		for (int sx = dx_s; sx <= dx_e; sx++)
		{
			for (int sy = dy_s; sy <= dy_e; sy++)
			{
				for (int so = ort_s; so <= ort_e; so++)
				{
					dOrtNbr = so * dOrtUnit * ort_sign;

					dx = sx+ l/2 + 1;
					dy = sy;
					::rotate(dx, dy, dOrtNbr);
					x = (int)(dx+l/2+1)*dxdy_sign;
					y = (int)(dy*dxdy_sign);
					pppCand[a_ind][l_ind][n].x = x;
					pppCand[a_ind][l_ind][n].y = y;
					pppCand[a_ind][l_ind][n].a = dOrtNbr;
					pppdProb[a_ind][l_ind][n] = 1.0/(1 + fabs(double(sx))*dPosZ + fabs(double(sy))*dPosZ + fabs(double(so))*dOrtZ);
					n++;
					assert(n <= 10000);
				}//for so
			}//for sy
		}//for sx
		cand_num[a_ind][l_ind] = n;
		memcpy(pBaseCand, pppCand[a_ind][l_ind], sizeof(SkBase)*cand_num[a_ind][l_ind]);
		memcpy(pdProb, pppdProb[a_ind][l_ind], sizeof(double)*cand_num[a_ind][l_ind]);
		return cand_num[a_ind][l_ind];
	}//if
	else if (a == R_BL_COCV || a == R_AB_COCV || a == L_BL_COCV || a == L_AB_COCV)// right below co-curve
	{
		assert(false);
		return 0;

		int dxdy_sign = 1;
		if ( a == L_BL_COCV || a == L_AB_COCV)
		{
			dxdy_sign = -1;
		}
		int ort_sign = 1;
		if (a == R_BL_COCV || a == L_AB_COCV)
			ort_sign = -1;

		int dx_s = -2;//dx start
		int dx_e = 4;//dx end
		int dy_s = -2;
		int dy_e = 2;
		int ort_s = 1;
		int ort_e = PI/2/dOrtUnit - 2;
		
		for (int sx = dx_s; sx <= dx_e; sx++)
		{
			for (int sy = dy_s; sy <= dy_e; sy++)
			{
				for (int so = ort_s; so <= ort_e; so++)
				{
					dOrtNbr = so * dOrtUnit * ort_sign;

					dx = (sx+l/2+1);
					dy = sy;
					::rotate(dx, dy, dOrtNbr);
					x = (int)((dx+l/2+1) * dxdy_sign);
					y = (int)(dy * dxdy_sign);
					pBaseCand[n].x = x;
					pBaseCand[n].y = y;
					pBaseCand[n].a = dOrtNbr;
					pdProb[n] = 1.0/(1 + fabs(double(sx))*dPosZ + fabs(double(sy))*dPosZ);// + fabs(so)*dOrtZ - 1);
					n++;
				}//for so
			}//for sy
		}//for sx
	}
	else if (a == BL_PRPD || a == AB_PRPD) // above perpendicular // below perpendicular
	{
		return 0;

		int dxdy_sign = 1;
		int ort_sign = 1;
		if ( a == BL_PRPD)
		{
			dxdy_sign = -1;
			ort_sign = -1;
		}

		int dx_s = -6 - l/2 - 1;//dx start
		int dx_e = -dx_s;//dx end
		int dy_s = w/2 + (l/2+1);
		int dy_e = dy_s + 4;
//		int ort_s = -1;
//		int ort_e = 1;
		int ort_s = -(PI/2/dOrtUnit - 2);
		int ort_e = -ort_s;
		
		for (int sx = dx_s; sx <= dx_e; sx++)
		{
			for (int sy = dy_s; sy <= dy_e; sy++)
			{
				for (int so = ort_s; so <= ort_e; so++)
				{

					dx = sx;
					dy = sy*dxdy_sign;//rotate a vertical bar
					x = (int)(dx);
					y = (int)(dy);
					pBaseCand[n].x = x;
					pBaseCand[n].y = y;
					dOrtNbr = (so*dOrtUnit+PI/2)  * ort_sign;
					pBaseCand[n].a = dOrtNbr;
					pdProb[n] = 1.0/(1 + fabs(double(so))*dOrtZ);
					n++;
				}//for so
			}//for sy
		}//for sx
	}
	else if (a == BL_PRLL || a == AB_PRLL) // below parallel // above parallel
	{
		return 0;

		int para_scale = 2;

		int dxdy_sign = 1;
		int ort_sign = 1;
		if ( a == BL_PRLL)
		{
			dxdy_sign = -1;
			ort_sign = -1;
		}

		int dx_s = -l;//dx start
		int dx_e = -dx_s;//dx end
//		int dy_s = (int)(g_dInhibitionDis);//w * 3 / 2 ;
		int dy_s = w * 3 / 2 ;
		int dy_e = dy_s + para_scale*l;
		int ort_s = -1;
		int ort_e = 1;
		
		for (int sx = dx_s; sx <= dx_e; sx++)
		{
			for (int sy = dy_s; sy <= dy_e; sy++)
			{
				for (int so = ort_s; so <= ort_e; so++)
				{
					dOrtNbr = so * dOrtUnit * ort_sign;

					dx = sx;
					dy = sy*dxdy_sign;
					x = (int)(dx);
					y = (int)(dy);
					pBaseCand[n].x = x;
					pBaseCand[n].y = y;
					pBaseCand[n].a = dOrtNbr;
					pdProb[n] = 1.0/(1 + fabs(sx/5.0f) + fabs(double(so))*dOrtZ);
					n++;
				}//for so
			}//for sy
		}//for sx
	}
	else
	{
		assert(false);//not considered
	}
	return n;
}
///////////////////
// skChooseOneAdrNbrBase
//	choose one AdrNbr base from "n" candidates "pBaseCand" with probabilities "pdProb"
int skChooseOneAdrNbrBase(SkBase *pBaseCand, //base candidates
						   double *pdProb, //the probability
						   int n,//candidate #
						   Matrix<oval_IntArray> *pOcpMat, //ocupaccy matrix
						   SkBaseMap *pBaseMap,//base map
						   SkFilterBank* pFB)//filter bank
{
	int nBound = 2;

	int wd = pBaseMap->getWidth();
	int ht = pBaseMap->getHeight();

	int best = -1;
	double bestProbCoef = -1;
	double probCoef;
	int x, y;
	float fPrllOcp, fLeft;
	int nOcp;
	for (int i = 0; i < n; i++)
	{
		SkBase &base = pBaseCand[i];
		x = base.x;
		y = base.y;
		if (x < nBound || x > wd-nBound || y < nBound || y > ht-nBound)
			continue;

		fPrllOcp = SkFilterCoef::parallelBaseOccupancy(x, y, base.k, pOcpMat, pBaseMap, pFB);
		fLeft = 1 - fPrllOcp;
		if (fLeft < 0 )
			fLeft = 0;
		if (fLeft > 1.0)
			fLeft = 1.0;

		nOcp = SkFilterCoef::baseOccupancy(x, y, pOcpMat);//at most two bases
//		probCoef = fabs(base.a*base.a) * pdProb[i] * fLeft;
		probCoef = fabs(base.a*base.a) * pdProb[i] * fLeft;//emphasize the coef, not spatial continuity, 05-23-2003
		if (bestProbCoef < probCoef && pdProb[i] >= 0.00001 && fLeft >= 0.00001 )//&& nOcp <=1)
		{
			bestProbCoef = probCoef;
			best = i;
		}
	}
	return best;
}

///////////////////
// skSaveBaseMap2ShowBottomUpProcess()
//	save base map to show bottom up process
int skSaveBaseMap2ShowBottomUpProcess(SkBaseMap *pBaseMap,//the current base map
 									  const char *pchProjName,//the project name
									  const char *pPreName, //pre-name
									  SkFilterBank* pFB, //filter bank
									  double dImgMean, double dImgStd)//image mean and variance
{
	int n = pBaseMap->getSize();

	char name[512], tmp[512], dir[512], dir_rct[512];
	sprintf(dir, "BupBsp");
	_mkdir(dir);
	sprintf(dir_rct, "BupBsp\\rct");
	_mkdir(dir_rct);

	int wd = pBaseMap->getWidth();
	int ht = pBaseMap->getHeight();

	Matrix<float> M(ht, wd);
	
	::changeNumTo4Char(tmp, n);

	sprintf(name, "%s\\%s_%s.bmp", dir_rct, pPreName, tmp);
	skSynthesizeImgFromBases(pBaseMap, &M, pFB, KER);
	skSaveMatrixAsBitmap(name, M, dImgMean, dImgStd);
	
	skSynthesizeImgFromBases(pBaseMap, &M, pFB, SMB);
	dImgMean = 255;
	dImgStd = 255;

	// mark boundary just for good looking
	int bound = 2;
	int r, c;
	//col boundary
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < bound; c++)
		{
			M(r,c) = -1;
			M(r,wd-c-1) = -1;
		}
	}
	//row boundary
	for (c = 0; c < wd; c++)
	{
		for (r = 0; r < bound; r++)
		{
			M(r,c) = -1;
			M(ht-r-1,c) = -1;
		}
	}

	sprintf(name, "%s\\%s_%s.bmp", dir, pPreName, tmp);
	skSaveMatrixAsBitmap(name, M, dImgMean, dImgStd);

	return 0;
}

///////////////////
// skSaveResidualErrorBaseByBase()
int skSaveResidualErrorBaseByBase(const char *name, SkBaseMap* pBaseMap, 
								  Matrix<float> *pImgOrg, SkFilterBank *pFB,
  								  double dImgMean, double dImgStd, const char* proj)
{
#define MAX_BASE_NUM 100000
	double pdResError[MAX_BASE_NUM];
	double pdCoef[MAX_BASE_NUM];
	memset(pdResError, 0, sizeof(double)*MAX_BASE_NUM);

	char name_tmp[512], no[32];

	Matrix<float> imgRes;
	imgRes = *pImgOrg;

	double dResErrorChg;

	int nBound = 5;
	int n = 0;
	pdResError[n] = skSquareSumOfMatrix(&imgRes, nBound);
	n++;

	SkBaseNode *pNode;
	int nPos = pBaseMap->getFirstPos();
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		
		dResErrorChg = skUpdateResidualImage(pNode->m_data, imgRes, pFB, ADD, nBound);
		pdResError[n] = pdResError[n-1] + dResErrorChg;
		pdCoef[n-1] = fabs(pNode->m_data.a);
		n++;
		if (n / 500 * 500 == n)
		{
			::changeNumTo4Char(no, n);
			sprintf(name_tmp, "skps\\%s_base_syn_%s.bmp", proj, no);
			::skSaveMatrixAsBitmap(name_tmp, *pImgOrg - imgRes, dImgMean, dImgStd);
		}
	}

	sprintf(name_tmp, "skps\\%s_base_syn.bmp", proj);
	::skSaveMatrixAsBitmap(name_tmp, *pImgOrg - imgRes, dImgMean, dImgStd);

	char proj0[512];
	strcpy(proj0, proj);
	char *temp;
	temp = strrchr(proj0, '_');//find '_', return the pointer to it
	*temp = '\0';


	sprintf(name_tmp, "%s_syn1.bmp", proj0);
	::skSaveMatrixAsBitmap(name_tmp, *pImgOrg - imgRes, dImgMean, dImgStd);

	sprintf(name_tmp, "skps\\%s_base_res.bmp", proj);
	::skSaveMatrixAsBitmap(name_tmp, imgRes, dImgMean, dImgStd);

	sprintf(name_tmp, "%s_res1.bmp", proj0);
	::skSaveMatrixAsBitmap(name_tmp, imgRes, dImgMean, dImgStd);

	::saveArray(pdResError, n, name);

	sprintf(name_tmp, "skps\\%s_base_coef.txt", proj);
	::saveArray(pdCoef, n-1, name_tmp);


	return 0;
}

///////////////////
// skSaveResidualErrorPatchByPatch()
int skSaveResidualErrorPatchByPatch(const char *name, SkBaseMap* pBaseMap, 
								  Matrix<float> *pImgOrg, SkFilterBank *pFB, 
								  double dImgMean, double dImgStd, const char* proj)
{
#define MAX_BASE_NUM 100000
	double pdResError[MAX_BASE_NUM];
	double pdCoef[MAX_BASE_NUM];
	memset(pdResError, 0, sizeof(double)*MAX_BASE_NUM);

	char name_tmp[512], no[32];

	Matrix<float> imgRes;
	imgRes = *pImgOrg;

	double dResErrorChg;

	int nBound =  5;
	int n = 0;
	pdResError[n] = skSquareSumOfMatrix(&imgRes, nBound);
	n++;

	SkBaseNode *pNode;
	int nPos = pBaseMap->getFirstPos();
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		
		double dCoef;
		dResErrorChg = skUpdateResidualImageByPatch(pNode->m_data, imgRes, pFB, nBound, dCoef);
		pdResError[n] = pdResError[n-1] + dResErrorChg;
		pdCoef[n-1] = dCoef;
		n++;

		if (n / 500 * 500 == n)
		{
			::changeNumTo4Char(no, n);
			sprintf(name_tmp, "skps\\%s_patch_syn_%s.bmp", proj, no);
			::skSaveMatrixAsBitmap(name_tmp, *pImgOrg - imgRes, dImgMean, dImgStd);
		}
	}


	sprintf(name_tmp, "skps\\%s_patch_syn.bmp", proj);
	::skSaveMatrixAsBitmap(name_tmp, *pImgOrg - imgRes, dImgMean, dImgStd);

	sprintf(name_tmp, "skps\\%s_patch_res.bmp", proj);
	::skSaveMatrixAsBitmap(name_tmp, imgRes, dImgMean, dImgStd);

	::saveArray(pdResError, n, name);

	sprintf(name_tmp, "skps\\%s_patch_coef.txt", proj);
	::saveArray(pdCoef, n-1, name_tmp);

	return 0;
}

///////////////////
// skSaveResidualErrorPatchByPatchWithDictionary()
int skSaveResidualErrorPatchByPatchWithDictionary(const char *name, SkBaseMap* pBaseMap, 
												  Matrix<float> *pImgOrg, SkFilterBank *pFB, 
												  double dImgMean, double dImgStd,
												  Matrix<float> &dict, //the dictionary
												  int nDictNo,
												  const char *proj)
{
#define MAX_BASE_NUM 100000
	double pdResError[MAX_BASE_NUM];
	memset(pdResError, 0, sizeof(double)*MAX_BASE_NUM);

	char name_tmp[512], no[32], dict_no[32];
	::changeNumTo2Char(dict_no, nDictNo);

	Matrix<float> imgRes;
	imgRes = *pImgOrg;

	double dResErrorChg;

	int nBound =  5;
	int n = 0;
	pdResError[n] = skSquareSumOfMatrix(&imgRes, nBound);
	n++;

	SkBaseNode *pNode;
	int nPos = pBaseMap->getFirstPos();
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		
		dResErrorChg = skUpdateResidualImageByPatchWithDictionary(pNode->m_data, imgRes, pFB, nBound, dict);
		pdResError[n] = pdResError[n-1] + dResErrorChg;
		n++;

		if (n / 500 * 500 == n)
		{
			::changeNumTo4Char(no, n);
			sprintf(name_tmp, "%s\\skps\\dict%s_syn_%s.bmp", proj, dict_no, no);
			::skSaveMatrixAsBitmap(name_tmp, *pImgOrg - imgRes, dImgMean, dImgStd);
		}
	}


	sprintf(name_tmp, "%s\\skps\\dict%s_syn.bmp", proj, dict_no);
	::skSaveMatrixAsBitmap(name_tmp, *pImgOrg - imgRes, dImgMean, dImgStd);

	sprintf(name_tmp, "%s\\skps\\dict%s_res.bmp", proj, dict_no);
	::skSaveMatrixAsBitmap(name_tmp, imgRes, dImgMean, dImgStd);

	::saveArray(pdResError, n, name);

	return 0;
}

///////////////////
// skSaveSketchGraph2CurveMapFile
int skSaveSketchGraph2CurveMapFile(const char *name, SkSketchGraph &sketchGraph)
{
	SkCurveMap curveMap;
	skSaveSketchGraph2CurveMap(sketchGraph, curveMap);
	curveMap.save(name);
	return 0;
}
///////////////////
// skSaveSketchGraph2CurveMap
int skSaveSketchGraph2CurveMap(SkSketchGraph &sketchGraph, SkCurveMap &curveMap)
{
	int x, y, h, t;
	SkCurve curve;
	SkContour *pCon = sketchGraph.getContourHead();
	while (pCon != 0)
	{
		SkNode *pNode = pCon->GetHeadNode();
		while (pNode != 0)
		{
			x = pNode->m_data.x;
			y = pNode->m_data.y;
			h = pNode->m_prfl.getSampleWidth()/2;    
			if (h < 8)
				h = 8;
			t = 1;
			curve.addPoint(x, y, h, t);
			pNode = pNode->m_pNext;
		}
		curveMap.add(curve);
		curve.clear();
		pCon = pCon->m_pNext;
	}
	curveMap.m_nImgHt = sketchGraph.m_nImgHt;
	curveMap.m_nImgWd = sketchGraph.m_nImgWd;

	curveMap.upsideDownPoints();

	return 0;
}
///////////////////
// skSavePatchesFromBaseMap()
int skSavePatchesFromBaseMap(SkBaseMap *pBaseMap, //base map
							 const char *pchProjName, //proj name
							 int patch_ht, int patch_wd,//patch hight and width
							 Matrix<float> &img_org,
							 Matrix<float> &img_wht)
{

	char prim_org_dir[32], prim_wht_dir[32];
	sprintf(prim_org_dir, "prim_org");
	sprintf(prim_wht_dir, "prim_wht");
	_mkdir(prim_org_dir);
	_mkdir(prim_wht_dir);

	::skSaveMatrixAsBitmap("org.bmp", img_org, 0, -1);
	::skSaveMatrixAsBitmap("wht.bmp", img_wht, 0, -1);

	char name[512], msg[512], no[32];

	Matrix<float> patch;
	float invalid_value = 0;

	int r0, c0;
	// for each base
	SkFilterBank *pFilterBank;
	int n = 0;
	SkBaseNode *pNode;
	int pos = pBaseMap->getFirstPos(), oldPos;
	while (pos >= 0)
	{
		pNode = pBaseMap->getNextNode(pos);
		if (!pNode->valid())
			continue;

		SkBase &base = pNode->m_data;
		if (fabs(base.a) < 3.0)
			continue;


		pFilterBank = base.getFilterBank();

		::changeNumTo4Char(no, n);

		int h = pNode->m_data.getHalf();
		
		// save patches
//		patch_wd = pFilterBank->getHalfValidLn(base.k)*2+1;//2*h+1;//
//		patch_ht = pFilterBank->getHalfValidWd(base.k)*2+1;//2*h+1;//
		patch_wd = 2*h+1;//
//		patch_ht = h/2*2+1;//
		patch_ht = h*2+1;//
		patch.SetDimension(patch_ht, patch_wd);

		c0 = pNode->getCol();
		r0 = pNode->getRow();
		double dOrt = pNode->m_data.getOrt();

		patch.Set(0);
		sprintf(name, "%s\\%s_prim_org_%s.bmp", prim_org_dir, pchProjName, no);
		::skGetRotatedPatch(img_org, dOrt, r0, c0, patch, invalid_value);
		::skSaveMatrixAsBitmap(name, patch, 0, -1);

		patch.Set(0);
		sprintf(name, "%s\\%s_prim_wht_%s.bmp", prim_wht_dir, pchProjName, no);
		::skGetRotatedPatch(img_wht, dOrt, r0, c0, patch, invalid_value);
		::skSaveMatrixAsBitmap(name, patch, 0, -1);

		n++;
	}
	return 0;
}

///////////////////
// skBestMatch2Dictionary()
//	 return the best no
int skBestMatch2Dictionary(Matrix<float> &dict, Matrix<float>& sample, double &a, double &b)
{
	int ht0, wd0;
	dict.GetDimension(ht0, wd0);

	int ht1, wd1;
	sample.GetDimension(ht1, wd1);

	assert(wd1 == wd0);

	a = 1.0; 
	b = 0.0;

	double dMinDis = 10000000;
	double dDis;
	int nMin = -1;
	int r, c;
	for (r = 0; r < ht0; r++)
	{
		dDis = 0;
		for (c = 0; c < wd0; c++)
		{
			dDis += pow(dict(r,c) - sample(0,c), 2);
		}
		if (dMinDis > dDis)
		{
			dMinDis = dDis;
			nMin = r;
		}
	}
	assert(nMin >= 0);
	return nMin;
}

int skComputeBestPatchSize4AverageModeling(Matrix<float> &mat)
{
	int wd, ht;
	mat.GetDimension(ht, wd);

	int max_dr = -1, max_dc = -1;
	double dMaxErrDecrease = 0, dErrDecrease;
	Matrix<float> patch, avg;
	int dr, dc;
	int wd1, ht1;
	for (dr = 0; dr < ht/2-1; dr++)
	{//min3
		ht1 = ht - 2*dr;
		for (dc = 0; dc < wd/2-2; dc++)
		{//min 5
			wd1 = wd - 2*dc;
			patch.SetDimension(ht1, wd1);
			::skCopyImagePart(&mat, &patch, wd/2, ht/2, 0, wd1-1, 0, ht1-1);
			::skGetMatrixRowAverage(patch, avg);
			dErrDecrease = skErrorDecreaseWithAverageModeling(patch, avg);
			if (dMaxErrDecrease < dErrDecrease)
			{
				dMaxErrDecrease = dErrDecrease;
				max_dr = dr;
				max_dc = dc;
			}
			
		}
	}

	assert(max_dr >= 0);
	ht1 = ht - 2*max_dr;
	wd1 = wd - 2*max_dc;

	patch.SetDimension(ht1, wd1);
	::skCopyImagePart(&mat, &patch, wd/2, ht/2, 0, wd1-1, 0, ht1-1);
	return 0;
}

