#ifndef _SK_BASEPURSUIT_H_
#define _SK_BASEPURSUIT_H_

#include "skFilter.h"
#include "skFilterCoef.h"
#include "skMap.h"
#include "skCurve.h"
#include "skSketchGraph.h"

class SkCompEntropyParam
{
public:
	int m_nScale;
	double m_dNoBasePrior;
public:
	SkCompEntropyParam();
	~SkCompEntropyParam();
	int read(const char *name);
	int save(const char *name);
};

class SkBasePursuitParam
{
public:
	int m_nBuildPyramid;// whether build the Lapalacian paramid, and operate on the whitened image
	double m_dPyramidKernalSize;//the Gaussian Kernal size to build the Lapalacian Pyramid
	int m_nMaxBaseNum;//the max base num
	double m_dMinBaseCoefHi;//the minimum base coef, Hi thresh
	double m_dMinBaseCoefLow;//the minimum base coef, low thresh
	int m_nUpdateRes;//whether to update the residual bases
	int m_nUseGestaltPrior;//whether use Gestalt prior, 0-no, general matching pursuit, 1-yes, sketching pursuit
	int m_nShowProcess;//show the process
	char m_pchFilterParamName[512];//the file name of the parameters for filters
	int m_nMaxProfileWd;//the max profile width
	double m_dInhibitionDis;//the distance of inhibition
	int m_bUseRes4Prfl;
	int m_nConsiderEntropy;
	int m_nSaveMaxCoef;
	int m_nSaveBasePatches;
public:
	SkBasePursuitParam();
	~SkBasePursuitParam();
	int read(const char *name);
	int save(const char *name);
	int initialize(int nSelect);

};

/////////////////////////////////////////////////////////////////////
// class IntegerStack
//	save the integers for bottom up
class SkIntegerStack
{
public:
	oval_IntArray m_idArray;
	int push(int id) { return m_idArray.add(id); }
	int pop() { int id = m_idArray.getAt(m_idArray.getSize()-1); 
					m_idArray.removeAt(m_idArray.getSize()-1); return id;}
	int getSize() { return m_idArray.getSize(); } 
	int getTop() { return m_idArray.getAt(m_idArray.getSize()-1);}
	int getAt(int i) { return m_idArray.getAt(i); }
};

int skSketchPursuit(const char *pchImgName, int nSelect);
int skSketchPursuit(Matrix<float> &imgOrg,//orignal image
					SkBasePursuitParam &param, //sketch pursuit parameters
					double dImgMean, double dImgStd, //image mean and std
					SkFilterBank* pFB, //filter bank
					const char *pchDir, //working dir
					const char *skps_dir, //sketch pursuit result dir
					const char *pchProjName);//project name
int skBottomUpBases(SkFilterCoef *pFC, //the FilterCoefs
					SkBaseMap *pBaseMap,//the base map to save all bases
					Matrix<float> &imgOrg, //orignal image, after noramlization, zero mean and unit variance
					Matrix<float> &imgRes, //residual image
					double dImgMean, double dImgStd,//original image mean and std
					SkBasePursuitParam &param,//parameters for the base pursuit
					const char *pchProjName);//the project name

int skBottomUpOneAdrNbrBaseWithPrior(int a, //the nbr #
									 int nNodeID, //current node ID
									 SkFilterCoef *pFC,//filter coef
									 SkBaseMap *pBaseMap, //the base map
									 Matrix<float> &imgRes,//residual image
									 SkFilterBank *pFB, //the filter bank
									 SkBasePursuitParam &param,//the parameters
									 SkIntegerStack &idStack,//the stack for ids
									 double dImgMean, double dImgStd,//image mean and std
 									 const char *pchProjName,//the project name
									 double dMinBaseCoef);
int skGetBaseIDNeedAdrNbr(SkIntegerStack & stack, int a, SkBaseMap *pBaseMap);
double skUpdateResidualImage(SkBase &base,//the base
						  Matrix<float> &imgRes, //the residual image
						  SkFilterBank *pFilterBank,//filter bank
						  int flag, int nBound=0);//ADD or REMOVE the base
int skSynthesizeImgFromBases(SkBaseMap *pBaseMap,//the base map
							 Matrix<float> *M, //the image
							 SkFilterBank *pFilterBank,//the filter bank
							 int flag);//KER or SMB symbolic
int skAddBase2Img(SkBaseNode *pBaseNode, //the base node will be added
				  Matrix<float> *M, //the image
				  SkFilterBank *pFilterBank, //the filter bank
				  int flag, //SMB - symbolic, SYN - synthesis
				  int nConsiderAtomOrt=0, //1-Yes, 0-Not, default-0, we rotate the base as its atomort before add it
				  int x0=0, int y0=0);//the atom center
int skPossibleAdrNbrBase(int nNodeID, //the base node id
						 int a, //nbr id
						 SkFilterCoef *pFC, //filter coef
						 SkBaseMap *pBaseMap, //base map
						 SkFilterBank *pFB, //filter bank
						 double dMinBaseCoef, //min base coef
						 SkBase* pBaseCand,//candidates
						 double *pdProb);//prob for each candidates
int skPossibleAdrNbrBase(int a, //nbr #
						SkBase* pBaseCand, //candidates
						double *pdProb, //probability
						int l, int w, //length and width of the base
						double dOrtUnit);//the orientaion unit
int skChooseOneAdrNbrBase(SkBase *pBaseCand, //base candidates
						   double *pdProb, //the probability
						   int n,//candidate #
						   Matrix<oval_IntArray> *pOcpMat, //ocupaccy matrix
						   SkBaseMap *pBaseMap,//base map
						   SkFilterBank* pFB);//filter bank

int skSaveBaseMap2ShowBottomUpProcess(SkBaseMap *pBaseMap,//the current base map
 									  const char *pchProjName,//the project name
									  const char *pPreName, //pre-name
									  SkFilterBank* pFB, //filter bank
									  double dImgMean, double dImgStd);//image mean and variance


int skSketchPursuitOnSelectedImage(Matrix<float> &imgOrg,//original image
								   double dImgMean0, double dImgStd0,//original image mean and std
								   const char* skps_dir,//sketch pursuit working directory
								   const char* pchDir, // working directory
								   const char* pchProjName,//project name
								   int nSelect);//0-original, 1-whitened, 2-edge strength, 3-entropy image

int skComputeEntropy(const char *pchImgName);
int skComputeEntropyImage(Matrix<float> &imgOrg,//original image
								   double dImgMean0, double dImgStd0,//original image mean and std
								   const char* pchDir, // working directory
								   const char* pchProjName);
int skSaveEntropyPatchWithScale(FILE *pFile);

int skSaveSketchGraph2CurveMapFile(const char *name, SkSketchGraph &sketchGraph);
int skSaveSketchGraph2CurveMap(SkSketchGraph &sketchGraph, SkCurveMap &curveMap);

int skSavePatchesFromBaseMap(SkBaseMap *pBaseMap, //base map
							 const char *pchProjName, //proj name
							 int ht_patch, int wd_patch,//patch hight and width
							 Matrix<float> &img_org,
							 Matrix<float> &img_wht);

double skUpdateResidualImageByPatch(SkBase &base,//the base
						  Matrix<float> &imgRes, //the residual image
						  SkFilterBank *pFilterBank,//filter bank
  						  int nBound, double &dCoef);
double skUpdateResidualImageByPatchWithDictionary(SkBase &base,//the base
						  Matrix<float> &imgRes, //the residual image
						  SkFilterBank *pFilterBank,//filter bank
						  int nBound,
						  Matrix<float>& dict);

int skSaveResidualErrorBaseByBase(const char *name, SkBaseMap* pBaseMap, 
								  Matrix<float> *pImgOrg, SkFilterBank *pFB,
  								  double dImgMean, double dImgStd, const char *proj);
int skSaveResidualErrorPatchByPatch(const char *name, SkBaseMap* pBaseMap, 
								  Matrix<float> *pImgOrg, SkFilterBank *pFB,
								  double dImgMean, double dImgStd, const char *proj);
int skSaveResidualErrorPatchByPatchWithDictionary(const char *name, SkBaseMap* pBaseMap, 
												  Matrix<float> *pImgOrg, SkFilterBank *pFB, 
												  double dImgMean, double dImgStd,
												  Matrix<float>& dict, //the dictionary
												  int nDictNo,
												  const char *proj);

int skBestMatch2Dictionary(Matrix<float> &dict, Matrix<float>& sample, double &a, double &b);
int skComputeBestPatchSize4AverageModeling(Matrix<float> &mat);

int skComputeDerivativeEntropy(Matrix<int>& img, const char *proj);


#endif
