#include <stdio.h>
#include <assert.h>

#include "inlinefunc.h"
#include "skCurve.h"
#include "skOutput.h"

int SkCurveMap::save(const char* name)
{
	FILE *pFile;
	pFile = fopen( name, "w" );
	if( pFile == NULL )
	{
		char msg[512];
		sprintf(msg, "Write curve map file %s failure", name);
		skOutput(msg);
		return 1;
	}
	fprintf( pFile, "image width = %d image height = %d \n", m_nImgWd, m_nImgHt );
	int nSize = m_curveArray.getSize();
	fprintf( pFile, "curve num = %d \n\n", nSize );
	int pt_num;
	for( int i = 0; i < nSize; i++ )
	{
		pt_num = m_curveArray[i].p.getSize();
		fprintf( pFile, "curve %d:\n", i);
		fprintf( pFile, "    point num = %d\n", pt_num );
		for (int j = 0; j < pt_num; j++)
		{
			fprintf( pFile, "        (%d, %d) h=%d t=%d\n", 
				m_curveArray[i].p[j].x, m_curveArray[i].p[j].y, 
				m_curveArray[i].h[j], m_curveArray[i].t[j] );
		}
		fprintf( pFile, "\n");
	}
	fclose( pFile );

	return 0;
}
int SkCurveMap::read(const char* name)
{
	FILE *pFile;
	pFile = fopen( name, "r" );
	if( pFile == NULL )
	{
		char msg[512];
		sprintf(msg, "Read curve map file %s failure", name);
		skOutput(msg);
		return 1;
	}
	fscanf( pFile, "image width = %d image height = %d \n", &m_nImgWd, &m_nImgHt );
	int nSize, nPtNum;
	fscanf( pFile, "curve num = %d \n\n", &nSize );
	m_curveArray.setSize(nSize);
	int j;
	int x, y, h, t;
	for( int i = 0; i < nSize; i++ )
	{
		fscanf( pFile, "curve %d:\n", &j);
		fscanf( pFile, "    point num = %d\n", &nPtNum );
		m_curveArray[i].p.setSize(nPtNum);
		m_curveArray[i].h.setSize(nPtNum);
		m_curveArray[i].t.setSize(nPtNum);
		for (int j = 0; j < nPtNum; j++)
		{
			fscanf( pFile, "        (%d, %d) h=%d t=%d\n", &x, &y, &h, &t );

			m_curveArray[i].p[j].x = x;
			m_curveArray[i].p[j].y = y;
			m_curveArray[i].h[j] = h;
			m_curveArray[i].t[j] = t;
		}
		fscanf( pFile, "\n");
	}
	fclose( pFile );
	return 0;
}
///////////////////
// upsideDownPoints()
// change the coordinates of the points
int SkCurveMap::upsideDownPoints()
{
	int ht = m_nImgHt;

	int i, j;
	int curveNum = m_curveArray.getSize(), pt_num;
	for (i = 0; i < curveNum; i++)
	{
		pt_num = m_curveArray[i].p.getSize();
		for (j = 0; j < pt_num; j++)
		{
			m_curveArray[i].p[j].y = ht - 1 - m_curveArray[i].p[j].y;
		}
	}
	return 0;
}
///////////////////
// setCurveHalfWidth()
//  change the half width for each point
int SkCurveMap::setCurveHalfWidth(int h)
{
	int ht = m_nImgHt;

	int i, j;
	int pt_num;
	int curveNum = m_curveArray.getSize();
	for (i = 0; i < curveNum; i++)
	{
		pt_num = m_curveArray[i].p.getSize();
		for (j = 0; j < pt_num; j++)
		{
			m_curveArray[i].h[j] = h;
		}
	}
	return 0;
}
///////////////////
// saveAsMask2Matrix()
// 0 ~ points on the curve, 255 ~ not the curve
int SkCurveMap::saveAsMask2Matrix(Matrix<int> &mat)
{
	int wd = m_nImgWd;
	int ht = m_nImgHt;
	mat.SetDimension(ht, wd);
	mat.Set(255);

	int x, y;
	int x1, y1, x2, y2;
	int pnX[4096], pnY[4096];

	int i, j, k;
	int line_num, n, w;
	int curveNum = m_curveArray.getSize();
	for (i = 0; i < curveNum; i++)
	{
		line_num = m_curveArray[i].getLineNum();
		for (j = 0; j < line_num; j++)
		{
			x1 = m_curveArray[i].p[j].x;
			y1 = m_curveArray[i].p[j].y;
			x2 = m_curveArray[i].p[j+1].x;
			y2 = m_curveArray[i].p[j+1].y;//start and end point
			w = (m_curveArray[i].h[j] + m_curveArray[i].h[j+1])/2 *2 +1;//line width
			n = ::rasterize_line_segment(x1, y1, x2, y2, pnX, pnY, w);
			assert(n <= 4096);
			for (k = 0; k < n; k++)
			{
				x = pnX[k];
				y = pnY[k];
				if (x < 0  || x >= wd || y < 0 || y >= ht)
					continue;
				mat(y, x) = 0;
			}
		}
	}
	return 0;
}
///////////////////
// scaleDownByFactor2()
int SkCurveMap::scaleDownByFactor2()
{
	m_nImgHt /= 2;
	m_nImgWd /= 2;

	int n = m_curveArray.getSize();
	for (int i = 0; i < n; i++)
	{
		int pt_num = m_curveArray[i].p.getSize();
		for (int j = 0; j < pt_num; j++)
		{
			m_curveArray[i].p[j].x /= 2;
			m_curveArray[i].p[j].y /= 2;
		}
	}
	return 0;
}

///////////////////
// 	removeShorCurve()
int SkCurveMap::removeShorCurve(double dMinCurveLength)
{
	int n = m_curveArray.getSize();
	for (int i = 0; i < n; i++)
	{
		double dLength = m_curveArray[i].getLength();
		if (dLength < dMinCurveLength)
		{
			m_curveArray.RemoveAt(i);
			i--;
			n--;
		}
	}
	return 0;
}
