#ifndef _SK_CURVE_H_
#define _SK_CURVE_H_
#include "skPrim.h"
#include "skMatrix.h"
#include "skTemplate.h"

class SkCurvePoint
{
public:
	int x, y;
};


class SkCurve
{
public:
	oval_Array<SkCurvePoint> p;//point
	oval_Array<int> h;//half with
	oval_Array<int> t;//type
public:
	SkCurve() { clear(); }
	void clear() {p.removeAll(); h.removeAll(); t.removeAll();}
	int addPoint(int x, int y, int w, int tt) { SkCurvePoint pt; pt.x = x; pt.y = y; p.add(pt); h.add(w); t.add(tt); return p.getSize(); }
	int removePoint(int b) {p.removeAt(b); h.removeAt(b); t.removeAt(b); return p.getSize(); }//b: index of point
	int getLineNum() { int n=p.getSize(); if (n <= 1) return 0; return n-1; }
	double getLength()
	{
		int n=p.getSize(); int x0=p[0].x; int y0=p[0].y;
		int x1=p[n-1].x; int y1=p[n-1].y;
		return sqrt(double((x1-x0)*(x1-x0)+(y1-y0)*(y1-y0)));
	}
	void reversePoints()
	{ 
		int n = p.getSize(); 
		for (int i = 0; i < n/2; i++)
		{
			::swap(p[i],p[n-i-1]);
			::swap(h[i],h[n-i-1]);
			::swap(t[i],t[n-i-1]);
		}
	}
	void getStartPoint(int &x, int &y){if (p.getSize() <= 0) return; x=p[0].x; y=p[0].y; }
	void getEndPoint(int &x, int &y){int n=p.getSize(); if (n < 1) return; x=p[n-1].x; y=p[n-1].y; }
};


class SkCurveMap
{
public:
	oval_Array<SkCurve> m_curveArray;
	int m_nImgWd, m_nImgHt;
public:
	SkCurveMap() { m_nImgWd = 0, m_nImgHt = 0;}
	int add(SkCurve &c) {m_curveArray.add(c); return m_curveArray.getSize(); }
	int removeLast() {m_curveArray.removeAt(m_curveArray.getSize()-1); return m_curveArray.getSize(); }
	void setImageSize(int w, int h) {m_nImgWd = w; m_nImgHt = h; }
	int save(const char* name);
	int read(const char* name);
	int getLineNum() { int s=0; int n=m_curveArray.getSize(); for (int i=0; i<n;i++) s+=m_curveArray[i].getLineNum(); return s; }
	int getCurveNum() { return m_curveArray.getSize(); }
	SkCurve &getCurveAt(int i) { return m_curveArray[i]; }
	int scaleDownByFactor2();
	int removeShorCurve(double dMinCurveLength);

	int saveAsMask2Matrix(Matrix<int> &mat);
	int upsideDownPoints();
	int setCurveHalfWidth(int h);
	void removeAll() { m_curveArray.removeAll(); }
};

#endif//_SK_PRIM_H_
