#include <direct.h>

#include "inlinefunc.h"
#include "skFilter.h"
#include "skMatrixIplOp.h"
#include "skIOImage.h"


double SkFilterBank::m_dLPNorm = 2;

/////////////////////////////////////////////////////////////////////
// class SkFilterBankParam

///////////////////
// Constructor
SkFilterBankParam::SkFilterBankParam()
{
	removeAllFilters();
}
///////////////////
// de-constructor
SkFilterBankParam::~SkFilterBankParam()
{
}
int SkFilterBankParam::removeAllFilters()
{
	m_nUseDeltaFunc = 0;//use delta func or not
	m_nUseFirDerX = 0;//use first derivative X or not
	m_nUseFirDerY = 0;//use first derivative Y or not
	m_nUseSecDerX = 0;//use second derivative X or not
	m_nUseSecDerY = 0;//use second derivative Y or not
	m_nLogNum = 0;//laplacian gaussian nume
	m_nDOOGScaleNum = 0;
	m_nUseDOOG = 0;//use DOOG or not
	m_nUseDOOG1 = 0;//use DOOG1 or not
	m_nGaborScaleNum = 0;
	m_nUseGSin = 0;//use Gabor sin or not
	m_nUseGCos = 0;//use Gabor cos or not
	m_nDogNum = 0;// dog num
	m_nUseImage = 0;//use image filter or not
	m_nImgFilterScaleNum = 0;
	m_nImgFilterTypeNum = 0;
	m_nImgFilterScaleNum = 0;
	m_nBarEdgeScaleNum = 0;
	m_nUseBar = 0;//use bar or not
	m_nUseEdge = 0;//use edge or not

	m_nFBType = SK_NOT_CARE_FB;
	return 0;
}
///////////////////
// read()
int SkFilterBankParam::read(const char *name)
{
	int i;
	FILE *pFile = fopen(name, "r");
	if (pFile == 0)
		return 1;
	fscanf(pFile, "Use Delta Func:               %d\n", &m_nUseDeltaFunc);//use delta func or not
	fscanf(pFile, "Use First Derivative X:       %d\n", &m_nUseFirDerX);//use first derivative X or not
	fscanf(pFile, "Use First Derivative Y:       %d\n", &m_nUseFirDerY);//use first derivative Y or not
	fscanf(pFile, "Use Second Derivative X:      %d\n", &m_nUseSecDerX);//use second derivative X or not
	fscanf(pFile, "Use Second Derivative Y:      %d\n", &m_nUseSecDerY);//use second derivative Y or not
	fscanf(pFile, "Use DOOG:               %d\n", &m_nUseDOOG);//use DOOG or not
	fscanf(pFile, "Use DOOG1:             %d\n", &m_nUseDOOG1);//use DOOG1 or not
	
	fscanf(pFile, "Laplacian Gaussian Num:       %d\n", &m_nLogNum);//laplacian gaussian nume
	fscanf(pFile, "    LOG Scales:             \n");
	fscanf(pFile, "        ");
	for (i = 0; i < m_nLogNum; i++)
	{
		fscanf(pFile, "%lf ", &(m_pdLogScales[i]));
	}
	fscanf(pFile, "\n");

	fscanf(pFile, "DOOG Scale Num:              %d\n", &m_nDOOGScaleNum);//scale num for DOOG filters
	
	fscanf(pFile, "    DOOG Scales:             \n");
	fscanf(pFile, "        ");
	for (i = 0; i < m_nDOOGScaleNum; i++)
	{
		fscanf(pFile, "%lf ", &(m_pdDOOGScales[i]));
	}
	fscanf(pFile, "\n");
	
	fscanf(pFile, "    DOOG Elongations:             \n");
	fscanf(pFile, "        ");
	for (i = 0; i < m_nDOOGScaleNum; i++)
	{
		fscanf(pFile, "%lf ", &(m_pdDOOGElongs[i]));
	}
	fscanf(pFile, "\n");
	
	fscanf(pFile, "    DOOG Rotations:             \n");
	fscanf(pFile, "        ");
	for (i = 0; i < m_nDOOGScaleNum; i++)
	{
		fscanf(pFile, "%d ", &(m_pnDOOGRotates[i]));
	}
	fscanf(pFile, "\n");
	
	fscanf(pFile, "DOG Num:                      %d\n", &m_nDogNum);// dog num
	
	fscanf(pFile, "    DOG Scales:             \n");
	fscanf(pFile, "        ");
	for (i = 0; i < m_nDogNum; i++)
	{
		fscanf(pFile, "%lf ", &(m_pdDogScales[i]));
	}
	fscanf(pFile, "\n");
	
	fscanf(pFile, "Sketch Width:                 %d\n", &m_nSketchWidth);//sketch width to draw symbolic

	fscanf(pFile, "\n");
	fscanf(pFile, "Use Bar:                      %d\n", &m_nUseBar);//use bar or not
	fscanf(pFile, "Use Edge:                     %d\n", &m_nUseEdge);//use edge or not
	fscanf(pFile, "Bar Edge Scale Num:           %d\n", &m_nBarEdgeScaleNum);//scale num for DOOG filters

	fscanf(pFile, "    Bar Edge Scales:             \n");
	fscanf(pFile, "        ");
	for (i = 0; i < m_nBarEdgeScaleNum; i++)
	{
		fscanf(pFile, "%lf ", &(m_pdBarEdgeScales[i]));
	}
	fscanf(pFile, "\n");
	
	fscanf(pFile, "    Bar Edge Elongations:             \n");
	fscanf(pFile, "        ");
	for (i = 0; i < m_nBarEdgeScaleNum; i++)
	{
		fscanf(pFile, "%lf ", &(m_pdBarEdgeElongs[i]));
	}
	fscanf(pFile, "\n");
	
	fscanf(pFile, "    Bar Edge Rotations:             \n");
	fscanf(pFile, "        ");
	for (i = 0; i < m_nBarEdgeScaleNum; i++)
	{
		fscanf(pFile, "%d ", &(m_pnBarEdgeRotates[i]));
	}
	fscanf(pFile, "\n");

	fclose(pFile);

	return 0;
}
///////////////////
// save()
int SkFilterBankParam::save(const char *name)
{
	int i;
	FILE *pFile = fopen(name, "w");
	if (pFile == 0)
		return 1;
	fprintf(pFile, "Use Delta Func:               %d\n", m_nUseDeltaFunc);//use delta func or not
	fprintf(pFile, "Use First Derivative X:       %d\n", m_nUseFirDerX);//use first derivative X or not
	fprintf(pFile, "Use First Derivative Y:       %d\n", m_nUseFirDerY);//use first derivative Y or not
	fprintf(pFile, "Use Second Derivative X:      %d\n", m_nUseSecDerX);//use second derivative X or not
	fprintf(pFile, "Use Second Derivative Y:      %d\n", m_nUseSecDerY);//use second derivative Y or not
	fprintf(pFile, "Use DOOG:                     %d\n", m_nUseDOOG);//use DOOG or not
	fprintf(pFile, "Use DOOG1:                    %d\n", m_nUseDOOG1);//use DOOG1 or not
	
	fprintf(pFile, "Laplacian Gaussian Num:       %d\n", m_nLogNum);//laplacian gaussian nume
	fprintf(pFile, "    LOG Scales:             \n");
	fprintf(pFile, "        ");
	for (i = 0; i < m_nLogNum; i++)
	{
		fprintf(pFile, "%.2f ", m_pdLogScales[i]);
	}
	fprintf(pFile, "\n");

	fprintf(pFile, "DOOG Scale Num:              %d\n", m_nDOOGScaleNum);//scale num for DOOG filters
	
	fprintf(pFile, "    DOOG Scales:             \n");
	fprintf(pFile, "        ");
	for (i = 0; i < m_nDOOGScaleNum; i++)
	{
		fprintf(pFile, "%.2lf ", (m_pdDOOGScales[i]));
	}
	fprintf(pFile, "\n");
	
	fprintf(pFile, "    DOOG Elongations:             \n");
	fprintf(pFile, "        ");
	for (i = 0; i < m_nDOOGScaleNum; i++)
	{
		fprintf(pFile, "%.2lf ", (m_pdDOOGElongs[i]));
	}
	fprintf(pFile, "\n");
	
	fprintf(pFile, "    DOOG Rotations:             \n");
	fprintf(pFile, "        ");
	for (i = 0; i < m_nDOOGScaleNum; i++)
	{
		fprintf(pFile, "%d ", (m_pnDOOGRotates[i]));
	}
	fprintf(pFile, "\n");
	
	fprintf(pFile, "DOG Num:                      %d\n", m_nDogNum);// dog num
	
	fprintf(pFile, "    DOG Scales:             \n");
	fprintf(pFile, "        ");
	for (i = 0; i < m_nDogNum; i++)
	{
		fprintf(pFile, "%.2lf ", (m_pdDogScales[i]));
	}
	fprintf(pFile, "\n");
	
	fprintf(pFile, "Sketch Width:                 %d\n", m_nSketchWidth);//sketch width to draw symbolic

	fprintf(pFile, "\n");
	fprintf(pFile, "Use Bar:                      %d\n", m_nUseBar);//use bar or not
	fprintf(pFile, "Use Edge:                     %d\n", m_nUseEdge);//use edge or not
	fprintf(pFile, "Bar Edge Scale Num:           %d\n", m_nBarEdgeScaleNum);//scale num for edge bar filters

	fprintf(pFile, "    Bar Edge Scales:             \n");
	fprintf(pFile, "        ");
	for (i = 0; i < m_nBarEdgeScaleNum; i++)
	{
		fprintf(pFile, "%.2lf ", (m_pdBarEdgeScales[i]));
	}
	fprintf(pFile, "\n");
	
	fprintf(pFile, "    Bar Edge Elongations:             \n");
	fprintf(pFile, "        ");
	for (i = 0; i < m_nBarEdgeScaleNum; i++)
	{
		fprintf(pFile, "%.2lf ", (m_pdBarEdgeElongs[i]));
	}
	fprintf(pFile, "\n");
	
	fprintf(pFile, "    Bar Edge Rotations:             \n");
	fprintf(pFile, "        ");
	for (i = 0; i < m_nBarEdgeScaleNum; i++)
	{
		fprintf(pFile, "%d ", (m_pnBarEdgeRotates[i]));
	}
	fprintf(pFile, "\n");

	int n = getTotalFilterNum();

	fprintf(pFile,"\nTotal Filter Num (computed not set by user):  %d\n", n);

	fclose(pFile);
	return 0;
}
int SkFilterBankParam::initilaize4PrimPursuit()//int nScale, int nScaleNum)
{
	removeAllFilters();
	m_nFBType = SK_PRIM_PURSUIT_FB;

	m_nBuildCorr = 0;
	m_nSketchWidth = 3;//sketch width to draw symbolic

	m_nLogNum = 5;//laplacian gaussian nume
	for (int i = 0; i < m_nLogNum; i++)
		m_pdLogScales[i] = i*2+3.0;

	m_nUseDOOG = 1;//use DOOG or not
	m_nUseDOOG1 = 1;//use DOOG1 or not
	m_nDOOGScaleNum = 3;//scale num for DOOG filters

	double dMaxElong = 6;
	double dElong = dMaxElong/pow(2.f, 1);//3
	m_pdDOOGScales[0]=1.0; m_pdDOOGScales[1]=1.5; m_pdDOOGScales[2]=2.0; m_pdDOOGScales[3]=3.0;
	m_pdDOOGElongs[0]=dElong; m_pdDOOGElongs[1]=dElong; m_pdDOOGElongs[2]=dElong; m_pdDOOGElongs[3]=dElong;
//	m_pdDOOGElongs[0]=3.0; m_pdDOOGElongs[1]=2.0; m_pdDOOGElongs[2]=1.5; m_pdDOOGElongs[3]=1.0;

		
	m_pnDOOGRotates[0]=18; m_pnDOOGRotates[1]=18; m_pnDOOGRotates[2]=18; m_pnDOOGRotates[3]=18;
	
	m_nDogNum = 0;// dog num
	for (int i = 0; i < m_nDogNum; i++)
		m_pdDogScales[i] = i+2.0;

	return 0;
}
int SkFilterBankParam::initilaize4REBDetection(int reb)//int nScale, int nScaleNum)
{
	removeAllFilters();
	m_nFBType = SK_REB_DETECTION;

	m_nBuildCorr = 0;
	m_nSketchWidth = 3;//sketch width to draw symbolic

	double dMaxElong = 6;
	double dElong = dMaxElong/pow(2.f, 1);//3
	int i;
	switch (reb)
	{
	case 0://rige
		m_nUseDOOG = 1;//use DOOG or not
		m_nDOOGScaleNum = 5;//scale num for DOOG filters
		m_pdDOOGScales[0]=1.0; m_pdDOOGScales[1]=1.5; m_pdDOOGScales[2]=2.0; m_pdDOOGScales[3]=3.0;m_pdDOOGScales[4]=4.0;m_pdDOOGScales[5]=5.0;
		m_pdDOOGElongs[0]= dElong; m_pdDOOGElongs[1]=dElong; m_pdDOOGElongs[2]=dElong; m_pdDOOGElongs[3]=dElong;m_pdDOOGElongs[4]=dElong;m_pdDOOGElongs[5]=dElong;
		m_pnDOOGRotates[0]=18; m_pnDOOGRotates[1]=18; m_pnDOOGRotates[2]=18; m_pnDOOGRotates[3]=18;m_pnDOOGRotates[4]=18;m_pnDOOGRotates[5]=18;
		break;
	case 1:
		m_nUseDOOG1 = 1;//use DOOG1 or not
		m_nDOOGScaleNum = 3;//scale num for DOOG filters
		m_pdDOOGScales[0]=1.0; m_pdDOOGScales[1]=1.5; m_pdDOOGScales[2]=2.0; m_pdDOOGScales[3]=3.0;
		m_pdDOOGElongs[0]= dElong; m_pdDOOGElongs[1]=dElong; m_pdDOOGElongs[2]=dElong; m_pdDOOGElongs[3]=dElong;
		m_pnDOOGRotates[0]=18; m_pnDOOGRotates[1]=18; m_pnDOOGRotates[2]=18; m_pnDOOGRotates[3]=18;
		break;
	case 2:
		m_nLogNum = 5;//laplacian gaussian nume
		for (i = 0; i < m_nLogNum; i++)
			m_pdLogScales[i] = i*2+3.0;
	}

	return 0;
}
int SkFilterBankParam::initilaize4BlobPursuit()
{
	removeAllFilters();

	m_nFBType = SK_BLOB_PURSUIT_FB;

	m_nBuildCorr = 0;
	m_nSketchWidth = 3;//sketch width to draw symbolic

	m_nDogNum = 5;// dog num

	for (int i = 0; i < m_nDogNum; i++)
		m_pdDogScales[i] = i+2.0;

	return 0;
}

int SkFilterBankParam::initilaize2ComputeEntropy(int nScale)
{
	removeAllFilters();
	m_nBuildCorr = 0;
	m_nSketchWidth = 3;//sketch width to draw symbolic

	m_nUseDOOG = 1;//use DOOG or not
	m_nUseDOOG1 = 1;//use DOOG1 or not
	m_nDOOGScaleNum = 1;//scale num for DOOG filters

	double dMaxElong = 6;
	double dElong = dMaxElong/pow(2.f, nScale);
	m_pdDOOGScales[0]=1.0; m_pdDOOGScales[1]=1.5; m_pdDOOGScales[2]=2.0;
	m_pdDOOGElongs[0]=dElong; m_pdDOOGElongs[1]=dElong/1.5; m_pdDOOGElongs[2]=dElong/2.0;

		
	m_pnDOOGRotates[0]=18; m_pnDOOGRotates[1]=18; m_pnDOOGRotates[1]=18;
	
	m_nDogNum = 0;// dog num
	return 0;
}
int SkFilterBankParam::initilaize4BasePursuit(int nSelect)
{
	removeAllFilters();
	m_nBuildCorr = 1;

	double m_k; 

	m_nSketchWidth = 3;//sketch width to draw symbolic

	int i;

	switch (nSelect)
	{
	case 0://orignal
		m_nDogNum = 0;// dog num
		m_nUseGSin = 1;//use GSin or not
		m_nUseGCos = 1;//use DOOG or not
		m_nGaborScaleNum = 3;//scale num for DOOG filters

	//m_nBuildCorr = 0;

		m_pdGaborK[0]=1.0; m_pdGaborK[1]=0.75; m_pdGaborK[2]=0.5; 
		m_pdGaborK[3]=1.0; m_pdGaborK[4]=0.5; m_pdGaborK[5]=0.25; 
		m_pdGaborK[6]=1; m_pdGaborK[7]=3;
//		m_k= 0.625;
//		m_pdGaborK[0]=m_k; m_pdGaborK[1]=m_k*sqrt(2.0); m_pdGaborK[2]=m_k*2; m_pdGaborK[3]=m_k*2*sqrt(2.0); m_pdGaborK[4]=1; m_pdGaborK[5]=3; m_pdGaborK[6]=1; m_pdGaborK[7]=3;
	//	m_pdGaborSigma[0]=2; m_pdGaborSigma[1]=3; m_pdGaborSigma[2]=4; m_pdGaborSigma[3]=4; m_pdGaborSigma[4]=15; m_pdGaborSigma[5]=5; m_pdGaborSigma[6]=18; m_pdGaborSigma[7]=6;
		// 1.5 octive bandwidth: sigma = 2.5/omega;
		for (i = 0; i < m_nGaborScaleNum; i++)
			m_pdGaborSigma[i] = 2.5;//1.5 octive
		//for (i = m_nGaborScaleNum/2; i < m_nGaborScaleNum; i++)
		//	m_pdGaborSigma[i] = PI;//1.0 octive

		m_pnGaborRotates[0]=6; m_pnGaborRotates[1]=6; m_pnGaborRotates[2]=6; 
		m_pnGaborRotates[3]=6; m_pnGaborRotates[4]=6; m_pnGaborRotates[5]=6; 
		m_pnGaborRotates[6]=6; m_pnGaborRotates[7]=6;


		//s6
		m_pdDOOGScales[0]=1.5; m_pdDOOGScales[1]=1.5; m_pdDOOGScales[2]=2.0; m_pdDOOGScales[3]=2.5; m_pdDOOGScales[4]=1.5; m_pdDOOGScales[5]=1.0; m_pdDOOGScales[6]=1.5; m_pdDOOGScales[7]=1.0;
		m_pdDOOGElongs[0]=4.0; m_pdDOOGElongs[1]=3.5; m_pdDOOGElongs[2]=2.6; m_pdDOOGElongs[3]=2.5; m_pdDOOGElongs[4]=2.5; m_pdDOOGElongs[5]=3.0; m_pdDOOGElongs[6]=2.0; m_pdDOOGElongs[7]=2.0;
		//s4
	//	m_pdDOOGScales[0]=1.0; m_pdDOOGScales[1]=0.8; m_pdDOOGScales[2]=1.0; m_pdDOOGScales[3]=0.75; m_pdDOOGScales[4]=1.5; m_pdDOOGScales[5]=1.0; m_pdDOOGScales[6]=1.5; m_pdDOOGScales[7]=1.0;
	//	m_pdDOOGElongs[0]=4.0; m_pdDOOGElongs[1]=5.0; m_pdDOOGElongs[2]=3.0; m_pdDOOGElongs[3]=4.0; m_pdDOOGElongs[4]=2.5; m_pdDOOGElongs[5]=3.0; m_pdDOOGElongs[6]=2.0; m_pdDOOGElongs[7]=2.0;
		
		m_pnDOOGRotates[0]=18; m_pnDOOGRotates[1]=12; m_pnDOOGRotates[2]=8; m_pnDOOGRotates[3]=12; m_pnDOOGRotates[4]=12; m_pnDOOGRotates[5]=12; m_pnDOOGRotates[6]=12; m_pnDOOGRotates[7]=6;
		break;
	case 1://whitened image
		m_nUseDOOG = 1;//use DOOG or not
		m_nUseDOOG1 = 0;//use DOOG1 or not
		m_nDOOGScaleNum = 2;//scale num for DOOG filters
		//bigger
//		m_pdDOOGScales[0]=1.5; m_pdDOOGScales[1]=2.0; m_pdDOOGScales[2]=1.0; m_pdDOOGScales[3]=1.5; m_pdDOOGScales[4]=1.5; m_pdDOOGScales[5]=1.0; m_pdDOOGScales[6]=1.5; m_pdDOOGScales[7]=1.0;
//		m_pdDOOGElongs[0]=2.7; m_pdDOOGElongs[1]=2.0; m_pdDOOGElongs[2]=4.0; m_pdDOOGElongs[3]=2.7; m_pdDOOGElongs[4]=2.5; m_pdDOOGElongs[5]=3.0; m_pdDOOGElongs[6]=2.0; m_pdDOOGElongs[7]=2.0;
		//low
		m_pdDOOGScales[0]=1.0; m_pdDOOGScales[1]=1.5; m_pdDOOGScales[2]=1.0; m_pdDOOGScales[3]=1.5; m_pdDOOGScales[4]=1.5; m_pdDOOGScales[5]=1.0; m_pdDOOGScales[6]=1.5; m_pdDOOGScales[7]=1.0;
		m_pdDOOGElongs[0]=4.0; m_pdDOOGElongs[1]=2.7; m_pdDOOGElongs[2]=4.0; m_pdDOOGElongs[3]=2.7; m_pdDOOGElongs[4]=2.5; m_pdDOOGElongs[5]=3.0; m_pdDOOGElongs[6]=2.0; m_pdDOOGElongs[7]=2.0;
		//high
//		m_pdDOOGScales[0]=1.5; m_pdDOOGScales[1]=2; m_pdDOOGScales[2]=1.0; m_pdDOOGScales[3]=1.5; m_pdDOOGScales[4]=1.5; m_pdDOOGScales[5]=1.0; m_pdDOOGScales[6]=1.5; m_pdDOOGScales[7]=1.0;
//		m_pdDOOGElongs[0]=2.0; m_pdDOOGElongs[1]=1.5; m_pdDOOGElongs[2]=3.0; m_pdDOOGElongs[3]=2.0; m_pdDOOGElongs[4]=2.5; m_pdDOOGElongs[5]=3.0; m_pdDOOGElongs[6]=2.0; m_pdDOOGElongs[7]=2.0;
	
		m_pnDOOGRotates[0]=18; m_pnDOOGRotates[1]=18; m_pnDOOGRotates[2]=18; m_pnDOOGRotates[3]=18; m_pnDOOGRotates[4]=12; m_pnDOOGRotates[5]=12; m_pnDOOGRotates[6]=12; m_pnDOOGRotates[7]=6;
//		m_pnDOOGRotates[0]=1; m_pnDOOGRotates[1]=1; m_pnDOOGRotates[2]=9; m_pnDOOGRotates[3]=12; m_pnDOOGRotates[4]=12; m_pnDOOGRotates[5]=12; m_pnDOOGRotates[6]=12; m_pnDOOGRotates[7]=6;
	
		m_nDogNum = 0;// dog num
		break;
	case 2://edge strength
		m_nUseDOOG = 0;//use DOOG or not
		m_nUseDOOG1 = 1;//use DOOG or not
		m_nDOOGScaleNum = 2;//scale num for DOOG filters
		//s6
	//	m_pdDOOGScales[0]=1.0; m_pdDOOGScales[1]=0.75; m_pdDOOGScales[2]=1.0; m_pdDOOGScales[3]=0.75; m_pdDOOGScales[4]=1.5; m_pdDOOGScales[5]=1.0; m_pdDOOGScales[6]=1.5; m_pdDOOGScales[7]=1.0;
	//	m_pdDOOGElongs[0]=6.0; m_pdDOOGElongs[1]=8.0; m_pdDOOGElongs[2]=3.0; m_pdDOOGElongs[3]=4.0; m_pdDOOGElongs[4]=2.5; m_pdDOOGElongs[5]=3.0; m_pdDOOGElongs[6]=2.0; m_pdDOOGElongs[7]=2.0;
		//s4
		m_pdDOOGScales[0]=1.0; m_pdDOOGScales[1]=1.5; m_pdDOOGScales[2]=1.0; m_pdDOOGScales[3]=0.75; m_pdDOOGScales[4]=1.5; m_pdDOOGScales[5]=1.0; m_pdDOOGScales[6]=1.5; m_pdDOOGScales[7]=1.0;
		m_pdDOOGElongs[0]=4.0; m_pdDOOGElongs[1]=2.6; m_pdDOOGElongs[2]=3.0; m_pdDOOGElongs[3]=4.0; m_pdDOOGElongs[4]=2.5; m_pdDOOGElongs[5]=3.0; m_pdDOOGElongs[6]=2.0; m_pdDOOGElongs[7]=2.0;
		
		m_pnDOOGRotates[0]=36; m_pnDOOGRotates[1]=36; m_pnDOOGRotates[2]=9; m_pnDOOGRotates[3]=12; m_pnDOOGRotates[4]=12; m_pnDOOGRotates[5]=12; m_pnDOOGRotates[6]=12; m_pnDOOGRotates[7]=6;
		break;
	case 3://entropy image
		m_nUseDOOG = 1;//use DOOG or not
		m_nUseDOOG1 = 0;//use DOOG1 or not
		m_nDOOGScaleNum = 2;//scale num for DOOG filters
		//s6
		m_pdDOOGScales[0]=1.0; m_pdDOOGScales[1]=0.75; m_pdDOOGScales[2]=1.0; m_pdDOOGScales[3]=0.75; m_pdDOOGScales[4]=1.5; m_pdDOOGScales[5]=1.0; m_pdDOOGScales[6]=1.5; m_pdDOOGScales[7]=1.0;
		m_pdDOOGElongs[0]=6.0; m_pdDOOGElongs[1]=8.0; m_pdDOOGElongs[2]=3.0; m_pdDOOGElongs[3]=4.0; m_pdDOOGElongs[4]=2.5; m_pdDOOGElongs[5]=3.0; m_pdDOOGElongs[6]=2.0; m_pdDOOGElongs[7]=2.0;
		//s4
	//	m_pdDOOGScales[0]=1.0; m_pdDOOGScales[1]=0.8; m_pdDOOGScales[2]=1.0; m_pdDOOGScales[3]=0.75; m_pdDOOGScales[4]=1.5; m_pdDOOGScales[5]=1.0; m_pdDOOGScales[6]=1.5; m_pdDOOGScales[7]=1.0;
	//	m_pdDOOGElongs[0]=4.0; m_pdDOOGElongs[1]=5.0; m_pdDOOGElongs[2]=3.0; m_pdDOOGElongs[3]=4.0; m_pdDOOGElongs[4]=2.5; m_pdDOOGElongs[5]=3.0; m_pdDOOGElongs[6]=2.0; m_pdDOOGElongs[7]=2.0;
		
		m_pnDOOGRotates[0]=18; m_pnDOOGRotates[1]=18; m_pnDOOGRotates[2]=9; m_pnDOOGRotates[3]=12; m_pnDOOGRotates[4]=12; m_pnDOOGRotates[5]=12; m_pnDOOGRotates[6]=12; m_pnDOOGRotates[7]=6;
		break;
	case 4://matching pursuit
		/*
		m_nDogNum = 0;// dog num
		m_nUseGSin = 1;//use GSin or not
		m_nUseGCos = 1;//use DOOG or not
		m_nGaborScaleNum = 3;//scale num for DOOG filters

	//m_nBuildCorr = 0;

		m_pdGaborK[0]=1.0; m_pdGaborK[1]=0.75; m_pdGaborK[2]=0.5; 
		m_pdGaborK[3]=1.0; m_pdGaborK[4]=0.5; m_pdGaborK[5]=0.25; 
		m_pdGaborK[6]=1; m_pdGaborK[7]=3;
//		m_k= 0.625;
//		m_pdGaborK[0]=m_k; m_pdGaborK[1]=m_k*sqrt(2.0); m_pdGaborK[2]=m_k*2; m_pdGaborK[3]=m_k*2*sqrt(2.0); m_pdGaborK[4]=1; m_pdGaborK[5]=3; m_pdGaborK[6]=1; m_pdGaborK[7]=3;
	//	m_pdGaborSigma[0]=2; m_pdGaborSigma[1]=3; m_pdGaborSigma[2]=4; m_pdGaborSigma[3]=4; m_pdGaborSigma[4]=15; m_pdGaborSigma[5]=5; m_pdGaborSigma[6]=18; m_pdGaborSigma[7]=6;
		// 1.5 octive bandwidth: sigma = 2.5/omega;
		for (i = 0; i < m_nGaborScaleNum; i++)
			m_pdGaborSigma[i] = 2.5;//1.5 octive
		//for (i = m_nGaborScaleNum/2; i < m_nGaborScaleNum; i++)
		//	m_pdGaborSigma[i] = PI;//1.0 octive

		m_pnGaborRotates[0]=6; m_pnGaborRotates[1]=6; m_pnGaborRotates[2]=6; 
		m_pnGaborRotates[3]=6; m_pnGaborRotates[4]=6; m_pnGaborRotates[5]=6; 
		m_pnGaborRotates[6]=6; m_pnGaborRotates[7]=6;
*/
		m_nUseDOOG = 1;//use GSin or not
		m_nUseDOOG1 = 1;//use DOOG or not
		m_nDOOGScaleNum = 4;//scale num for DOOG filters

		//s6
		m_pdDOOGScales[0]=0.75; m_pdDOOGScales[1]=1.0; m_pdDOOGScales[2]=1.5; m_pdDOOGScales[3]=2.0; m_pdDOOGScales[4]=2.5; m_pdDOOGScales[5]=1.0; m_pdDOOGScales[6]=1.5; m_pdDOOGScales[7]=1.0;
		m_pdDOOGElongs[0]=3.0; m_pdDOOGElongs[1]=3.0; m_pdDOOGElongs[2]=3.0; m_pdDOOGElongs[3]=3.0; m_pdDOOGElongs[4]=4.0; m_pdDOOGElongs[5]=3.0; m_pdDOOGElongs[6]=2.0; m_pdDOOGElongs[7]=2.0;
//		m_pdDOOGScales[0]=1.0; m_pdDOOGScales[1]=1.5; m_pdDOOGScales[2]=2.0; m_pdDOOGScales[3]=2.0; m_pdDOOGScales[4]=2.5; m_pdDOOGScales[5]=1.0; m_pdDOOGScales[6]=1.5; m_pdDOOGScales[7]=1.0;
//		m_pdDOOGElongs[0]=3.0; m_pdDOOGElongs[1]=3.0; m_pdDOOGElongs[2]=3.0; m_pdDOOGElongs[3]=4.0; m_pdDOOGElongs[4]=4.0; m_pdDOOGElongs[5]=3.0; m_pdDOOGElongs[6]=2.0; m_pdDOOGElongs[7]=2.0;
		//s4
	//	m_pdDOOGScales[0]=1.0; m_pdDOOGScales[1]=0.8; m_pdDOOGScales[2]=1.0; m_pdDOOGScales[3]=0.75; m_pdDOOGScales[4]=1.5; m_pdDOOGScales[5]=1.0; m_pdDOOGScales[6]=1.5; m_pdDOOGScales[7]=1.0;
	//	m_pdDOOGElongs[0]=4.0; m_pdDOOGElongs[1]=5.0; m_pdDOOGElongs[2]=3.0; m_pdDOOGElongs[3]=4.0; m_pdDOOGElongs[4]=2.5; m_pdDOOGElongs[5]=3.0; m_pdDOOGElongs[6]=2.0; m_pdDOOGElongs[7]=2.0;
		
		m_pnDOOGRotates[0]=18; m_pnDOOGRotates[1]=18; m_pnDOOGRotates[2]=18; m_pnDOOGRotates[3]=18; m_pnDOOGRotates[4]=18; m_pnDOOGRotates[5]=12; m_pnDOOGRotates[6]=12; m_pnDOOGRotates[7]=6;
		
		m_nDogNum = 3;
		
		break;
	case 5://image filter matching pursuit

		m_nImgFilterTypeNum = 1;
		m_nImgFilterScaleNum = 3;

		m_pdImgFilterScales[0]=1; m_pdImgFilterScales[1]=1.5; m_pdImgFilterScales[2]=1*1.5*1.5; 
		m_pdImgFilterScales[3]=1.2*1.2; m_pdImgFilterScales[4]=2; m_pdImgFilterScales[5]=2.5;
		m_pdImgFilterScales[6]=3; m_pdImgFilterScales[7]=4;
		
		m_pdImgFilterElongs[0]=1.0; m_pdImgFilterElongs[1]=1.0; m_pdImgFilterElongs[2]=1.0; 
		m_pdImgFilterElongs[3]=1.0; m_pdImgFilterElongs[4]=1.0; m_pdImgFilterElongs[5]=1.0; 
		m_pdImgFilterElongs[6]=1.0; m_pdImgFilterElongs[7]=1.0;

		m_pnImgFilterRotates[0]=18; m_pnImgFilterRotates[1]=18; m_pnImgFilterRotates[2]=18; 
		m_pnImgFilterRotates[3]=18; m_pnImgFilterRotates[4]=18; m_pnImgFilterRotates[5]=18; 
		m_pnImgFilterRotates[6]=18; m_pnImgFilterRotates[7]=18;
/*
		m_nUseBar = 1;//use bar or not
		m_nUseEdge = 0;//use edge or not
		m_nBarEdgeScaleNum = 1;//scale num for DOOG filters
		m_pdBarEdgeScales[0]=1; m_pdBarEdgeScales[1]=3; m_pdBarEdgeScales[2]=1; m_pdBarEdgeScales[3]=3; m_pdBarEdgeScales[4]=1; m_pdBarEdgeScales[5]=3; m_pdBarEdgeScales[6]=1; m_pdBarEdgeScales[7]=3;
		m_pdBarEdgeElongs[0]=5; m_pdBarEdgeElongs[1]=5; m_pdBarEdgeElongs[2]=12; m_pdBarEdgeElongs[3]=4; m_pdBarEdgeElongs[4]=15; m_pdBarEdgeElongs[5]=5; m_pdBarEdgeElongs[6]=18; m_pdBarEdgeElongs[7]=6;
		m_pnBarEdgeRotates[0]=18; m_pnBarEdgeRotates[1]=9; m_pnBarEdgeRotates[2]=12; m_pnBarEdgeRotates[3]=12; m_pnBarEdgeRotates[4]=15; m_pnBarEdgeRotates[5]=15; m_pnBarEdgeRotates[6]=18; m_pnBarEdgeRotates[7]=18;
*/		break;
	default:
		assert(false);
		break;
	}
	for (i = 0; i < m_nDogNum; i++)
		m_pdDogScales[i] = i+2.0;
	return 0;
}

int SkFilterBankParam::initilaize4TextureClustering()
{
	removeAllFilters();
	m_nBuildCorr = 0;

	m_nUseDeltaFunc = 1;//use delta func or not
	m_nUseFirDerX = 1;//use first derivative X or not
	m_nUseFirDerY = 1;//use first derivative Y or not
	m_nUseSecDerX = 1;//use second derivative X or not
	m_nUseSecDerY = 1;//use second derivative Y or not
	m_nLogNum = 0;//laplacian gaussian nume
	m_nUseDOOG = 0;//use DOOG or not
	m_nUseDOOG1 = 0;//use DOOG1 or not
	m_nDOOGScaleNum = 0;//scale num for DOOG filters
	m_nDogNum = 0;// dog num
	m_nSketchWidth = 3;//sketch width to draw symbolic

	m_pdDOOGScales[0]=0.75; m_pdDOOGScales[1]=1.0; m_pdDOOGScales[2]=1.5; m_pdDOOGScales[3]=2.0; m_pdDOOGScales[4]=2.5; m_pdDOOGScales[5]=0.75; m_pdDOOGScales[6]=0.5; m_pdDOOGScales[7]=0.75;
	m_pdDOOGElongs[0]=2.0; m_pdDOOGElongs[1]=2.0; m_pdDOOGElongs[2]=2.0; m_pdDOOGElongs[3]=2.0; m_pdDOOGElongs[4]=2; m_pdDOOGElongs[5]=2.0; m_pdDOOGElongs[6]=1.5; m_pdDOOGElongs[7]=1.0;
	m_pnDOOGRotates[0]=4; m_pnDOOGRotates[1]=6; m_pnDOOGRotates[2]=6; m_pnDOOGRotates[3]=6; m_pnDOOGRotates[4]=6; m_pnDOOGRotates[5]=6; m_pnDOOGRotates[6]=6; m_pnDOOGRotates[7]=6;

	for (int i = 0; i < m_nDogNum; i++)
		m_pdDogScales[i] = i+2.0;

	return 0;
}


int SkFilterBankParam::initilaize4FameModel()
{
	removeAllFilters();
	m_nBuildCorr = 0;

	int i;
	m_nUseDeltaFunc = 1;//use delta func or not
	// below for simple test 
	m_nUseFirDerX = 1;//use first derivative X or not
	m_nUseFirDerY = 1;//use first derivative Y or not
	m_nUseSecDerX = 1;//use second derivative X or not
	m_nUseSecDerY = 1;//use second derivative Y or not
	m_nLogNum = 3;//laplacian gaussian nume
	for (i = 0; i < m_nLogNum; i++)
		m_pdLogScales[i] = i*2+1.0;
	m_nUseDOOG = 1;//use DOOG or not
	m_nUseDOOG1 = 1;//use DOOG1 or not
	m_nDOOGScaleNum = 1;//scale num for DOOG filters
	m_nDogNum = 0;// dog num
	m_nSketchWidth = 3;//sketch width to draw symbolic

	m_pdDOOGScales[0]=0.75; m_pdDOOGScales[1]=1.0; m_pdDOOGScales[2]=1.5; m_pdDOOGScales[3]=2.0; m_pdDOOGScales[4]=2.5; m_pdDOOGScales[5]=0.75; m_pdDOOGScales[6]=0.5; m_pdDOOGScales[7]=0.75;
	m_pdDOOGElongs[0]=2.0; m_pdDOOGElongs[1]=2.0; m_pdDOOGElongs[2]=2.0; m_pdDOOGElongs[3]=2.0; m_pdDOOGElongs[4]=2; m_pdDOOGElongs[5]=2.0; m_pdDOOGElongs[6]=1.5; m_pdDOOGElongs[7]=1.0;
	m_pnDOOGRotates[0]=2; m_pnDOOGRotates[1]=6; m_pnDOOGRotates[2]=6; m_pnDOOGRotates[3]=6; m_pnDOOGRotates[4]=6; m_pnDOOGRotates[5]=6; m_pnDOOGRotates[6]=6; m_pnDOOGRotates[7]=6;

	for (i = 0; i < m_nDogNum; i++)
		m_pdDogScales[i] = i+2.0;

	m_nUseBar = 0;//use bar or not
	m_nUseEdge = 0;//use edge or not
	m_nBarEdgeScaleNum = 4;//scale num for DOOG filters
	m_pdBarEdgeScales[0]=1; m_pdBarEdgeScales[1]=3; m_pdBarEdgeScales[2]=1; m_pdBarEdgeScales[3]=3; m_pdBarEdgeScales[4]=1; m_pdBarEdgeScales[5]=3; m_pdBarEdgeScales[6]=1; m_pdBarEdgeScales[7]=3;
	m_pdBarEdgeElongs[0]=6; m_pdBarEdgeElongs[1]=2; m_pdBarEdgeElongs[2]=9; m_pdBarEdgeElongs[3]=3; m_pdBarEdgeElongs[4]=12; m_pdBarEdgeElongs[5]=4; m_pdBarEdgeElongs[6]=15; m_pdBarEdgeElongs[7]=5;
	m_pnBarEdgeRotates[0]=6; m_pnBarEdgeRotates[1]=6; m_pnBarEdgeRotates[2]=9; m_pnBarEdgeRotates[3]=9; m_pnBarEdgeRotates[4]=12; m_pnBarEdgeRotates[5]=12; m_pnBarEdgeRotates[6]=15; m_pnBarEdgeRotates[7]=15;

	return 0;
}

int SkFilterBankParam::initilaize2ComputeFilterResponseEntropy()
{
	removeAllFilters();
	m_nBuildCorr = 0;

	m_nSketchWidth = 3;//sketch width to draw symbolic

	m_nUseDeltaFunc = 1;//use delta func or not
	m_nUseFirDerX = 1;//use first derivative X or not
	m_nUseFirDerY = 1;//use first derivative Y or not
//	m_nUseSecDerX = 1;//use second derivative X or not
//	m_nUseSecDerY = 1;//use second derivative Y or not
//	m_nLogNum = 3;//laplacian gaussian nume

//	m_nUseGCos = 1;//use bar or not
//	m_nUseGSin = 1;//use edge or not
//	m_nGaborScaleNum = 3;//scale num for Gabor filters
	m_pdGaborK[0]=0.625; m_pdGaborK[1]=0.625*sqrt(2.0); m_pdGaborK[2]=1; m_pdGaborK[3]=1; m_pdGaborK[4]=1; m_pdGaborK[5]=3; m_pdGaborK[6]=1; m_pdGaborK[7]=3;
//	m_pdGaborSigma[0]=2; m_pdGaborSigma[1]=3; m_pdGaborSigma[2]=4; m_pdGaborSigma[3]=4; m_pdGaborSigma[4]=15; m_pdGaborSigma[5]=5; m_pdGaborSigma[6]=18; m_pdGaborSigma[7]=6;
	m_pdGaborSigma[0]=2; m_pdGaborSigma[1]=2; m_pdGaborSigma[2]=2; m_pdGaborSigma[3]=2; m_pdGaborSigma[4]=15; m_pdGaborSigma[5]=5; m_pdGaborSigma[6]=18; m_pdGaborSigma[7]=6;
	m_pnGaborRotates[0]=6; m_pnGaborRotates[1]=6; m_pnGaborRotates[2]=6; m_pnGaborRotates[3]=6; m_pnGaborRotates[4]=15; m_pnGaborRotates[5]=15; m_pnGaborRotates[6]=18; m_pnGaborRotates[7]=18;
	return 0;
}

///////////////////
// getTotalFilterNum()
int SkFilterBankParam::getTotalFilterNum()
{

	int s = 0;
	s += m_nUseDeltaFunc;
	s += m_nUseFirDerX;
	s += m_nUseFirDerY;
	s += m_nUseSecDerX;
	s += m_nUseSecDerY;
	s += m_nLogNum;

	int i;
	for (i = 0; i < m_nDOOGScaleNum; i++)
	{
		s += m_pnDOOGRotates[i]*m_nUseDOOG + m_pnDOOGRotates[i]*m_nUseDOOG1;
	}
	s += m_nDogNum;
	for (i = 0; i < m_nBarEdgeScaleNum; i++)
	{
		s += m_pnBarEdgeRotates[i]*m_nUseBar + m_pnBarEdgeRotates[i]*m_nUseEdge;
	}
	for (i = 0; i < m_nImgFilterScaleNum; i++)
	{
		s += m_pnImgFilterRotates[i]*m_nImgFilterTypeNum;
	}
	for (i = 0; i < m_nGaborScaleNum; i++)
	{
		s += m_pnGaborRotates[i]*m_nUseGSin + m_pnGaborRotates[i]*m_nUseGCos;
	}
	return s;
}



///////////////////
// saveIplConvKernelFP()
int SkFilterBank::saveIplConvKernelFP(FILE *pFile, cv::Ptr<cv::Mat> iplKer)
{
	fprintf(pFile, "nCols = %d, nRows = %d, anchorX = %d, anchory = %d\n", iplKer->cols, iplKer->rows, (iplKer->cols-1)/2, (iplKer->rows-1)/2);
	fprintf(pFile, "values are:\n");
	for (int i = 0; i < iplKer->rows; i++)
	{
		for (int j = 0; j < iplKer->cols; j++)
			fprintf(pFile, "%.5f ", iplKer->at<float>(i, j));
		fprintf(pFile, "\n");
	}
	fprintf(pFile, "\n");
	return 0;
}
///////////////////
// readIplConvKernelFP()
int SkFilterBank::readIplConvKernelFP(FILE *pFile, cv::Ptr<cv::Mat> iplKer)
{
	int cols, rows, anchorX, anchorY;

	fscanf(pFile, "nCols = %d, nRows = %d, anchorX = %d, anchory = %d\n", &cols, &rows, &(anchorX), &(anchorY));
	fscanf(pFile, "values are:\n");

	if ( iplKer.empty() )
	{
		iplKer = new cv::Mat();
	}

	iplKer->create(rows, cols, CV_32FC1);

	for (int i = 0; i < rows; i++)
	{
		for (int j = 0; j < cols; j++)
			fscanf(pFile, "%f ", &(iplKer->at<float>(i, j)));
		fscanf(pFile, "\n");
	}
	fscanf(pFile, "\n");
	return 0;
}

/////////////////////////////////////////////////////////////////////
// Functions to build filters
void skNormalize2Norm1(float *B, int size, double LP_norm)
{ 
	assert(LP_norm > 0);
	// L2 norm or L1 norm
	double d = 0;
	int i, j;
	for (i=0; i<size; i++)
		for (j=0; j<size; j++)
		{
				d += pow((double)fabs(B[i*size+j]), LP_norm);//LP Norm
		}
	d = pow(d, 1/LP_norm);
	for (i=0; i<size; i++)
		for (j=0; j<size; j++)
			B[i*size+j] = (float)(B[i*size+j]/d);    // normalized to norm 1
}

///////////////////
// Gaussian
// g = G(x, s)
// Gaussian density truncated to (-3, 3) with 0 ending 
// x: the difference from the mean
// s: the scale, sigma
double SkFilterBank::G(double x, double s)
{
	double d,p,x1,g;
	d = 3; p = .9974; 
	x1 = x/s; 
	if (fabs(x1)<d)
	{ 
	   g = (exp(-x1*x1/2) - exp(-d*d/2))/sqrt(2*PI)/s; 
	   return g/(p-d*2/sqrt(2*PI)*exp(-d*d/2));//normalize the sum to one
	}
	else return 0; 
}
///////////////////
// intensity()
//	intensity
//	function [B] = intensity()
//	B: the base matrix
cv::Ptr<cv::Mat> SkFilterBank::intensity()
{
	int size = 1;	// the size of the base matrix 
	float *B=new float[size*size];

	B[0] = 1;

	cv::Ptr<cv::Mat> ker(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return ker;
}

///////////////////
// symIntensity()
//	intensity symbolic
//	function [B] = symIntensity()
//	B: the base matrix
cv::Ptr<cv::Mat> SkFilterBank::symIntensity()
{
	int size = 1;	// the size of the base matrix 
	float *B=new float[size*size];

	B[0] = -1;

	cv::Ptr<cv::Mat> sym(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return sym;
}

///////////////////
// firDer()
//	first derivative
//	function [B] = firDer(int direction)
//	direction: 0-x, 1-y
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::firDer(int direction)
{
	long i,j,size;
	double d;
	size = 3;	// the size of the base matrix 
	float *B=new float[size*size];

	//B is 1-d vector saving one column and the other
	// 0 3 6
	// 1 4 7
	// 2 5 8
	// 0 0 0
	// -1 1 0
	// 0 0 0 
	if (direction == 0)//x
	{
		B[0] = 0; B[1] = 0; B[2] = 0;
		B[3] = -1; B[4] = 1; B[5] = 0;
		B[6] = 0; B[7] = 0; B[8] = 0;
	}
	else if (direction == 1)//y
	{
		B[0] = 0; B[1] = -1; B[2] = 0;
		B[3] = 0; B[4] = 1; B[5] = 0;
		B[6] = 0; B[7] = 0; B[8] = 0;
	}
	else
		assert(false);

	skNormalize2Norm1(B, size, m_dLPNorm);

	cv::Ptr<cv::Mat> ker(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return ker;
}

///////////////////
// symFirDer()
//	first derivative symbolic
//	function [B] = symFirDer(int direction)
//	direction: 0-x, 1-y
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::symFirDer(int direction)
{
	int size = 3;	// the size of the base matrix 
	float *B=new float[size*size];

	//B is 1-d vector saving one column and the other
	// 0 3 6
	// 1 4 7
	// 2 5 8
	// 0 0 0
	// -1 1 0
	// 0 0 0 
	if (direction == 0)//x
	{
		B[0] = 0; B[1] = 0; B[2] = 0;
		B[3] = -1; B[4] = -1; B[5] = -1;
		B[6] = 0; B[7] = 0; B[8] = 0;
	}
	else if (direction == 1)//y
	{
		B[0] = 0; B[1] = -1; B[2] = 0;
		B[3] = 0; B[4] = -1; B[5] = 0;
		B[6] = 0; B[7] = -1; B[8] = 0;
	}
	else
		assert(false);

	cv::Ptr<cv::Mat> sym(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return sym;
}

///////////////////
// secDer()
//	second derivative
//	function [B] = secDer(int direction)
//	direction: 0-x, 1-y
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::secDer(int direction)
{
	long i,j,size;
	double d;
	size = 3;	// the size of the base matrix 
	float *B=new float[size*size];

	//B is 1-d vector saving one column and the other
	// 0 3 6
	// 1 4 7
	// 2 5 8
	// 0 0 0
	// -1 2 -1
	// 0 0 0 
	if (direction == 0)//x
	{
		B[0] = 0; B[1] = 0; B[2] = 0;
		B[3] = -1; B[4] = 2; B[5] = -1;
		B[6] = 0; B[7] = 0; B[8] = 0;
	}
	else if (direction == 1)//y
	{
		B[0] = 0; B[1] = -1; B[2] = 0;
		B[3] = 0; B[4] = 2; B[5] = 0;
		B[6] = 0; B[7] = -1; B[8] = 0;
	}
	else 
		assert(false);


	skNormalize2Norm1(B, size, m_dLPNorm);


	cv::Ptr<cv::Mat> ker(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return ker;
}

///////////////////
// symSecDer()
//	second derivative symbolic
//	function [B] = symSecDer(int direction)
//	direction: 0-x, 1-y
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::symSecDer(int direction)
{
	int size = 3;	// the size of the base matrix 
	float *B=new float[size*size];

	//B is 1-d vector saving one column and the other
	// 0 3 6
	// 1 4 7
	// 2 5 8
	// 0 0 0
	// -1 2 -1
	// 0 0 0 
	if (direction == 0)//x
	{
		B[0] = 0; B[3] = 0; B[6] = 0;
		B[1] = -1; B[4] = -1; B[7] = -1;
		B[2] = 0; B[5] = 0; B[8] = 0;
	}
	else if (direction == 1)//y
	{
		B[0] = 0; B[3] = -1; B[6] = 0;
		B[1] = 0; B[4] = -1; B[7] = 0;
		B[2] = 0; B[5] = -1; B[8] = 0;
	}
	else 
		assert(false);

	cv::Ptr<cv::Mat> sym(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return sym;
}
///////////////////
// LoG()
//	Laplacian Gaussian
//	function [B] = LoG(int scale)
//	scale: 0-3*3, 1-5*5, 2-7*7
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::LoG(double scale)
{
	long i,j,size;
	double d;

	double half_size = scale + 1;
	size = 2*half_size + 1;
	float *B=new float[size*size];


	double sigma = half_size/2.0;
	double x, y;
	double mean = 0, std = 0;
	double sigma_2 = sigma*sigma;
	for (i=0; i<size; i++)
	{
		y = i-half_size;
		for (j=0; j<size; j++)
		{
			 x = j-half_size;
			 B[i*size+j] = (x*x+y*y-sigma_2)*exp(-(x*x+y*y)/(2.0*sigma_2))/sigma_2;
			 mean += B[i*size+j];
		}
	}
	mean /= size*size;
	for (i=0; i<size; i++)
		for (j=0; j<size; j++)
			 B[i*size+j] -= mean;


	skNormalize2Norm1(B, size, m_dLPNorm);


	cv::Ptr<cv::Mat> ker(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;

	return ker;
}

///////////////////
// symLoG()
//	Laplacian Gaussian symbolic
//	function [B] = symLoG(int scale)
//	scale: 0-3*3, 1-5*5, 2-7*7
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::symLoG(double scale)
{
	long i,j,size;

	double half_size = scale + 1;
	size = 2*half_size + 1;
	float *B=new float[size*size];


	double sigma = half_size/2.0;
	double x, y;
	for (i=0; i<size; i++)
	{
		y = i-half_size;
		for (j=0; j<size; j++)
		{
			 x = j-half_size;
			 if (x*x + y*y <= half_size*half_size && x*x + y*y > (half_size-1)*(half_size-1))
				 B[i*size+j] = -1;
			 else
				 B[i*size+j] = 0;
		}
	}

	cv::Ptr<cv::Mat> sym(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return sym;
}
///////////////////
// dog()
//	difference of gaussian
//	function [B] = dog(sigma)
//	Center-sorround DOG base
//	sigma: scale
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::dog(double sigma)
{
	long i,j,size;
	double c,r,s,x,y,d;
	size = (int) (sigma*6);	// the size of the base matrix 
	c = sigma*3;			// the half size
	if ((size%2)==0)        // the size of B should be odd 
		size++;
	float *B=new float[size*size];

	float mean = 0;
	r = .9; s = 1;          // the sizes of center and sorround 
	for (i = 0;i<size;i++)
	{
		for (j = 0;j<size;j++)
		{
			x = (i-c)/sigma;
			y = (j-c)/sigma;    // scale x and y
			B[i*size+j] = (float)(G(x, r)*G(y, r) - G(x, s)*G(y, s));
							// difference between center and sorround
			mean += B[i*size+j];
		}
	}
	mean /= size*size;
	for (i=0; i<size; i++)
		for (j=0; j<size; j++)
			 B[i*size+j] -= mean;


	skNormalize2Norm1(B, size, m_dLPNorm);


	cv::Ptr<cv::Mat> ker(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return ker;
}
///////////////////
// symbdog() 
//	difference of gaussian - symbolic
//	function [C] = dog(sigma)
//	Center-sorround DOG base
//	sigma: scale
//	C: the symbolic base matrix
cv::Ptr<cv::Mat>  SkFilterBank::symbdog(double sigma)
{
	int i,j,size;
	double c,r,x,y;
	size = (int) (sigma*6);	// the size of the base matrix 
	c = sigma*3;			// the half size
	if ((size%2)==0)        // the size of B should be odd 
		size++;
	float *C=new float[size*size];

	for (i = 0;i<size;i++)
	{
		for (j = 0;j<size;j++)
		{
			x = (i-c);
			y = (j-c);
			r = sqrt(x*x+y*y); 
			if ((r<=sigma+1) && (r>=sigma-1)) 
				C[i*size+j]= -1;       // a circle
//			if ((r<=sigma+3) && (r>=sigma+1)) 
//				C[i*size+j]= 1;       // a circle
			else 
				C[i*size+j]=0;
		}
	}
	cv::Ptr<cv::Mat> sym(new cv::Mat(size, size, CV_32FC1, C));
	//delete [] C;
	return sym;	
}

///////////////////
// doog
//	[B] = doog(sigma, orientation, aspect)
//	Even DOOG base, i.e., second derivative 
//	sigma: scale of the Gaussian in x-direction 
//	orientation: in degree
//	aspect ratio: y:x
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::doog(double sigma,double orientation,double aspect)
{
	
	long i,j,size;
	double c,d,theta,costheta,sintheta,r,x,y;

	size = (int)(sigma * 4 * aspect)+1;     // size of the base
	c = sigma * 2 * aspect ;        // half-size
	theta = (PI * orientation)/180.;    // orientation in degree
	costheta=cos(theta);
	sintheta=sin(theta);
	
	if ((size%2)==0)         // the size of B should be odd 
		size++;
	float *B=new float[size*size];

	double mean = 0;
	r = aspect;           // aspect ratio
	for (i = 0;i<size;i++){
		for (j = 0;j<size;j++){
//			x = ((i-c) * costheta + (j-c) * sintheta)/sigma;
//			y = ((c-i) * sintheta + (j-c) * costheta)/sigma;
			x = ((i-c) * costheta + (c-j) * sintheta)/sigma;
			y = ((i-c) * sintheta + (j-c) * costheta)/sigma;
							  // rotate and scale x and y
			B[i*size+j] = (float)((2*G(x, 1)-G(x-1, 1)-G(x+1, 1))*G(y, r)); 
							  // second derivative 
			mean += B[i*size+j];
		}
	}
	mean /= size*size;
	for (i=0; i<size; i++)
		for (j=0; j<size; j++)
			 B[i*size+j] -= mean;
	

	skNormalize2Norm1(B, size, m_dLPNorm);


	cv::Ptr<cv::Mat> ker(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return ker;
}

///////////////////
// symbdoog()
//	[C] = doog(sigma, orientation, aspect) - symbolic
//	Even DOOG base, i.e., second derivative 
//	sigma: scale of the Gaussian in x-direction 
//	orientation: in degree
//	aspect ratio: y:x
//	C: the symbolic base matrix
cv::Ptr<cv::Mat>  SkFilterBank::symbdoog(double sigma,double orientation,double aspect)
{
	int nHalfShowWidth = m_param.m_nSketchWidth / 2;

	long i,j,size;
	double c,theta,costheta,sintheta,x,y;

	size = (int)(sigma * 4 * aspect)+1;     // size of the base
	c = sigma * 2 * aspect ;        // half-size
	theta = (PI * orientation)/180.;    // orientation in degree
	costheta=cos(theta);
	sintheta=sin(theta);

	if ((size%2)==0)         // the size of B should be odd 
		size++;
	float *C=new float[size*size];

	int fabsx, fabsy;
	for (i = 0;i<size;i++){
		for (j = 0;j<size;j++){
//			x = ((i-c) * costheta + (j-c) * sintheta);
//			y = ((c-i) * sintheta + (j-c) * costheta);
			x = ((i-c) * costheta + (c-j) * sintheta);
			y = ((i-c) * sintheta + (j-c) * costheta);
			if (x >= 0)
				fabsx = int(x+0.5);
			else
				fabsx = int(fabs(x-0.5));
			if (y >= 0)
				fabsy = fabs(y+0.5);
			else
				fabsy = fabs(y-0.5);
//			if ((fabs(x)<1.2)&&(fabs(y/sigma)<1.5*aspect))
			if (fabsx<=nHalfShowWidth && fabsy<=FILTER_VALID_LENTH_RATIO*c+1)
				C[i*size+j] = (float)(-1);      // a thin bar 	
			else C[i*size+j]=(float)0;
		}
	}
	cv::Ptr<cv::Mat> sym(new cv::Mat(size, size, CV_32FC1, C));
	//delete []C;
	return sym;
}

///////////////////
// doog1()
//	[B] = doog1(sigma, orientation, aspect)
//	Odd DOOG base, i.e., first derivative 
//	sigma: scale of the Gaussian in x-direction 
//	orientation: in degree
//	aspect ratio: y:x
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::doog1(double sigma,double orientation,double aspect)
{

	long i,j,size;
	double c,d,theta,costheta,sintheta,r,x,y;

	size = (int)(sigma * 4 * aspect)+1;     // size of the base
	c = sigma * 2 * aspect ;        // half-size
	theta = (PI * orientation)/180;    // orientation in degree
	costheta=cos(theta);
	sintheta=sin(theta);
	
	if ((size%2)==0)         // the size of B should be odd 
		size++;
	float *B=new float[size*size];

	double mean = 0;
	r = aspect;           // aspect ratio
	for (i = 0;i<size;i++){
		for (j = 0;j<size;j++){
			if (i==c && j == c)
				i += 0;
//			x = ((i-c) * costheta + (j-c) * sintheta)/sigma;
//			y = ((c-i) * sintheta + (j-c) * costheta)/sigma;
			x = ((i-c) * costheta + (c-j) * sintheta)/sigma;
			y = ((i-c) * sintheta + (j-c) * costheta)/sigma;
								 // rotate and scale x and y
			B[i*size+j] = (float)((G(x-1, 1)-G(x+1, 1))*G(y, r));      
                         // first derivative 
			mean += B[i*size+j];
		}
	}
	mean /= size*size;
	for (i=0; i<size; i++)
		for (j=0; j<size; j++)
			 B[i*size+j] -= mean;
	

	skNormalize2Norm1(B, size, m_dLPNorm);


	cv::Ptr<cv::Mat> ker(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return ker;
}

///////////////////
// symbdoog1
//	[C] = doog1(sigma, orientation, aspect) - symbolic
//	Odd DOOG base, i.e., first derivative 
//	sigma: scale of the Gaussian in x-direction 
//	orientation: in degree
//	aspect ratio: y:x
//	C: the symbolic base matrix
cv::Ptr<cv::Mat>  SkFilterBank::symbdoog1(double sigma,double orientation,double aspect)
{
	int nHalfShowWidth = m_param.m_nSketchWidth / 2;

	long i,j, size;
	double c,theta,costheta,sintheta,x,y;

	size = (int)(sigma * 4 * aspect)+1;     // size of the base
	c = sigma * 2 * aspect ;        // half-size
	theta = (PI * orientation)/180;    // orientation in degree
	costheta=cos(theta);
	sintheta=sin(theta);
	
	if ((size%2)==0)         // the size of B should be odd 
		size++;
	float *C=new float[size*size];
	
	int fabsx, fabsy;
	for (i = 0;i<size;i++){
		for (j = 0;j<size;j++){
//			x = ((i-c) * costheta + (j-c) * sintheta);
//			y = ((c-i) * sintheta + (j-c) * costheta);
			x = ((i-c) * costheta + (c-j) * sintheta);
			y = ((i-c) * sintheta + (j-c) * costheta);
			if (x >= 0)
				fabsx = int(x+0.5);
			else
				fabsx = int(fabs(x-0.5));
			if (y >= 0)
				fabsy = fabs(y+0.5);
			else
				fabsy = fabs(y-0.5);

//			x = ((i-c) * costheta + (j-c) * sintheta)/sigma;
//			y = ((c-i) * sintheta + (j-c) * costheta)/sigma;
//			if ((fabs(x)<1.2)&(fabs(y/sigma)<1.5*aspect))
			if (fabsx<=nHalfShowWidth &&(fabsy<=FILTER_VALID_LENTH_RATIO*c+1))
//			   C[i*size+j] = (float)((x>=0)*2-1);  // a thin edge 
			   C[i*size+j] = (float)(-1);  // a thin edge 
			else C[i*size+j]=0;
		}
	}
	cv::Ptr<cv::Mat> sym(new cv::Mat(size, size, CV_32FC1, C));
	
	//delete []C;
	return sym;
}

///////////////////
// GSine
//	[B] = GSine(k, sigma, orientation)
//	Gabor sine base,  
//  k: freqency of sine
//	sigma: scale of the Gaussian in x-direction 
//	ort: in degree
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::GSine(double k, double sigma,double ort)
{

	int even = 0;
	Matrix<float> M;
	M.SetGabor(k, sigma, ort, even);

	int wd, ht;
	M.GetDimension(ht, wd);
	assert(ht == wd);
	assert(ht / 2 * 2 + 1 == ht);
	
	int size = ht;
	float *B=new float[size*size];

	long i,j;
	double mean = 0;
	for (i = 0;i<size;i++)
	{
		for (j = 0;j<size;j++)
		{
			B[i*size+j] = M(i,j);
			mean += B[i*size+j];
		}
	}

	mean /= size*size;
	for (i=0; i<size; i++)
		for (j=0; j<size; j++)
			 B[i*size+j] -= mean;
	
	skNormalize2Norm1(B, size, m_dLPNorm);


	cv::Ptr<cv::Mat> ker(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return ker;
}


///////////////////
// symbGSine
//	[B] = GSine(k, sigma, orientation)
//	Gabor sine base,  
//  k: freqency of sine
//	sigma: scale of the Gaussian in x-direction 
//	ort: in degree
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::symbGSine(double k, double sigma,double ort)
{
	return GSine(k, sigma, ort);
}

///////////////////
// GCosine
//	[B] = GSine(k, sigma, orientation)
//	Gabor sine base,  
//  k: freqency of sine
//	sigma: scale of the Gaussian in x-direction 
//	ort: in degree
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::GCosine(double k, double sigma,double ort)
{
	int even = 1;
	Matrix<float> M;
	M.SetGabor(k, sigma, ort, even);
/*
	static kk = 0;
	char name[512];
	sprintf(name, "cos_tmp%d.bmp", kk);
	::skSaveMatrixAsBitmap(name, M, 0, -1);
	kk++;
*/
	int wd, ht;
	M.GetDimension(ht, wd);
	assert(ht == wd);
	assert(ht / 2 * 2 + 1 == ht);
	
	int size = ht;
	float *B=new float[size*size];

	long i,j;
	double mean = 0;
	for (i = 0;i<size;i++)
	{
		for (j = 0;j<size;j++)
		{
			B[i*size+j] = M(i,j);
			mean += B[i*size+j];
		}
	}

	mean /= size*size;
	for (i=0; i<size; i++)
		for (j=0; j<size; j++)
			 B[i*size+j] -= mean;
	
/*
	d=0;
	for (i=0; i<size; i++)
		for (j=0; j<size; j++)
			d += B[i*size+j]*B[i*size+j];
	d=sqrt(d);
	for (i=0; i<size; i++)
		for (j=0; j<size; j++)
			B[i*size+j] = (float)(B[i*size+j]/d);    // normalized to norm 1
*/
	skNormalize2Norm1(B, size, m_dLPNorm);


	cv::Ptr<cv::Mat> ker(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return ker;
}


///////////////////
// symbGCosine
//	[B] = GCosine(k, sigma, orientation)
//	Gabor sine base,  
//  k: freqency of sine
//	sigma: scale of the Gaussian in x-direction 
//	ort: in degree
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::symbGCosine(double k, double sigma,double ort)
{
	return GCosine(k, sigma, ort);
}


///////////////////
// bar()
//	[B] = bar(sigma, orientation, aspect)
//	sigma: scale of the bar width, true width = (int)(sigma + 0.5); 
//	orientation: in degree
//	aspect ratio: x:y, true length = w*aspect
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::bar(double sigma,double orientation,double aspect)
{
	int size;
	size = ((int)(sigma * aspect+0.5)/2)*2+1;     // size of the base, it is odd
	int c = size/2;        // half-size
	float *B=new float[size*size];
	memset(B, 0, sizeof(float)*size*size);

	int w = (int)(sigma + 0.5);

	double theta = (PI * orientation)/180;    // orientation in degree

	double x = size/2;
	double y = 0;
	::rotate(x, y, theta);

	int absx, absy;
	if (x > 0)
		absx = (int) (x+0.5);
	else
		absx = (int) (x-0.5);
	if (y > 0)
		absy = (int) (y+0.5);
	else
		absy = (int) (y-0.5);

	int x0, y0, x1, y1;
	x0 = c - absx;
	y0 = c - absy;
	x1 = c + absx;
	y1 = c + absy;

	int pnX[4096], pnY[4096];
	int n = ::rasterize_line_segment(x0, y0, x1, y1, pnX, pnY, w);
	float value = sqrt(1.0/n);
	for (int i = 0; i < n; i++)
	{
		if (pnX[i] < 0 || pnX[i] >= size || pnY[i] < 0 || pnY[i] >= size)
			continue;
		B[pnY[i]*size+pnX[i]] = value;
	}
	

	cv::Ptr<cv::Mat> ker(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return ker;
}

///////////////////
// symBar
//	[C] = symbBar(sigma, orientation, aspect) - symbolic
//	sigma: scale of the bar width 
//	orientation: in degree
//	aspect ratio: x:y
//	B: the symbolic base matrix
cv::Ptr<cv::Mat>  SkFilterBank::symBar(double sigma,double orientation,double aspect)
{
	int size;
	size = ((int)(sigma * aspect+0.5)/2)*2+1;     // size of the base, it is odd
	int c = size/2;        // half-size
	float *B=new float[size*size];

	int w = (int)(sigma + 0.5);

	w = m_param.m_nSketchWidth;


	double theta = (PI * orientation)/180;    // orientation in degree

	double x = size/2;
	double y = 0;
	::rotate(x, y, theta);

	int absx, absy;
	if (x > 0)
		absx = (int) (x+0.5);
	else
		absx = (int) (x-0.5);
	if (y > 0)
		absy = (int) (y+0.5);
	else
		absy = (int) (y-0.5);

	int x0, y0, x1, y1;
	x0 = c - absx;
	y0 = c - absy;
	x1 = c + absx;
	y1 = c + absy;

	int pnX[4096], pnY[4096];
	int n = ::rasterize_line_segment(x0, y0, x1, y1, pnX, pnY, w);
	int i;
	for (i = 0; i < size*size; i++)
		B[i] = 0;
	for (i = 0; i < n; i++)
	{
		if (pnX[i] < 0 || pnX[i] >= size || pnY[i] < 0 || pnY[i] >= size)
			continue;
		B[pnY[i]*size+pnX[i]] = -1;
	}
	

	cv::Ptr<cv::Mat> sym(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return sym;
}
///////////////////
// edge()
//	[B] = edge(sigma, orientation, aspect)
//	sigma: scale of the edge width 
//	orientation: in degree
//	aspect ratio: x:y
//	B: the base matrix
cv::Ptr<cv::Mat>  SkFilterBank::edge(double sigma,double orientation,double aspect)
{
	int size;
	size = ((int)(sigma * aspect+0.5)/2)*2+1;     // size of the base, it is odd
	int c = size/2;        // half-size
	float *B=new float[size*size];
	memset(B, 0, sizeof(float)*size*size);

	int w = (int)(sigma + 0.5);

	double theta,costheta,sintheta,x,y;

	theta = (PI * orientation)/180;    // orientation in degree
	costheta=cos(theta);
	sintheta=sin(theta);


	int i, j;
	int fabsy;
	for (i = 0; i<size; i++)
	{
		for (j = 0;j<size;j++)
		{
			x = ((i-c) * costheta + (c-j) * sintheta);
			y = ((i-c) * sintheta + (j-c) * costheta);

			if (y >= 0)
				fabsy = fabs(y+0.5);
			else
				fabsy = fabs(y-0.5);

			if (x>=w && x < 0 &&(fabsy<=c+1))
				B[i*size+j] = (float)(-1);  // a thin edge 
			else if (x == 0 &&(fabsy<=c+1))
				B[i*size+j] = (float)(0);
			else 
				B[i*size+j] = (float)(1);
		}
	}
	

	cv::Ptr<cv::Mat> ker(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return ker;
}

///////////////////
// symEdge
//	[C] = symbEdge(sigma, orientation, aspect) - symbolic
//	sigma: scale of the bar width 
//	orientation: in degree
//	aspect ratio: x:y
//	B: the symbolic base matrix
cv::Ptr<cv::Mat>  SkFilterBank::symEdge(double sigma,double orientation,double aspect)
{
	return edge(sigma, orientation, aspect);
}
//apect = resize ht, keep wd
// pImg should be normalized to zero mean and 1 variance
cv::Ptr<cv::Mat> SkFilterBank::imageFilter(double scale,double orientation,double aspect, Matrix<float> *pImg)
{
	int wd, ht;
	pImg->GetDimension(ht, wd);

	double d_wd_new, d_ht_new;
	d_wd_new = wd * scale * aspect * 1.3;
	d_ht_new = ht * scale * 1.3;

	int wd_new = (int)(d_wd_new + 0.5);
	int ht_new = (int)(d_ht_new + 0.5);

	int max_size =  MAX(wd_new, ht_new);

	int size = max_size /2*2 + 1;// size of the base, it is odd

	int c = size/2;        // half-size

	float *B=new float[size*size];
	memset(B, 0, sizeof(float)*size*size);

	double theta,costheta,sintheta,x,y;

	theta = (PI * orientation)/180;    // orientation in degree
	costheta=cos(theta);
	sintheta=sin(theta);


	double out_region_value = 0;
	int i, j;
	int fabsy;
	for (i = 0; i<size; i++)
	{//y
		for (j = 0;j<size;j++)
		{//x
			x = ((j-c) * costheta + (c-i) * sintheta);
			y = ((j-c) * sintheta + (i-c) * costheta);

			x /= scale;
			y /= scale*aspect;

			x += wd/2;
			y += ht/2;

			double v;
			if (x < 0 || x > wd-1 || y < 0 || y > ht-1)
				v = 0;
			else
				v = ::interpolationValue(*pImg, x, y, out_region_value);

			B[i*size+j] = (float)v;  
		}
	}
	
	// normalize to norm 1 vector
	double mean = 0;
	for (i=0; i<size; i++)
		for (j=0; j<size; j++)
			 mean += B[i*size+j];

	mean /= size*size;
	for (i=0; i<size; i++)
		for (j=0; j<size; j++)
			 B[i*size+j] -= mean;

/*
	d=0;
	for (i=0; i<size; i++)
		for (j=0; j<size; j++)
			d += B[i*size+j]*B[i*size+j];
	d=sqrt(d);
	for (i=0; i<size; i++)
		for (j=0; j<size; j++)
			B[i*size+j] = (float)(B[i*size+j]/d);    // normalized to norm 1
*/
	skNormalize2Norm1(B, size, m_dLPNorm);

	cv::Ptr<cv::Mat> ker(new cv::Mat(size, size, CV_32FC1, B));

	//delete []B;
	return ker;
}
cv::Ptr<cv::Mat> SkFilterBank::symImageFilter(double scale,double orientation,double aspect, Matrix<float> *pImg)
{
	return imageFilter(scale, orientation, aspect, pImg);
}

/////////////////////////////////////////////////////////////////////
// class SkFilter

///////////////////
// constructor
SkFilter::SkFilter()
{ 
	m_pIplKer = 0; 
	m_pIplSym = 0; 
	m_nHalf = 0; 
	m_nSub = 0;
	m_nType = -1;
//	m_dScale = -1;
	m_nScale = -1;
	m_dOrt = 0;
	m_dAspect = -1;

}
///////////////////
// deconstructor
SkFilter::~SkFilter()
{ 
	
}

///////////////////
// operator =
SkFilter& SkFilter::operator = (const SkFilter& f)
{
	assert(false);//not correctly implemented yet
	if(this != &f)
	{
		m_pIplKer = f.m_pIplKer;//couldn't just assign the pointer
		m_pIplSym = f.m_pIplSym;//couldn't just assign the pointer
		m_nType = f.m_nType;
		m_nHalf = f.m_nHalf;
		m_nSub = f.m_nSub;
//		m_dScale = f.m_dScale;
		m_nScale = f.m_nScale;
		m_dOrt = f.m_dOrt;
		m_dAspect = f.m_dAspect;
	}
	return *this;
};

///////////////////
// save
int SkFilter::save(FILE *pFile)
{
	fprintf(pFile, "Type = %d, Half = %d, Sub = %d\n", m_nType, m_nHalf, m_nSub);
//	fprintf(pFile, "Scale = %.5f, Ort = %.5f, Aspect = %.5f\n", m_dScale, m_dOrt, m_dAspect);
	fprintf(pFile, "Scale = %d, Ort = %.5f, Aspect = %.5f\n", m_nScale, m_dOrt, m_dAspect);
	fprintf(pFile, "Ker Matrix: \n");
	SkFilterBank::saveIplConvKernelFP(pFile, m_pIplKer);
	fprintf(pFile, "Sym Matrix: \n");
	SkFilterBank::saveIplConvKernelFP(pFile, m_pIplSym);
	return 0;
}
///////////////////
// read
int SkFilter::read(FILE *pFile)
{
	fscanf(pFile, "Type = %d, Half = %d, Sub = %d\n", &m_nType, &m_nHalf, &m_nSub);
//	float fScale;
	int nScale;
	float fOrt, fAspect;
//	fscanf(pFile, "Scale = %f, Ort = %f, Aspect = %f\n", &fScale, &fOrt, &fAspect);
//	m_dScale = fScale;
	fscanf(pFile, "Scale = %d, Ort = %f, Aspect = %f\n", &nScale, &fOrt, &fAspect);
	m_nScale = nScale;
	m_dOrt = fOrt;
	m_dAspect = fAspect;

		
	fscanf(pFile, "Ker Matrix: \n");
	SkFilterBank::readIplConvKernelFP(pFile, m_pIplKer);

	fscanf(pFile, "Sym Matrix: \n");
	SkFilterBank::readIplConvKernelFP(pFile, m_pIplSym);
	return 0;
}


/////////////////////////////////////////////////////////////////////
// class SkFilterBank

///////////////////
// constructor

SkFilterBank::SkFilterBank()
{
	m_pFilters = 0;
	m_ppCorr = 0;

//	m_nTotalFilterNum = 0;
}

///////////////////
// deconstructor
SkFilterBank::~SkFilterBank()
{
	if (m_pFilters != 0)
		delete [] m_pFilters;
	if (m_ppCorr != 0)
		deallocateCorr();
}

///////////////////
// deallocateCorr
void SkFilterBank::deallocateCorr()
{
	int n = getTotalFilterNum();
	for (int i = 0; i < n*n; i++)
	{
		if (0 != m_ppCorr[i])
			delete m_ppCorr[i];
	}
	delete [] m_ppCorr;
}
/*
int SkFilterBank::getValidWidth(int k)
{
	int nType = getType(k);
	if (nType == DOOG || nType == DOOG1)
	{
		int nScale = getScale(k);
		return ((int)(m_param.m_pdDOOGScales[nScale]*8))/2*2-1;
	}
	else if (nType == BAR || nType == SK_EDGE)
	{
		int nScale = getScale(k);
		return (int)(m_param.m_pdBarEdgeScales[nScale]*m_param.m_pdBarEdgeElongs[nScale]+0.5)/2*2+1;
	}
	else
	{
		return getValidLength(k);
	}
	assert(false);
	return -1;
}
*/
double SkFilterBank::getfScale(int k)
{ 
	int nType = getType(k);
	if (nType == DOOG || nType == DOOG1)
		return m_param.m_pdDOOGScales[m_pFilters[k].m_nScale]; 
	else if (nType == BAR || nType == SK_EDGE)
		return m_param.m_pdBarEdgeScales[m_pFilters[k].m_nScale]; 
	else if (nType == DOG)
		return m_param.m_pdDogScales[m_pFilters[k].m_nScale];
	else if (nType == IMAGE)
		return m_param.m_pdImgFilterScales[m_pFilters[k].m_nScale];
	else if (nType == LOG)
	{
		return m_param.m_pdLogScales[m_pFilters[k].m_nScale];
	}
	else
	{
		assert(false);
		return 1.0;
	}

}		   

///////////////////
// buildFilters()
int SkFilterBank::buildFilters()
{

	int n = getTotalFilterNum();

//	m_nTotalFilterNum = n;

	if (n <= 0)
		return 0;

	m_nImgBond = -1;

	if (m_pFilters != 0)
		delete [] m_pFilters;
	m_pFilters = new SkFilter[n];

	int ind = 0;
	int j, e, R;
	double endX, endY;

	// intensity
	if (m_param.m_nUseDeltaFunc)
	{
		m_pFilters[ind].m_pIplKer = intensity(); 
		m_pFilters[ind].m_pIplSym = symIntensity();// the symbolic base 

		m_pFilters[ind].m_nType = INTENSITY;//type
		m_pFilters[ind].m_nHalf = (m_pFilters[ind].m_pIplKer->rows-1)/2;  // half-size 
		m_pFilters[ind].m_nSub =1 ;						// subsampling 
	//		m_pFilters[ind].m_dScale = m_pdDOOGScales[i];		//scale
		m_pFilters[ind].m_nScale = 0;		//scale
		m_pFilters[ind].m_dAspect = 1;	//aspect ratio
		m_pFilters[ind].m_dOrt = 0;	//angle, in radian, 0~PI
		m_pFilters[ind].m_dOrtUnit = PI/2;	//angle unit
		m_pFilters[ind].m_dHalfValidWd = 0;
		m_pFilters[ind].m_dHalfValidLn = 0;


		endX = m_pFilters[ind].m_nHalf*FILTER_VALID_LENTH_RATIO;
		endY = 0;
		::rotate(endX, endY, m_pFilters[ind].m_dOrt);
		if (endX >= 0) m_pFilters[ind].m_nEndX = (int)(endX+0.5);
		else m_pFilters[ind].m_nEndX = (int)(endX-0.5);
		if (endY >= 0) m_pFilters[ind].m_nEndY = (int)(endY+0.5);
		else m_pFilters[ind].m_nEndY = (int)(endY-0.5);


		if (m_nImgBond < m_pFilters[ind].m_nHalf)
			m_nImgBond = m_pFilters[ind].m_nHalf;
		ind++; 
	}



	//first derivative
	int i;
	int e1, e2;
	if (m_param.m_nUseFirDerX)
		e1 = 0;
	else
		e1 = 1;
	if (m_param.m_nUseFirDerY)
		e2 = 1;
	else
		e2 = 0;
	for (i = e1; i <= e2; i++)
	{
		m_pFilters[ind].m_pIplKer = firDer(i); 
		m_pFilters[ind].m_pIplSym = symFirDer(i);// the symbolic base 
	
		m_pFilters[ind].m_nType = FIR_DER;//type
		m_pFilters[ind].m_nHalf = (m_pFilters[ind].m_pIplKer->rows-1)/2;  // half-size 
		m_pFilters[ind].m_nSub =1 ;						// subsampling 
//		m_pFilters[ind].m_dScale = m_pdDOOGScales[i];		//scale
		m_pFilters[ind].m_nScale = i;		//scale
		m_pFilters[ind].m_dAspect = 1;	//aspect ratio
		m_pFilters[ind].m_dOrt = PI/2 * i;	//angle, in radian, 0~PI
		m_pFilters[ind].m_dOrtUnit = PI/2;	//angle unit
		m_pFilters[ind].m_dHalfValidWd = 1;
		m_pFilters[ind].m_dHalfValidLn = 0;

		m_pppdFilterIndex[FIR_DER][i][0] = ind;
		m_ppdOrtUnit[FIR_DER][i] = PI/2;

		endX = m_pFilters[ind].m_nHalf*FILTER_VALID_LENTH_RATIO;
		endY = 0;
		::rotate(endX, endY, m_pFilters[ind].m_dOrt);
		if (endX >= 0) m_pFilters[ind].m_nEndX = (int)(endX+0.5);
		else m_pFilters[ind].m_nEndX = (int)(endX-0.5);
		if (endY >= 0) m_pFilters[ind].m_nEndY = (int)(endY+0.5);
		else m_pFilters[ind].m_nEndY = (int)(endY-0.5);


		if (m_nImgBond < m_pFilters[ind].m_nHalf)
			m_nImgBond = m_pFilters[ind].m_nHalf;
		ind++; 
	}

	//second derivative
	if (m_param.m_nUseSecDerX)
		e1 = 0;
	else
		e1 = 1;
	if (m_param.m_nUseSecDerY)
		e2 = 1;
	else
		e2 = 0;
	for (i = e1; i <= e2; i++)
	{
		m_pFilters[ind].m_pIplKer = secDer(i); 
		m_pFilters[ind].m_pIplSym = symSecDer(i);// the symbolic base 
	
		m_pFilters[ind].m_nType = SEC_DER;//type
		m_pFilters[ind].m_nHalf = (m_pFilters[ind].m_pIplKer->rows-1)/2;  // half-size 
		m_pFilters[ind].m_nSub =1 ;						// subsampling 
//		m_pFilters[ind].m_dScale = m_pdDOOGScales[i];		//scale
		m_pFilters[ind].m_nScale = i;		//scale
		m_pFilters[ind].m_dAspect = 1;	//aspect ratio
		m_pFilters[ind].m_dOrt = PI/2 * i;	//angle, in radian, 0~PI
		m_pFilters[ind].m_dOrtUnit = PI/2;	//angle unit
		m_pFilters[ind].m_dHalfValidWd = 1;
		m_pFilters[ind].m_dHalfValidLn = 0;

		m_pppdFilterIndex[SEC_DER][i][0] = ind;
		m_ppdOrtUnit[SEC_DER][i] = PI/2;

		endX = m_pFilters[ind].m_nHalf*FILTER_VALID_LENTH_RATIO;
		endY = 0;
		::rotate(endX, endY, m_pFilters[ind].m_dOrt);
		if (endX >= 0) m_pFilters[ind].m_nEndX = (int)(endX+0.5);
		else m_pFilters[ind].m_nEndX = (int)(endX-0.5);
		if (endY >= 0) m_pFilters[ind].m_nEndY = (int)(endY+0.5);
		else m_pFilters[ind].m_nEndY = (int)(endY-0.5);


		if (m_nImgBond < m_pFilters[ind].m_nHalf)
			m_nImgBond = m_pFilters[ind].m_nHalf;
		ind++; 
	}

	//LOG
	int nLoGNum = m_param.m_nLogNum;
	e = LOG;
	for (i = 0; i < nLoGNum; i++)
	{
		double scale = m_param.m_pdLogScales[i];
		m_pFilters[ind].m_pIplKer = LoG(scale); 	
		m_pFilters[ind].m_pIplSym = symLoG(scale);// the symbolic base 	
		m_pFilters[ind].m_nType = e;//type
		m_pFilters[ind].m_nHalf = (m_pFilters[ind].m_pIplKer->rows-1)/2;  // half-size 
		m_pFilters[ind].m_nSub =1 ;						// subsampling 
//		m_pFilters[ind].m_dScale = m_pdDOOGScales[i];		//scale
		m_pFilters[ind].m_nScale = i;		//scale
		m_pFilters[ind].m_dAspect = 1;	//aspect ratio
		m_pFilters[ind].m_dOrt = PI/2 * i;	//angle, in radian, 0~PI
		m_pFilters[ind].m_dOrtUnit = PI/2;	//angle unit
		m_pFilters[ind].m_dHalfValidWd = i+1;
		m_pFilters[ind].m_dHalfValidLn = i+1;

		m_pppdFilterIndex[e][i][0] = ind;
		m_ppdOrtUnit[e][i] = PI/2;

		endX = m_pFilters[ind].m_nHalf*FILTER_VALID_LENTH_RATIO;
		endY = 0;
		::rotate(endX, endY, m_pFilters[ind].m_dOrt);
		if (endX >= 0) m_pFilters[ind].m_nEndX = (int)(endX+0.5);
		else m_pFilters[ind].m_nEndX = (int)(endX-0.5);
		if (endY >= 0) m_pFilters[ind].m_nEndY = (int)(endY+0.5);
		else m_pFilters[ind].m_nEndY = (int)(endY-0.5);


		if (m_nImgBond < m_pFilters[ind].m_nHalf)
			m_nImgBond = m_pFilters[ind].m_nHalf;
		ind++; 
	}

	// DOOG and DOOG1
	if (m_param.m_nUseDOOG)
		e1 = DOOG;
	else
		e1 = DOOG1;
	if (m_param.m_nUseDOOG1)
		e2 = DOOG1;
	else
		e2 = DOOG;

	for ( i = 0; i < m_param.m_nDOOGScaleNum; i++)
	{		// a ridgelet system: the finer the scale,
		R = m_param.m_pnDOOGRotates[i];		// the more enlongate the bases, and the more orientations
		for (j = 0; j < R; j++)
		{  // the orientations
			for ( e = e1; e <= e2; e++)
			{// 0: odd/sin and 1: even/cos
				if (e == DOOG)
				{
					m_pFilters[ind].m_pIplKer = doog(m_param.m_pdDOOGScales[i], j*180.0/R, m_param.m_pdDOOGElongs[i]); 
					m_pFilters[ind].m_pIplSym = symbdoog(m_param.m_pdDOOGScales[i], j*180.0/R, m_param.m_pdDOOGElongs[i]);// the symbolic base 
					//the kernel is already rotated
				}
				else
				{
					m_pFilters[ind].m_pIplKer = doog1(m_param.m_pdDOOGScales[i], j*180.0/R, m_param.m_pdDOOGElongs[i]); 
					m_pFilters[ind].m_pIplSym = symbdoog1(m_param.m_pdDOOGScales[i], j*180.0/R, m_param.m_pdDOOGElongs[i]); 
//					m_pFilters[ind].m_dOrt = j*360./R;	//angle
				}
				m_pFilters[ind].m_nType = e;//type
				m_pFilters[ind].m_nHalf = (m_pFilters[ind].m_pIplKer->rows-1)/2;  // half-size 
				m_pFilters[ind].m_nSub =1 ;						// subsampling 
//				m_pFilters[ind].m_dScale = m_pdDOOGScales[i];		//scale
				m_pFilters[ind].m_nScale = i;		//scale
				m_pFilters[ind].m_dAspect = m_param.m_pdDOOGElongs[i];	//aspect ratio
//				m_pFilters[ind].m_dOrt = j*PI/R + PI/2;	//angle, in radian, 0~PI
				m_pFilters[ind].m_dOrt = j*PI/R;	//angle, in radian, 0~PI
				m_pFilters[ind].m_dOrtUnit = PI/R;	//angle unit
				m_pFilters[ind].m_dHalfValidWd = (m_param.m_pdDOOGScales[i]*2-1);//*2+1;
				m_pFilters[ind].m_dHalfValidLn = m_pFilters[ind].m_nHalf*FILTER_VALID_LENTH_RATIO;

				m_pppdFilterIndex[e][i][j] = ind;
				m_ppdOrtUnit[e][i] = PI/R;

				endX = m_pFilters[ind].m_nHalf*FILTER_VALID_LENTH_RATIO;
				endY = 0;
				::rotate(endX, endY, m_pFilters[ind].m_dOrt);
				if (endX >= 0) m_pFilters[ind].m_nEndX = (int)(endX+0.5);
				else m_pFilters[ind].m_nEndX = (int)(endX-0.5);
				if (endY >= 0) m_pFilters[ind].m_nEndY = (int)(endY+0.5);
				else m_pFilters[ind].m_nEndY = (int)(endY-0.5);


				if (m_nImgBond < m_pFilters[ind].m_nHalf)
					m_nImgBond = m_pFilters[ind].m_nHalf;
				ind++; 
		   }
	   }
	}
  
	// difference of Gaussian
	e = DOG;
	for (i = 0; i < m_param.m_nDogNum; i++)
	{
		m_pFilters[ind].m_pIplKer = dog(m_param.m_pdDogScales[i]);  
		m_pFilters[ind].m_pIplSym = symbdog(m_param.m_pdDogScales[i]);

		m_pFilters[ind].m_nType = e;//type
		m_pFilters[ind].m_nHalf = (m_pFilters[ind].m_pIplKer->rows-1)/2;  // half-size 
		m_pFilters[ind].m_nSub =1 ;						// subsampling 
//		m_pFilters[ind].m_dScale = m_pdDogScales[i];		//scale
		m_pFilters[ind].m_nScale = i;		//scale
		m_pFilters[ind].m_dOrt = 0;	//angle
		m_pFilters[ind].m_dAspect = 1;	//aspect ratio
		m_pFilters[ind].m_dHalfValidWd = m_pFilters[ind].m_nHalf;
		m_pFilters[ind].m_dHalfValidLn = m_pFilters[ind].m_nHalf;

		m_pppdFilterIndex[e][i][0] = ind;
		m_ppdOrtUnit[e][i] = PI;

		endX = m_pFilters[ind].m_nHalf;//*FILTER_VALID_LENTH_RATIO;
		endY = 0;
		if (endX >= 0) m_pFilters[ind].m_nEndX = (int)(endX+0.5);
		else m_pFilters[ind].m_nEndX = (int)(endX-0.5);
		if (endY >= 0) m_pFilters[ind].m_nEndY = (int)(endY+0.5);
		else m_pFilters[ind].m_nEndY = (int)(endY-0.5);

		if (m_nImgBond < m_pFilters[ind].m_nHalf)
			m_nImgBond = m_pFilters[ind].m_nHalf;
		
		ind++; 
	}

	// bar and edge
	if (m_param.m_nUseBar)
		e1 = BAR;
	else
		e1 = SK_EDGE;
	if (m_param.m_nUseEdge)
		e2 = SK_EDGE;
	else
		e2 = BAR;

	for ( i = 0; i < m_param.m_nBarEdgeScaleNum; i++)
	{		// a ridgelet system: the finer the scale,
		R = m_param.m_pnBarEdgeRotates[i];		// the more enlongate the bases, and the more orientations
		for (j = 0; j < R; j++)
		{  // the orientations
			for ( e = e1; e <= e2; e++)
			{// 0: odd/sin and 1: even/cos
				if (e == BAR)
				{
					m_pFilters[ind].m_pIplKer = bar(m_param.m_pdBarEdgeScales[i], j*180.0/R, m_param.m_pdBarEdgeElongs[i]); 
					m_pFilters[ind].m_pIplSym = symBar(m_param.m_pdBarEdgeScales[i], j*180.0/R, m_param.m_pdBarEdgeElongs[i]);// the symbolic base 
					//the kernel is already rotated
				}
				else
				{
					m_pFilters[ind].m_pIplKer = edge(m_param.m_pdBarEdgeScales[i], j*180.0/R, m_param.m_pdBarEdgeElongs[i]); 
					m_pFilters[ind].m_pIplSym = symEdge(m_param.m_pdBarEdgeScales[i], j*180.0/R, m_param.m_pdBarEdgeElongs[i]); 
//					m_pFilters[ind].m_dOrt = j*360./R;	//angle
				}
				m_pFilters[ind].m_nType = e;//type
				m_pFilters[ind].m_nHalf = (m_pFilters[ind].m_pIplKer->rows-1)/2;  // half-size 
				m_pFilters[ind].m_nSub =1 ;						// subsampling 
//				m_pFilters[ind].m_dScale = m_pdDOOGScales[i];		//scale
				m_pFilters[ind].m_nScale = i;		//scale
				m_pFilters[ind].m_dAspect = m_param.m_pdBarEdgeElongs[i];	//aspect ratio
				m_pFilters[ind].m_dOrt = j*PI/R;	//angle, in radian, 0~PI
				m_pFilters[ind].m_dOrtUnit = PI/R;	//angle unit
				m_pFilters[ind].m_dHalfValidWd = m_param.m_pdBarEdgeScales[i];
				m_pFilters[ind].m_dHalfValidLn = m_pFilters[ind].m_nHalf;

				m_pppdFilterIndex[e][i][j] = ind;
				m_ppdOrtUnit[e][i] = PI/R;

				endX = m_pFilters[ind].m_nHalf;
				endY = 0;
				::rotate(endX, endY, m_pFilters[ind].m_dOrt);
				if (endX >= 0) m_pFilters[ind].m_nEndX = (int)(endX+0.5);
				else m_pFilters[ind].m_nEndX = (int)(endX-0.5);
				if (endY >= 0) m_pFilters[ind].m_nEndY = (int)(endY+0.5);
				else m_pFilters[ind].m_nEndY = (int)(endY-0.5);


				if (m_nImgBond < m_pFilters[ind].m_nHalf)
					m_nImgBond = m_pFilters[ind].m_nHalf;
				ind++; 
		   }
	   }
	}


	// image filters
	e = IMAGE;
	char name[512], no[32], clu_no[32];
	::changeNumTo2Char(clu_no, m_param.m_nImgFilterTypeNum);
	Matrix<int> pImgChn[3], pImgChnMask[3];
	Matrix<float> imgFilter;
	int nFilterWd, nFilterHt;
	for (i = 0; i < m_param.m_nImgFilterTypeNum; i++)
	{
		e = e + i;//a bug might be here
		::changeNumTo3Char(no, i);
		sprintf(name, "filters\\img_filters\\cluster%s\\img_filter_%s.bmp", clu_no, no);
		::skLoadImage2YCrCb(name, pImgChn);
		sprintf(name, "filters\\img_filters\\cluster%s\\img_filter_%s_mask.bmp", clu_no, no);
		::skLoadImage2YCrCb(name, pImgChnMask);

//		::skSaveImageFromYCrCb("temp.bmp", pImgChn, 3);

		constructImageFilterWithMask(imgFilter, pImgChn[0], pImgChnMask[0]);

		pImgChn[0].GetDimension(nFilterHt, nFilterWd);

		::skUpsideDownSkMatrix(imgFilter);

//		::skSaveMatrixAsBitmap("temp2.bmp", imgFilter, -1, -1);

		R = m_param.m_pnImgFilterRotates[i];		// the more enlongate the bases, and the more orientations

		for (int k = 0; k < m_param.m_nImgFilterScaleNum; k++)
		{
			for (j = 0; j < R; j++)
			{  // the orientations
				m_pFilters[ind].m_pIplKer = imageFilter(m_param.m_pdImgFilterScales[k], j*180.0/R, m_param.m_pdImgFilterElongs[k], &imgFilter); 
				m_pFilters[ind].m_pIplSym = symImageFilter(m_param.m_pdImgFilterScales[k], j*180.0/R, m_param.m_pdImgFilterElongs[k], &imgFilter); // the symbolic base 

				m_pFilters[ind].m_nType = IMAGE;//type
				m_pFilters[ind].m_nHalf = (m_pFilters[ind].m_pIplKer->rows-1)/2;  // half-size 
				m_pFilters[ind].m_nSub =1 ;						// subsampling 
//				m_pFilters[ind].m_dScale = m_pdDOOGScales[i];		//scale
				m_pFilters[ind].m_nScale = k;		//scale
				m_pFilters[ind].m_dAspect = m_param.m_pdImgFilterElongs[k];	//aspect ratio
//				m_pFilters[ind].m_dOrt = j*PI/R + PI/2;	//angle, in radian, 0~PI
				m_pFilters[ind].m_dOrt = j*PI/R;	//angle, in radian, 0~PI
				m_pFilters[ind].m_dOrtUnit = PI/R;	//angle unit
				m_pFilters[ind].m_dHalfValidWd = m_pFilters[ind].m_nHalf;
				m_pFilters[ind].m_dHalfValidLn = m_pFilters[ind].m_nHalf;

				m_pppdFilterIndex[e][k][j] = ind;
				m_ppdOrtUnit[e][k] = PI/R;

				endX = m_pFilters[ind].m_nHalf*FILTER_VALID_LENTH_RATIO;
				endY = 0;
				::rotate(endX, endY, m_pFilters[ind].m_dOrt);
				if (endX >= 0) m_pFilters[ind].m_nEndX = (int)(endX+0.5);
				else m_pFilters[ind].m_nEndX = (int)(endX-0.5);
				if (endY >= 0) m_pFilters[ind].m_nEndY = (int)(endY+0.5);
				else m_pFilters[ind].m_nEndY = (int)(endY-0.5);


				if (m_nImgBond < m_pFilters[ind].m_nHalf)
					m_nImgBond = m_pFilters[ind].m_nHalf;
				ind++; 
		   }
	   }
	}


	// GSin and GCos
	if (m_param.m_nUseGSin)
		e1 = GSIN;
	else
		e1 = GCOS;
	if (m_param.m_nUseGCos)
		e2 = GCOS;
	else
		e2 = GSIN;

	for ( i = 0; i < m_param.m_nGaborScaleNum; i++)
	{		// a ridgelet system: the finer the scale,
		R = m_param.m_pnGaborRotates[i];		// the more enlongate the bases, and the more orientations
		for (j = 0; j < R; j++)
		{  // the orientations
			for ( e = e1; e <= e2; e++)
			{// 0: odd/sin and 1: even/cos
				if (e == GSIN)
				{
					m_pFilters[ind].m_pIplKer = GSine(m_param.m_pdGaborK[i], m_param.m_pdGaborSigma[i], j*180.0/R); 
					m_pFilters[ind].m_pIplSym = symbGSine(m_param.m_pdGaborK[i], m_param.m_pdGaborSigma[i], j*180.0/R); 
					//the kernel is already rotated
				}
				else
				{
					m_pFilters[ind].m_pIplKer = GCosine(m_param.m_pdGaborK[i], m_param.m_pdGaborSigma[i], j*180.0/R); 
					m_pFilters[ind].m_pIplSym = symbGCosine(m_param.m_pdGaborK[i], m_param.m_pdGaborSigma[i], j*180.0/R); 
//					m_pFilters[ind].m_dOrt = j*360./R;	//angle
				}
				m_pFilters[ind].m_nType = e;//type
				m_pFilters[ind].m_nHalf = (m_pFilters[ind].m_pIplKer->rows-1)/2;  // half-size 
				m_pFilters[ind].m_nSub =1 ;						// subsampling 
//				m_pFilters[ind].m_dScale = m_pdDOOGScales[i];		//scale
				m_pFilters[ind].m_nScale = i;		//scale
				m_pFilters[ind].m_dAspect = m_param.m_pdGaborSigma[i];// sigma for gabor
//				m_pFilters[ind].m_dOrt = j*PI/R + PI/2;	//angle, in radian, 0~PI
				m_pFilters[ind].m_dOrt = j*PI/R;	//angle, in radian, 0~PI
				m_pFilters[ind].m_dOrtUnit = PI/R;	//angle unit
				m_pFilters[ind].m_dHalfValidWd = m_param.m_pdGaborK[i];
				m_pFilters[ind].m_dHalfValidLn = m_pFilters[ind].m_nHalf*FILTER_VALID_LENTH_RATIO;

				m_pppdFilterIndex[e][i][j] = ind;
				m_ppdOrtUnit[e][i] = PI/R;

				endX = m_pFilters[ind].m_nHalf*FILTER_VALID_LENTH_RATIO;
				endY = 0;
				::rotate(endX, endY, m_pFilters[ind].m_dOrt);
				if (endX >= 0) m_pFilters[ind].m_nEndX = (int)(endX+0.5);
				else m_pFilters[ind].m_nEndX = (int)(endX-0.5);
				if (endY >= 0) m_pFilters[ind].m_nEndY = (int)(endY+0.5);
				else m_pFilters[ind].m_nEndY = (int)(endY-0.5);

				if (m_nImgBond < m_pFilters[ind].m_nHalf)
					m_nImgBond = m_pFilters[ind].m_nHalf;
				ind++; 
		   }
	   }
	}


	m_nImgBond = 0;//not consider boundary effect
	assert(n == ind);
	if (m_param.m_nBuildCorr)
		m_ppCorr = corr(m_pFilters,n); // the correlation (inhibition) matrix 
	else
		m_ppCorr = 0;
	return n;
}

// imgFilter has zero mean and 1 vairance
int SkFilterBank::constructImageFilterWithMask(Matrix<float> &imgFilter,//computed filter
											   Matrix<int> &img,//original image
											   Matrix<int> &imgMask)//0-invalid, 1-valid
{
//	::skCopyMatrix(img, imgFilter);

	int wd, ht;
	img.GetDimension(ht, wd);

	imgFilter.SetDimension(ht, wd);
	imgFilter.Set(0);

	double dSum = 0;
	int n = 0;//valid point num

	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (imgMask(r,c) != 0)
			{
				dSum += img(r,c);
				n++;
			}
		}
	}
	if (n <= 0)
	{
		imgFilter.Set(0);
		return 1;
	}
	dSum /= n;//mean

	double dStd = 0;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (imgMask(r,c) != 0)
			{
				dStd += (img(r,c)-dSum)*(img(r,c)-dSum);
			}
		}
	}
	dStd = sqrt(dStd/n);
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (imgMask(r,c) != 0)
				imgFilter(r,c) = (img(r,c)-dSum)/dStd;
			else
				imgFilter(r,c) = 0;
		}
	}

	return 0;
}
///////////////////
// getFilterIndex()
// for given filter type, scale, and ort, find the index k
// return k
int SkFilterBank::getFilterIndex(int nType, int nScale, double dOrt)
{// stupid search
	double dOrtUnit = m_ppdOrtUnit[nType][nScale];
	int nOrt = (int)(dOrt / dOrtUnit + 0.5);
	if (nType == BAR || nType == SK_EDGE)
	{
		int R = m_param.m_pnBarEdgeRotates[nScale];		// the more enlongate the bases, and the more orientations
		if (nOrt >= R)
			nOrt -= R;
	}
	else if (nType == DOOG || nType == DOOG1)
	{
		int R = m_param.m_pnDOOGRotates[nScale];		// the more enlongate the bases, and the more orientations
		if (nOrt >= R)
			nOrt -= R;
	}
	else
		nOrt = 0;
	assert(nOrt >= 0 && nOrt < MAX_FILTER_ORT_NUM);
	return m_pppdFilterIndex[nType][nScale][nOrt];
}


///////////////////
// corr
// C = corr(m_pFilters)
//---------------------------------------------------------
// Compute correlation or inhibition coefficients so that 
// after the synthesis coefficient of a base is updated, 
// the analysis coefficients of all the overlapping bases 
// can be updated without computing the remainder image. 
//---------------------------------------------------------
// m_pFilters: the bank of filters
// C: the correlation matrix
// n: the number of filters
Matrix<float> **SkFilterBank::corr(SkFilter *pFilters, int n)
{
	printf("        Corr for filters is processed....\n");//just for debugging

	IplImage *I,*cr;
	int i,j,k,l,hi,hj,hi2;
	float p,*val;
	cv::Ptr<cv::Mat> kr;
	
	Matrix<float> **C = new Matrix<float>*[n*n];
	memset(C, 0, sizeof(Matrix<float>*) * n * n);
	for (i = 0;i<n;i++){ 
		hi = pFilters[i].m_nHalf;   // half-size of filter i
		for (j = 0;j<n;j++){
			hj = pFilters[j].m_nHalf;    // half-size of filter j

			I=cvCreateImage(cvSize(2*(hi+hj)+1, 2*(hi+hj)+1), IPL_DEPTH_32F, 1);

			cr=cvCreateImage(cvSize(2*(hi+hj)+1, 2*(hi+hj)+1), IPL_DEPTH_32F, 1);
		

		//	iplGetConvKernelFP(m_pFilters[i].m_pIplKer,&k,&l,&ax,&ay,&val);
			cv::Mat imgI(I);

			hi2=2*hi+1;
			pFilters[i].m_pIplKer->copyTo( imgI(cv::Range(hj, hj+hi2), cv::Range(hj, hj+hi2)) ); 
						
			// I is the image patch of filter i at (0, 0)
			// the domain of I collects all positions of filter j
			// that can overlap with filter i at (0, 0)
			//	char st[55];
			//sprintf(st,"i%02dj%02d.txt",i,j);
			cv::Mat imgcr(cr);

			kr=pFilters[j].m_pIplKer;

			cv::filter2D(imgI, imgcr, CV_32F, *kr);
			//iplConvolve2DFP(I,cr,&kr,1,IPL_SUM);
//			ipl32FImage2FMatrix(cr, C[i*n+j]);
			skIpl32F2Matrix(cr, C[i*n+j]);
			// C{i,j} is how filter i at (0,0) inhibites filter j at 
			// locations overlapping with filter i
			cvReleaseImage(&I);
			cvReleaseImage(&cr);
	   }//for n
	}//for n
	return C;
}
///////////////////
// saveFiltersAsBmp()
//	save filters as bitmap images	
int SkFilterBank::saveFiltersAsBmp(const char *pchType)
{

	char name[128], dir[512], no[32];
	sprintf(dir,"filters");
	_mkdir(dir);

	int n = getTotalFilterNum();
	double dImgMean = 0;
	double dImgStd = -1;
	for (int i = 0; i < n; i++)
	{
		::changeNumTo3Char(no, i);
		sprintf(name, "%s\\%s_filter%s.bmp", dir, pchType, no);
//		saveFilterAsBmp(name, i, dImgMean, dImgStd, KER);
		sprintf(name, "%s\\%s_filter%s_syb.bmp", dir, pchType, no);
//		saveFilterAsBmp(name, i, dImgMean, dImgStd, SMB);
	}
	sprintf(name, "%s\\%s_filters.bmp", dir, pchType);
	saveAllFiltersInOneBmp(name, dImgMean, dImgStd, KER);
	sprintf(name, "%s\\%s_filters_syb.bmp", dir, pchType);
	saveAllFiltersInOneBmp(name, dImgMean, dImgStd, SMB);

//	saveFiltersAsMatrix();

	return n;
}

///////////////////
// 	saveFiltersAsMatrix()
//	save filters as matrix files	
int SkFilterBank::saveFiltersAsMatrix()
{
	char name[128], dir[512], no[32];
	sprintf(dir,"filters");
	_mkdir(dir);

	Matrix<float> M;
	int n = getTotalFilterNum();

	//save the filter num
	sprintf(name, "%s\\filter_num.txt", dir);
	FILE *pFile;
	if (pFile=fopen(name, "w"))
		fprintf(pFile, "%d", n);
	fclose(pFile);

	int h;
	int syn = KER;
	for (int i = 0; i < n; i++)
	{
		::changeNumTo3Char(no, i);
		sprintf(name, "%s\\filter%s.txt", dir, no);
		h = m_pFilters[i].m_nHalf;
		M.SetDimension(2*h+1, 2*h+1);
		int x0 = h, y0 = h;
		copyFilter2Matrix(i, M, x0, y0, syn);
		skSaveMatrix(name, M);
	}
	return n;
}
///////////////////
// saveFilterAsBmp()
//	save the filter i as bitmap image	
//	syn - KER or SMB
int SkFilterBank::saveFilterAsBmp(const char* pchName, int i, double dImgMean, double dImgStd, int  syn)
{
	int n = getTotalFilterNum();
	if (i < 0 || i >= n)
		return -1;
	int h = m_pFilters[i].m_nHalf;
	Matrix<float> M(2*h+1, 2*h+1);
	M.Zero();
	int x0 = h, y0 = h;
	copyFilter2Matrix(i, M, x0, y0, syn);
	skSaveMatrixAsBitmap(pchName, M, dImgMean, dImgStd);
	return 0;
}
///////////////////
// saveAllFiltersInOneBmp()
//	save all filters in one bitmap image	
//	syn - KER or SMB
int SkFilterBank::saveAllFiltersInOneBmp(const char* pchName, double dImgMean, double dImgStd, int syn)
{
	int n = getTotalFilterNum();
	int m = (int)sqrt(double(n));//rows and colums
	if (n != m*m)
		m = m+1;
	int m2 = n/m;
	if (m2 * m != n)
		m2 += 1;
	int i;
	int max_h = -1, h;
	for (i = 0; i < n; i++)
	{//find max half
		h = m_pFilters[i].m_nHalf;
		if (max_h < h)
			max_h = h;
	}
	int size = 2*(max_h+1) + 1;
	int space = 3;

	int wd = m * (size+space) + space;
	int ht = m2 * (size+space) + space;
	
	Matrix<float> M(ht, wd);
	M.Set(0);
	int r, c, x0, y0;
	for (i = 0; i < n; i++)
	{
		r = i/m;
		c = i%m;
		x0 = c * (size+space) + size/2 + space;
		y0 = r * (size+space) + size/2 + space;
		copyFilter2Matrix(i, M, x0, y0, syn);
	}
	float max_f = M.GetMax();
	// draw seperate lines
	int x1, y1, x2, y2;
	for (i = 0; i <= m2; i++)
	{
		x1 = 0; 
		x2 = wd;
		y1 = i * (size+space);
		y2 = y1 + space;
		for (r = y1; r < y2; r++)
			for (c = x1; c < x2; c++)
				M(r,c) = max_f;
	}
	for (i = 0; i <= m; i++)
	{
		y1 = 0; 
		y2 = ht;
		x1 = i * (size+space);
		x2 = x1 + space;
		for (r = y1; r < y2; r++)
			for (c = x1; c < x2; c++)
				M(r,c) = max_f;
	}
	// last several not used square set to blank
	int nBlank = m*m2 - n;
	x2 = wd;
	x1 = x2 - nBlank*(size+space);
	y2 = ht;
	y1 = y2 - (size+space);
	for (r = y1; r < y2; r++)
		for (c = x1; c < x2; c++)
			M(r,c) = max_f;

	skSaveMatrixAsBitmap(pchName, M, dImgMean, dImgStd);
	return n;
}
///////////////////
// copyFilter2Matrix()
//	copy filter i to a matrix M, 
//	x0, y0 - the center position of the filter in M
//	syn - KER or SMB
int SkFilterBank::copyFilter2Matrix(int i, Matrix<float>& M, int x0, int y0, int syn)
{
	int ht = M.rows();
	int wd = M.cols();

	int h = m_pFilters[i].m_nHalf;
	cv::Ptr<cv::Mat> f;
	if (syn == KER)
		f = m_pFilters[i].m_pIplKer;
	else
		f = m_pFilters[i].m_pIplSym;
	int fx, fy;
	for (fy = -h; fy <= h; fy++)
	{
		int nYOffset = (fy+h)*(2*h+1);
		if (fy+y0 < 0 || fy+y0 >= ht)
			continue;
		for (fx = -h; fx <= h; fx++)
		{
			if (fx+x0 < 0 || fx+x0 >= wd)
				continue;
			M(fy+y0, fx+x0) = f->at<float>(fy+h, fx+h); //[nYOffset+fx+h];
		}
	}
	return 0;
}
///////////////////
// getMaxHalf()
//	get max half of all filters
int SkFilterBank::getMaxHalf()
{
	int nMaxHalf = 0;
	int n = getTotalFilterNum();
	int h;
	for (int i = 0; i < n; i++)
	{
		h = getHalf(i);
		if (nMaxHalf < h)
			nMaxHalf = h;
	}
	return nMaxHalf;
}























































