#ifndef _SK_FILTER_H_
#define _SK_FILTER_H_

#include <stdio.h>
#include "cv.h"
#include "skMatrix.h"

#ifndef PI
#define PI 3.14169265
#endif

#ifndef FILTER_VALID_LENTH_RATIO
#define FILTER_VALID_LENTH_RATIO 0.5
#endif

#define MAX_FILTER_TYPE 20
#define MAX_FILTER_NUM 1024

#define MAX_DOOG_SCALE_NUM 10
#define MAX_GABOR_SCALE_NUM 10
#define MAX_DOG_NUM 16
#define MAX_LOG_NUM 16
#define MAX_BAR_EDGE_SCALE_NUM 90
#define MAX_IMAGE_FILTER_SCALE_NUM 10
#define MAX_FILTER_ORT_NUM 36
#define MAX_FILTER_SCALE_NUM 90//(MAX_DOOG_SCALE_NUM > MAX_BAR_EDGE_SCALE_NUM ? MAX_DOOG_SCALE_NUM : MAX_BAR_EDGE_SCALE_NUM)

class SkFilterBank;

#ifndef SMB 
#define SMB 0
#endif

#ifndef KER
#define KER 1
#endif


/////////////////////////////////////////////////////////////////////
// class SkFilter
class SkFilter
{
#define DOOG 0 // first derivative of Gaussian
#define DOOG1  1 // second derivative of Gaussian
#define DOG   2 // difference of Gaussian
#define INTENSITY 3//intensity
#define FIR_DER 4 // first derivative
#define SEC_DER 5 // second derivative
#define LOG 6 // laplacian Gaussian
#define BAR 7 // bar
#define SK_EDGE 8 // edge
#define GSIN 9
#define GCOS 10
#define IMAGE 11 //filter from image

public:
	cv::Ptr<cv::Mat> m_pIplKer;
	cv::Ptr<cv::Mat> m_pIplSym; //symbolic representation

	int m_nType; //DOOG, DOOG1, DOG
	int m_nHalf; //half size
	double m_dHalfValidWd, m_dHalfValidLn;
	int m_nSub; //sub sampling

	int m_nScale;//scale
	double m_dOrt;//orientation, 0~PI
	double m_dAspect;//aspect ratio
	double m_dOrtUnit;//ort unit for this kind of filter

	int m_nEndX;
	int m_nEndY;

	SkFilter();
	~SkFilter();
	SkFilter& operator=(const SkFilter& f);
	cv::Ptr<cv::Mat>  getPointer2Ker() { return m_pIplKer; }
	cv::Ptr<cv::Mat>  getPointer2Sym() { return m_pIplSym; }
	int getType() { return m_nType; }
	int getHalf() { return m_nHalf; }
	int getSub() { return m_nSub; }
	int getScale() { return m_nScale; }
	double getOrt() { return m_dOrt; }//orientation, 0~PI
	double getOrtUnit() { return m_dOrtUnit; }
	double getAspect() { return m_dAspect; }
	double getHalfValidWd() { return m_dHalfValidWd; }
	double getHalfValidLn() { return m_dHalfValidLn; }
	int save(FILE *pFile);
	int read(FILE *pFile);

	int getEndX() { return m_nEndX; }
	int getEndY() { return m_nEndY; }

};

class SkFilterBankParam
{
#define SK_NOT_CARE_FB -1
#define SK_PRIM_PURSUIT_FB 0
#define SK_BLOB_PURSUIT_FB 1
#define SK_REB_DETECTION   3
public:
	int m_nBuildCorr; 
	int m_nFBType;//prim pursuit, blob pursuit, etc

	int m_nUseDeltaFunc;//use delta func or not
	int m_nUseFirDerX;//use first derivative X or not
	int m_nUseFirDerY;//use first derivative Y or not
	int m_nUseSecDerX;//use second derivative X or not
	int m_nUseSecDerY;//use second derivative Y or not

	int m_nLogNum;//laplacian gaussian nume
	double m_pdLogScales[MAX_LOG_NUM];//scales of Dogs
	
	int m_nUseDOOG;//use DOOG or not
	int m_nUseDOOG1;//use DOOG1 or not
	int m_nDOOGScaleNum;//scale num for DOOG filters
	double m_pdDOOGScales[MAX_DOOG_SCALE_NUM];//scales of DOOG filters
	double m_pdDOOGElongs[MAX_DOOG_SCALE_NUM];//englongations of DOOG filters
	int m_pnDOOGRotates[MAX_DOOG_SCALE_NUM];//rotates of DOOG filters
	
	int m_nUseGSin;//use GSin or not
	int m_nUseGCos;//use GCos or not
	int m_nGaborScaleNum;//scale num for Gabor filters
	double m_pdGaborK[MAX_GABOR_SCALE_NUM];//K of Gabor filters
	double m_pdGaborSigma[MAX_GABOR_SCALE_NUM];//sigma of Gabor filters
	int m_pnGaborRotates[MAX_GABOR_SCALE_NUM];//rotates of Gabor filters
	
	int m_nUseImage;//use image as filter or not
	int m_nImgFilterTypeNum;
	int m_nImgFilterScaleNum;
	double m_pdImgFilterScales[MAX_IMAGE_FILTER_SCALE_NUM];//scales of img filters
	double m_pdImgFilterElongs[MAX_IMAGE_FILTER_SCALE_NUM];//englongations of img filters
	int m_pnImgFilterRotates[MAX_IMAGE_FILTER_SCALE_NUM];//rotates of img filters
	
	int m_nDogNum;// dog num
	double m_pdDogScales[MAX_DOG_NUM];//scales of Dogs

	int m_nSketchWidth;//sketch width to draw symbolic
	
	int m_nUseBar;//use bar or not
	int m_nUseEdge;//use edge or not
	int m_nBarEdgeScaleNum;//scale num for bar and edges
	double m_pdBarEdgeScales[MAX_BAR_EDGE_SCALE_NUM];//scales of bar-edge filters
	double m_pdBarEdgeElongs[MAX_BAR_EDGE_SCALE_NUM];//englongations of bar-edge filters
	int m_pnBarEdgeRotates[MAX_BAR_EDGE_SCALE_NUM];//rotates of bar-edge filters
	
public:
	SkFilterBankParam();
	~SkFilterBankParam();
	int getTotalFilterNum();
	int read(const char *name);
	int save(const char *name);
	int initilaize4REBDetection(int reb);//int nScale, int nScaleNum);
	int initilaize4BasePursuit(int nSelect);
	int initilaize4PrimPursuit();//int nScale, int nScaleNum);
	int initilaize4BlobPursuit();
	int initilaize2ComputeEntropy(int nScale);
	int initilaize4FameModel();
	int initilaize4TextureClustering();
	int initilaize2ComputeFilterResponseEntropy();
	int removeAllFilters();
};

/////////////////////////////////////////////////////////////////////
// class SkFilterBank
class  SkFilterBank
{
// an overcomplete 136 linear bases of dots, bars, and edges
// they are to be used for symbolic image representation
// the final goal is a representational theory of primal sketch 
private:
	double m_pdOrtUnit[MAX_FILTER_NUM];
	int m_pppdFilterIndex[MAX_FILTER_TYPE][MAX_FILTER_SCALE_NUM][MAX_FILTER_ORT_NUM];
	double m_ppdOrtUnit[MAX_FILTER_TYPE][MAX_FILTER_SCALE_NUM];
	int m_nTotalFilterNum;
	int m_nImgBond;//max half filter size
public:
	SkFilterBankParam m_param;

public:
	SkFilter *m_pFilters;
	Matrix<float> **m_ppCorr;

	static double m_dLPNorm;//1:L1 norm, 2: L2 norm, 0.5: L-0.5 norm etc
public:
	SkFilterBank();
	~SkFilterBank();

	void setDOOGScaleNum(int nScaleNum) { m_param.m_nDOOGScaleNum = nScaleNum; }
	int  getDOOGScaleNum() { return m_param.m_nDOOGScaleNum; }
	void setDogNum(int nDogNum) { m_param.m_nDogNum = nDogNum; }
	int	 getTotalFilterNum(){return m_param.getTotalFilterNum();}
	int  getFilterTypeNum() { return MAX_FILTER_TYPE; }
	int getImgBond() { return m_nImgBond; }
	int getType(int k) { return m_pFilters[k].getType(); }
	double getOrt(int k) { return m_pFilters[k].getOrt(); }//orientation, [0,PI)
	double getOrtUnit(int k) { return m_pFilters[k].getOrtUnit(); }

	int getHalf(int k) { return m_pFilters[k].getHalf(); }
	double getAspect(int k) { return m_pFilters[k].getAspect(); }
	int getScale(int k) { return m_pFilters[k].getScale(); }
	double getfScale(int k); 
	double getHalfValidWd(int k) { return  m_pFilters[k].getHalfValidWd(); }
	double getHalfValidLn(int k) { return  m_pFilters[k].getHalfValidLn(); }

	int getFilterIndex(int nType, int nScale, double dOrt);

	int buildFilters();
	Matrix<float> **corr(SkFilter *pFilters, int n);

	void deallocateCorr();
	int readParameters(const char *name) {return m_param.read(name);}
	int saveParameters(const char *name) {return m_param.save(name);}
	int save(const char *name){return m_param.save(name);}
	int saveFiltersAsBmp(const char *pchType);
	int saveFiltersAsMatrix();
	int saveFilterAsBmp(const char* pchName, int i, double dImgMean, double dImgStd, int syn);
	int saveAllFiltersInOneBmp(const char* pchName, double dImgMean, double dImgStd, int syn);
	int copyFilter2Matrix(int i, Matrix<float>& M, int x0, int y0, int syn);

	int getEndX(int k) { return m_pFilters[k].getEndX(); }
	int getEndY(int k) { return m_pFilters[k].getEndY(); }
	int getMaxHalf();

	// filter functions
public:
	static int saveIplConvKernelFP(FILE *pFile, cv::Ptr<cv::Mat> iplKer);
	static int readIplConvKernelFP(FILE *pFile, cv::Ptr<cv::Mat> iplKer);
	static double G(double x, double s);
	static cv::Ptr<cv::Mat> intensity();
	static cv::Ptr<cv::Mat> symIntensity();
	static cv::Ptr<cv::Mat> firDer(int direction);
	static cv::Ptr<cv::Mat> symFirDer(int direction);
	static cv::Ptr<cv::Mat> secDer(int direction);
	static cv::Ptr<cv::Mat> symSecDer(int direction);
	static cv::Ptr<cv::Mat> LoG(double scale);
	static cv::Ptr<cv::Mat> symLoG(double scale);
	static cv::Ptr<cv::Mat> dog(double sigma);
	static cv::Ptr<cv::Mat> symbdog(double sigma);
	static cv::Ptr<cv::Mat> doog(double sigma,double orientation,double aspect);
	static cv::Ptr<cv::Mat> doog1(double sigma,double orientation,double aspect);
	static cv::Ptr<cv::Mat> GSine(double k, double sigma, double ort);
	static cv::Ptr<cv::Mat> GCosine(double k, double sigma, double ort);
	static cv::Ptr<cv::Mat> symbGSine(double k, double sigma, double ort);
	static cv::Ptr<cv::Mat> symbGCosine(double k, double sigma, double ort);

	cv::Ptr<cv::Mat> symbdoog(double sigma,double orientation,double aspect);//different filter banks may use different sketch width 
	cv::Ptr<cv::Mat> symbdoog1(double sigma,double orientation,double aspect);//different filter banks may use different sketch width 
	cv::Ptr<cv::Mat> symBar(double sigma,double orientation,double aspect);

	static cv::Ptr<cv::Mat> bar(double sigma,double orientation,double aspect);
	static cv::Ptr<cv::Mat> edge(double sigma,double orientation,double aspect);
	static cv::Ptr<cv::Mat> symEdge(double sigma,double orientation,double aspect);
	static cv::Ptr<cv::Mat> imageFilter(double sigma,double orientation,double aspect, Matrix<float> *pImg);
	static cv::Ptr<cv::Mat> symImageFilter(double sigma,double orientation,double aspect, Matrix<float> *pImg);
	static int constructImageFilterWithMask(Matrix<float> &imgFilter,//computed filter
											Matrix<int> &img,//original image
											Matrix<int> &imgMask);//1-valid

}; 

#endif