#include <direct.h>

#include "skFilterCoef.h"
#include "skMatrixIplOp.h"
#include "skGaussian.h"
#include "skOutput.h"


int skPropagateOneBase(float m, float delta, Matrix<float> &mag, int k, int r, int c, SkFilterBank *pFB)
{
	int ht, wd;
	mag.GetDimension(ht, wd);

	int dx1, dy1, dx2, dy2;
	int x1, y1, x2, y2;

	int h = pFB->getHalfValidWd(k);
	if (h < 1)
		h = 1;
	
	dx1 = pFB->getEndX(k);
	dy1 = pFB->getEndY(k);
	dx2 = -dx1;
	dy2 = -dy1;

	x1 = c + dx1; 
	x2 = c + dx2;
	y1 = r + dy1;
	y2 = r + dy2;

	bool horizon = false;
	if (abs(x1-x2) > abs(y1-y2))
		horizon = true;


	int pnX[2096], pnY[2096];

	int n = ::rasterize_line_segment(x1, y1, x2, y2, pnX, pnY);
	assert( n <= 2096);

	int i, j;
	int x, y;
	for (i = 0; i < n; i++)
	{//for each center point
		x = pnX[i];
		y = pnY[i];
		if (x < 0 || x >= wd || y < 0 || y >= ht)
			continue;
		mag(y,x) += m * delta;

		for (j = -h; j <= h; j++)
		{//for each inhibited point
			if (j == 0)
				continue;
			x1 = x;
			y1 = y;
			if (horizon)
				y1 = y+j;
			else
				x1 = x+j;
			if (x1 < 0 || x1 >= wd || y1 < 0 || y1 >= ht)
				continue;
			mag(y1,x1) += -m * delta/h;
			if (mag(y1,x1) < 0)
				mag(y1,x1) = 0;
		}
	}
	return 0;
}

int skBeliefProp4Magnitude(Matrix<float> &mag0, Matrix<int> &max_k, SkFilterBank *pFB)
{
	int ht, wd;
	mag0.GetDimension(ht, wd);

	Matrix<float> mag;
	mag = mag0;

	int iter = 5;
	float delta = 0.3;

	int i, r, c;
	for (i = 0; i < iter; i++)
	{
		mag0 = mag;
		for (r = 0; r < ht; r++)
		{
			for (c = 0; c < wd; c++)
			{
				skPropagateOneBase(mag0(r,c), delta, mag, max_k(r,c), r, c, pFB);
			}
		}
		char name[512];
		sprintf(name, "mag_%d.bmp", i);
//		::skSaveMatrixAsBitmap(name, mag, 0, -1);
	}
	mag0 = mag;
	return 0;
}


/////////////////////////////////////////////////////////////////////
// class SkFilterCoef
////////////////////////////////////////////////////////////////////
// class SkFilterCoef

///////////////////
// constructor()
SkFilterCoef::SkFilterCoef()
{
	m_pFilterBank = 0;
	m_pIplImage = 0;
	m_pMatImage = 0;
	for (int i = 0; i < MAX_FILTER_NUM; i++)
	{
		m_ppRemainCoef[i] = 0;
//		m_ppFilterProb[i] = 0;
	}
//	m_nDivide = 5;
	m_nDivide = 1;
	m_bConsiderEntropy = false;
//	m_dImgSigma = 3.0;
	m_dNoBasePrior = 0.99;
	m_nMaxScaleOfEntropy = 1000;
	m_nEntropyScale = 1;
	sprintf(m_pchProjName, "NotGiven");
	m_nSaveMaxCoef = 0;
}

///////////////////
// deconstructor()
SkFilterCoef::~SkFilterCoef()
{
	m_pFilterBank = 0;
	m_pIplImage = 0;
	for (int i = 0; i < MAX_FILTER_NUM; i++)
	{
		if (0 != m_ppRemainCoef[i])
			delete m_ppRemainCoef[i];
//		if (0 != m_ppFilterProb[i])
//			delete m_ppFilterProb[i];
	}
}
void SkFilterCoef::setAllBlocksUnvisited()
{
	m_nBlockLeft = m_nDivide*m_nDivide;
	memset(m_pbBlockLeft, true, sizeof(bool)*m_nBlockLeft);
}
int SkFilterCoef::setBlockDivideByBlockSize(int nBlockSize)
{
	if (m_pMatImage == 0)
	{
		m_nDivide = 1;
		return m_nDivide;
	}
	int wd, ht;
	m_pMatImage->GetDimension(ht, wd);
	int nMin = MIN(wd, ht);
	m_nDivide = nMin / nBlockSize;
	if (m_nDivide < 1)
		m_nDivide = 1;
	return m_nDivide;
}
///////////////////
// initializeCoefficencies()
int SkFilterCoef::initializeCoefficencies()
{
	IplImage *I0=0;
	if (m_pIplImage == 0)
	{
		assert(m_pMatImage != 0);
		skMatrix2Ipl32FImage(m_pMatImage, I0);
	}
	else
		I0 = m_pIplImage;

	assert(m_pFilterBank != 0);
	int n = m_pFilterBank->getTotalFilterNum();
	int wd = I0->width;
	int ht = I0->height;

	cv::Mat img0(I0);
	cv::Mat imgFiltered(img0.size(), CV_32FC1);
	
	for (int i = 0; i < n; i++)
	{		
		
		if ( m_pFilterBank->m_pFilters[i].m_nType==INTENSITY )
		{
			img0.copyTo( imgFiltered );
		}
		else
		{
			cv::filter2D(img0, imgFiltered, CV_32F, *(m_pFilterBank->m_pFilters[i].m_pIplKer), cv::Point(-1,-1), 0, cv::BORDER_CONSTANT);
		}				
	
		IplImage I = imgFiltered;
		skIpl32F2Matrix(&I, m_ppRemainCoef[i]);
	}
	
	if (m_pIplImage == 0)
		cvReleaseImage(&I0);

	setAllBlocksUnvisited();

	if (m_nSaveMaxCoef)
	{
		computeAndSaveMaxCoef();
//		computeNoBaseRatio();


/*
		::skComputeLocalProfile(*m_pMatImage, m_ort, m_clg, m_hfht, m_hfln);

		char dir[32], name[512];
		sprintf(dir, "edge");
		_mkdir(dir);

		sprintf(name, "%s\\%s_dir.bmp", dir, m_pchProjName);
		Matrix<float> tmp;
		tmp = m_ort*180/PI;
		::skSaveMatrixAsBitmap(name, tmp, 0, 1);

		sprintf(name, "%s\\%s_half_ln.bmp", dir, m_pchProjName);
		::skSaveMatrixAsBitmap(name, m_hfln, 0, -1);
		sprintf(name, "%s\\%s_half_ht.bmp", dir, m_pchProjName);
		::skSaveMatrixAsBitmap(name, m_hfht, 0, -1);

		sprintf(name, "%s\\%s_cl_gain.bmp", dir, m_pchProjName);
		::skSaveMatrixAsBitmap(name, m_clg, 0, -1);

		int ht, wd;
		m_clg.GetDimension(ht, wd);
		Matrix<float> tmp_mat;
		tmp_mat.SetDimension(ht, wd);

		float hfwd, hfln, clg;
		int r, c;
		for (r = 0; r < ht; r++)
		{
			for (c = 0; c < wd; c++)
			{
				hfwd = m_hfht(r,c);
				hfln = m_hfln(r,c);
				clg = m_clg(r,c);
				tmp_mat(r,c) = clg/hfln;
			}
		}
		sprintf(name, "%s\\%s_cl_gain_norm.bmp", dir, m_pchProjName);
		::skSaveMatrixAsBitmap(name, tmp_mat, 0, -1);

  */
	}
	if (m_bConsiderEntropy)
	{
		for (int i = 0; i < n; i++)
		{
//			if (m_ppFilterProb[i] == 0)
//				m_ppFilterProb[i] = new Matrix<float>;
//			m_ppFilterProb[i]->SetDimension(ht, wd);
		}
		m_entropy.SetDimension(ht, wd);
		computeEntropy4EachPixel();
	}
	return 0;
}

///////////////////
// updateBaseMap()
int SkFilterCoef::updateBaseMap(SkBaseMap &baseMap, int flag)
{
	int nPos = baseMap.getFirstPos();
	SkBaseNode *pBaseNode;
	while (nPos >= 0)
	{
		pBaseNode = baseMap.getNextNode(nPos);
		if (!pBaseNode->valid())
			continue;
		updateOneBase(pBaseNode->m_data, flag);
//		::TakeABreath();
	}
	return 0;
}

///////////////////
// findLargestCoeffBase()
float SkFilterCoef::findLargestCoeffBase(SkBase &base, Matrix<oval_IntArray> *pOcpMat, SkBaseMap *pBaseMap)
{
	int nBound = 2;
	assert(nBound >= 0);

	int n = m_pFilterBank->getTotalFilterNum();
	int wd = getWidth();
	int ht = getHeight();
	float fMax = -100000000; 
	float f;
	float fOcp, fLeft;
	for (int i = 0; i < n; i++)
	{//for i, every filter
		for (int r = nBound; r < ht-nBound; r++)
		{
			for (int c = nBound; c < wd-nBound; c++)
			{
				f = (*m_ppRemainCoef[i])(r, c);

				if (pOcpMat != 0)
				{
					fLeft = 1 - parallelBaseOccupancy(c, r, i, pOcpMat, pBaseMap, m_pFilterBank);
					if (fLeft > 1.0)
						fLeft = 1.0;
					if (fLeft < 0)
						fLeft = 0;
//					if (m_pFilterBank->getType(i) == DOG)
//						fLeft *= 0.5;
//					if (i >= n-g_nDogNum && fabs(f) < g_dMinBaseAbsCoef*2)
//						continue;//for heart image, we don't use white DOG, 11-15-2002
				}
				else
					fLeft = 1.0;

				f *= fLeft;
//				if (f > 0)//specially for the star image and the text image, 11-15-2002
//					continue;
				if (fMax < fabs(f))
				{
					fMax = float(fabs(f));
					base.k = i;
					base.x = c;
					base.y = r;
				}
			}
		}
	}//for i, every filter
	base.a = (*m_ppRemainCoef[base.k])(base.y, base.x);
	return fMax;
}

///////////////////
// findLargestCoefBase()
//	consider base occupancy for parrallel inhibition
//	and dMinCoef to search divided blocks
float SkFilterCoef::findLargestCoefBase(SkBase &base, Matrix<oval_IntArray> *pOcpMat, SkBaseMap *pBaseMap, double dMinCoef)
{
	int wd = getWidth();
	int ht = getHeight();

	int nTotalBlocks = m_nDivide*m_nDivide;

	int nSelect = -1;
	int nRand;
	int i, ss;
	while (true)
	{
		nRand = (int)(rand()/(RAND_MAX+1.0)*m_nBlockLeft);
		ss = 0;
		nSelect = -1;
		for (i = 0; i < nTotalBlocks; i++)
		{
			if (m_pbBlockLeft[i])
			{
				if (ss == nRand)
				{
					nSelect = i;
					break;
				}
				ss++;
			}
		}
		if (nSelect >= 0)
		{
			findLargestCoefBase(base, pOcpMat, pBaseMap, m_nDivide, nSelect, dMinCoef);
			if (fabs(base.a) < dMinCoef)
			{
				m_pbBlockLeft[nSelect] = false;
				m_nBlockLeft--;
			}
			else 
				break;
		}
		else
		{
			base.a = 0;
			break;
		}
	}

	return fabs(base.a);
}

///////////////////
// findLargestCoefBase()
//	consider base occupancy for parrallel inhibition
//	and dMinCoef to search divided blocks
float SkFilterCoef::findLargestCoefBase(SkBase &base, Matrix<oval_IntArray> *pOcpMat, SkBaseMap *pBaseMap, int nDivide, int nSelect, double dMinCoef)
{
	int nBound = 5;

	base.a = 0;
	base.k = -1;
	base.x = -1;
	base.y = -1;

	int wd = getWidth();
	int ht = getHeight();
	int n = m_pFilterBank->getTotalFilterNum();

	assert(nSelect >= 0 && nSelect < nDivide*nDivide);
	int cc = nSelect % nDivide;
	int rr = nSelect / nDivide;

	int x1 = cc * wd / nDivide;
	int y1 = rr * ht / nDivide;
	int x2 = (cc+1) * wd / nDivide;
	int y2 = (rr+1) * ht / nDivide;

	if (x1 < 0) x1 = 0;
	if (x1 >= wd) x1 = wd - 1;
	if (y1 < 0) y1 = 0;
	if (y1 >= ht) y1 = ht - 1;
	if (x2 < 0) x2 = 0;
	if (x2 >= wd) x2 = wd - 1;
	if (y2 < 0) y2 = 0;
	if (y2 >= ht) y2 = ht - 1;


	float fMax = 0.1; 
	float f;
	float fOcp, fLeft;
	bool bFind = false;
	for (int i = 0; i < n; i++)
	{//for i, every filter
		for (int r = y1; r <= y2; r++)
		{
			if (r < nBound || r >= ht-nBound)
				continue;
			for (int c = x1; c <= x2; c++)
			{
				if (c < nBound || c >= wd-nBound)
					continue;
				f = (*m_ppRemainCoef[i])(r, c);
				if (fabs(f) < dMinCoef)
					continue;

				if (pOcpMat != 0)
				{
					fLeft = 1 - parallelBaseOccupancy(c, r, i, pOcpMat, pBaseMap, m_pFilterBank);
					if (fLeft > 1.0)
						fLeft = 1.0;
					if (fLeft < 0)
						fLeft = 0;
				}
				else
					fLeft = 1.0;

				f *= fLeft;
				if (fMax < fabs(f))
				{
					fMax = float(fabs(f));
					base.k = i;
					base.x = c;
					base.y = r;
					bFind = true;
				}
			}
		}
	}//for i, every filter
	if (bFind)
		base.a = (*m_ppRemainCoef[base.k])(base.y, base.x);
	return fMax;
}


///////////////////
// maxAbsCoef()
//	at pixel (r,c), return the max abs coef
float SkFilterCoef::maxAbsCoef(int r, int c)
{
	int n = m_pFilterBank->getTotalFilterNum();
	float max_coef = 0, coef;
	for (int i = 0; i < n; i++)
	{
		coef = fabs((*m_ppRemainCoef[i])(r, c));
		if (coef > max_coef)
			max_coef = coef;
	}
	return max_coef;
}

///////////////////
// baseOccupancy()
//	return the occupancy of one point - the number of bases overlapped there
int SkFilterCoef::baseOccupancy(int x, int y, Matrix<oval_IntArray> *pOcpMat)
{
	oval_IntArray &array = (*pOcpMat)(y,x);
	int size = array.getSize();
	return (float)size;
}

///////////////////
// parallelBaseOccupancy()
//	"pOcpMat" saves the bases which are nearby (closed to g_dInhibitionDis)
//	return 1.0 if (x,y) is occupied by a almost parallel base
float SkFilterCoef::parallelBaseOccupancy(int x, int y, int k, Matrix<oval_IntArray> *pOcpMat, SkBaseMap *pBaseMap, SkFilterBank *pFB)
{
	double dTh = PI / 4;

	oval_IntArray &array = (*pOcpMat)(y,x);
	int n = array.getSize();
	if (n <= 0)
		return 0;
	double dOrt = pFB->getOrt(k);
	double dOrtOcp, dDeltaOrt;
	int id;
	SkBase base;
	for (int i = 0; i < n; i++)
	{
		id = array.getAt(i);
		base = *(pBaseMap->getAt(id));
		dOrtOcp = pFB->getOrt(base.k);
		dDeltaOrt = dOrt - dOrtOcp;
		assert(dDeltaOrt > -PI && dDeltaOrt < PI);
		if (dDeltaOrt < 0)
			dDeltaOrt = fabs(dDeltaOrt);
		if (dDeltaOrt >= PI/2)
			dDeltaOrt = PI - dDeltaOrt;

		if (dDeltaOrt < dTh)
			return 1;
	}
	return 0;
}

///////////////////
// findLargestCoeffBase()
//	i: filter index = base.k
float SkFilterCoef::findLargestCoeffBase(SkBase &base, int i)
{
	int n = m_pFilterBank->getTotalFilterNum();
	assert(i >= 0 && i < n);

	int wd = getWidth();//m_pIplImage->width;
	int ht = getHeight();//m_pIplImage->height;
	float fMax = -100000000; 
	float f;
	for (int r = 0; r < ht; r++)
	{
		for (int c = 0; c < wd; c++)
		{
			f = (*m_ppRemainCoef[i])(r, c);
//			if (f > 0)//specially for the star image and the text image, 11-15-2002
//				continue;
//			if (i >= n-g_nDogNum && f > 0)
//				continue;//for heart image, we don't use white DOG
			if (fMax < fabs(f))
			{
				fMax = float(fabs(f));
				base.k = i;
				base.x = c;
				base.y = r;
			}
		}
	}
	base.a = (*m_ppRemainCoef[base.k])(base.y, base.x);
	return fMax;
}

///////////////////
// findLargestCoeffBase()
float SkFilterCoef::findLargestCoeffBase(SkBase &base, int x1, int y1, int x2, int y2, int k)
{//open interval [x1,x2), [y1, y2)
	int n = m_pFilterBank->getTotalFilterNum();
	int wd = m_ppRemainCoef[0]->cols();
	int ht = m_ppRemainCoef[0]->rows();
	if (x1 < 0) x1 = 0;
	if (x1 >= wd) x1 = wd - 1;
	if (y1 < 0) y1 = 0;
	if (y1 >= ht) y1 = ht - 1;
	if (x2 < 0) x2 = 0;
	if (x2 >= wd) x2 = wd - 1;
	if (y2 < 0) y2 = 0;
	if (y2 >= ht) y2 = ht - 1;

	float fMax = -100000000; 
	float f;
	int ks = 0, ke = n-1;
	if (k >= 0)
	{
		ks = k;
		ke = k;
	}

	for (int i = ks; i <= ke; i++)
	{//for i, every filter
		for (int r = y1; r < y2; r++)
		{
			for (int c = x1; c < x2; c++)
			{
				f = (*m_ppRemainCoef[i])(r, c);
				if (fMax < fabs(f))
				{
					fMax = float(fabs(f));
					base.k = i;
					base.x = c;
					base.y = r;
				}
			}
		}
	}//for i, every filter
	base.a = (*m_ppRemainCoef[base.k])(base.y, base.x);
	return fMax;
}

///////////////////
// findLargestCoeffCertainBase()
//	Find the largest Coeff base of certain type 'nFilterType' 
//		in the region [x1, x2), [y1, y2)
//	and save it in 'base'
//	return 'base.a'
float SkFilterCoef::findLargestCoeffCertainBase(SkBase &base, int x1, int y1, int x2, int y2, int nFilterType)
{
	int t;
	float f, max_absf = 0;
	SkBase base_tmp;
	base_tmp.setFilterBank(base.getFilterBank());
	int n = m_pFilterBank->getTotalFilterNum();
	for (int i = 0; i < n; i++)
	{//for i, every filter
		t = m_pFilterBank->getType(i);
		if (t == nFilterType)
		{
			f = findLargestCoeffBase(base_tmp, x1, y1, x2, y2, i);
			if (max_absf < fabs(f))
			{
				base = base_tmp;
				max_absf = fabs(f);
			}
		}
	}
	return max_absf;
}


///////////////////
// findLargestCoeffBase()
int SkFilterCoef::updateOneBase(SkBase base, int flag)
{
	float a;
	if (flag == ADD)
		a = base.a;
	else
		a = -base.a;
	int n = m_pFilterBank->getTotalFilterNum();
	int wd = getWidth();//m_pIplImage->width;
	int ht = getHeight();//m_pIplImage->height;
	int mh = m_pFilterBank->m_pFilters[base.k].getHalf();
	int mx = base.x;
	int my = base.y;

	float corr;
	for (int i = 0; i < n; i++)  // update analysis coefficients 
	{//all filters at this point
		int h = m_pFilterBank->m_pFilters[i].getHalf();
		for (int r = MAX(0, my-mh-h); r <= MIN(ht-1, my+mh+h); r++)
		{
			for (int c = MAX(0, mx-mh-h); c <= MIN(wd-1, mx+mh+h); c++)
			{
				corr = (*(m_pFilterBank->m_ppCorr[base.k*n+i]))(r-my+mh+h, c-mx+mh+h);
				(*m_ppRemainCoef[i])(r,c) -= a * corr;
			}// the center one will be zero   
		}
	} 
	return 0;
}
///////////////////
// proposeOneRandom4Birth()
//	given a center (x0,y0) and a search radius, propose one random base for birth
//	probability proportional to absceof
//	save it in "base". base.x and base.y are absolute value, not relative to (x0,y0)
//	return the proposal probability  
//		1) search all bases with coef >= dMinCoef
//		2) if exists such bases, draw one from probability which is proportional to abscoef
//		3) if not exists, randomly draw one base (k, x, y)
//		4) the coef is draw from a Gaussion distribution N(remain_coef, 1)
double SkFilterCoef::proposeOneRandomBase4Birth(SkBase& base, int x0, int y0, double dSearchRadius, double dMinCoef)
{
	int n = m_pFilterBank->getTotalFilterNum();
	int wd = m_ppRemainCoef[0]->cols();
	int ht = m_ppRemainCoef[0]->rows();
	int x1 = int(x0 - dSearchRadius);
	int y1 = int(y0 - dSearchRadius);
	int x2 = int(x0 + dSearchRadius);
	int y2 = int(y0 + dSearchRadius);

	if (x1 < 0) x1 = 0;
	if (x1 >= wd) x1 = wd - 1;
	if (y1 < 0) y1 = 0;
	if (y1 >= ht) y1 = ht - 1;
	if (x2 < 0) x2 = 0;
	if (x2 >= wd) x2 = wd - 1;
	if (y2 < 0) y2 = 0;
	if (y2 >= ht) y2 = ht - 1;

	int w = x2 - x1 + 1;
	int h = y2 - y1 + 1;

	double *pdF = new double[n*w*h];
	double *pdProb = new double[n*w*h];
	int *pnK = new int[n*w*h];
	int *pnX = new int[n*w*h];
	int *pnY = new int[n*w*h];

	float f;
	int ks = 0, ke = n-1;

	int s = 0;//large base num
	for (int i = ks; i <= ke; i++)
	{//for i, every filter
		for (int r = y1; r <= y2; r++)
		{
			for (int c = x1; c <= x2; c++)
			{
				f = (*m_ppRemainCoef[i])(r, c);
				if (fabs(f) >= dMinCoef )
				{
					pdF[s] = -log(fabs(f));
					pnK[s] = i;
					pnX[s] = c;
					pnY[s] = r;
					s++;
				}
			}
		}
	}//for i, every filter
	double dProbDrawn;
	if (s <= 0)
	{//random sample k,x,y
		base.k = int(rand()*n/(RAND_MAX+1.0));
		base.x = x0 + int(rand()*w/(RAND_MAX+1.0)-w/2);
		base.y = y0 + int(rand()*h/(RAND_MAX+1.0)-h/2);
		dProbDrawn = 1.0/(n*w*h);
	}
	else
	{//draw from energy/coef
		double dTemp = 0.1;
		::changeEnergy2Probability(pdF,pdProb, s, dTemp);
		int nDrawn = ::samplingFromProbability(pdProb, s, &dProbDrawn);
		base.k = pnK[nDrawn];
		base.x = pnX[nDrawn];
		base.y = pnY[nDrawn];
	}
	double dDelta = CGaussian::getOneSampleFromGaussian();
	dProbDrawn *= CGaussian::gaussian1D(dDelta, 1.0);
	base.a = (*m_ppRemainCoef[base.k])(base.y, base.x) + dDelta;
	
	delete[] pdF;
	delete[] pdProb;
	delete[] pnK;
	delete[] pnX;
	delete[] pnY;
	return dProbDrawn;

}
///////////////////
// proposalOfOneRandomBase4Birth()
//	given a center (x0,y0) and a search radius, find the proposal of birth one give base
//	this base should have been removed from the SkFilterCoef
//	base.x and base.y are absolute value, not relative to (x0,y0)
//	probability proportional to absceof
//	return the proposal probability, 
//		1) if fabs(base.a) >= dMinCoef, search all bases with coef >= dMinCoef, compute the prob of drawing it
//		2) if not, consider it is randomly drawn, compute the prob
double SkFilterCoef::proposalOfOneRandomBase4Birth(SkBase& base, int x0, int y0, double dSearchRadius, double dMinCoef)
{
	int n = m_pFilterBank->getTotalFilterNum();
	int wd = m_ppRemainCoef[0]->cols();
	int ht = m_ppRemainCoef[0]->rows();
	int x1 = int(x0 - dSearchRadius);
	int y1 = int(y0 - dSearchRadius);
	int x2 = int(x0 + dSearchRadius);
	int y2 = int(y0 + dSearchRadius);
	if (x1 < 0) x1 = 0;
	if (x1 >= wd) x1 = wd - 1;
	if (y1 < 0) y1 = 0;
	if (y1 >= ht) y1 = ht - 1;
	if (x2 < 0) x2 = 0;
	if (x2 >= wd) x2 = wd - 1;
	if (y2 < 0) y2 = 0;
	if (y2 >= ht) y2 = ht - 1;

	if (base.x < x1 || base.x > x2 || base.y < y1 || base.y > y2)
		return 0;//not inside the region

	int w = x2 - x1 + 1;
	int h = y2 - y1 + 1;
	double dProbDrawn;
	if (fabs(base.a) >= dMinCoef)
	{//
		double *pdF = new double[n*w*h];
		double *pdProb = new double[n*w*h];

		float f;
		int ks = 0, ke = n-1;

		int s = 0;//large base num
		int used_s = -1;
		for (int i = ks; i <= ke; i++)
		{//for i, every filter
			for (int r = y1; r <= y2; r++)
			{
				for (int c = x1; c <= x2; c++)
				{
					f = (*m_ppRemainCoef[i])(r, c);
					if (i == base.k && r == base.y && c == base.x)
					{
						used_s = s;
						pdF[s] = -log(fabs(f));
						s++;
					}
					else if (fabs(f) >= dMinCoef )
					{
						pdF[s] = -log(fabs(f));
						s++;
					}
				}
			}
		}//for i, every filter
		assert(used_s >= 0);
		delete[] pdF;
		delete[] pdProb;
	}
	else
	{
		dProbDrawn = 1.0/(n*w*h);
	}

	double dDelta = base.a - (*m_ppRemainCoef[base.k])(base.y, base.x);
	dProbDrawn *= CGaussian::gaussian1D(dDelta, 1.0);
	return dProbDrawn;
}


///////////////////
// computeAndSaveMaxCoef()
//	compute the max coef and save it to file
int SkFilterCoef::computeAndSaveMaxCoef()
{
	char dir[32];
	sprintf(dir, "edge");
//	_mkdir(dir);

	int wd, ht;
	m_pMatImage->GetDimension(ht, wd);
	m_maxmag.SetDimension(ht, wd);
	m_maxmagLog.SetDimension(ht, wd);
	m_max_size_Log.SetDimension(ht, wd);

	SkFilterBank *pFB = m_pFilterBank;
//	Matrix<int> max_k;
	m_max_k.SetDimension(ht, wd);
	m_ort.SetDimension(ht, wd);
	m_normal.SetDimension(ht, wd);
	m_hfht.SetDimension(ht, wd);
	m_hfln.SetDimension(ht, wd);

	int n = m_pFilterBank->getTotalFilterNum();
	
	int nFBType = m_pFilterBank->m_param.m_nFBType;


	m_maxmagLog.Set(0);
	m_max_size_Log.Set(0);

	double dMaxMag, dMag, coef1, coef2;
	int r, c;
	int k, mk;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			dMaxMag = -100000;
			if (nFBType == SK_PRIM_PURSUIT_FB)
			{//filter pursuit filter bank
				int start, end;
				//get the DOOG and DOOG1 indices
				for (k = 0; k < n; k++)
				{
					if (m_pFilterBank->getType(k) == DOOG 
						|| m_pFilterBank->getType(k) == DOOG1)
						break;
				}
				start = k;
				for (k = n-1; k >=0; k--)
				{
					if (m_pFilterBank->getType(k) == DOOG 
						|| m_pFilterBank->getType(k) == DOOG1)
						break;
				}
				end = k;
//				assert((end-start+1)/2*2 == end-start+1);//must be pairwised
				for (k = start; k <= end; k +=2)//combine DOOG and DOOG1
				{
					coef1 = getCoef(k, r, c);
					coef2 = getCoef(k+1, r, c);
	//				dMag = sqrt((coef1*coef1 + coef2*coef2))/(pFB->getHalfValidLn(k)*2+1);
					dMag = sqrt((coef1*coef1 + coef2*coef2));
					if (dMaxMag < dMag)
					{
						dMaxMag = dMag;
						if (fabs(coef1) >= fabs(coef2) )
							mk = k;
						else
							mk = k+1;
					}
				}
				//get the LoG
				int mk_log = -1;
				for (k = 0; k < n; k++)
				{
					if (m_pFilterBank->getType(k) == LOG)
						break;
				}
				start = k;
				for (k = n-1; k >=0; k--)
				{
					if (m_pFilterBank->getType(k) == LOG )
						break;
				}
				end = k;
				if (end >= start)
				{
					double dMaxMagLog = -1000000;
					for (k = start; k <= end; k ++)//combine DOOG and DOOG1
					{
						coef1 = getCoef(k, r, c);
						if (dMaxMagLog < coef1)
						{
							dMaxMagLog = coef1;
							mk_log = k;
						}
					}
					m_maxmagLog(r,c) = dMaxMagLog;
					double size = m_pFilterBank->getfScale(mk_log);
					m_max_size_Log(r,c) = size;
				}
			}// primitive pursuit
			else
			{//all others
				for (k = 0; k < n; k++)
				{
					coef1 = getCoef(k, r, c);
					dMag = fabs(coef1);
					if (dMaxMag < dMag)
					{
						dMaxMag = dMag;
						mk = k;
					}
				}
			}
			m_maxmag(r,c) = (float)dMaxMag;
			m_max_k(r,c) = mk;
			m_ort(r,c) = m_pFilterBank->getOrt(mk);
			m_hfht(r,c) = m_pFilterBank->getHalfValidWd(mk);
			m_hfln(r,c) = m_pFilterBank->getHalfValidLn(mk);
		}
	}

	char name[512];
	if (m_nSaveMaxCoef)
	{
/*		sprintf(name, "%s\\%s_mag0.bmp", dir, m_pchProjName);
		::skSaveMatrixAsBitmap(name, m_maxmag, 0, -1);
		sprintf(name, "%s\\%s_edge_mag0.bmp", dir, m_pchProjName);
		::skSaveMatrixAsBitmap(name, m_maxmag, 0, -1);

		sprintf(name, "%s\\%s_blob_mag0.bmp", dir, m_pchProjName);
		::skSaveMatrixAsBitmap(name, m_maxmagLog, 0, -1);

		sprintf(name, "%s\\%s_blob_size0.bmp", dir, m_pchProjName);
		::skSaveMatrixAsBitmap(name, m_max_size_Log, 0, -1);
*/	}

	Matrix<float> tmp, zeros;
	zeros.SetDimension(ht, wd);
	zeros.Zero();


	tmp = zeros - m_maxmag;
	if (m_nSaveMaxCoef)
	{
//		sprintf(name, "%s\\%s_mag_inv.bmp", dir, m_pchProjName);
//		::skSaveMatrixAsBitmap(name, tmp, 0, -1);
	}

	if (nFBType == SK_PRIM_PURSUIT_FB 
		|| nFBType == SK_REB_DETECTION)
	{
		char dir[32], name[512];
		sprintf(dir, "edge");
//		_mkdir(dir);

		//
		float ort;
		for (r = 0; r < ht; r++)
		{
			for (c = 0; c < wd; c++)
			{
				ort = -m_ort(r,c) + PI/2;// tangent change to normal,
				while(ort >= 2*PI)
					ort -= 2*PI;
				while (ort < 0)
					ort += 2*PI;
				m_normal(r,c) = ort*(180/PI / 2.0);//0~180, represents 0~360 degree
			}
		}
/*		sprintf(name, "%s\\%s_dir0.bmp", dir, m_pchProjName);
		::skSaveMatrixAsBitmap(name, m_normal, 0, 1);

		sprintf(name, "%s\\%s_edge_dir.bmp", dir, m_pchProjName);
		::skSaveMatrixAsBitmap(name, m_normal, 0, 1);

		sprintf(name, "%s\\%s_half_ln0.bmp", dir, m_pchProjName);
		::skSaveMatrixAsBitmap(name, m_hfln, 0, -1);
		sprintf(name, "%s\\%s_half_ht0.bmp", dir, m_pchProjName);
		::skSaveMatrixAsBitmap(name, m_hfht, 0, -1);
*/	}

	return 0;
}
///////////////////
// computeNoBaseRatio()
//	compute the log ratio of no-base and one base
int SkFilterCoef::computeNoBaseRatio()
{
	char dir[32], name[512];
	sprintf(dir, "edge");
	_mkdir(dir);

	int wd, ht;
	m_pMatImage->GetDimension(ht, wd);
	m_nobase.SetDimension(ht, wd);

	double dSqureRoot2PI = sqrt(2*PI);

	double dNoBasePrior = 0.9;
	double dLogNoBasePrior = log(dNoBasePrior);
//	double dLogBasePrior = log((1.0-dNoBasePrior)/n);
	double dLogBasePrior = log(1.0-dNoBasePrior);

	int n = m_pFilterBank->getTotalFilterNum();

	//get bases images
	Matrix<float> *pBases = new Matrix<float>[n];
	int k;
	int w, h, nHalf;
	for (k = 0; k < n; k++)
	{
		nHalf = m_pFilterBank->getHalf(k);
		w = h = 2*nHalf + 1;
		pBases[k].SetDimension(h, w);
		m_pFilterBank->copyFilter2Matrix(k, pBases[k], nHalf, nHalf, KER);
	}

	
	double dMean;
	double dSigma0, dSigma;
	int r, c;
	int mk;
	Matrix<float> patch, res;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			mk = m_max_k(r,c);
			nHalf = m_pFilterBank->getHalf(mk);
			w = h = 2*nHalf + 1;
			patch.SetDimension(h, w);

			//get local patch first
			patch.Set(SK_INFINITE);
			skCopyImage(m_pMatImage, &patch, c-nHalf, r-nHalf);
			int nValidPoint = skNumOfSmallerValueInMatrix(patch, SK_INFINITE/2);
			assert(nValidPoint > 0);
			//replace invalid patch with zero
			skReplaceValueInMatrix(patch, SK_INFINITE, 0, SK_INFINITE/2);
//			double dSumSqErr0 = skSquareSumOfMatrix(&patch, 0);//sum of the sqare errer
//			dSigma0 = sqrt(dSumSqErr0/nValidPoint);//the std of it
			::skGetMatrixMeanAndStd(&patch, dMean, dSigma0);
			//iid gaussian model, log P = log
			double dLogProb0 = dLogNoBasePrior - nValidPoint/2 - nValidPoint*dSqureRoot2PI*dSigma0;

			double dCoef1, dCoef2;
			double dLogProb = -SK_INFINITE;

			dCoef1 = getCoef(mk, r, c);
			dCoef2 = getCoef(mk+1, r, c);
			// compute by know coef, faster, the same result except the boundaries (some small errors)
//			dSigma= sqrt((dSumSqErr0 - dCoef1*dCoef1 - dCoef2*dCoef2)/nValidPoint);
			//compute sigma by compute residual
			res = patch - pBases[mk]*dCoef1 + pBases[mk]*dCoef2;
			skReplaceValueInMatrix(res, SK_INFINITE, 0, SK_INFINITE/2);
//			double dSumSqErr = skSquareSumOfMatrix(&res, 0);
//			dSigma = sqrt(dSumSqErr/nValidPoint);
			::skGetMatrixMeanAndStd(&res, dMean, dSigma);

			dLogProb = dLogBasePrior - nValidPoint/2 - nValidPoint*dSqureRoot2PI*dSigma;//save it just for debugging
			double log_ratio = dLogProb0 - dLogProb;
//			double temp = 50.0;
//			double tmp = exp(log_ratio/temp);
//			m_nobase(r, c) = log(tmp);
			m_nobase(r, c) = log_ratio;// /50;

		}
	}
	sprintf(name, "%s\\%s_nobase.bmp", dir, m_pchProjName);
	::skSaveMatrixAsBitmap(name, m_nobase, 0, -1);
	sprintf(name, "%s\\%s_nobase.txt", dir, m_pchProjName);
	::skSaveMatrix(name, m_nobase);

	Matrix<float> nobase_wt;
	nobase_wt.SetDimension(ht, wd);

	double min_nobase = m_nobase.GetMin();
	for (r = 0; r < ht; r++)
	{
		for ( c = 0; c < wd; c++)
		{
			assert(m_hfht(r,c) > -1);
//			nobase_wt(r,c) = m_nobase(r,c) / (m_hfht(r,c) + 1.0);
//			if (m_nobase(r,c) > 0)
//				nobase_wt(r,c) = 0;
//			else
				nobase_wt(r,c) = 0;//sqrt(m_nobase(r,c)-min_nobase);
//				nobase_wt(r,c) = log(m_nobase(r,c)-min_nobase+1.0);
//			nobase_wt(r,c) = m_nobase(r,c);


		}
	}
	sprintf(name, "%s\\%s_nobase_cut.bmp", dir, m_pchProjName);
//	skHistogramEqualization(nobase_wt, nEquHist);
	::skSaveMatrixAsBitmap(name, nobase_wt, 0, -1);

	delete []pBases;

	Matrix<float> mag;
	mag = m_nobase * (-1);

	sprintf(name, "%s\\%s_1base.bmp", dir, m_pchProjName);
	::skSaveMatrixAsBitmap(name, mag, 0, -1);
	sprintf(name, "%s\\%s_edge_mag.bmp", dir, m_pchProjName);
	::skSaveMatrixAsBitmap(name, mag, 0, -1);
	sprintf(name, "%s\\%s_1base.txt", dir, m_pchProjName);
	::skSaveMatrix(name, mag);

//	skComputeSketchFromMagByProfile(*m_pMatImage, mag, m_ort, m_max_k, m_pFilterBank);

	return 0;
}


double skGetTightFrameImageModelEntropy(Matrix<float> &org, Matrix<float> &syn, int &nValidNum)
{
	nValidNum = 0;
	int nOrgPtNum = 0;
	double dProj = 0;//projection
	double dSynNorm = 0, dOrgNorm = 0;
	int wd, ht;
	org.GetDimension(ht, wd);
	int r, c;
	float sy, or;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			or = org(r,c);
			if (or >= SK_INFINITE)
				continue;
			nOrgPtNum++;
			dOrgNorm += or * or;
			if (syn(r, c) == 0)
				continue;
			sy = syn(r,c);
			nValidNum++;
			dProj +=  or * sy;
			dSynNorm += sy * sy;
		}
	}
	if (nOrgPtNum == 0)
	{
		nValidNum = 0;
		return 0;
	}
	double dSigma;
	if (dSynNorm == 0 || nValidNum == 0)
	{
		dSigma = sqrt(dOrgNorm/nOrgPtNum);
	}
	else
	{
		double dScale = 2*dProj/dSynNorm;
		dSigma = 0;
		for (r = 0; r < ht; r++)
		{
			for (c = 0; c < wd; c++)
			{
				or = org(r,c);
				if (or >= SK_INFINITE)
					continue;
				if (syn(r, c) == 0)
					continue;
				sy = syn(r,c);
				dSigma += (or-sy)*(or-sy);
			}
		}
		dSigma = sqrt(dSigma/nValidNum);
	}
	return 0.5 + log(sqrt(2*PI)*dSigma);
}
///////////////////
// discoverSketchHalfHtWdByProfile
int discoverSketchHalfHtWdByProfile(Matrix<float> &org, 
									Matrix<float> &ort, //orientation for each point, in degree
									Matrix<float> &hfHt, 
									Matrix<float> &hfWd, 
									Matrix<float> &std0, //std of original image
									Matrix<float> &std1)//std after subtract profile
{
	int ht, wd;
	org.GetDimension(ht, wd);
	std0.SetDimension(ht, wd);
	std1.SetDimension(ht, wd);

	int nMaxHalfHt = 7;//mad ht 15
	int nMaxHalfWd = 4;//max wd 9

	int nMaxHt = 2*nMaxHalfHt + 1;
	int nMaxWd = 2*nMaxHalfWd + 1;

	Matrix<float> patch(nMaxHt, nMaxWd);

	float invalid_value = SK_INFINITE;

	double dOrt;
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			dOrt = ort(r,c)*PI/180;
			skGetRotatedPatch(org, dOrt, r, c, patch, invalid_value);
		}
	}
	return 0;
}
///////////////////
// skComputeLocalProfile()
//	compute the local profile
//given orientation of local pixel, compute the local profile, the ort will be updated
int skComputeLocalProfile(Matrix<float> &org, //original image
						  Matrix<float> &ort, //orientation, radius
  						  Matrix<float> &clg, //coding lenght gain
						  Matrix<float> &hf_ht, //half height
						  Matrix<float> &hf_ln) //half length
{
	int wd, ht;
	org.GetDimension(ht, wd);
	clg.SetDimension(ht, wd);
	hf_ht.SetDimension(ht, wd);
	hf_ln.SetDimension(ht, wd);

	int nMaxHfHt = 3;
	int nMinHfHt = 3;

	int nMaxHfLn = 6;
	int nMinHfLn = 6;

	double dOrt;
	double dClg;
	int nHfHt, nHfLn;
	char msg[512];
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			dOrt = ort(r,c);
			dClg = skComputeBestLocalProfile(org, r, c, dOrt, nHfHt, nHfLn, 
				nMaxHfHt, nMinHfHt, nMaxHfLn, nMinHfLn);
			clg(r,c) = dClg;
			ort(r,c) = dOrt;
			hf_ht(r,c) = nHfHt;
			hf_ln(r,c) = nHfLn;
		}
		sprintf(msg, "r=%d\n", r);
		::skOutput(msg);
	}

	double dMin = clg.GetMin();
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			clg(r,c) = sqrt(sqrt(clg(r,c)-dMin));
		}
	}

	// draw skeches
	Matrix<float> sk[5];
	for (int i = 0; i < 5; i++)
	{
		sk[i].SetDimension(ht, wd);
		sk[i].Set(0);
	}

	int x, y, x1, y1, x2, y2;
	double dx, dy;
	int pnX[1024], pnY[1024];
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			dOrt = ort(r,c);
			nHfLn = hf_ln(r,c);
			nHfHt = hf_ht(r,c);

			dx = nHfLn;
			dy = 0;
			::rotate(dx, dy, dOrt);

			x1 = c + dx;
			y1 = r + dy;
			x2 = c - dx;
			y2 = r - dy;

			int n = ::rasterize_line_segment(x1, y1, x2, y2, pnX, pnY, nHfHt);
			for (int i = 0; i < n; i++)
			{
				x = pnX[i];
				if (x < 0 || x >= wd)
					continue;
				y = pnY[i];
				if (y < 0 || y >= ht)
					continue;
				for (int j = 0; j < 5; j++)
				{
					if (nHfLn >= 2*j+2)
						sk[j](y,x)=sk[j](y,x) + 1;
				}				
			}
		}
	}


	char name[512];
	for (int j = 0; j < 5; j++)
	{
		sprintf(name, "edge\\sk_minkn_%d.bmp", 2*j);
		::skSaveMatrixAsBitmap(name, sk[j], 0, -1);
	}
	return 0;
}
///////////////////
// skComputeLocalProfile()
//	compute the local profile at pixel (r0,c0), return best coding lenth per pixel
//given orientation of local pixel, compute the local profile, the ort will be updated
double skComputeBestLocalProfile(Matrix<float> &org, //original image
						  int r0, int c0, // point considered
						  double &dOrt0, //orentation in radius
						  int &nHfHt, int &nHfLn,//half height, length
						  int nMaxHfHt, int nMinHfHt,
						  int nMaxHfLn, int nMinHfLn)
{
	int nHfSearch = 2;
	double dHfSearchOrt = 5.0*PI/180;
	double dSearchUnit = dHfSearchOrt/nHfSearch;

	float invalid_value = 0;

	int ds;
	double dOrt;
	int hf_ht, hf_ln;//test hf
	int ht, ln;
	Matrix<float> patch;
	double clg, clg_max = -1000000;
	int ds_max = -1, hf_ht_max = -1, hf_ln_max = -1;
	for (ds = -nHfSearch; ds <= nHfSearch; ds++)
	{
		dOrt = dOrt0 + ds*dSearchUnit;
		for (hf_ht = nMinHfHt; hf_ht <= nMaxHfHt; hf_ht++)
		{
			ht = 2*hf_ht + 1;
			for (hf_ln = nMinHfLn; hf_ln <= nMaxHfLn; hf_ln++)
			{
				ln = 2*hf_ln + 1;
				patch.SetDimension(ht, ln);
				::skGetRotatedPatch(org, dOrt, r0, c0, patch, invalid_value);
				clg = skCodingLengthGainWithProfile(patch);
				if (clg_max < clg)
				{
					clg_max = clg;
					ds_max = ds;
					hf_ht_max = hf_ht;
					hf_ln_max = hf_ln;
				}
			}
		}
	}
	dOrt0 = dOrt0 + ds_max *dSearchUnit;
	nHfHt = hf_ht_max;
	nHfLn = hf_ln_max;

	return clg_max;	
}
double skCodingLengthGainWithProfile(Matrix<float> &patch)
{
	Matrix<float> avg;
	::skGetMatrixRowAverage(patch, avg);

	int ht, wd;
	patch.GetDimension(ht, wd);

	Matrix<float> res;//residaul
	res.SetDimension(ht, wd);
	::setSameValue4EachRow(res, avg);
	res = patch - res;

	double dSumSq0 = patch.sumsq();
	double dSumSq1 = res.sumsq();

	double dSigma0 = sqrt(dSumSq0/(ht*wd));
	double dSigma1 = sqrt(dSumSq1/(ht*wd));

	double dSigma = 1.0;
//	double dDeltaDL = (dSumSq0-dSumSq1)/(2*dSigma*dSigma);///(wd);
//	double dDeltaDL = (-dSumSq1)/(2*dSigma*dSigma);///(wd);
//	double dDeltaDL = dSumSq0/(dSumSq1+0.001);

	double dSqureRoot2PI = sqrt(2*PI);
	double dDeltaDL = wd*ht*dSqureRoot2PI*(dSigma0-dSigma1);;


	double dCodeingProfilePixelLength = 1.0;
	dDeltaDL -= ht * dCodeingProfilePixelLength;

	return dDeltaDL;

}
///////////////////
// computeEntropy4EachPixel()
//	compute the entropy for each pixel
int SkFilterCoef::computeEntropy4EachPixel()
{

	//weighted no base

	double dValidCoef = 2.0;
	double dBaseCodingLength =  10.0;

	char dir[512], name[512];
	int wd, ht;
	m_entropy.GetDimension(ht, wd);

	Matrix<float> sq_err(ht, wd);//, nobase_ratio(ht, wd);


	int n = m_pFilterBank->getTotalFilterNum();
	assert(n > 1);
	int nHalf = m_pFilterBank->getMaxHalf();
	
	int nEquHist = 1024;
	//double dNoBasePrior = m_dNoBasePrior;

	double dSqureRoot2PI = sqrt(2*PI);
	double dNoBasePrior = 0.9;
	double dLogNoBasePrior = log(dNoBasePrior);
	double dLogBasePrior = log((1.0-dNoBasePrior)/n);
//	double dLogBasePrior = log(1.0-dNoBasePrior);

//	nHalf = nHalf/2;

	int w, h;
	w = h = 2*nHalf + 1;

	Matrix<float> patch, res, sum_base;
	patch.SetDimension(h, w);
	res.SetDimension(h, w);
	sum_base.SetDimension(h, w);

	double pdProb[512], pdLogProb[512];
	double pdCoef[512];
	assert(n <= 512);

	int nn = n/2; //+1;//number of base set n/2 +1 considers the no-base
	double dEntropy, dSumSqErr0, dSumSqErr;
	double dMaxEntropy = log(double(nn));

	int r, c;
	int k;
	int nValidPoint;
	double dSigma0, dSigma;
	double dLogProb0, dLogProb;

	//get bases images
	Matrix<float> *pBases = new Matrix<float>[n];
	for (k = 0; k < n; k++)
	{
		pBases[k].SetDimension(h, w);
		m_pFilterBank->copyFilter2Matrix(k, pBases[k], nHalf, nHalf, KER);
	}

	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			//get local patch first
			patch.Set(SK_INFINITE);
			skCopyImage(m_pMatImage, &patch, c-nHalf, r-nHalf);
			nValidPoint = skNumOfSmallerValueInMatrix(patch, SK_INFINITE/2);
			assert(nValidPoint > 0);
			//replace invalid patch with zero
			skReplaceValueInMatrix(patch, SK_INFINITE, 0, SK_INFINITE/2);
			dSumSqErr0 = skSquareSumOfMatrix(&patch, 0);//sum of the sqare errer
			dSigma0 = sqrt(dSumSqErr0/nValidPoint);//the std of it
			//iid gaussian model, log P = log
			//p(I|no base)=p(
			dLogProb0 = dLogNoBasePrior - nValidPoint/2 - nValidPoint*dSqureRoot2PI*dSigma0;
			
			sq_err(r,c) = dSigma0;

			patch.Set(SK_INFINITE);//re-copy again
			skCopyImage(m_pMatImage, &patch, c-nHalf, r-nHalf);


//			sum_base.Set(0);
//			int nUsedBase = 0;
			double dCoef1, dCoef2;
			dLogProb = -SK_INFINITE;
			for (k = 0; k < n/2; k++)
			{
//				pdCoef[k] = getCoef(k, r, c);
				dCoef1 = getCoef(k, r, c);
				dCoef2 = getCoef(k+1, r, c);
				// (1) compute sigma by compute residual
//				res = patch - pBases[k]*pdCoef[k];
//				skReplaceValueInMatrix(res, SK_INFINITE, 0, SK_INFINITE/2);
//				dSumSqErr = skSquareSumOfMatrix(&res, 0);
//				dSigma = sqrt(dSumSqErr/nValidPoint);
				// (2) compute by know coef, faster, the same result except the boundaries (some small errors)
				dSigma = sqrt((dSumSqErr0 - dCoef1*dCoef1 - dCoef2*dCoef2)/nValidPoint);
				pdLogProb[k] = dLogBasePrior - nValidPoint/2 - nValidPoint*dSqureRoot2PI*dSigma;//save it just for debugging
				if (dLogProb < pdLogProb[k])
					dLogProb = pdLogProb[k];
				// (3) compute entropy by coding length
//				if (fabs(pdCoef[k]) >= dValidCoef)
//				{
//					nUsedBase++;
//					sum_base = sum_base + pBases[k]*pdCoef[k];
//				}
			}
//			int max_k = m_max_k(r,c);
//			double coef = getCoef(max_k, r, c);
//			dSigma = sqrt((dSumSqErr0 - coef*coef)/nValidPoint);
//			dLogProb = dLogBasePrior - nValidPoint/2 - nValidPoint*dSqureRoot2PI*dSigma;

			for (k = 0; k < n; k++)
			{
//				(*m_ppFilterProb[k])(r,c) = pdProb[k];
			}

			if (nn * 2 != n)//consider no-base
				pdLogProb[nn-1] =  dLogProb0;
			::changeSign(pdLogProb, nn);
			::changeEnergy2Probability(pdLogProb, pdProb, nn);
			dEntropy = oneDEntropy(pdProb, nn);
			m_entropy(r,c) = dEntropy/log(double(nn));
		}
	}

	delete[] pBases;

	sprintf(dir, "edge");
	_mkdir(dir);


//	skHistogramEqualization(m_entropy, 1024);
	sprintf(name, "%s\\%s_entropy.bmp", dir, m_pchProjName);
	::skSaveMatrixAsBitmap(name, m_entropy, 0, -1);
	sprintf(name, "%s\\%s_entropy.txt", dir, m_pchProjName);
	::skSaveMatrix(name, m_entropy);

	sprintf(name, "%s\\%s_sq_err.bmp", dir, m_pchProjName);
	::skSaveMatrixAsBitmap(name, sq_err, 0, -1);



	Matrix<float> sig0, sig1;//sigmas before profile and after profile
	discoverSketchHalfHtWdByProfile(*m_pMatImage, m_ort, m_hfht, m_hfln, sig0, sig1); 

	Matrix<float> ent_mag;
	ent_mag.SetDimension(ht, wd);

	double tmp;
	for (r = 0; r < ht; r++)
	{
		for ( c = 0; c < wd; c++)
		{
			tmp = m_maxmag(r,c) * (1.0 - m_entropy(r,c));
			if (tmp > 10.0)
				tmp = 10.0;
			ent_mag(r,c) = tmp;
		}
	}
	sprintf(name, "%s\\%s_entropy_mag.bmp", dir, m_pchProjName);
//	skHistogramEqualization(ent_mag, nEquHist);
	::skSaveMatrixAsBitmap(name, ent_mag, 0, -1);

	Matrix<float> ent_nobase;
	ent_nobase.SetDimension(ht, wd);
	for (r = 0; r < ht; r++)
	{
		for ( c = 0; c < wd; c++)
		{
			ent_nobase(r,c) = m_entropy(r,c) * m_nobase(r,c);
		}
	}
	sprintf(name, "%s\\%s_entropy_nobase.bmp", dir, m_pchProjName);
//	skHistogramEqualization(ent_nobase, nEquHist);
	::skSaveMatrixAsBitmap(name, ent_nobase, 0, -1);

	Matrix<float> ent_nobase_mag;
	ent_nobase_mag.SetDimension(ht, wd);
	for (r = 0; r < ht; r++)
	{
		for ( c = 0; c < wd; c++)
		{
			ent_nobase_mag(r,c) = (1-m_entropy(r,c)) * (1-m_nobase(r,c)) * m_maxmag(r,c)/20;
		}
	}
	sprintf(name, "%s\\%s_entropy_nobase_mag.bmp", dir, m_pchProjName);
//	skHistogramEqualization(ent_nobase_mag, nEquHist);
	::skSaveMatrixAsBitmap(name, ent_nobase_mag, 0, -1);

	return 0;
}


///////////////////
// saveLowHighEntropyPatches()
//	save the images pathes with high or low entropy
int SkFilterCoef::saveLowHighEntropyPatches(double dImgMean, double dImgStd)
{

	int nMaxScaleOfEntropy = m_nMaxScaleOfEntropy;
	char dir0[512], dir1[512];
	sprintf(dir0, "entropy");
	_mkdir(dir0);

	int wd, ht;
	m_pMatImage->GetDimension(ht, wd);

	Matrix<float> minEn(ht, wd);
	minEn.Set(1);

	int nMaxFilterHalf = m_pFilterBank->getMaxHalf();
	int f = nMaxFilterHalf;

	int w, h;
//	w = h = 2 * 2 * nMaxFilterHalf + 1;
	w = h = 2 * f + 1;
	Matrix<float> M(h, w);

	int index;
	char name[512], pchNo[32], pchIndex[32], pchX[32], pchY[32];
	int pnPatchNum[4096];
	assert(nMaxScaleOfEntropy <= 4096);
	memset(pnPatchNum, 0, sizeof(int) * 4096);

	double min_en;
	int min_dr, min_dc;

	int r, c, dr, dc;
	int r0, c0, rr, cc;
	for (r = f; r < ht; r += f)
	{
		for (c = f; c < wd; c += f)
		{
			rr = r;
			cc = c;
			while (true)
			{
				min_dr = 0;
				min_dc = 0;
				min_en = 2.0;
				for (dr = -f; dr <= f; dr++)
				{
					r0 = rr + dr;
					if (r0 < 0 || r0 >= ht)
						continue;
					for (dc = -f; dc <= f; dc++)
					{
						c0 = cc + dc;
						if (c0 < 0 || c0 >= wd)
							continue;
						if (min_en > m_entropy(r0, c0))
						{
							min_en = m_entropy(r0, c0);
							min_dr = dr;
							min_dc = dc;
						}
					}
				}
				assert(min_dr >= -f && min_dc >= -f); 
				if (min_dr == 0 && min_dc == 0)
					break;
				rr = rr + min_dr;
				cc = cc + min_dc;
			}
			if (rr < 2*f || rr >= ht-2*f || cc < 2*f || cc >= wd - 2*f)
				continue;//cross the boundares

			minEn(rr,cc) = 0;
		}
	}
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (minEn(r, c) != 0)
				continue;

			::skCopyImage(m_pMatImage, &M, c-f, r-f);
			index = (int)(m_entropy(r,c) * nMaxScaleOfEntropy);
			if (pnPatchNum[index] >= 100)
				continue;

			::changeNumTo3Char(pchIndex, index);
			sprintf(dir1, "%s\\%s", dir0, pchIndex);
			_mkdir(dir1);
			::changeNumTo4Char(pchNo, pnPatchNum[index]);
			::changeNumTo3Char(pchX, c);
			::changeNumTo3Char(pchY, r);
			sprintf(name, "%s\\%s_en%s_%s_x%s_y%s.bmp", dir1, m_pchProjName, pchIndex, pchNo, pchX, pchY);
			::skSaveMatrixAsBitmap(name, M, dImgMean, dImgStd);
			pnPatchNum[index]++;
		}
	}
	return 0;
}

int skSaveFilterResponses(SkFilterCoef &fc, int level, const char *pProjectName)
{
	char dir[512];
	sprintf(dir, "FilterResp");
	_mkdir(dir);

	double dImgMean = -1; 
	double dImgStd = -1;

	char name[512], no[512];
	Matrix<float> *M;
	int n = fc.m_pFilterBank->getTotalFilterNum();
	for (int i = 0; i < n; i++)
	{
		if (i < 10)
			sprintf(no, "00%d", i);
		else if (i < 100)
			sprintf(no, "0%d", i);
		else
			sprintf(no, "%d", i);

		M = fc.m_ppRemainCoef[i];
		sprintf(name, "%s\\%s_FR_level%d_%s.bmp", dir, pProjectName, level, no);
		::skSaveMatrixAsBitmap(name, *M, dImgMean, dImgStd);
	}
	return n;

}
int skReadFilterResponse( Matrix<float> &rsps, int level, const char *pProjectName, int i)
{
	char dir[512];
	sprintf(dir, "FilterResp");
	_mkdir(dir);

	char name[512], no[512];

	if (i < 10)
		sprintf(no, "00%d", i);
	else if (i < 100)
		sprintf(no, "0%d", i);
	else
		sprintf(no, "%d", i);

	sprintf(name, "%s\\%s_FR_level%d_%s.bmp", dir, pProjectName, level, no);
	::skReadBitmap(name, &rsps);

	return 0;

}

