#ifndef _SK_FILTER_COEF_H_
#define _SK_FILTER_COEF_H_


#include "skFilter.h"
#include "skBase.h"
#include "skMap.h"


#define SK_MAX_COEF_BLOCK 900
class SkDivideImage
{
private:
	bool m_pbBlockLeft[SK_MAX_COEF_BLOCK];
	int m_nDivide;
	int m_nBlockLeft;
	int m_wd, m_ht;
public:
	void initialize(int ht, int wd, int nBlockSize) 
	{ 
		setHtWd(ht, wd);
		setBlockDivideByBlockSize(nBlockSize);
		setAllBlocksUnvisited();
	}
	void setHtWd(int h, int w) { m_ht = h; m_wd = w; }
	void setnDivide(int d) { m_nDivide = d; } 
	void setAllBlocksUnvisited()
	{
		m_nBlockLeft = m_nDivide*m_nDivide;
		memset(m_pbBlockLeft, true, sizeof(bool)*m_nBlockLeft);
	}
	int setBlockDivideByBlockSize(int nBlockSize)
	{
		int nMin = MIN(m_wd, m_ht);
		m_nDivide = nMin / nBlockSize;
		if (m_nDivide < 1)
			m_nDivide = 1;
		return m_nDivide;
	}

	int getNextBlock(int &x1, int &y1, int &x2, int &y2)
	{
		x2 = -1;
		y2 = -1;
		x1 = -1;
		y1 = -1;

		if (m_nBlockLeft <= 0)
			return -1;

		int nTotalBlocks = m_nDivide*m_nDivide;

		int nSelect = -1;
		int nRand;
		int i, ss;
		nRand = (int)(rand()/(RAND_MAX+1.0)*m_nBlockLeft);
		ss = 0;
		nSelect = -1;
		for (i = 0; i < nTotalBlocks; i++)
		{
			if (m_pbBlockLeft[i])
			{
				if (ss == nRand)
				{
					nSelect = i;
					break;
				}
				ss++;
			}

		}
		assert(nSelect >= 0);
		int nDivide = m_nDivide;
		int cc = nSelect % nDivide;
		int rr = nSelect / nDivide;

		int wd = m_wd;
		int ht = m_ht;

		x1 = cc * wd / nDivide;
		y1 = rr * ht / nDivide;
		x2 = (cc+1) * wd / nDivide;
		y2 = (rr+1) * ht / nDivide;

		if (x1 < 0) x1 = 0;
		if (x1 >= wd) x1 = wd - 1;
		if (y1 < 0) y1 = 0;
		if (y1 >= ht) y1 = ht - 1;
		if (x2 < 0) x2 = 0;
		if (x2 >= wd) x2 = wd - 1;
		if (y2 < 0) y2 = 0;
		if (y2 >= ht) y2 = ht - 1;

		return nSelect;
	}
	void setBlockNotLeft(int i)
	{
		m_pbBlockLeft[i] = false;
		m_nBlockLeft--;
	}
};

/////////////////////////////////////////////////////////////////////
// class SkFilterCoef
class SkFilterCoef
{
#ifndef ADD
#define ADD 0
#endif

#ifndef REMOVE 
#define REMOVE 1
#endif


public:
	SkFilterBank *m_pFilterBank;
	IplImage *m_pIplImage;
	Matrix<float> *m_pMatImage;
	Matrix<float> *m_ppRemainCoef[MAX_FILTER_NUM];//the analysis coefficient of filter i for remainder image
private:
	//the variables here just for searching bases block by block
	bool m_pbBlockLeft[SK_MAX_COEF_BLOCK];
	int m_nDivide;
	int m_nBlockLeft;
	Matrix<float> m_entropy;//entropy for each pixel
	Matrix<float> m_maxcoef;//max coefficients
	Matrix<float> m_nobase;//max coef magnitude
	Matrix<float> m_clg;//coding length gain
	bool m_bConsiderEntropy;
	double m_dNoBasePrior;
	int m_nMaxScaleOfEntropy;
	char m_pchProjName[512];
	int m_nEntropyScale;
	bool m_nSaveMaxCoef;
public:
	Matrix<float> m_maxmag;//max coef magnitude
	Matrix<float> m_maxmagLog;//max log magnitude
	Matrix<float> m_max_size_Log;//max size of log
	Matrix<int> m_max_k;//the filter type with max coef magnitude
	Matrix<float> m_ort;//orientation of each pixel, in radius
	Matrix<float> m_normal;//normal of each pixel in degree divided by 2
	Matrix<float> m_hfln;//half length of that filter
	Matrix<float> m_hfht;//half height of that filter

public:
	SkFilterCoef();
	~SkFilterCoef();
	SkFilterBank *getFilterBank(){ return m_pFilterBank; }
	void setAllBlocksUnvisited();
	void	setFilterBank(SkFilterBank *pFilterBank) { m_pFilterBank = pFilterBank; }
	void	setIplImage(IplImage *pIplImage) { m_pIplImage = pIplImage; }
	void	setMatrixImage(Matrix<float> *pMatImage) { m_pMatImage = pMatImage; }
	void	setConsiderEntropy(bool b) { m_bConsiderEntropy = b; }
	void	setNoBasePrior(double s) { m_dNoBasePrior = s; }
	void	setMaxScaleOfEntropy(int s) { m_nMaxScaleOfEntropy = s; }
	void	setProjName(const char *name) { strcpy(m_pchProjName, name); }
	int		getMaxScaleOfEntropy() { return m_nMaxScaleOfEntropy; }
	bool	considerEntropy() { return m_bConsiderEntropy; }
	double  getNoBasePrior() { return m_dNoBasePrior; }
	const char *getProjName() { return m_pchProjName; }
	int		getEntropyScale() { return m_nEntropyScale; }
	void	setEntropyScale(int s) { m_nEntropyScale = s; }
	void	setSaveMaxCoef(int b) { m_nSaveMaxCoef = b; }

	int		initializeCoefficencies();
	int		bottomUpBases(int nBaseNum, double dMinBaseCoef, SkBaseMap *m_pBaseMap);
	float	findLargestCoeffBase(SkBase &base, Matrix<oval_IntArray> *pOcpMat=0, SkBaseMap *pBaseMap=0);
	float	findLargestCoeffBase(SkBase &base, int i);
	float	findLargestCoeffBase(SkBase &base, int x1, int y1, int x2, int y2, int k=-1);
	float	findLargestCoeffCertainBase(SkBase &base, int x1, int y1, int x2, int y2, int nFilterType);

	int		updateOneBase(SkBase base, int flag);//flag = ADD, or REMOVE
	int		updateBaseMap(SkBaseMap &baseMap, int flag);
	int		getWidth() { return m_ppRemainCoef[0]->cols(); }
	int		getHeight() { return m_ppRemainCoef[0]->rows(); }
	double	proposeOneRandomBase4Birth(SkBase& base, int x0, int y0, double dSearchRadius, double dMinCoef);
	double	proposalOfOneRandomBase4Birth(SkBase& base, int x0, int y0, double dSearchRadius, double dMinCoef);
	static float parallelBaseOccupancy(int x, int y, int k, Matrix<oval_IntArray> *pOcpMat, SkBaseMap *pBaseMap, SkFilterBank *pFB);
	static int baseOccupancy(int x, int y, Matrix<oval_IntArray> *pOcpMat);

	float getCoef(int k, int r, int c){return (*m_ppRemainCoef[k])(r,c); }
	float maxAbsCoef(int r, int c); 
	void setCoef(int k, int r, int c, double v){(*m_ppRemainCoef[k])(r,c)=(float)v; }

	float findLargestCoefBase(SkBase &base, Matrix<oval_IntArray> *pOcpMat, SkBaseMap *pBaseMap, int nDivide, int nSelect, double dMinCoef);
	float findLargestCoefBase(SkBase &base, Matrix<oval_IntArray> *pOcpMat, SkBaseMap *pBaseMap, double dMinCoef);

	void setBlockDivide(int d) { m_nDivide = d; }
	int getBlockDivide() { return m_nDivide; }
	int setBlockDivideByBlockSize(int nBlockSize);

	//entropy
	double getEntropy(int r, int c) { return m_entropy(r,c); }
	int saveLowHighEntropyPatches(double dImgMean, double dImgStd);
	int computeAndSaveMaxCoef();
	int computeNoBaseRatio();
	int computeEntropy4EachPixel();

};


int skSaveFilterResponses(SkFilterCoef &fc, int level, const char *pProjectName);
int skReadFilterResponse( Matrix<float> &rsps, int level, const char *pProjectName, int i);
int skComputeLocalProfile(Matrix<float> &org, //original image
						  Matrix<float> &ort, //orientation, radius
  						  Matrix<float> &clg, //coding lenght gain
						  Matrix<float> &hf_ht, //half height
						  Matrix<float> &hf_ln); //half length
double skComputeBestLocalProfile(Matrix<float> &org, //original image
						  int r0, int c0, // point considered
						  double &dOrt0, //orentation in radius
						  int &nHfHt, int &nHfLn,//half height, length
						  int nMaxHfHt, int nMinHfht,
						  int nMaxHfLn, int nMinHfLn);
double skCodingLengthGainWithProfile(Matrix<float> &patch);

#endif