#ifndef _GAUSSIAN_H_
#define _GAUSSIAN_H_

#include <math.h>
#include <stdlib.h>

#ifndef PI 
#define PI 3.14159265
#endif
class CGaussian
{
public:
	static double gaussian1D(double x, double dSigma=1.0)
	{
		static double dSR2Pi = sqrt(2*PI);
		if (dSigma == 0)
			if (x == 0) return 1.0;
			else return 0;
		else
			return exp(-x*x / (2*dSigma*dSigma))/(dSR2Pi*dSigma);
	}
	static double gaussian(double x, double y, double dSigmaX, double dSigmaY, int nDim)
	{ 
		static double dSR2Pi = sqrt(2*PI);
		if (nDim == 0)
		{
			if (dSigmaX == 0)
				if (x==0)return 1.0;
				else return 0;
			else
				return 1.0/(dSR2Pi*dSigmaX)*exp( -x*x / (2*dSigmaX*dSigmaX) );
		}
		else
		{
			if (dSigmaX == 0 && dSigmaY == 0)
				if (x==0) return 1.0;
				else return 0;
			if (dSigmaY == 0)
				return 1.0/(sqrt(2*PI)*dSigmaX)*exp( -x*x / (2*dSigmaX*dSigmaX) );
			if (dSigmaX == 0)
				return 1.0/(sqrt(2*PI)*dSigmaY)*exp( -y*y / (2*dSigmaY*dSigmaY) );
			return 1.0/(2*PI*dSigmaX*dSigmaY) * exp( -x*x / (2*dSigmaX*dSigmaX)-y*y / (2*dSigmaY*dSigmaY) ); 
		}
	}
	static double getOneSampleFromGaussian(int seed = 0)
	{// copy from "Numerical recipes in C", pp289
		//get two uniform number in (-1,1) first
		static int iset = 0;
		static double gset;
		double fac, rsq, v1, v2;
		
		if (seed < 0)
			iset = 0;//reinitialize
		if (iset == 0)
		{
			while (true)
			{
				v1 = 2.0 * rand()/(RAND_MAX+1.0) - 1.0;
				v2 = 2.0 * rand()/(RAND_MAX+1.0) - 1.0;
				rsq = v1*v1 + v2*v2;
				if (rsq < 1.0 && rsq != 0.0)
					break;//find
			}
			fac = sqrt(-2.0 * log (rsq) / rsq);
			gset = v1*fac;//now make the Box-Muller transformation to get two normal deviates, return one and save the other for next time
			iset = 1;
			return v2*fac;
		}
		else
		{
			iset = 0;
			return gset;
		}
	}
};

#endif//_GAUSSIAN_H_