#include <direct.h>

#include "skOutput.h"
#include "skPrimitive.h"
#include "skMatrixIplOp.h"
#include "skIOImage.h"

#include "skGeneralEdge.h"

///////////////////
// skCanny
int skGeneralEdgeDetector(const char *pchImgName)
{
	DWORD Time0 = ::GetTickCount();
	

	char pchDir0[512];
	_getcwd(pchDir0, 512);

	char name[512], pchProjName[512], msg[512], dir[64];

	sprintf(msg, "\n General edge detection for image %s ......  ", pchImgName);
	::skOutput(msg);

	strcpy(pchProjName, pchImgName);
	::removeFileTypeInName(pchProjName);

	//load image
	Matrix<int> pImgChn[3];
	Matrix<unsigned char> image;
	Matrix<float> dir_rad;//direction in radius
	Matrix<int> mag;
	Matrix<int> tmp_int;
	Matrix<float> tmp_float;

	::skLoadImage2YCrCb(pchImgName, pImgChn);
	::skCopyMatrix(pImgChn[0], image);
	int wd, ht;
	pImgChn[0].GetDimension(ht, wd);
	Matrix<unsigned char > edge(ht, wd);

	_mkdir(pchProjName);
	_chdir(pchProjName);

	sprintf(dir, "edge");
	_mkdir(dir);

//	sprintf(name, "%s\\%s_dir.bmp", dir, pchProjName);
	sprintf(name, "%s\\%s_edge_dir.bmp", dir, pchProjName);
	::skLoadImage2YCrCb(name, pImgChn);
	::skCopyMatrix(pImgChn[0], tmp_float);
	tmp_float = tmp_float * 2.0;

//	::skSaveImageFromYCrCb("dir_tmp.bmp", &tmp_float, 1);

	tmp_float = tmp_float * (PI/180.0);//change it to radian
	::skCopyMatrix(tmp_float, dir_rad);

	sprintf(name, "%s\\%s_edge_mag.bmp", dir, pchProjName);
	::skLoadImage2YCrCb(name, pImgChn);
	::skCopyMatrix(pImgChn[0], mag);

	unsigned char *pImg = image.data;
	unsigned char *pEdge = edge.data;
	float *pDirRad = dir_rad.data;
	int *pMag = mag.data;
	Matrix<float> tmp_float255(ht, wd);
	tmp_float255.Set(255);



	char low_no[32], high_no[32];

	double dThreshHi;
	double dThreshLow;
	
	double dMean, dStd;

	for (dThreshHi = 0.90; dThreshHi >= 0.10; dThreshHi -= 0.10)
	{
		dThreshLow = dThreshHi * 0.4;
		::changeNumTo3Char(high_no, (int)(dThreshHi*1000+0.5));
		::changeNumTo3Char(low_no, (int)(dThreshLow*1000+0.5));

		skGeneralEdgeDetector(pImg, ht, wd, dThreshLow, dThreshHi, pEdge, pDirRad, pMag);

		sprintf(name, "%s\\%s_edge_l%s_h%s_gen.bmp", dir, pchProjName, low_no, high_no);
		::skCopyMatrix(edge, tmp_int);
		::skSaveImageFromYCrCb(name, &tmp_int, 1);

		skThinnerEdgeSketch(edge, dir_rad, mag);

		//save edge
		sprintf(name, "%s\\%s_edge_l%s_h%s_thin.bmp", dir, pchProjName, low_no, high_no);
		::skCopyMatrix(edge, tmp_int);
//		::skSaveImageFromYCrCb(name, &tmp_int, 1);
	}


	_chdir(pchDir0);//change back to original dir

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "  time:   %d min %.3fs \n ",nMin, dSec);
	::skOutput(msg);
	return 0;
}

const char g_pchCannyParamName[] = "canny_param.txt";
const char g_pchCannyWorkDir[] = "canny";

SkCannyEdgeParam::SkCannyEdgeParam()
{
	m_dSigmaStart = 0.6;
	m_dSigmaEnd   = 1.8;
	m_dSigmaStep  = 0.6;

	m_dHiThreshStart = 0.20;
	m_dHiThreshEnd = 0.95;
	m_dHiThreshStep = 0.05;
}
int SkCannyEdgeParam::read(const char *name)
{
	FILE *pFile;
	pFile = fopen(name, "r");
	if (pFile == 0)
		return 1;
	
	fscanf(pFile, "sigma start:          %lf\n", &m_dSigmaStart);// start sigma
	fscanf(pFile, "sigma end:            %lf\n", &m_dSigmaEnd);// end sigma
	fscanf(pFile, "sigma step:           %lf\n", &m_dSigmaStep);// sigma step
	
	fscanf(pFile, "hi thresh start:      %lf\n", &m_dHiThreshStart);// start hi thresh
	fscanf(pFile, "hi thresh end:        %lf\n", &m_dHiThreshEnd);// end hi thresh
	fscanf(pFile, "hi thresh step:       %lf\n", &m_dHiThreshStep);// step hi thresh
	fclose(pFile);
	return 0;
}
int SkCannyEdgeParam::save(const char *name)
{
	FILE *pFile;
	pFile = fopen(name, "w");
	if (pFile == 0)
		return 1;
	
	fprintf(pFile, "sigma start:          %.3f\n", m_dSigmaStart);// start sigma
	fprintf(pFile, "sigma end:            %.3f\n", m_dSigmaEnd);// end sigma
	fprintf(pFile, "sigma step:           %.3f\n", m_dSigmaStep);// sigma step
	
	fprintf(pFile, "hi thresh start:      %.3f\n", m_dHiThreshStart);// start hi thresh
	fprintf(pFile, "hi thresh end:        %.3f\n", m_dHiThreshEnd);// end hi thresh
	fprintf(pFile, "hi thresh step:       %.3f\n", m_dHiThreshStep);// step hi thresh
	fclose(pFile);
	return 0;
}

int skSfuCannyEdgeDetector(const char *pchImgName)
{
	DWORD Time0 = ::GetTickCount();
	

	char pchDir0[512];
	_getcwd(pchDir0, 512);

	char name[512], pchProjName[512], msg[512], work_dir[64];

	sprintf(msg, "\n SFU Canny edge detection for image %s ...... ", pchImgName);
	::skOutput(msg);

	strcpy(pchProjName, pchImgName);
	::removeFileTypeInName(pchProjName);

	//load image
	Matrix<int> pImgChn[3];
	::skLoadImage2YCrCb(pchImgName, pImgChn);
	int wd, ht;
	pImgChn[0].GetDimension(ht, wd);

	Matrix<unsigned char> image(ht, wd), edge(ht, wd);
	Matrix<float> dir_rad(ht, wd);//direction in radius
	Matrix<int> mag(ht, wd);

	::skCopyMatrix(pImgChn[0], image);
	unsigned char *pImg = image.data;
	unsigned char *pEdge = edge.data;
	float *pDirRad = dir_rad.data;
	int *pMag = mag.data;
	Matrix<int> tmp_int;
	Matrix<float> tmp_float;
	Matrix<float> tmp_float255(ht, wd);
	tmp_float255.Set(255);
	Matrix<float> tmp_float0(ht, wd);
	tmp_float255.Set(0);


	// read canny parameters
	SkCannyEdgeParam param;
	if (param.read(g_pchCannyParamName) != 0)
		param.save(g_pchCannyParamName);

	_mkdir(pchProjName);
	_chdir(pchProjName);


	sprintf(work_dir, g_pchCannyWorkDir);
	_mkdir(work_dir);

	char low_no[32], high_no[32], sigma_no[32];

	double dThreshHi;
	double dThreshLow;
	
	double sigma ;//1,2,3
	double dMean, dStd;

	for (sigma = param.m_dSigmaStart; sigma <= param.m_dSigmaEnd; sigma += param.m_dSigmaStep)
	{

		sprintf(sigma_no, "%3.1f", sigma);
		for (dThreshHi = param.m_dHiThreshStart; dThreshHi <= param.m_dHiThreshEnd; dThreshHi += param.m_dHiThreshStep)
		{
			dThreshLow = dThreshHi * 0.4;
			::changeNumTo3Char(high_no, (int)(dThreshHi*1000+0.5));
			::changeNumTo3Char(low_no, (int)(dThreshLow*1000+0.5));
			

			sfu_canny_image(pImg, ht, wd, sigma, dThreshLow, dThreshHi, pEdge, pDirRad, pMag);

			//save edge
			sprintf(name, "%s\\%s_canny_edge_s%s_l%s_h%s.bmp", work_dir, pchProjName, sigma_no, low_no, high_no);
			::skCopyMatrix(edge, tmp_int);
			::skSaveImageFromYCrCb(name, &tmp_int, 1);
		}
		//save edge strength
		sprintf(name, "%s\\%s_canny_edge_sth_s%s.bmp", work_dir, pchProjName, sigma_no);
		::skCopyMatrix(mag, tmp_float);
//		dMean = 128.0; dStd = 32.0;
//		::skNormalizeMatrix2ExpectedMeanStd(tmp_float, dMean, dStd);
//		::skSaveImageFromYCrCb(name, &tmp_float, 1);
		::skSaveMatrixAsBitmap(name, mag, 0, -1);

//		tmp_float = tmp_float255 - tmp_float; 
		tmp_float = tmp_float0 - tmp_float; 
		sprintf(name, "%s\\%s_canny_edge_sth_inv_s%s.bmp", work_dir, pchProjName, sigma_no);//inverse edge strength
		::skSaveMatrixAsBitmap(name, tmp_float, 0, -1);
//		::skSaveImageFromYCrCb(name, &tmp_float, 1);

		dir_rad = dir_rad * 180 / (2*PI);//dir_rad : 0~2*PI, scale down 
		sprintf(name, "%s\\%s_canny_edge_dir_s%s.bmp", work_dir, pchProjName, sigma_no);//inverse edge strength
		::skSaveImageFromYCrCb(name, &dir_rad, 1);
//		::skSaveMatrixAsBitmap(name, dir_rad, 0, -1);

	}


	_chdir(pchDir0);//change back to original dir

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "  time:   %d min %.3fs \n ",nMin, dSec);
	::skOutput(msg);
	return 0;
}

///////////////////
// skGeneralEdgeDetector()
void skGeneralEdgeDetector(unsigned char *image, int rows, int cols,
         float tlow, float thigh, unsigned char *edge, float *dir_radians, int *magnitude)
{ 
   unsigned char *nms;        /* Points that are local maximal magnitude. */
   int *smoothedim,     /* The image after gaussian smoothing.      */
             *delta_x,        /* The first devivative image, x-direction. */
             *delta_y;        /* The first derivative image, y-direction. */
   int r, c, pos;


   /****************************************************************************
   * Perform non-maximal suppression.
   ****************************************************************************/
   if((nms = (unsigned char *) calloc(rows*cols,sizeof(unsigned char)))==NULL){
      fprintf(stderr, "Error allocating the nms image.\n");
      exit(1);
   }

//   if(VERBOSE) printf("Doing the non-maximal suppression.\n");
//   if((nms = (unsigned char *) calloc(rows*cols,sizeof(unsigned char)))==NULL){
//      fprintf(stderr, "Error allocating the nms image.\n");
//      exit(1);
//   }
   if((delta_x = (int *) calloc(rows*cols,sizeof(int)))==NULL){
      fprintf(stderr, "Error allocating the nms image.\n");
      exit(1);
   }
   if((delta_y = (int *) calloc(rows*cols,sizeof(int)))==NULL){
      fprintf(stderr, "Error allocating the nms image.\n");
      exit(1);
   }

   skApproximateDeltaXYFromDirection(dir_radians, delta_x, delta_y, rows, cols);

   non_max_supp(magnitude, delta_x, delta_y, rows, cols, nms);

   /****************************************************************************
   * Use hysteresis to mark the edge pixels.
   ****************************************************************************/
//   if(VERBOSE) printf("Doing hysteresis thresholding.\n");

   apply_hysteresis(magnitude, nms, rows, cols, tlow, thigh, edge);

   /****************************************************************************
   * Free all of the memory that we allocated except for the edge image that
   * is still being used to store out result.
   ****************************************************************************/
   free(delta_x);
   free(delta_y);
   free(nms);
}
///////////////////
// skApproximateDeltaXYFromDirection()
int skApproximateDeltaXYFromDirection(float *dir_rad,//direction in radius, 0~PI
									  int* delta_x,//delta x
									  int* delta_y,//delat y
									  int rows, int cols)//rows and cols
{
   int r, c, pos;
   pos = 0;
   for (r = 0; r < rows; r++)
   {
	   for (c = 0; c < cols; c++)
	   {
		   skApproximateDeltaXYFromDirection(dir_rad[pos], delta_x[pos], delta_y[pos]);
		   pos++;
	   }
   }

	return 0;
}
///////////////////
// skApproximateDeltaXYFromDirection()
int skApproximateDeltaXYFromDirection(float dir_rad,//direction in radius, 0 ~ 2*PI
									  int &delta_x,//delta x
									  int &delta_y)//delat y
{
	//dir_rad=atan(dy/dx)

	int scale = 100;
	float tan_value = tan(dir_rad);
	if (dir_rad > PI/2 && dir_rad <= PI*3/2)
	{
		delta_x = -scale;
		delta_y = tan_value * delta_x;
	}
	else 
	{
		delta_x = scale;
		delta_y = tan_value * delta_x;
	}

	delta_y = -delta_y;//change coordiant system
	 return 0;
}


///////////////////
// 	
int skThinnerEdgeSketch(Matrix<unsigned char> &edge,//the edge
						Matrix<float> &dir_rad,//direction
						Matrix<int> &mag)//magnitude
{
	return 0;
}
