#include "cv.h"
#include "highgui.h"
#include "cvaux.h"
#include "skIOImage.h"
#include "skOutput.h"

void skCvtColor( const IplImage* src, IplImage* dst, int code );

///////////////////
// skCopyGrayCVImage2SkMatrix
int skCopyCVImage2SkMatrix(IplImage *pIplImg, Matrix<int> *pSkImg, int nChannel)
{
	assert(pIplImg->nChannels == nChannel);

	char msg[512];
	CvMat  *pMatImg;
    CvMat  imgStub;
	int coi;

	pMatImg = cvGetMat( pIplImg, &imgStub, &coi );

	uchar *pData = pMatImg->data.ptr;
	CvSize size = cvGetSize( pMatImg );
	int nWdStep = pMatImg->step;

	int wd = size.width;
	int ht = size.height;

	int r, c, k;

	for (k = 0; k < nChannel; k++)
		pSkImg[k].SetDimension(ht, wd);

	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			for (k = 0; k < nChannel; k++)
			{
				pSkImg[k](r, c) = (int)((pData + r*nWdStep)[c*nChannel+k]);
			}
		}
	}

	return nChannel;
}
///////////////////
// skCopyGrayCVImage2SkMatrix
int skCopyCVImage2SkMatrix(IplImage *pIplImg, Matrix<float> *pSkImg, int nChannel)
{
	assert(pIplImg->nChannels == nChannel);

	char msg[512];
	CvMat  *pMatImg;
    CvMat  imgStub;
	int coi;

	pMatImg = cvGetMat( pIplImg, &imgStub, &coi );

	uchar *pData = pMatImg->data.ptr;
	CvSize size = cvGetSize( pMatImg );
	int nWdStep = pMatImg->step;

	int wd = size.width;
	int ht = size.height;

	int r, c, k;

	for (k = 0; k < nChannel; k++)
		pSkImg[k].SetDimension(ht, wd);

	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			for (k = 0; k < nChannel; k++)
			{
				pSkImg[k](r, c) = (float)((pData + r*nWdStep)[c*nChannel+k]);
			}
		}
	}

	return nChannel;
}

///////////////////
// return channel number
int skCopySkMatrix2CVImage(Matrix<int> *pSkImg, IplImage *pIplImg, int nChannel)
{
	assert(pIplImg->nChannels == nChannel);

	char msg[512];
	CvMat  *pMatImg;
    CvMat  imgStub;
	int coi;

	pMatImg = cvGetMat( pIplImg, &imgStub, &coi );

	uchar *pData = pMatImg->data.ptr;
	CvSize size = cvGetSize( pMatImg );
	int nWdStep = pMatImg->step;

	int wd, ht;
	int k;
	pSkImg[0].GetDimension(ht, wd);
	assert( wd == size.width && ht == size.height);

	int value;
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			for (k = 0; k < nChannel; k++)
			{
				value = pSkImg[k](r, c);
				if (value < 0)
					(pData + r*nWdStep)[c*nChannel + k] = (uchar)0;
				else if (value > 255)
					(pData + r*nWdStep)[c*nChannel + k] = (uchar)255;
				else
					(pData + r*nWdStep)[c*nChannel + k] = (uchar)value;
			}
		}
	}

	return nChannel;
}
///////////////////
// return channel number
int skCopySkMatrix2CVImage(Matrix<float> *pSkImg, IplImage *pIplImg, int nChannel)
{
	assert(pIplImg->nChannels == nChannel);

	char msg[512];
	CvMat  *pMatImg;
    CvMat  imgStub;
	int coi;

	pMatImg = cvGetMat( pIplImg, &imgStub, &coi );

	uchar *pData = pMatImg->data.ptr;
	CvSize size = cvGetSize( pMatImg );
	int nWdStep = pMatImg->step;

	int wd, ht;
	int k;
	pSkImg[0].GetDimension(ht, wd);
	assert( wd == size.width && ht == size.height);

	int value;
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			for (k = 0; k < nChannel; k++)
			{
				value = pSkImg[k](r, c);
				if (value < 0)
					(pData + r*nWdStep)[c*nChannel + k] = (uchar)0;
				else if (value > 255)
					(pData + r*nWdStep)[c*nChannel + k] = (uchar)255;
				else
					(pData + r*nWdStep)[c*nChannel + k] = (uchar)value;
			}
		}
	}

	return nChannel;
}

///////////////////
// return channel number
// pChnMat should be allocated as a matrix
int skLoadImage2YCrCb(const char *name, Matrix<int> *pChnMat)
{
	char msg[512];

	IplImage *pIplImg = cvLoadImage(name, -1);//1-color, 0-gray,-1~as it be


	IplImage *pYCrCb = 0, *tmp = 0;
	if (pIplImg == 0)
	{
		sprintf(msg, "\n Couldn't find image %s\n", name);
		skOutput(msg);
		throw("int skLoadImage2YCrCb(const char *name, Matrix<int> *pChnMat) failed"); // Qi Ren
		return -1;
	}

	int channel = pIplImg->nChannels;
	if (channel == 1)
	{
		skCopyCVImage2SkMatrix(pIplImg, pChnMat, channel);
	}
	else if (channel == 3)
	{
//		Y=0.299*R + 0.587*G + 0.114*B
//		Cr=(R-Y)*0.713 + 128
//		Cb=(B-Y)*0.564 + 128
//
//		R=Y + 1.403*(Cr - 128)
//		G=Y - 0.344*(Cb - 128) - 0.714*(Cr - 128)
//		B=Y + 1.773*(Cb - 128)
		pYCrCb = cvCreateImage(cvGetSize(pIplImg), IPL_DEPTH_8U , 3);//YCrCb image
		skCvtColor(pIplImg, pYCrCb, CV_RGB2YCrCb);
//		cvCvtColor(pIplImg, pYCrCb, CV_RGB2YCrCb);
		
//		skCvtColor(pYCrCb, pIplImg, CV_YCrCb2RGB);
//		cvSaveImage("tmp002.bmp", pIplImg);

		skCopyCVImage2SkMatrix(pYCrCb, pChnMat, channel);
	}
	else
		assert(false);

	cvReleaseImage( &pIplImg );
	if (pYCrCb != 0)
		cvReleaseImage( &pYCrCb );

	return channel;
}
///////////////////
// return channel number
int skSaveImageFromYCrCb(const char *name, Matrix<int> *pChnMat, int channel)
{
	char msg[512];
	int wd, ht;
	pChnMat[0].GetDimension(ht, wd);
	if (ht <= 0 || wd <= 0)
		return 1;

	CvSize size;
	size.width = wd;
	size.height = ht;

	IplImage *pIplImg = 0;
	IplImage *pRGB = 0;
	if (channel == 1)
	{
		pIplImg = cvCreateImage(size, IPL_DEPTH_8U, 1);//YCrCb image
		skCopySkMatrix2CVImage(pChnMat, pIplImg, channel);
		cvSaveImage(name, pIplImg);
	}
	else if (channel == 3)
	{
//		Y=0.299*R + 0.587*G + 0.114*B
//		Cr=(R-Y)*0.713 + 128
//		Cb=(B-Y)*0.564 + 128
//
//		R=Y + 1.403*(Cr - 128)
//		G=Y - 0.344*(Cb - 128) - 0.714*(Cr - 128)
//		B=Y + 1.773*(Cb - 128)
		pIplImg = cvCreateImage(size, IPL_DEPTH_8U , 3);//YCrCb image
		skCopySkMatrix2CVImage(pChnMat, pIplImg, channel);
		pRGB = cvCreateImage(size, IPL_DEPTH_8U , 3);//YCrCb image
		skCvtColor(pIplImg, pRGB, CV_YCrCb2RGB);
//		cvCvtColor(pIplImg, pRGB, CV_YCrCb2RGB);
		cvSaveImage(name, pRGB);
	}
	else
		assert(false);

	if (pIplImg != 0)
		cvReleaseImage( &pIplImg );
	if (pRGB != 0)
		cvReleaseImage( &pRGB );

	return channel;
}

// return channel number
// pChnMat should be allocated as a matrix
int skLoadImage2YCrCb(const char *name, Matrix<float> *pChnMat)
{
	char msg[512];

	IplImage *pIplImg = cvLoadImage(name, -1);//1-color, 0-gray,-1~as it be


	IplImage *pYCrCb = 0, *tmp = 0;
	if (pIplImg == 0)
	{
		sprintf(msg, "\n Couldn't find image %s\n", name);
		skOutput(msg);
		throw("int skLoadImage2YCrCb(const char *name, Matrix<float> *pChnMat) failed"); // Qi Ren
		return -1;
	}

	int channel = pIplImg->nChannels;
	if (channel == 1)
	{
		skCopyCVImage2SkMatrix(pIplImg, pChnMat, channel);
	}
	else if (channel == 3)
	{
//		Y=0.299*R + 0.587*G + 0.114*B
//		Cr=(R-Y)*0.713 + 128
//		Cb=(B-Y)*0.564 + 128
//
//		R=Y + 1.403*(Cr - 128)
//		G=Y - 0.344*(Cb - 128) - 0.714*(Cr - 128)
//		B=Y + 1.773*(Cb - 128)
		pYCrCb = cvCreateImage(cvGetSize(pIplImg), IPL_DEPTH_8U , 3);//YCrCb image
		skCvtColor(pIplImg, pYCrCb, CV_RGB2YCrCb);
//		cvCvtColor(pIplImg, pYCrCb, CV_RGB2YCrCb);
		
//		skCvtColor(pYCrCb, pIplImg, CV_YCrCb2RGB);
//		cvSaveImage("tmp002.bmp", pIplImg);

		skCopyCVImage2SkMatrix(pYCrCb, pChnMat, channel);
	}
	else
		assert(false);

	cvReleaseImage( &pIplImg );
	if (pYCrCb != 0)
		cvReleaseImage( &pYCrCb );

	return channel;
}
///////////////////
// return channel number
int skSaveImageFromYCrCb(const char *name, Matrix<float> *pChnMat, int channel)
{
	char msg[512];
	int wd, ht;
	pChnMat[0].GetDimension(ht, wd);
	if (ht <= 0 || wd <= 0)
		return 1;

	CvSize size;
	size.width = wd;
	size.height = ht;

	IplImage *pIplImg = 0;
	IplImage *pRGB = 0;
	if (channel == 1)
	{
		pIplImg = cvCreateImage(size, IPL_DEPTH_8U, 1);//YCrCb image
		skCopySkMatrix2CVImage(pChnMat, pIplImg, channel);
		cvSaveImage(name, pIplImg);
	}
	else if (channel == 3)
	{
//		Y=0.299*R + 0.587*G + 0.114*B
//		Cr=(R-Y)*0.713 + 128
//		Cb=(B-Y)*0.564 + 128
//
//		R=Y + 1.403*(Cr - 128)
//		G=Y - 0.344*(Cb - 128) - 0.714*(Cr - 128)
//		B=Y + 1.773*(Cb - 128)
		pIplImg = cvCreateImage(size, IPL_DEPTH_8U , 3);//YCrCb image
		skCopySkMatrix2CVImage(pChnMat, pIplImg, channel);
		pRGB = cvCreateImage(size, IPL_DEPTH_8U , 3);//YCrCb image
		skCvtColor(pIplImg, pRGB, CV_YCrCb2RGB);
//		cvCvtColor(pIplImg, pRGB, CV_YCrCb2RGB);
		cvSaveImage(name, pRGB);
	}
	else
		assert(false);

	if (pIplImg != 0)
		cvReleaseImage( &pIplImg );
	if (pRGB != 0)
		cvReleaseImage( &pRGB );

	return channel;
}

void skCvtColor( const IplImage* src, IplImage* dst, int code )
{
	assert(src->nChannels == 3);

	CvMat  *pMatSrc, *pMatDst;
    CvMat  srcStub, dstStub;
	int coi;

	pMatSrc = cvGetMat( src, &srcStub, &coi );
	pMatDst = cvGetMat( dst, &dstStub, &coi );

	uchar *pDataSrc = pMatSrc->data.ptr;
	CvSize sizeSrc = cvGetSize( pMatSrc );
	uchar *pDataDst = pMatDst->data.ptr;
	CvSize sizeDst = cvGetSize( pMatDst );
	assert(sizeSrc.height == sizeDst.height && sizeSrc.width == sizeDst.width);

	int nWdStep = pMatSrc->step;

	int wd = sizeSrc.width;
	int ht = sizeDst.height;

	int value;
	int r, c, i;
	uchar R, G, B, Y, Cr, Cb;
	double d_R, d_G, d_B, d_Y, d_Cr, d_Cb;

	double dSCr = 0.713;
	double dSCb = 0.564;

//	double dSCr = 0.20;
//	double dSCb = 0.10;

	if (code == CV_RGB2YCrCb)
	{
	//Y=0.299*R + 0.587*G + 0.114*B
	//Cr=(R-Y)*0.713 + 128
	//Cb=(B-Y)*0.564 + 128
		for (r = 0; r < ht; r++)
		{
			for (c = 0; c < wd; c++)
			{
				// Qi Ren: the channel order in bitmap is BGR 09-03-2004
				// Qi Ren: proper channel seqience according to header infor 09-20-2004

				if ( strcmp(src->channelSeq, "RGB" ) == 0 )
				{
					R = (pDataSrc + r*nWdStep)[c*3];
					G = (pDataSrc + r*nWdStep)[c*3+1];
					B = (pDataSrc + r*nWdStep)[c*3+2];
				}
				else
				{
					assert( strcmp(src->channelSeq, "BGR" ) == 0 );
					B = (pDataSrc + r*nWdStep)[c*3];
					G = (pDataSrc + r*nWdStep)[c*3+1];
					R = (pDataSrc + r*nWdStep)[c*3+2];
				}
				

				d_Y=0.299*R + 0.587*G + 0.114*B;
//				d_Cr=(R-Y)*0.713 + 128;
//				d_Cb=(B-Y)*0.564 + 128;
				d_Cr=((double)R-d_Y)*dSCr + 128.0;
				d_Cb=((double)B-d_Y)*dSCb + 128.0;

				if (d_Y < 0) 
					Y = 0;
				else if (d_Y > 255) 
					Y = 255;
				else 
					Y = (uchar)d_Y;

				if (d_Cr < 0) 
					Cr = 0;
				else if (d_Cr > 255) 
					Cr = 255;
				else 
					Cr = (uchar)d_Cr;

				if (d_Cb < 0) 
					Cb = 0;
				else if (d_Cb > 255) 
					Cb = 255;
				else 
					Cb = (uchar)d_Cb;

				(pDataDst + r*nWdStep)[c*3] = Y;
				(pDataDst + r*nWdStep)[c*3+1] = Cr;
				(pDataDst + r*nWdStep)[c*3+2] = Cb;
			}
		}
	}
	else if (code == CV_YCrCb2RGB)
	{
	//Y=0.299*R + 0.587*G + 0.114*B
	//Cr=(R-Y)*0.713 + 128
	//Cb=(B-Y)*0.564 + 128

	//R=Y + 1.0/0.713*(Cr - 128)
	//G=Y - 0.114/(0.564*0.587)*(Cb - 128) - 0.299/(0.713*0.587)*(Cr - 128)
	//B=Y + 1.0/0.564*(Cb - 128)
		for (r = 0; r < ht; r++)
		{
			for (c = 0; c < wd; c++)
			{
				Y = (pDataSrc + r*nWdStep)[c*3];
				Cr = (pDataSrc + r*nWdStep)[c*3+1];
				Cb = (pDataSrc + r*nWdStep)[c*3+2];

//				d_R=Y + 1.0/0.713*(Cr - 128);
//				d_G=Y - 0.114/(0.564*0.587)*(Cb - 128) - 0.299/(0.713*0.587)*(Cr - 128);
//				d_B=Y + 1.0/0.564*(Cb - 128);

				d_R=Y + 1.0/dSCr*(Cr - 128.0);
				d_G=Y - 0.114/(dSCb*0.587)*(Cb - 128.0) - 0.299/(dSCr*0.587)*(Cr - 128.0);
				d_B=Y + 1.0/dSCb*(Cb - 128.0);

				if (d_R < 0) 
					R = 0;
				else if (d_R > 255) 
					R = 255;
				else 
					R = (uchar)d_R;

				if (d_G < 0) 
					G = 0;
				else if (d_G > 255) 
					G = 255;
				else 
					G =	(uchar)d_G;

				if (d_B < 0) 
					B = 0;
				else if (d_B > 255) 
					B = 255;
				else 
					B = (uchar)d_B;

				(pDataDst + r*nWdStep)[c*3] = R;
				(pDataDst + r*nWdStep)[c*3+1] = G;
				(pDataDst + r*nWdStep)[c*3+2] = B;
			}
		}
	}
	else
		assert(false);

}



