#include <direct.h>
#include <time.h>

#include "skMatrixIplOp.h"
#include "skFilter.h"
#include "skFilterCoef.h"
#include "skOutput.h"

#include "skImgSeg.h"

#include "skLocalHist.h"

/////////////////////////////////////////////////////////////////////
// class SkImgSegParam
SkImgSegParam::SkImgSegParam()
{
	m_nLocalHistHalfSize = 7;
	m_nSubSampleRate = 1;
}
SkImgSegParam::~SkImgSegParam()
{
}
int SkImgSegParam::read(const char *name)
{
	FILE *pFile;
	pFile = fopen(name, "r");
	if (pFile == 0)
		return 1;

	fscanf(pFile, "LocalHistHalfSize:      %d\n", &m_nLocalHistHalfSize);// half size for the local histogram
	fscanf(pFile, "SubSampleRate:          %d\n", &m_nSubSampleRate);// sub sample rate

	fclose(pFile);
	return 0;
}
int SkImgSegParam::save(const char *name)
{
	FILE *pFile;
	pFile = fopen(name, "w");
	if (pFile == 0)
		return 1;
	
	fprintf(pFile, "LocalHistHalfSize:      %d\n", m_nLocalHistHalfSize);// half size for the local histogram
	fprintf(pFile, "SubSampleRate:          %d\n", m_nSubSampleRate);// sub sample rate
	
	fclose(pFile);
	return 0;
}
///////////////////
// skImageSegmentationSketch()
// need x.bmp, x_con.bmp; output: x_cls.bmp, x_cls_show.bmp
int skImageSegmentationSketch(const char *pchImgName, int nMaxClusterNum)
{
	char pchProjName[512], name[512], pchDir[512];
	char pchDir0[512];
	_getcwd(pchDir0, 512);

	char msg[512];
	sprintf(msg, "\n      Image Segmentation for %s begins ..............\n", pchImgName);
	skOutput(msg);

	int nReturn;
	//read original image
	Matrix<float> org;
	nReturn = skReadBitmap(pchImgName, &org);
	if (nReturn != 0)
	{
		sprintf(msg, "      Couldn't read image %s\n", pchImgName);
		skOutput(msg);
		return 1;
	}
	int wd, ht;
	org.GetDimension(ht, wd);
	// normalize the image to 0 mean 1 variance
	double dImgMean=0, dImgStd = -1;
	skNormalizeMatrix2ZeroMean1Std(&org, dImgMean, dImgStd);

	// retreive the project name and directory name
	strcpy(name, pchImgName);
	char *temp;
	temp = strrchr(name, '.');//find '.', return the pointer to it
	*temp = '\0';
	strcpy(pchProjName, name);
	strcpy(pchDir, name);

	_mkdir(pchDir);
	_chdir(pchDir);//change current dir as the image dir

	// read residual image
	Matrix<float> res1;
	sprintf(name, "%s_res1.bmp", pchProjName);
	nReturn = skReadBitmap(name, &res1);
	if (nReturn != 0)
	{
		sprintf(msg, "      Couldn't read image %s,\n      use original", name);
		skOutput(msg);
		res1 = org;
//		return 1;
	}
	else
		res1 = (res1-dImgMean)/dImgStd;//normalize



	// read the constraint file
	Matrix<float> con;
	sprintf(name, "%s_con.bmp", pchProjName);
	nReturn = skReadBitmap(name, &con);
//	assert(nReturn == 0);
	if (nReturn != 0)
	{
		sprintf(msg, "      Couldn't read image %s\n", name);
		skOutput(msg);
		return 1;
	}


	// write or read paramters
	SkImgSegParam param;
	sprintf(name, "%s_imgseg_param.txt", pchProjName);
//	if (param.read(name) != 0)
	param.save(name);//if read fails, then save the default

	//filters
	SkFilterBankParam fb_param;
	sprintf(name, "%s_imgseg_filter_param.txt", pchProjName);

	// construct FRAME filters and compute the filter responses
	SkFilterBank filterBank, *pFB = &filterBank;
//	if (pFB->readParameters(name) != 0)
		pFB->m_param.initilaize4TextureClustering();
	pFB->saveParameters(name);//no such parameter file, save the default
	pFB->buildFilters();

	DWORD Time0 = ::GetTickCount();

	Matrix<float> ones(ht, wd), texture;
	ones.Set(255);
	texture = ones - con;

	//interpolation of the sketchable part
	sprintf(msg, "\n      Interpolation begins ...");
	::skOutput(msg);
	skInterpolation(&texture, &res1);
	sprintf(name, "%s_seg_init.bmp", pchProjName);
	::skSaveMatrixAsBitmap(name, res1, dImgMean, dImgStd);
	sprintf(msg, "\n      Interpolation ends");
	::skOutput(msg);


	SkFilterCoef fc;
	fc.setFilterBank(pFB);
//	fc.setMatrixImage(&org); 
	fc.setMatrixImage(&res1); 
	sprintf(name, "%s_imgseg", pchProjName);
	fc.setProjName(name);
	fc.initializeCoefficencies();

	int nHalfLocalSize = param.m_nLocalHistHalfSize;
	int nSub = param.m_nSubSampleRate;
	SkLocalHist localHist;
	localHist.setHalfLocalSize(nHalfLocalSize);
	localHist.setSubSampleRate(nSub);
	localHist.build(&fc, &texture);//texture - 0: is valid

	Matrix<float> cluster_sm;//small clusters with sub sampling
	
	// clustering on texture local histogram
	SkTextureCluster::clustering(&localHist, cluster_sm, nMaxClusterNum);
	
	
	Matrix<float> cluster(ht, wd);
	cluster.Set(0);

	nSub = localHist.m_nSub;
	int r, c, r1, c1;
	for (r = 0; r < ht; r++)
	{
		r1 = r / nSub;
		for (c = 0; c < wd; c++)
		{
			c1 = c / nSub;
			if (fabs(texture(r,c)) < 0.001)
			{
				cluster(r,c) = cluster_sm(r1, c1);
			}
		}
	}

	sprintf(name, "%s_cls.bmp", pchProjName);
	::skSaveMatrixAsBitmap(name, cluster, 0, 1);
	sprintf(name, "%s_cls_show.bmp", pchProjName);
	::skSaveMatrixAsBitmap(name, cluster, 0, -1);

	int nClusterNum = cluster.GetMax();
	sprintf(msg, "\n      Total region num: %d", nClusterNum + 1);
	::skOutput(msg);

	_chdir(pchDir0);

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "\n      Total image segmentation time for image %s:   %d min %.3fs \n ", pchImgName, nMin, dSec);
	::skOutput(msg);

	return 0;
}
