#include <direct.h>
#include <time.h>

#include "skJulesz.h"
#include "skFilter.h"
#include "skFilterCoef.h"
#include "skOutput.h"
#include "skGaussian.h"
#include "skMatrixIplOp.h"
#include "skBasePursuit.h"
#include "skImgSeg.h"
#include "skPrimitive.h"
#include "skPrimPursuit.h"

/////////////////////////////////////////////////////////////////////
// class SkJuleszParam

///////////////////
// constructor
SkJuleszParam::SkJuleszParam()
{
	m_nUseFrameLearnLambda = 0;//use the frame model and learn the lambda
	m_dLambdaLearnRate = 10000;//the learn rate for the lambda
	m_nSynWd = -1;
	m_nSynHt = -1;//synthesized image width and height
	m_nUseFilterSelection = 1;//select first m_nFilterNum filters for each region
	m_nFilterNum = 100;//filter num used in modeling
	m_nFirstFixedFilterNum = 100;//first filters: intensity, dx, dy, ddx, ddy, DOG 1, DOG 2, 
	m_nUseRegionClustering = 0;//automatically cluster regions 
	m_nMaxRegionNum = 1;//maximum region number
	m_nBoundCond = 0;//boundary conditions
	m_nUseAnealing = 1;//use anealing or not, if not greedy 
	m_dStartTemp = 0.5;//if anealing, start temperature
	m_dEndTemp = 0.2;// and end temperature
	m_nAnealingSteps = 10;//anealing steps which reducing the temperature
	m_nSweepsPerTemp = 2;//sampling sweeps per each temperature
	m_nImgGrayLevels = 32;//256;//8;//256;
	m_nFirDerMaxBinNum = 13;//the max bin num for first derivative
	m_nSecDerMaxBinNum = 13;//the max bin num for second derivative
	m_nLogMaxBinNum = 13;//the max bin num for Log
	m_nDOOGMaxBinNum = 13;//the max bin num for DOOG filters
}

///////////////////
// de-constructor
SkJuleszParam::~SkJuleszParam()
{
}

///////////////////
// read()
int SkJuleszParam::read(const char *name)
{
	FILE *pFile;
	pFile = fopen(name, "r");
	if (pFile == 0)
		return 1;
	fscanf(pFile, "UseFrameLearnLambda:     %d\n", &m_nUseFrameLearnLambda);//use the frame model and learn the lambda
	fscanf(pFile, "LambdaLearnRate:         %lf\n", &m_dLambdaLearnRate);//the learn rate for the lambda
	fscanf(pFile, "Syn image width:         %d\n", &m_nSynWd);
	fscanf(pFile, "Syn image width:         %d\n", &m_nSynHt);//synthesized image width and height
	fscanf(pFile, "Use Filter Selection:    %d\n", &m_nUseFilterSelection);//select first m_nFilterNum filters for each region
	fscanf(pFile, "Filter Num:              %d\n", &m_nFilterNum);//filter num used in modeling
	fscanf(pFile, "FirstFixedFilterNum:     %d\n", &m_nFirstFixedFilterNum);//filter num used in modeling
	fscanf(pFile, "Use Region Clustering:   %d\n", &m_nUseRegionClustering);//automatically cluster regions 
	fscanf(pFile, "Max Region Num:          %d\n", &m_nMaxRegionNum);//maximus region number
	fscanf(pFile, "Boundary Condition:      %d\n", &m_nBoundCond);//boundary conditions
	fscanf(pFile, "Use Anealing:            %d\n", &m_nUseAnealing);//use anealing or not, if not greedy 
	fscanf(pFile, "Start Temperature:       %lf\n", &m_dStartTemp);//if anealing, start temperature
	fscanf(pFile, "End Temperature:         %lf\n", &m_dEndTemp);// and end temperature
	fscanf(pFile, "Anealing steps:          %d\n", &m_nAnealingSteps);//anealing steps which reducing the temperature
	fscanf(pFile, "Sweeps Per Temperature:  %d\n", &m_nSweepsPerTemp);//sampling sweeps per each temperature
	fscanf(pFile, "Image Gray Levels:       %d\n", &m_nImgGrayLevels);//the gray levels of input image
	fscanf(pFile, "FirDerMaxBinNum:         %d\n", &m_nFirDerMaxBinNum);//the max bin num for first derivative
	fscanf(pFile, "SecDerMaxBinNum:         %d\n", &m_nSecDerMaxBinNum);//the max bin num for second derivative
	fscanf(pFile, "LogMaxBinNum:            %d\n", &m_nLogMaxBinNum);//the max bin num for Log
	fscanf(pFile, "DOOGMaxBinNum:          %d\n", &m_nDOOGMaxBinNum);//the max bin num for Gabor filters
	fclose(pFile);
	return 0;
}
///////////////////
// save()
int SkJuleszParam::save(const char *name)
{
	FILE *pFile;
	pFile = fopen(name, "w");
	if (pFile == 0)
		return 1;
	fprintf(pFile, "UseFrameLearnLambda:     %d\n", m_nUseFrameLearnLambda);//use the frame model and learn the lambda
	fprintf(pFile, "LambdaLearnRate:         %.5lf\n", m_dLambdaLearnRate);//the learn rate for the lambda
	fprintf(pFile, "Syn image width:         %d\n", m_nSynWd);
	fprintf(pFile, "Syn image width:         %d\n", m_nSynHt);//synthesized image width and height
	fprintf(pFile, "Use Filter Selection:    %d\n", m_nUseFilterSelection);//select first m_nFilterNum filters for each region
	fprintf(pFile, "Filter Num:              %d\n", m_nFilterNum);//filter num used in modeling
	fprintf(pFile, "FirstFixedFilterNum:     %d\n", m_nFirstFixedFilterNum);//filter num used in modeling
	fprintf(pFile, "Use Region Clustering:   %d\n", m_nUseRegionClustering);//automatically cluster regions 
	fprintf(pFile, "Max Region Num:          %d\n", m_nMaxRegionNum);//maximus region number
	fprintf(pFile, "Boundary Condition:      %d\n", m_nBoundCond);//boundary conditions
	fprintf(pFile, "Use Anealing:            %d\n", m_nUseAnealing);//use anealing or not, if not greedy 
	fprintf(pFile, "Start Temperature:       %.3lf\n", m_dStartTemp);//if anealing, start temperature
	fprintf(pFile, "End Temperature:         %.3lf\n", m_dEndTemp);// and end temperature
	fprintf(pFile, "Anealing steps:          %d\n", m_nAnealingSteps);//anealing steps which reducing the temperature
	fprintf(pFile, "Sweeps Per Temperature:  %d\n", m_nSweepsPerTemp);//sampling sweeps per each temperature
	fprintf(pFile, "Image Gray Levels:       %d\n", m_nImgGrayLevels);//the gray levels of input image
	fprintf(pFile, "FirDerMaxBinNum:         %d\n", m_nFirDerMaxBinNum);//the max bin num for first derivative
	fprintf(pFile, "SecDerMaxBinNum:         %d\n", m_nSecDerMaxBinNum);//the max bin num for second derivative
	fprintf(pFile, "LogMaxBinNum:            %d\n", m_nLogMaxBinNum);//the max bin num for Log
	fprintf(pFile, "DOOGMaxBinNum:          %d\n", m_nDOOGMaxBinNum);//the max bin num for Gabor filters

	fclose(pFile);
	return 0;
}


int skJuleszSamplingWithThreeModels(const char *pchImgName) // image file name
{
	//primitive pursuit
	// need x.bmp, output: x_con.bmp, x_pro.bmp
//	SkPrimPursuit pp;
//	pp.primPursuitOnOneImage(pchImgName);

	// build k-gon primitives
	SkPrimManager primMan;
	skBuildPrimitiveMap(pchImgName, &primMan);//find primitive map after finishing
//
	// need x.bmp, x_con.bmp, x_pro.bmp; output x_syn0.bmp
//	skInterpolationWithProfiles(pchImgName);

	// need x.bmp, x_syn0.bmp; output x_res0.bmp, x_syn1.bmp, x_res1.bmp
//	skComputeResidaulAfterDiffusionAndMatchingPursuit(pchImgName);

	// need x.bmp, x_con.bmp; output: x_cls.bmp, x_cls_show.bmp
//	int nMaxClusterNum = 7;
//	skImageSegmentationSketch(pchImgName, nMaxClusterNum);

	// need: x.bmp, x_res1.bmp, x_con.bmp, x_pro.bmp, x_cls.bmp; output: x_syn2.bmp
//	skJuleszSamplingOnResidualOfInterpolationAndMatchingPursuit(pchImgName);


	// need x.bmp, x_syn0.bmp, x_syn1.bmp, x_syn2.bmp, x_con.bmp; output: x_syn_all.bmp
//	skCombinSynResults(pchImgName);
	return 0;
}

///////////////////
// need x_con.bmp, x_pro.bmp
// output x_syn0.bmp
int skInterpolationWithProfiles(const char *pchImgName)
{
	char pchDir0[512];
	_getcwd(pchDir0, 512);

	char msg[512];

	int nReturn;
	//read original image
	Matrix<float> org;
	nReturn = skReadBitmap(pchImgName, &org);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", pchImgName);
		skOutput(msg);
		return 1;
	}
	int wd, ht;
	org.GetDimension(ht, wd);
	// normalize the image to 0 mean 1 variance
	double dImgMean = 0, dImgStd = 1;
//	skNormalizeMatrix(&org, dImgMean, dImgStd);


	// retreive the project name and directory name
	char pchProjName[512], name[512], pchDir[512];
	strcpy(name, pchImgName);
	char *temp;
	temp = strrchr(name, '.');//find '.', return the pointer to it
	*temp = '\0';
	strcpy(pchProjName, name);
	strcpy(pchDir, name);

	_mkdir(pchDir);

	_chdir(pchDir);//change current dir as the image dir

	nReturn = skSaveMatrixAsBitmap(pchImgName, org, dImgMean, dImgStd);


	Matrix<float> orgCon, orgPro;//original constraints, profile, clusters
	Matrix<float> syn(ht,wd);// synthesized

	Matrix<float> *pSynCon = &orgCon;
	Matrix<float> *pSynPro = &orgPro;

	sprintf(name, "%s_con.bmp", pchProjName);
	nReturn = skReadBitmap(name, &orgCon);
	assert (nReturn == 0);

	sprintf(name, "%s_pro.bmp", pchProjName);
	nReturn = skReadBitmap(name, &orgPro);
	assert (nReturn == 0);

	sprintf(msg, "\n\n Interpolation for image %s begins .... \n", pchImgName);
	skOutput(msg);

	DWORD Time0 = ::GetTickCount();

	int w=wd, h=ht;
	pSynCon->GetDimension(h, w);
	int i, j;

	//compute image mean and std
	double mean=0, std=0;
//	skGetMatrixMeanAndStd(&org, mean, std);
	int n = 0;
	for (i = 0; i < h; i++)
	{
		for ( j = 0; j < w; j++)
		{
			if ((*pSynCon)(i,j) != 0)
			{//not on con, 
				mean += org(i,j);
				n++;
			}
		}
	}
	if (n != 0)
	{
		mean /= n;
		for (i = 0; i < h; i++)
		{
			for ( j = 0; j < w; j++)
			{
				if ((*pSynCon)(i,j) != 0)
				{
					std += (org(i,j)-mean)*(org(i,j)-mean);
				}
			}
		}
		std = sqrt(std/n);
		//discritized
		mean = (int) (mean + 0.5);
		std = (int) (std + 0.5);
	}
	else
	{
		skGetMatrixMeanAndStd(&org, mean, std);
		//discritized
		mean = (int) (mean + 0.5);
		std = (int) (std + 0.5);
	}
	//generate syn
	for (i = 0; i < h; i++)
	{
		for ( j = 0; j < w; j++)
		{
			if ((*pSynCon)(i,j) == 0)
				syn(i,j) = (*pSynPro)(i,j);
			else//randomize non-fixed part
			{
				syn(i,j) = CGaussian::getOneSampleFromGaussian()*std + mean;
//						syn(i,j) = 0;
//						syn(i,j) = (*pSynPro)(i,j);
				syn(i,j) = (int)syn(i,j);
			}
		}
	}
	skInterpolation(pSynCon, &syn);

	sprintf(name, "%s_syn0.bmp", pchProjName);
	nReturn = skSaveMatrixAsBitmap(name, syn, dImgMean, dImgStd);

	for (i = 0; i < h; i++)
	{
		for ( j = 0; j < w; j++)
		{
			if ((*pSynCon)(i,j) != 0)
				syn(i,j) = 256;
		}
	}
	sprintf(name, "%s_syn0_only.bmp", pchProjName);
	nReturn = skSaveMatrixAsBitmap(name, syn, dImgMean, dImgStd);

	_chdir(pchDir0);

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "\n Interpolation time for image %s:   %d min %.3fs \n ", pchImgName, nMin, dSec);
	::skOutput(msg);

	return 0;
}

/////////////////////////////////////////////////////////////////////
// 
///////////////////
// skJuleszSampling() - a general method
//	given an image file name and parameters
//  for given X.bmp
//		X_con.bmp, X_pro.bmp, X_cls.bmp are checked
//		if (X_con.bmp doesn't exist) then no constraints
//		X_syn_con.bmp, X_syn_pro.bmp, X_syn_cls.bmp are also checked
int skJuleszSampling(const char *pchImgName) // image file name
{	

	char pchDir0[512];
	_getcwd(pchDir0, 512);

	char msg[512];

	int nReturn;
	//read original image
	Matrix<float> org;
	nReturn = skReadBitmap(pchImgName, &org);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", pchImgName);
		skOutput(msg);
		return 1;
	}
	int wd, ht;
	org.GetDimension(ht, wd);
	// normalize the image to 0 mean 1 variance
	double dImgMean = 0, dImgStd = 1;
//	skNormalizeMatrix(&org, dImgMean, dImgStd);

	// retreive the project name and directory name
	char pchProjName[512], name[512], pchDir[512];

	std::string strImgFile(pchImgName);

	std::string strDir, strImgBaseName;

	int idx0 = strImgFile.find_last_of('\\');
	int idx1 = strImgFile.find_last_of('.');

	strDir = strImgFile.substr(0, idx1);
	strDir += '\\';

	strImgBaseName = strImgFile.substr(idx0+1, idx1-idx0-1);

	strcpy( pchDir, strDir.c_str() );
	strcpy( pchProjName, strImgBaseName.c_str() );

	_mkdir(pchDir);

	_chdir(pchDir);//change current dir as the image dir

	nReturn = skSaveMatrixAsBitmap(pchImgName, org, dImgMean, dImgStd);

	// write or read paramters
	SkJuleszParam param;
	char pchParamName[512];
	sprintf(pchParamName, "%s_julesz_param.txt", pchProjName);
//	param.read(pchParamName);
	param.save(pchParamName);//if read fails, then save the default

	sprintf(param.m_pchFilterParamName, "%s%s_julesz_filter_param.txt", pchDir, pchProjName);

	dImgStd = skResetImageGrayLevel(org, param.m_nImgGrayLevels);
	dImgMean = 0; 

	sprintf(name, "%s_scaled.bmp", pchProjName);
	::skSaveMatrixAsBitmap(name, org, dImgMean, dImgStd);

	//save a copy to synthesized folder, just for comparision
	char syn_dir[32];
	sprintf(syn_dir, "syn");
	_mkdir(syn_dir);
	sprintf(name, "%s\\%s_scaled.bmp", syn_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, org, dImgMean, dImgStd);


	// read all possilbe provided images
//	Matrix<float> orgCon, orgPro, orgCls;//original constraints, profile, clusters
//	Matrix<float> syn(ht,wd), synCon, synPro, synCls;// synthesized
//	Matrix<float> *pOrgCon=&orgCon, *pOrgPro=&orgPro, *pOrgCls=&orgCls;//original constraints, profile, clusters
//	Matrix<float> *pSynCon=&synCon, *pSynPro=&synPro, *pSynCls=&synCls;// synthesized ones

	Matrix<float> orgCon, orgPro, orgCls;//original constraints, profile, clusters
	Matrix<float> syn(ht,wd), synCon, synPro, synCls;// synthesized
	Matrix<float> *pOrgCon=&orgCon, *pOrgPro=&orgPro, *pOrgCls=&orgCls;//original constraints, profile, clusters
	Matrix<float> *pSynCon=&synCon, *pSynPro=&synPro, *pSynCls=&synCls;// synthesized ones


	sprintf(name, "%s_con.bmp", pchProjName);
	nReturn = skReadBitmap(name, &orgCon);
	if (nReturn != 0)
		pOrgCon = 0;

	sprintf(name, "%s_pro.bmp", pchProjName);
	nReturn = skReadBitmap(name, &orgPro);
	if (nReturn != 0)
		pOrgPro = 0;
	else
//		skNormalizeMatrix(&orgPro, dImgMean, dImgStd);
		skResetImageGrayLevel(orgPro, param.m_nImgGrayLevels);

	sprintf(name, "%s_cls.bmp", pchProjName);
	nReturn = skReadBitmap(name, &orgCls);
	if (nReturn != 0)
		pOrgCls = 0;


	sprintf(name, "%s_syn_con.bmp", pchProjName);
	nReturn = skReadBitmap(name, &synCon);
	if (nReturn != 0)
		pSynCon = pOrgCon;

	sprintf(name, "%s_syn_pro.bmp", pchProjName);
	nReturn = skReadBitmap(name, &synPro);
	if (nReturn != 0)
		pSynPro = pOrgPro;
//	else
//		skNormalizeMatrix(&synPro, dImgMean, dImgStd);
		skResetImageGrayLevel(synPro, param.m_nImgGrayLevels);

	sprintf(name, "%s_syn_cls.bmp", pchProjName);
	nReturn = skReadBitmap(name, &synCls);
	if (nReturn != 0)
		pSynCls = pOrgCls;

	sprintf(msg, "\n\nJuelesz sampling for image %s begins .... \n", pchImgName);
	skOutput(msg);

	DWORD Time0 = ::GetTickCount();

	sprintf(name, "%s_syn.bmp", pchProjName);
	nReturn = skReadBitmap(name, &syn);
	if (nReturn != 0)//
	{//syn not provided
		if (pSynPro != 0 && pSynCon != 0)//if profile provided, use it
		{ 
			skComputeInitSynByInterpolation(org, pSynCon, pSynPro, syn);
		}
		else//profiles not provided
		{//randomize syn
			skComputeInitSynByRandom(param.m_nImgGrayLevels, syn);
		}
		sprintf(name, "%s_syn_init.bmp", pchProjName);
		nReturn = skSaveMatrixAsBitmap(name, syn, dImgMean, dImgStd);
	}//synthesized image not 
	else
//		skNormalizeMatrix(&syn, dImgMean, dImgStd);
		skResetImageGrayLevel(syn, param.m_nImgGrayLevels);
	// julesz sampling
	int nNumSampling = skJuleszSampling(&org, &syn, param, dImgMean, dImgStd, pchDir, pchProjName, pOrgCon, pSynCon, pOrgPro, pSynPro, pOrgCls, pSynCls);

	_chdir(pchDir0);

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "\n Total Julesz sampling time for image %s:   %d min %.3fs \n ", pchImgName, nMin, dSec);
	::skOutput(msg);

	return nNumSampling;
}
int	skComputeInitSynByRandom(int nLevels, 
							Matrix<float> &syn)
{
	int w, h;
	syn.GetDimension(h, w);
	int i, j;
	for (i = 0; i < h; i++)
	{
		for ( j = 0; j < w; j++)
		{
			syn(i,j) = (nLevels*rand())/(RAND_MAX+1.0);
			syn(i,j) = (int)syn(i,j);
		}
	}
	return 0;
}

int	skComputeInitSynByInterpolation(Matrix<float> &org,
									Matrix<float> *pSynCon, 
									Matrix<float> *pSynPro, 
									Matrix<float> &syn)
{
	int w, h;
	pSynCon->GetDimension(h, w);
	syn.SetDimension(h, w);
	int i, j;

	//compute image mean and std
	double mean=0, std=0;
	int n = 0;
	for (i = 0; i < h; i++)
	{
		for ( j = 0; j < w; j++)
		{
			if ((*pSynCon)(i,j) != 0)
			{
				mean += org(i,j);
				n++;
			}
		}
	}
	if (n != 0)
	{
		mean /= n;
		for (i = 0; i < h; i++)
		{
			for ( j = 0; j < w; j++)
			{
				if ((*pSynCon)(i,j) != 0)
				{
					std += (org(i,j)-mean)*(org(i,j)-mean);
				}
			}
		}
		std = sqrt(std/n);
		//discritized
		mean = (int) (mean + 0.5);
		std = (int) (std + 0.5);
	}
	else
	{
		skGetMatrixMeanAndStd(&org, mean, std);
		//discritized
		mean = (int) (mean + 0.5);
		std = (int) (std + 0.5);
	}
	//generate syn
	for (i = 0; i < h; i++)
	{
		for ( j = 0; j < w; j++)
		{
			if ((*pSynCon)(i,j) == 0)
				syn(i,j) = (*pSynPro)(i,j);
			else//randomize non-fixed part
			{
				syn(i,j) = CGaussian::getOneSampleFromGaussian()*std + mean;
//						syn(i,j) = 0;
//						syn(i,j) = (*pSynPro)(i,j);
				syn(i,j) = (int)syn(i,j);
			}
		}
	}
	skInterpolation(pSynCon, &syn);
	return 0;
}

int	skComputeInitSynByClusterAverage(Matrix<float> &org,
									Matrix<float> &con,
									Matrix<float> &pro,
									Matrix<float> &cls, 
									Matrix<float> &syn)
{
	int wd, ht;
	org.GetDimension(ht, wd);
	syn.SetDimension(ht, wd);

	//find mean for each cluster
	int pnNum[1024];
	double pdSum[1024];
	memset(pnNum, 0, sizeof(int)*1024);
	memset(pdSum, 0, sizeof(double)*1024);

	int nMaxC = -1;
	int x, y;
	int c;
	for (y = 0; y < ht; y++)
	{
		for (x = 0; x < wd; x++)
		{
			c = (int)(cls(y,x) + 0.5);
			if ( c > nMaxC)
				nMaxC = c;
			assert(c <=1024);
			pdSum[c] += org(y,x);
			pnNum[c]++;
		}
	}
	for (c = 0; c <= nMaxC; c++)
	{
		if (pnNum[c] != 0)
			pdSum[c] /= pnNum[c];
		else
			pdSum[c] = 0;
	}
	for (y = 0; y < ht; y++)
	{
		for (x = 0; x < wd; x++)
		{
			c = (int)(cls(y,x) + 0.5);
			if (fabs(con(y,x)) < 0.001)//constraint
				syn(y,x) = pro(y,x);
			else
				syn(y,x) = pdSum[c];
		}
	}
	return 0;
}
///////////////////
// need: x_res1.bmp, x_con.bmp, x_pro.bmp, x_cls.bmp, 
// output: x_syn2.bmp
int skJuleszSamplingOnResidualOfInterpolationAndMatchingPursuit(const char *pchImgName) // image file name
{	
	char pchDir0[512];
	_getcwd(pchDir0, 512);

	char msg[512];

	int nReturn;

	// retreive the project name and directory name
	char pchProjName[512], name[512], pchDir[512];
	strcpy(name, pchImgName);
	char *temp;
	temp = strrchr(name, '.');//find '.', return the pointer to it
	*temp = '\0';
	strcpy(pchProjName, name);
	strcpy(pchDir, name);

	_mkdir(pchDir);

	_chdir(pchDir);//change current dir as the image dir

	//read original image
	Matrix<float> org;
	sprintf(name, "%s_res1.bmp", pchProjName);
//	nReturn = skReadBitmap(pchImgName, &org);
	nReturn = skReadBitmap(name, &org);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", name);
		skOutput(msg);
		return 1;
	}
	int wd, ht;
	org.GetDimension(ht, wd);
	// normalize the image to 0 mean 1 variance
	double dImgMean = 0, dImgStd = 1;
//	skNormalizeMatrix(&org, dImgMean, dImgStd);

//	nReturn = skSaveMatrixAsBitmap(pchImgName, org, dImgMean, dImgStd);

	// write or read paramters
	SkJuleszParam param;
	char pchParamName[512];
	sprintf(pchParamName, "%s_julesz_param.txt", pchProjName);
//	param.read(pchParamName);
	param.save(pchParamName);//if read fails, then save the default

	sprintf(param.m_pchFilterParamName, "%s_julesz_filter_param.txt", pchDir, pchProjName);

	param.m_nImgGrayLevels = 32;
	dImgStd = skResetImageGrayLevel(org, param.m_nImgGrayLevels);
	dImgMean = 0; 

	sprintf(name, "%s_res1_scaled.bmp", pchProjName);
	::skSaveMatrixAsBitmap(name, org, dImgMean, dImgStd);

	//save a copy to synthesized folder, just for comparision
	char syn_dir[32];
	sprintf(syn_dir, "syn");
	_mkdir(syn_dir);
	sprintf(name, "%s\\%s_scaled.bmp", syn_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, org, dImgMean, dImgStd);


	// read all possilbe provided images

	Matrix<float> orgCon, orgPro, orgCls;//original constraints, profile, clusters
	Matrix<float> syn(ht,wd), synCon, synPro, synCls;// synthesized
	Matrix<float> *pOrgCon=&orgCon, *pOrgPro=&orgPro, *pOrgCls=&orgCls;//original constraints, profile, clusters
	Matrix<float> *pSynCon=&synCon, *pSynPro=&synPro, *pSynCls=&synCls;// synthesized ones


	sprintf(name, "%s_con.bmp", pchProjName);
	nReturn = skReadBitmap(name, &orgCon);
	assert(nReturn == 0);

	// not use provided profile, use the interpolated one
	sprintf(msg, " \n            Interpolating sketchable part in res1.bmp .... \n");
	skOutput(msg);

	orgPro = org;
	Matrix<float> con_inv(ht,wd), mat255(ht,wd);//inverted constraints
	mat255.Set(255);
	con_inv = mat255 - orgCon;

	::skInterpolation(&con_inv, &orgPro);
	sprintf(name, "%s_res1_pro.bmp", pchProjName);
	nReturn = ::skSaveMatrixAsBitmap(name, orgPro, dImgMean, dImgStd);


	sprintf(name, "%s_cls.bmp", pchProjName);
	nReturn = skReadBitmap(name, &orgCls);
	if(nReturn != 0)
	{
		assert(false);
		orgCls.SetDimension(ht, wd);
		orgCls.Set(0);
	}



	synCon = orgCon;
	synPro = orgPro;
	synCls = orgCls;

	sprintf(msg, "\n\nJuelesz sampling for image %s begins .... \n", pchImgName);
	skOutput(msg);

	DWORD Time0 = ::GetTickCount();

	sprintf(name, "%s_syn.bmp", pchProjName);
	nReturn = skReadBitmap(name, &syn);
//	assert(nReturn != 0);
	if (nReturn != 0)//
	{//syn not provided
		if (pSynPro != 0 && pSynCon != 0)//if profile provided, use it
		{ 
			int w, h;
			pSynCon->GetDimension(h, w);
			syn.SetDimension(h, w);
			int i, j;

			//compute image mean and std
			double mean=0, std=0;
//			skGetMatrixMeanAndStd(&org, mean, std);
			int n = 0;
			for (i = 0; i < h; i++)
			{
				for ( j = 0; j < w; j++)
				{
					if ((*pSynCon)(i,j) != 0)
					{
						mean += org(i,j);
						n++;
					}
				}
			}
			if (n != 0)
			{
				mean /= n;
				for (i = 0; i < h; i++)
				{
					for ( j = 0; j < w; j++)
					{
						if ((*pSynCon)(i,j) != 0)
						{
							std += (org(i,j)-mean)*(org(i,j)-mean);
						}
					}
				}
				std = sqrt(std/n);
				//discritized
				mean = (int) (mean + 0.5);
				std = (int) (std + 0.5);
			}
			else
			{
				skGetMatrixMeanAndStd(&org, mean, std);
				//discritized
				mean = (int) (mean + 0.5);
				std = (int) (std + 0.5);
			}
			//generate syn
			for (i = 0; i < h; i++)
			{
				for ( j = 0; j < w; j++)
				{
					if ( (i==246 && j == 55)
						|| (i ==161 && j==51))
						i+= 0;

					if ((*pSynCon)(i,j) == 0)
						syn(i,j) = (*pSynPro)(i,j);
					else//randomize non-fixed part
					{
						syn(i,j) = CGaussian::getOneSampleFromGaussian()*std/2 + mean;
//						syn(i,j) = 0;
//						syn(i,j) = org(i,j);
						syn(i,j) = (int)syn(i,j);
					}
				}
			}
//			skInterpolation(pSynCon, &syn);
			sprintf(name, "%s_syn.bmp", pchProjName);
//			nReturn = skSaveMatrixAsBitmap(name, syn, dImgMean, dImgStd);
		}
		else//profiles not provided
		{//randomize syn
			double mean, std;
			skGetMatrixMeanAndStd(&org, mean, std);

			mean = (int) (mean + 0.5);
			std = (int) (std + 0.5);

			int w, h;
			syn.GetDimension(h, w);
			int i, j;
			for (i = 0; i < h; i++)
			{
				for ( j = 0; j < w; j++)
				{
//					syn(i,j) = CGaussian::getOneSampleFromGaussian()*std + mean;
					syn(i,j) = (param.m_nImgGrayLevels*rand())/(RAND_MAX+1.0);
					syn(i,j) = (int)syn(i,j);
				}
			}
			sprintf(name, "%s_syn.bmp", pchProjName);
//			nReturn = skSaveMatrixAsBitmap(name, syn, dImgMean, dImgStd);
		}
	}//synthesized image not 
	else
	{
//		skNormalizeMatrix(&syn, dImgMean, dImgStd);
		skResetImageGrayLevel(syn, param.m_nImgGrayLevels);
	}
	// julesz sampling
	skJuleszSampling(&org, &syn, param, dImgMean, dImgStd, pchDir, pchProjName, pOrgCon, pSynCon, pOrgPro, pSynPro, pOrgCls, pSynCls);

	_chdir(pchDir0);

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "\n Total Julesz sampling time for image %s:   %d min %.3fs \n ", pchImgName, nMin, dSec);
	::skOutput(msg);

	return 0;
}

///////////////////
// need x.bmp, x_syn0.bmp
// output x_res0.bmp, x_syn1.bmp, x_res1.bmp
int skComputeResidaulAfterDiffusionAndMatchingPursuit(const char *pchImgName) // image file name
{	
	char pchDir0[512];
	_getcwd(pchDir0, 512);

	char msg[512];

	int nReturn;
	//read original image
	Matrix<float> org;
	nReturn = skReadBitmap(pchImgName, &org);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", pchImgName);
		skOutput(msg);
		return 1;
	}
	int wd, ht;
	org.GetDimension(ht, wd);
	// normalize the image to 0 mean 1 variance
	double dImgMean = 0, dImgStd = 1;
	::skNormalizeMatrix2ZeroMean1Std(&org, dImgMean, dImgStd);


	// retreive the project name and directory name
	char pchProjName[512], name[512], pchDir[512];
	strcpy(name, pchImgName);
	char *temp;
	temp = strrchr(name, '.');//find '.', return the pointer to it
	*temp = '\0';
	strcpy(pchProjName, name);
	strcpy(pchDir, name);

	_mkdir(pchDir);

	_chdir(pchDir);//change current dir as the image dir

	nReturn = skSaveMatrixAsBitmap(pchImgName, org, dImgMean, dImgStd);

	Matrix<float> syn0;
	sprintf(name, "%s_syn0.bmp", pchProjName);
	nReturn = skReadBitmap(name, &syn0);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", name);
		skOutput(msg);
		return 1;
	}

	// 
	syn0 = (syn0 - dImgMean)/dImgStd;

	Matrix<float> res;
	res = org - syn0;


	sprintf(name, "%s_res0.bmp", pchProjName);
	nReturn = ::skSaveMatrixAsBitmap(name, res, dImgMean, dImgStd);


	char skps_dir[32];
	sprintf(skps_dir, "skps");
	_mkdir(skps_dir);

	int nSelect = 4;//matching pursuit
	// matching pursuit
	skSketchPursuitOnSelectedImage(res,//original image
								   dImgMean, dImgStd,//original image mean and std
								   skps_dir,//sketching pursuit working directory
								   pchDir, // working directory
								   pchProjName,//project name
								   nSelect);//0-original, 1-whitened, 2-edge strength, 3-entropy image

	Matrix<float> syn1;
	sprintf(name, "%s_syn1.bmp", pchProjName);
	nReturn = skReadBitmap(name, &syn1);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", name);
		skOutput(msg);
		return 1;
	}

	syn1 = (syn1-dImgMean)/dImgStd;

	Matrix<float> syn01;
	syn01 = syn0 + syn1;

	sprintf(name, "%s_syn01.bmp", pchProjName);
	nReturn = ::skSaveMatrixAsBitmap(name, syn01, dImgMean, dImgStd);

	_chdir(pchDir0);
	return 0;
}

// need x.bmp, x_syn0.bmp, x_syn1.bmp, x_syn2.bmp, x_con.bmp
// output: x_syn_all.bmp
int skCombinSynResults(const char *pchImgName)
{
		char pchDir0[512];
	_getcwd(pchDir0, 512);

	char msg[512];

	int nReturn;
	//read original image
	Matrix<float> org;
	nReturn = skReadBitmap(pchImgName, &org);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", pchImgName);
		skOutput(msg);
		return 1;
	}
	int wd, ht;
	org.GetDimension(ht, wd);
	// normalize the image to 0 mean 1 variance
	double dImgMean = 0, dImgStd = 1;
	::skNormalizeMatrix2ZeroMean1Std(&org, dImgMean, dImgStd);


	// retreive the project name and directory name
	char pchProjName[512], name[512], pchDir[512];
	strcpy(name, pchImgName);
	char *temp;
	temp = strrchr(name, '.');//find '.', return the pointer to it
	*temp = '\0';
	strcpy(pchProjName, name);
	strcpy(pchDir, name);

	_mkdir(pchDir);

	_chdir(pchDir);//change current dir as the image dir

	nReturn = skSaveMatrixAsBitmap(pchImgName, org, dImgMean, dImgStd);

	Matrix<float> syn0;
	sprintf(name, "%s_syn0.bmp", pchProjName);
	nReturn = skReadBitmap(name, &syn0);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", name);
		skOutput(msg);
		return 1;
	}
	syn0 = (syn0 - dImgMean)/dImgStd;

	Matrix<float> syn1;
	sprintf(name, "%s_syn1.bmp", pchProjName);
	nReturn = skReadBitmap(name, &syn1);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", name);
		skOutput(msg);
		return 1;
	}
	syn1 = (syn1 - dImgMean)/dImgStd;
	//
	Matrix<float> synRes;
	sprintf(name, "%s_syn2.bmp", pchProjName);
	nReturn = skReadBitmap(name, &synRes);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", name);
		skOutput(msg);
		return 1;
	}
	synRes = (synRes - dImgMean)/dImgStd;


	Matrix<float> con;
	sprintf(name, "%s_con.bmp", pchProjName);
	nReturn = skReadBitmap(name, &con);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", name);
		skOutput(msg);
		return 1;
	}

	Matrix<float> synAll(ht, wd);
	synAll.Set(0);

	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (con(r,c) == 0 )//0: constrainted, 1:to be syn
			{
				synAll(r,c) = syn0(r,c) + syn1(r,c);//255;//
			}
			else
			{
				synAll(r,c) = syn0(r,c) + syn1(r,c) + synRes(r,c);
			}
		}
	}



	sprintf(name, "%s_syn_txtr.bmp", pchProjName);
	nReturn = ::skSaveMatrixAsBitmap(name, synAll, dImgMean, dImgStd);


	_chdir(pchDir0);

	return 0;
}


///////////////////
// skJuleszSamplingSketch() - only for primal sketch model sampling
//	given a image file name and parameters
//  for given X.bmp
//		X_con.bmp, X_pro.bmp, X_cls.bmp are checked
//		if (X_con.bmp doesn't exist) then no constraints
//		X_syn_con.bmp, X_syn_pro.bmp, X_syn_cls.bmp are also checked
int skJuleszSamplingSketch(const char *pchImgName) // image file name
{
	char pchDir0[512];
	_getcwd(pchDir0, 512);

	char msg[512];

	int nReturn;
	//read original image
	Matrix<float> org;
	nReturn = skReadBitmap(pchImgName, &org);
	if (nReturn != 0)
	{
		sprintf(msg, "      Couldn't read image %s\n", pchImgName);
		skOutput(msg);
		return 1;
	}
	int wd, ht;
	org.GetDimension(ht, wd);
	// normalize the image to 0 mean 1 variance
	double dImgMean = 0, dImgStd = 1;
//	skNormalizeMatrix(&org, dImgMean, dImgStd);


	// retreive the project name and directory name
	char pchProjName[512], name[512], pchDir[512];
	std::string strImgFile(pchImgName);

	std::string strDir, strImgBaseName;

	int idx0 = strImgFile.find_last_of('\\');
	int idx1 = strImgFile.find_last_of('.');

	strDir = strImgFile.substr(0, idx1);
	strDir += '\\';

	strImgBaseName = strImgFile.substr(idx0+1, idx1-idx0-1);

	strcpy( pchDir, strDir.c_str() );
	strcpy( pchProjName, strImgBaseName.c_str() );

	_mkdir(pchDir);

	_chdir(pchDir);//change current dir as the image dir

	nReturn = skSaveMatrixAsBitmap(pchImgName, org, dImgMean, dImgStd);

	// write or read paramters
	SkJuleszParam param;
	char pchParamName[512];
	sprintf(pchParamName, "%s_julesz_param.txt", pchProjName);
//	param.read(pchParamName);
	param.save(pchParamName);//if read fails, then save the default

	sprintf(param.m_pchFilterParamName, "%s_julesz_filter_param.txt", pchDir, pchProjName);

	dImgStd = skResetImageGrayLevel(org, param.m_nImgGrayLevels);
	dImgMean = 0; 

	sprintf(name, "%s_scaled.bmp", pchProjName);
	::skSaveMatrixAsBitmap(name, org, dImgMean, dImgStd);

	//save a copy to synthesized folder, just for comparision
	char syn_dir[32];
	sprintf(syn_dir, "syn");
	_mkdir(syn_dir);
	sprintf(name, "%s\\%s_scaled.bmp", syn_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, org, dImgMean, dImgStd);


	// read all possilbe provided images

	Matrix<float> orgCon, orgPro, orgCls;//original constraints, profile, clusters
	Matrix<float> syn, synCon, synPro, synCls;// synthesized


	sprintf(name, "%s_con.bmp", pchProjName);
	nReturn = skReadBitmap(name, &orgCon);
	if (nReturn != 0)
		return -1;
	synCon = orgCon;


	sprintf(name, "%s_pro.bmp", pchProjName);
	nReturn = skReadBitmap(name, &orgPro);
	if (nReturn != 0)
		return -1;
	synPro = orgPro;

	skResetImageGrayLevel(orgPro, param.m_nImgGrayLevels);

	sprintf(name, "%s_cls.bmp", pchProjName);
	nReturn = skReadBitmap(name, &orgCls);
	if (nReturn != 0)
		return -1;
	synCls = orgCls;


	sprintf(msg, "\n      Juelesz sampling for image %s begins .... \n", pchImgName);
	skOutput(msg);

	DWORD Time0 = ::GetTickCount();

	sprintf(name, "%s_syn.bmp", pchProjName);
	nReturn = skReadBitmap(name, &syn);
	if (nReturn != 0)//
	{//syn not provided
		skComputeInitSynByClusterAverage(org, orgCon, orgPro, orgCls, syn);
		sprintf(name, "%s_syn_init.bmp", pchProjName);
		nReturn = skSaveMatrixAsBitmap(name, syn, dImgMean, dImgStd);
	}//synthesized image not 
	else
		skResetImageGrayLevel(syn, param.m_nImgGrayLevels);
	// julesz sampling
	skJuleszSampling(&org, &syn, param, dImgMean, dImgStd, pchDir, pchProjName, &orgCon, &synCon, &orgPro, &synPro, &orgCls, &synCls);

	_chdir(pchDir0);

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "\n      Total Julesz sampling time for image %s:   %d min %.3fs \n ", pchImgName, nMin, dSec);
	::skOutput(msg);

	return 0;
}

///////////////////
// skJuleszSampling()
int skJuleszSampling(Matrix<float> *pOrg, //original image
					 Matrix<float> *pSyn, //synthesized
					 SkJuleszParam &param, //parameters
					 double dImgMean, double dImgStd,//image mean and std
					 const char* pchDir, //work directory
					 const char* pchProjName, //project name
					 Matrix<float> *pOrgCon, //constraints for orignal image
					 Matrix<float> *pSynCon, //constraints for synthesized image
					 Matrix<float> *pOrgPro, //profile for orignal image
					 Matrix<float> *pSynPro, //profile for synthesized image
					 Matrix<float> *pOrgCls, //clusters of orignal image 
					 Matrix<float> *pSynCls)//clusters of orignal image 
{
	int wd, ht;//orginal wd ht
	pOrg->GetDimension(ht, wd);

	int w, h;//syn wd, ht
	pSyn->GetDimension(h, w);
	if (param.m_nSynWd > 0 && param.m_nSynHt > 0)
	{
		if (w != param.m_nSynWd || h != param.m_nSynHt)
		{
			pSyn->SetDimension(ht, wd);
			pSyn->Zero();
			w = param.m_nSynWd;
			h = param.m_nSynHt;
		}
	}
	Matrix<float> orgCon, orgPro, orgCls;//original constraints, profile, clusters
	Matrix<float> synCon, synPro, synCls;// synthesized
	if (pOrgCon == 0)
	{
		orgCon.SetDimension(ht, wd);
		orgCon.Set(1);//0 - is valid for constraint
	}
	else
		orgCon = *pOrgCon;

	if (pSynCon == 0)
	{
		synCon.SetDimension(h, w);
		synCon.Set(1);//0 - is valid for constraint
	}
	else
		synCon = *pSynCon;

	if (pOrgPro == 0)
	{
		orgPro = *pOrg;
	}
	else
		orgPro = *pOrgPro;

	if (pSynPro == 0)
	{
		synPro = *pSyn;
	}
	else
		synPro = *pSynPro;

	if (pOrgCls == 0)
	{
		orgCls.SetDimension(ht, wd);
		orgCls.Set(0);//only one cluster
	}
	else
		orgCls = *pOrgCls;

	if (pSynCls == 0)
	{
		synCls.SetDimension(h, w);
		synCls.Set(0);//only one cluster
	}
	else
		synCls = *pSynCls;
	
	skJuleszSampling(*pOrg, *pSyn, param, dImgMean, dImgStd, pchDir, pchProjName, orgCon, synCon, orgPro, synPro, orgCls, synCls);

	return 0;
}
///////////////////
// skJuleszSampling()
int skJuleszSampling(Matrix<float> &org, //original image
					 Matrix<float> &syn, //synthesized
					 SkJuleszParam &param, //parameters
					 double dImgMean, double dImgStd,//image mean and std
					 const char* pchDir, //work directory
					 const char* pchProjName, //project name
					 Matrix<float> &orgCon, //constraints for orignal image
					 Matrix<float> &synCon, //constraints for synthesized image
					 Matrix<float> &orgPro, //profile for orignal image
					 Matrix<float> &synPro, //profile for synthesized image
					 Matrix<float> &orgCls, //clusters of orignal image 
					 Matrix<float> &synCls)//clusters of orignal image 
{
	char msg[512], name[512];

	SkFilterCoef orgFC;
	// construct FRAME filters and compute the filter responses
	SkFilterBank filterBank, *pFB = &filterBank;
//	if (pFB->readParameters(param.m_pchFilterParamName) != 0)
		pFB->m_param.initilaize4FameModel();
	pFB->saveParameters(param.m_pchFilterParamName);//no such parameter file, save the default


	if (pFB->m_param.m_nUseDeltaFunc != 1)
	{
		sprintf(msg, "\nFor Julesz ensemble sampling, Delta Func must be used. Change the %s parameter setting.", param.m_pchFilterParamName);
		skOutput(msg);
		return 1;
	}

	pFB->buildFilters();
	char pchType[32];
	sprintf(pchType, "jlsz");
	pFB->saveFiltersAsBmp(pchType);

	char pchProjNameNew[512];
	sprintf(pchProjNameNew, "%s_jlsz", pchProjName);

	orgFC.setFilterBank(pFB);
	orgFC.setMatrixImage(&org); 
	orgFC.setProjName(pchProjNameNew);
	orgFC.initializeCoefficencies();

	skSaveFilterResponses(orgFC, 8, pchProjName);

	CAdaptHis **ppOrgHis, **ppSynHis;
	int nObsClusterNum = (int)(orgCls.GetMax() + 1 + 0.5);
	int nSynClusterNum = (int)(synCls.GetMax() + 1 + 0.5);
	assert(nSynClusterNum <= nObsClusterNum);
	int nClusterNum = nSynClusterNum;
	int nFilterNum = pFB->getTotalFilterNum();

	int i, j;
	ppOrgHis = new CAdaptHis*[nObsClusterNum];
	for (i = 0; i < nObsClusterNum; i++)
		ppOrgHis[i] = new CAdaptHis[nFilterNum];

	ppSynHis = new CAdaptHis*[nSynClusterNum];
	for (i = 0; i < nSynClusterNum; i++)
		ppSynHis[i] = new CAdaptHis[nFilterNum];

	int nMaxChosenFilter = MIN(param.m_nFilterNum, nFilterNum);
	assert(nFilterNum*nClusterNum <= 128*128);
	int pnOrder[128*128];

	int nBuildHistFromSample = 1;
	skComputeFilterResponseHistogram(&orgFC, &orgCon, &orgCls, ppOrgHis, nBuildHistFromSample, param, nClusterNum, nFilterNum);

	for (i = 0; i < nObsClusterNum; i++)
		skComputeOrder4FilterSelection(ppOrgHis[i], pnOrder+nFilterNum*i, nFilterNum, param.m_nFirstFixedFilterNum);
	skSaveFilters4EachCluster(org, orgCls, pnOrder, nClusterNum, nFilterNum, pFB, pchProjName, dImgMean, dImgStd);


	// only compare valid/selected filters
	for (i = 0; i < nMaxChosenFilter; i++)
	{
		for (j = 0; j < nClusterNum; j++)
		{
			ppOrgHis[j][pnOrder[nFilterNum*j+i]].setValid(true);
		}
	}
	for (i = nMaxChosenFilter; i < nFilterNum; i++)
	{
		for (j = 0; j < nClusterNum; j++)
		{
			ppOrgHis[j][pnOrder[nFilterNum*j+i]].setValid(false);
		}
	}

	int nMaxFilterHalf = -1, nHalf;
	for (i = 0; i < nClusterNum; i++)
	{
		for (j = 0; j < nFilterNum; j++)
		{
			if (!ppOrgHis[i][j].valid())
				continue;
			nHalf = pFB->getHalf(j);
			if (nMaxFilterHalf < nHalf)
				nMaxFilterHalf = nHalf;
		}
	}
	sprintf(msg, "\n Max Filter Half = %d\n", nMaxFilterHalf);
	::skOutput(msg);


	//initialize syn with respect to org intensity histogram
//	skInitializeSynImageWRTOrgIntensityHist(syn, &synCon, &synCls, ppOrgHis, nClusterNum, nFilterNum);

	// compute histograms
	int index = 0;
	
	SkFilterCoef synFC;

	synFC.setFilterBank(pFB);
	synFC.setMatrixImage(&syn);
	synFC.initializeCoefficencies();
//	SaveResults::saveFilterResponses(synFC, 9, pchProjName);

	skCountFilterResponseHistogram(&synFC, ppOrgHis, ppSynHis, &synCon, &synCls, nClusterNum, nFilterNum);

	char pType[32];
	sprintf(pType, "org");
	skSaveFilterResponseHistograms(ppOrgHis, nClusterNum, nFilterNum, pchProjName, pchDir, pType, index);
	sprintf(pType, "syn");
	skSaveFilterResponseHistograms(ppSynHis, nClusterNum, nFilterNum, pchProjName, pchDir, pType, index);
//	double pdFilterWeight[4096];
//	assert(nClusterNum*nFilterNum <= 4096);
//	decideFilterWeight(pdFilterWeight,pHisObs, pHisSyn, nClusterNum, nFilterNum);

	int nNumSampling = skSampleJulesEnsemble(ppOrgHis, ppSynHis, &synFC, &syn, &synCon, &synCls, pchProjName, pchDir, dImgMean, dImgStd, 
		nMaxChosenFilter, param, nClusterNum, nFilterNum, nMaxFilterHalf);

	for (i = 0; i < nObsClusterNum; i++)
		delete[] ppOrgHis[i];
	delete[] ppOrgHis;
	for (i = 0; i < nSynClusterNum; i++)
		delete[] ppSynHis[i];
	delete[] ppSynHis;


	return nNumSampling;
}


///////////////////
// skComputeFilterResponseHistogram()
int skComputeFilterResponseHistogram(
				SkFilterCoef *pFC, //filter coef
				Matrix<float> *pCon, //constraint matrix 0-fixed
				Matrix<float> *pCls, //cluster matrix 0~nClusterNum-1
				CAdaptHis **ppHis,
				int nBuildHistFromSample,//build the histogram from the sample
				SkJuleszParam &param, //parameters
				int nClusterNum,//region cluster num
				int nFilterNum)//total filternum
{
	int wd = pCon->cols();
	int ht = pCon->rows();

	int i, j, r, c;
	int nPixNum;
	int m = 0;
	double dMinStep = 0.0001;
	int nMaxBinNum;
	double dLowerLimit=-3.0, dUpperLimit=3.0;
	double dFR;//filter response
	for (i = 0; i < nClusterNum; i++)
	{
		nPixNum = skNumOfValueInMatrix(*pCls, i);
		for (j = 0; j < nFilterNum; j++)
		{
			switch (j)
			{
			case 0:
//				dMinStep = 256.0/param.m_nImgGrayLevels;
				dMinStep = 1.0;
				nMaxBinNum = 256;//param.m_nImgGrayLevels;
				break;
			case 1:
			case 2:
				dMinStep = 0.1;
				nMaxBinNum = param.m_nFirDerMaxBinNum;
				break;
			case 3:
			case 4:
				dMinStep = 0.1;
				nMaxBinNum = param.m_nSecDerMaxBinNum;
				break;
			case 5:
			case 6:
				dMinStep = 0.1;
				nMaxBinNum = param.m_nLogMaxBinNum;
				break;
			default:
				dMinStep = 0.1;
				nMaxBinNum = param.m_nDOOGMaxBinNum;
				break;
			}

			if (nBuildHistFromSample==1)
			{//build the histogram from the sample
				ppHis[i][j].SetDimension(1);
				ppHis[i][j].SetMaxSampleNum(nPixNum);
//				ppHis[i][j].SetZeroMustBeOneBin(true);
				ppHis[i][j].SetZeroMustBeOneBin(false);
			}
			else
			{//build the histogram by upper and lower limits
//				LocalHist::setHistogramLowerUpperLimitsAndBinNum(j, dLowerLimit,  dUpperLimit, nBinNum);
				ppHis[i][j].buildByLimitsAndBinNum(&dLowerLimit, &dUpperLimit, &nMaxBinNum);
				ppHis[i][j].setSaveSample(false);
			}
			for (r = 0; r < ht; r++)
			{
				for (c = 0; c < wd; c++)
				{
					if ((*pCon)(r,c) == 0)//constrainted point
						continue;
					if ( fabs((*pCls)(r,c)-i) > 0.5)
						continue;
					dFR = pFC->getCoef(j, r, c);
					ppHis[i][j].AddOneSample(&dFR);
				}
			}
			if (nBuildHistFromSample==1)
			{//build the histogram from the sample
				ppHis[i][j].buildFromSamplesAndMinStepAndMaxBinNum(&dMinStep, &nMaxBinNum);
			}
			else
			{//build the histogram by upper and lower limits
				ppHis[i][j].CalculateHistogram();
			}
		}
	}
	return 0;
}

///////////////////
// skComputeOrder4FilterSelection()
int skComputeOrder4FilterSelection(CAdaptHis *pHis, int *pnOrder, int nFilterNum, int nFirtFixedFilterNum)
{
	int i, j, k;

	assert(nFilterNum <= 1024);
	double pdInfo[1024];
	int n = nFilterNum - nFirtFixedFilterNum;

	int *pnOrderTmp = pnOrder;

	k = 0;
	for (j = 0; j < nFirtFixedFilterNum; j++)
		pnOrderTmp[j] = j;
	for (j = nFirtFixedFilterNum; j < nFilterNum; j++)
	{//first five automatically being used
		pdInfo[k] = skInformationGainOfOneFilterHistogram(pHis[j]);
		k++;
	}
	::sortDecreaselyWithTrackingOrder(pdInfo, n, pnOrderTmp+nFirtFixedFilterNum);
	for (j = 0; j < n; j++)
		pnOrderTmp[j+nFirtFixedFilterNum] += nFirtFixedFilterNum;

	return 0;
}
///////////////////
// skInformationGainOfOneFilterHistogram()
double skInformationGainOfOneFilterHistogram(CAdaptHis &his)
{
	double dBinStep;
	his.getBinStep(&dBinStep);
	return his.computeAbsWeightCenter()*dBinStep;
}

////////////////////////
int skInitializeSynImageWRTOrgIntensityHist(
							Matrix<float> &syn, // original image
							Matrix<float> *pSynCon, //constraint image 0- fixed
							Matrix<float> *pSynCls,//cluster image
							CAdaptHis **ppOrgHis, // observe histogram
							int nClusterNum, //cluster num
							int nFilterNum)//filter num
{
	char msg[512];

	int wd, ht;
	syn.GetDimension(ht, wd);

	int nBinNum;
	double *pdProb, dProbDrawn;
	double value;
	int x, y;
	for (y = 0; y < ht; y++)
	{
		for (x = 0; x < wd; x++)
		{
			if ((*pSynCon)(y,x) == 0)
				continue;//fixed point

			int c = (int)((*pSynCls)(y,x) + 0.5);
			assert(c < nClusterNum);

			pdProb = ppOrgHis[c][0].getHis();
			nBinNum = ppOrgHis[c][0].GetBinNum();
			int nBin = ::samplingFromProbability(pdProb, nBinNum, &dProbDrawn);
			ppOrgHis[c][0].Bin2Real(nBin, &value);
			syn(y, x) = value;
		}
	}

	return 0;

}
///////////////////
// skRediscretizeImageAndUpdateFCandHistogram()
int skRediscretizeImageAndUpdateFCandHistogram(	
							Matrix<float> *pImg, // original image
							Matrix<float> *pCon, //constraint image 0- fixed
							Matrix<float> *pCls,//cluster image
							SkFilterCoef *pFC, //synthesized filter Coefs
							CAdaptHis **ppHis, // observe histogram
							int nClusterNum, //cluster num
							int nFilterNum,//filter num
							int nMaxFilterHalf)//the max half size of valid filter
{
	char msg[512];

	int wd = pCon->cols();
	int ht = pCon->rows();

	int x, y;

	double old_value, new_value;
	int nBin;

	int c;
	for (y = 0; y < ht; y++)
	{
		for (x = 0; x < wd; x++)
		{
			if ((*pCon)(y,x) == 0)
				continue;//fixed point

			c = (int)((*pCls)(y,x) + 0.5);
			assert(c < nClusterNum);

			old_value = (*pImg)(y,x);
			nBin = ppHis[c][0].Real2Bin(&old_value);
			ppHis[c][0].Bin2Real(nBin, &new_value);
			if (new_value != old_value)
			{
				skChangeOnePoint(x, y, new_value, ppHis, pFC, pImg, pCon, pCls, nClusterNum, nFilterNum, nMaxFilterHalf);
				(*pImg)(y,x) = new_value;
			}
		}
	}

	return 0;
}
///////////////////
// sampleJulesEnsemble()
int skSampleJulesEnsemble(	CAdaptHis **ppOrgHis, // observe histogram
							CAdaptHis **ppSynHis, //synthesized histogram
							SkFilterCoef *pSynFC, //synthesized filter Coefs
							Matrix<float> *pSyn, // synthesized image
							Matrix<float> *pSynCon, //constraint image 0- fixed
							Matrix<float> *pSynCls,//cluster image
							const char *pProjName,//project name
							const char *pchDir,//directory name
							double dImgMean, double dImgStd,//image mean and std
//							int *pnOrder,//the filters which are chosen
							int nMaxChosenFilter,//max chosen filter num
							SkJuleszParam& param,//parameters
							int nClusterNum,//region cluster num
							int nFilterNum,//total filternum
							int nMaxFilterHalf)//the max half size of valid filter
{
	char pType[32];
	CAdaptHis** ppLambda = 0;
	if (param.m_nUseFrameLearnLambda)
	{
		ppLambda = new CAdaptHis*[nClusterNum];
		for (int i = 0; i < nClusterNum; i++)
		{
			ppLambda[i] = new CAdaptHis[nFilterNum];
			for (int j = 0; j < nFilterNum; j++)
			{
				if (ppOrgHis[i][j].valid() )
				{
					ppLambda[i][j] = ppOrgHis[i][j];//only copy bin settings
					ppLambda[i][j].clearSamplesAndHistogram();//set the histogram--lambdas as zero
				}
				else
				{
					ppLambda[i][j].setValid(false);
				}
			}
		}
		skUpdateFrameLambda(ppLambda, ppOrgHis, ppSynHis, nClusterNum, nFilterNum, param.m_dLambdaLearnRate);
		sprintf(pType, "lbd");
		skSaveFilterResponseHistograms(ppLambda, nClusterNum, nFilterNum, pProjName, pchDir, pType, 0);
	}


	char msg[512];
	double pdEnergy[1024];

	double dStartTemp = param.m_dStartTemp;
	double dEndTemp = param.m_dEndTemp;
	double dTemp;
	int s, nSweeps = param.m_nSweepsPerTemp;
	int t, nTemp = param.m_nAnealingSteps;//temperature num

	sprintf(msg, "\n\n Sampling Julesz ensemble begins ---------------------------------------");
	skOutput(msg);
	sprintf(msg, "\n       nSweeps = %d, nTemp = %d, startTemp = %.2f, endTemp = %.2f\n", nSweeps, nTemp, dStartTemp, dEndTemp);
	skOutput(msg);
	// compute the histogram di

	int i;

	char syn_dir[32];
	sprintf(syn_dir, "syn");
	_mkdir(syn_dir);

	char name[512], pPre[128];
	sprintf(pType, "syn");
	int n = 0;//total steps=sweeps*tempNum

	::changeNumTo3Char(pPre, n);
	sprintf(name, "%s\\%s_syn_%s.bmp", syn_dir, pProjName, pPre);
	::skSaveMatrixAsBitmap(name, *pSyn, dImgMean, dImgStd);
	sprintf(name, "%s_syn.bmp", pProjName);
	::skSaveMatrixAsBitmap(name, *pSyn, dImgMean, dImgStd);

	printf("mean = %.2f std=%.2f\n\n", dImgMean, dImgStd);

	pdEnergy[n] = skComputeHistogramDifference(nClusterNum, nFilterNum, ppOrgHis, ppSynHis);
	n++;
	bool bBreak = false;
	for (t = 0; t < nTemp; t++)
	{
		if (param.m_nUseAnealing)
			dTemp = dEndTemp + (dStartTemp-dEndTemp)*exp(-t*5.0/nTemp);
		else
			dTemp = 0;

		for (s = 0; s < nSweeps; s++)
		{
			skSampleJulesEnsemble(ppOrgHis, ppSynHis, pSynFC, pSyn, pSynCon, pSynCls, dTemp, 
				nMaxChosenFilter, nClusterNum, nFilterNum, nMaxFilterHalf, ppLambda);
		
			skCheckHist(ppSynHis, pSynFC, pSyn, pSynCon, pSynCls, nMaxChosenFilter,nClusterNum, nFilterNum);
			skCheckFilterCoef(pSynFC, pSyn, pSynCon, nMaxChosenFilter);

			//energy 
			pdEnergy[n] = skComputeHistogramDifference(nClusterNum, nFilterNum, ppOrgHis, ppSynHis);
			sprintf(name, "%s\\%s_energy.txt", syn_dir, pProjName);
			::saveArray(pdEnergy, n+1, name);
			sprintf(name, "histogram\\%s_energy.txt", pProjName);
			::saveArray(pdEnergy, n+1, name);
			//progress
			sprintf(msg, "\n  time=%3d  sweep=%3d temp=%4.2f e=%10.4f e2=%7.2f\n", t, s, dTemp, pdEnergy[n]); 
			skOutput(msg);
			// syn image
			::changeNumTo3Char(pPre, n);
			sprintf(name, "%s\\%s_syn_%s.bmp", syn_dir, pProjName, pPre);
			::skSaveMatrixAsBitmap(name, *pSyn, dImgMean, dImgStd);
			sprintf(name, "%s_syn.bmp", pProjName);
			::skSaveMatrixAsBitmap(name, *pSyn, dImgMean, dImgStd);

			Matrix<float> syn_only = *pSyn;
			int ht, wd;
			pSyn->GetDimension(ht, wd);
			int r, c;
			for (r = 0; r < ht; r++)
			{
				for (c = 0; c < wd; c++)
				{
					if ((*pSynCon)(r,c) == 0)
						syn_only(r,c) = param.m_nImgGrayLevels;
				}
			}
			sprintf(name, "%s\\%s_syn_only_%s.bmp", syn_dir, pProjName, pPre);
			skSaveMatrixAsBitmap(name, syn_only, dImgMean, dImgStd);
			sprintf(name, "%s_syn_only.bmp", pProjName);
			skSaveMatrixAsBitmap(name, syn_only, dImgMean, dImgStd);

		
			n++;
//			::TakeABreath();

			if ( n > 1 )
			{
				if ( fabs(pdEnergy[n-1]-pdEnergy[n-2]) < 0.00001 * fabs(pdEnergy[n-2])
					|| n >= 30 )
				{
					bBreak = true;
					break;
				}
			}
		}
		//syn histogram
		sprintf(pType, "syn");
		skSaveFilterResponseHistograms(ppSynHis, nClusterNum, nFilterNum, pProjName, pchDir, pType, t+1);
		// frame lambda
		if (param.m_nUseFrameLearnLambda)
		{
			skUpdateFrameLambda(ppLambda, ppOrgHis, ppSynHis, nClusterNum, nFilterNum, param.m_dLambdaLearnRate);
			sprintf(pType, "lbd");
			skSaveFilterResponseHistograms(ppLambda, nClusterNum, nFilterNum, pProjName, pchDir, pType, t+1);
		}

		if (bBreak)
			break;
	}

//	sprintf(name, "%s_syn2.bmp", pProjName);
//	skSaveMatrixAsBitmap(name, *pSyn, dImgMean, dImgStd);

//	sprintf(name, "%s_syn2_only.bmp", pProjName);
	Matrix<float> syn_only = *pSyn;
	int ht, wd;
	pSyn->GetDimension(ht, wd);
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if ((*pSynCon)(r,c) == 0)
				syn_only(r,c) = param.m_nImgGrayLevels;
		}
	}
//	skSaveMatrixAsBitmap(name, syn_only, dImgMean, dImgStd);


	if (ppLambda != 0)
	{
		for (int i = 0; i < nClusterNum; i++)
		{
			delete[] ppLambda[i];
		}
		delete[] ppLambda;
	}

	return n-1;

}
///////////////////
// skSampleJulesEnsemble()
int skSampleJulesEnsemble(	CAdaptHis **ppOrgHis, // observe histogram
							CAdaptHis **ppSynHis, //synthesized histogram
							SkFilterCoef *pSynFC, //synthesized filter Coefs
							Matrix<float> *pSyn, // synthesized image
							Matrix<float> *pCon, //constraint image 0- fixed
							Matrix<float> *pCls,//cluster image
							//int *pnOrder,//the filters which are chosen
							double dTemp,//temprature
							int nMaxChosenFilter,//max chosen filter num
							int nClusterNum,//region cluster num
							int nFilterNum,//total filternum
							int nMaxFilterHalf,//the max half size of valid filter
							CAdaptHis **ppLambda)//parameters lambda of the frame model, if 0, use Julesz model
{
	char msg[512];

	int wd = pCon->cols();
	int ht = pCon->rows();

	double pdEnergy[512];//histogram difference 
	double pdInten[512];//intensity
	double pdProb[512];

	//assert(nBinNum <= 512);

	int x, y;
	int b;
	int nIntenBinNum;//intensity bin num
	int c;
	double value, pdProbDrawn;
	double sigma_2 = 1.0/(ht*wd);//0.0005;

	Matrix<int> M;
	M.SetDimension(ht, wd);
	M.Zero();
	int n = ht*wd;
	for (y = 0; y < ht; y++)
	{
		for (x = 0; x < wd; x++)
		{
			if ((*pCon)(y,x) == 0)
			{
				M(y,x) = 1;//used
				n--;
			}
		}
	}

	DWORD time0, time1;
	time0 = ::GetTickCount();

//	time_t time0, time1;
//	time(&time0);



	while (true)
	{//random sample x, y
		int nRand = (int)(rand()/(RAND_MAX+1.0)*n);
		int nn = 0;
		bool bFind = false;
		for (y = 0; y < ht; y++)
		{
			for (x = 0; x < wd; x++)
			{
				if (M(y,x) != 1)
				{//not used
					if (nn == nRand)
					{
						bFind = true;
						break;
					}
					nn++;
				}
			}
			if (bFind) break;
		}
		assert(bFind);
		
//		x = 118;
//		y = 58;

		n--;
		M(y,x) = 1;
		
		
//		assert((*pCon)(y,x) != 0);

		if ((*pCon)(y,x) == 0)
			continue;//fixed point
		c = (int)((*pCls)(y,x) + 0.5);
		assert(c < nClusterNum);

		double old_value = (*pSyn)(y,x);
		int old_bin = ppOrgHis[c][0].Real2Bin(&old_value);

		int nSampleBinNum = 13;
		int nTrueBinNum = ppOrgHis[c][0].GetBinNum();//reserved pHisObs[0] as intensity
		int nOutlierBin = 2;
		int nStartBin = old_bin - nSampleBinNum/2;
		int nEndBin = old_bin + nSampleBinNum/2;
		if (nStartBin < nOutlierBin/2)
		{
			nStartBin = nOutlierBin/2;
			nEndBin = nStartBin + nTrueBinNum - 1; 
		}
		if (nEndBin > nTrueBinNum-1 - nOutlierBin/2)
		{
			nEndBin = nTrueBinNum-1 - nOutlierBin/2;
		}

		nSampleBinNum = nEndBin - nStartBin + 1;
		int bb, b = 0;
		for (bb = nStartBin; bb <= nEndBin; bb++)//two bins for outliers
		{
//			ppOrgHis[c][0].Bin2RealRandom(bb, &value);
			ppOrgHis[c][0].Bin2Real(bb, &value);
			pdInten[b] = value;
			b++;
		}//for b - each bin of intensity
		skNewEnergyAfterChangingOnePoint(x, y, pdInten, pdEnergy, nSampleBinNum, ppOrgHis, ppSynHis, 
			pSynFC, pSyn, pCon, pCls, nMaxChosenFilter, nClusterNum, nFilterNum, nMaxFilterHalf, ppLambda);
		//consider old value
		int old_b = nSampleBinNum;
		pdInten[old_b] = old_value;

		int nUseOldInten = 0;

		double dUsedTemp;
		if (ppLambda == 0)
		{
			if (nUseOldInten)
				pdEnergy[old_b] = skComputeHistogramDifference(nClusterNum, nFilterNum, ppOrgHis, ppSynHis);
			double dMagicTemp = skMagicTempratureByWhichBiggestProbIsAsTwiceAsSecondBiggest(pdEnergy, nSampleBinNum+nUseOldInten);
			dUsedTemp = dTemp*dMagicTemp;
		}
		else
		{
			if (nUseOldInten)
				pdEnergy[old_b] = skComputeFrameEnergy(nClusterNum, nFilterNum, ppSynHis, ppLambda);
			dUsedTemp = 1.0;
		}
		if (dTemp > 0.000001)
		{
			::changeEnergy2Probability(pdEnergy, pdProb, nSampleBinNum+nUseOldInten, dUsedTemp );
			b = ::samplingFromProbability(pdProb, nSampleBinNum+nUseOldInten, &pdProbDrawn);
		}
		else
		{
			b = ::indexOfMin(pdEnergy, nSampleBinNum+nUseOldInten);
			pdProbDrawn = 1.0;
		}

		value = pdInten[b];
		if (value != (*pSyn)(y,x))
		{
			skChangeOnePoint(x, y, value, ppSynHis, pSynFC, pSyn, pCon, pCls, nClusterNum, nFilterNum, nMaxFilterHalf);
			(*pSyn)(y,x) = value;
//			skCheckFilterCoef(pSynFC, pSyn, pCon, nMaxChosenFilter);
//			skCheckHist(ppSynHis, pSynFC, pSyn, pCon, pCls, nMaxChosenFilter,nClusterNum, nFilterNum);
			//only for debug
//			double dDiff = skComputeHistogramDifference(nClusterNum, nFilterNum, ppOrgHis, ppSynHis);
//			assert(fabs(dDiff - pdHisDiff[b]) < 0.0000001);
		}

//		if (n == 31000)
//			break;
		if (n/1000 * 1000 == n)  
		{
			time1 = ::GetTickCount();
			sprintf(msg, "n=%d  time=%d ms  prob draw = %.5f bin %d -> %d\n",n, time1-time0, pdProbDrawn, old_bin, b+nStartBin);
			skOutput(msg);
			time0 = time1;
		}
/*		if	(x==118 && y == 518) 
		{
			time1 = ::GetTickCount();
			sprintf(msg, "**** n=%d  (%d,%d)  time=%d ms  prob draw = %.5f bin %d -> %d\n",n, x, y, time1-time0, pdProbDrawn, old_bin, b+nStartBin);
			skOutput(msg);
			sprintf(msg, "");
			skOutput(msg);
		}
*/		if (n == 0)
			break;

//		::TakeABreath();
	}

	return 0;
}

///////////////////
// histogramDifferenceAfterAddingOneBase()
double skNewEnergyAfterChangingOnePoint(	int x0, int y0,//the position of the point
											double* pdValue,//new values
											double* pdEnergy,//
											int nValueNum,//value num
											CAdaptHis **ppOrgHis, // observe histogram
											CAdaptHis **ppSynHis, //synthesized histogram
											SkFilterCoef *pSynFC, //synthesized filter Coefs
											Matrix<float> *pSyn, // synthesized image
											Matrix<float> *pCon, //constraint image 0- fixed
											Matrix<float> *pCls,//cluster image
											int nMaxChosenFilter,//max chosen filter num
											int nClusterNum,//region cluster num
											int nFilterNum,//total filternum
											int nMaxFilterHalf,//the max half size of valid filter
											CAdaptHis **ppLambda)//parameters lambda of the frame model, if 0, use Julesz model
{
//	skCheckHist(ppSynHis, pSynFC, pSyn, pCon, pCls, nMaxChosenFilter,nClusterNum, nFilterNum);

	assert(nValueNum > 0);

	int wd = pCon->cols();
	int ht = pCon->rows();

	double old_value = (*pSyn)(y0,x0);

	double new_value;

	if (nValueNum == 1)
	{
		new_value = pdValue[0];
		if (new_value == old_value)
		{
			pdEnergy[0] = skComputeHistogramDifference(nClusterNum, nFilterNum, ppOrgHis, ppSynHis);
			return pdEnergy[0];
		}
	}

	//use temporary histogram
	CAdaptHis ***pppHisSynTmp = new CAdaptHis**[nValueNum];
	int i,j,k;
	for (k = 0; k < nValueNum; k++)
	{
		pppHisSynTmp[k] =  new CAdaptHis*[nClusterNum];
		for (i = 0; i < nClusterNum; i++)
		{
			pppHisSynTmp[k][i] = new CAdaptHis[nFilterNum];
		}
	}

	for (k = 0; k < nValueNum; k++)
	{
		for (i = 0; i < nClusterNum; i++)
		{
			for (j = 0; j < nFilterNum; j++)
			{
				if (ppOrgHis[i][j].valid())
					pppHisSynTmp[k][i][j] = ppSynHis[i][j];//all the parameters, cell counts and histogram are copied, not samples
				else
					pppHisSynTmp[k][i][j].setValid(false);
			}
		}
	}


	SkFilterBank *pFB = pSynFC->m_pFilterBank;
	int h, size;//half and full size of filter
	int dx, dy;//relative to (x0, y0)
	int x, y;
	double old_rsp, new_rsp;//old, new filter response
	int old_bin, new_bin;//old, new bin no in the histograms
	cv::Ptr<cv::Mat> f;//pointer to filter values
	double f_value;

	int nMaxHalf = nMaxFilterHalf;

	int nPos;
	for (dy = -nMaxHalf; dy <= nMaxHalf; dy++)
	{
		y = y0 + dy;
		if (y < 0 || y >= ht)
			continue;
		for (dx = -nMaxHalf; dx <= nMaxHalf; dx++)
		{
			x = x0 + dx;
			if (x < 0 || x >= wd)
				continue;
			if ((*pCon)(y, x) == 0)
				continue; 
			i = (int)((*pCls)(y,x) + 0.5);
			assert(i < nClusterNum);

			for (j = 0; j < nFilterNum; j++)
			{
				if(!ppSynHis[i][j].valid())
					continue;
				h = pFB->m_pFilters[j].getHalf();//half of filter size
				if (h < abs(dx) || h < abs(dy))
					continue;//the filter not cover this pixel (x0,y0)
				size = 2*h + 1;
				f = pFB->m_pFilters[j].m_pIplKer;
				nPos = (h-dy)*size+(h-dx);
//				assert(nPos >= 0 && nPos < size*size);
				f_value = f->at<float>(h-dy, h-dx); //[nPos];
				if ( f_value == 0)
					continue;

				old_rsp = pSynFC->getCoef(j, y, x);
				old_bin = ppSynHis[i][j].Real2Bin(&old_rsp);

				for (k = 0; k < nValueNum; k++)
				{
					new_rsp = (float)(old_rsp + (pdValue[k]-old_value)*f_value);
					new_bin = pppHisSynTmp[k][i][j].Real2Bin(&new_rsp);

					if (new_bin != old_bin)
					{//change the histogram
						pppHisSynTmp[k][i][j].changePointOfCell(old_bin, REMOVE);
						pppHisSynTmp[k][i][j].changePointOfCell(new_bin, ADD);
					}
				}
			}
		}
	}

	for (k = 0; k < nValueNum; k++)
	{
		for (i = 0; i < nClusterNum; i++)
		{
			for (j = 0; j < nFilterNum; j++)
			{
				if (ppSynHis[i][j].valid())
					pppHisSynTmp[k][i][j].CalculateHistogram();
			}
		}
	}
	if (ppLambda == 0)
	{//no lambda hense not Frame model but julesz model, compute the histogram difference as energy
		for (k = 0; k < nValueNum; k++)
		{
			pdEnergy[k] = skComputeHistogramDifference(nClusterNum, nFilterNum, ppOrgHis, pppHisSynTmp[k]);
		}
	}
	else
	{//frame model, energy = \sum{lambda*Histogram}
		for (k = 0; k < nValueNum; k++)
		{
			pdEnergy[k] = skComputeFrameEnergy(nClusterNum, nFilterNum, pppHisSynTmp[k], ppLambda);
		}
	}

	for (k = 0; k < nValueNum; k++)
	{
		for (i = 0; i < nClusterNum; i++)
		{
			delete[] pppHisSynTmp[k][i];
		}
		delete[] pppHisSynTmp[k];
	}
	delete[] pppHisSynTmp;

//	skCheckHist(ppSynHis, pSynFC, pSyn, pCon, pCls, nMaxChosenFilter,nClusterNum, nFilterNum);
	return 0;
}

///////////////////
// histogramDifferenceAfterAddingOneBase()
double skCheckHist(	CAdaptHis **ppSynHis, //synthesized histogram
					SkFilterCoef *pSynFC, //synthesized filter Coefs
					Matrix<float> *pSyn, // synthesized image
					Matrix<float> *pCon, //constraint image 0- fixed
					Matrix<float> *pCls,//cluster image
					int nMaxChosenFilter,//max chosen filter num
					int nClusterNum,//region cluster num
					int nFilterNum)//total filternum
{
	return 0;

	int wd = pCon->cols();
	int ht = pCon->rows();

	//use temporary histogram
	int i, j;
	CAdaptHis **ppSynHisTmp =  new CAdaptHis*[nClusterNum];
	for (i = 0; i < nClusterNum; i++)
	{
		ppSynHisTmp[i] = new CAdaptHis[nFilterNum];
	}

	skCountFilterResponseHistogram(
				pSynFC, //filter coef
				ppSynHis,//observed histogram
				ppSynHisTmp,//observed histogram
				pCon, //constraint matrix 0-fixed
				pCls, //cluster matrix 0~nClusterNum-1
				nClusterNum,//region cluster num
				nFilterNum);//total filternum

	for (i = 0; i < nClusterNum; i++)
	{
		for (j = 0; j < nFilterNum; j++)
		{
			if (ppSynHis[i][j].valid())
			{
				assert(ppSynHisTmp[i][j] == ppSynHis[i][j]);
			}
		}
	}

	for (i = 0; i < nClusterNum; i++)
	{
		delete[] ppSynHisTmp[i];
	}
	delete[] ppSynHisTmp;

	return 0;
}
bool skCheckFilterCoef(SkFilterCoef *pFC, 
					   Matrix<float>* pImg, 
					   Matrix<float>* pCon,
					   int nFilterNum)
{
	return true;

	SkFilterBank *pFB = pFC->getFilterBank();

	double dError = 0.00001;
	SkFilterCoef fc;
	fc.setFilterBank(pFB);
	fc.setMatrixImage(pImg);
	fc.initializeCoefficencies();

	int n = pFB->getTotalFilterNum();
	n = MIN(nFilterNum, n);

	int wd, ht;
	pImg->GetDimension(ht, wd);
	int r, c, k;
	float f0, f1;
	int nBond = 7;
	for (r = nBond; r < ht-nBond; r++)
	{
		for (c = nBond; c < wd-nBond; c++)
		{
			if ((*pCon)(r, c) == 0)
				continue;
			for (k = 0; k < n; k++)
			{
				f0 = pFC->getCoef(k, r, c);
				f1 = fc.getCoef(k, r, c);
				if (fabs(f0-f1) > dError)
//					c += 0;
//					return false;
					assert(false);
			}
		}
	}
	return true;
}
///////////////////
// changeOnePoint()
int skChangeOnePoint(	int x0, int y0,//the position of the point
						double new_value,//new value
						CAdaptHis **ppSynHis, //synthesized histogram
						SkFilterCoef *pSynFC, //synthesized filter Coefs
						Matrix<float> *pSyn, // synthesized image
						Matrix<float> *pCon, //constraint image 0- fixed
						Matrix<float> *pCls,//cluster image
						int nClusterNum,//region cluster num
						int nFilterNum,//total filternum
						int nMaxFilterHalf)//the max half size of valid filter
{

	if( x0 == 54 && y0 == 29)
		x0 += 0;

	int nHisNum = nClusterNum*nFilterNum;

	int wd = pCon->cols();
	int ht = pCon->rows();

	double old_value = (*pSyn)(y0,x0);

	if (new_value == old_value)
	{
		return 0;
	}

	int c, m; 
	int k;//filter index
	int i,j;
	int h, size;//half and full size of filter
	int dx, dy;//relative to (x0, y0)
	int x, y;
	double old_rsp, new_rsp;//old, new filter response
	int old_bin, new_bin;//old, new bin no in the histograms
	float f_value;//pointer to filter values
	cv::Ptr<cv::Mat> f;

	SkFilterBank *pFB = pSynFC->m_pFilterBank;
	int nMaxHalf = nMaxFilterHalf;
	int nPos;
	for (dy = -nMaxHalf; dy <= nMaxHalf; dy++)
	{
		y = y0 + dy;
		if (y < 0 || y >= ht)
			continue;
		for (dx = -nMaxHalf; dx <= nMaxHalf; dx++)
		{
			x = x0 + dx;
			if (x < 0 || x >= wd)
				continue;
			if ((*pCon)(y, x) == 0)
				continue;
			c = (int)((*pCls)(y,x) + 0.5);
			assert(c < nClusterNum);

			for (j = 0; j < nFilterNum; j++)
			{
				if (!ppSynHis[c][j].valid())
					continue;
//				k = pnOrder[c*nFilterNum + i];
				k = j;
				h = pFB->m_pFilters[k].getHalf();//half of filter size
				if (h < abs(dx) || h < abs(dy))
					continue;//the filter not cover this pixel (x0,y0)
				size = 2*h + 1;
				f = pFB->m_pFilters[k].m_pIplKer;
				nPos = (h-dy)*size+(h-dx);
//				assert(nPos >= 0 && nPos < size*size);
				f_value = f->at<float>(h-dy, h-dx); //[nPos];
				if (f_value == 0)
					continue;

				old_rsp = pSynFC->getCoef(k, y, x);
				old_bin = ppSynHis[c][j].Real2Bin(&old_rsp);

				new_rsp = old_rsp + (new_value-old_value)*f_value;
				pSynFC->setCoef(k, y, x, new_rsp);
				new_rsp = pSynFC->getCoef(k, y, x);
				new_bin = ppSynHis[c][j].Real2Bin(&new_rsp);

				if (new_bin != old_bin)
				{//change the histogram
					ppSynHis[c][j].changePointOfCell(old_bin, REMOVE);
					ppSynHis[c][j].changePointOfCell(new_bin, ADD);
				}
			}//for i
		}//for dx
	}//for dy

	for (c = 0; c < nClusterNum; c++)
	{
		for (i = 0; i < nFilterNum; i++)
		{
			if (ppSynHis[c][i].valid())
				ppSynHis[c][i].CalculateHistogram();
		}
	}
	return 0;
}

///////////////////
// skCountFilterResponseHistogram()
//	the bins have been built from the ppOrgHis, only count the histogram
int skCountFilterResponseHistogram(
				SkFilterCoef *pFC, //filter coef
				CAdaptHis **ppOrgHis,//observed histogram
				CAdaptHis **ppSynHis,//observed histogram
				Matrix<float> *pCon, //constraint matrix 0-fixed
				Matrix<float> *pCls, //cluster matrix 0~nClusterNum-1
				int nClusterNum,//region cluster num
				int nFilterNum)//total filternum
{
	int wd = pCon->cols();
	int ht = pCon->rows();

	int i, j, r, c;
	int nPixNum;
	double dFR;//filter response
	for (i = 0; i < nClusterNum; i++)
	{
		nPixNum = skNumOfValueInMatrix(*pCls, i);
		for (j = 0; j < nFilterNum; j++)
		{
			if (ppOrgHis[i][j].valid())
			{
				ppSynHis[i][j] = ppOrgHis[i][j];
				ppSynHis[i][j].clearSamplesAndHistogram();

				for (r = 0; r < ht; r++)
				{
					for (c = 0; c < wd; c++)
					{
						if ((*pCon)(r,c) == 0)//constrainted point
							continue;
						if ( fabs((*pCls)(r,c)-i) > 0.5)
							continue;
						dFR = pFC->getCoef(j, r, c);
						ppSynHis[i][j].changePointOfCell(&dFR, ADD);
					}
				}
				ppSynHis[i][j].CalculateHistogram();
			}
			else
				ppSynHis[i][j].setValid(false);
		}
	}
	return 0;
}
///////////////////
// skComputeHistogramDifference()
double skComputeHistogramDifference(int nClusterNum, int nFilterNum, CAdaptHis **ppOrgHis, CAdaptHis **ppSynHis)
{
	double pdHisDiff[128*128];
	skComputeHistogramDifference(nClusterNum, nFilterNum, pdHisDiff, ppOrgHis, ppSynHis);
	double dHisDiff = ::sum(pdHisDiff, nClusterNum*nFilterNum);
	return dHisDiff;
}

///////////////////
// skComputeHistogramDifference()
int skComputeHistogramDifference(int nClusterNum, int nFilterNum, double *pdHisDiff, CAdaptHis **ppOrgHis, CAdaptHis **ppSynHis)
{
	int c, k;
	int m = 0;
	for (c = 0; c < nClusterNum; c++)
	{
		for (k = 0; k < nFilterNum; k++)
		{
			if (ppOrgHis[c][k].valid())
			{
				pdHisDiff[m] = ppOrgHis[c][k].histogramDifference(&(ppSynHis[c][k]));
			}
			else
			{
				pdHisDiff[m] = 0;
			}
			m++;
		}
	}
	return m;
}
///////////////////
// skComputeFrameEnergy()
double skComputeFrameEnergy(int nClusterNum, int nFilterNum, CAdaptHis **ppHis, CAdaptHis **ppLambda)
{
	double pdEnergy[128*128];
	skComputeFrameEnergy(nClusterNum, nFilterNum, pdEnergy, ppHis, ppLambda);
	double dEnergy = ::sum(pdEnergy, nClusterNum*nFilterNum);
	return dEnergy;
}

///////////////////
// skComputeFrameEnergy()
int skComputeFrameEnergy(int nClusterNum, int nFilterNum, double *pdEnergy, CAdaptHis **ppHis, CAdaptHis **ppLambda)
{
	int c, k;
	int m = 0;
	for (c = 0; c < nClusterNum; c++)
	{
		for (k = 0; k < nFilterNum; k++)
		{
			if (ppHis[c][k].valid())
			{
				pdEnergy[m] = ppHis[c][k].histogramProduct(&(ppLambda[c][k]));
			}
			else
			{
				pdEnergy[m] = 0;
			}
			m++;
		}
	}
	return m;
}

///////////////////
// skUpdateFrameLambda()
int skUpdateFrameLambda(CAdaptHis** ppLambda,//frame parameter lambda
						CAdaptHis** ppOrgHis,//observed histogram
						CAdaptHis** ppSynHis,//synthesized histogram
						int nClusterNum,//cluster num
						int nFilterNum,//filter num
						double dLearnRate)//learn rate
{
	double *pLambda, *pOrg, *pSyn;
	double mean;
	int c, k, b, bin_num;
	for (c = 0; c < nClusterNum; c++)
	{
		for (k = 0; k < nFilterNum; k++)
		{
			if (ppOrgHis[c][k].valid())
			{
				pLambda = ppLambda[c][k].getHis();
				pOrg = ppOrgHis[c][k].getHis();
				pSyn = ppSynHis[c][k].getHis();

				bin_num = ppOrgHis[c][k].GetBinNum();
				for (b = 0; b < bin_num; b++)
				{
					pLambda[b] += (pSyn[b] - pOrg[b]) * dLearnRate;
				}
				::changeArray2ZeroMean(pLambda, bin_num);
			}
		}
	}
	return 0;
}

///////////////////
// skSaveFilterResponseHistograms()
int skSaveFilterResponseHistograms(CAdaptHis** ppHis, int nClusterNum, int nFilterNum, const char *pProjName, const char *pchDir, const char *pType, int index)
{
	char dir[512], name[512];
	sprintf(dir, "histogram");
	_mkdir(dir);

	char cn[32], fn[32], id[32];
	::changeNumTo3Char(id, index);
	int i, j;
	// save histogram
	for (i = 0; i < nClusterNum; i++)
	{
		::changeNumTo2Char(cn, i);
		for (j = 0; j < nFilterNum; j++)
		{
			if (ppHis[i][j].valid())
			{
				::changeNumTo2Char(fn, j);
				sprintf(name, "%s\\%s_his_%s_c%s_f%s_%s.txt", dir, pProjName, pType, cn, fn, id);
				ppHis[i][j].saveHistogram(name);
			}
		}
	}
	return 0;
}

///////////////////
// skSaveFilters4EachCluster()
int skSaveFilters4EachCluster(	Matrix<float> &img, //matrix of orginal image
								 Matrix<float> &cluster, //matrix of clusters
								 int *pnOrder, //the filter order for each cluster 
								 int nClusterNum, int nFilterNum, //cluster and filter num
								 SkFilterBank *pFB,//the filter bank
								 const char *pProjName, //the project name
								 int dImgMean, int dImgStd)//image mean and std to scale the image back
{
	char dir[512], name[512];
	sprintf(dir, "cluster");
	_mkdir(dir);


	Matrix<float> tmp;
	int cc;
	for (cc = 0; cc < nClusterNum; cc++)
	{
		{//just seperate two parts of the code
		tmp = img;
		int w, h;
		img.GetDimension(h, w);

		for (int r = 0; r < h; r++)
		{
			for (int c = 0; c < w; c++)
			{
				if (fabs(cluster(r,c)-cc) > 0.001)
					tmp(r,c) = 255;
			}
		}
		if (cc < 10)
			sprintf(name, "%s\\%s_cls_0%d_cluster.bmp", dir, pProjName, cc);
		else
			sprintf(name, "%s\\%s_cls_%d_cluster.bmp", dir, pProjName, cc);
		::skSaveMatrixAsBitmap(name, tmp, dImgMean, dImgStd);
		}

		{//below code just copy from oval_FilterBank
		// save filters in one image
		if (cc < 10)
			sprintf(name, "%s\\%s_cls_0%d_filter.bmp", dir, pProjName, cc);
		else
			sprintf(name, "%s\\%s_cls_%d_filter.bmp", dir, pProjName, cc);
		int n = nFilterNum;
		int m = (int)sqrt(double(n));//rows and colums
		if (n != m*m)
			m = m+1;
		int m2 = n/m;
		if (m2 * m != n)
			m2 += 1;
		int i;
		int max_h = -1, h;
		for (i = 0; i < n; i++)
		{//find max half
			h = pFB->m_pFilters[i].m_nHalf;
			if (max_h < h)
				max_h = h;
		}
		int size = 2*(max_h+1) + 1;
		int space = 3;

		int wd = m * (size+space) + space;
		int ht = m2 * (size+space) + space;;
		
		Matrix<float> M(ht, wd);
		M.Set(0);
		int r, c, x0, y0;
		int syn = KER;
		for (i = 0; i < n; i++)
		{
			r = i/m;
			c = i%m;
			x0 = c * (size+space) + size/2 + space;
			y0 = r * (size+space) + size/2 + space;
			pFB->copyFilter2Matrix(pnOrder[cc*nFilterNum+i], M, x0, y0, syn);
		}
		float max_f = M.GetMax();
		// draw seperate lines
		int x1, y1, x2, y2;
		for (i = 0; i <= m2; i++)
		{
			x1 = 0; 
			x2 = wd;
			y1 = i * (size+space);
			y2 = y1 + space;
			for (r = y1; r < y2; r++)
				for (c = x1; c < x2; c++)
					M(r,c) = max_f;
		}
		for (i = 0; i <= m; i++)
		{
			y1 = 0; 
			y2 = ht;
			x1 = i * (size+space);
			x2 = x1 + space;
			for (r = y1; r < y2; r++)
				for (c = x1; c < x2; c++)
					M(r,c) = max_f;
		}
		// last several not used square set to blank
		int nBlank = m*m2 - n;
		x2 = wd;
		x1 = x2 - nBlank*(size+space);
		y2 = ht;
		y1 = y2 - (size+space);
		for (r = y1; r < y2; r++)
			for (c = x1; c < x2; c++)
				M(r,c) = max_f;

		::skSaveMatrixAsBitmap(name, M, 0, -1);
		}
	}
	return 0;
}







