#ifndef _SK_JULESZ_H_
#define _SK_JULESZ_H_

#include "skMatrixIplOp.h"
#include "skFilterCoef.h"
#include "skAdaptHis.h"

class SkJuleszParam
{
// variables
public:
	int m_nUseFrameLearnLambda;//use the frame model and learn the lambda
	double m_dLambdaLearnRate;//the learn rate for the lambda
	int m_nSynWd;
	int m_nSynHt;//synthesized image width and height
	int m_nUseFilterSelection;//select first m_nFilterNum filters for each region
	int m_nFilterNum;//filter num used in modeling
	int m_nFirstFixedFilterNum;//first filters: intensity, dx, dy, ddx, ddy, DOG 1, DOG 2, 
	int m_nUseRegionClustering;//automatically cluster regions 
	int m_nMaxRegionNum;//maximus region number
	int m_nBoundCond;//boundary conditions
	int m_nUseAnealing;//use anealing or not, if not greedy 
	double m_dStartTemp;//if anealing, start temperature
	double m_dEndTemp;// and end temperature
	int m_nAnealingSteps;//anealing steps which reducing the temperature
	int m_nSweepsPerTemp;//sampling sweeps per each temperature
	char m_pchFilterParamName[512];
	int m_nImgGrayLevels;//the gray levels of input image
	int m_nFirDerMaxBinNum;//the max bin num for first derivative
	int m_nSecDerMaxBinNum;//the max bin num for second derivative
	int m_nLogMaxBinNum;//the max bin num for Log
	int m_nDOOGMaxBinNum;//the max bin num for DOOG filters
// operations:
public:
	SkJuleszParam();
	~SkJuleszParam();
	int read(const char *name);
	int save(const char *name);
};


int skJuleszSampling(const char *pchImgName);
int skJuleszSamplingSketch(const char *pchImgName);//only for sketch

int	skComputeInitSynByInterpolation(Matrix<float> &org,
									Matrix<float> *pSynCon, 
									Matrix<float> *pSynPro, 
									Matrix<float> &syn);
int	skComputeInitSynByRandom(int nLevels, 
							Matrix<float> &syn);
int	skComputeInitSynByClusterAverage(Matrix<float> &org,
									Matrix<float> &con,
									Matrix<float> &pro,
									Matrix<float> &cls, 
									Matrix<float> &syn);
int skJuleszSampling(Matrix<float> *pOrg, //original image
					 Matrix<float> *pSyn, //synthesized
					 SkJuleszParam &param, //parameters
					 double dImgMean, double dImgStd,//image mean and std
					 const char* pchDir, //work directory
					 const char* pchProjName, //project name
					 Matrix<float> *pOrgCon, //constraints for orignal image
					 Matrix<float> *pSynCon, //constraints for synthesized image
					 Matrix<float> *pOrgPro, //profile for orignal image
					 Matrix<float> *pSynPro, //profile for synthesized image
					 Matrix<float> *pOrgCls, //clusters of orignal image 
					 Matrix<float> *pSynCls);//clusters of orignal image 

int skJuleszSampling(Matrix<float> &org, //original image
					 Matrix<float> &syn, //synthesized
					 SkJuleszParam &param, //parameters
					 double dImgMean, double dImgStd,//image mean and std
					 const char* pchDir, //work directory
					 const char* pchProjName, //project name
					 Matrix<float> &orgCon, //constraints for orignal image
					 Matrix<float> &synCon, //constraints for synthesized image
					 Matrix<float> &orgPro, //profile for orignal image
					 Matrix<float> &synPro, //profile for synthesized image
					 Matrix<float> &orgCls, //clusters of orignal image 
					 Matrix<float> &synCls);//clusters of orignal image 
int skComputeFilterResponseHistogram(
				SkFilterCoef *pFC, //filter coef
				Matrix<float> *pCon, //constraint matrix 0-fixed
				Matrix<float> *pCls, //cluster matrix 0~nClusterNum-1
				CAdaptHis **ppHis,//detination hist
				int nBuildHistFromSample,//build the histogram from the sample
				SkJuleszParam &param,
				int nClusterNum,//region cluster num
				int nFilterNum);//total filternum

int skComputeOrder4FilterSelection(CAdaptHis *pHis, int *pnOrder, int nFilterNum, int nFirtFixedFilterNum);
double skInformationGainOfOneFilterHistogram(CAdaptHis &his);

int skSampleJulesEnsemble(	CAdaptHis **ppOrgHis, // observe histogram
							CAdaptHis **ppSynHis, //synthesized histogram
							SkFilterCoef *pSynFC, //synthesized filter Coefs
							Matrix<float> *pSyn, // synthesized image
							Matrix<float> *pSynCon, //constraint image 0- fixed
							Matrix<float> *pSynCls,//cluster image
							const char *pProjName,//project name
							const char *pchDir,//directory name
							double dImgMean, double dImgStd,//image mean and std
							int nMaxChosenFilter,//max chosen filter num
							SkJuleszParam& param,//parameters
							int nClusterNum,//region cluster num
							int nFilterNum,//total filternum
							int nMaxFilterHalf);//the max half size of valid filter
int skSampleJulesEnsemble(	CAdaptHis **ppOrgHis, // observe histogram
							CAdaptHis **ppSynHis, //synthesized histogram
							SkFilterCoef *pSynFC, //synthesized filter Coefs
							Matrix<float> *pSyn, // synthesized image
							Matrix<float> *pCon, //constraint image 0- fixed
							Matrix<float> *pCls,//cluster image
							double dTemp,//temprature
							int nMaxChosenFilter,//max chosen filter num
							int nClusterNum,//region cluster num
							int nFilterNum,//total filternum
							int nMaxFilterHalf,//the max half size of valid filter
							CAdaptHis **ppLambda=0);//parameters lambda of the frame model, if 0, use Julesz model
double skNewEnergyAfterChangingOnePoint(	int x0, int y0,//the position of the point
													double* pdValue,//new values
													double* pdEnergy,//Energy
													int nValueNum,//value num
													CAdaptHis **ppOrgHis, // observe histogram
													CAdaptHis **ppSynHis, //synthesized histogram
													SkFilterCoef *pSynFC, //synthesized filter Coefs
													Matrix<float> *pSyn, // synthesized image
													Matrix<float> *pCon, //constraint image 0- fixed
													Matrix<float> *pCls,//cluster image
													int nMaxChosenFilter,//max chosen filter num
													int nClusterNum,//region cluster num
													int nFilterNum,//total filternum
													int nMaxFilterHalf,//the max half size of valid filter
													CAdaptHis **ppLambda=0);//parameters lambda of the frame model, if 0, use Julesz model
int skChangeOnePoint(	int x0, int y0,//the position of the point
						double new_value,//new value
						CAdaptHis **ppSynHis, //synthesized histogram
						SkFilterCoef *pSynFC, //synthesized filter Coefs
						Matrix<float> *pSyn, // synthesized image
						Matrix<float> *pCon, //constraint image 0- fixed
						Matrix<float> *pCls,//cluster image
						int nClusterNum,//region cluster num
						int nFilterNum,//total filternum
						int nMaxFilterHalf);//the max half size of valid filter
int skCountFilterResponseHistogram(
				SkFilterCoef *pFC, //filter coef
				CAdaptHis **pObsHis,//observed histogram
				CAdaptHis **pSynHis,//observed histogram
				Matrix<float> *pCon, //constraint matrix 0-fixed
				Matrix<float> *pCls, //cluster matrix 0~nClusterNum-1
				int nClusterNum,//region cluster num
				int nFilterNum);//total filternum
int skRediscretizeImageAndUpdateFCandHistogram(	
							Matrix<float> *pImg, // original image
							Matrix<float> *pCon, //constraint image 0- fixed
							Matrix<float> *pCls,//cluster image
							SkFilterCoef *pFC, //synthesized filter Coefs
							CAdaptHis **ppHis, // observe histogram
							int nClusterNum, //cluster num
							int nFilterNum,//filter num
							int nMaxFilterHalf);//the max half size of valid filter

double skComputeHistogramDifference(int nClusterNum, int nFilterNum, CAdaptHis **ppOrgHis, CAdaptHis **ppSynHis);
int skComputeHistogramDifference(int nClusterNum, int nFilterNum, double *pdHisDiff, CAdaptHis **ppOrgHis, CAdaptHis **pSynHis);
double skComputeFrameEnergy(int nClusterNum, int nFilterNum, CAdaptHis **ppHis, CAdaptHis **ppLambda);
int skComputeFrameEnergy(int nClusterNum, int nFilterNum, double *pdEnergy, CAdaptHis **ppHis, CAdaptHis **ppLambda);
int skUpdateFrameLambda(CAdaptHis** ppLambda,//frame parameter lambda
						CAdaptHis** ppOrgHis,//observed histogram
						CAdaptHis** ppSynHis,//synthesized histogram
						int nClusterNum,//cluster num
						int nFilterNum,//filter num
						double dLearnRate);//learn rate

int skSaveFilterResponseHistograms(CAdaptHis** ppHis, int nClusterNum, int nFilterNum, const char *pProjName, const char *pchDir, const char *pType, int index);
int skSaveFilters4EachCluster(	Matrix<float> &img, //matrix of orginal image
								 Matrix<float> &cluster, //matrix of clusters
								 int *pnOrder, //the filter order for each cluster 
								 int nClusterNum, int nFilterNum, //cluster and filter num
								 SkFilterBank *pFB,//the filter bank
								 const char *pProjName, //the project name
								 int dImgMean, int dImgStd);//image mean and std to scale the image back

double skCheckHist(	CAdaptHis **ppSynHis, //synthesized histogram
					SkFilterCoef *pSynFC, //synthesized filter Coefs
					Matrix<float> *pSyn, // synthesized image
					Matrix<float> *pCon, //constraint image 0- fixed
					Matrix<float> *pCls,//cluster image
					int nMaxChosenFilter,//max chosen filter num
					int nClusterNum,//region cluster num
					int nFilterNum);//total filternum
bool skCheckFilterCoef(SkFilterCoef *pFC, 
					   Matrix<float>* pImg, 
					   Matrix<float>* pCon,
					   int nFilterNum);

int skJuleszSamplingWithThreeModels(const char *pchImgName); // image file name
int skInterpolationWithProfiles(const char *pchImgName);
int skComputeResidaulAfterDiffusionAndMatchingPursuit(const char *pchImgName); // image file name
int skJuleszSamplingOnResidualOfInterpolationAndMatchingPursuit(const char *pchImgName); // image file name
int skCombinSynResults(const char *pchImgName);

int skInitializeSynImageWRTOrgIntensityHist(
							Matrix<float> &syn, // original image
							Matrix<float> *pSynCon, //constraint image 0- fixed
							Matrix<float> *pSynCls,//cluster image
							CAdaptHis **ppOrgHis, // observe histogram
							int nClusterNum, //cluster num
							int nFilterNum);//filter num

#endif