#include "skLocalHist.h"
#include "skClustering.h"

///////////////////
// constructor
SkLocalHist::SkLocalHist()
{
	m_pHisMat = 0;
	m_nHalfLocalSize = 7;//the half local size on which the histograms apply
	m_nSub = 3;//sub sample image, to save the space, we don't compute the hist for each pixel, only for sub-sample ones
}

///////////////////
// constructor
SkLocalHist::~SkLocalHist()
{
	if (m_pHisMat != 0)
		delete []m_pHisMat;
}


int SkLocalHist::setHistogramLowerUpperLimitsAndBinNum(int k, double &dLowerLimit, double &dUpperLimit, int &nBinNum)
{
	switch(k)
	{
	case 0://delta
		dLowerLimit = -1.5;
		dUpperLimit = 1.5;
		nBinNum = 7;
		break;
	case 1:
	case 2://first derivative of X and Y
		dLowerLimit = -2;
		dUpperLimit = 2;
		nBinNum = 7;
		break;
	case 3:
	case 4://LoG scale 1 and 2
	default:
		dLowerLimit = -3;
		dUpperLimit = 3;
		nBinNum = 7;
		break;
	}
	return 0;
}


///////////////////
// buildLocalHistogram()
int SkLocalHist::build(SkFilterCoef *pFC,//the FilterCoefs which the SkLocalHist is builded on
					 Matrix<float> *pValid)//if pValid != 0, only build on those pixels (*pValid)(y,x)=0
											// if pValid ==0, build for all the pixels
{
	int nBinNum;
	double dLowerLimit;
	double dUpperLimit;

	int nHalf = m_nHalfLocalSize;

	setFilterCoef(pFC);
	int n = pFC->m_pFilterBank->getTotalFilterNum();
	m_nFilterNum = n;
	assert(n > 0);
	if (m_pHisMat != 0)
		delete []m_pHisMat;
	m_pHisMat = new Matrix<CAdaptHis>[n];

	int wd = pFC->getWidth();//width of orignal image
	int ht = pFC->getHeight();//height of orignal image

	int w = (wd+m_nSub-1) / m_nSub;//width of histogram image
	int h = (ht+m_nSub-1) / m_nSub;//height of histogram image

	int k;
	int r, c, dr, dc, r0, c0, r1, c1;
	int i, j;
	double coef;
	for (k = 0; k < n; k++)
	{
		m_pHisMat[k].SetDimension(h, w);
		setHistogramLowerUpperLimitsAndBinNum(k, dLowerLimit, dUpperLimit, nBinNum);
		for (r = 0; r < h; r++)
		{
			r0 = r*m_nSub + m_nSub/2;
			for (c = 0; c < w; c++)
			{
				c0 = c*m_nSub + m_nSub/2;
				// consider skip this point or not
				bool bSkip = true;
				for (i = -m_nSub/2; i <= m_nSub/2; i++)
				{
					if (r0+i < 0 || r0+i >= ht)
						continue;
					for (j = -m_nSub/2; j <= m_nSub/2; j++)
					{
						if (c0+j < 0 || c0+j >= wd)
							continue;
						if (pValid != 0)
							if (fabs((*pValid)(r0+i,c0+j)) <= 0.001)
								bSkip = false;
					}
				}
				if (bSkip)
				{
					m_pHisMat[k](r,c).setValid(false);
					continue;
				}

				m_pHisMat[k](r,c).buildByLimitsAndBinNum(&dLowerLimit, &dUpperLimit, &nBinNum);
				m_pHisMat[k](r,c).setSaveSample(false);
				for (dr = -nHalf; dr <= nHalf; dr++)
				{
					r1 = r0 + dr;
					if (r1 < 0 || r1 >= ht) 
						continue;//not count
					for (dc = -nHalf; dc < nHalf; dc++)
					{
						c1 = c0 + dc;
						if (c1 < 0 || c1 >= wd)
							continue;
//						if (fabs((*pValid)(r0+i,c0+j)) > 0.001)//not considered
//							continue;
						if (fabs((*pValid)(r1,c1)) > 0.001)//not considered
							continue;

						coef = pFC->getCoef(k, r1, c1);
						m_pHisMat[k](r,c).AddOneSample(&coef);
					}
				}
				m_pHisMat[k](r,c).CalculateHistogram();
			}
		}
	}
	return 0;
}


/////////////////////////////////////////////////////////////////////
// SkTextureCluster
///////////////////
// clustering()
//	return the cluster num
int SkTextureCluster::clustering(SkLocalHist *pLocalHis, //the local histogram matrix
								Matrix<float> &cluster,//the clusters for each pixel, 0-background, not clustered
								int nMaxClusterNum)//max cluster num
{
	int nFilterNum = pLocalHis->getFilterNum();//filter num
	assert(nFilterNum > 0);
	int wd = pLocalHis->getHistImgWidth();//histogram image width
	int ht = pLocalHis->getHistImgHeight();//histogram image height

	int k = 0;//filter index, the first one, should be delta funtion, 05-16-2003

	int r, c;//row colummn index
	int i;//general index 
	int d;//index of dimension
	int n;//index of data
	int j;//index of cluster

	// find the first valid histogram, use it to compute all the information of original data
	int nNum = 0;//data num - valid point with histogram
	int nDim = 0;//data dimension
	int *pR = new int[wd*ht];
	int *pC = new int[wd*ht];//save the valid rows and cols
	bool bFindValidHist = false;
	int nFilter4Feature = MIN(5, nFilterNum);
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (pLocalHis->m_pHisMat[k](r,c).valid())
			{
				pR[nNum] = r;
				pC[nNum] = c;
				nNum++;
				if (!bFindValidHist)
				{//just find the first valid histogram and find ou the data dimension
					for (k = 0; k < nFilter4Feature; k++)
						nDim += pLocalHis->m_pHisMat[k](r,c).GetBinNum();
				}
				bFindValidHist = true;
			}
			k = 0;
		}
	}


	int nClusterNum;//final cluster num
	int nMinClusterSize = 8;
//	int nMaxClusterNum = ht*wd/(nMinClusterSize*40);//max cluster num

//	nMaxClusterNum = MIN(nMaxClusterNum, 5);

	double *pdData = new double[nNum*nDim];//save the original data in 1-dim
	double *pdWeight = new double[nDim];//weight for the data 
	double *pdCC = new double[nMaxClusterNum*nDim];//the cluster centers
	int	   *pnCluster = new int[nNum];//the cluster index for each data
	memset(pnCluster, -1, sizeof(int)*nNum);

	double *p = pdData;//dynamic pointer to data memory
	double *pHist;//pointer to the histogram
	int nBinNum;
	for (n = 0; n < nNum; n++)
	{
		r = pR[n];
		c = pC[n];
		for (k = 0; k < nFilter4Feature; k++)
		{
			pHist = pLocalHis->m_pHisMat[k](r,c).getHis();
			nBinNum = pLocalHis->m_pHisMat[k](r,c).GetBinNum();
			memcpy(p, pHist, sizeof(double)*nBinNum);
			p += nBinNum;
		}
	}
	for (d = 0; d < nDim; d++)
		pdWeight[d] = 1.0;

	double dMinClusterDisRate = 0.0;//the higher the less clusters
	nClusterNum = skRobustCluster(pdData, nDim, nNum, pdWeight, pdCC, nMaxClusterNum, nMinClusterSize, dMinClusterDisRate, pnCluster);

	// set the clusters
	cluster.SetDimension(ht, wd);
	cluster.Set(0);
	for (n = 0; n < nNum; n++)
	{
		r = pR[n];
		c = pC[n];
		j = pnCluster[n];
		cluster(r, c) = j+1;
	}

	//release memory
	delete[] pnCluster;
	delete[] pdCC;
	delete[] pdWeight;
	delete[] pdData;
	delete[] pR;
	delete[] pC;

	return 0;
}
