#include "skMap.h"
#include "inlinefunc.h"

// the property used to sort the outnbr
// indexType: 0-distance, 1-direction, 2-BiBaseProb, 3-BiBaseCluster, 4-nbrID
int SkOutNbr::m_nKey = 0;
// class SkOutNbr: token out nbr
// constructor
SkOutNbr::SkOutNbr()
{
	m_nNbrID = -1;
	m_nNbrBranchID = -1;
	m_nSelfBranchID = -1;
	m_dCenterDis = 0;
	m_dCenterDir = 0;
	m_bValid = false;
	m_bChecked =  false;
	m_nCluster = -1;//the cluster to which this bi-base outnbr belongs
	m_dProb = 0;//the probabilty of this bi-base outnbr
	m_nRelationType = -1;
}

// deconstructor
SkOutNbr::~SkOutNbr()
{
}

// equal
bool SkOutNbr::operator == (SkOutNbr &outNbr)
{
	if (m_nNbrID == outNbr.getNbrID()
		&& m_nNbrBranchID == outNbr.getNbrBranchID()
		&& m_nSelfBranchID == outNbr.getSelfBranchID() )
		return true;
	else
		return false;
}

// greater equal than
bool SkOutNbr::operator >= (SkOutNbr &outNbr)
{
	switch (m_nKey)
	{
		// the property used to sort the outnbr
		// indexType: 0-distance, 1-direction, 2-BiBaseProb, 3-BiBaseCluster, 4-nbrID
	case 0://dis
	default:
		if (m_dCenterDis >= outNbr.getCenterDis())
			return true;
		else
			return false;
		break;
	case 1://dir
		if (m_dCenterDir >= outNbr.getCenterDir())
			return true;
		else
			return false;
		break;
	case 2://prob
		if (m_dProb >= outNbr.getProb())
			return true;
		else
			return false;
		break;
	case 3://cluster
		if (m_nCluster >= outNbr.getCluster())
			return true;
		else
			return false;
		break;
	case 4://nbrID
		if (m_nNbrID >= outNbr.getNbrID())
			return true;
		else
			return false;
		break;
	}
}

// less equal than
bool SkOutNbr::operator <= (SkOutNbr &outNbr)
{
	switch (m_nKey)
	{
		// the property used to sort the outnbr
		// indexType: 0-distance, 1-direction, 2-BiBaseProb, 3-BiBaseCluster, 4-nbrID
	case 0://dis
	default:
		if (m_dCenterDis <= outNbr.getCenterDis())
			return true;
		else
			return false;
		break;
	case 1://dir
		if (m_dCenterDir <= outNbr.getCenterDir())
			return true;
		else
			return false;
		break;
	case 2://prob
		if (m_dProb <= outNbr.getProb())
			return true;
		else
			return false;
		break;
	case 3://cluster
		if (m_nCluster <= outNbr.getCluster())
			return true;
		else
			return false;
		break;
	case 4://nbrID
		if (m_nNbrID <= outNbr.getNbrID())
			return true;
		else
			return false;
		break;
	}
}



/////////////////////////////////////////////////////////////////////
// class SkNodeNbrs: all nbrs of a token
int SkNodeNbrs::m_pnMaxNbrNum[] = { MAX_NBR_NUM, MAX_PRT_NUM, MAX_CHD_NUM, MAX_ADR_NUM};//50;//40;//20; Feb. 27, 2001
// constructor
SkNodeNbrs::SkNodeNbrs()
{
	memset(m_pnDrctNbrArray, -1, sizeof(int)*MAX_NBR_NUM);

	// set adr nbr num and checked status
	int nbrType = ADR_NBR;
	int n = getMaxNbrNum(nbrType);
	setNbrNum(nbrType, n);
}
// deconstructor
SkNodeNbrs::~SkNodeNbrs()
{
}

bool SkNodeNbrs::operator == (SkNodeNbrs &nbrs)
{
	if (!(m_inNbrArray == (*(nbrs.getPointer2InNbrArray())) ) )
		return false;
	int i;
	for (i = 0; i < MAX_NBR_TYPE; i++)
		if (!(m_pNbrArray[i] == (*(nbrs.getPointer2NbrArray(i))) ))
			return false;
	
	for (i = 0; i < MAX_NBR_TYPE; i++)
		if (m_pnMaxNbrNum[i] != nbrs.getMaxNbrNum(i))
			return false;

	int *pn = nbrs.getDrctNbrArrayPointer();
	for (i = 0; i < m_pnMaxNbrNum[0]; i++)
		if (m_pnDrctNbrArray[i] != pn[i])
			return false;
	return true;
}

///////////////////
// addNbr() - nbrType: 0-OUT_NBR, 1-PRT_NBR, 2-CHD_NBR
// add one nbr
// return: the No in the nbr array after adding, -1 if not being added
int	SkNodeNbrs::addNbr(SkOutNbr &outNbr, int nbrType, int n)
{
	outNbr.setValid(true);
	if (n >=0 )
	{
		m_pNbrArray[nbrType].setAt(n, outNbr);//insert without extension
		return n;
	}

	int nNbrNum = getNbrNum(nbrType);
	int i=0;
	for ( i = nNbrNum-1; i >= 0; i--)
	{// find the position to insert first
		if (outNbr >= *(getNbrPointerAt(i,nbrType)) )
			break;
	}
	if (nNbrNum < m_pnMaxNbrNum[nbrType])
	{// not get enough out nbr yet
		m_pNbrArray[nbrType].AddAt(i+1, outNbr);// insert with extension
		return i+1;
	}
	else
	{// already get enough out nbrs, remove the last one if needed
		if (i < nNbrNum-1)
		{// smaller than the last one
			m_pNbrArray[nbrType].RemoveAt(nNbrNum-1);//remove the last one first
			m_pNbrArray[nbrType].AddAt(i+1, outNbr);// insert with extension
			return i+1;
		}
		else
			return -1;
	}
}

////////////////////
// setNbrAt() - nbrType: 0-OUT_NBR, 1-PRT_NBR, 2-CHD_NBR, 3-ADR_NBR
// set one nbr of type "nbrType" at "i" 
//	if the array size is less than i, then set to max nbr size
// return: 1 success, -1 else
int	SkNodeNbrs::setNbrAt(int i, SkOutNbr &outNbr, int nbrType)
{
	int nNbrNum = getNbrNum(nbrType);
	if (nNbrNum <= i)
		setNbrNum(nbrType, getMaxNbrNum(nbrType));

	m_pNbrArray[nbrType].setAt(i, outNbr);
	return 1;
}

////////////////////
// add one in nbr
// return in nbr num after adding
int	SkNodeNbrs::addInNbr(int nNbrID)
{
	m_inNbrArray.Add(nNbrID);
	return 0;//m_inNbrArray.GetSize();//To save time, just return 0; The size is useless. Feb. 19, 2001
}

int	SkNodeNbrs::hasDrctNbr(int nNbrID)
{
	int nDrctNum = m_pnMaxNbrNum[OUT_NBR];//CNbrManager::getNbrDrctNum();
	int nPos, nTrueNbrID;
	for (int i = 0; i < nDrctNum; i++)
	{
		nPos = m_pnDrctNbrArray[i];
		if (nPos < 0)
			continue;
		nTrueNbrID = m_pNbrArray[OUT_NBR].GetAt(nPos).getNbrID();
		if (nTrueNbrID == nNbrID)
			return i;
	}
	return -1;
}

///////////////////
// sort the nbrs
// nbrType: 0-outnbr, 1-prtnbr, 2-chdnbr, 3-inNbr;  
// indexType: 0-distance, 1-direction, 2-BiBaseProb, 3-BiBaseCluster, 4-nbrID
int SkNodeNbrs::sortNbr(int nbrType, int indexType)
{
	if (nbrType == 3)//inNbr
	{
		int *pInNbr= m_inNbrArray.GetPointerAt(0);;
		int n = m_inNbrArray.GetSize();
		::qksort(pInNbr, 0, n-1);
		return n;//return directly
	}

	SkOutNbr::setKey(indexType);

	SkOutNbrArray *pNbrArray = &(m_pNbrArray[nbrType]);
	SkOutNbr *pNbr = pNbrArray->GetPointerAt(0);
	int n = pNbrArray->GetSize();

	qksort(pNbr, 0, n-1);

	if (indexType == 2)//sort bi-base prob by decreasing
		pNbrArray->Reverse();

	SkOutNbr::resetKey();//set as default
	return 0;
}


///////////////////
// nextAdrNbrNeed2Pursue()
//	find the next addressed nbr need to pursue
int SkNodeNbrs::nextNbrNeed2Pursue(int nbrType)
{
	int n = getNbrNum(nbrType);
	bool bChecked;
	int nFind = -1;
	for (int a = 0; a < n; a++)
	{
		bChecked = getNbrChecked(a, nbrType);
		if (!bChecked)
		{
			nFind = a;
			break;
		}
	}
	return nFind;
}
///////////////////
//	getAdrNbrChecked()
//	get the nbr checked or not
bool SkNodeNbrs::getNbrChecked(int a, int nbrType)
{
	return m_pNbrArray[nbrType].getAt(a).getChecked();
}
///////////////////
//	setAdrNbrChecked()
//	set the adr nbr checked or not
void SkNodeNbrs::setNbrChecked(int a, bool c, int nbrType)
{
	m_pNbrArray[nbrType].getAt(a).setChecked(c);
}
///////////////////
//	setAdrNbrChecked()
//	set the adr nbr checked or not
void SkNodeNbrs::setNbrChecked(bool c, int nbrType)
{
	int n = getNbrNum(nbrType);
	for (int a = 0; a < n; a++)
		setNbrChecked(a, c, nbrType);
}
///////////////////
//	getNbrValid()
//	get the nbr Valid or not
bool SkNodeNbrs::getNbrValid(int i, int nbrType)
{
	return m_pNbrArray[nbrType].getAt(i).valid();
}
///////////////////
//	setNbrValid()
//	set the adr nbr Valid or not
void SkNodeNbrs::setNbrValid(bool b, int nbrType)
{
	int n = getNbrNum(nbrType);
	for (int a = 0; a < n; a++)
		setNbrValid(a, b, nbrType);
}
///////////////////
//	setNbrValid()
//	set the adr nbr Valid or not
void SkNodeNbrs::setNbrValid(int i, bool b, int nbrType)
{
	m_pNbrArray[nbrType].getAt(i).setValid(b);
}
///////////////////
// adrNbrInverse()
int SkNodeNbrs::adrNbrInverse(int a)
{
	int a_i = -1;
	switch(a)
	{
	case R_CNTN: // right continuity
		a_i = L_CNTN;
		break;
	case R_BL_COCV: // right below co-curve
		a_i = L_AB_COCV;
		break;
	case R_AB_COCV: // right above co-curve
		a_i = L_BL_COCV;
		break;
	case BL_PRPD: // below perpendicular
//		a_i = -1;
		a_i = AB_PRPD;
		break;
	case AB_PRPD: // above perpendicular
//		a_i = -1;
		a_i = BL_PRPD;
		break;
	case BL_PRLL: // below parallel
		a_i = AB_PRLL;
		break;
	case AB_PRLL: // above parallel
		a_i = BL_PRLL;
		break;
	case L_BL_COCV: // left below co-curve
		a_i = R_AB_COCV;
		break;
	case L_AB_COCV: // left above co-curve
		a_i = R_BL_COCV;
		break;
	case L_CNTN: // left continuity
		a_i = R_CNTN;
		break;
	default:
		assert(false);
		break;
	}
	return a_i;
}

///////////////////
// getValidNbrNum()
int SkNodeNbrs::getValidNbrNum(int nbrType)
{
	int n = getNbrNum(nbrType);
	int s = 0;
	for (int i = 0; i < n; i++)
	{
		if (getNbrValid(i, nbrType))
			s += 1;
	}
	return s;

}
