#ifndef _SK_MAP_H_
#define _SK_MAP_H_

#include <stdio.h>
#include "skTemplate.h"
#include "skSparseArray.h"
#include "skFilter.h"
#include "inlinefunc.h"
#include "skFilter.h"



#include "skBase.h"

#ifndef SkOutNbrArray
#define SkOutNbrArray oval_Array<SkOutNbr>
#endif

#ifndef ADD
#define ADD 0
#endif

#ifndef REMOVE 
#define REMOVE 1
#endif

#ifndef MAX_NBR_NUM
#define MAX_NBR_NUM 16
#endif

#ifndef MAX_PRT_NUM
#define MAX_PRT_NUM 1
#endif

#ifndef MAX_CHD_NUM
#define MAX_CHD_NUM 32
#endif

#ifndef MAX_ADR_NUM
#define MAX_ADR_NUM 2//10
#endif

//nbrType 0:out, 1:prt, 2:chd
#ifndef MAX_NBR_TYPE
#define MAX_NBR_TYPE 4
#endif

#ifndef OUT_NBR
#define OUT_NBR 0
#endif

#ifndef PRT_NBR
#define PRT_NBR 1
#endif

#ifndef CHD_NBR
#define CHD_NBR 2
#endif

#ifndef ADR_NBR
#define ADR_NBR 3
#endif


#ifndef L_CNTN
#define L_CNTN 0 // left continuity
#endif

#ifndef R_CNTN
#define R_CNTN 1 // right continuity
#endif

#ifndef BL_PRPD
#define BL_PRPD 2 // below perpendicular
#endif

#ifndef AB_PRPD
#define AB_PRPD 3 // above perpendicular
#endif

#ifndef R_BL_COCV
#define R_BL_COCV 4 // right below co-curve
#endif

#ifndef L_AB_COCV
#define L_AB_COCV 5 // left above co-curve
#endif

#ifndef R_AB_COCV
#define R_AB_COCV 6 // right above co-curve
#endif

#ifndef L_BL_COCV
#define L_BL_COCV 7 // left below co-curve
#endif

#ifndef BL_PRLL
#define BL_PRLL 8 // below parallel
#endif

#ifndef AB_PRLL
#define AB_PRLL 9 // above parallel
#endif


/////////////////////////////////////////////////////////////////////
// class SkOutNbr: token out nbr
class SkOutNbr// define the out nbr of a token
{
#define SK_PRLL_ORT_TH PI/18
#define SK_PRLL_DIS_TH 2

#define SK_PRLL			0 //parallel
#define SK_EXTND_PRLL	1 //parallel after extension
#define SK_CROSS		2 //cross
#define SK_EXTND_CROSS	3 //cross after extension
#define SK_CNTN			4 //continue

private:
	int		m_nNbrID;
	int		m_nNbrBranchID;
	int		m_nSelfBranchID;
	bool	m_bValid;//valid our nbr or not
	bool	m_bChecked;//check this our nbr or not
	double	m_dCenterDis;//center distance
	double	m_dCenterDir;//center direction
	int		m_nCluster;//the cluster which this outnbr belongs to

	double	m_dProb;//the probabilty of this outnbr
	static int m_nKey;// the property used to sort the outnbr
				// indexType: 0-distance, 1-direction, 2-BiBaseProb, 3-BiBaseCluster, 4-nbrID
	// below are for branches
	int     m_nRelationType;
	double  m_dDeltaOrt;//[0,pi/2) the smallest cross angle
	double  m_dGap;//min distance between any two points
	double	m_dDeltaR;//parallel distance
public:
	SkOutNbr();

	~SkOutNbr();

	bool operator == (SkOutNbr &outNbr);
	bool operator >= (SkOutNbr &outNbr);
	bool operator <= (SkOutNbr &outNbr);
	int		getNbrID() { return m_nNbrID; }
	void	setNbrID(int nID) { m_nNbrID = nID; }
	int		getNbrBranchID() { return m_nNbrBranchID; }
	void	setNbrBranchID(int nID) { m_nNbrBranchID = nID; }
	int		getSelfBranchID() { return m_nSelfBranchID; }
	void	setSelfBranchID(int nID) { m_nSelfBranchID = nID; }
	double	getCenterDis() { return m_dCenterDis; }
	void	setCenterDis(double dDis) { m_dCenterDis = dDis; }
	double	getCenterDir() { return m_dCenterDir; }
	void	setCenterDir(double dDir) { m_dCenterDir = dDir; }
	void	setValid(bool b) { m_bValid = b; }
	bool	valid() { return m_bValid; }
	void	setChecked(bool c) { m_bChecked = c; }
	bool	getChecked() { return m_bChecked; }
	
	void	setCluster(int c) { m_nCluster = c; }
	int		getCluster() { return m_nCluster; }
	void	setProb(double p) { m_dProb = p; }
	double	getProb() { return m_dProb; }

	void setDeltaOrt(double dDeltaOrt) { m_dDeltaOrt = dDeltaOrt; }
	void setGap(double dGap) { m_dGap = dGap;}
	void setRelationType(int t) { m_nRelationType = t; }
	void setDeltaR(double dr) { m_dDeltaR = dr; }
	double getDeltaOrt() { return m_dDeltaOrt; }
	double getGap() { return m_dGap; }
	int getRelationType() { return m_nRelationType; }
	double getDeltaR() { return m_dDeltaR; }

	static void	setKey(int k) { m_nKey = k; }
	static int	getKey() { return m_nKey; }
	static void	resetKey() { m_nKey = 0; }//set key as default - 0, distance

};

/////////////////////////////////////////////////////////////////////
// class SkNodeNbrs: all nbrs of a node
class SkNodeNbrs
{

private:
public:
	oval_IntArray	m_inNbrArray;
	SkOutNbrArray	m_pNbrArray[MAX_NBR_TYPE];//0-out nbr array, 1-parent nbr array, 2-child nbr array, 3-addressed nbr
	static int		m_pnMaxNbrNum[MAX_NBR_TYPE];//0-out nbr num, 1-parent nbr num, 2-child nbr num, 3-addressed nbr num
	int				m_pnDrctNbrArray[MAX_NBR_NUM];//It generally won't be more than 12 directions. A defect left.

private:
	SkNodeNbrs(SkNodeNbrs &tn) {}//not allow implict copy

public:
	SkNodeNbrs();
	~SkNodeNbrs();
	bool	operator == (SkNodeNbrs &nbrs);
	// in nbrs
	int		addInNbr(int nNbrID);
	int		getInNbrNum() { return m_inNbrArray.GetSize(); }
	int		getInNbrIDAt(int i) { return m_inNbrArray.GetAt(i); }
	int		removeInNbr(int nNbrID) { m_inNbrArray.Remove(nNbrID); return 0; }
	bool	hasInNbr(int nNbrID) { return m_inNbrArray.HasMember(nNbrID); }
	void	removeAllInNbrs(){ m_inNbrArray.RemoveAll(); }
	oval_IntArray *getPointer2InNbrArray() { return &m_inNbrArray; }
	// drct nbrs, direction nbrs selected from out nbrs
	int		getDrctNbrIDAt(int i) { return getNbrIDAt(m_pnDrctNbrArray[i], OUT_NBR); }
	int		*getDrctNbrArrayPointer() { return m_pnDrctNbrArray; }
	int		getDrctNbrNo(int nDrctNo) { return m_pnDrctNbrArray[nDrctNo]; }
	double	getDrctNbrCenterDis(int nDrctNbrNo) { return getNbrCenterDis(m_pnDrctNbrArray[nDrctNbrNo], OUT_NBR); }
	double	getDrctNbrCenterDir(int nDrctNbrNo) { return getNbrCenterDir(m_pnDrctNbrArray[nDrctNbrNo], OUT_NBR); }
	int		hasDrctNbr(int nNbrID);
	// general nbrs, 0-out nbr, 1-prt nbr, 2-chd nbr, 3-adr nbr
	int		addNbr(SkOutNbr &outNbr, int nbrType=0, int n=0);//nbrType 0:out, 1:prt, 2:chd, default:0-out
	int		addNbrAt(SkOutNbr &outNbr, int i, int nbrType=0);//nbrType 0:out, 1:prt, 2:chd, default:0-out
	int		getNbrNum(int nbrType=0) {return m_pNbrArray[nbrType].GetSize();}
	int		getMaxNbrNum(int nbrType=0) {return m_pnMaxNbrNum[nbrType];}
	void	setNbrNum(int nbrType, int n) {m_pNbrArray[nbrType].setSize(n);}
	int		setNbrAt(int i, SkOutNbr &outNbr, int nbrType);
	static void setMaxNbrNum(int n, int nbrType=0) { m_pnMaxNbrNum[nbrType] = n; }
	SkOutNbr *getNbrPointerAt(int i, int nbrType=0) {return m_pNbrArray[nbrType].GetPointerAt(i); }
	SkOutNbr &getNbr(int i, int nbrType=0) { return m_pNbrArray[nbrType][i]; }
	int		getNbrIDAt(int i, int nbrType=0) { if (i<0||i>=getNbrNum(nbrType)||!getNbrValid(i,nbrType)) return -1; return getNbrPointerAt(i, nbrType)->getNbrID(); }
	double	getMaxNbrDis(int nbrType=0) { if (getNbrNum(nbrType)<=0) return 0; return getNbrPointerAt(getNbrNum(nbrType)-1,nbrType)->getCenterDis(); }
	double	getNbrCenterDis(int nNbrNo, int nbrType=0) { return m_pNbrArray[nbrType].GetPointerAt(nNbrNo)->getCenterDis(); }
	double	getNbrCenterDir(int nNbrNo, int nbrType=0) { return m_pNbrArray[nbrType].GetPointerAt(nNbrNo)->getCenterDir(); }
	SkOutNbrArray *getPointer2NbrArray(int nbrType=0) { return &(m_pNbrArray[nbrType]); }

	bool	hasNbr(int nNbrID, int nbrType=0) { SkOutNbr outNbr; outNbr.setNbrID(nNbrID); return m_pNbrArray[nbrType].HasMember(outNbr); }
	int		removeNbr(int nNbrID, int nbrType=0) {bool b=false; int i=0; for ( i = 0; i < m_pNbrArray[nbrType].GetSize(); i++) if (getNbrIDAt(i,nbrType) == nNbrID) { removeNbrAt(i,nbrType); b=true;break; } assert(b);return i; }
	int		removeNbrAt(int nPos, int nbrType=0) { m_pNbrArray[nbrType].RemoveAt(nPos); return 0; }

	void	removeAllNbrs() { removeAllInNbrs(); for(int i=0;i<3;i++) removeAllNbrs(i);}
	void	removeAllNbrs(int nbrType) { m_pNbrArray[nbrType].RemoveAll();}

	void	setNbrCluster(int o, int c, double dProb, int nbrType=0) { m_pNbrArray[nbrType].GetAt(o).setCluster(c); m_pNbrArray[nbrType].GetAt(o).setProb(dProb); }//m_pnOutNbrCluster[o] = c; m_pdOutNbrProb[o]=dProb; }
	int		getNbrCluster(int o, int nbrType=0) { return m_pNbrArray[nbrType].GetAt(o).getCluster(); }//m_pnOutNbrCluster[o]; }
	double	getNbrProb(int o, int nbrType=0) { return m_pNbrArray[nbrType].GetAt(o).getProb(); }//m_pdOutNbrProb[o]; }

	int		sortNbr(int nbrType, int indexType);//nbrType: 0-outnbr, 1-prtnbr, 2-chdnbr, 3-inNbr; indexType: 0-distance, 1-direction, 2-BiBaseProb, 3-BiBaseCluster, 4-nbrID

	int		nextNbrNeed2Pursue(int nbrType);
	void	setNbrChecked(bool c, int nbrType);
	void	setNbrChecked(int a, bool c, int nbrType);
	bool	getNbrChecked(int a, int nbrType);

	static int adrNbrInverse(int a);

	int		getValidNbrNum(int nbrType);
	bool	getNbrValid(int i, int nbrType);
	void	setNbrValid(int i, bool b, int nbrType);
	void	setNbrValid(bool b, int nbrType);

};

template<class TYPE>
class SkMap;


#define FREE_ELEC -1//free electron
#define NUCLEUS 0
#define BIND_ELEC 1//bind electron

/////////////////////////////////////////////////////////////////////
// class SkMapNode: the node in a map
template<class TYPE>
class SkMapNode
{
public:
	TYPE			m_data;
	SkNodeNbrs		m_nbrs;
	int				m_nID;
	bool			m_bValid;
	int				m_nType; // node type, FREE_ELECTRON -1, NUCLEUS 0, BIND_ELECTRON 1

	int				m_n1GonClusterNo;// the 1-gon cluster no of this node
	int				m_nNbrType;// generated from which gon-type, or grouped into which gon-type
	double			m_dAtomProb;// the probability of this atom, add on 12-28-2001
	int				m_nMoveFlag;// the flag for moving this node
	double			m_dAtomOrt;//the Atom orientation, which this node belongs to -PI~PI
	int				m_nAtomID;// the atom ID which the node and its children will be classified to
public:
	SkMapNode() { m_bValid=true; m_nNbrType=-1; m_nMoveFlag=0; 
						m_dAtomProb=0; m_dAtomOrt=0; m_nAtomID=-1; m_nType=FREE_ELEC; m_n1GonClusterNo=-1;}
	SkMapNode(SkMapNode<TYPE> &n) {*this = n;}

	int		read(FILE *pFile);
	int		save(FILE *pFile);

	int		getMoveFlag() { return m_nMoveFlag; }
	void	setMoveFlag(int f) { m_nMoveFlag = f; }

	int		getID() { return m_nID; }
	void	setID(int nID) { m_nID = nID; }
	void	setValid(bool b) { m_bValid = b; }
	bool	valid() { return m_bValid; }
	int		getRow() { return m_data.y; }
	int		getCol() { return m_data.x; }
	void	setRow(int row) { m_data.y=row; }
	void	setCol(int col) { m_data.x=col; }

	int		getType() { return m_nType; }
	void	setType(int t) { m_nType = t; }
	int		get1GonClusterNo() { return m_n1GonClusterNo; }
	void	set1GonClusterNo(int c) { m_n1GonClusterNo = c; }

	void	setNbrType(int t) { m_nNbrType = t; }
	int		getNbrType() { return m_nNbrType; }
	// atom property
	void	setAtomID(int t) { m_nAtomID = t; }
	int		getAtomID() { return m_nAtomID; }
	void	setAtomProb(double p) { m_dAtomProb = p; }
	double  getAtomProb() { return m_dAtomProb; }
	void	setAtomOrt(double o) { m_dAtomOrt = o; }
	double	getAtomOrt() { return m_dAtomOrt; }

	double	getRealOrt(double *pdSign=0);
	double	convertFilterOrt2CounterAtomOrt();

	//nbrType: 0-outnbr, 1-prtnbr, 2-chdnbr, 3-inNbr; indexType: 0-distance, 1-direction, 2-BiBaseProb, 3-BiBaseCluster, 4-nbrID
	int		sortNbr(int nbrType, int indexType) { return m_nbrs.sortNbr(nbrType, indexType); }

	// nbrs
	// in nbr
	int		addInNbr(int nNbrID) { return m_nbrs.addInNbr(nNbrID); }
	int		removeInNbr(int nNbrID) { return m_nbrs.removeInNbr(nNbrID); }
	void	removeAllInNbrs(){ m_nbrs.removeAllInNbrs(); }
	int		getInNbrNum() { return m_nbrs.getInNbrNum(); }
	int		getInNbrIDAt(int i) { return m_nbrs.getInNbrIDAt(i); }
	bool	hasInNbr(int nNbrID) { return m_nbrs.hasInNbr(nNbrID); }
	// drct nbr
	int		*getDrctNbrArrayPointer() { return m_nbrs.getDrctNbrArrayPointer(); }
	int		getDrctNbrNo(int nDrctNo) { return m_nbrs.getDrctNbrNo(nDrctNo); }
	int		getDrctNbrIDAt(int i) { return m_nbrs.getDrctNbrIDAt(i); }
	double	getDrctNbrCenterDis(int nOutNbrNo) { return m_nbrs.getDrctNbrCenterDis(nOutNbrNo); }
	double	getDrctNbrCenterDir(int nOutNbrNo) { return m_nbrs.getDrctNbrCenterDir(nOutNbrNo); }
	int		hasDrctNbr(int nNbrID) { return m_nbrs.hasDrctNbr(nNbrID); }

	// general nbrs, 0-outnbr, 1-prtnbr, 2-chdnbr,
	int		addNbr(SkOutNbr &outNbr,int nbrType=0, int n=-1){return m_nbrs.addNbr(outNbr,nbrType,n);}
	int		getNbrNum(int nbrType=0) { return m_nbrs.getNbrNum(nbrType); }
	int		getMaxNbrNum(int nbrType=0) { return m_nbrs.getMaxNbrNum(nbrType); }
	void	setMaxNbrNum(int n,int nbrType=0) { m_nbrs.setMaxNbrNum(n,nbrType); }
	int		getNbrIDAt(int i,int nbrType=0) { return m_nbrs.getNbrIDAt(i,nbrType); }
	double	getMaxNbrDis(int nbrType=0) { return m_nbrs.getMaxNbrDis(nbrType); }
	double	getNbrCenterDis(int nOutNbrNo,int nbrType=0) { return m_nbrs.getNbrCenterDis(nOutNbrNo,nbrType); }
	double	getNbrCenterDir(int nOutNbrNo,int nbrType=0) { return m_nbrs.getNbrCenterDir(nOutNbrNo,nbrType); }
	SkOutNbr &getNbr(int i,int nbrType=0) { return m_nbrs.getNbr(i,nbrType); }

	int		removeNbrAt(int i,int nbrType=0) {return m_nbrs.removeNbrAt(i,nbrType);}
	int		removeNbr(int nID,int nbrType=0) { return m_nbrs.removeNbr(nID,nbrType); }
	void	removeAllNbrs() { m_nbrs.removeAllNbrs(); }
	void	removeAllNbrs(int nbrType){ m_nbrs.removeAllNbrs(nbrType); }

	bool	hasNbr(int nNbrID,int nbrType=0) { return m_nbrs.hasNbr(nNbrID,nbrType); }

	void	setNbrCluster(int o, int c, double dProb,int nbrType=0) { m_nbrs.setNbrCluster(o, c, dProb,nbrType); }
	int		getNbrCluster(int o,int nbrType=0) { return m_nbrs.getNbrCluster(o,nbrType); }
	double	getNbrProb(int o,int nbrType=0) { return m_nbrs.getNbrProb(o,nbrType); }

	// May 19, 2002
	int		resetBaseIndexPosByAtomOrt(int x0, int y0);

	// Dec 17, 2002
	int		nextNbrNeed2Pursue(int nbrType) {return m_nbrs.nextNbrNeed2Pursue(nbrType); }
	bool	getNbrChecked(int a, int nbrType) {return m_nbrs.getNbrChecked(a,nbrType);}
	void	setNbrChecked(int a, bool c, int nbrType) {m_nbrs.setNbrChecked(a,c,nbrType);}
	void	setNbrChecked(bool c, int nbrType) {m_nbrs.setNbrChecked(c,nbrType);}

	bool	getNbrValid(int a, int nbrType) {return m_nbrs.getNbrValid(a,nbrType);}
	void	setNbrValid(int a, bool c, int nbrType) {m_nbrs.setNbrValid(a,c,nbrType);}
	void	setNbrValid(bool c, int nbrType) {m_nbrs.setNbrValid(c,nbrType);}

	int		getValidNbrNum(int nbrType) { return m_nbrs.getValidNbrNum(nbrType); }

};

///////////////////
// save
template<class TYPE>
int SkMapNode<TYPE>::save(FILE *pFile)
{
	m_data.save(pFile);
	return 0;
}
///////////////////
// read
template<class TYPE>
int SkMapNode<TYPE>::read(FILE *pFile)
{
	m_data.read(pFile);

	return 0;
}



///////////////////
// getRealOrt()
//	return the real ort based on the atom ort and the filter ort
//  real ort: the real ort of the base, equals atomOrt + FilterOrt. [0,PI)
//  atom ort: the ort of the atom which this basenode belongs to. [-PI,PI)
//  filter ort:  the ort of the filter. [0,PI)
//  *pdSign - 1.0: coef is not needed to change, -1.0 the coef is needed to change, because
//			for DOOG1(sin) bases, more than 180 rotation means coef*(-1)
template<class TYPE>
double SkMapNode<TYPE>::getRealOrt(double *pdSign)
{

	double dSign = 1.0;
	double dRealOrt = m_data.getOrt();//filter ort [0,PI)
	if (m_data.getType() == 2)//DOG
	{
		assert(dRealOrt == 0);
		if (pdSign != 0)
			*pdSign = 1.0;
		return dRealOrt;// the DOG Ort is not dependent on AtomOrt
	}

	//filter ort [0,PI)
	//m_dAtomOrt [-PI,PI)
	//dRealOrt: [0,PI)
	dRealOrt += m_dAtomOrt;//here [-PI,2PI)
	if (dRealOrt >= PI)
	{
		dRealOrt -= PI;//here [-PI,PI)
		if (m_data.getType() == 0)//DOOG, odd, DOOG1)
			dSign = -1.0;// DOOG, odd, may change the sign of coef
	}
	if (dRealOrt < 0)
	{
		dRealOrt += PI;//here [0,PI)
		if (m_data.getType() == DOOG)//DOOG)
			dSign = -1.0;
	}
	if (dRealOrt > PI - 5.0*PI/180)
	{//special case, to zero, when almost PI
		dRealOrt = 0;
		if (m_data.getType() == DOOG)//DOOG)
			dSign = -1.0;// DOOG may change the sign of coef
	}

	if (pdSign != 0)
		*pdSign = dSign;
	return dRealOrt;
}

///////////////////
// convertFilterOrt2CounterAtomOrt()
//	This counter-atom ort will make the base to have 0-orientation
//	dFilterOrt: [0,PI)
//	dAtomOrt: [-PI,PI)
template<class TYPE>
inline double SkMapNode<TYPE>::convertFilterOrt2CounterAtomOrt()
{
	double dAtomOrt;
	double dFilterOrt = m_data.getOrt();
	int nFilterType = m_data.getType();
	float coef = m_data.a;
	switch (nFilterType)
	{
	case DOOG:
		if (coef >= 0)
			dAtomOrt = -dFilterOrt;
		else
			dAtomOrt = PI - dFilterOrt;
		break;
	case DOOG1:
		if (dFilterOrt < PI/2.0)
			dAtomOrt = -dFilterOrt;
		else
			dAtomOrt = PI - dFilterOrt;
		break;
	case DOG:
		assert(fabs(dFilterOrt) < 0.000001);
		dAtomOrt = 0;
		break;
	}
	if (dAtomOrt == PI)
		dAtomOrt = -PI;
	return dAtomOrt;
}

//////////////////
// resetBaseIndexByAtomOrt()
template<class TYPE>
inline int SkMapNode<TYPE>::resetBaseIndexPosByAtomOrt(int x0, int y0)
{
	double dAtomOrt = getAtomOrt();
	SkFilterBank *pFilterBank = m_data.getFilterBank();
	if (dAtomOrt != 0)
	{
		double dSign;
		double ort = getRealOrt(&dSign);//ort
		double fscale = m_data.getfScale();//fscale
		int type = (int)(m_data.getType());//type
		int ind;
		if (type != DOG)
		{
			m_data.a = float(m_data.a*dSign);
			ind = pFilterBank->getFilterIndex(type, fscale, ort);
			assert(type == pFilterBank->getType(ind));
			assert(fabs(fscale-pFilterBank->getfScale(ind)) < 0.5 );
			m_data.k = ind;
		}
		double dx = m_data.x - x0;
		double dy = m_data.y - y0;
		::rotate(dx, dy, -dAtomOrt);//because the X-Y system are different
		m_data.x = (int)(x0 + dx);
		m_data.y = (int)(y0 + dy);
	}
	return m_data.k;
}


/////////////////////////////////////////////////////////////////////
// class SkMap: the template class of Map

//class CTOManager;
//#include "TOManager.h"
//template <class TYPE> class CTOManager;

template<class TYPE>
class SkMap
{
	typedef SkMapNode<TYPE> MapNode_TYPE;
private:
	int m_nWidth;
	int m_nHeight;
	Matrix<oval_IntArray> m_occupyMat;
public:
	CSparseArray<MapNode_TYPE> *m_pNodeArray;
//	CTOManager<TYPE> *m_pTOManager;

	SkMap<TYPE>* m_pParentMap;//pinter to parent map
	SkMap<TYPE>* m_pChildMap;//pointer to child map

public:
	SkMap(){ m_pNodeArray = new CSparseArray<MapNode_TYPE>(100);  
				m_nWidth = 10; m_nHeight = 10; m_pParentMap = 0; m_pChildMap = 0;} //m_pTOManager = 0;
	~SkMap() { delete m_pNodeArray; }
	SkMap<TYPE> & operator = (SkMap<TYPE> &m);

	void setParentMap(SkMap<TYPE>* pParent) { m_pParentMap = pParent; }
	SkMap<TYPE>* getParentMap() { return m_pParentMap; }
	void setChildMap(SkMap<TYPE>* pChild) { m_pChildMap = pChild; }
	SkMap<TYPE>* getChildMap() { return m_pChildMap; }

	int				getWidth() { return m_nWidth; }
	int				getHeight() { return m_nHeight; }
//	void			setWidth(int nWidth) { m_nWidth = nWidth; }
//	void			setHeight(int nHeight) { m_nHeight = nHeight; }
	void			setDimension(int nWidth, int nHeight); 

	int				getFirstPos() { return m_pNodeArray->getFirstPos(); }
	TYPE			*getNext(int& nPos) { return &(m_pNodeArray->getNext(nPos)->m_data); }
	TYPE			*getAt(int nID) { return &(m_pNodeArray->getAt(nID)->m_data); }
	MapNode_TYPE	*getNextNode(int& nPos) { return m_pNodeArray->getNext(nPos); }
	MapNode_TYPE	*getNodeAt(int nID) { assert(nID >= 0); return m_pNodeArray->getAt(nID); }
	double			getDensity() { return (double)(getSize())/(getWidth()*getHeight()); }
	int				getSize() { return m_pNodeArray->getSize(); }
	int				getValidNodeNum();
	int				getValidMaxID();
	
	void			removeAll() { if (m_pNodeArray!=0) m_pNodeArray->removeAll(); m_occupyMat.Zero();}
	int				add(TYPE data);
	int				removeNode(MapNode_TYPE *pNode);
	int				removeInvalid();
	int				addNode(MapNode_TYPE &node);
	
	int				read(const char *Name);
	int				save(const char *Name);

	void			setValid(bool b);
	void			setPointer2FilterBank(SkFilterBank* pFB);
	void			setNbrValid(SkMapNode<TYPE> *pMapNode, bool b, int nbrType=0);

	int				setElectronsAndNucleus(double dMinNucleusCoef);

	int				getBaseNumOfFilterType(int *pnNum);
	// TOManager related
//	int				getCenterOccupyNode(int nRow, int nCol, int *&pnTokenID) { return m_pTOManager->getCenterOccupyNode(nRow, nCol, pnTokenID); }
	//only for debug
	bool			checkAtomID();

	// occupy matrix
	int				updateOccupyMatrix(SkMapNode<TYPE> *pNode, int flag, double dInhibitionDis);
//	void			addOccupancy(int r, int c, int id) { m_occupyMat(r,c).add(id); }
	void			updateOccupancy(int r, int c, int id, int flag) { if (flag==ADD) m_occupyMat(r,c).add(id); else m_occupyMat(r,c).remove(id);}
	Matrix<oval_IntArray>*  getOccupyMatrix() { return &m_occupyMat; }
	oval_IntArray &	getOccupancy(int r, int c) { return m_occupyMat(r,c); }
	int				buildOccupyMatrix(double dInhibitionDis);
	int				clearOccupyMatrix();

	// filter bank
	int				setFilterBank4AllValidBases(SkFilterBank *pFB);

	int				scaleCoef(double dScale);

};

///////////////////
// add one node to node array
template<class TYPE>
int SkMap<TYPE>::add(TYPE data)
{
	SkMapNode<TYPE> node;
	node.m_data = data;
	return addNode(node);
}

///////////////////
// remove one node from node array
template<class TYPE>
int SkMap<TYPE>::addNode(SkMapNode<TYPE> &node)
{
	int nID = m_pNodeArray->add(node);
	SkMapNode<TYPE>* pNode = m_pNodeArray->getAt(nID);
	pNode->setID(nID);
//	updateOccupyMatrix(pNode, m_occupyMat, ADD);
	return nID;
}

///////////////////
// remove one node from node array
template<class TYPE>
int SkMap<TYPE>::removeNode(SkMapNode<TYPE> *pNode)
{
	int nID = pNode->getID();
	pNode->setValid(false);
	m_pNodeArray->removeAt(nID);
//	updateOccupyMatrix(pNode, m_occupyMat, REMOVE);
	return nID;
}
///////////////////
// remove one node from node array
template<class TYPE>
int SkMap<TYPE>::removeInvalid()
{
	int n = 0;
	int nPos = getFirstPos();
	SkMapNode<TYPE> *pMapNode;
	while (nPos >= 0)
	{
		pMapNode = getNextNode(nPos);
		if (!pMapNode->valid())
		{
			removeNode(pMapNode);
			n++;
		}
	}
	return n;
}
///////////////////
// remove one node from node array
template<class TYPE>
SkMap<TYPE> & SkMap<TYPE>::operator = (SkMap<TYPE> &m)
{	
	setDimension(m.getWidth(), m.getHeight());

	*m_pNodeArray =  *(m.m_pNodeArray);
	m_occupyMat = m.m_occupyMat;

	return *this;
}

///////////////////
// save the results
template<class TYPE>
int SkMap<TYPE>::save( const char *pchName)
{
	FILE *f;
	if( (f=fopen(pchName, "w"))==0 )
		return 1;
	//save image width and height
	int nWidth = getWidth();
	int nHeight = getHeight();
	fprintf(f, "ImgWidth: %d; ImgHeight: %d\n", nWidth, nHeight);
	// save token number and cluster num
	int nNodeNum = m_pNodeArray->getSize();
	fprintf(f, "NodeNum: %d\n", nNodeNum);
	SkMapNode<TYPE> *pNode;
	int nPos = getFirstPos();
	while (nPos >= 0)
	{//save both valid and invalid
		pNode = getNextNode(nPos);
		pNode->save(f);
	}
	fclose(f);
	return 0;
}

///////////////////
// read the results
template<class TYPE>
int SkMap<TYPE>::read( const char *pchName)
{
	FILE *f;
	if( (f=fopen(pchName, "r"))==0 )
		return 1;
	//save image width and height
	fscanf(f, "ImgWidth: %d; ImgHeight: %d\n", &m_nWidth, &m_nHeight);
	// save token number and cluster num
	int nNodeNum;
	fscanf(f, "NodeNum: %d\n", &nNodeNum);
	SkMapNode<TYPE> node;
	removeAll();
	for (int i = 0; i < nNodeNum; i++)
	{
		node.read(f);
		addNode(node);
	}
	fclose(f);
	return 0;
}

///////////////////
// set image dimensions
template<class TYPE>
void SkMap<TYPE>::setDimension(int nWidth, int nHeight)
{
	m_nWidth = nWidth;
	m_nHeight = nHeight;

	m_occupyMat.SetDimension(nHeight, nWidth);
	m_occupyMat.Zero();


}

///////////////////
// set valid of each node
template<class TYPE>
void SkMap<TYPE>::setValid(bool b)
{
	int nPos = getFirstPos();
	SkMapNode<TYPE> *pMapNode;
	while (nPos >= 0)
	{
		pMapNode = getNextNode(nPos);
		pMapNode->setValid(b);
	}
}

///////////////////
// set valid of each node
template<class TYPE>
int SkMap<TYPE>::scaleCoef(double dScale)
{
	int nPos = getFirstPos();
	SkMapNode<TYPE> *pMapNode;
	float coef;
	while (nPos >= 0)
	{
		pMapNode = getNextNode(nPos);
		coef = pMapNode->m_data.a;
		coef = exp(fabs(coef)) * coef / fabs(coef) / 10;
		pMapNode->m_data.a = coef;
	}
	return 0;
}

///////////////////
// set valid of each node
template<class TYPE>
void SkMap<TYPE>::setPointer2FilterBank(SkFilterBank *pFB)
{
	int nPos = getFirstPos();
	SkMapNode<TYPE> *pMapNode;
	while (nPos >= 0)
	{
		pMapNode = getNextNode(nPos);
		pMapNode->m_data.setFilterBank(pFB);
	}
}
///////////////////
template<class TYPE>
int SkMap<TYPE>::getValidNodeNum()
{
	int nPos = getFirstPos();
	SkMapNode<TYPE> *pMapNode;
	int n = 0;
	while (nPos >= 0)
	{
		pMapNode = getNextNode(nPos);
		if (pMapNode->valid())
			n++;
	}
	return n;
}
///////////////////
template<class TYPE>
int SkMap<TYPE>::getValidMaxID()
{
	int nPos = getFirstPos();
	SkMapNode<TYPE> *pMapNode;
	int max_id = -1;
	while (nPos >= 0)
	{
		pMapNode = getNextNode(nPos);
		if (!pMapNode->valid())
			continue;
		int id = pMapNode->getID();
		if (id > max_id)
			max_id = id;
	}
	return max_id;
}
///////////////////
// set out nbr of one node valid
template<class TYPE>
void SkMap<TYPE>::setNbrValid(SkMapNode<TYPE> *pMapNode, bool b, int nbrType)
{
	SkMapNode<TYPE> *pNbrNode;
	int nOutNbr = pMapNode->getNbrNum(nbrType);
	for (int i = 0; i < nOutNbr; i++)
	{
		int nNbrID = pMapNode->getNbrIDAt(i, nbrType);
		pNbrNode = getNodeAt(nNbrID);
		pNbrNode->setValid(b);
	}
}
///////////////////
// setElectronsAndNucleus()
//	if fabs(coef) >= dMinNucleusCoef then consider it as a nucleus
//	otherwise as electrons
//	return number of nucleus
template<class TYPE>
int	SkMap<TYPE>::setElectronsAndNucleus(double dMinNucleusCoef)
{
	int n = 0;
	SkMapNode<TYPE> *pNode;
	int nPos = getFirstPos();
	while (nPos >= 0)
	{
		pNode = getNextNode(nPos);
		if (!pNode->valid())
			continue;
		if (fabs(pNode->m_data.a) >= dMinNucleusCoef)
		{
			pNode->setType(NUCLEUS);
			n++;
		}
		else
			pNode->setType(FREE_ELEC);
	}
	return n;
}

///////////////////
// set out nbr of one node valid
template<class TYPE>
bool SkMap<TYPE>::checkAtomID()
{
	SkMapNode<TYPE> *pNode;
	int nPos = getFirstPos();
	while (nPos >= 0)
	{
		pNode = getNextNode(nPos);
		if (!pNode->valid())
			continue;
		assert(pNode->getAtomID() >= 0);
	}
	return true;
}

///////////////////
// getBaseNumOfType()
//	get the base num of each filter type, save them in pnNum
//	return base filter type num
template<class TYPE>
int SkMap<TYPE>::getBaseNumOfFilterType(int *pnNum)
{
	int nMaxFilterType = 3;
	for (int i = 0; i < nMaxFilterType; i++)
		pnNum[i] = 0;

	int n = -1;
	int t;
	SkMapNode<TYPE> *pNode;
	int nPos = getFirstPos();
	while (nPos >= 0)
	{
		pNode = getNextNode(nPos);
		if (!pNode->valid())
			continue;
		t = pNode->m_data.getType();
		assert(t >= 0 && t < nMaxFilterType);
		pnNum[t]++;
		if (n < t+1)
			n = t+1;
	}
	return n;
}

///////////////////
// updateOccupyMatrix()
template<class TYPE>
int SkMap<TYPE>::updateOccupyMatrix(SkMapNode<TYPE> *pNode, int flag, double dInhibitionDis)
{
	if( dInhibitionDis < 0)
		return 0;
	Matrix<oval_IntArray> &mat = m_occupyMat;
 
	int id = pNode->getID();

	int wd = getWidth();
	int ht = getHeight();

	int x0 = pNode->m_data.x;
	int y0 = pNode->m_data.y;
	int x, y;

	int l = pNode->m_data.getValidLength();
	int nInhDis = (int)dInhibitionDis;
	double dOrt = pNode->m_data.getOrt();
	int xs = -l;
	int xe = -xs;
	int ys = -nInhDis;
	int ye = -ys;


  // filling by polygonFilling, overcome the precision problem
	oval_Array<oval_Point> polyVetArray, regionPtArray;
	oval_Point pt;
	double xx, yy;

	xx = xs;
	yy = ys;
	::rotate(xx, yy, dOrt);
	if (xx >= 0)
		pt.x = (int)(xx+x0+0.5);
	else
		pt.x = (int)(xx+x0-0.5);
	if (yy >= 0)
		pt.y = (int)(yy+y0+0.5);
	else
		pt.y = (int)(yy+y0-0.5);
	polyVetArray.add(pt);

	xx = xs;
	yy = ye;
	::rotate(xx, yy, dOrt);
	if (xx >= 0)
		pt.x = (int)(xx+x0+0.5);
	else
		pt.x = (int)(xx+x0-0.5);
	if (yy >= 0)
		pt.y = (int)(yy+y0+0.5);
	else
		pt.y = (int)(yy+y0-0.5);
	polyVetArray.add(pt);

	xx = xe;
	yy = ye;
	::rotate(xx, yy, dOrt);
	if (xx >= 0)
		pt.x = (int)(xx+x0+0.5);
	else
		pt.x = (int)(xx+x0-0.5);
	if (yy >= 0)
		pt.y = (int)(yy+y0+0.5);
	else
		pt.y = (int)(yy+y0-0.5);
	polyVetArray.add(pt);

	xx = xe;
	yy = ys;
	::rotate(xx, yy, dOrt);
	if (xx >= 0)
		pt.x = (int)(xx+x0+0.5);
	else
		pt.x = (int)(xx+x0-0.5);
	if (yy >= 0)
		pt.y = (int)(yy+y0+0.5);
	else
		pt.y = (int)(yy+y0-0.5);
	polyVetArray.add(pt);

	int n = ::polygonFilling(polyVetArray, regionPtArray);
	for (int i = 0; i < n; i++)
	{
		pt = regionPtArray.getAt(i);
		x = pt.x;
		y = pt.y;
		if (x < 0 || x >= wd || y < 0 || y >= ht) 
			continue;
		if (flag == ADD)
			mat(y,x).add(id);
		else
			mat(y,x).remove(id);
	}

	return 0;
}

///////////////////
// buildOccupyMatrix()
template<class TYPE>
int SkMap<TYPE>::buildOccupyMatrix(double dInhibitionDis)
{
	SkMapNode<TYPE> *pNode;
	int nPos = getFirstPos();
	while (nPos >= 0)
	{
		pNode = getNextNode(nPos);
		if (!pNode->valid())
			continue;
		updateOccupyMatrix(pNode, m_occupyMat, ADD, dInhibitionDis);
	}
	return 0;
}

///////////////////
// clearOccupyMatrix()
template<class TYPE>
int SkMap<TYPE>::clearOccupyMatrix()
{
	int wd, ht;
	m_occupyMat.GetDimension(ht, wd);
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			m_occupyMat(r,c).removeAll();
		}
	}
	return 0;
}

///////////////////
// setFilterBank4AllBases()
template<class TYPE>
int SkMap<TYPE>::setFilterBank4AllValidBases(SkFilterBank *pFB)
{
	SkMapNode<TYPE> *pNode;
	int nPos = getFirstPos();
	while (nPos >= 0)
	{
		pNode = getNextNode(nPos);
		if (!pNode->valid())
			continue;
		pNode->m_data.setFilterBank(pFB);
	}
	return 0;
}

typedef SkMap<SkBase> SkBaseMap;
typedef SkMapNode<SkBase> SkBaseNode;

#endif // _SK_MAP_H_