#ifndef _SK_MATRIX_H
#define _SK_MATRIX_H

#include <iostream>
#include <fstream>
#include <memory.h>
#include <stdlib.h>
#include <math.h>
#include <new.h>
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>


typedef unsigned char Octet;
#define LPI	3.14159265359

typedef double* pdouble;


/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////


template<class Tp>
class Matrix
{

public:
		//CONSTRUCTION
		Matrix();
		Matrix( int row, int col);
		Matrix(const Matrix<Tp>& x);
        ~Matrix();
		
		//OPERATORS
		Matrix<Tp>&		operator=(const Matrix<Tp>& x );
		Tp&				operator()(const int row, const int col);
		Tp&				operator()(const long index);
        Matrix<Tp>	operator+(const Matrix<Tp>& x);
		Matrix<Tp>	operator-(const Matrix<Tp>& x);
		Matrix<Tp>	operator*(const Matrix<Tp>& x);
		void			Set(Tp t);


		Matrix<Tp>			operator+(const Tp y);
		Matrix<Tp>			operator-(const Tp y);
		Matrix<Tp>			operator*(const Tp y);
		Matrix<Tp>			operator/(const Tp y);
		Matrix<Tp>&			operator^(const Matrix<Tp> y);
		
		int				SetDimension(const int row, const int col);
		void			GetDimension(int& row, int& col) const; 
		long			rows() const;
		long			cols() const;
		int				nx() const{return ncol;};
		int				ny() const{return nrow;};
		inline long		size(void) const {return rows()*cols();};
		int		MatrixFree();
		
		inline bool		Valid(const int row, const int col);
		inline bool		Valid(const int row);
		Matrix<Tp>			T() const;
		Matrix<Tp>			rot90(int K=1) const;
		virtual void	Identity(void);					// set this matrix to be I matrix
		double			Det() const;	
		double			DetNoZero() const;
   		Matrix<Tp>			Inv() const;
		virtual void	Zero(void);						// set this matrix to be zero matrix
		inline Tp		Data(const int row, const int col) const;
		inline Tp		Data(const int index) const;
				Tp		SubPixel(const double x, const double y) const;
		inline	void	init_value(Tp value);
		virtual Tp		GetMin(void) const;
		virtual Tp		GetMax(void) const;
		virtual Tp		Sum(void) const;
		virtual Tp		sumsq(void) const;
		virtual Tp		Mean(void) const;
		virtual Tp		Var(void) const;
		virtual Matrix<Tp>	mabs(void) const;
		virtual void	GetPos(const long index, long &j, long &i);
		virtual Tp		DotProduct(const Matrix<Tp> &kern, const long jshift=0, const long ishift=0);
		virtual Tp		DotProductR(const Matrix<Tp> &kern, const long jshift=0, const long ishift=0);
		virtual Tp		Dist(const Matrix<Tp> &a);

		// convolution
		virtual Matrix<Tp>	Conv(const Matrix<Tp>& kern);		// two dimensioanl convolution
		virtual Matrix<Tp>	ConvHor(const Matrix<Tp>& kern);	// horizontal convolution 
		virtual Matrix<Tp>	ConvVer(const Matrix<Tp>& kern);	// vertical convolution
		virtual Matrix<Tp>	Conv2(const Matrix<Tp>& kern);	
		virtual void	NormToZero(void);				// make sum up to zero
		virtual bool	SameColValue(Tp &x, const int i=0);

		// kernel generation
		virtual void	SetGaussian(int dim=1, double sigma=2, int direvitive=0);	// Gaussian kernel
		virtual void	SetGabor(const double k, const double sigma, const double theta=0 /* in degree */);
		virtual void	SetGabor(const double k, const double sigma, const double theta=0 /* in degree */, const int even = 0);
		virtual void	SetLaplaicanOfGaussian(const double sigma);


		// pdf functions
		Tp				PdfNorm1(const Matrix &x);
		Tp				PdfNorm2(const Matrix &x);

protected:
        friend int		LUdcmp(Matrix<Tp>& X, Matrix<Tp>& indx, double& d);
		friend int		LUbksb(Matrix<Tp>& X, Matrix<Tp>& indx, Matrix<Tp>& B);
		friend void		Eigen(Matrix<Tp>& a, Matrix<Tp>& eigenvalue, Matrix<Tp>& eigenvector);
		friend void		SVD(Matrix<Tp>& a, Matrix<Tp>& U, Matrix& D, Matrix<Tp>& V);

private:
		int MatrixAlloc(const int row, const int col);

protected:
		Tp		**pprow;
public:
		int		nrow, ncol ;
		Tp		*data;

};

template<class Tp1, class Tp2> void assign_matrix(Matrix<Tp1>& x1,const Matrix<Tp2>& x2);


template<class Tp>
Matrix<Tp>::Matrix()
{  //null matrix
	nrow = 0; ncol = 0;
	data = NULL;
	pprow = NULL;
}

template<class Tp>
Matrix<Tp>::Matrix(int row, int col)
{	
	Matrix();

	//initialize matrix
	if (row > 0 && col >0)
	{
		nrow = row;
		ncol = col;
		if (MatrixAlloc(nrow, ncol) )
		{
			std::cerr<<"\n Memory allocation Error ! " ;
			exit(-1);
		}
	}
}

template<class Tp>
Matrix<Tp>::Matrix(const Matrix<Tp>& x)
{//copy constructor

	nrow = x.nrow; ncol = x.ncol;
	MatrixAlloc(nrow, ncol);  //allocate memory

	for (int i = 0; i < rows(); i++)
		for (int j = 0; j < cols(); j++)
			(*this)(i, j) = x.Data(i, j);
}


template<class Tp>
Matrix<Tp>::~Matrix()
{  //delete the data
	MatrixFree(); //free the memory
}

template<class Tp>
int Matrix<Tp>::MatrixFree()
{  //null matrix

	if (data != NULL)
		delete[] data;           // STEP 2: DELETE THE ROWS
	if (pprow != NULL)
		delete[] pprow;

	data = NULL;
	pprow = NULL;
	nrow = ncol = 0;

	return 0;
}

template<class Tp>
int Matrix<Tp>::MatrixAlloc(const int row, const int col)
{
	//	return MatrixAlloc(((long)row) * col);
	try
	{ 								//test for exceptions
		data = new Tp[long(row)*long(col)];	//allocate memory for rows
		pprow = new Tp*[row];
		for (int i = 0; i < row; i++)
			pprow[i] = data + i * col;

	}
	catch (double*)
	{  // ENTER THIS BLOCK ONLY IF xalloc IS THROWN.
		return -1;
	}
	return 0;
}


template<class Tp>
long Matrix<Tp>::rows() const
{ 
	return nrow;
}

template<class Tp>
long Matrix<Tp>::cols() const
{ 
	return ncol;
}

template<class Tp>
inline bool	Matrix<Tp>::Valid(const int row, const int col)
{
	if (row>=0 && row<rows() && col>=0 && col<cols())
		return true;
	else
		return false;
}

template<class Tp>
inline bool	Matrix<Tp>::Valid(const int row)
{
	if (row>=0 && row<size())
		return true;
	else
		return false;
}

template<class Tp>
int Matrix<Tp>::SetDimension(const int row, const int col)
{  //set the dimension and allocates the memory of the matrix

	//delete the existing data;
	if (nrow != row || ncol != col)
	{
		MatrixFree();
		nrow = row;
		ncol = col;

		MatrixAlloc(row, col);
	}
	return 0; //could be used to return the success
	// or failure of allocation
}

template<class Tp>
void Matrix<Tp>::GetDimension(int& row, int& col) const
{  //report the dimension of the matrix
	row = nrow;
	col = ncol;
}

template<class Tp>
Matrix<Tp> Matrix<Tp>::operator+(const Matrix<Tp>& x)
{  //add two matrices

	Matrix<Tp> temp(x.nrow, x.ncol); //intialize the temporary matrix

	for(int i = 0; i < x.rows(); i++)
		for(int j = 0; j < x.cols(); j++)
			temp(i, j) = Data(i, j) + x.Data(i, j); //add element by element

	return temp; //return the new matrix
}


template<class Tp>
Matrix<Tp> Matrix<Tp>::operator+(const Tp y)
{  
	//add a constant to a matrices
	Matrix<Tp> temp(rows(), cols()); //intialize the temporary matrix

	for(int i = 0; i < nrow; i++)
		for(int j = 0; j < ncol; j++)
			temp(i, j) = (*this)(i,j)+y;  //add element by element

	return temp; //return the matrix
}


template<class Tp>
Matrix<Tp> Matrix<Tp>::operator-(const Matrix<Tp>& x)
{  //subtract two matrix
	Matrix<Tp> temp(x.rows(),x.cols()); //intialize the temporary matrix

	for (int i = 0; i < x.nrow; i++)
		for (int j = 0; j<x.ncol; j++)
			temp(i, j) = Data(i, j)- x.Data(i, j); //sub element by element

	return temp; //return the new matrix
}


template<class Tp>
Matrix<Tp> Matrix<Tp>::operator-(const Tp y)
{  //subtract a constant from a matrix
	Matrix<Tp> temp(rows(), cols()); //intialize the temporary matrix

	for (int i = 0; i < nrow; i++)
		for (int j = 0; j < ncol; j++ )
			temp(i,j) = (*this)(i, j)-y; //subtract element by element

	return temp; //return the matrix
}


template<class Tp>
Matrix<Tp> Matrix<Tp>::operator*(const Matrix<Tp>& y)
{  //multiply two matrices
	Tp			sum;
	Matrix<Tp>	temp(nrow,y.ncol); //intialize the temporary matrix

	for (int i = 0; i < nrow; i++)
		for (int j = 0; j < y.ncol; j++)
		{
			sum = 0;
			for(int k = 0; k < ncol; k++)
				sum = sum+Data(i, k)*Data(k, j); //multiply

			temp(i, j) = sum ;
		}

		return temp; //return the result of multiplication
}
template<class Tp>
Matrix<Tp> Matrix<Tp>::operator*(const Tp y)
{  //multiply by a scalar constant
	Matrix<Tp> temp(rows(), cols()); //intialize the temporary matrix

	for (int i = 0; i < nrow; i++)
		for (int j = 0; j < ncol; j++)
			temp(i,j) = (*this)(i, j)*y; //scale element by element

	return temp; //return the matrix
}

template<class Tp>
Matrix<Tp>&	Matrix<Tp>::operator^(const Matrix<Tp> y)
{
#ifdef _DEBUG
	assert(rows()==y.rows() && cols()==y.cols());
#endif

	for (long i=0; i<size(); i++)
		(*this)(i) = (*this)(i)*y.Data(i);
	return *this;
}

template<class Tp>
Matrix<Tp> Matrix<Tp>::operator/(const Tp y)
{  //multiply by a scalar constant
	Matrix<Tp> temp(rows(), cols()); //intialize the temporary matrix

	for (int i = 0; i < nrow; i++)
		for (int j = 0; j < ncol; j++)
			temp(i,j) = (*this)(i, j)/y; //scale element by element

	return temp; //return the matrix
}

template<class Tp>
Tp& Matrix<Tp>::operator()(const int row, const int col)
{	
#ifdef _DEBUG
	assert(row < rows() && row >= 0);
	assert(col < cols() && col >= 0);
#endif

	//	return data[((long)row) * cols() + col];
	return pprow[row][col];
}

template<class Tp>
Tp& Matrix<Tp>::operator()(const long index)
{	
#ifdef _DEBUG
	assert(index<((long)rows()) * cols() && index>=0);
#endif

	return data[index];
}

template<class Tp>
Matrix<Tp>&	Matrix<Tp>::operator=(const Matrix<Tp>& x)
{//performs assignment statement

	//beware of self assignments : x = x
	if ( this != &x )  
	{
		register int i,j;

		if (rows()!=x.rows() || cols()!=x.cols())
		{
			//delete the existing data;
			MatrixFree();

			nrow = x.nrow; ncol = x.ncol;
			MatrixAlloc(nrow, ncol); //allocate memory
		}

		for(i=0; i<nrow; i++)
			for(j=0; j<ncol; j++)
				(*this)(i, j) = (Tp)(x.Data(i, j));
	}
	return *this;
}

template<class Tp1, class Tp2>
void assign_matrix(Matrix<Tp1>& x1,const Matrix<Tp2>& x2)
{
	x1.SetDimension(x2.rows(), x2.cols());

	for(int i = 0; i < x2.rows(); i++)
		for(int j = 0; j < x2.cols(); j++)
			x1(i, j) = (Tp1)(x2.Data(i, j));
}

template<class Tp>
Matrix<Tp> Matrix<Tp>::T() const
{//transposes of a matrix

	Matrix temp(ncol,nrow); //intialize a temp matrix

	for (int i = 0; i < ncol; i++)
		for (int j = 0; j < nrow; j++)
			temp(i, j) = Data(j, i); //transpose elements

	return temp;
}
template<class Tp>
Matrix<Tp> Matrix<Tp>::rot90(int K) const{
	//	  ROT90  Rotate matrix 90 degrees.
	//    ROT90() is the 90 degree counterclockwise rotation of matrix A.
	//    ROT90(K) is the K*90 degree rotation of A, K = +-1,+-2,...

	//    Example,
	//        A = [1 2 3      B = rot90(A) = [ 3 6
	//             4 5 6 ]                     2 5
	//                                         1 4 ]
	int i,j;
	Matrix temp; //intialize a temp matrix
	int k = K%4;
	if (k < 0)
		k += 4;
	if (k==1){
		temp.SetDimension(ncol,nrow);
		for (i = 0; i < ncol; i++)
			for (j = 0; j < nrow; j++)
				temp(i, j) = Data(j, ncol-i-1); //rotate elements

		return temp;
	}
	if (k==2){
		temp.SetDimension(nrow,ncol);
		for (i = 0; i <nrow ; i++)
			for (j = 0; j < ncol; j++)
				temp(i, j) = Data(nrow-i-1, ncol-j-1); //rotate elements

		return temp;
	}
	if (k==3){
		temp.SetDimension(ncol,nrow);
		for (i = 0; i < ncol; i++)
			for (j = 0; j < nrow; j++)
				temp(i, j) = Data(nrow-j-1, i); //rotate elements

		return temp;
	}
	temp.SetDimension(nrow,ncol);
	for (i = 0; i <nrow ; i++)
		for (j = 0; j < ncol; j++)
			temp(i, j) = Data(i, j); 

	return temp;	
}


// set this matrix to be I matrix
template<class Tp>
void Matrix<Tp>::Identity(void)
{
	for (int i = 0; i < rows();  i++)
		for (int j = 0; j < cols();  j++)
		{
			if( i==j) 
				(*this)(i,j) = 1;
			else 
				(*this)(i,j) = 0;
		}
}


// set this matrix to be zero matrix					
template<class Tp>
void Matrix<Tp>::Zero(void)						
{
	for (int i = 0; i < rows();  i++)
		for (int j = 0; j < cols();  j++)
			(*this)(i,j) = 0;
}

template<class Tp>
void Matrix<Tp>::Set(Tp t)						
{
	for (int i = 0; i < rows();  i++)
		for (int j = 0; j < cols();  j++)
			(*this)(i,j) = t;
}


template<class Tp>
Tp Matrix<Tp>::Data(const int row, const int col) const
{
#ifdef _DEBUG
	assert(row < rows() && row >= 0);
	assert(col < cols() && col >= 0);
#endif

	//	return data[((long)row) * cols() + col];
	return pprow[row][col];
}
////////////////////////////////////////////////////////////////
template<class Tp>
Tp Matrix<Tp>::SubPixel(const double x, const double y) const
{	//interpolates the pixels, 
	int ix=(int)x,iy=(int)y;
	int ny=rows(),nx=cols();
	double a,p1,p2;
	Tp lt,rt,lb,rb; //lt=left top, rb=right bottom and so on

	if ((ix<0)||(ix>=nx)||(iy<0)||(iy>=ny))	
		return 0;

	lt=Data(iy,ix);

	if (ix+1<nx)	
		rt=Data(iy,ix+1);
	else 
		rt=0;
	if (iy+1<ny) 
		lb=Data(iy+1,ix);
	else 
		lb=0;
	if ((ix+1<nx)&&(iy+1<ny)) 
		rb=Data(iy+1,ix+1);
	else 
		rb=0;
	a=x-ix;
	p1=(1-a)*lt+a*rt;
	p2=(1-a)*lb+a*rb;
	a=y-iy;
	return (Tp)((1-a)*p1+a*p2);
}

template<class Tp>
Tp Matrix<Tp>::Data(const int index) const
{
#ifdef _DEBUG
	assert(index>=0 && index<size());
#endif

	return data[index];
}

template<class Tp>
void Matrix<Tp>::GetPos(const long index, long &j, long &i)
{
	j = index/(cols());
	i = index-j*cols();
}

template<class Tp>
inline	void Matrix<Tp>::init_value(Tp value)
{
	register int i,j;

	if (data != NULL)
	{
		for (i = 0; i < rows(); i++)
			for (j = 0; j < cols(); j++)
				(*this)(i, j) = value;
	}
}


///////////////////////////////////////////////////
// two dimensioanl convolution
template<class Tp>
//Matrix<Tp> Matrix<Tp>::Conv(const Matrix<Tp>& kern)
Matrix<Tp> Matrix<Tp>::Conv(const Matrix& kern)
{
	Matrix<Tp>	I;
	long i,j,i1,j1,i2,j2,m1,n1,m2,n2;

	I.SetDimension(rows(), cols());

	I.Zero();
	m1 = kern.rows();
	n1 = kern.cols();

	m2 = rows();
	n2 = cols();

	for (j=0; j<m2; j++)
		for (i=0; i<n2; i++)
		{
			for (j1=0; j1<m1; j1++)
				for (i1=0; i1<n1; i1++)
				{
					j2 = j-(j1-(m1-1)/2);
					i2 = i-(i1-(n1-1)/2);
					if (j2>=0 && j2<m2 &&
						i2>=0 && i2<n2)
						I(j,i) = I(j,i)+Data(j2,i2)*kern.Data(j1,i1);
				}
		}
		return I;
}

// horizontal convolution 
template<class Tp>
Matrix<Tp> Matrix<Tp>::ConvHor(const Matrix& kern)
{
	Matrix<Tp>	I;

	I.SetDimension(rows(), cols());
	long i,j,k,l,m;

	I.Zero();
	l = kern.cols();
	for (j=0; j<rows(); j++)
		for (i=0; i<cols(); i++)
		{
			for (k=0; k<l; k++)
			{
				m = i-(k-(l-1)/2);
				if (m>=0 && m<cols())
					I(j,i) = I(j,i)+Data(j,m)*kern.Data(0,k);
			}
		}
		return I;
}

// vertical convolution
template<class Tp>
Matrix<Tp> Matrix<Tp>::ConvVer(const Matrix& kern)
{
	Matrix<Tp>	I;

	// get tranpose of matrix
	I = T();
	I = I.ConvHor(kern.T());
	return I.T();
}

template<class Tp>
Matrix<Tp> Matrix<Tp>::Conv2(const Matrix& kern)
{
	Matrix<Tp> I;

	I = ConvHor(kern);
	I = I.ConvVer(kern.T());
	return I;
}

inline double	gauss(double x, double sigma)
{return exp(-x*x/(2*sigma*sigma)) / (sigma*sqrt(2*LPI));};
inline double	dgauss(double x, double sigma)
{return -x*gauss(x,sigma)/(sigma*sigma);};
inline double	d2gauss(double x, double sigma)
{return (x*x/pow(sigma,4)-1/pow(sigma,2))*gauss(x,sigma);};

//////////////////////////////////////////////////////
// kernel generation
/*   dim : dimension 1: one
2: two
sigma: sigmal value
direvite: 0: 
1: first
2: second
*/
template<class Tp>
void Matrix<Tp>::SetGaussian(int dim, double sigma, int direvitive)
{
	Matrix<Tp>	op;
	int			size,i,j,u;
	double		x,y;

	size = ((int)sigma)*6+1;
	u = (size-1)/2;

	assert(size > 0);

	// gaussian kernel
	if (dim == 1)
	{
		op.SetDimension(1,size);
		op.Zero();
		for (i=0; i<size; i++)
		{
			x = double(i-u);
			switch (direvitive)
			{
			case 0:	// Gaussian function
				op(0,i) = (Tp)gauss(x,sigma);
				break;
			case 1:	// First dirivtive of Gaussian
				op(0,i) = (Tp)dgauss(x,sigma);
				break;
			case 2: // Second dirivtive of Gaussian
				op(0,i) = (Tp)d2gauss(x,sigma);
				break;
			}
		}
	}
	else
	{
		double	std2=sigma*sigma;
		double	h1;

		op.SetDimension(size,size);
		op.Zero();

		switch(direvitive)
		{
		case 2:
			for (j=0; j<size; j++)
				for (i=0; i<size; i++)
				{
					x = double(j-u);
					y = double(i-u);

					h1 = exp(-(x*x+y*y)/(2*std2));
					op(j,i) = (Tp)(h1*(x*x+y*y-2*std2)/(2*LPI*(pow(sigma,6))));
				}
				break;
		}
	}
	*this = op;
}

// Gabor Filter
// make gabor filter                                                   2
//               k*k                k*k       2                  sigma
//  Pk(x) = --------------exp(- -------------x )(exp(ikx)-exp(- --------)
//          (sigma*sigma)       (sigma*sigma)                      2
//
template<class Tp>
void Matrix<Tp>::SetGabor(const double k, const double sigma, 
						  const double theta /* in degree */)
{
	int half_size,w,h,i,j;

	// size of the filter
	if (k>1e-10)
		half_size = (int)floor(sigma/k*1.5);
	else
		half_size = (int)floor(sigma*1.5);
	w = 2*half_size+1;
	h = 2*half_size+1;

	// initilization
	// gaussian kernel
	Matrix<Tp>	F(1,w),G;
	double		x,sigma_s,k_s;

	k_s = k*k;
	sigma_s = sigma*sigma;
	for (i=0; i<w; i++)
	{
		x = (double)(i-half_size);
		x = x*x;
		F(i) = (Tp)(k_s/sigma_s*exp(-k_s/(2*sigma_s)*x));
	}
	G = F.T()*F;

	// sinusoid kernel
	// to grad
	Matrix<Tp>		P(h,w);
	Matrix<double>	r(2,2),temp(2,1),u;
	double			angle;
	angle = theta/180*LPI;
	r(0,0) = cos(angle);
	r(0,1) = -sin(angle);
	r(1,0) = sin(angle);
	r(1,1) = cos(angle);

	for (j=0; j<h; j++)
		for (i=0; i<w; i++)
		{
			temp(0) = j-half_size;
			temp(1) = i-half_size;
			u = r*temp;
			P(j,i) = (Tp)(cos(u(0)*k));
		}

		// Gabor filter
		(*this) = G^P;
		(*this) = (*this)/Sum();
}

// Gabor Filter
// make gabor filter                                                   2
//               k*k                k*k       2                  sigma
//  Pk(x) = --------------exp(- -------------x )(exp(ikx)-exp(- --------)
//          (sigma*sigma)       2*(sigma*sigma)                      2
//
template<class Tp>
void Matrix<Tp>::SetGabor(const double k, const double sigma, 
						  const double theta /* in degree */, const int even)
{
	int half_size,w,h,i,j;

	// size of the filter
	if (k>1e-10)
		//		half_size = (int)floor(sigma/k*1.5);
		half_size = (int)floor(sigma/k*4);
	else
		//		half_size = (int)floor(sigma*1.5);
		half_size = (int)floor(sigma*4);
	w = 2*half_size+1;
	h = 2*half_size+1;

	// initilization
	// gaussian kernel
	Matrix<Tp>	F(1,w),G;
	double		x,sigma_s,k_s;

	k_s = k*k;
	sigma_s = sigma*sigma;
	for (i=0; i<w; i++)
	{
		x = (double)(i-half_size);
		x = x*x;
		F(i) = (Tp)(k_s/sigma_s*exp(-k_s/(2*sigma_s)*x));
	}
	G = F.T()*F;

	// sinusoid kernel
	// to grad
	Matrix<Tp>		P(h,w);
	Matrix<double>	r(2,2),temp(2,1),u;
	double			angle;
	angle = theta/180.0*LPI;
	r(0,0) = cos(angle);
	r(0,1) = -sin(angle);
	r(1,0) = sin(angle);
	r(1,1) = cos(angle);

	for (j=0; j<h; j++)
		for (i=0; i<w; i++)
		{
			temp(0) = j-half_size;
			temp(1) = i-half_size;
			u = r*temp;//u(0)-y, u(1)-x; CCW rotate
			if (even)
				P(j,i) = (Tp)(cos(u(0)*k));
			else
				P(j,i) = (Tp)(sin(u(0)*k));
		}

		// Gabor filter
		(*this) = G^P;
		//	printf("sum=%.10f\n", Sum());
		//	(*this) = (*this)/Sum();
}

template<class Tp>
void Matrix<Tp>::SetLaplaicanOfGaussian(const double sigma)
{
	int		half_size,w,h,i,j;
	double	x,y;

	half_size = (int)floor(sigma/2.0);
	w = 2*half_size+1;
	h = 2*half_size+1;

	SetDimension(h,w);
	for (j=0; j<h; j++)
		for (i=0; i<w; i++)
		{
			x = j-half_size; y = i-half_size;
			(*this)(j,i) = (Tp)((x*x+y*y-sigma*sigma)*exp(-(x*x+y*y)/(2.0*sigma*sigma))/pow(sigma,2.0));
		}
		Tp sum=(Tp)Sum();
		for (j=0; j<h; j++)
			for (i=0; i<w; i++)
				(*this)(j,i) = (Tp)(((*this)(j,i)-sum/((double)size()))*10.0);
}

template<class Tp>
Tp Matrix<Tp>::GetMin(void) const
{
	Tp value;

	if (size()>0)
	{
		long i;
		value = Data(0);
		for (i=1; i<size(); i++)
			if (Data(i)<value)
				value = Data(i);
	}

	return value;
}

template<class Tp>
Tp Matrix<Tp>::GetMax(void) const
{
	Tp value;

	if (size()>0)
	{
		long i;
		value = Data(0);
		for (i=1; i<size(); i++)
			if (Data(i)>value)
				value = Data(i);
	}

	return value;
}
template<class Tp>
Tp Matrix<Tp>::Sum(void) const
{
	Tp	value;
	value = 0;
	for (long i=0; i<size(); i++)
		value = value+Data(i);

	return value;
}
template<class Tp>
Tp Matrix<Tp>::sumsq(void) const
{
	Tp	value;
	value = 0;
	for (long i=0; i<size(); i++)
		value = value+Data(i)*Data(i);

	return value;
}
// make sum up to zero
template<class Tp>
void Matrix<Tp>::NormToZero(void)
{
	Tp		sum=Sum(),g,z;
	long	i;

	z = 0;
	g = (Tp)((sum)/((double)size()));
	if (sum != z)
		for (i=1; i<size(); i++)
			(*this)(i) = (Tp)(Data(i)-g);
}

template<class Tp>
Tp Matrix<Tp>::Mean(void) const
{
	Tp	value;
	long temp;

	value = 0;
	temp = size();
	if (temp>0)
		value = (Tp)(Sum()/temp);
	return value;
}
template<class Tp>
Tp Matrix<Tp>::Var(void) const
{
	Tp	value,mean=Mean();
	long i,len;

	value = 0;
	len = size();
	if (len>1){
		for (i=0;i<len;i++)
			value += (Tp)((Data(i)-mean)*(Data(i)-mean));
		value=(Tp)(value/(len-1));
	}
	return value;
}

template<class Tp>
Matrix<Tp> Matrix<Tp>::mabs(void) const
{
	Matrix<Tp> I;
	Tp			z;

	z = 0;
	I.SetDimension(rows(),cols());
	for (long i=0; i<size(); i++)
	{
		if (Data(i)<z)	I(i) = z-Data(i);
		else			I(i) = Data(i);
	}
	return I;
}

template<class Tp>
Tp Matrix<Tp>::DotProduct(const Matrix<Tp> &kern, const long jshift, const long ishift)
{
	Tp		value;
	long	i,j,k,l;

	value = 0;
	if (size()<kern.size())
	{
		for (j=0; j<rows(); j++)
			for (i=0; i<cols(); i++)
			{
				k = j-jshift;
				l = i-ishift;
				if (k>=0 && k<kern.rows() && l>=0 && l<kern.cols())
					value = value+kern.Data(k,l)*Data(j,i);
			}
	}
	else
	{
		for (k=0; k<kern.rows(); k++)
			for (l=0; l<kern.cols(); l++)
			{
				j = k+jshift;
				i = l+ishift;
				if (j>=0 && j<rows() && i>=0 && i<cols())
					value = value+kern.Data(k,l)*Data(j,i);
			}
	}
	return value;
}

template<class Tp>
Tp Matrix<Tp>::DotProductR(const Matrix<Tp> &kern, const long jshift, const long ishift)
{
	Tp		value;
	long	i,j,k,l;

	value = 0;
	if (size()<kern.size())
	{
		for (j=0; j<rows(); j++)
			for (i=0; i<cols(); i++)
			{
				k = (j-jshift+2*kern.rows())%kern.rows();
				l = (i-ishift+2*kern.cols())%kern.cols();
				value = value+kern.Data(k,l)*Data(j,i);
			}
	}
	else
	{
		for (k=0; k<kern.rows(); k++)
			for (l=0; l<kern.cols(); l++)
			{
				j = (k+jshift+2*rows())%rows();
				i = (l+ishift+2*cols())%cols();
				value = value+kern.Data(k,l)*Data(j,i);
			}
	}
	return value;
}

template<class Tp>
Tp Matrix<Tp>::PdfNorm1(const Matrix &x)
{
	Tp result=0;

#ifdef _DEBUG
	assert(x.size() == size());
#endif

	for (long i=0; i<size(); i++)
		result += __min(Data(i),x.Data(i));
	return result;
}

template<class Tp>
Tp Matrix<Tp>::PdfNorm2(const Matrix &x)
{
	Tp result=0;

#ifdef _DEBUG
	assert(x.size() == size());
#endif

	for (long i=0; i<size(); i++)
		result += (Data(i)-x.Data(i))*(Data(i)-x.Data(i));
	return result;
}

template<class Tp>
double Matrix<Tp>::Det() const
{//returns the value of the determinanat

	double d; //interchange flag

	Matrix<Tp> temp(*this); 		//make a copy of the original matrix
	Matrix<Tp> index(1,temp.rows());

#ifdef _DEBUG
	assert(nrow==ncol);
#endif

	if( LUdcmp(temp,index,d)) //decompose the matrix
		return 0;

	for( int j=0; j<nrow; j++) 
		d *= temp(j,j);
	return d;//fabs(d);
}

template<class Tp>
double Matrix<Tp>::DetNoZero() const
{
	double det = Det();
	if (det<1.0)
		return 1.0;
	else
		return det;
}

template<class Tp>
Matrix<Tp> Matrix<Tp>::Inv() const
{//returns the inverse of the matrix

	double	d; //interchange flag
	int		i,j;

	Matrix<Tp> temp(*this); 		//make a copy of the original matrix
	Matrix<Tp> col(rows(),1) ;     //temporary variable
	Matrix<Tp> index(1,rows()) ;
	Matrix<Tp> result(rows(),cols());    //the final result

	if( LUdcmp(temp, index, d) )
	{    //decompose the matrix
		std::cerr<<"\n\n\t Error Singluar matrix...."<<std::endl;
		return result;
	}

	for(j=0; j<rows(); j++) 
	{
		col.init_value(0);
		col(j,0) = 1;
		LUbksb(temp,index,col);
		for(i=0; i<nrow; i++)
			result(i,j) = col(i,0);
	}
	return result;
}


#define TINY 1.0e-20;

template<class Tp>
int LUdcmp(Matrix<Tp>& X, Matrix<Tp>& indx, double& d)
{
	int i,imax, j, k;
	double big, dum, sum, temp;

	Matrix<Tp> vv(1,X.nrow); //vv stores the implicit scaling of each nrow

	d = 1.0;						//no nrow interchange yet
	for (i=0;i<X.nrow;i++)    //loop to get implicit scaling information
	{
		big = 0.0;
		for (j=0;j<X.ncol;j++)
			if ((temp=fabs(X(i,j))) > big) big=temp;

		if (big == 0.0)
		{
			cerr<<"Singular matrix in routine LUDCMP"<<endl;
			return -1;
		}

		vv(0,i)=1.0/big;  //save the scaling
	}

	for (j=0;j<X.nrow;j++) 
	{
		for (i=0;i<j;i++) 
		{
			sum = X(i,j);
			for (k=0;k<i;k++) sum -= X(i,k)*X(k,j);
			X(i,j)=sum;
		}
		big=0.0;
		for (i=j;i<X.nrow;i++)
		{
			sum = X(i,j);
			for (k=0;k<j;k++)
				sum -= X(i,k)*X(k,j);

			X(i,j)=sum;
			if ( (dum=vv(0,i)*fabs(sum)) >= big)
			{
				big=dum;
				imax=i;
			}
		}

		if (j != imax)
		{
			for (k=0;k<X.nrow;k++) 
			{
				dum = X(imax,k);
				X(imax,k)= X(j,k);
				X(j,k)=dum;
			}
			d = -(d);            			// change the parity of d
			vv(0,imax)=vv(0,j);				// Also interchange the scale factor
		}

		indx(0,j) = imax;
		if (X(j,j) == 0.0) X(j,j)=TINY;
		if (j != (X.nrow-1))
		{
			dum = 1.0/(X(j,j));
			for (i=j+1;i<X.nrow;i++) X(i,j) *= dum;
		}
	}
	return 0;
}
#undef TINY

template<class Tp>
int LUbksb(Matrix<Tp>& X, Matrix<Tp>& indx, Matrix<Tp>& B)
{
	int i, ii=-1, ip, j;
	double sum;

	for (i=0;i<X.nrow;i++)
	{
		ip  = (int) indx(0,i);
		sum = B(ip, 0);
		B(ip, 0) = B(i,0);
		if (ii>=0)
			for (j=ii;j<=(i-1);j++) sum -= X(i,j)*B(j,0);
		else if (sum) ii=i;
		B(i,0)=sum;
	}

	for (i=(X.nrow-1);i>=0;i--)
	{
		sum = B(i,0);
		for (j=i+1;j<X.nrow;j++) sum -= X(i,j)*B(j,0);
		B(i,0)=sum/X(i,i);
	}

	return 0;
}

template<class Tp>
bool Matrix<Tp>::SameColValue(Tp &x, const int i)
{
	int		j=0;

	if (rows()>0 && i<cols())
		x = Data(0,i);

	for (j=1; j<rows(); j++)
	{
		if (!(Data(j,i)==Data(0,i)))
			return false;
	}
	return true;
}


template<class Tp>
Tp Matrix<Tp>::Dist(const Matrix<Tp> &a)
{
	Matrix<Tp>	temp;

	if (a.cols()!=1)		return 0;
	if (cols()!=1)			return 0;
	if (rows()!=a.rows())	return 0;

	temp = *this-a;
	temp = temp.T()*temp;

	return temp.Data(0);
}

int jacobi(double **a, int n, double *d, double **v, int *nrot);
void svdcmp(double **a,int m,int n,double *w,double **v);

template<class Tp>
void Eigen(Matrix<Tp>& mx, Matrix<Tp>& eigenvalue, Matrix<Tp>& eigenvector)
{
	int i,j,n,*nrot;
	double **a,*d,**v;

	if (mx.rows()!=mx.cols())
		return;

	// allocate memory
	a = new pdouble[mx.rows()+1];
	for (j=0; j<mx.rows()+1; j++)
		a[j] = new double[mx.cols()+1];

	d = new double[mx.rows()+1];

	v = new pdouble[mx.rows()+1];
	for (j=0; j<mx.rows()+1; j++)
		v[j] = new double[mx.cols()+1];

	nrot = new int[mx.rows()+1];

	n = mx.rows();

	eigenvalue.SetDimension(n,1);
	eigenvector.SetDimension(n,n);

	// copy data
	for (j=0; j<mx.rows(); j++)
		for (i=0; i<mx.cols(); i++)
			a[j+1][i+1] = (double)mx(j,i);
	// compute eigen values and eigen vectors
	jacobi(a,n,d,v,nrot);

	for (j=0; j<mx.rows(); j++)
	{
		eigenvalue(j) = d[j+1];
		for (i=0; i<mx.cols(); i++)
			eigenvector(j,i) = (Tp)(v[j+1][i+1]);
	}

	// free memory
	for (j=0; j<mx.rows(); j++)
	{
		delete []a[j];
		delete []v[j];
	}
	delete []a;
	delete []d;
	delete []v;
	delete []nrot;


}
template<class Tp>
void SVD(Matrix<Tp>& mx, Matrix<Tp>& U, Matrix<Tp>& D, Matrix<Tp>& V)
{
	int i,j,n,*nrot,m;
	double **a,*d,**v;

	// allocate memory
	a = new pdouble[mx.rows()+1];
	for (j=0; j<mx.rows()+1; j++)
		a[j] = new double[mx.cols()+1];

	d = new double[mx.rows()+1];

	v = new pdouble[mx.rows()+1];
	for (j=0; j<mx.rows()+1; j++)
		v[j] = new double[mx.cols()+1];

	nrot = new int[mx.rows()+1];

	m = mx.rows();
	n = mx.cols();

	U.SetDimension(m,n);
	D.SetDimension(n,n);D.Zero();
	V.SetDimension(n,n);

	// copy data
	for (j=0; j<mx.rows(); j++)
		for (i=0; i<mx.cols(); i++)
			a[j+1][i+1] = (double)mx(j,i);
	// compute svd
	svdcmp(a,m,n,d,v);

	for (i=0; i<n; i++){
		D(i,i) = d[i+1];
		for (j=0; j<m; j++)
		{
			U(j,i) = (Tp)(a[j+1][i+1]);
		}
		for (j=0; j<n; j++)
		{
			V(j,i) = (Tp)(v[j+1][i+1]);
		}
	}

	// free memory
	for (j=0; j<mx.rows(); j++)
	{
		delete []a[j];
		delete []v[j];
	}
	delete []a;
	delete []d;
	delete []v;
}


#endif
