#ifndef _SK_MATRIX_IPL_H_
#define _SK_MATRIX_IPL_H_

#include "cv.h"
#include "highgui.h"

#include "skMatrix.h"
#include "skTemplate.h"

///////////////////
// readMatrix()
template<class T>
int skReadMatrix(const char* filename, Matrix<T> &M)
{
	int i, j;
	FILE *fp;
	int rows, cols;

	if((fp = fopen(filename, "r"))!=NULL)
	{
		fscanf(fp, "%d %d\n", &rows, &cols);
		M.SetDimension(rows, cols);
		for(i=0; i<rows; i++) 
		{
			for(j=0; j<cols; j++)
			{
				double p;
				fscanf(fp, "%lf ", &p);
				M(i, j) = (T)p;
			}
			fscanf(fp, "\n");
		}
	    fclose(fp);
		return 0;
	}

	return -1;
}

///////////////////
// saveMatrix()
template<class T>
int skSaveMatrix(const char* filename, Matrix<T> &M)
{
	int i, j;
	FILE *fp;
	int rows, cols;

	rows = M.rows();
	cols = M.cols();
	if((fp = fopen(filename, "w"))!=NULL)
	{
		fprintf(fp, "%d %d\n", rows, cols);
		for(i=0; i<rows; i++) 
		{
				for(j=0; j<cols; j++)
				{
					fprintf(fp, "%.5lf ", M(i, j));
				}
				fprintf(fp, "\n");
		}
	    fclose(fp);
		return 0;
	}

	return -1;
}

///////////////////
// skNormalizeMatrix2ZeroMean1Std()
//	find dMean and dStd which is the mean and standard deviation of the image, 
//	then normalize to zero mean and variance
template<class T>
int skNormalizeMatrix2ZeroMean1Std(Matrix<T> *F, double &dMean, double &dStd)
{

	int nBound = 0;

	int h = F->rows();
	int w = F->cols();

	if (h <= nBound || w <= nBound)
		nBound = 0;

	float p;
	int i, j;
	double s = 0;
	for(i = nBound; i < h-nBound; i++)
	{
		for(j = nBound; j < w-nBound; j++)
		{
			s += (*F)(i,j);
		}
	}
	s /= (h-nBound*2)*(w-nBound*2);
	double std = 0;
	for(i = nBound; i < h-nBound; i++)
	{
		for(j = nBound; j < w-nBound; j++)
		{
			std += ((*F)(i, j) - s) * ((*F)(i, j) - s);
		}
	}
	std = sqrt( std/((h-nBound*2)*(w-nBound*2)-1) );//variance

	//update values, all of them
	for(i = 0; i < h; i++)
	{
		for(j = 0; j < w; j++)
		{
			p = (float)(((*F)(i, j) - s)/std);
			(*F)(i, j) = p;
		}
	}
	dMean = s;
	dStd = std;
	return 0;
}
///////////////////
// skOffsetAndScaleMatrix()
//		new_value = (old_value - dMean) / dStd;
template<class T> 
int skOffsetAndScaleMatrix( Matrix<T> *F, double dMean, double dStd)
{
	int h = F->rows();
	int w = F->cols();

	int i, j;
	for(i = 0; i < h; i++)
	{
		for(j = 0; j < w; j++)
		{
			(*F)(i,j) = ((*F)(i,j) - dMean)/dStd;
		}
	}
	return 0;
}

///////////////////
// skGetMatrixMeanAndStd()
//		find dMean and dStd which is the mean and standard deviation of the image
template<class T>
int skGetMatrixMeanAndStd(Matrix<T> *F, double &dMean, double &dStd)
{
	int h = F->rows();
	int w = F->cols();

	float p;
	int i, j;
	double s = 0;
	for(i = 0; i < h; i++)
	{
		for(j = 0; j < w; j++)
		{
			s += (*F)(i,j);
		}
	}
	s /= (h*w);
	double std = 0;
	for(i = 0; i < h; i++)
	{
		for(j = 0; j < w; j++)
		{
			std += ((*F)(i, j) - s) * ((*F)(i, j) - s);
		}
	}
	std = sqrt( std/(h*w) );//variance
	
	dMean = s;
	dStd = std;

	return 0;
}

///////////////////
// skNormalizeMatrix2ExpectedMeanStd()
template<class T>
int skNormalizeMatrix2ExpectedMeanStd(Matrix<T> &M, double dExpMean, double dExpStd)
{
	double dMean, dStd;
	skGetMatrixMeanAndStd(&M, dMean, dStd);

	double dTranStd = dStd/dExpStd;
	double dTranMean = dMean - dExpMean*dTranStd;

	skOffsetAndScaleMatrix(&M, dTranMean, dTranStd);

	double dMean_new, dStd_new;
	skGetMatrixMeanAndStd(&M, dMean_new, dStd_new);


	return 0;
}
									 
///////////////////
// skIpl2Matrix()
//	any ipl image to Matrix
template<class T>
int skIpl32F2Matrix(IplImage *I, Matrix<T> *&M)
{
	int i,j,h=I->height,w=I->width;
	float f;
	if (M == 0)
		M = new Matrix<T>;

	M->SetDimension(h,w);
	
	CvScalar pixel;

	for(i=0;i<w;i++)
		for(j=0;j<h;j++)
		{
			//iplGetPixel(I,i,j,&f);			
			pixel = cvGet2D(I, j, i);

			(*M)(j,i)=(T)pixel.val[0];
		}
	return 0;
}

///////////////////
// skFMatrix2Ipl32FImage()
//	Matrix to ipl 32 float
template<class T>
int skMatrix2Ipl32FImage(Matrix<T> *F, IplImage *&I)
{
	int wd = F->cols();
	int ht = F->rows();
	if (I == 0)
	{
		I = cvCreateImage(cvSize(wd, ht), IPL_DEPTH_32F, 1);
	}
	else
	{
		if (I->depth != IPL_DEPTH_32F || I->width != wd || I->height != ht)
		{
			cvReleaseImage(&I);

			I = cvCreateImage(cvSize(wd, ht), IPL_DEPTH_32F, 1);
		}
	}
	float p;
	for(int i = 0; i < ht; i++)
	{
		for(int j = 0; j < wd; j++)
		{
			p = (*F)(i, j);

			cvSet2D(I, i, j, cv::Scalar(p));
			//iplPutPixel(I, j, i, &p);
		}
	}
	return 0;
}

///////////////////
// skScaleIplFP()
//	fMin, fMax - min and max value after scaling, for saving image they should be 0, 255
//	if dStd > 0 then use mean and std provided to scale up the image
//		new_value = old_value*dStd + dMean
//	otherwise not use provided, just find the min and max in the image and scale them to (fMin, fMax)
//		new_value = (old_value-min)*(fMax-fMin)/(max-min) + fMin;
inline int skScaleIplFP(IplImage *I, float fMin, float fMax, double dMean, double dStd)
{
	assert(I->depth == IPL_DEPTH_32F);
	bool bUseMeanStd;
	if (dStd > 0)
		bUseMeanStd = true;
	else
		bUseMeanStd = false;

	int h = I->height;
	int w = I->width;
	int i,j;
	float f=0;
	
	double max = 0;
	double min = 0;
	
	cv::Point minLoc, maxLoc;


	cv::Mat img(I);
	cv::minMaxLoc(img, &min, &max, &minLoc, &maxLoc);

	f = img.at<float>(0, 0);

	if (max > min)
	{
		if ( !bUseMeanStd )
		{
			cv::normalize(img, img, fMax, fMin, CV_MINMAX);
		}
		else
		{
			cv::Mat imgScaled;
			img.convertTo(imgScaled, img.type(), dStd, dMean);

			cv::threshold(imgScaled, imgScaled, fMax, fMax, cv::THRESH_TRUNC);
	

			imgScaled *= -1;

			cv::threshold(imgScaled, imgScaled, -fMin, -fMin, cv::THRESH_TRUNC);

			imgScaled *=-1;		

			imgScaled.copyTo(img);

		}

	}
	else 
	{
		if (!bUseMeanStd)
			img = cv::Scalar((fMin+fMax)/2);			
		else
		{
			f = (float)(f * dStd + dMean);
			img = cv::Scalar(f);		
		}
	}
	return 0;
}
///////////////////
// skConvertIpl32F2Ipl8U()
//	skConvert IPL 32F float to IPL 8U
inline int skConvertIpl32F2Ipl8U(IplImage *I, IplImage *J)
{
	assert(I->depth == IPL_DEPTH_32F);
	assert(J->depth == IPL_DEPTH_8U);
	assert(I->width == J->width);
	assert(I->height == J->height);
	
	cv::Mat img8U(J);
	cv::Mat img32F(I);

	img32F.convertTo(img8U, CV_32FC(img32F.channels()));

	return 0;
}

///////////////////
// skConvertIpl8U2Ipl32F
//	skConvert IPL 8U float to IPL 32F
inline int skConvertIpl8U2Ipl32F(IplImage *I, IplImage *J)
{
	assert(I->depth == IPL_DEPTH_8U);
	assert(J->depth == IPL_DEPTH_32F);
	assert(I->width == J->width);
	assert(I->height == J->height);
	
	cv::Mat img8U(I);
	cv::Mat img32F(J);

	img8U.convertTo(img32F, CV_32FC(img8U.channels()));

	return 0;
}
///////////////////
// skUpsideDownIplImage()
inline int skUpsideDownIplImage(IplImage *I)
{
	if (I == 0)
		return -1;
	
	cv::Mat img(I);
	cv::flip(img, img, 1);
	return 0;
}
///////////////////
// skUpsideDownSkMatrix()
template<class T>
int skUpsideDownSkMatrix(Matrix<T> &M)
{
	int w, h;
	M.GetDimension(h, w);

	T p0, p1;
	int i, j;
	for(i = 0; i < h/2; i++)
	{
		for(j = 0; j < w; j++)
		{
			p0 = M(i, j);
			p1 = M(h-i-1, j);
			M(i, j) = p1;
			M(h-i-1, j) = p0;
		}
	}
	return 0;
}

///////////////////
// skSaveMatrixAsBitmap()
//  save a matrix to a bitmap
//	pFileName - file name
//	M - is supposed to be zero mean and one variance gray lelvel image matrix
//	dImgMean, dImgStd - the the mean and std of original image
//	dImgMean and dImgStd are given to scale up the image
//	if dImgStd < 0, then find the max and min in the image and scale it up to 0~255
//	bottom-left origin
template<class T>
int skSaveMatrixAsBitmap(const char *pFileName, Matrix<T> &M, double dImgMean, double dImgStd)
{	
	IplImage *I = 0;
	skMatrix2Ipl32FImage(&M, I);
	skScaleIplFP(I, 0, 255, dImgMean, dImgStd);
	
	cv::Mat img(I);

	cv::Mat imgSave;

	img.convertTo(imgSave, CV_8UC(img.channels()));

	imwrite(pFileName, imgSave);
		
	return 0;
}

///////////////////
// skReadBitmap()
//  read a bitmap to a matrix
//	pFileName - file name
template<class T>
int skReadBitmap(const char *pFileName, Matrix<T> *pM)
{
	cv::Mat image( cv::imread(pFileName, -1) );
	if (image.empty())
	{
		printf("The image file %s is not found.\n", pFileName);
		return -1;
	}

	int w = image.cols;
	int h = image.rows;
	IplImage *I = cvCreateImage( cvSize(w, h), IPL_DEPTH_32F, 1);

	IplImage orig = image;

	skConvertIpl8U2Ipl32F(&orig, I);

	skIpl32F2Matrix(I, pM);

	cvReleaseImage(&I);
	
	return 0;
}



///////////////////
// skNumOfValueInMatrix
template<class T>
int skNumOfValueInMatrix(Matrix<T> &M, float v)
{
	int n = 0;
	int wd, ht;
	M.GetDimension(ht, wd);
	float error = 0.0001;
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (fabs(M(r,c)-v) < error)
				n++;
		}
	}
	return n;
}

///////////////////
// skNumOfValueInMatrix
template<class T>
int skNumOfSmallerValueInMatrix(Matrix<T> &M, float v)
{
	int n = 0;
	int wd, ht;
	M.GetDimension(ht, wd);
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (M(r,c) < v) 
				n++;
		}
	}
	return n;
}
///////////////////
// skNumOfValueInMatrix
template<class T>
int skReplaceValueInMatrix(Matrix<T> &M, float v, float new_v, float error)
{
	int n = 0;
	int wd, ht;
	M.GetDimension(ht, wd);
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (fabs(M(r,c)-v) < error)
				M(r,c) = new_v;
		}
	}
	return n;
}

///////////////////
// reutrn the square error change
//	skSubtractImageAndReturnSquareSumChange()
template<class T>
double skSubtractImageAndReturnSquareSumChange(Matrix<T> *M0,//orginal image, subtract from
									  Matrix<T> *M, //image subtracted by 
									  int dx, int dy,//offset, M relative to M0
									  int nBound)
{
	int wd0, ht0, wd, ht;
	M->GetDimension(ht, wd);
	M0->GetDimension(ht0, wd0);

	double dErrorChg = 0;
	int x, y, x0, y0;
	for (y = 0; y < ht; y++)
	{
		y0 = y + dy;
		if (y0 < nBound || y0 >= ht0-nBound)
			continue;
		for (x = 0; x < wd; x++)
		{
			x0 = x + dx;
			if (x0  < nBound || x0 >= wd0-nBound)
				continue;
			if ((*M)(y,x) == 0)
				continue;
			dErrorChg -= (*M0)(y0, x0) * (*M0)(y0, x0);
			(*M0)(y0, x0) -= (*M)(y,x);
			dErrorChg += (*M0)(y0, x0) * (*M0)(y0, x0);

		}
	}
	return dErrorChg;
}

// reutrn the square error change
//	skSubtractImageAndReturnSquareSumChange()
template<class T>
double skSubtractImageAndReturnSquareSumChangeWithInhibition(Matrix<T> *M0,//orginal image, subtract from
									  Matrix<T> *M, //image subtracted by 
									  int dx, int dy,//offset, M relative to M0
									  int nBound)
{
	int wd0, ht0, wd, ht;
	M->GetDimension(ht, wd);
	M0->GetDimension(ht0, wd0);

	double dErrorChg = 0;
	int x, y, x0, y0;
	for (y = 0; y < ht; y++)
	{
		y0 = y + dy;
		if (y0 < nBound || y0 >= ht0-nBound)
			continue;
		for (x = 0; x < wd; x++)
		{
			x0 = x + dx;
			if (x0  < nBound || x0 >= wd0-nBound)
				continue;
			if ((*M)(y,x) == 0)
				continue;
			if ((*M0)(y0, x0) == 0)
				continue;
			dErrorChg -= (*M0)(y0, x0) * (*M0)(y0, x0);
			(*M0)(y0, x0) -= (*M)(y,x);
			dErrorChg += (*M0)(y0, x0) * (*M0)(y0, x0);
			(*M0)(y0, x0) = 0;

		}
	}
	return dErrorChg;
}


///////////////////
//	skCopyImage()
//	copy one image M0 to the other image M
//	with offset (dx, dy)
template<class T>
double skCopyImage(Matrix<T> *M0,//orginal image,
				   Matrix<T> *M, //new image
				  int dx, int dy)//offset, M relative to M0
{
	int wd0, ht0, wd, ht;
	M0->GetDimension(ht0, wd0);
	M->GetDimension(ht, wd);
//	M->Zero();

	int x, y, x0, y0;
	for (y = 0; y < ht; y++)
	{
		y0 = y + dy;
		if (y0 < 0 || y0 >= ht0)
			continue;
		for (x = 0; x < wd; x++)
		{
			x0 = x + dx;
			if (x0  < 0 || x0 >= wd0)
				continue;
			(*M)(y,x) = (*M0)(y0, x0);
		}
	}
	return 0;
}

///////////////////
//	skCopyImage()
//	copy one image to the other
//	with offset (dx, dy)
template<class T>
double skCopyImagePart(Matrix<T> *M0,//orginal image, copy from
				   Matrix<T> *M, //new image
				  int dx, int dy,//offset, M relative to M0
				  int x_s, int x_e,//start and end of x in M, [x_s,x_e], close interval, in M
				  int y_s, int y_e)//start and end of y in M, [y_s,y_e], close interval, in M
{
	int wd0, ht0, wd, ht;
	M0->GetDimension(ht0, wd0);
	M->GetDimension(ht, wd);
//	M->Zero();


	int x, y, x0, y0;
	for (y = y_s; y <= y_e; y++)
	{
		y0 = y + dy;
		if (y0 < 0 || y0 >= ht0)
			continue;
		for (x = x_s; x <= x_e; x++)
		{
			x0 = x + dx;
			if (x0  < 0 || x0 >= wd0)
				continue;
			if (y>= 0 && y < ht && x >= 0 && x < wd)
				(*M)(y,x) = (*M0)(y0, x0);
		}
	}
	return 0;
}
///////////////////
// skSquareSumOfMatrix()
//	get the square sum of the matrix
//  the pixel in the boundary range is not considerred
template<class T>
double skSquareSumOfMatrix(Matrix<T> *M,  int nBound)
{
	int wd, ht;
	M->GetDimension(ht, wd);

	double dSum = 0;
	int x, y;
	for (y = nBound; y < ht-nBound; y++)
	{
		for (x = nBound; x < wd-nBound; x++)
		{
			dSum += (*M)(y, x) * (*M)(y, x);
		}
	}
	return dSum;
}

///////////////////
// skInterpolation()
template<class T>
int skInterpolation( Matrix<T> *pCon,//the constrainted image, 0-fixed pixel, other-should be changed
						    Matrix<float> *pSyn)//synthesized image
{
	int wd, ht;
	pSyn->GetDimension(ht, wd);

	int r, c;

	double dMaxError = 0.0001;
	double dError = 0.02;
	float fOldValue;
	double dKeep = 0.7;

	int nHalf = 1;
	int dr, dc, r1, c1;
	int n;
	double dSum;
	while (dError > dMaxError)
	{
		dError = 0;
		
		for (r = 0; r < ht; r++)
		{
			for (c = 0; c < wd; c++)
			{
				n = 0;
				dSum = 0;
				if (fabs(double((*pCon)(r,c))) >= 0.001)
				{
					for (dr = -nHalf; dr <= nHalf; dr++)
					{
						r1 = r + dr;
						if (r1 < 0 || r1 >= ht)
							continue;
						for (dc = -nHalf; dc <= nHalf; dc++)
						{
							c1 = c + dc;
							if (c1 < 0 || c1 >= wd)
								continue;
							dSum += (*pSyn)(r1, c1);
							n++;
						}
					}
					if(n > 0)
					{
						fOldValue = (*pSyn)(r,c);
						(*pSyn)(r,c) = dSum / n;
						dError += fabs(fOldValue - (*pSyn)(r,c));
					}
					else
						(*pSyn)(r,c) = 0;
				}
			}
		}
	}
	return 0;
}

///////////////////
// skScaleMatrix()
//	scale the matrix by a factor of s
template<class T>
int skScaleMatrix(Matrix<T> *pOrg, Matrix<T> *pNew, int s)
{
	int wd, ht, wd1, ht1;
	pOrg->GetDimension(ht, wd);
	pNew->GetDimension(ht1, wd1);

	int wd_new = wd*s, ht_new=ht*s;

	if (wd1 != wd_new || ht1 != ht_new)
		pNew->SetDimension(ht_new, wd_new);

	if (s == 1)
	{
		*pNew = *pOrg;
		return 0;
	}

	int r, c, dr, dc;
	double value;
	if (s > 1)
	{
		//       gv1           
		//   gh1     gh2
		//		 gv2
		// gh=(h2*gh1+h1*gh2)/(h1+h2)
		// gv=(v2*gv1+v1*gv2)/(v1+v2)
		// g= (h*gv+v*gh)/(h+v)
		// here h1=dc, h2=s-dc
		//		v1=dr, v2=s-dr
		//	gv1=((r,c)*(s-dc)+(r,c+1)*dc)/s 
		//	etc
		double gv1, gv2, gh1, gh2, gv, gh, g;
		double v00, v01, v10, v11;
		for (r = 0; r < ht; r++)
		{
			for (c = 0; c < wd; c++)
			{
				for (dr = 0; dr < s; dr++)
				{
					for (dc = 0; dc < s; dc++)
					{
						if (r < ht-1 && c < wd-1)
						{
							v00 = (*pOrg)(r,c);
							v01 = (*pOrg)(r,c+1);
							v10 = (*pOrg)(r+1,c);
							v11 = (*pOrg)(r+1,c+1);

						}
						else 
						{
							v00 = (*pOrg)(r,c);
							if (c == wd-1)
								v01 = v00 + v00 - (*pOrg)(r,c-1);
							else
								v01 = (*pOrg)(r,c+1);
							if (r == ht-1)
								v10 = v00 + v00 - (*pOrg)(r-1,c);
							else
								v10 = (*pOrg)(r+1,c);
							if (c == wd-1)
								v11 = v10 + v10 - (*pOrg)(r+1,c);
							else
								v11 = v01 + v01 - (*pOrg)(r-1,c+1);
						}
						gv1 = (v00 * (s-dc) + v01*dc)/(s*1.0);
						gv2 = (v10 * (s-dc) + v11*dc)/(s*1.0);
						gh1 = (v00 * (s-dr) + v10*dr)/(s*1.0);
						gh2 = (v01 * (s-dr) + v11*dr)/(s*1.0);
						gv = (gv1*(s-dr) + gv2*dr)/(s*1.0);
						gh = (gh1*(s-dc) + gh2*dc)/(s*1.0);

						g = (gv+gh)/2.0;
						(*pNew)(r+dr,c+dc) = (float)g;
					}
				}
			}
		}
	}
	else
	{
		assert(false);//not implement yet
	}
	return 0;
}

///////////////////
// skDrawSeperatedLinesAndNoOccupiedBlocks()
template<class T>
int skDrawSeperatedLinesAndNoOccupiedBlocks(Matrix<T> &mat,//the matrix
											T line_value,//the value set to be the lines
											T nocover_value,
											int nRow, int nCol,//rows and cols of seperated icons/blocks
											int nTotalNum,//total number of blocks
											int size, //block size
											int space)// space between blocks
{
	int ht, wd;
	mat.GetDimension(ht, wd);
	// draw seperate lines
	int x1, y1, x2, y2;
	int i;
	int r, c;
	for (i = 0; i <= nRow; i++)
	{
		x1 = 0; 
		x2 = wd;
		y1 = i * (size+space);
		y2 = y1 + space;
		for (r = y1; r < y2; r++)
			for (c = x1; c < x2; c++)
			{
				mat(r,c) = line_value;
			}
	}
	for (i = 0; i <= nCol; i++)
	{
		y1 = 0; 
		y2 = ht;
		x1 = i * (size+space);
		x2 = x1 + space;
		for (r = y1; r < y2; r++)
			for (c = x1; c < x2; c++)
			{
				mat(r,c) = line_value;
			}
	}
	// last several not used square set to blank
	int nBlank = nCol*nRow - nTotalNum;
	x2 = wd;
	x1 = x2 - nBlank*(size+space);
	y2 = ht;
	y1 = y2 - (size+space);
	for (r = y1; r < y2; r++)
		for (c = x1; c < x2; c++)
			{
				mat(r, c) = nocover_value;
			}
	return 0;
}
///////////////////
// skDrawSeperatedLinesAndNoOccupiedBlocks()
template<class T>
int skHistogramEqualization(Matrix<T> &mat,//the matrix
							int nBinNum)
{
	assert(nBinNum > 1);
	int ht, wd;
	mat.GetDimension(ht, wd);

	double dMax = mat.GetMax();
	double dMin = mat.GetMin();

	double dStep = (dMax-dMin)/(nBinNum-1);
	int *pnHist = new int[nBinNum];
	int *pnNewBin = new int[nBinNum];
	memset(pnHist, 0, sizeof(int)*nBinNum);

	double dValue;
	int nBin;
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			dValue = mat(r,c);
			nBin = (int)((dValue - dMin)/dStep);
			assert(nBin >= 0 && nBin < nBinNum);
			pnHist[nBin]++;
		}
	}

	int nSum = 0;
	int nNewBin;
	for (nBin = 0; nBin < nBinNum; nBin++)
	{
		nNewBin = (int)(nSum * nBinNum / (double)(wd*ht));
		nSum += pnHist[nBin];
		pnNewBin[nBin] = nNewBin;
	}

	int tmp[1024];
	for (int i = 0; i < 1024; i++)
	{
		tmp[i] = pnHist[i];
	}
	for (i = 0; i < 1024; i++)
	{
		tmp[i] = pnNewBin[i];
	}
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			dValue = mat(r,c);
			nBin = (int)((dValue - dMin)/dStep);
			assert(nBin >= 0 && nBin < nBinNum);
			dValue = dMin + pnNewBin[nBin] * dStep;
			mat(r,c) = dValue;
		}
	}

	delete []pnHist;
	delete []pnNewBin;

	return 0;
}

template<class T>
double skResetImageGrayLevel(Matrix<T> &img, int nLevels)
{
	double dScale = 256.0/nLevels;
	int ht, wd;
	img.GetDimension(ht, wd);
	for (int r = 0; r < ht; r++)
	{
		for (int c = 0; c < wd; c++)
		{
			img(r,c) = (int)(img(r,c)/dScale+0.5);
		}
	}
	return dScale;
}



template<class T>
double interpolationValue(Matrix<T> &mat, double x, double y, double out_region_value)
{
	int wd, ht;
	mat.GetDimension(ht, wd);

	if (x < 0 || x > wd-1 || y < 0 || y > ht-1)
		return out_region_value;

	int x1 = (int)x;
	int x2 = x1 + 1;
	int y1 = (int)y;
	int y2 = y1 + 1;

	int xx[4], yy[4];
	xx[0] = x1; yy[0] = y1;
	xx[1] = x1; yy[1] = y2;
	xx[2] = x2; yy[2] = y1;
	xx[3] = x2; yy[3] = y2;
	for (int i = 0; i < 4; i++)
	{
		if (x-xx[i] == 0 && y-yy[i] == 0)
		{
			return mat(yy[i], xx[i]);
		}
	}

	if (x1 < 0 || x1 >= wd || y1 <0 || y1 >= ht)
		return out_region_value;
	if (x2 < 0 || x2 >= wd || y2 <0 || y2 >= ht)
		return out_region_value;


	double d[4];
	double sum = 0, d_sum = 0;
	double inv_d;
	for (int i = 0; i < 4; i++)
	{
		d[i] = sqrt((x-xx[i])*(x-xx[i]) + (y-yy[i])*(y-yy[i]));
		if (d[i] == 0)
			return mat(yy[i], xx[i]);
		inv_d = 1.0/d[i];
		d_sum += inv_d;
		sum += mat(yy[i], xx[i]) * inv_d;
	}
	return sum/d_sum;
}
template<class T>
T nearestValue(Matrix<T> &mat, double x, double y)
{
	int wd, ht;
	mat.GetDimension(ht, wd);

	int x1 = (int)x;
	int x2 = x1 + 1;
	int y1 = (int)y;
	int y2 = y1 + 1;

	if (x1 < 0 || x1 >= wd || y1 <0 || y1 >= ht)
		return -10000;
	if (x2 < 0 || x2 >= wd || y2 <0 || y2 >= ht)
		return -10000;

	int xx[4], yy[4];
	xx[0] = x1; yy[0] = y1;
	xx[1] = x1; yy[1] = y2;
	xx[2] = x2; yy[2] = y1;
	xx[3] = x2; yy[3] = y2;

	double d[4];
	double min_d = wd + ht;
	T min_v;
	for (int i = 0; i < 4; i++)
	{
		d[i] = sqrt((x-xx[i])*(x-xx[i]) + (y-yy[i])*(y-yy[i]));
		if (min_d > d[i])
		{
			min_d = d[i];
			min_v = mat(yy[i], xx[i]);
		}
	}
	return min_v;
}
template<class T>
T meanValueOf4Neighbor(Matrix<T> &mat, double x, double y)
{
	int wd, ht;
	mat.GetDimension(ht, wd);

	int x1 = (int)x;
	int x2 = x1 + 1;
	int y1 = (int)y;
	int y2 = y1 + 1;

	if (x1 < 0 || x1 >= wd || y1 <0 || y1 >= ht)
		return -10000;
	if (x2 < 0 || x2 >= wd || y2 <0 || y2 >= ht)
		return -10000;

	int xx[4], yy[4];
	xx[0] = x1; yy[0] = y1;
	xx[1] = x1; yy[1] = y2;
	xx[2] = x2; yy[2] = y1;
	xx[3] = x2; yy[3] = y2;

	T m = mat(yy[0], xx[0]);//min
	for (int i = 1; i < 4; i++)
	{
		if (m > mat(yy[i], xx[i]) )
			m = mat(yy[i], xx[i]);
	}
	return m;

}
template<class T1, class T2>
int skCopyMatrix(Matrix<T1> &src, Matrix<T2> &dst)
{
	int wd, ht;
	src.GetDimension(ht, wd);
	dst.SetDimension(ht, wd);

	int x, y;
	for ( y = 0; y < ht; y++)
	{
		for (x = 0; x < wd; x++)
		{
			dst(y,x) = (T2)(src(y,x));
		}
	}

	return 0;
}
template<class T>
bool skMatrixHasNegativeElements(Matrix<T> &M)
{
	int wd, ht;
	M.GetDimension(ht, wd);

	int x, y;
	for ( y = 0; y < ht; y++)
	{
		for (x = 0; x < wd; x++)
		{
			if (M(y,x) < 0)
				return true;
		}
	}

	return false;
}
template<class T>
int skMedianFilter(Matrix<T> &M, int nHalf)
{
	int wd, ht;
	M.GetDimension(ht, wd);

	Matrix<T> M_bak;
	M_bak = M;

	int size = 2*nHalf + 1;
	T *data = new T[size*size];
	int x, y, dx, dy, x0, y0;
	int n;//valid point num
	for ( y0 = 0; y0 < ht; y0++)
	{
		for (x0 = 0; x0 < wd; x0++)
		{
			n = 0;
			for (dy = -nHalf; dy <= nHalf; dy++)
			{
				y = y0 + dy;
				if (y < 0 || y >= ht)
					continue;
				for (dx = -nHalf; dx <= nHalf; dx++)
				{
					x = x0 + dx;
					if (x < 0 || x >= wd)
						continue;
					data[n] = M_bak(y,x);
					n++;
				}
			}
			::sort(n, data, 1);
			M(y0, x0) = data[n/2];
		}
	}

	delete []data;
	return 0;
}
///////////////////
// skChangeDirectBetweenLeftBottomCoordAndLeftTopCoord()
//	assume M is 0~180 in the left-bottom coordinate
//	change is to left top cooridate
template<class T>
int skChangeDirectBetweenLeftBottomCoordAndLeftTopCoord(Matrix<T> &M)
{
	int wd, ht;
	M.GetDimension(ht, wd);

	int x, y;
	for (y = 0; y < ht; y++)
	{
		for (x = 0; x < wd; x++)
		{
			if (M(y,x) != 0)
				M(y,x) = 180 - M(y,x);
		}
	}
	return 0;
}

///////////////////
// skGetMaxHistValue()
//	get the value with the max histogram
//	 assume the value between 0~255, other value doesn't count
inline int skGetMaxHistValue(Matrix<int> &M)
{
	int ht, wd;
	M.GetDimension(ht, wd);

	int hist[256];
	memset(hist, 0, sizeof(int) * 256);

	int v;
	int y, x;
	for (y = 0; y < ht; y++)
	{
		for (x = 0; x < wd; x++)
		{
			v = M(y, x);
			if (v < 0 || v > 255)
				continue;
			hist[v]++;
		}
	}

	int max_hist = 0;
	int max_v = -1;
	int i;
	for (i = 0; i < 256; i++)
	{
		if (max_hist < hist[i])
		{
			max_hist = hist[i];
			max_v = i;
		}
	}
	return max_v;
}


///////////////////
// skScaleImage()
// dScale -- down scale
// nDownScaleType -- 0
template<class T>
int skScaleImage(Matrix<T> &img, Matrix<T> &s_img, double dScale, int nDownScaleType)
{
	int wd, ht;
	img.GetDimension(ht, wd);

	int s_wd, s_ht;
	s_wd = (int)(wd / dScale + 0.5);
	s_ht = (int)(ht / dScale + 0.5);

	s_img.SetDimension(s_ht, s_wd);

	double r0, c0;
	int r, c;
	double dValue;
	double out_region_value = 0;

	if (nDownScaleType == 1)
	{//interpolation
		for (r = 0; r < s_ht; r++)
		{
			r0 = r*dScale;
			for (c = 0; c < s_wd; c++)
			{
				c0 = c*dScale;
				dValue = ::interpolationValue(img, c0, r0, out_region_value);
				s_img(r, c) = (T)dValue;
			}
		}
	}
	else if (nDownScaleType == 0)
	{//average
		int dr, dc;
		for (r = 0; r < s_ht; r++)
		{
			r0 = r*dScale;
			for (c = 0; c < s_wd; c++)
			{
				c0 = c*dScale;
				int nn = 0;
				double sum = 0;
				for (dr = 0; dr < dScale; dr++)
				{
					int r0_d = r0 + dr;
					if (r0_d >= ht)
						continue;
					for (dc = 0; dc < dScale; dc++)
					{
						int c0_d = c0 + dc;
						if (c0_d >= wd)
							continue;
						nn++;
						sum += img(r0_d, c0_d);
					}
				}
				if (nn > 0)
					dValue = sum / nn;
				else 
					dValue = out_region_value;
				s_img(r, c) = (T)dValue;
			}
		}
	}
	else
		assert(false);
	return 0;
}
// torus boundary
template<class T0>
Matrix<T0> skDirectiveX(Matrix<T0> &M0)
{
	int wd, ht;
	M0.GetDimension(ht, wd);
	Matrix<T0> M;
	M.SetDimension(ht, wd);

	int r, c;
	for (r = 0; r < ht; r++)
	{
		int r1 = r + 1;
		if (r1 >= ht)
			r1 -= ht;
		for (c = 0; c < wd; c++)
		{
			int c1 = c + 1;
			if (c1 >= wd)
				c1 -= wd;
			M(r,c) = M0(r,c1)-M0(r,c);
		}
	}
	return M;
}
template<class T0>
Matrix<T0> skDirectiveY(Matrix<T0> &M0)
{
	int wd, ht;
	M0.GetDimension(ht, wd);
	Matrix<T0> M;
	M.SetDimension(ht, wd);

	int r, c;
	for (r = 0; r < ht; r++)
	{
		int r1 = r + 1;
		if (r1 >= ht)
			r1 -= ht;
		for (c = 0; c < wd; c++)
		{
			int c1 = c + 1;
			if (c1 >= wd)
				c1 -= wd;
			M(r,c) = M0(r1,c)-M0(r,c);
		}
	}
	return M;
}

template<class T>
int skComputeDXHistogram(Matrix<T> &M0, double *pdHist, int bin_num)
{
	Matrix<T> DX;
	DX =  skDirectiveX(M0);

	memset(pdHist, 0, sizeof(double)*bin_num);

	int wd, ht;
	M0.GetDimension(ht, wd);

	int r, c;
	int dx;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			dx = DX(r,c);
			dx += bin_num/2;
			if (dx < 0)
				dx = 0;
			if (dx >= bin_num)
				dx = bin_num - 1;
			pdHist[dx]++;
		}
	}

	for (int i = 0; i < bin_num; i++)
	{
		pdHist[i] /= (wd*ht);
	}
	return 0;

}
template<class T>
double skDXEntropy(Matrix<T> &M0)
{
	double pdHist[512];
	skComputeDXHistogram(M0, pdHist, 512);
	double dEnt = skComputeEntropy(pdHist, 512);
	return dEnt;
}

template<class T>
Matrix<T> skSqureRootOfSqureSum(Matrix<T> &M1, Matrix<T> &M2)
{
	int wd1, ht1;
	M1.GetDimension(ht1, wd1);
	int wd2, ht2;
	M2.GetDimension(ht2, wd2);

	assert(wd1 == wd2 && ht1 == ht2);

	Matrix<T> M;
	M.SetDimension(ht1, wd1);

	int r, c;
	for (r = 0; r < ht1; r++)
	{
		for (c = 0; c < wd1; c++)
		{
			M(r,c) = sqrt(double(M1(r,c)*M1(r,c) + M2(r,c)*M2(r,c)));
		}
	}
	return M;
}

template<class T1, class T2>
int skGetRotatedPatch(Matrix<T1> &org, //org image
					  double dOrt,//rotated angel, in radius
					  int r0, int c0, //the center position 
					  Matrix<T2> &patch, // final patch
					  double invalid_value)
{
	int ht0, wd0, ht1, wd1;

	org.GetDimension(ht0, wd0);
	patch.GetDimension(ht1, wd1);


	int r1_c = ht1/2;
	int c1_c = wd1/2;

	double dValue;
	double dr1, dc1, dr0, dc0;
	int r1, c1;
	for (r1 = 0; r1 < ht1; r1++)
	{
		for (c1 = 0; c1 < wd1; c1++)
		{
			dr1 = r1 - r1_c;
			dc1 = c1 - c1_c;
			::rotate(dc1, dr1, dOrt);
			dr0 = r0 + dr1;
			dc0 = c0 + dc1;
			dValue = ::interpolationValue(org, dc0, dr0, invalid_value);
			patch(r1,c1) =  (T2)dValue;
		}
	}
	return c1_c;
}

template<class T1, class T2>
int skGetHalfRotatedPatch(Matrix<T1> &org, //org image
					  double dOrt,//rotated angel, in radius
					  int r0, int c0, //the center position 
					  Matrix<T2> &patch, // final patch
					  double invalid_value,
					  int end)//end indicator 0~left, 1-right
{
	int ht0, wd0, ht1, wd1;

	org.GetDimension(ht0, wd0);
	patch.GetDimension(ht1, wd1);


	int r1_c = ht1/2;

	int c1_c;
	if (end == 0)
	{
		c1_c = wd1-1;
	}
	else
	{
		c1_c = 0;
	}

	T2 dValue;
	double dr1, dc1, dr0, dc0;
	int r1, c1;
	for (r1 = 0; r1 < ht1; r1++)
	{
		for (c1 = 0; c1 < wd1; c1++)
		{
			dr1 = r1 - r1_c;
			dc1 = c1 - c1_c;
			::rotate(dc1, dr1, dOrt);
			dr0 = r0 + dr1;
			dc0 = c0 + dc1;
			dValue = ::interpolationValue(org, dc0, dr0, invalid_value);
			patch(r1,c1) =  dValue;
		}
	}
	return c1_c;
}

template<class T>
int skGetMatrixRowAverage(Matrix<T> &org,
					  Matrix<T> &rowAvg)
{
	int wd, ht;
	org.GetDimension(ht, wd);
	rowAvg.SetDimension(ht, 1);

	int r, c;
	for (r = 0; r < ht; r++)
	{
		T sum = 0;
		for (c = 0; c < wd; c++)
		{
			sum += org(r,c);
		}
		if (wd == 0)
			rowAvg(r, 0) = 0;
		else
			rowAvg(r, 0) =  sum/wd;

	}
	return 0;
}

template<class T>
int setSameValue4EachRow(Matrix<T> &mat,
					  Matrix<T> &rowValue)
{
	int wd, ht;
	mat.GetDimension(ht, wd);

	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			mat(r,c) = rowValue(r, 0);
		}
	}
	return 0;
}
template<class T>
int setSameValue4EachRow(Matrix<T> &mat,
					  Matrix<T> &rowValue, int start)
{
	int wd, ht;
	mat.GetDimension(ht, wd);

	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			mat(r,c) = rowValue(r+start, 0);
		}
	}
	return 0;
}

template<class T>
int skKeepRowsValid(Matrix<T> &mat, int v_start, int v_end, 
					  double invalid_value)
{
	int wd, ht;
	mat.GetDimension(ht, wd);

	int r, c;
	for (r = 0; r < v_start; r++)
	{
		for (c = 0; c < wd; c++)
		{
			mat(r,c) = invalid_value;
		}
	}
	for (r = v_end+1; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			mat(r,c) = invalid_value;
		}
	}
	return 0;
}


template<class T>
double skMultipleRowAverage(Matrix<T> &mat,
					  Matrix<T> &rowValue)
{
	double sum = 0;
	double dAmp = 0;

	int wd, ht;
	mat.GetDimension(ht, wd);

	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			sum += mat(r,c) * rowValue(r, 0);
		}
		dAmp += rowValue(r, 0) * rowValue(r, 0);
	}
	dAmp *= wd;
	dAmp = sqrt(dAmp);

	return sum/dAmp;
}

template<class T>
int skErrorDecreaseWithAverageModeling(Matrix<T> &mat,
					  Matrix<T> &rowValue)
{
	double dErrDecrease = 0;
	int wd, ht;
	mat.GetDimension(ht, wd);

	T v;
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			v = mat(r,c);
			dErrDecrease += v*v - (v-rowValue(r, 0))*(v-rowValue(r,0));
		}
	}
	return 0;
}
template<class T>
T skFindMaxPosition(Matrix<T> &mat,
					  int x1, int y1, int x2, int y2, int &r0, int &c0)
{
	int ht, wd;
	mat.GetDimension(ht, wd);
	T m = -1000000;
	r0 = -1;
	c0 = -1;

	int r, c;

	for (r = y1; r <= y2; r++)
	{
		for (c = x1; c <= x2; c++)
		{
			if (mat(r,c) > m)
			{
				m = mat(r,c);
				r0 = r;
				c0 = c;
			}
		}
	}

	assert(r0 >= 0);

	return m;
}

inline double skComputeEntropy(double *pdProb, int  n)
{
	double dEnt = 0;
	for (int i = 0; i < n; i++)
	{
		if (pdProb[i] > 0)
			dEnt += -pdProb[i]* log(pdProb[i])/log(2.0);
	}
	return dEnt;
}

template<class T>
T skInerProduct(Matrix<T> &M1, Matrix<T> M2)
{
	int ht1, wd1, ht2, wd2;

	M1.GetDimension(ht1, wd1);
	M2.GetDimension(ht2, wd2);

	assert(ht1 == ht2 && wd1 ==  wd2);

	T sum = 0;
	int r, c;
	for (r = 0; r < ht1; r++)
	{
		for (c = 0; c < wd1; c++)
		{		
			sum += M1(r,c) * M2(r,c);
		}
	}
	return sum;
}

// for each column, sum up squre, then get mean then sqrt
// don't take out the mean
template<class T>
int	skGetColumnSqrtMeanSquareSum(Matrix<T> &mat, double *pdData)
{
	int ht, wd;
	mat.GetDimension(ht, wd);
	int r, c;
	for (c = 0; c < wd; c++)
	{
		double dSum = 0;
		for (r = 0; r < ht; r++)
		{
			dSum += mat(r,c)*mat(r,c);
		}
		dSum = sqrt(dSum/ht);
		pdData[c] = dSum;
	}
	return 0;
}
// for each column, sum up squre, then get mean then sqrt
template<class T>
int	skGetColumnStd(Matrix<T> &mat, double *pdData)
{
	int ht, wd;
	mat.GetDimension(ht, wd);
	int r, c;
	for (c = 0; c < wd; c++)
	{
		double mean = 0;
		for (r = 0; r < ht; r++)
		{
			mean += mat(r,c);
		}
		mean /= ht;
		double sum = 0;
		for (r = 0; r < ht; r++)
		{
			sum += (mat(r,c)-mean)*(mat(r,c)-mean);
		}
		sum = sqrt(sum/ht);
		pdData[c] = sum;
	}
	return 0;
}
template<class T>
bool skNonNegativeElementMatrix(Matrix<T> &M)
{
	int ht, wd;
	M.GetDimension(ht, wd);

	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (M(r,c) < 0)
				return false;
		}
	}
	return true;
}
template<class T>
bool skMatrixHasMemeber(Matrix<T> &M, T v0)
{
	int ht, wd;
	M.GetDimension(ht, wd);

	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (M(r,c) == v0)
				return true;
		}
	}
	return false;
}
inline int skChangeDoubleRegionPoints2IntRegionPoints(double *pdX, double *pdY, int n, int *pnX, int *pnY)
{
	int nMinX=1000000, nMaxX=-1000000;
	int nMinY=1000000, nMaxY=-1000000;
	int x, y;
	int i;
	for (i = 0; i < n; i++)
	{
		x = (int)(pdX[i] + 0.5);
		y = (int)(pdY[i] + 0.5);
		if (nMinX > x)
			nMinX = x;
		if (nMaxX < x)
			nMaxX = x;
		if (nMinY > y)
			nMinY = y;
		if (nMaxY < y)
			nMaxY = y;
	}

	int wd = nMaxX - nMinX + 1;
	int ht = nMaxY - nMinY + 1;

	Matrix<int> M(ht, wd);
	M.Zero();

	for (i = 0; i < n; i++)
	{
		x = (int)(pdX[i] + 0.5);
		y = (int)(pdY[i] + 0.5);
		M(y-nMinY, x-nMinX) = 1;
	}

	// center part
	int r, c;
	for (r = 1; r < ht-1; r++)
	{
		for (c = 1; c < wd-1; c++)
		{
			if (M( r, c) == 0)
			if (M(r, c-1) + M(r, c+1) + M(r+1, c) + M(r-1, c) >=3)
				M(r,c) = 1;
		}
	}
	//four corners
	if (ht > 1 && wd > 1)
	{
		if (M(0,1) && M(1,0))
			M(0,0) = 1;
		if (M(0, wd-2) && M(1, wd-1))
			M(0, wd-1) = 1;
		if (M(ht-2,0) && M(ht-1,1))
			M(ht-1,0) = 1;
		if (M(ht-2, wd-1) && M(ht-1, wd-2))
			M(ht-1, wd-1) = 1;
	}
	else if (ht == 1 && wd > 1)
	{
		if (M(0,1))
			M(0,0) = 1;
		if (M(0,wd-2))
			M(0,wd-1) = 1;
	}
	else if (wd == 1 && ht > 1)
	{
		if (M(1,0))
			M(0,0) = 1;
		if (M(ht-2,0))
			M(ht-1,0) = 1;
	}
	// four boundary lines
	r = 0;
	for (c = 1; c < wd-1; c++)
	{
		if (ht >= 2)
		{
			if (M( r, c) == 0)
			if (M(r,c-1) && M(r,c+1) && M(r+1,c))
				M(r,c) = 1;
		}
		else
		{
			if (M( r, c) == 0)
			if (M(r,c-1) && M(r,c+1))
				M(r,c) = 1;
		}
	}

	r = ht-1;
	for (c = 1; c < wd-1; c++)
	{
		if (ht >= 2)
		{
			if (M( r, c) == 0)
			if (M(r,c-1) && M(r,c+1) && M(r-1,c))
				M(r,c) = 1;
		}
		else
		{
			if (M( r, c) == 0)
			if (M(r,c-1) && M(r,c+1))
				M(r,c) = 1;
		}
	}

	c = 0;
	for (r = 1; r < ht-1; r++)
	{
		if (wd >= 2)
		{
			if (M( r, c) == 0)
			if (M(r-1, c) && M(r+1, c) && M(r, c+1))
				M(r,c) = 1;
		}
		else
		{
			if (M( r, c) == 0)
			if (M(r-1, c) && M(r+1, c))
				M(r,c) = 1;
		}
	}

	c = wd-1;
	for (r = 1; r < ht-1; r++)
	{
		if (wd >= 2)
		{
			if (M( r, c) == 0)
			if (M(r-1, c) && M(r+1, c) && M(r, c-1))
				M(r,c) = 1;
		}
		else
		{
			if (M( r, c) == 0)
			if (M(r-1, c) && M(r+1, c))
				M(r,c) = 1;
		}
	}

	int m = 0;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (M(r,c))
			{
				pnX[m] = c+nMinX;
				pnY[m] = r+nMinY;
				m++;
			}
		}
	}
	return m;
}

template<class T1, class T2>
int skBlobNoneMaxSuppresion(Matrix<T1> &sth, Matrix<T2> &mx)
{//max: 0-valid
	int n = 0;
	int wd, ht;
	sth.GetDimension(ht, wd);
	mx.SetDimension(ht,wd);

	int r0, c0, r1, c1;
	int dr, dc;

	for (r0 = 0; r0 < ht; r0++)
	{
		for (c0 = 0; c0 < wd; c0++)
		{
			double dSth0 = sth(r0,c0);
			bool bBreak = false;
			for (dr = -1; dr <= 1; dr++)
			{
				for (dc = -1; dc <= 1; dc++)
				{
					if (dr == 0 && dc == 0)
						continue;
					r1 = r0 + dr;
					c1 = c0 + dc;
					if (r1 < 0 || r1 >= ht || c1 < 0 || c1 >= wd)
						continue;
					double dSth1 = sth(r1, c1);
					if (dSth1 >= dSth0)
					{
						bBreak = true;
						break;
					}
				}
				if (bBreak)
					break;
			}
			if (bBreak)
				mx(r0,c0) = 1;
			else
			{
				mx(r0,c0) = 0;
				n++;
			}
		}
	}
	return n;
}

//  mx is already found
//  we want to reduce the mx number by thresholding
//  return the reduced number
template<class T1, class T2>
int skTrimmingByThresholding(Matrix<T1> &sth, Matrix<T2> &mx, double dThresh)
{//max: 0-valid
	int dn = 0;
	int wd, ht;
	sth.GetDimension(ht, wd);

	int r0, c0, r1, c1;
	int dr, dc;

	for (r0 = 0; r0 < ht; r0++)
	{
		for (c0 = 0; c0 < wd; c0++)
		{
			double dSth0 = sth(r0,c0);
			if (dSth0 <= dThresh)
			{
				mx(r0,c0) = 1;
				dn++;
			}
		}
	}
	return dn;
}

template<class T1, class T2, class T3>
int skDrawBlobs(Matrix<T1>& pos, Matrix<T2> & sz, Matrix<T3> &dst)
{//pos: 0-valid
	int n = 0;
	int ht, wd;
	pos.GetDimension(ht, wd);

	dst.SetDimension(ht, wd);
	dst.Set(1);//0-valid, 1:invalid

	int r, c;

	int *pnX = new int[ht*wd];
	int *pnY = new int[ht*wd];

	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (pos(r,c) != 0)
				continue;
			n++;
			int rd = sz(r,c);
			int m = ::rasterize_pie(c, r, rd, pnX, pnY);
			for (int i = 0; i < m; i++)
			{
				int x = pnX[i];
				int y = pnY[i];
				if (x < 0 || x >= wd || y < 0 || y >= ht)
					continue;
				dst(y,x) = 0;
			}
		}
	}
	delete []pnX;
	delete []pnY;

	return n;
}
template<class T1, class T2, class T3>
int skCopyConstraintedImage(Matrix<T1> &org, Matrix<T2> &dst, Matrix<T3> &con)
{//con: 0-valid
	int wd, ht;
	org.GetDimension(ht, wd);
	
	int r, c;
	for ( r = 0; r < ht; r++)
	{
		for ( c = 0; c < wd; c++)
		{
			if (con(r,c) == 0)
			{
				dst(r,c) = org(r,c);
			}
		}
	}
	return 0;
}
// return mask =  0 pixel number
template<class T1, class T2>
int skComputeMaskedImageMeanStd(Matrix<T1> & mask, Matrix<T2> &mat, double &dMean, double &dStd)
{
	int ht, wd;
	mask.GetDimension(ht, wd);

	int r, c;
	int n = 0;

	dMean = 0; 
	dStd = 0;
	for (r = 0; r < ht; r++)
	{
		for ( c = 0; c < wd; c++)
		{
			if (mask(r,c) != 0)
				continue;
			n++;
			dMean += mat(r,c);
		}
	}
	if ( n == 0)
	{
		dMean = 0;
		dStd = -1;//invalid
		return n;
	}
	dMean /= n;

	for ( r = 0; r < ht; r++)
	{
		for ( c = 0; c < wd; c++)
		{
			if (mask(r,c) != 0)
				continue;
			dStd += (mat(r,c)-dMean)*(mat(r,c)-dMean);
		}
	}
	dStd /= n;
	dStd = sqrt(dStd);

	return n;
}
template<class T1, class T2>
int skCountValueInMatrix(T1 &mat, T2 v)
{
	int n = 0;
	int ht, wd;
	mat.GetDimension(ht, wd);

	int r, c; 
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (mat(r,c) == v)
				n++;
		}
	}
	return n;
}

template<class T1, class T2>
int skMarkPointsAsDots(Matrix<T1> &mat, T2 v)
{
	int n = 0;
	int ht, wd;
	mat.GetDimension(ht, wd);

	Matrix<T1> tmp;
	tmp = mat;

	int dr[4] = {-1, 0, 1, 0};
	int dc[4] = {0, -1, 0, 1};
	int r, c, r1, c1; 
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (tmp(r,c) == v)
			{
				for (int i = 0; i < 4; i++)
				{
					r1 = r + dr[i];
					c1 = c + dc[i];
					if (r1 < 0 || r1 >= ht || c1 < 0 || c1 >= wd)
						continue;
					mat(r1,c1) = v;
				}
				n++;
			}
				
		}
	}
	return n;
}
///////////////////
// return the number of points removed (set from valid to invalid)
// 0: marked (valid) points
template<class T1, class T2, class T3>
int skRemoveIsolatedPoints(Matrix<T1> &mat, T2 valid, T3 invalid)
{
	int ht, wd;
	mat.GetDimension(ht, wd);
	
	int dr[8] = {-1, 0, 1, -1, 0, 1, -1, 1};
	int dc[8] = {-1, -1, -1, 1, 1, 1, 0, 0};

	int n = 0;
	int r, c, r1, c1;
	for( r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (mat(r,c) == valid) 
			{
				bool bConnected = false;
				for (int i = 0; i < 8; i++)
				{
					r1 = r + dr[i];
					c1 = c + dc[i];
					if (r1 < 0 || r1 >= ht || c1 < 0 || c1 >= wd)
						continue;
					if (mat(r1, c1) == valid)
					{
						bConnected = true;
						break;
					}
				}
				if (!bConnected)
				{
					mat(r,c) = invalid;
					n++;
				}
			}//if valid
		}//for c
	}//for r
	return n;
}
template<class T1, class T2>
int skAddBoundary(Matrix<T1> &mat, int bound, T2 v)
{
	int ht, wd;
	mat.GetDimension(ht, wd);
	if (ht <= bound || wd <= bound)
	{
		mat.Set(v);
		return 1;
	}
	int r, c;
	//col boundary
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < bound; c++)
		{
			mat(r,c) = v;
		}
		for (c = wd-bound; c < wd; c++)
		{
			mat(r,c) = v;
		}
	}
	//row boundary
	for (c = 0; c < wd; c++)
	{
		for (r = 0; r < bound; r++)
		{
			mat(r,c) = v;
		}
		for (r = ht-bound; r < ht; r++)
		{
			mat(r,c) = v;
		}
	}
	return 0;
}

template<class T>
int skRotatAMatrix(Matrix<T> &M0, Matrix<T> &M1, double dOrt)
{
	int ht, wd;
	M0.GetDimension(ht, wd);
	int hf = ht/2;

	int xx,yy;
	double x, y;
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			x = c - hf;
			y = r - hf;
			::rotate(x, y, -dOrt);
			if (x >= 0) 
				xx = (int)(x+hf+0.5);
			else
				xx = (int)(x+hf-0.5);
			if (y >= 0) 
				yy = (int)(y+hf+0.5);
			else
				yy = (int)(y+hf-0.5);
			if (xx >= 0 && xx < wd && yy >= 0 && yy < ht)
				M1(r,c) = M0(yy,xx);
		}
	}
	return 0;
}

inline int skDrawRectangle(Matrix<int>* pImgChan, int nChannel, 
					double dl, double dt, double dr, double db)
{
	if (nChannel <= 0)
		return -1;
	int ht, wd;
	pImgChan[0].GetDimension(ht, wd);

	int l = (int)(dl + 0.5);
	int t = (int)(dt + 0.5);
	int r = (int)(dr + 0.5);
	int b = (int)(db + 0.5);

	if (l > r)
		::swap(l,r);
	if (t > b)
		::swap(t,b);

	if (l < 0) l = 0;
	if (l >= wd) l = wd - 1;
	if (r <= 1) r = 1;
	if (r > wd) r = wd;

	if (t < 0) t = 0;
	if (t >= ht) t = ht - 1;
	if (b <= 1) b = 1;
	if (b > ht) b = ht;


	int x, y;
	int c;
	// horizontal line
	for (x = l; x < r; x++)
	{
		y = t;
		pImgChan[0](y,x) = 0;
		for (c = 1; c < nChannel; c++)
			pImgChan[c](y,x) = 128;
		y = b-1;
		pImgChan[0](y,x) = 0;
		for (c = 1; c < nChannel; c++)
			pImgChan[c](y,x) = 128;
	}
	// vertical line
	for (y = t; y < b; y++)
	{
		x = l;
		pImgChan[0](y,x) = 0;
		for (c = 1; c < nChannel; c++)
			pImgChan[c](y,x) = 128;
		x = r-1;
		pImgChan[0](y,x) = 0;
		for (c = 1; c < nChannel; c++)
			pImgChan[c](y,x) = 128;
	}
	return 0;
}
template<class T>
int skRead2OvalArray(const char* name, oval_Array<T> &arr)
{
	FILE *pFile;
	pFile = fopen(name, "r");
	if (pFile == 0)
		return -1;

	arr.removeAll();

	double value;
	while (fscanf(pFile, "%lf", &value) != EOF)
	{
		T v = value;
		arr.add(v);
	}
	fclose(pFile);
	return 0;
}
template<class T1, class T2, class T3>
int skSetValueInMatrixMask(Matrix<T1> &M, Matrix<T2> &mask, T3 value)
{
	int ht, wd;
	M.GetDimension(ht, wd);
	int n = 0;
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (mask(r,c) != 0)
				continue;
			M(r,c) = value;
			n++;
		}
	}
	return n;
}

#endif
