#include "skNbrManager.h"
#include "inlineFunc.h"

///////////////////
// addNbr()
//	Add pNbr as the "nNbrType" nbr of pNode 
//	the position of nbr, if -1 then resort nbr
int SkNbrManager::addNbr(SkBaseNode *pNode, SkBaseNode *pNbr, int nNbrType, int n)
{
	SkOutNbr nbr;
	int nNbrID = pNbr->getID();
	double dDis;
	if (nNbrType == OUT_NBR || ADR_NBR)
		dDis = ::distanceBetweenPoints(pNode->getRow(), pNode->getCol(), pNbr->getRow(), pNbr->getCol());
	else if (nNbrType == CHD_NBR)
		dDis = ::distanceBetweenPoints(pNode->getRow()*2, pNode->getCol()*2, pNbr->getRow(), pNbr->getCol());
	else if (nNbrType == PRT_NBR)
		dDis = ::distanceBetweenPoints(pNode->getRow(), pNode->getCol(), pNbr->getRow()*2, pNbr->getCol()*2);
	else 
		assert(false);
	nbr.setNbrID(nNbrID);
	nbr.setCenterDis(dDis);
	nbr.setValid(true);
	nbr.setChecked(true);
	pNode->addNbr(nbr, nNbrType, n);
	return pNode->getNbrNum(nNbrType);
}

///////////////////
// constructParentChildNbrhood()
//	consider all the nodes in the child base map, search the nearest parent for each of them
//	return 0
int SkNbrManager::constructParentChildNbrhood(SkBaseMap* pMapPrt, SkBaseMap* pMapChd)
{
	// get image width 
	int nWdPrt = pMapPrt->getWidth();
	int nHtPrt = pMapPrt->getHeight();
	int nWdChd = pMapChd->getWidth();
	int nHtChd = pMapChd->getHeight();
	// get scale factor
	int nScale = (nWdChd-1)/nWdPrt + 1;
	// set parent-child relationship
	pMapPrt->setChildMap(pMapChd);
	pMapChd->setParentMap(pMapPrt);
	// find the parents of one child
	// consider all tokens

	SkBaseNode *pBaseNode;
	// remove all parents for the nodes in the child base map
	int nPos = pMapChd->getFirstPos();
	while (nPos >= 0)
	{
		pBaseNode = pMapChd->getNextNode(nPos);
		if (!pBaseNode->valid())
			continue;
		pBaseNode->removeAllNbrs(PRT_NBR);
	}
	// remove all children for the nodes in the parent base map
	nPos = pMapPrt->getFirstPos();
	while (nPos >= 0)
	{
		pBaseNode = pMapPrt->getNextNode(nPos);
		if (!pBaseNode->valid())
			continue;
		pBaseNode->removeAllNbrs(CHD_NBR);
	}
	// consider all the nodes in the child base map, search the parent for each of them
	nPos = pMapChd->getFirstPos();
	while (nPos >= 0)
	{
		pBaseNode = pMapChd->getNextNode(nPos);
		if (!pBaseNode->valid())
			continue;
		constructParentChildNbrhood(pBaseNode, pMapChd, pMapPrt, nScale);
	}
	return 0;
}
///////////////////
// constructParentChildNbrhood()
//	consider all the nodes in the child base map, search the nearest parent for each of them
//	return parents num
int SkNbrManager::constructParentChildNbrhood(SkBaseNode* pBaseNode, SkBaseMap* pMapChd, SkBaseMap* pMapPrt, int nScale)
{
	pBaseNode->removeAllNbrs(CHD_NBR);
	double dDis;//in the child image plane
	SkOutNbr prtNbr;// out nbr

	int nPrtID;
	int nWidth = pMapChd->getWidth();
	int nHeight = pMapChd->getHeight();

	double dMaxDis = 12;

	int nPos = pMapPrt->getFirstPos();
	SkBaseNode *pPrtNode;
	while (nPos >= 0)
	{
		pPrtNode = pMapPrt->getNextNode(nPos);
		if (!pPrtNode->valid())
			continue;
		nPrtID = pPrtNode->getID();
//		dDis = ::distanceBetweenPointsInAImage(pBaseNode->getRow(), pBaseNode->getCol(), pPrtNode->getRow()*nScale, pPrtNode->getCol()*nScale, nWidth, nHeight);
		dDis = ::distanceBetweenPoints(pBaseNode->getRow(), pBaseNode->getCol(), pPrtNode->getRow()*nScale, pPrtNode->getCol()*nScale);
		if (dDis > dMaxDis)
			continue;
		prtNbr.setNbrID(nPrtID);
		prtNbr.setCenterDis(dDis);
		pBaseNode->addNbr(prtNbr, PRT_NBR);//the prt nbr will be sorted when added
	}//while (nPos >= 0)
// set current node as child of its parents
	addNodeAsChildOfItsParents(pBaseNode, pMapPrt);
	return pBaseNode->getNbrNum(PRT_NBR);
}
///////////////////
// addNodeAsChildOfItsParents()
//	add the current node, as child node to its parent nodes
//	return parents num
int SkNbrManager::addNodeAsChildOfItsParents(SkBaseNode* pBaseNode, SkBaseMap* pMapPrt)
{
	int nPrtNum = pBaseNode->getNbrNum(PRT_NBR);
	SkOutNbr prtNbr, chdNbr;// out nbr
	int i, id;
	id = pBaseNode->getID();
	chdNbr.setNbrID(id);
	SkBaseNode *pPrtNode;
	for (i = 0; i < nPrtNum; i++)
	{
		prtNbr = pBaseNode->getNbr(i, PRT_NBR);
		pPrtNode = pMapPrt->getNodeAt(prtNbr.getNbrID());
		chdNbr.setCenterDis(prtNbr.getCenterDis());
		pPrtNode->addNbr(chdNbr, CHD_NBR);
	}
	return nPrtNum;
}
///////////////////
// findNearestNode()
//	Find the top nMaxOutNbrNum nodes which are close to point (x0, y0)
//	the nodes are ordered
//  return node num
int SkNbrManager::findNearestNode(SkBaseMap* pMap, int x0, int y0, SkBaseNode** pNodes, int nMaxOutNbrNum, double dMinCoef)
{
	double dDis;//in 
	SkOutNbr outNbr;// out nbr
	SkBaseNode tempNode;
	tempNode.setMaxNbrNum(nMaxOutNbrNum, OUT_NBR);
	int nID;
	int nWidth = pMap->getWidth();
	int nHeight = pMap->getHeight();
	int nPos = pMap->getFirstPos();
	SkBaseNode *pNode;
	while (nPos >= 0)
	{
		pNode = pMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		if (fabs(pNode->m_data.a) <= dMinCoef)
			continue;
		dDis = ::distanceBetweenPoints(pNode->getRow(), pNode->getCol(), y0, x0);
		nID = pNode->getID();
		outNbr.setNbrID(nID);
		outNbr.setCenterDis(dDis);
		tempNode.addNbr(outNbr, OUT_NBR);//the prt nbr will be sorted when added
	}//while (nPos >= 0)
	int num = tempNode.getNbrNum(OUT_NBR);
	int i;
	for (i = 0; i < num; i++)
	{
		nID = tempNode.getNbrIDAt(i, OUT_NBR);
		pNodes[i] = pMap->getNodeAt(nID);
	}
	return num;
}
///////////////////
// findCloseNodeIDs()
//	Find the node IDs which are close to point (x0, y0) within dMaxDis
//	the nodes are not ordered
//	return nbr num
int SkNbrManager::findCloseNodeIDs(int x0, int y0, int *pnNbrIDs, SkBaseMap *pBaseMap, double dMaxDis)
{
	int n = 0;
	int x, y;
	SkBaseNode *pNode;
	int  nPos =  pBaseMap->getFirstPos();
	double dDis;
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		if (pNode->getAtomID() >= 0)//only search nodes not clustered yet
			continue;
		x = pNode->getCol();
		y = pNode->getRow();

		dDis = ::distanceBetweenPoints(y, x, y0, x0);
		if (dDis <= dMaxDis)
		{
			pnNbrIDs[n] = pNode->getID();
			n++;
		}
	}
	return n;
}

///////////////////
// constructNbrhood()
//	construct the out-in nbrhood for all the bases in pBaseMap,
//	the out nbr num is limited to 'nMaxOutNbrNum'
//	if nPartial==1 then only consider bases with partial order, 
//	which means only consider those bases with (x,y) s.t. y>y0 || (y==y0&&x>x0)
//	return 0
int SkNbrManager::constructNbrhood(SkBaseMap* pBaseMap, int nMaxOutNbrNum, int nPartial)
{
	// try to find pNode as pNode0's nbrs
	SkOutNbr outNbr;
	int nPos;
	SkBaseNode *pNode;
	nPos = pBaseMap->getFirstPos();
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		constructNbrhood(pNode, pBaseMap, nMaxOutNbrNum, nPartial);
		addNodeAsInNbr2ItsOutNbrs(pNode, pBaseMap);
	}//while (nPos >= 0)
	return 0;
}
///////////////////
// constructNbrhood()
//	construct the out-in nbrhood for pNode0
//	the out nbr num is limited to 'nMaxOutNbrNum'
//	if nPartial==1 then only consider bases with partial order, 
//	which means only consider those bases with (x,y) s.t. y>y0 || (y==y0&&x>x0)
//	return 0
//	not addNodeAsInNbr2ItsOutNbrs
int SkNbrManager::constructNbrhood(SkBaseNode *pNode0, SkBaseMap* pBaseMap, int nMaxOutNbrNum, int nPartial)
{
	// try to find pNode as pNode0's nbrs
	pNode0->removeAllNbrs();

	SkOutNbr outNbr;
	double dDis;
	int x0, y0, x, y;
	int nID0, nID;

	pNode0->setMaxNbrNum(nMaxOutNbrNum, OUT_NBR);
	nID0 = pNode0->getID();
	x0 = pNode0->getCol();
	y0 = pNode0->getRow();

	int  nPos;
	SkBaseNode *pNode;
	nPos =  pBaseMap->getFirstPos();
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		if (pNode == pNode0)
			continue;
		x = pNode->getCol();
		y = pNode->getRow();

		if (nPartial == 1)
		{
			if (y > y0) continue;
			if (y == y0)
				if (x > x0)
					continue;
		}
		dDis = ::distanceBetweenPoints(y, x, y0, x0);
		nID = pNode->getID();
		outNbr.setNbrID(nID);
		outNbr.setCenterDis(dDis);
		pNode0->addNbr(outNbr, OUT_NBR);
	}//while (nPos >= 0)
	return 0;
}
///////////////////
// addNodeAsInNbr2ItsOutNbrs()
//	add current node - pNode as in nbr to its out nbrs
//	return out node num
int SkNbrManager::addNodeAsInNbr2ItsOutNbrs(SkBaseNode *pNode, SkBaseMap *pMap)
{
	int nID = pNode->getID();
	int nOutNbrNum = pNode->getNbrNum(OUT_NBR);
	int nNbrID;
	SkBaseNode *pNbrNode;
	for (int o = 0; o < nOutNbrNum; o++)
	{
		nNbrID = pNode->getNbrIDAt(o, OUT_NBR);
		pNbrNode = pMap->getNodeAt(nNbrID);
		pNbrNode->addInNbr(nID);
	}
	return nOutNbrNum;
}

SkBaseNode *SkNbrManager::findFirstBase(SkBaseMap *pBaseMap)
{
	int x0 = pBaseMap->getWidth();
	int y0 = pBaseMap->getHeight();
	int x, y;

	int  nPos =  pBaseMap->getFirstPos();
	SkBaseNode *pNode, *pNode0;
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		x = pNode->getCol();
		y = pNode->getRow();

		if (y < y0 || (y==y0 && x < x0))
		{
			pNode0 = pNode;
			y0 = y;
			x0 = x;
		}
	}//while (nPos >= 0)
	return pNode0;
}

///////////////////
// after()
//	check if the position of pNode is 'after' (x0, y0)
//	'after' means y>y0 || y==y0 && x >= x0
bool SkNbrManager::after(SkBaseNode *pNode, int x0, int y0)
{
	int x = pNode->getCol();
	int y = pNode->getRow();
	if (y > y0 || (y==y0 && x >= x0))
		return true;
	else
		return false;
}
///////////////////
// after()
//	check if the position of pNode is 'after' the position of pNode0
//	'after' means y>y0 || y==y0 && x >= x0
bool SkNbrManager::after(SkBaseNode *pNode, SkBaseNode *pNode0)
{
	int x0 = pNode0->getCol();
	int y0 = pNode0->getRow();
	return after(pNode, x0, y0);
}
///////////////////
// existNode()
//	check if there exists any node around (x0,y0) within the distance of dDis
bool SkNbrManager::existNode(int x0, int y0, SkBaseMap *pBaseMap, double dDis)
{
	int x, y;
	int  nPos =  pBaseMap->getFirstPos();
	SkBaseNode *pNode;
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		x = pNode->getCol();
		y = pNode->getRow();
		if (::distanceBetweenPoints(y, x, y0, x0) < dDis)
			return true;
	}//while (nPos >= 0)
	return false;
}
///////////////////
// moveAdjacientBases()
//	check if there are two bases are too adjacient (distance less than dMinDis)
//	if so, then remove (set as invalid) one of the two bases with smaller fabs-coef
int SkNbrManager::moveAdjacientBases(SkBaseMap *pBaseMap, double dMinDis)
{
	int n = 0;
	double dDis;
	int nOutNbrNum;
	int nPos =  pBaseMap->getFirstPos();
	SkBaseNode *pNode, *pNodeNbr;
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		nOutNbrNum = pNode->getNbrNum(OUT_NBR);
		if (nOutNbrNum < 1)
			continue;
		dDis = pNode->getNbrCenterDis(0, OUT_NBR);
		if (dDis >= dMinDis)
			continue;
		pNodeNbr = pBaseMap->getNodeAt(pNode->getNbrIDAt(0, OUT_NBR));
		if (fabs(pNode->m_data.a) > fabs(pNodeNbr->m_data.a) )
		{
			pNodeNbr->setValid(false);
			pNodeNbr->setMoveFlag(1);
		}
		else
		{
			pNode->setValid(false);
			pNode->setMoveFlag(1);
		}
		n++;
	}//while (nPos >= 0)
	return 0;
}

///////////////////
// sortNbr()
//	sort the basenode nbrs
//	nbrType: 0-outnbr, 1-prtnbr, 2-chdnbr, 3-inNbr; 
//	indexType: 0-distance, 1-direction, 2-BiBaseProb, 3-BiBaseCluster, 4-nbrID, 5-uniBaseProb, 6-uniBaseCluster
int SkNbrManager::sortNbr(SkBaseMap *pBaseMap, int nbrType, int indexType)
{
	int  nPos =  pBaseMap->getFirstPos();
	SkBaseNode *pNode;
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		if (indexType != 5 && indexType != 6)
			pNode->sortNbr(nbrType, indexType);
		else
			sortNbrByUniBaseProperty(pNode, pBaseMap, nbrType, indexType);
	}
	return 0;
}

///////////////////
// sortNbrByUniBaseProperty()
//	sort nbr by uniBaseProperty
//	nbrType: 0-outnbr, 1-prtnbr, 2-chdnbr; 
//	indexType: (must be)5-uniBaseCluster(increasing order), 6-Base Coef absolute value, all others not defined
int SkNbrManager::sortNbrByUniBaseProperty(SkBaseNode *pNode, SkBaseMap *pBaseMap, int nbrType, int indexType)
{
	assert(indexType == 5 || indexType == 6);//must be uniBaseProperty
	assert(nbrType != 3);//couldn't be inNbr

	SkBaseMap *pNbrMap;
	switch (nbrType)
	{
	case 0://out nbr
		pNbrMap = pBaseMap;
		break;
	case 1://prt nbr
		pNbrMap = pBaseMap->getParentMap();
		break;
	case 2:
		pNbrMap = pBaseMap->getChildMap();
		break;
	default:
		assert(false);
		break;
	}
	SkOutNbrArray *pNbrArray = &(pNode->m_nbrs.m_pNbrArray[nbrType]);
	SkBaseNode *pNbr1, *pNbr2;
	int t1, t2;
	double a1, a2;
	int nNbrNum = pNbrArray->GetSize();
	int i, j;
	for (i = 0; i < nNbrNum-1; i++)
	{
		pNbr1 = pNbrMap->getNodeAt(pNode->getNbrIDAt(i, nbrType));
		t1 = pNbr1->get1GonClusterNo();
		a1 = pNbr1->m_data.a;
		for (j = i+1; j < nNbrNum; j++)
		{
			pNbr2 = pNbrMap->getNodeAt(pNode->getNbrIDAt(j, nbrType));
			t2 = pNbr2->get1GonClusterNo();
			a2 = pNbr2->m_data.a;
			if (indexType == 5)
			{// sort by uniBase type
				if (t1 > t2 || (t1==t2 && fabs(a1) < fabs(a2)) )// swap the nbr
				{
					::swap(pNbrArray->GetAt(i), pNbrArray->GetAt(j));
					t1 = t2;
					a1 = a2;
				}
			}
			else if (indexType == 6)
			{// sort by uniBase coef (absolute value)
				if (fabs(a1) < fabs(a2))// swap the nbr
				{
					::swap(pNbrArray->GetAt(i), pNbrArray->GetAt(j));
					t1 = t2;
					a1 = a2;
				} 
			}
		}
	}

	// below only for debug
	for (i = 0; i < nNbrNum; i++)
	{
		pNbr1 = pNbrMap->getNodeAt(pNode->getNbrIDAt(i, nbrType));
		int nType = pNbr1->get1GonClusterNo();
		float a = pNbr1->m_data.a;
	}
	return 0;
}

///////////////////
// setElectronsAsNearestNuclusOutNbr()
//	For each electrons, find the nearest nucleus within dMaxDis, 
//	and set it as the nucleus' out nbr
//	return 0;
int SkNbrManager::setElectronsAsNearestNucleusOutNbr(SkBaseMap* pBaseMap, double dMaxDis)
{
	int  nPos =  pBaseMap->getFirstPos();
	SkBaseNode *pNode;
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		if (pNode->getType() == NUCLEUS)
			continue;
		setElectronsAsNearestNucleusOutNbr(pNode, pBaseMap, dMaxDis);
	}
	return 0;
}

///////////////////
// setElectronsAsNearestNuclusOutNbr()
//	For the electron - pBaseNode0, find the nearest nucleus within dMaxDis, 
//	and set it as the nucleus' out nbr
//	return 0;
int SkNbrManager::setElectronsAsNearestNucleusOutNbr(SkBaseNode* pNode0, SkBaseMap* pBaseMap, double dMaxDis)
{
	assert(pNode0->getType() != NUCLEUS);

	int x0 = pNode0->getCol();
	int y0 = pNode0->getRow();
	int nID = pNode0->getID();
	int x, y;
	double dDis, dMinDis = 10000;

	int  nPos =  pBaseMap->getFirstPos();
	SkBaseNode *pNode, *pMinDisNode = 0;
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		if (pNode == pNode0)
			continue;
		if (pNode->getType() != NUCLEUS)
			continue;
		x = pNode->getCol();
		y = pNode->getRow();
		dDis = ::distanceBetweenPoints(y, x, y0, x0);
		if (dDis > dMaxDis)
			continue;
		if (dDis < dMinDis)
		{
			dMinDis = dDis;
			pMinDisNode = pNode;
		}
	}
	if (pMinDisNode != 0)
	{
		SkOutNbr outNbr;
		outNbr.setNbrID(nID);
		outNbr.setCenterDis(dMinDis);
		pMinDisNode->addNbr(outNbr, OUT_NBR);//add pNode0 to pNode as out nbr
	}
	else//no nodes close to pNode0, then set it as nucleus
		pNode0->setType(NUCLEUS);
	return 0;
}
///////////////////
// setElectronsAndNucleusAndBuildOutNbrhood()
//	1). set all bases as FREE_ELEC
//	2). Check each base according to its ID (coefs are ordered roughly by IDs)
//		If it is still FREE_ELEC then set it as NUCLEUS, 
//		and search all the other bases within dMaxDis which are still FREE_ELEC
//		and set them as its electron-out nbrs (also set out-nbrs as BIND_ELEC
//	3). Repeat 2) until all bases are not FREE_ELEC
//	return 0;
int SkNbrManager::setElectronsAndNucleusAndBuildOutNbrhood(SkBaseMap* pBaseMap, double dMaxDis)
{
	int  nPos =  pBaseMap->getFirstPos();
	SkBaseNode *pNode;
	//set all type as free-electrons
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		pNode->setType(FREE_ELEC);
		pNode->removeAllNbrs(OUT_NBR);
	}

	nPos =  pBaseMap->getFirstPos();
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		if (pNode->getType() != FREE_ELEC)
			continue;
		setElectronsAndNucleusAndBuildOutNbrhood(pNode, pBaseMap, dMaxDis);
	}
	return 0;
}

///////////////////
// setElectronsAndNucleusAndBuildOutNbrhood()
//	1). search all the other bases within dMaxDis of base pNode0 which are still FREE_ELEC
//	2). set them as pNode0's electron-out nbrs (also set out-nbrs as BIND_ELEC)
//	return 0;
int SkNbrManager::setElectronsAndNucleusAndBuildOutNbrhood(SkBaseNode* pNode0, SkBaseMap* pBaseMap, double dMaxDis)
{
	assert(pNode0->getType() == FREE_ELEC);
	pNode0->setType(NUCLEUS);

	int x0 = pNode0->getCol();
	int y0 = pNode0->getRow();
	int nHalf = pNode0->m_data.getHalf();
	int x, y;
	double dDis;

	int nNbrType = OUT_NBR;
	int  nPos =  pBaseMap->getFirstPos();
	SkBaseNode *pNode, *pMinDisNode = 0;
	while (nPos >= 0)
	{
		pNode = pBaseMap->getNextNode(nPos);
		if (!pNode->valid())
			continue;
		if (pNode->getType() != FREE_ELEC)
			continue;
		if (pNode == pNode0)//never happen
			continue;
		x = pNode->getCol();
		y = pNode->getRow();
		dDis = ::distanceBetweenPoints(y, x, y0, x0);
		if (dDis > dMaxDis)
			continue;
		
//		if (dDis > nHalf)
//			continue;// consider the filter size, only combine those which cover the centers of each other

		int nID = pNode->getID();
		SkOutNbr outNbr;
		outNbr.setNbrID(nID);
		outNbr.setCenterDis(dDis);
		int nMaxNbrNum = pNode0->getMaxNbrNum(nNbrType);
		int nNbrNum = pNode0->getNbrNum(nNbrType);
		if (nNbrNum < nMaxNbrNum)
		{//not get enough nbrs yet
			pNode0->addNbr(outNbr, nNbrType);
			pNode->setType(BIND_ELEC);
		}
		else
		{//already get enough neighbors
			int nLastNbrID = pNode0->getNbrIDAt(nMaxNbrNum-1, nNbrType);
			assert(nLastNbrID >= 0);
			SkBaseNode *pLastNbr;
			if (nLastNbrID >=0 )
				pLastNbr = pBaseMap->getNodeAt(nLastNbrID);
			int nNbrPos = pNode0->addNbr(outNbr, nNbrType);//add pNode0 to pNode as out nbr
			if (nNbrPos >= 0)//valid nbr
			{
				pNode->setType(BIND_ELEC);
				assert(pLastNbr != 0);
				{
					pLastNbr->setType(FREE_ELEC);
				}
			}
		}
	}
	return 0;
}

///////////////////
// removeThisNodeFromItsNbrs()
//	This Node will be removed from the base map, it should be removed from its nbrs too
int SkNbrManager::removeThisNodeFromItsNbrs(SkBaseNode *pNode, SkBaseMap* pBaseMap, int nbrType)
{
	int n = pNode->getNbrNum(nbrType);
	int nbrID;
	SkBaseNode *pNbr;
	int a, a_i;
	if (nbrType == ADR_NBR)
	{
		for (a = 0; a < MAX_ADR_NUM; a++)
		{
			nbrID = pNode->getNbrIDAt(a, nbrType);
			if (nbrID < 0)
				continue;
			pNbr = pBaseMap->getNodeAt(nbrID);
			a_i = SkNodeNbrs::adrNbrInverse(a);
			pNbr->setNbrValid(a_i, false, nbrType);
		}
	}
	else
	{
		assert(false);
	}
	return 0;
}

