#include "inlineFunc.h"
#include "skOutput.h"
#include "skIOImage.h"
#include "skMatrixIplOp.h"

#include "skFilter.h"
#include "skFilterCoef.h"
#include "skGeneralEdge.h"

#include "skPrimitive.h"
#include "skPrimPursuit.h"

#include "cv.h"
#include "highgui.h"

const char g_pchSketchParamName[] = "sketch_param.txt";
const char g_pchSketchWorkDir[] = "curves";
const char g_pchProposalWorkDir[] = "proposal";

int g_nHalfProfileWd = 3;


/////////////////////////////////////////////////////////////////////
// class SkPrimPursuitParam

SkPrimPursuitParam::SkPrimPursuitParam()
{
	//only for saving results
	m_nSaveIntermediateResults = 0;
 	m_nSaveCurveByCurveStep = 1;

	//constrols line segment error to form curve
	m_dPixTol = 1.0;//0.0~3.0, 
	m_dAngTol = 0.05;//0~0.3
	m_nLinkRad = 2;//0~5

	//the threshold for stop searching all possible strokes
	m_dMagThresh1 = 0.25;//[0.0, 1.0]


	//the min image CLG to keep strokes
	m_dMinImgCLG = 25;//0.0~40.0
	m_dExtensionDist4Refine = 7.0;//1~30, extension distance to refine the graph
	
	m_dExtRegionHalfLeft = 1.0;//1.0~5.0
	m_nExtTrapezoidleRegion = 0;//trapezoid region (1) or rectangle (0)

	m_dMinTotalCLG = 5;//the total CLG to accept the graph change, not used for version sketch_v69

}

SkPrimPursuitParam::~SkPrimPursuitParam()
{
}
int SkPrimPursuitParam::read(const char *name)
{
	FILE *pFile = fopen(name, "r");
	if (pFile == 0)
		return 1;

	fscanf(pFile, "MagThresh1:                      %.2f\n", &m_dMagThresh1);
	fscanf(pFile, "MinImgCodingLengthGain:          %.2f\n", &m_dMinImgCLG);
	fscanf(pFile, "MinTotalCodingLengthGain:        %.2f\n\n", &m_dMinTotalCLG);

	fscanf(pFile, "Pixel tolerance(for edge link):  %0.2f\n", &m_dPixTol);
	fscanf(pFile, "Angel tolerance(for edge link):  %0.2f\n", &m_dAngTol);
	fscanf(pFile, "Linking radius(for edge link):   %d\n\n", &m_nLinkRad);

	fscanf(pFile, "Extension Dist. for refine:      %.2f\n", &m_dExtensionDist4Refine);
	fscanf(pFile, "Extension Dist. (half left):     %.2f\n", &m_dExtRegionHalfLeft);
	fscanf(pFile, "Extension(rect. or trapezoid):   %d\n\n", &m_nExtTrapezoidleRegion);

	fscanf(pFile, "Save Curve By Curve Step:        %d\n", &m_nSaveCurveByCurveStep);

	fclose(pFile);

	return 0;
}
int SkPrimPursuitParam::save(const char *name)
{
	FILE *pFile = fopen(name, "w");
	if (pFile == 0)
		return 1;
	
	fprintf(pFile, "MagThresh1:                      %.2f\n", m_dMagThresh1);
	fprintf(pFile, "MinImgCodingLengthGain:          %.2f\n", m_dMinImgCLG);
	fprintf(pFile, "MinTotalCodingLengthGain:        %.2f\n\n", m_dMinTotalCLG);

	fprintf(pFile, "Pixel tolerance(for edge link):  %0.2f\n", m_dPixTol);
	fprintf(pFile, "Angel tolerance(for edge link):  %0.2f\n", m_dAngTol);
	fprintf(pFile, "Linking radius(for edge link):   %d\n\n", m_nLinkRad);
	
	fprintf(pFile, "Extension Dist. for refine:      %.2f\n", m_dExtensionDist4Refine);
	fprintf(pFile, "Extension Dist. (half left):     %.2f\n", m_dExtRegionHalfLeft);
	fprintf(pFile, "Extension(rect. or trapezoid):   %d\n\n", m_nExtTrapezoidleRegion);

	fprintf(pFile, "Save Curve By Curve Step:        %d\n", m_nSaveCurveByCurveStep);

	fclose(pFile);

	return 0;
}
int skOutputPrimPursuitParam(SkPrimPursuitParam &param)
{
	char msg[512];
	sprintf(msg, "MagThresh1:                      %.2f\n", param.m_dMagThresh1);
	::skOutput(msg);
//	sprintf(pFile, "Min Branch Length:               %d\n", m_nMinBranchLength);
	sprintf(msg, "Save Curve By Curve Step:        %d\n", param.m_nSaveCurveByCurveStep);
	::skOutput(msg);

	return 0;
}

/////////////////////////////////////////////////////////////////////
int skPrimPursuit(FILE *pFile)
{
	char ppchImgName[1024][128];
	char name[512], msg[512], proj[512], no[32];
	
	///////////////
	// get file names
	int n = 0;
	while (fscanf(pFile, "%s", ppchImgName[n]) != EOF)
	{
		n++;
		if (n >= 1024)
			break;
	}

	sprintf(msg, "\n  Primitive Pursuit on %d images......\n", n);
	::skOutput(msg);

	SkPrimPursuit pp;
	// read the scale parameter
	int i; 
	for (i = 0; i < n; i++)
	{
		pp.initializePrimPursuit(ppchImgName[i]);

		pp.m_nProcStep = 3;

		pp.primPursuitWithoutPrior();
		pp.savePursuitResults(pp.m_nProcStep);

		pp.completePrimPursuit();
	}
	return n;

}
/////////////////////////////////////////////////////////////////////
int skReadAndSaveProfileSetInOne(FILE *pFile)
{
	char ppchImgName[1024][128];
	char name[512], msg[512], pchProjName[512], no[32];
	
	///////////////
	// get file names
	int n = 0;
	while (fscanf(pFile, "%s", ppchImgName[n]) != EOF)
	{
		n++;
		if (n >= 1024)
			break;
	}

	sprintf(msg, "\n  Reading and Saving Profile set on %d images......\n", n);
	::skOutput(msg);
	
	if (n == 0)
		return 0;

	// read image
	Matrix<int> pImgChn[3];
	int nReturn = ::skLoadImage2YCrCb(ppchImgName[0], pImgChn);
	int ht, wd;
	pImgChn[0].GetDimension(ht, wd);
	if (ht <= 0 || wd <= 0)
		return 1;

	Matrix<float> imgOrg;//save the gray image 
	::skCopyMatrix(pImgChn[0], imgOrg);

	// normalize image
	double dImgMean0 = 0, dImgStd0 = -1;
	::skNormalizeMatrix2ZeroMean1Std(&imgOrg, dImgMean0, dImgStd0);//-1 std will lead to find the mean and std


	char pchDir0[512];
	_getcwd(pchDir0, 512);

	char work_dir[32];
	sprintf(work_dir, "prim");

	SkPrimPursuitParam param;
	sprintf(name, g_pchSketchParamName);
	param.read(name);

	int step = 3;
	char type[512];
	double dMTh = param.m_dMagThresh1;///255;
	int nMTh = (int)(dMTh*100);
	char mth_no[32];
	::changeNumTo2Char(mth_no, nMTh);
	char w_no[32];
	int hf = g_nHalfProfileWd;
	::changeNumTo2Char(w_no, 2*hf+1);
	sprintf(type, "%d_m%s_w%s", step, mth_no, w_no);

	// read the scale parameter
	int i; 
	SkProfileSet prflSet3;
//	SkProfileSet prflSet4;
	for (i = 0; i < n; i++)
	{
		sprintf(msg, "                   Reading %s ...... ", ppchImgName[i]);
		skOutput(msg);
		//get project name and change dir
		strcpy(pchProjName, ppchImgName[i]);
		::removeFileTypeInName(pchProjName);
		_mkdir(pchProjName);
		_chdir(pchProjName);

		_mkdir(work_dir);

		SkProfileSet prflSet;
		sprintf(name, "%s\\%s_%s_prfl_set.txt", work_dir, pchProjName, type);
		prflSet.readFromText(name);

		prflSet3.add(prflSet);
//		prflSet4.add(prflSet);

		sprintf(msg, "done! \n", ppchImgName[i]);
		skOutput(msg);

		_chdir(pchDir0);
	}

	sprintf(msg, "\n Saving the profiles, please wait for a while .... \n");
	skOutput(msg);

	sprintf(type, "3");
	prflSet3.sort();
	prflSet3.save2Files(type, dImgMean0, dImgStd0);

	skSaveSomeBlobEdgeRidgeSamplesFromPrflSet(prflSet3, type, dImgMean0, dImgStd0);

//	sprintf(type, "4");
//	prflSet4.sort();
//	prflSet4.save2Files(type, dImgMean0, dImgStd0);

	return n;
}
int skSaveSomeBlobEdgeRidgeSamplesFromPrflSet(SkProfileSet&prflSet, const char *type, 
											  double dImgMean0, double dImgStd0)
{
	char work_dir[32],dir[512];
	sprintf(work_dir, "1D_prim");
	_mkdir(work_dir);

	sprintf(dir, "%s\\blob", work_dir);
	_mkdir(dir);
	sprintf(dir, "%s\\edge", work_dir);
	_mkdir(dir);
	sprintf(dir, "%s\\end", work_dir);
	_mkdir(dir);
	sprintf(dir, "%s\\ridge", work_dir);
	_mkdir(dir);
	sprintf(dir, "%s\\multi_ridge", work_dir);
	_mkdir(dir);

	char name[512], msg[512];

	int nPrflNum = prflSet.m_rmArray.getSize();
	int hf0 = 11;
	int ht0 = 2*hf0 + 1;
	int wd0 = ht0;

	int ht = 2*g_nHalfProfileWd+1;
	if (nPrflNum > 0)
	{
		SkRidgeModel &rm = prflSet.m_rmArray[0];
		ht = rm.getDataLength();
	}
	
	double *dv, *dvm;
	int i, j;

	// get the profile matrix first
	Matrix<float> prflMat(ht0,wd0), prflMatM(ht0,wd0);

	char no[32];

	int nBlobNum = 0;
	int nEndNum = 0;
	int nEdgeNum = 0;
	int nRidgeNum = 0;
	int nMultiRidgeNum = 0;
	// save as bmp with diff seg num
	for (i = 0; i < nPrflNum; i++)
	{
		SkRidgeModel &rm = prflSet.m_rmArray[i];
		dv = rm.getOrgIntensity();
		dvm = rm.getCenterredModelIntensity();
		int nStepNum = rm.getStepNum();
		if (nStepNum == 2)
		{//edge
			::changeNumTo4Char(no, nBlobNum);
			prflMatM.Set(0);
			skSaveABlobFromProfile(prflMatM, dvm, ht, wd0/2, ht0/2);
			sprintf(name, "%s\\blob\\%s_blob_%s.bmp", work_dir, type, no);
			::skSaveMatrixAsBitmap(name, prflMatM, dImgMean0, dImgStd0);
			nBlobNum++;

			::changeNumTo4Char(no, nEdgeNum);
			double dOrt = PI*rand()/(RAND_MAX+1.0);
			skSaveAnEdgeFromProfile(prflMatM, dvm, ht, dOrt);
			sprintf(name, "%s\\edge\\%s_edge_%s.bmp", work_dir, type, no);
			::skSaveMatrixAsBitmap(name, prflMatM, dImgMean0, dImgStd0);
			nEdgeNum++;
		}
		else if (nStepNum == 3)
		{
			::changeNumTo4Char(no, nEndNum);
			double dOrt = PI*rand()/(RAND_MAX+1.0);
			dOrt = 0;
			skSaveAnEndFromProfile(prflMatM, dvm, ht, dOrt);
			sprintf(name, "%s\\end\\%s_end_%s.bmp", work_dir, type, no);
			::skSaveMatrixAsBitmap(name, prflMatM, dImgMean0, dImgStd0);
			nEndNum++;

			::changeNumTo4Char(no, nRidgeNum);
			dOrt = PI*rand()/(RAND_MAX+1.0);
			dOrt = 0;
			skSaveARidgeFromProfile(prflMatM, dvm, ht, dOrt);
			sprintf(name, "%s\\ridge\\%s_ridge_%s.bmp", work_dir, type, no);
			::skSaveMatrixAsBitmap(name, prflMatM, dImgMean0, dImgStd0);
			nRidgeNum++;
		}
		else if (nStepNum > 3)
		{
			double dOrt;
			::changeNumTo4Char(no, nMultiRidgeNum);
			dOrt = PI*rand()/(RAND_MAX+1.0);
			dOrt = 0;
			skSaveARidgeFromProfile(prflMatM, dvm, ht, dOrt);
			sprintf(name, "%s\\multi_ridge\\%s_multi_ridge_%s.bmp", work_dir, type, no);
			::skSaveMatrixAsBitmap(name, prflMatM, dImgMean0, dImgStd0);
			nMultiRidgeNum++;
		}
	}//for prfl
	return 0;
}
///////////////////
// skRidgeEdgeBlobDetection()
// input: x.bmp
// output:
//	x_ridge_mag0.bmp
//	x_ridge_dir0.bmp
//	x_ridge_size0.bmp
//	x_edge_mag0.bmp
//	x_edge_dir0.bmp
//	x_edge_size0.bmp
//	x_blob_mag0.bmp
//	x_blob_size0.bmp
int skRidgeEdgeBlobDetection(const char * pchImgName)
{
	char name[512], msg[512], no[32], pchProjName[512];

	sprintf(msg, "\n   Ridge-Edge-Blob detection  on image %s......\n", pchImgName);
	::skOutput(msg);

	DWORD Time0 = ::GetTickCount();

	char pchDir0[512];
	_getcwd(pchDir0, 512);

	// read image
	Matrix<int> pImgChn[3];
	int nReturn = ::skLoadImage2YCrCb(pchImgName, pImgChn);
	int ht, wd;
	pImgChn[0].GetDimension(ht, wd);
	if (ht <= 0 || wd <= 0)
		return 1;

	Matrix<float> imgOrg;//save the gray image 
	::skCopyMatrix(pImgChn[0], imgOrg);

	// normalize image
	double dImgMean0 = 0, dImgStd0 = -1;
	::skNormalizeMatrix2ZeroMean1Std(&imgOrg, dImgMean0, dImgStd0);//-1 std will lead to find the mean and std

	//get project name and change dir
	strcpy(pchProjName, pchImgName);
	::removeFileTypeInName(pchProjName);
	
	_mkdir(pchProjName);
	_chdir(pchProjName);

	int reb = 0;//0~ridge, 1~edge, 2~blob
	for (reb = 0; reb < 3; reb++)
	{
		skREBDetection(imgOrg, reb, pchProjName);
	}

	char work_dir[512];
	sprintf(work_dir, g_pchProposalWorkDir);
	_mkdir(work_dir);

	Matrix<float> ridge_mag, edge_mag;
	sprintf(name, "%s\\%s_ridge_mag0.dat", work_dir, pchProjName);
	::skReadMatrix(name, ridge_mag);
	sprintf(name, "%s\\%s_edge_mag0.dat", work_dir, pchProjName);
	::skReadMatrix(name, edge_mag);

	Matrix<float> ridge_dir, edge_dir;
	sprintf(name, "%s\\%s_ridge_dir0.bmp", work_dir, pchProjName);
	::skReadBitmap(name, &ridge_dir);
	sprintf(name, "%s\\%s_edge_dir0.bmp", work_dir, pchProjName);
	::skReadBitmap(name, &edge_dir);

	Matrix<float> re_mag(ht,wd), re_dir(ht, wd);
	Matrix<int> re_type(ht,wd);
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (ridge_mag(r,c) > edge_mag(r,c) )
			{
				re_mag(r,c) = ridge_mag(r,c);
				re_type(r,c) = 0;
				re_dir(r,c) = ridge_dir(r,c);
			}
			else
			{
				re_mag(r,c) = edge_mag(r,c);
				re_type(r,c) = 1;
				re_dir(r,c) = edge_dir(r,c);
			}
		}
	}

	sprintf(name, "%s\\%s_re_mag0.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, re_mag, 0, -1);
	sprintf(name, "%s\\%s_re_type0.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, re_type, 0, -1);

	Matrix<int> re_mag_int;
	sprintf(name, "%s\\%s_re_mag0.bmp", work_dir, pchProjName);
	::skReadBitmap(name, &re_mag_int);

	Matrix<int> sketch;
	Matrix<int> mag1;

	skNonmaximumSupressionFromMagAndDir(re_mag_int, re_dir, sketch, mag1);
	sprintf(name, "%s\\%s_re_mag1.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, mag1, 0, 1);
	sprintf(name, "%s\\%s_re0.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, sketch, 0, 1);
	

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, " done!  %d min %.3fs \n", nMin, dSec);
	::skOutput(msg);

	_chdir(pchDir0);
	return 0;
}
int skREBDetection(Matrix<float> &imgOrg, int reb, const char* pchProjName)
{
	char msg[512];
	char name[512];
	char rebType[256];
	SkFilterBank filterBank, *pFB = &filterBank;
	switch (reb)
	{
	case 0:
	default:
		sprintf(name, "sk_ridge_filter.txt");
		sprintf(rebType, "ridge");
		break;
	case 1:
		sprintf(name, "sk_edge_filter.txt");
		sprintf(rebType, "edge");
		break;
	case 2:
		sprintf(name, "sk_blob_filter.txt");
		sprintf(rebType, "blob");
		break;
	}

	sprintf(msg, "      %s detection ......\n", rebType);
	skOutput(msg);

	pFB->m_param.m_nFBType = SK_REB_DETECTION;
	pFB->m_param.initilaize4REBDetection(reb);
//	if (pFB->readParameters(name) !=0)
//		pFB->saveParameters(name);

	char filter_type[32];
	SkFilterBank::m_dLPNorm = 0.9;//LP norm
	pFB->buildFilters();
	pFB->saveFiltersAsBmp(rebType);

	SkFilterCoef *pFC = new SkFilterCoef;
	
	pFC->setProjName(pchProjName);
	pFC->setConsiderEntropy(false);
	pFC->setSaveMaxCoef(true);//save the max coef
	pFC->setFilterBank(pFB);
	pFC->setMatrixImage(&imgOrg); 
	pFC->setBlockDivideByBlockSize(2000);
	pFC->initializeCoefficencies();

	// save the sketch strength
	char work_dir[512];
	sprintf(work_dir, g_pchProposalWorkDir);
	_mkdir(work_dir);

	sprintf(name, "%s\\%s_%s_mag0.bmp", work_dir, pchProjName, rebType);
	::skSaveMatrixAsBitmap(name, pFC->m_maxmag, 0, -1);
	
	sprintf(name, "%s\\%s_%s_mag0.dat", work_dir, pchProjName, rebType);
	::skSaveMatrix(name, pFC->m_maxmag);

	sprintf(name, "%s\\%s_%s_dir0.bmp", work_dir, pchProjName, rebType);
	::skSaveMatrixAsBitmap(name, pFC->m_normal, 0, 1);

	sprintf(name, "%s\\%s_%s_size0.bmp", work_dir, pchProjName, rebType);
	::skSaveMatrixAsBitmap(name, pFC->m_hfht, 0, 1);
	// save an inversed magnitude just for visualization
	int ht, wd;
	imgOrg.GetDimension(ht,wd);
	Matrix<float> tmp, zeros;
	zeros.SetDimension(ht, wd);
	zeros.Zero();
	tmp = zeros - pFC->m_maxmag;
	sprintf(name, "%s\\%s_%s_mag0_inv.bmp", work_dir, pchProjName, rebType);
	::skSaveMatrixAsBitmap(name, tmp, 0, -1);

	delete pFC;

	if (reb == 0 || reb == 1)
	{
		// get diretion in dx dy
		Matrix<float> dir;
		// read dir 
		sprintf(name, "%s\\%s_%s_dir0.bmp", work_dir, pchProjName, rebType);
		::skReadBitmap(name, &dir);//in degree by two
		// read sketch strength magnitude
		Matrix<int> mag;
		sprintf(name, "%s\\%s_%s_mag0.bmp", work_dir, pchProjName, rebType);
		::skReadBitmap(name, &mag);
		Matrix<int> sketch;
		Matrix<int> mag1;

		skNonmaximumSupressionFromMagAndDir(mag, dir, sketch, mag1);

		sprintf(name, "%s\\%s_%s_mag1.bmp", work_dir, pchProjName, rebType);
		::skSaveMatrixAsBitmap(name, mag1, 0, 1);
		sprintf(name, "%s\\%s_%s0.bmp", work_dir, pchProjName, rebType);
		::skSaveMatrixAsBitmap(name, sketch, 0, 1);
	}
	else
	{
		// deal with blobs
		Matrix<int> blobProposal;
		Matrix<float> blobStrength;
		
		sprintf(name, "%s\\%s_%s_mag0.bmp", work_dir, pchProjName, rebType);
		::skReadBitmap(name, &blobStrength);

		Matrix<float> filter(3,3);
		filter(0,0) = 0; filter(0,1) = 1; filter(0,2) = 0;
		filter(1,0) = 1; filter(1,1) = 4; filter(1,2) = 1; 
		filter(2,0) = 0; filter(2,1) = 1; filter(2,2) = 0;

		filter = filter/8.0;
		blobStrength = blobStrength.Conv(filter);

		int n0 = skBlobNoneMaxSuppresion(blobStrength, blobProposal);

		sprintf(name, "%s\\%s_blob0.bmp", work_dir, pchProjName);
		::skSaveMatrixAsBitmap(name, blobProposal, 0, -1);

		double dMaxBlobSth = blobStrength.GetMax();
		double dThresh = dMaxBlobSth *3 / 4;
		int dn = ::skTrimmingByThresholding(blobStrength, blobProposal, dThresh);

		Matrix<int> blobSize;
		sprintf(name, "%s\\%s_blob_size0.bmp", work_dir, pchProjName);
		::skReadBitmap(name, &blobSize);

		Matrix<int> blob_show;
		::skDrawBlobs(blobProposal, blobSize, blob_show);

		sprintf(name, "%s\\%s_blob1.bmp", work_dir, pchProjName);
		::skSaveMatrixAsBitmap(name, blob_show, 0, -1);
	}

	return 0;
}
int skNonmaximumSupressionFromMagAndDir(Matrix<int> &mag, Matrix<float> &dir, 
			Matrix<int> &sketch, Matrix<int> &mag1)
{
	int ht, wd;
	mag.GetDimension(ht, wd);
	sketch.SetDimension(ht, wd);

	mag1 = mag;

	sketch.Set(255);
	Matrix<float> rad;

	Matrix<int> dx(ht,wd), dy(ht,wd);
	// compute dx and dy
	rad = dir * (2 * PI/180.0);
	float *dir_rad = rad.data;
	int *delta_x = dx.data;
	int *delta_y = dy.data;

	skApproximateDeltaXYFromDirection(dir_rad,//direction in radius, 0~PI
									  delta_x,//delta x
									  delta_y,//delat y
									  ht, wd);//rows and cols



	//non-maximal suppression
	int *magnitude;
	magnitude = mag.data;

	Matrix<unsigned char> nmsMat(ht, wd);
	unsigned char *nms;
	nms = nmsMat.data;
	non_max_supp(magnitude, delta_x, delta_y, ht, wd, nms);


	//setNonMaxMagAsZero
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (nms[r*wd+c] == POSSIBLE_EDGE)
			{
				sketch(r,c) = 0;
			}
			else 
			{
				mag1(r,c) = 0;
			}
		}
	}

	skThinnerBoundary(sketch);
	return 0;

}
///////////////////
// skComputeSketchStrength()
// input: x.bmp
// output:
//	x_sketch_mag0.bmp
//	x_sketch_dir0.bmp
//	x_sketch_half_ln0.bmp
//	x_sketch_half_ht0.bmp
//	x_blob_mag0.bmp
//	x_blob_size0.bmp
//	x_filter_type0.bmp
int skComputeSketchStrength(const char * pchImgName)
{
	char name[512], msg[512], no[32], pchProjName[512];

	sprintf(msg, "\n   Compute Sketch Strength and Proposal  on image %s......\n", pchImgName);
	::skOutput(msg);
	sprintf(msg, "                  strength ......");
	::skOutput(msg);

	DWORD Time0 = ::GetTickCount();

	char pchDir0[512];
	_getcwd(pchDir0, 512);

	// read image
	Matrix<int> pImgChn[3];
	int nReturn = ::skLoadImage2YCrCb(pchImgName, pImgChn);
	int ht, wd;
	pImgChn[0].GetDimension(ht, wd);
	if (ht <= 0 || wd <= 0)
		return 1;

	Matrix<float> imgOrg;//save the gray image 
	::skCopyMatrix(pImgChn[0], imgOrg);

	// normalize image
	double dImgMean0 = 0, dImgStd0 = -1;
	::skNormalizeMatrix2ZeroMean1Std(&imgOrg, dImgMean0, dImgStd0);//-1 std will lead to find the mean and std

	//get project name and change dir
	strcpy(pchProjName, pchImgName);
	::removeFileTypeInName(pchProjName);
	
	_mkdir(pchProjName);
	_chdir(pchProjName);

	// build filter bank
//	sprintf(msg, "            buidling filters......\n");
//	::skOutput(msg);

	SkFilterBank filterBank, *pFB = &filterBank;
	sprintf(name, "sk_filter.txt");
//	pFB->m_param.m_nFBType = SK_PRIM_PURSUIT_FB;
	pFB->m_param.m_nFBType = SK_REB_DETECTION;
	pFB->m_param.initilaize4PrimPursuit();
	if (pFB->readParameters(name) !=0)
	{
		pFB->saveParameters(name);
	}

	char filter_type[32];
	sprintf(filter_type, "sketch");
	SkFilterBank::m_dLPNorm = 1.0;//LP norm
	pFB->buildFilters();
	pFB->saveFiltersAsBmp(filter_type);

//	sprintf(msg, "            computing coefs......\n");
//	::skOutput(msg);
	// build filter coef
	SkFilterCoef *pFC = new SkFilterCoef;
	
	pFC->setProjName(pchProjName);
	pFC->setConsiderEntropy(false);
	pFC->setSaveMaxCoef(true);//save the max coef
	pFC->setFilterBank(pFB);
	pFC->setMatrixImage(&imgOrg); 
	pFC->setBlockDivideByBlockSize(2000);
	pFC->initializeCoefficencies();

	// save the sketch strength
	char work_dir[512];
	sprintf(work_dir, g_pchProposalWorkDir);
	_mkdir(work_dir);

	sprintf(name, "%s\\%s_sketch_mag0.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, pFC->m_maxmag, 0, -1);

	sprintf(name, "%s\\%s_sketch_dir0.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, pFC->m_normal, 0, 1);

	sprintf(name, "%s\\%s_sketch_half_ln0.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, pFC->m_hfln, 0, 1);
	sprintf(name, "%s\\%s_sketch_half_ht0.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, pFC->m_hfht, 0, 1);

	sprintf(name, "%s\\%s_blob_mag0.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, pFC->m_maxmagLog, 0, -1);

	sprintf(name, "%s\\%s_blob_size0.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, pFC->m_max_size_Log, 0, 1);

	sprintf(name, "%s\\%s_filter_type0.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, pFC->m_max_k, 0, 1);

	// save an inversed magnitude just for visualization
	Matrix<float> tmp, zeros;
	zeros.SetDimension(ht, wd);
	zeros.Zero();
	tmp = zeros - pFC->m_maxmag;
	sprintf(name, "%s\\%s_sketch_mag0_inv.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, tmp, 0, -1);

	delete pFC;

	_chdir(pchDir0);

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, " done!  %d min %.3fs \n", nMin, dSec);
	::skOutput(msg);

	return 0;
}
///////////////////
// skComputeSketchProposal(const char * pchImgName)
// input: 
//		x_sketch_dir0.bmp, the normal direction
//		x_sketch_mag0.bmp, the sketch strength
//		x_blob_mag0.bmp, the blob strength
//		x_blob_size0.bmp, the correspondent blob size
// output:
//		x_sketch_sketch0.bmp, proposed sketch
//		x_sketch_mag1.bmp, correspondent magnitude
//		x_sketch_blob0.bmp, proposed blob
//		x_sketch_blob1.bmp, blobs after a thresholding
int skComputeSketchProposal(const char * pchImgName)
{
	char name[512], msg[512], no[32], pchProjName[512];

	sprintf(msg, "                  proposal ......");
	::skOutput(msg);

	DWORD Time0 = ::GetTickCount();

	char pchDir0[512];
	_getcwd(pchDir0, 512);

	// read image
	Matrix<int> pImgChn[3];
	int nReturn = ::skLoadImage2YCrCb(pchImgName, pImgChn);
	int ht, wd;
	pImgChn[0].GetDimension(ht, wd);
	if (ht <= 0 || wd <= 0)
		return 1;

	Matrix<float> imgOrg;//save the gray image 
	::skCopyMatrix(pImgChn[0], imgOrg);

	//get project name and change dir
	strcpy(pchProjName, pchImgName);
	::removeFileTypeInName(pchProjName);
	
	_mkdir(pchProjName);
	_chdir(pchProjName);

	char work_dir[512];
	strcpy(work_dir, g_pchProposalWorkDir);
	_mkdir(work_dir);

	// get diretion in dx dy
	Matrix<float> dir, rad;
	Matrix<int> dx(ht,wd), dy(ht,wd);
	// read dir 
	sprintf(name, "%s\\%s_sketch_dir0.bmp", work_dir, pchProjName);
	::skReadBitmap(name, &dir);//in degree by two
	// compute dx and dy
	rad = dir * (2 * PI/180.0);
	float *dir_rad = rad.data;
	int *delta_x = dx.data;
	int *delta_y = dy.data;

	skApproximateDeltaXYFromDirection(dir_rad,//direction in radius, 0~PI
									  delta_x,//delta x
									  delta_y,//delat y
									  ht, wd);//rows and cols


	// read sketch strength magnitude
	Matrix<int> mag;
	sprintf(name, "%s\\%s_sketch_mag0.bmp", work_dir, pchProjName);
	::skReadBitmap(name, &mag);
	

	//non-maximal suppression
	int *magnitude;
	magnitude = mag.data;

	Matrix<unsigned char> nmsMat(ht, wd);
	unsigned char *nms;
	nms = nmsMat.data;
	non_max_supp(magnitude, delta_x, delta_y, ht, wd, nms);

	Matrix<int> sketch(ht,wd);
	sketch.Set(255);

	//setNonMaxMagAsZero
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (nms[r*wd+c] == POSSIBLE_EDGE)
			{
				sketch(r,c) = 0;
			}
			else 
			{
				mag(r,c) = 0;
			}
		}
	}

	skThinnerBoundary(sketch);

	sprintf(name, "%s\\%s_sketch_mag1.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, mag, 0, 1);
	sprintf(name, "%s\\%s_sketch0.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, sketch, 0, 1);

	// deal with blobs
	Matrix<int> blobProposal;
	Matrix<float> blobStrength;
	
	sprintf(name, "%s\\%s_blob_mag0.bmp", work_dir, pchProjName);
	::skReadBitmap(name, &blobStrength);

	Matrix<float> filter(3,3);
	filter(0,0) = 0; filter(0,1) = 1; filter(0,2) = 0;
	filter(1,0) = 1; filter(1,1) = 4; filter(1,2) = 1; 
	filter(2,0) = 0; filter(2,1) = 1; filter(2,2) = 0;

	filter = filter/8.0;
	blobStrength = blobStrength.Conv(filter);

	int n0 = skBlobNoneMaxSuppresion(blobStrength, blobProposal);

	sprintf(name, "%s\\%s_blob0.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, blobProposal, 0, -1);

	double dMaxBlobSth = blobStrength.GetMax();
	double dThresh = dMaxBlobSth *3 / 4;
	int dn = ::skTrimmingByThresholding(blobStrength, blobProposal, dThresh);

	Matrix<int> blobSize;
	sprintf(name, "%s\\%s_blob_size0.bmp", work_dir, pchProjName);
	::skReadBitmap(name, &blobSize);

	Matrix<int> blob_show;
	::skDrawBlobs(blobProposal, blobSize, blob_show);

	sprintf(name, "%s\\%s_blob1.bmp", work_dir, pchProjName);
	::skSaveMatrixAsBitmap(name, blob_show, 0, -1);

	_chdir(pchDir0);

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, " done!  %d min %.3fs \n", nMin, dSec);
	::skOutput(msg);
	return 0;
}




/////////////////////////////////////////////////////////////////////
// class SkPrimPursuit

///////////////////
// constructor and deconstructor
SkPrimPursuit::SkPrimPursuit()
{
}
SkPrimPursuit::~SkPrimPursuit()
{
}

///////////////////
int SkPrimPursuit::initializePrimPursuit(const char * pchImgName)
{
	char name[512], msg[512], no[32];

//	m_Time0 = ::GetTickCount();

	_getcwd(m_pchDir0, 512);

	sprintf(msg, "\n   Sketching Pursuit on image %s......\n", pchImgName);
	::skOutput(msg);

	// read image
	Matrix<int> pImgChn[3];
	int nReturn = ::skLoadImage2YCrCb(pchImgName, pImgChn);
	int ht, wd;
	pImgChn[0].GetDimension(ht, wd);
	if (ht <= 0 || wd <= 0)
		return 1;

	::skCopyMatrix(pImgChn[0], m_imgOrg);

	// normalize image
	m_dImgMean0 = 0, m_dImgStd0 = -1;
	::skNormalizeMatrix2ZeroMean1Std(&m_imgOrg, m_dImgMean0, m_dImgStd0);//-1 std will lead to find the mean and std

	//get project name and change dir
	strcpy(m_pchProjName, pchImgName);
	::removeFileTypeInName(m_pchProjName);

	_mkdir(m_pchProjName);
	_chdir(m_pchProjName);

	::skSaveMatrixAsBitmap(pchImgName, m_imgOrg, m_dImgMean0, m_dImgStd0);

	// read the parameters
	//strcpy(name, g_pchSketchParamName);
	//if (m_param.read(name) != 0)
	//	m_param.save(name);

//	m_param.m_dMagThresh1 *= 255;//provided is percentage
	

	SkBranchProfile::m_dImgMean0 = m_dImgMean0;
	SkBranchProfile::m_dImgStd0 = m_dImgStd0;

	SkPrimBranch::m_nHalfWd = g_nHalfProfileWd;//profile hald width


	return 0;
}

///////////////////
int SkPrimPursuit::primPursuitWithoutPrior()
{
	char msg[512];
	sprintf(msg, "                   without prior...... ");
	::skOutput(msg);

	DWORD Time0 = ::GetTickCount();
	///////////////////////////////////////////////////
	// hysterisis growing
	//3333333333333333333333333333333333333333333333333333333333333
	initialize4HysteresisCurveGrowing();
	initializePrimitiveManager();


	primPursuitWithFilterCoef();

	m_primMan.buildBranchSegments();
	m_primMan.buildBranchProfiles();
	m_primMan.computeImageCodingLengthGain4EachBranch();
	m_primMan.sortBrachesByCodeingLengthGain();

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "  %d min %.3fs\n", nMin, dSec);
	::skOutput(msg);

	// output
//	savePursuitResults(m_nProcStep);

	return 0;
}
int SkPrimPursuit::combineConnectedFreeEndPointsIntoOneBranch()
{
	m_primMan.combineConnectedFreeEndPointsIntoOneBranch();

	m_primMan.buildBranchSegments();
	m_primMan.buildBranchProfiles();
	m_primMan.computeImageCodingLengthGain4EachBranch();
	m_primMan.sortBrachesByCodeingLengthGain();
	return 0;
}

///////////////////
int SkPrimPursuit::primPursuitByGraphOperations()
{
	char msg[512];

	DWORD Time0 = ::GetTickCount();
	int nRemoved, nTotal;

	nTotal = m_primMan.getBranchNum();
//	sprintf(msg, "                  %d total curves. \n", nTotal);
//	::skOutput(msg);
	sprintf(msg, "                  with graph operations......\n");
	::skOutput(msg);

	// add curves one by one
	refineCurvesByGraphOperations();

	m_primMan.buildBranchSegments();
	m_primMan.buildBranchProfiles();
	m_primMan.computeImageCodingLengthGain4EachBranch();
	m_primMan.sortBrachesByCodeingLengthGain();



	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "  %d min %.3fs \n", nMin, dSec);
	::skOutput(msg);

	// output
//	savePursuitResults(m_nProcStep);
	return 0;
}
/*
///////////////////
int SkPrimPursuit::primPursuitWithPrior()
{
	char msg[512];

	DWORD Time0 = ::GetTickCount();
	int nRemoved, nTotal;

	nTotal = m_primMan.getBranchNum();
//	sprintf(msg, "                  %d total curves. \n", nTotal);
//	::skOutput(msg);
	sprintf(msg, "                  with prior......\n");
	::skOutput(msg);
//4444444444444444444444444444444444444444444444444444444444444444
//	blobPursuitWithFilterCoef();


	// add curves one by one
	refineCurvesOneByOne();
//	refineCurvesOneByOne_woimg();
	

	m_primMan.buildBranchSegments();
	m_primMan.buildBranchProfiles();
	m_primMan.computeImageCodingLengthGain4EachBranch();
	m_primMan.sortBrachesByCodeingLengthGain();



	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "  %d min %.3fs \n", nMin, dSec);
	::skOutput(msg);

	// output
//	savePursuitResults(m_nProcStep);
	return 0;
}
*/
///////////////////
int SkPrimPursuit::removeLowImageCLGBranches(double dMinImgCLG)
{
	SkBranchMap &branchMap = m_primMan.m_branchMap;
	int n = branchMap.getValidNodeNum();
	int i;
	for (i = 0; i < n; i++)
	{
		int id = m_primMan.m_sortedBhIDs[i];

		SkBranchNode *pBhNode = branchMap.getNodeAt(id);
		if (!pBhNode->valid())
			continue;//all are invalid first

		double dImgCLG = pBhNode->m_data.getImageCodingLengthGain();
		if (dImgCLG < dMinImgCLG)
			pBhNode->setValid(false);
	}
	return 0;

}
/*
///////////////////
int SkPrimPursuit::refineCurvesOneByOne_woimg()
{
	SkBranchMap &branchMap = m_primMan.m_branchMap;
	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;

	int n = branchMap.getValidNodeNum();


	int ht, wd;
	m_imgOrg.GetDimension(ht, wd);

	Matrix<oval_IntArray>& extOcp = m_primMan.m_extOcp;//extension occupy branch id
	Matrix<oval_IntArray>& extEndOcp = m_primMan.m_extEndOcp;//the extension end of correspondent branch
	Matrix<oval_IntArray>& conOcp = m_primMan.m_conOcp;//the constraints (7 pixel width ) occupy
	Matrix<SkPixCandArray>& rec_cand = m_primMan.m_pixCandMat; //reconstruction candidate
	Matrix<int> &vpOcp = m_primMan.m_vpID;
	Matrix<oval_IntArray>& skOcp = m_primMan.m_branchIDs;//the sketch (1 pixel width) occupy

	m_primMan.m_vpMap.removeAll();//saves the virtural primitives
	m_primMan.initializeOcpIDMatrices();
	extOcp.SetDimension(1,1); extOcp.SetDimension(ht, wd);
	extEndOcp.SetDimension(1,1); extEndOcp.SetDimension(ht, wd);
	conOcp.SetDimension(1,1); conOcp.SetDimension(ht, wd);
	rec_cand.SetDimension(1,1); rec_cand.SetDimension(ht, wd);
	

#define MAX_EXT_PIXEL 32*32
#define MAX_SK_PIXEL 1024*4
#define MAX_PRFL_WD 13

	int pnExtPtNum[2];
	int ppnExtX[2][MAX_EXT_PIXEL], ppnExtY[2][MAX_EXT_PIXEL];
	int skPtNum;
	int pnSkX[MAX_SK_PIXEL], pnSkY[MAX_SK_PIXEL];
	int conPtNum;
	int pnConX[MAX_PRFL_WD*MAX_SK_PIXEL], pnConY[MAX_PRFL_WD*MAX_SK_PIXEL];


	m_primMan.m_branchMap.removeInvalid();
	m_primMan.m_branchMap.setValid(false);

	oval_PointArray extVpPosArr;
	oval_IntArray extVpIDArr;

	oval_PointArray extSkPosArr;
	oval_Array<oval_IntArray> extSkIDArr;
	
	oval_PointArray extExtPosArr;
	oval_Array<oval_IntArray> extExtIDArr;
	oval_Array<oval_IntArray> extExtEndArr;

	oval_Array<SkEndConnection> cnctArr;

	m_primMan.m_bUpdateExtCon = true;

	int hf_prfl = g_nHalfProfileWd;
	int extDis = m_param.m_dExtensionDist4Refine;//four overhead
	int i;
	int e;
	SkIntPoint endPt;
	int nPossCnct;
	SkEndConnection validCnct;
	int nKeeped = 0;
	for (i = 0; i < n; i++)
	{
		int id = m_primMan.m_sortedBhIDs[i];

		pBhNode = branchMap.getNodeAt(id);
//		if (!pBhNode->valid())
//			continue;//all are invalid first
		pBranch = &(pBhNode->m_data);

		if (pBranch->getPointNum() <= 1 )
			continue;//one pixel only
		if (pBranch->getPointNum() == 2)
		{// has two same pixel
			SkIntPoint &pt1 = pBranch->getEndPoint(0);
			SkIntPoint &pt2 = pBranch->getEndPoint(1);
			if (pt1.x == pt2.x && pt1.y == pt2.y)
				continue;
		}
		if (pBranch->getPrimID(0) == pBranch->getPrimID(1)
			&& pBranch->getPrimID(0) >= 0)
		{//closed curve but less than 4 points
			if (pBranch->getPointNum() <= 4)
				continue;
		}

		double dImgCLG = pBranch->getImageCodingLengthGain();
		if (dImgCLG < m_param.m_dMinImgCLG 
			&& pBranch->getPointNum() < m_param.m_dExtensionDist4Refine
			&& pBranch->getPrimID(0) < 0
			&& pBranch->getPrimID(1) < 0)
		{//low image CLG & short & two free end
			continue;
		}

		if (dImgCLG < m_param.m_dMinImgCLG - 1.0)
			break;

		for (e = 0; e < 2; e++)
			pBranch->setPrimID(e,-1);//free all end

		findSketchPixels(pBranch, skPtNum, pnSkX, pnSkY);
		if (wholellyInhibited(skPtNum, pnSkX, pnSkY, conOcp))
		{//inhibition testing, leads to combing parallel ones
			pBhNode->setValid(false);
			continue;
		}
		updateGeneralOcp(skOcp, id, skPtNum, pnSkX, pnSkY, ADD);
		pBhNode->setValid(true);

		findConPixels(pBranch, conPtNum, pnConX, pnConY, hf_prfl);

		for (e = 0; e < 2; e++)
		{
			pBranch = branchMap.getAt(id);// memory might be reallocated
			if (pBranch->getPrimID(e) >= 0)
				continue;//not free end
			endPt = pBranch->getEndPoint(e);
			//check 1~2 pixel connections 
			bool bFindValidCnct = false;
//			for (int k = 1; k < 2; k++)
			{
//				if (k == 0)
//					findNearbyPixels(pBranch, e, pnExtPtNum[e], ppnExtX[e], ppnExtY[e]);
//				else
					findExtensionPixels(pBranch, e, pnExtPtNum[e], ppnExtX[e], ppnExtY[e], extDis);

				findCrossedVP(pnExtPtNum[e], ppnExtX[e], ppnExtY[e], 
								vpOcp, extVpPosArr, extVpIDArr);
				findCrossedSk(pnExtPtNum[e], ppnExtX[e], ppnExtY[e], 
								skOcp, extSkPosArr, extSkIDArr);
				findCrossedExt(pnExtPtNum[e], ppnExtX[e], ppnExtY[e],
								extOcp, extEndOcp, extExtPosArr, extExtIDArr, extExtEndArr);
				findAllPossibleConnections(cnctArr, id, e, 
											extVpPosArr, extVpIDArr, //VP
											extSkPosArr, extSkIDArr, //sk
											extExtPosArr, extExtIDArr, extExtEndArr);
				removeDuplicateConnection(cnctArr);
				m_primMan.removeCrossedConnection(cnctArr);
				nPossCnct = cnctArr.getSize();
				m_primMan.computeConnectionEnergy_woimg(cnctArr);
				sortConnectionEnergy(cnctArr);
				if (nPossCnct > 0)
				{
					validCnct = cnctArr[0];
					bFindValidCnct = true;
					if (m_param.m_nSaveIntermediateResults)
					{
						char msg[512];
						m_primMan.outputEndConnectionInfo(msg, validCnct);
						skOutput(msg);
					}
				}
				else 
					bFindValidCnct = false;
			}
			if (bFindValidCnct)
			{//there are possible connections,
				SkEndConnection &cnct = validCnct;//choose the best one
				if (cnct.m_type == 0)
				{//VP
					//only update the con, not ext
					updateGeneralOcp(conOcp, id, conPtNum, pnConX, pnConY, ADD);
					if(m_primMan.extendBranchEnd2AJunction(id, e, cnct.m_x, cnct.m_y)==0)
						updateGeneralOcp(conOcp, id, endPt.x, endPt.y, cnct.m_x, cnct.m_y, ADD);
					m_primMan.updatePixCand4ABranch(pBranch, rec_cand, ADD, false);
				}
				else if (cnct.m_type == 1)
				{//sk
					m_primMan.extendBranchEnd2ABranch(id, e,  cnct.m_x,  cnct.m_y);
					e = -1;//we need to check the two ends of this branch again Nov. 18

					m_primMan.buildBranchSegments();//re-build all segments
					m_primMan.buildBranchProfiles();//rebuild all profile models
					m_primMan.computePixCand(rec_cand);//re-compute pixel cand
					recomputeExtConOcp(m_primMan.m_branchMap, conOcp,extOcp, 
						extEndOcp, hf_prfl, extDis);//recompute ext and con ocp
				}
				else if (cnct.m_type == 2)
				{// sk extension
					int id1 = cnct.m_id1;
					int end1 = cnct.m_end1;
					SkBranchNode *pBhNode1 = branchMap.getNodeAt(id1);
					findExtensionPixels(&(pBhNode1->m_data), end1, pnExtPtNum[end1], ppnExtX[end1], 
						ppnExtY[end1], extDis);

					updateExtensionOcp(extOcp, extEndOcp, id1, end1, 
						pnExtPtNum[end1], ppnExtX[end1], ppnExtY[end1], REMOVE);
					m_primMan.extendBranchEnd2ABranchExtension(id, e, id1, end1,  cnct.m_x,  cnct.m_y);
					
					//re-build all
					m_primMan.buildBranchSegments();//re-build all segments
					m_primMan.buildBranchProfiles();//rebuild all profile models
					m_primMan.computePixCand(rec_cand);//re-compute pixel cand
					recomputeExtConOcp(m_primMan.m_branchMap, conOcp,extOcp, 
						extEndOcp, hf_prfl, extDis);//recompute ext and con ocp
				}
				else 
				{
					assert(false);
				}
			}//with potential extension
			else
			{//no possible connection, 
				// consider inhibition, parrellism
				// .... to be added

				// add all ocps, might be multiple added
				updateExtensionOcp(extOcp, extEndOcp, id, e, 
					pnExtPtNum[e], ppnExtX[e], ppnExtY[e], ADD);
				updateGeneralOcp(conOcp, id, conPtNum, pnConX, pnConY, ADD);
				m_primMan.updatePixCand4ABranch(pBranch, rec_cand, ADD, false);
			}

		}//for each end
		// consider other extension to this sketch
		oval_IntArray bhIDArr, bhEndArr;
		oval_Array<oval_PointArray> ptArrayArr;
		findOtherExtension2ThisSketch(skPtNum, pnSkX, pnSkY, id, extOcp, extEndOcp, bhIDArr, bhEndArr, ptArrayArr);
		int nOpenEnd = bhIDArr.getSize();
		for (int kk = 0; kk < nOpenEnd; kk++)
		{
			findAllPossibleConnections(cnctArr, bhIDArr[kk], bhEndArr[kk], ptArrayArr[kk], id);
			removeDuplicateConnection(cnctArr);
			m_primMan.removeCrossedConnection(cnctArr);
			m_primMan.computeConnectionEnergy_woimg(cnctArr);
			sortConnectionEnergy(cnctArr);
			nPossCnct = cnctArr.getSize();
			bool bFindValidCnct = false;
			if (nPossCnct > 0)
			{
				validCnct = cnctArr[0];
				bFindValidCnct = true;
				char msg[512];
				m_primMan.outputEndConnectionInfo(msg, validCnct);
			}
			if (bFindValidCnct)
			{//there are possible connections,
				if (validCnct.m_type == 1)
				{//sk
					int nOpenID = validCnct.m_id0;
					int nOpenEnd = validCnct.m_end0;
					SkBranchNode *pOpenBhNode = m_primMan.m_branchMap.getNodeAt(nOpenID);
					endPt = pOpenBhNode->m_data.getEndPoint(nOpenEnd);
					
					// that open end will be closed, remove all extension ocp
					findExtensionPixels(&(pOpenBhNode->m_data), nOpenEnd, pnExtPtNum[nOpenEnd], ppnExtX[nOpenEnd], ppnExtY[nOpenEnd], extDis);
					updateExtensionOcp(extOcp, extEndOcp, nOpenID, nOpenEnd, 
						pnExtPtNum[nOpenEnd], ppnExtX[nOpenEnd], ppnExtY[nOpenEnd], REMOVE);

					m_primMan.extendBranchEnd2ABranch(nOpenID, nOpenEnd,  validCnct.m_x,  validCnct.m_y);//both sk, con are updated

					m_primMan.buildBranchSegments();//re-build all segments
					m_primMan.buildBranchProfiles();//rebuild all profile models
					m_primMan.computePixCand(rec_cand);//re-compute pixel cand
					recomputeExtConOcp(m_primMan.m_branchMap, conOcp,extOcp, 
						extEndOcp, hf_prfl, extDis);
					//only update the con, not ext
				}
				else 
				{
					assert(false);
				}
			}//valid extension
		}//for each open end 
//		saveRefinedSketchImage(rec_cand, nKeeped);
//		saveRefinedCurves(nKeeped);
		if (nKeeped == 6)
			nKeeped += 0;
		nKeeped++;
	}//for each branch/curve
	return 0;
}
*/
///////////////////
int SkPrimPursuit::refineCurvesByGraphOperations()
{
	/*
	SkBranchMap &branchMap = m_primMan.m_branchMap;
	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;

	int ht, wd;
	m_imgOrg.GetDimension(ht, wd);

	int hf_prfl = g_nHalfProfileWd;
	int extDis = m_param.m_dExtensionDist4Refine;//four overhead


	m_primMan.m_bUpdateExtCon = true;
	m_primMan.computePixCand(rec_cand);//re-compute pixel cand
	m_primMan.recomputeOcpIDMatrices();
	m_primMan.recomputeExtConOcp(hf_prfl, extDis);//recompute ext and con ocp
	

#define MAX_EXT_PIXEL 32*32
#define MAX_SK_PIXEL 1024*4
#define MAX_PRFL_WD 13

	int pnExtPtNum[2];
	int ppnExtX[2][MAX_EXT_PIXEL], ppnExtY[2][MAX_EXT_PIXEL];
	int skPtNum;
	int pnSkX[MAX_SK_PIXEL], pnSkY[MAX_SK_PIXEL];
	int conPtNum;
	int pnConX[MAX_PRFL_WD*MAX_SK_PIXEL], pnConY[MAX_PRFL_WD*MAX_SK_PIXEL];



	oval_PointArray extVpPosArr;
	oval_IntArray extVpIDArr;

	oval_PointArray extSkPosArr;
	oval_Array<oval_IntArray> extSkIDArr;
	
	oval_PointArray extExtPosArr;
	oval_Array<oval_IntArray> extExtIDArr;
	oval_Array<oval_IntArray> extExtEndArr;

	oval_Array<SkEndConnection> cnctArr;


	int i;
	int e;
	SkIntPoint endPt;
	int nPossCnct;
	SkEndConnection validCnct;
	int nKeeped = 0;
	int n = branchMap.getValidNodeNum();
	for (i = 0; i < n; i++)
	{
		int id = m_primMan.m_sortedBhIDs[i];

		pBhNode = branchMap.getNodeAt(id);
		if (!pBhNode->valid())
			continue;//all are invalid first
		pBranch = &(pBhNode->m_data);

		if (pBranch->getPointNum() <= 1 )
		{
			m_primMan.removeOneBranch(id);
			recomputeExtConOcp(m_primMan.m_branchMap, conOcp,extOcp, 
					extEndOcp, hf_prfl, extDis);//recompute ext and con ocp
			continue;//one pixel only
		}
		if (pBranch->getPointNum() == 2)
		{// has two same pixel
			SkIntPoint &pt1 = pBranch->getEndPoint(0);
			SkIntPoint &pt2 = pBranch->getEndPoint(1);
			if (pt1.x == pt2.x && pt1.y == pt2.y)
			{
				m_primMan.removeOneBranch(id);
				recomputeExtConOcp(m_primMan.m_branchMap, conOcp,extOcp, 
						extEndOcp, hf_prfl, extDis);//recompute ext and con ocp
				continue;
			}
		}
		if (pBranch->getPrimID(0) == pBranch->getPrimID(1)
			&& pBranch->getPrimID(0) >= 0)
		{//closed curve but less than 4 points
			if (pBranch->getPointNum() <= 4)
			{
				m_primMan.removeOneBranch(id);
				recomputeExtConOcp(m_primMan.m_branchMap, conOcp,extOcp, 
						extEndOcp, hf_prfl, extDis);//recompute ext and con ocp
				continue;
			}
		}

		double dImgCLG = pBranch->getImageCodingLengthGain();
		if (dImgCLG < m_param.m_dMinImgCLG 
			&& pBranch->getPointNum() < m_param.m_dExtensionDist4Refine
			&& pBranch->getPrimID(0) < 0
			&& pBranch->getPrimID(1) < 0)
		{//low image CLG & short & two free end
			m_primMan.removeOneBranch(id);
			recomputeExtConOcp(m_primMan.m_branchMap, conOcp,extOcp, 
					extEndOcp, hf_prfl, extDis);//recompute ext and con ocp
			continue;
		}


		findSketchPixels(pBranch, skPtNum, pnSkX, pnSkY);
//		if (wholellyInhibited(skPtNum, pnSkX, pnSkY, conOcp))
//		{//inhibition testing, leads to combing parallel ones
//			m_primMan.removeOneBranch(id);
//			recomputeExtConOcp(m_primMan.m_branchMap, conOcp,extOcp, 
//					extEndOcp, hf_prfl, extDis);//recompute ext and con ocp
//			continue;
//		}

		findConPixels(pBranch, conPtNum, pnConX, pnConY, hf_prfl);

		for (e = 0; e < 2; e++)
		{
			pBhNode = branchMap.getNodeAt(id);
			if (!pBhNode->valid())
				continue;//all are invalid first
			if (pBranch->getPrimID(e) >= 0)
				continue;//not free end
			endPt = pBranch->getEndPoint(e);
			//check 1~2 pixel connections 
			bool bFindValidCnct = false;
			findExtensionPixels(pBranch, e, pnExtPtNum[e], ppnExtX[e], ppnExtY[e], extDis);

			findCrossedVP(pnExtPtNum[e], ppnExtX[e], ppnExtY[e], 
							vpOcp, extVpPosArr, extVpIDArr);
			findCrossedSk(pnExtPtNum[e], ppnExtX[e], ppnExtY[e], 
							skOcp, extSkPosArr, extSkIDArr);
			findCrossedExt(pnExtPtNum[e], ppnExtX[e], ppnExtY[e],
							extOcp, extEndOcp, extExtPosArr, extExtIDArr, extExtEndArr);
			findAllPossibleConnections(cnctArr, id, e, 
										extVpPosArr, extVpIDArr, //VP
										extSkPosArr, extSkIDArr, //sk
										extExtPosArr, extExtIDArr, extExtEndArr);
			removeDuplicateConnection(cnctArr);
			m_primMan.removeCrossedConnection(cnctArr);
			m_primMan.computeConnectionEnergy(cnctArr);
			sortConnectionEnergy(cnctArr);
			nPossCnct = cnctArr.getSize();
			if (nPossCnct > 0)
			{
				validCnct = cnctArr[0];
				bFindValidCnct = true;
				char msg[512];
				sprintf(msg, "%4d:",nKeeped);
				skOutput(msg);
				m_primMan.outputEndConnectionInfo(msg, validCnct);
				skOutput(msg);
			}
			else 
				bFindValidCnct = false;
		
			if (bFindValidCnct && -validCnct.m_dEnergy >= m_param.m_dMinTotalCLG)
			{//there are possible connections,
				SkEndConnection &cnct = validCnct;//choose the best one
				if (cnct.m_type == 0)
				{//VP
					//only update the con, not ext
					if(m_primMan.extendBranchEnd2AJunction(id, e, cnct.m_x, cnct.m_y)==0)
					{
						//re-build all
						m_primMan.buildBranchSegments();//re-build all segments
						m_primMan.buildBranchProfiles();//rebuild all profile models
						m_primMan.computePixCand(rec_cand);//re-compute pixel cand
						m_primMan.recomputeOcpIDMatrices();
						recomputeExtConOcp(m_primMan.m_branchMap, conOcp,extOcp, 
							extEndOcp, hf_prfl, extDis);//recompute ext and con ocp

					}
				}
				else if (cnct.m_type == 1)
				{//sk
					if (m_primMan.extendBranchEnd2ABranch(id, e,  cnct.m_x,  cnct.m_y) >= 0)
					{
						e = -1;//we need to check the two ends of this branch again Nov. 18

						m_primMan.buildBranchSegments();//re-build all segments
						m_primMan.buildBranchProfiles();//rebuild all profile models
						m_primMan.computePixCand(rec_cand);//re-compute pixel cand
						m_primMan.recomputeOcpIDMatrices();
						recomputeExtConOcp(m_primMan.m_branchMap, conOcp,extOcp, 
							extEndOcp, hf_prfl, extDis);//recompute ext and con ocp
					}
				}
				else if (cnct.m_type == 2)
				{// sk extension
					int id1 = cnct.m_id1;
					int end1 = cnct.m_end1;

					if (m_primMan.extendBranchEnd2ABranchExtension(id, e, id1, end1,  cnct.m_x,  cnct.m_y) >= 0)
					{
						//re-build all
						m_primMan.buildBranchSegments();//re-build all segments
						m_primMan.buildBranchProfiles();//rebuild all profile models
						m_primMan.computePixCand(rec_cand);//re-compute pixel cand
						m_primMan.recomputeOcpIDMatrices();
						recomputeExtConOcp(m_primMan.m_branchMap, conOcp,extOcp, 
							extEndOcp, hf_prfl, extDis);//recompute ext and con ocp
					}
				}
				else 
				{
					assert(false);
				}
			}//with potential extension
			else
			{//no possible connection, 
				// consider inhibition, parrellism
				// .... to be added

			}

		}//for each end
		// consider other extension to this sketch
		saveRefinedSketchImage(rec_cand, nKeeped);
		saveRefinedCurves(nKeeped);
		if (nKeeped == 6)
			nKeeped += 0;
		nKeeped++;
	}//for each branch/curve
	*/
	return 0;
}
///////////////////
int SkPrimPursuit::diffuseBranchPoints()
{
	m_primMan.diffuseBranchPoints();

//	m_primMan.buildBranchSegments();
	m_primMan.buildBranchProfiles();
	m_primMan.computeImageCodingLengthGain4EachBranch();
	m_primMan.sortBrachesByCodeingLengthGain();

	return 0;
}
int SkPrimPursuit::removeSmallImgCLGBranches()
{	// remove branches with small img CLG
	m_primMan.removeSmallImgCLGBranches(m_param.m_dMinImgCLG);
//	m_primMan.combineConnectedFreeEndPointsIntoOneBranch();
//	m_primMan.removeDegree1And2VP();

	m_primMan.buildBranchSegments();//re-build all segments
	m_primMan.buildBranchProfiles();//rebuild all profile models
	m_primMan.computeImageCodingLengthGain4EachBranch();
	m_primMan.sortBrachesByCodeingLengthGain();
	return 0;
}
///////////////////
int SkPrimPursuit::refineSketchGraphByRemovingRedundentPoints()
{
	m_primMan.recomputeOcpIDMatrices();
	Matrix<int> mat;
	changeOcp2Binary(m_primMan.m_branchIDs, mat);


	::skThinnerBoundary(mat);

	initializePrimitiveManager();
	m_primMan.setBound(mat);
	m_primMan.buildPrimitiveMap();

	m_primMan.buildBranchSegments();
	m_primMan.buildBranchProfiles();
	m_primMan.computeImageCodingLengthGain4EachBranch();
	m_primMan.sortBrachesByCodeingLengthGain();

	return 0;

}
///////////////////
int SkPrimPursuit::combineConnectedVirtualPrimitives()
{
	m_primMan.recomputeOcpIDMatrices();
	m_primMan.combineConnectedVirtualPrimitives();

	m_primMan.buildBranchSegments();
	m_primMan.buildBranchProfiles();
	m_primMan.computeImageCodingLengthGain4EachBranch();
	m_primMan.sortBrachesByCodeingLengthGain();

	return 0;

}



///////////////////
int SkPrimPursuit::findCrossedVP(int num, int *pnX, int *pnY, Matrix<int> &vpOcp, 
								 oval_PointArray& ptArr, oval_IntArray &idArr)
{
	ptArr.removeAll();
	idArr.removeAll();

	int ht, wd;
	vpOcp.GetDimension(ht, wd);
	int i;
	for (i = 0; i < num; i++)
	{
		int x = pnX[i];
		int y = pnY[i];
		if (x < 0 || x >= wd || y < 0 || y >= ht)
			continue;
		int id = vpOcp(y,x);
		if (id >= 0)
		{
			oval_Point p;
			p.x = x; 
			p.y = y;
			if (!idArr.hasMember(id))
			{
				ptArr.add(p);
				idArr.add(id);
			}
		}
	}
	return ptArr.getSize();
}
///////////////////
int SkPrimPursuit::findCrossedSk(int num, int *pnX, int *pnY, Matrix<oval_IntArray> &skOcp, 
								 oval_PointArray& ptArr, oval_Array<oval_IntArray> &idArr)
{	
	ptArr.removeAll();
	idArr.removeAll();
	int ht, wd;
	skOcp.GetDimension(ht, wd);
	int i;
	for (i = 0; i < num; i++)
	{
		int x = pnX[i];
		int y = pnY[i];
		if (x < 0 || x >= wd || y < 0 || y >= ht)
			continue;
		if (skOcp(y,x).getSize() > 0)
		{
			oval_Point p;
			p.x = x; 
			p.y = y;
			ptArr.add(p);
			idArr.add(skOcp(y,x));
		}
	}
	return ptArr.getSize();
}
///////////////////
int SkPrimPursuit::findCrossedExt(int num, int *pnX, int *pnY, Matrix<oval_IntArray> &extOcp,
								  Matrix<oval_IntArray> &extEndOcp,
								 oval_PointArray& ptArr, oval_Array<oval_IntArray> &idArr,
								 oval_Array<oval_IntArray> &endArr)
{
	ptArr.removeAll();
	idArr.removeAll();
	endArr.removeAll();

	int ht, wd;
	extOcp.GetDimension(ht, wd);
	int i;
	for (i = 0; i < num; i++)
	{
		int x = pnX[i];
		int y = pnY[i];
		if (x < 0 || x >= wd || y < 0 || y >= ht)
			continue;
		if (extOcp(y,x).getSize() > 0)
		{
			oval_Point p;
			p.x = x; 
			p.y = y;
			ptArr.add(p);
			idArr.add(extOcp(y,x));
			endArr.add(extEndOcp(y,x));
		}
	}
	return ptArr.getSize();
}
///////////////////
int SkPrimPursuit::findAllPossibleConnections(oval_Array<SkEndConnection> &cnctArr, 
											  int id0, int end0, //the interested branch id and end
											  oval_PointArray &extVpPosArr, 
											  oval_IntArray &extVpIDArr, //VP
											  oval_PointArray &extSkPosArr,
											  oval_Array<oval_IntArray> &extSkIDArr, //sk
											  oval_PointArray &extExtPosArr, 
											  oval_Array<oval_IntArray> &extExtIDArr, 
											  oval_Array<oval_IntArray> &extExtEndArr)//ext
{
	cnctArr.RemoveAll();

	SkEndConnection cnct;

	int id1, end1;
	int i;
	//for all VP
	int vpNum = extVpPosArr.getSize();
	for (i = 0; i < vpNum; i++)
	{
		oval_Point &p = extVpPosArr[i];

		cnct.m_id0 = id0;
		cnct.m_end0 = end0;
		cnct.m_id1 = extVpIDArr[i];
		cnct.m_end1 = -1;
		cnct.m_x = p.x;
		cnct.m_y = p.y;

		cnct.m_type = 0;//connection type, 0~VP, 1~curve, 2~curve extension

		cnctArr.add(cnct);

	}
	// for all SK
	int skNum = extSkPosArr.getSize();
	for (i = 0; i < skNum; i++)
	{
		oval_Point &p = extSkPosArr[i];
		oval_IntArray &skIDArr = extSkIDArr[i];
		int n = skIDArr.getSize();

		if (n > 1)
		{
			int vpID = m_primMan.m_vpID(p.y,p.x);
//			assert(vpID >= 0);
//			assert(extVpIDArr.hasMember(vpID));
			continue;
		}
		assert(n == 1);

		cnct.m_id0 = id0;
		cnct.m_end0 = end0;
		cnct.m_id1 = skIDArr[0];
		cnct.m_end1 = -1;
		cnct.m_x = p.x;
		cnct.m_y = p.y;

		cnct.m_type = 1;//connection type, 0~VP, 1~curve, 2~curve extension

		cnctArr.add(cnct);

	}
	// for all ext
	int extNum = extExtPosArr.getSize();
	for (i = 0; i < extNum; i++)
	{
		oval_Point &p = extExtPosArr[i];

		oval_IntArray &extIDArr = extExtIDArr[i];
		oval_IntArray &extEndArr = extExtEndArr[i];
		int n = extIDArr.getSize();
		for (int k = 0; k < n; k++)
		{
			cnct.m_id0 = id0;
			cnct.m_end0 = end0;
			cnct.m_id1 = extIDArr[k];
			cnct.m_end1 = extEndArr[k];
			cnct.m_x = p.x;
			cnct.m_y = p.y;

			cnct.m_type = 2;//connection type, 0~VP, 1~curve, 2~curve extension

			cnctArr.add(cnct);
		}

	}
	return 0;
}

///////////////////
// consider one branch (id1) is overlapped with extensions 0f (id0, en0)
int SkPrimPursuit::findAllPossibleConnections(oval_Array<SkEndConnection> &cnctArr, 
											  int id0, int end0, //
											  oval_PointArray &crossPtArr,//the points on the branch (id1)
											  int id1)
{
	cnctArr.RemoveAll();

	SkEndConnection cnct;

	int nPtNum = crossPtArr.getSize();
	int i;
	for (i = 0; i < nPtNum; i++)
	{
		oval_Point & p = crossPtArr[i];

		cnct.m_id0 = id0;
		cnct.m_end0 = end0;
		cnct.m_id1 = id1;
		cnct.m_end1 = -1;
		cnct.m_x = p.x;
		cnct.m_y = p.y;

		cnct.m_type = 1;//connection type, 0~VP, 1~curve, 2~curve extension

		cnctArr.add(cnct);

	}
	return cnctArr.getSize();

}
///////////////////
int SkPrimPursuit::findOtherExtension2ThisSketch(int nSkPtNum, int *pnSkX, int *pnSkY, int id,//the point on the sketch
												 Matrix<oval_IntArray> &extOcp, 
												 Matrix<oval_IntArray> &extEndOcp,//the extension branch id and end ocps
												 oval_IntArray &bhIDArr, //found branch id array
												 oval_IntArray &bhEndArr,//according end array
												 oval_Array<oval_PointArray> &ptArrayArr)//the points for each id & end
{
	int ht, wd;
	oval_Point p;
	extOcp.GetDimension(ht, wd);
	oval_PointArray ptArr0;//no pt
	int i;
	for (i = 0; i < nSkPtNum; i++)
	{
		int x = pnSkX[i];
		int y = pnSkY[i];
		p.x = x;
		p.y = y;
		if (x < 0 || x >= wd || y < 0 || y >= ht)
			continue;
		oval_IntArray &idArr0 = extOcp(y,x);
		oval_IntArray &endArr0 = extEndOcp(y,x);
		int n = idArr0.getSize();
		for (int j = 0; j < n; j++)
		{
			int id0 = idArr0[j];
			int end0 = endArr0[j];//get id and end at this point

			if (id == id0)
				continue;//not consider itself

			//below just for that ext/con bug (not kept consistant)
			// the id and end in the extOcp might always be valid
			SkPrimBranch *pBranch = m_primMan.m_branchMap.getAt(id0);
			if (pBranch->getPrimID(end0) >= 0)
				continue;
			
			int m = bhIDArr.getSize();
			bool bExist = false;
			int nExistPos = -1;
			for (int k = 0; k < m; k++)
			{//check already exist or not
				if (id0 == bhIDArr[k] && end0 == bhEndArr[k])
				{//exist
					bExist = true;
					nExistPos = k;
				}
			}
			if (!bExist)
			{//new id and end point
				bhIDArr.add(id0);
				bhEndArr.add(end0);
				int pos = ptArrayArr.add(ptArr0);
				ptArrayArr[pos].add(p);
			}
			else
			{
				assert(nExistPos >= 0);
				ptArrayArr[nExistPos].add(p);//just add a new point
			}
		}//for each ext at this point
	}//for each point on the sketch
	return bhIDArr.getSize();
}


int SkPrimPursuit::removeDuplicateConnection(oval_Array<SkEndConnection> &cnctArray)
{
	int nRemoved = 0;
	for (int i = 0; i < cnctArray.getSize()-1; i++)
	{
		for (int j = i+1; j < cnctArray.getSize(); j++)
		{
			if (cnctArray[i] == cnctArray[j])
			{
				cnctArray.removeAt(j);
				j--;
				nRemoved++;
			}
		}
		
	}
	return nRemoved;
}


///////////////////
bool SkPrimPursuit::wholellyInhibited(int nPtNum, int *pnX, int *pnY, Matrix<oval_IntArray> &ocp)
{
	int ht, wd;
	ocp.GetDimension(ht, wd);

	int x, y;
	for (int i = 0; i < nPtNum; i++)
	{
		x = pnX[i];
		y = pnY[i];
		if (x < 0 || x >= wd || y < 0 || y >= ht)
			continue;
		if (ocp(y,x).getSize() == 0)
			return false;
	}
	return true;
}
///////////////////
int SkPrimPursuit::sortConnectionEnergy(oval_Array<SkEndConnection> &cnctArray)
{
	int n = cnctArray.getSize();
	for (int i = 0; i < n-1; i++)
		for (int j = i+1; j < n; j++)
			if (cnctArray[i].m_dEnergy > cnctArray[j].m_dEnergy)
				::swap(cnctArray[i], cnctArray[j]);
	return 0;
}								 

///////////////////
int SkPrimPursuit::saveRefinedCurves(int i)
{
	char work_dir[512];
	sprintf(work_dir, g_pchSketchWorkDir);
	_mkdir(work_dir);

	int nSaveStep = m_param.m_nSaveCurveByCurveStep;
	int nProcStep = m_nProcStep;//process step

	double dMTh = m_param.m_dMagThresh1;///255;
	int nMTh = (int)(dMTh*100);
	char mth_no[32];	::changeNumTo2Char(mth_no, nMTh);

	char type[32];

	sprintf(type, "%d_m%s", nProcStep, mth_no);

	char no[32], name[32];
	int num = i+1; 
	Matrix<int> mat;
	changeOcp2Binary(m_primMan.m_branchIDs, mat);
	if (num/nSaveStep * nSaveStep == num)
	{
		::changeNumTo4Char(no, num);
		sprintf(name, "%s\\%s_s%s.bmp", work_dir, m_pchProjName, no);
		::skSaveMatrixAsBitmap(name, mat, 0, -1);
	}
	return 0;
}
///////////////////
int SkPrimPursuit::saveRefinedSketchImage(Matrix<SkPixCandArray> &rec_cand, int i)
{
	char work_dir[512];
	sprintf(work_dir, g_pchSketchWorkDir);
	_mkdir(work_dir);

	int nSaveStep = m_param.m_nSaveCurveByCurveStep;

	char no[32], name[32];
	int num = i+1; 
	int ht, wd;
	getImageDimension(ht, wd);
	Matrix<float> mat;
	mat.SetDimension(ht, wd);
	mat.Set(0);

	skComputeImageFromPixCandByMixing(rec_cand, mat);
	if (num/nSaveStep * nSaveStep == num)
	{
		::changeNumTo4Char(no, num);
		sprintf(name, "%s\\%s_rec_%s.bmp", work_dir, m_pchProjName, no);
		::skSaveMatrixAsBitmap(name, mat, m_dImgMean0, m_dImgStd0);
	}
	return 0;
}
///////////////////
int SkPrimPursuit::changeOcp2Binary(Matrix<oval_IntArray> &ocp, Matrix<int> &mat)
{
	int ht, wd;
	ocp.GetDimension(ht, wd);
	mat.SetDimension(ht, wd);
	int x, y;
	for (y = 0; y < ht; y++)
	{
		for (x = 0; x < wd; x++)
		{
			if (ocp(y,x).getSize() <= 0)
				mat(y,x) = 255;
			else
				mat(y,x) = 0;
		}
	}
	return 0;
}
///////////////////
int SkPrimPursuit::completePrimPursuit()
{
	_chdir(m_pchDir0);

	return 0;
}

///////////////////
int SkPrimPursuit::savePursuitResults(int step)
{
	char msg[512];
	sprintf(msg, "\n      saving results ......\n");
	::skOutput(msg);
	double dMTh = m_param.m_dMagThresh1;///255;
	int nMTh = (int)(dMTh*100);
	char mth_no[32];
	::changeNumTo2Char(mth_no, nMTh);
	char type[32];
	char w_no[32];
	int hf = SkPrimBranch::m_nHalfWd;
	::changeNumTo2Char(w_no, 2*hf+1);

	sprintf(type, "%d_m%s", step, mth_no);
	m_primMan.saveSketchGraphAsBmp(type,1, this);
	saveSketchImage(type);

	if (m_param.m_nSaveIntermediateResults)
		m_primMan.saveCurveByCurve(type, m_param.m_nSaveCurveByCurveStep);

	sprintf(type, "%d_m%s_w%s", step, mth_no, w_no);
	buildProfileSet();
	saveProfileSet(type);

	if (m_param.m_nSaveIntermediateResults)
		m_primMan.saveSingleJoints(type);

	sprintf(type, "%d", step);
//	m_primMan.saveResidualImagesStokeByStroke(type, m_param.m_nSaveCurveByCurveStep);
	return 0;
}

int SkPrimPursuit::initialize4HysteresisCurveGrowing()
{
	char name[512];

	char work_dir[32];
	strcpy(work_dir, g_pchProposalWorkDir);
	_mkdir(work_dir);

	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);


	//read sketch strength mag
	sprintf(name, "%s\\%s_sketch0.bmp", work_dir, m_pchProjName);
	::skReadBitmap(name, &m_edge);

	sprintf(name, "%s\\%s_sketch_mag1.bmp", work_dir, m_pchProjName);
	::skReadBitmap(name, &m_mag);

	sprintf(name, "%s\\%s_filter_type0.bmp", work_dir, m_pchProjName);
	::skReadBitmap(name, &m_filterType);

	m_magRes = m_mag;

	skRemoveIsolatedPoints(m_edge, 0, 255);

	//setNonMaxMagAsZero
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (m_edge(r,c) == 0)
			{
				m_edge(r,c) = POSSIBLE_EDGE;
			}
			else 
			{
				m_magRes(r,c) = -1;
				m_edge(r,c) = NOEDGE;
			}
		}
	}

	//At the border we say there can not be an edge so it makes the
    // follow_edges algorithm more efficient to not worry about tracking an
    // edge off the side of the image.
	for( r = 0; r <ht; r++)
	{
		m_edge(r,0) = NOEDGE;
		m_edge(r,wd-1) = NOEDGE;
	}
	for( c = 0; c <wd; c++)
	{
		m_edge(0,c) = NOEDGE;
		m_edge(ht-1,c) = NOEDGE;
	}

	return 0;
}

int SkPrimPursuit::initializePrimitiveManager()
{

	int ht, wd;
	m_imgOrg.GetDimension(ht, wd);

//	m_primMan.setBound(m_edgeFinal);
	m_primMan.setOrgImg(&m_imgOrg);
	m_primMan.setOrgImgMeanStd(m_dImgMean0, m_dImgStd0);
	m_primMan.setDir("");
	m_primMan.setProjName(m_pchProjName);
//	m_primMan.setMaxBranchLength(7);
//	m_primMan.setSaveLargeBranch(1);//only save large branch
//	m_primMan.setSaveLargeBranch(0);//save all branch
	m_primMan.removeAll();
	m_primMan.initializeOcpIDMatrices();
	m_primMan.m_param = m_param;

	m_primManTmp.setOrgImg(&m_imgOrg);
	m_primManTmp.m_param = m_param;


	return 0;
}
///////////////////
// primPursuitWithFilterCoef
int SkPrimPursuit::primPursuitWithFilterCoef()
{
	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);
	
	int nBlockSize = 50;
	SkDivideImage di;
	di.initialize(ht, wd, nBlockSize);

	int x1, y1, x2, y2;
	int block;

	int nReturn;
	while (true)
	{
		block = di.getNextBlock(x1, y1, x2, y2);
		if (block < 0)
			break;
		nReturn = primPursuitWithFilterCoefInOneBlockAndHysteresis(x1, y1, x2, y2);
		if (nReturn == 2)//max mag too small
			di.setBlockNotLeft(block);
	}
	return 0;
}

///////////////////
// primPursuitWithFilterCoef
int SkPrimPursuit::blobPursuitWithFilterCoef()
{
	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);
	
	int nBlockSize = 30;
	SkDivideImage di;
	di.initialize(ht, wd, nBlockSize);

	int x1, y1, x2, y2;
	int block;

	int nReturn;
//	while (true)
// {
//		block = di.getNextBlock(x1, y1, x2, y2);
//		if (block < 0)
//			break;
//		nReturn = blobPursuitWithFilterCoefInOneBlock(x1, y1, x2, y2);
//		if (nReturn == 2)//max mag too small
//			di.setBlockNotLeft(block);
//	}

	int r0 = 79;
	int c0 = 54;
	SkIntPoint p(c0, r0);
	SkPrimBlob blob;
	blob.setRefPoint(p);

	double dRad = 3;
	blob.setRadius(dRad);

	m_primMan.m_blobMap.add(blob);


	
	
	
	SkBranchMap &bhMap = m_primMan.m_branchMap;
	SkBranchNode *pBhNode;
	SkIntPoint pt[2];
	double dMaxDis = 5;
	// build nbrhoods
	int nPos = bhMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = bhMap.getNextNode(nPos);
		
		if (!pBhNode->valid())
			continue;
		bool bClose = true;
		for (int e = 0; e < 2; e++)
		{
			pt[e] = pBhNode->m_data.getEndPoint(e);
			if (::distanceBetweenPoints(pt[e].y, pt[e].x, r0, c0) > dMaxDis)
			{
				bClose = false;
				break;
			}
		}
		if (bClose)
			m_primMan.removeOneBranch(pBhNode->getID());
	}
	return 0;
}

void followEdges(unsigned char *edgemapptr, int *edgemagptr, int *edgetypeptr, int lowval, int cols, int nFilterType)
{
   int *tempmagptr;
   int *temptypeptr;
   unsigned char *tempmapptr;
   int i;
   float thethresh;
   int x[8] = {1,1,0,-1,-1,-1,0,1},
       y[8] = {0,1,1,1,0,-1,-1,-1};

   for(i=0;i<8;i++){
      tempmapptr = edgemapptr - y[i]*cols + x[i];
      tempmagptr = edgemagptr - y[i]*cols + x[i];
      temptypeptr = edgetypeptr - y[i]*cols + x[i];

      if((*tempmapptr == POSSIBLE_EDGE) && (*tempmagptr > lowval)
//		  && (*edgetypeptr == nFilterType) 
		)
	  {
         *tempmapptr = (unsigned char) EDGE;
         followEdges(tempmapptr,tempmagptr, temptypeptr, lowval, cols, nFilterType);
      }
   }
}
///////////////////
int SkPrimPursuit::primPursuitWithFilterCoefInOneBlockAndHysteresis(int x1, int y1, int x2, int y2)
{
	static int nCurveNum = 0;

	char name[512];
	int ht, wd;
	m_imgOrg.GetDimension(ht, wd);


	char sketch_dir[512];
	sprintf(sketch_dir, "sketches");
	_mkdir(sketch_dir);

	int r0, c0;

	// find the best position
	double dMaxMag;
	dMaxMag = ::skFindMaxPosition(m_magRes, x1, y1, x2, y2, r0, c0);
	if (dMaxMag < m_param.m_dMagThresh1*255)
		return 2;

	int r1, c1, r2, c2;//start and end points
	double dOrt1, dOrt2;//start and end orientation

	Matrix<unsigned char> edge0;//previous edge
	edge0 = m_edge;

	Matrix<int> edge_new;
	edge_new.SetDimension(ht, wd);
	edge_new.Set(255);

	//find all edges from the current point
	int highthreshold = m_magRes(r0,c0);
	int lowthreshold = highthreshold*0.5;
//	int lowthreshold = 0;
//	int lowthreshold = m_nLowEdgeThresh;
	int pos = r0*wd + c0;
	int nFilterType = m_filterType(r0, c0);
	if(m_edge(r0,c0) == POSSIBLE_EDGE)
	{
		m_edge(r0,c0) = EDGE;
        followEdges((m_edge.data+pos), (m_magRes.data+pos), (m_filterType.data+pos), lowthreshold, wd, nFilterType);
	}

	//get the new edges, update m_imgRes
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (edge0(r,c) != m_edge(r,c) )
			{
				edge_new(r,c) = 0;
				m_magRes(r,c) = -1;
			}
		}
	}

	::skThinnerBoundary(edge_new);

	addPrimitivesFromPointMap(edge_new);

	return 0;
}


///////////////////
int SkPrimPursuit::addPrimitivesFromPointMap(Matrix<int> &ptMat)
{// alert!!! This easy coding makes the speed extremely slow!!!!
	m_primManTmp.removeAll();
	m_primManTmp.setBound(ptMat);
	m_primManTmp.buildPrimitiveMap();
	m_primManTmp.buildBranchSegments();
	
	int ht, wd;
	m_imgOrg.GetDimension(ht, wd);

	Matrix<int> con;
	con.SetDimension(ht, wd);
	con.Set(255);
	m_primManTmp.saveSketchableLattice(con);//0 is valid
	skSetValueInMatrixMask(m_magRes, con, -1);//inhibition

	m_primMan.addPrimitivesFromOtherManager(m_primManTmp);
	// considering inhibition Oct. 01, 



	return 0;
}





int SkPrimPursuit::buildBranchNbrs()
{
	SkBranchMap &bhMap = m_primMan.m_branchMap;
	SkBranchNode *pBhNode;
	// build nbrhoods
	int nPos = bhMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = bhMap.getNextNode(nPos);
		
		if (!pBhNode->valid())
			continue;
		int id = pBhNode->getID();
		buildBranchNbrs(pBhNode);
	}
	return 0;
}
int SkPrimPursuit::buildBranchNbrs(SkBranchNode *pBhNode0)
{

	// find the nbrs, 
	// 1) extended, crossed, parallel

	//define the searching area of the branch

	int pnBranchIDs[1024], pnVpIDs[1024];
	int nNbrBranch, nNbrVp;

	SkBranchMap &bhMap = m_primMan.m_branchMap;

	getPossibleNbrBranchesAndVirtualPrimitives(pBhNode0, pnBranchIDs, nNbrBranch, pnVpIDs, nNbrVp);

	assert(nNbrBranch <= 1024);
	assert(nNbrVp <= 1024);
	int i;

	SkPrimBranch &bh0 = pBhNode0->m_data;
	double dOrt0 = bh0.getOrt();

	double dR0 = bh0.getR();

	double dOrt, dDeltaOrt, dR, dDeltaR;
	int id;
	SkBranchNode *pBhNode;
	SkOutNbr outNbr;
	int nRelationType = -1;
	for (i = 0; i < nNbrBranch; i++)
	{
		id = pnBranchIDs[i];
		outNbr.setNbrID(id);

		pBhNode = bhMap.getNodeAt(id);
		SkPrimBranch &bh = pBhNode->m_data;

		dOrt = bh.getOrt();
		dDeltaOrt = ::skAbsoluteAngleDifferenceOfTwoLines(dOrt0, dOrt);
		dDeltaOrt = fabs(dDeltaOrt);
		outNbr.setDeltaOrt(dDeltaOrt);

		dR = bh.getR();

		dDeltaR = dR - dR0;
		dDeltaR = fabs(dDeltaR);

		outNbr.setDeltaR(dDeltaR);
		
		if (dDeltaOrt <= SK_PRLL_ORT_TH 
			 &&dDeltaR >= SK_PRLL_DIS_TH)
			 nRelationType = SK_PRLL;
		else
			nRelationType = -1;

		outNbr.setRelationType(nRelationType);

		pBhNode0->addNbr(outNbr);
	}

	return 0;
}
int SkPrimPursuit::getPossibleNbrBranchesAndVirtualPrimitives(SkBranchNode *pBhNode, 
															 int *pnBranchIDs, int &nNbrBranch,
															 int *pnVpIDs, int &nNbrVp)
{
	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);

	int id0 = pBhNode->getID();

	SkPrimBranch &bh = pBhNode->m_data;
	SkIntPoint p[2];
	for (int e = 0; e < 2; e++)
		p[e] = bh.getEndPoint(e);

	int x0 = p[0].x, y0 = p[0].y;
	int x1 = p[1].x, y1 = p[1].y;

	double dLength = ::lineLength(p[0].x, p[0].y, p[1].x, p[1].y);
	double dOrt0 = atan2(double(y0-y1), x0-x1);//the orientation of this profile
	double dSearchStick = dLength*3/2;
	double dSearchBack = -dSearchStick;
	double dSearchHfWd = 5;

	SkIntPoint pc;
	pc.x = (p[0].x + p[1].x) / 2;
	pc.y = (p[0].y + p[1].y) / 2;

	int m = skGetRotatedRectangleRegion(pc.x, pc.y, dOrt0, dSearchBack, dSearchStick, 
		-dSearchHfWd, dSearchHfWd, m_pnX, m_pnY);
	assert(m <= 1024*1024);

	getPossibleNbrBranchesAndVirtualPrimitives(pnBranchIDs, nNbrBranch, pnVpIDs, nNbrVp, m_pnX, m_pnY, m, id0);
	return 0;
}

int SkPrimPursuit::getPossibleNbrBranchesAndVirtualPrimitives(int *pnBranchIDs, int &nNbrBranch,
															 int *pnVpIDs, int &nNbrVp,
															 int *pnX, int *pnY, int m, int id0)
{
	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);

	Matrix<oval_IntArray> &bhID = m_primMan.m_branchIDs;
	Matrix<int> &vpID = m_primMan.m_vpID;
	// search if there is a prfl
	nNbrBranch = 0;
	nNbrVp = 0;
	for (int k = 0; k < m; k++)
	{
		int x, y;
		x = pnX[k];
		y = pnY[k];
		if (x < 0 || x >= wd || y < 0 || y >= ht)
			continue;
		int n = bhID(y,x).GetSize();
		for (int i = 0; i < n; i++)
		{
			if (id0 == bhID(y,x).getAt(i))
				continue;//itsself
			bool bExist = false;
			for (int j = 0; j < nNbrBranch; j++)
			{
				if (pnBranchIDs[j] == bhID(y,x).getAt(i))
				{
					bExist = true;
					break;
				}
			}
			if (!bExist)
			{
				pnBranchIDs[nNbrBranch] = bhID(y,x).getAt(i);
				nNbrBranch++;
			}//check existing
		}//for i
		if (vpID(y,x) >= 0)
		{
			bool bExist = false;
			for (int j = 0; j < nNbrVp; j++)
			{
				if (pnVpIDs[j] == vpID(y,x) )
				{
					bExist = true;
					break;
				}
			}
			if (!bExist)
			{
				pnVpIDs[nNbrVp] = vpID(y,x);
				nNbrVp++;
			}//check existing
		}
	}//for k
	return 0;
}

int SkPrimPursuit::getPtArray2PtArrayDistance(SkIntPointArray& ptArray0, 
											  SkIntPointArray& ptArray, 
											  double *pdDis)
{
	int n0 = ptArray0.getSize();
	int n = ptArray.getSize();

	SkIntPoint p0, p;
	for (int i = 0; i < n0; i++)
	{
		p0 = ptArray0[i];
		double dis, min_dis = 1000000;
		for (int j = 0; j < n; j++)
		{
			p = ptArray[j];
			dis = ::sqrtDistance(p0.x, p0.y, p.x, p.y);
			if (dis < min_dis)
				min_dis = dis;
		}
		pdDis[i] = min_dis;
	}
	return n0;
}

int SkPrimPursuit::getExtendCrossBranchesOrVirtualPrimitives(int &nCrossBhID, int &nCrossVpID, 
															 SkIntPoint &p0, SkIntPoint &p1,
															 SkIntPoint &p_cross)
{
	int ht, wd;
	m_imgOrg.GetDimension(ht, wd);
;
	int n = ::rasterize_line_segment_keeping_order(p0.x, p0.y, p1.x, p1.y, m_pnX, m_pnY);
	int wd_dir;//the width direction
	if (abs(p0.x-p1.x) <= abs(p0.y-p1.y))
		wd_dir = 0;//x
	else
		wd_dir = 1;//y

	bool bBreak = false;
	SkIntPoint p;
	int dd[5] = {0, -1, 1, -2, 2};
	for (int i = 1; i < n; i++)//skip the first point
	{
		for (int j = 0; j < 3; j++)//consider two points, for the 45 degree line miss-cross bug
		{
			p.x = m_pnX[i];
			p.y = m_pnY[i];
			if (wd_dir == 0)
				p.x += dd[j];
			else
				p.y += dd[j];
			if(p.x < 0 || p.x >= wd || p.y < 0 || p.y >= ht)
				continue;
			if (p == p0)
				continue;

			int vpID = m_primMan.m_vpID(p.y, p.x);
			if (vpID >= 0)
			{//find a joint
				nCrossVpID = vpID;
				p_cross = p;
				return 1;
			}
			int nBranchNum = m_primMan.m_branchIDs(p.y, p.x).getSize();
//			assert(nBranchNum <= 1);
			if (nBranchNum == 1)
			{
				nCrossBhID = m_primMan.m_branchIDs(p.y, p.x).getAt(0);
				p_cross = p;
				return 1;
			}
		}
	}
	return 0;
}


///////////////////
// value 0-valid point
int SkPrimPursuit::getTotalSegmentNum()
{
	SkBranchMap &bhMap = m_primMan.m_branchMap;
	SkBranchNode *pBhNode;

	// compute linearity and R0 and ort first
	int nSum = 0;
	int nPos = bhMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = bhMap.getNextNode(nPos);
		
		if (!pBhNode->valid())
			continue;
		SkPrimBranch &bh = pBhNode->m_data;
		int nSegNum = bh.getSegNum();

		nSum += nSegNum;
	}
	return nSum;
}


///////////////////
// value 0-valid point
int SkPrimPursuit::saveSketchImage(const char *type)
{
	char name[512];
	char work_dir[512];
	sprintf(work_dir, "sketches");
	_mkdir(work_dir);

	int ht, wd;
	m_imgOrg.GetDimension(ht,wd);
	// get skethable lattice / constraints
	Matrix<int> con;
	con.SetDimension(ht, wd);
	con.Set(255);
	m_primMan.saveSketchableLattice(con);

	sprintf(name, "%s\\%s_%s_sk_con.bmp", work_dir, m_pchProjName, type);
	::skSaveMatrixAsBitmap(name, con, 0, -1);

	sprintf(name, "%s_con.bmp", m_pchProjName);
	::skSaveMatrixAsBitmap(name, con, 0, -1);

	Matrix<float> sk_img_org;
	sk_img_org.SetDimension(ht, wd);
	sk_img_org.Set(0);
	::skCopyConstraintedImage(m_imgOrg, sk_img_org, con);

	sprintf(name, "%s\\%s_%s_sk_img_org.bmp", work_dir, m_pchProjName, type);
	::skSaveMatrixAsBitmap(name, sk_img_org, m_dImgMean0, m_dImgStd0);

	bool bSharp = false;
	Matrix<float> sk_img_rec;
	m_primMan.saveSketchImageReconstructed(sk_img_rec, bSharp);

	sprintf(name, "%s\\%s_%s_sk_img_rec.bmp", work_dir, m_pchProjName, type);
	::skSaveMatrixAsBitmap(name, sk_img_rec, m_dImgMean0, m_dImgStd0);
	sprintf(name, "%s_pro.bmp", m_pchProjName);
	::skSaveMatrixAsBitmap(name, sk_img_rec, m_dImgMean0, m_dImgStd0);

	Matrix<float> sk_img_res;
	sk_img_res = sk_img_org - sk_img_rec;

	sprintf(name, "%s\\%s_%s_sk_img_res.bmp", work_dir, m_pchProjName, type);
	::skSaveMatrixAsBitmap(name, sk_img_res, m_dImgMean0, m_dImgStd0);

	Matrix<float> sk_img_sharp_rec;
	bSharp = true;
	m_primMan.saveSketchImageReconstructed(sk_img_sharp_rec, bSharp);

	sprintf(name, "%s\\%s_%s_sk_img_rec_sharp.bmp", work_dir, m_pchProjName, type);
	::skSaveMatrixAsBitmap(name, sk_img_sharp_rec, m_dImgMean0, m_dImgStd0);

	sk_img_sharp_rec = sk_img_sharp_rec + sk_img_res;
	Matrix<int> mat255;
	mat255.SetDimension(ht,wd);
	mat255.Set(255);
	Matrix<int> con_inv;
	con_inv = mat255 - con;
	::skCopyConstraintedImage(m_imgOrg, sk_img_sharp_rec, con_inv);

	sprintf(name, "%s\\%s_%s_img_rec_sharp.bmp", work_dir, m_pchProjName, type);
	::skSaveMatrixAsBitmap(name, sk_img_sharp_rec, m_dImgMean0, m_dImgStd0);

	sprintf(name, "%s\\%s_%s_sk_img_coding.txt", work_dir, m_pchProjName, type);
	FILE *pFile;
	pFile = fopen(name, "w");
	if (pFile == 0)
	{
		char msg[512];
		sprintf(msg, "Can open file %s to write. Program returns. \n", name);
		return 1;
	}
	fprintf(pFile, "ImgWd = %d, ImgHt = %d\n", wd, ht);
	fprintf(pFile, "ImgMean = %.2f, ImgStd = %.2f\n", m_dImgMean0, m_dImgStd0);

	// save the variance of the org and res too
	double dImgMean, dImgStd;
	double dImgMean1, dImgStd1;
	int nn = ::skComputeMaskedImageMeanStd(con, sk_img_org, dImgMean, dImgStd);
	dImgMean1 = m_dImgMean0 + dImgMean * m_dImgStd0;
	dImgStd1 = dImgStd * m_dImgStd0;
	fprintf(pFile, "Sketchable size: %d\n", nn);
	fprintf(pFile, "OrgSkMean = %.5f, OrgSkStd = %.5f\n", dImgMean1, dImgStd1);

	::skComputeMaskedImageMeanStd(con, sk_img_res, dImgMean, dImgStd);
	dImgMean1 = m_dImgMean0 + dImgMean * m_dImgStd0;
	dImgStd1 = dImgStd * m_dImgStd0;
	fprintf(pFile, "ResSkMean = %.5f, ResSkStd = %.5f\n", dImgMean1, dImgStd1);

	fclose(pFile);

	return 0;
}

///////////////////
// value 0-valid point
int SkPrimPursuit::buildProfileSet()
{

	m_prflSet.clear();

	SkBranchMap &bhMap = m_primMan.m_branchMap;
	SkBranchNode *pBhNode;

	// compute linearity and R0 and ort first
	SkLinelet line;
	SkIntPoint p;
	int nPos = bhMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = bhMap.getNextNode(nPos);
		
		if (!pBhNode->valid())
			continue;
		SkPrimBranch &bh = pBhNode->m_data;
		int nSegNum = bh.getSegNum();
		
		for (int i = 0; i < nSegNum; i++)
		{
			SkSegment *pSeg = bh.getSegment(i);
			SkBranchProfile &prfl = pSeg->m_prfl;
			SkRidgeModel &rm = prfl.m_ridgeModel;
			for (int e = 0; e < 2; e++)
			{
				p = pSeg->getEndPoint(e);
				line.x[e] = p.x;
				line.y[e] = p.y;
			}

			int nSegLength = prfl.getSampleNum();

			m_prflSet.add(rm, nSegLength, line);
		}
	}

	return 0;
}

///////////////////
// saveProfileSet()
int SkPrimPursuit::saveProfileSet(const char *type)
{
	char new_type[512];
	sprintf(new_type, "%s_%s", m_pchProjName, type);

	m_prflSet.orderStepAndCenterProfiles();
	m_prflSet.sort();
	m_prflSet.save2Files(new_type, m_dImgMean0, m_dImgStd0);
	return 0;
}

/////////////////////////////////////////////////////////////////////
// class SkProfileSet
///////////////////
int SkProfileSet::add(SkProfileSet &prflSet)
{
	int i, n;
	n = prflSet.m_rmArray.getSize();
	int num;
	for (i = 0; i < n; i++)
	{
		SkRidgeModel &rm = prflSet.m_rmArray[i];
		num = prflSet.m_smpNumArray[i];
		SkLinelet &line = prflSet.m_lineletArray[i];
		add(rm, num, line);
	}
	return m_rmArray.getSize(); 
}
///////////////////
// order the step with black first then white
// center the profile
int SkProfileSet::orderStepAndCenterProfiles()
{
	int nPrflNum = m_rmArray.getSize();
	if (nPrflNum <= 0)
		return 1;

	int i;
	for (i = 0; i < nPrflNum; i++)
	{
		SkRidgeModel &rm = m_rmArray[i];
		rm.sortStepMean();
	}
	return 0;
}

///////////////////
int SkProfileSet::sort()
{
	int nPrflNum = m_rmArray.getSize();
	if (nPrflNum <= 0)
		return 1;

	SkRidgeModel tmp;
	int i, j;
	for (i = 0; i < nPrflNum-1; i++)
	{
		SkRidgeModel &rm1 = m_rmArray[i];
		for (j = i+1; j < nPrflNum; j++)
		{
			SkRidgeModel &rm2 = m_rmArray[j];
			if (rm1 > rm2)
			{
				swap(rm1, rm2);
				swap(m_smpNumArray[i], m_smpNumArray[j]);
				swap(m_lineletArray[i], m_lineletArray[j]);
			}
		}
	}

	return 0;
}
///////////////////
// save2Files()
int SkProfileSet::save2Files(const char *type, double dImgMean0, double dImgStd0)
{
	char work_dir[32];
	sprintf(work_dir, "prim");
	_mkdir(work_dir);

	char name[512];

	sprintf(name, "%s\\%s_prfl_set.txt", work_dir, type);
	saveAsText(name);

//	saveHistograms(type, dImgMean0, dImgStd0);
	return 0;
}
///////////////////
// save2Files()
int SkProfileSet::saveAsText(const char *name)
{
	char msg[512];

	int nPrflNum = m_rmArray.getSize();
	int ht = 7;
	if (nPrflNum > 0)
	{
		SkRidgeModel &rm = m_rmArray[0];
		ht = rm.getDataLength();
	}

	FILE *pFile = fopen(name, "w");
	if (pFile == 0)
	{
		sprintf(msg, "\n Couldn't open file %s to write. Program stops. \n", name);
		return 1;
	}
	fprintf(pFile, "prfl num: %d\n", nPrflNum);

	int i, j;
	for (i = 0; i < nPrflNum; i++)
	{
		SkRidgeModel &rm = m_rmArray[i];
		// save the properties as text
		int nSampleNum = m_smpNumArray[i];
		int nStepNum = rm.getStepNum();
		assert(nStepNum >= 0 && nStepNum <= ht);
		SkLinelet &line = m_lineletArray[i];

		fprintf(pFile, "prfl %d: end points (%d,%d) (%d,%d); sample num %d; step num %d; steps: ", 
			i, line.x[0], line.y[0], line.x[1], line.y[1], nSampleNum, nStepNum);
		for (j = 0; j < nStepNum; j++)
		{
			int st = rm.getStepStart(j);
			int en = rm.getStepEnd(j);
			double dMean = rm.getStepMean(j);
			fprintf(pFile, "(%d,%d, %.5f) ", st, en, dMean);
		}
		fprintf(pFile, "\n");
		
	}
	fclose(pFile);
	return 0;
}
int SkProfileSet::readFromText(const char *name)
{
	char msg[512];
	FILE *pFile = fopen(name, "r");
	if (pFile == 0)
	{
		sprintf(msg, "\n Couldn't open file %s to read. Program stops. \n", name);
		return 1;
	}

	int nPrflNum;
	fscanf(pFile, "prfl num: %d\n", &nPrflNum);

	m_rmArray.setSize(nPrflNum);
	m_smpNumArray.setSize(nPrflNum);
	m_lineletArray.setSize(nPrflNum);

	int i, j;
	for (i = 0; i < nPrflNum; i++)
	{
		int nSampleNum;
		int nStepNum;
		int x0, y0, x1, y1;
		int id;

		fscanf(pFile, "prfl %d: end points (%d,%d) (%d,%d); sample num %d; step num %d; steps: ", 
			&id, &x0, &y0, &x1, &y1, &nSampleNum, &nStepNum);
		m_lineletArray[i].x[0] = x0;
		m_lineletArray[i].y[0] = y0;
		m_lineletArray[i].x[1] = x1;
		m_lineletArray[i].y[1] = y1;
		m_smpNumArray[i] = nSampleNum;

		m_rmArray[i].m_nDataLength =  2 * g_nHalfProfileWd + 1;
		m_rmArray[i].m_nStepNum = nStepNum;

		for (j = 0; j < nStepNum; j++)
		{
			int st;//start of this step
			int en;//end of this step, difference is the step width
			double dMean;
			fscanf(pFile, "(%d,%d, %lf) ", &st, &en, &dMean);
			m_rmArray[i].m_pnStepSt[j] = st;
			m_rmArray[i].m_pnStepEn[j] = en;
			m_rmArray[i].m_pdStepMean[j] = dMean;
		}
		fscanf(pFile, "\n");
		m_rmArray[i].computeModelIntensity();
		m_rmArray[i].centerModelIntensity();
		m_rmArray[i].setOriginalIntensityAsCenterredModel();
	}
	fclose(pFile);
	return 0;	
}


///////////////////
// saveHistograms()
int SkProfileSet::saveHistograms(const char *type, double dImgMean0, double dImgStd0)
{
	char work_dir[32];
	sprintf(work_dir, "prim");
	_mkdir(work_dir);

	char name[512], msg[512];

	int nPrflNum = m_rmArray.getSize();
	int ht = 2*g_nHalfProfileWd+1;
	if (nPrflNum > 0)
	{
		SkRidgeModel &rm = m_rmArray[0];
		ht = rm.getDataLength();
	}
	
	int nEdgeStepNum = 256;
	int nMaxLineletLength = 256;
	int *pnStepNumHist = new int[ht+1]; memset(pnStepNumHist, 0, sizeof(int)*(ht+1));
	int *pnStepNumPrflCount = new int[ht+1]; memset(pnStepNumPrflCount, 0, sizeof(int)*(ht+1));
	int *pnEdgeScaleHist = new int[ht]; memset(pnEdgeScaleHist, 0, sizeof(int)*ht);
	int *pnRidgeWidthHist = new int[ht]; memset(pnRidgeWidthHist, 0, sizeof(int)*ht);
	int *pnEdgeStepHist = new int[nEdgeStepNum]; memset(pnEdgeStepHist, 0, sizeof(int)*nEdgeStepNum);
	int *pnLineletLengthHist = new int[nMaxLineletLength]; memset(pnLineletLengthHist, 0, sizeof(int)*nMaxLineletLength);


	double *dv, *dvm;
	int i, j;
	for (i = 0; i < nPrflNum; i++)
	{
		SkRidgeModel &rm = m_rmArray[i];
		// save the properties as text
		int nSampleNum = m_smpNumArray[i];
		int nStepNum = rm.getStepNum();
		assert(nStepNum >= 0 && nStepNum <= ht);

		// compute the histogram statistics 

		pnStepNumHist[nStepNum] += nSampleNum;
		pnStepNumPrflCount[nStepNum]++;

		for (j = 0; j < nStepNum -1; j++)
		{
			double dMean1 = rm.getStepMean(j);
			double dMean2 = rm.getStepMean(j+1);
			double dDeltaMean = fabs(dMean1 - dMean2);
			int m = dDeltaMean * dImgStd0;
			if (m >= nEdgeStepNum) 
				m = nEdgeStepNum-1;
			if (m < 0 )
				m = 0;
			pnEdgeStepHist[m] += nSampleNum;

			int s = rm.getStepStart(j+1) - rm.getStepEnd(j) - 1;
			assert(s >= 0 && s < ht);
			pnEdgeScaleHist[s] += nSampleNum;
		}

		for (j = 1; j < nStepNum -1; j++)
		{
			int st = rm.getStepStart(j);
			int en = rm.getStepEnd(j);
			int l = en - st + 1; 
			assert(l < ht);
			pnRidgeWidthHist[l] += nSampleNum;
		}
		if (nSampleNum >= nMaxLineletLength) 
			nSampleNum = nMaxLineletLength-1;
		pnLineletLengthHist[nSampleNum] ++;
	}

	// get the profile matrix first
	Matrix<double> prflMat, prflMatM;

	// save as bmp with diff seg num
	int s = 2;
	while (pnStepNumPrflCount[s] <= 0)
		s++;
	int nCurPrflNum = pnStepNumPrflCount[s];
	prflMat.SetDimension(ht, nCurPrflNum); prflMat.Set(0);
	prflMatM.SetDimension(ht, nCurPrflNum); prflMatM.Set(0);

	int ii = 0;//the count of profiles
	int ii_nr = 0;//the count of not redundant profiles
	for (i = 0; i < nPrflNum; i++)
	{

		SkRidgeModel &rm = m_rmArray[i];
		dv = rm.getOrgIntensity();
		dvm = rm.getCenterredModelIntensity();
		if (ii_nr == 0)
		{//the first profile
			for (j = 0; j < ht; j++)
			{
				prflMat(j,ii) = dv[j];
				prflMatM(j,ii) = dvm[j];
			}
			ii++;
			ii_nr++;
		}
		else
		{//compare current profile with the previous one
			double dDiff = 0;
			for (j = 0; j < ht; j++)
				dDiff += fabs(dvm[j] - prflMatM(j,ii_nr-1));
			dDiff *= dImgStd0;
			if (dDiff/ht > 10)
			{//non-redundant profile
				for (j = 0; j < ht; j++)
				{
					prflMat(j,ii_nr) = dv[j];
					prflMatM(j,ii_nr) = dvm[j];
				}
				ii++;
				ii_nr++;
			}
			else
				ii++;
		}

		if (ii == pnStepNumPrflCount[s])
		{//got the s

			sprintf(name, "%s\\%s_all_prfl_s%d.bmp", work_dir, type, s);
			::skSaveMatrixAsBitmap(name, prflMat, dImgMean0, dImgStd0);
			sprintf(name, "%s\\%s_all_prfl_s%d.txt", work_dir, type, s);
			::skSaveMatrix(name, prflMat);
			
			sprintf(name, "%s\\%s_all_prflM_s%d.bmp", work_dir, type, s);
			::skSaveMatrixAsBitmap(name, prflMatM, dImgMean0, dImgStd0);
			sprintf(name, "%s\\%s_all_prflM_s%d.txt", work_dir, type, s);
			::skSaveMatrix(name, prflMatM);

			s++;
			while (pnStepNumPrflCount[s] <= 0 && s < ht )
			{
				s++;
			}
			nCurPrflNum = pnStepNumPrflCount[s];
			if (nCurPrflNum <= 0)
				break;
			prflMat.SetDimension(ht, nCurPrflNum); prflMat.Set(0);
			prflMatM.SetDimension(ht, nCurPrflNum); prflMatM.Set(0);
			ii = 0;
			ii_nr = 0;
		}//if got the number
		
	}//for prfl


	sprintf(name, "%s\\%s_step_num_hist.txt", work_dir, type);
	::saveArray(pnStepNumHist, ht+1, name);

	sprintf(name, "%s\\%s_step_num_prfl_count.txt", work_dir, type);
	::saveArray(pnStepNumPrflCount, ht+1, name);

	sprintf(name, "%s\\%s_edge_scale_hist.txt", work_dir, type);
	::saveArray(pnEdgeScaleHist, ht, name);

	sprintf(name, "%s\\%s_ridge_width_hist.txt", work_dir, type);
	::saveArray(pnRidgeWidthHist, ht, name);

	sprintf(name, "%s\\%s_edge_step_hist.txt", work_dir, type);
	::saveArray(pnEdgeStepHist, nEdgeStepNum, name);

	sprintf(name, "%s\\%s_linelet_length_hist.txt", work_dir, type);
	::saveArray(pnLineletLengthHist, nMaxLineletLength, name);

	delete []pnStepNumHist;
	delete []pnStepNumPrflCount;
	delete []pnEdgeScaleHist;
	delete []pnRidgeWidthHist;
	delete []pnEdgeStepHist;
	delete []pnLineletLengthHist;

	saveLineletOrienataionHistogram(type);
	return 0;
}

int SkProfileSet::saveLineletOrienataionHistogram(const char *type)
{
	const double g_dMinSketchLength4OrtHist = 5;//only for saving

	char work_dir[32];
	sprintf(work_dir, "prim");
	_mkdir(work_dir);

	char name[512], msg[512];

	int n = m_lineletArray.getSize();
	int i;
	int x0, y0, x1, y1;
	double dLength, dOrt;
	int pnHist[180];
	memset(pnHist, 0, sizeof(int)*180);
	for (i = 0; i < n; i++)
	{
		x0 = m_lineletArray[i].x[0];
		y0 = m_lineletArray[i].y[0];
		x1 = m_lineletArray[i].x[1];
		y1 = m_lineletArray[i].y[1];

		dLength = ::distanceBetweenPoints(y0, x0, y1, x1);
		if (dLength < g_dMinSketchLength4OrtHist)
			continue;
		dOrt = ::directionOfTwoPoints(y0, x0, y1, x1);
		if (dOrt >= PI)
			dOrt = 2*PI - dOrt;

		int nOrt = (int)(dOrt * 180 / PI);
		if (nOrt < 0)
			nOrt = 0;
		if (nOrt > 180)
			nOrt = 179;
		pnHist[nOrt]++;

	}
	sprintf(name,"%s\\%s_linelet_ort_hist.txt", work_dir, type);
	::saveArray(pnHist, 180, name);
	return 0;
}


int skSaveABlobFromProfile(Matrix<float> &mat, double *v, int ht0, int nXOffset, int nYOffset)
{
	int ht, wd;
	mat.GetDimension(ht, wd);
//	int hf = ht/2;

//	mat.Set(0);

	double x, y;
	int r, c;
	int bound = 1;
	for (r = bound; r < ht-bound; r++)
	{
		for (c = bound; c < wd-bound; c++)
		{
			x = c-nXOffset;
			y = r-nYOffset;
			double dOrt = atan2(y,x);
			::rotate(x,y,-dOrt);
			int xx = x + 0.5;
			if (xx >= 0 && xx <ht0)
				mat(r,c) = v[xx];
		}
	}
	return 0;
}
int skSaveAnEdgeFromProfile(Matrix<float> &mat, double *v, int ht0, double dOrt)
{
	int ht, wd;
	mat.GetDimension(ht, wd);

	int hf = ht/2;
	int hf0 = ht0/2;

	Matrix<float> M(ht, wd);

	int bound = (int)(rand()*5/(RAND_MAX+1.0));
	//get the profile image
	M.Set(0);
	int r, c;
	for (r = -hf0; r <= hf0; r++)
	{
		if (r + hf >= ht || r + hf < 0)
			continue;
		for (c = bound; c < wd-bound; c++)
		{
			M(r+hf,c) = v[r+hf0];
		}
	}

	// rotate
	mat.Set(0);
	skRotatAMatrix(M, mat, dOrt);

	return 0;
}
int skSaveAnEndFromProfile(Matrix<float> &mat, double *v, int ht0, double dOrt0)
{
	int ht, wd;
	mat.GetDimension(ht, wd);

	int hf = ht/2;
	int hf0 = ht0/2;

	Matrix<float> M(ht, wd);

	int bound = 1;
	//get the profile image
	M.Set(0);
	int r, c;
	for (r = -hf0; r <= hf0; r++)
	{
		if (r + hf >= ht || r + hf < 0)
			continue;
		for (c = bound; c <= hf; c++)
		{
			M(r+hf,c) = v[r+hf0];
		}
	}
	double x, y;
	// the blured end
	for (r = -hf0; r <= hf0; r++)
	{
		for (c = hf+1; c < hf+1+hf0; c++)
		{
			x = c-hf;
			y = r;
			double dOrt = atan2(y,x);
			::rotate(x,y,-dOrt);
			int xx = x + 0.5;
			if (xx>0 && xx <=hf0)
			{
				double v1 = v[hf0-xx];
				double v2 = v[hf0+xx];
				double v_new = ((hf0+r)*v2 + (hf0-r)*v1)/(2*hf0);
				M(r+hf,c) = v_new;
			}
		}
	}
	// rotate
	mat.Set(0);
	skRotatAMatrix(M, mat, dOrt0);

	return 0;
}
int skSaveARidgeFromProfile(Matrix<float> &mat, double *v, int ht0, double dOrt)
{
	skSaveAnEdgeFromProfile(mat, v, ht0, dOrt);
	return 0;
}
/////////////////////////////////////////////////////////////////////
int skSortAllCodingLengthGain(FILE *pFile)
{
	char ppchImgName[1024][128];
	char name[512], msg[512], pchProjName[512], no[32];
	
	///////////////
	// get file names
	int n = 0;
	while (fscanf(pFile, "%s", ppchImgName[n]) != EOF)
	{
		n++;
		if (n >= 1024)
			break;
	}

	sprintf(msg, "\n  Sort All Coding Length Gain on %d images......\n", n);
	::skOutput(msg);


	oval_Array<float> sthArray;
	FILE *fp;
	// read the scale parameter
	int i; 
	for (i = 0; i < n; i++)
	{
		sprintf(msg, "             %s ...",ppchImgName[i]);
		::skOutput(msg);

		strcpy(pchProjName, ppchImgName[i]);
		::removeFileTypeInName(pchProjName);

		sprintf(name, "%s\\%s_3_m10_stroke_sth.txt", pchProjName, pchProjName);
		fp = fopen(name, "r");

		if (fp == 0)
			continue;

		float value;

		int numInOne = 0;
		while (fscanf(fp, "%f ", &value) != EOF)
		{
			sthArray.add(value);
			numInOne++;
		}
		fclose(fp);

		sprintf(msg, " %d curves, done!\n",numInOne);
		::skOutput(msg);
	}
	int num = sthArray.getSize();
	float *pData = sthArray.GetPointerAt(0);
	::sort(num, pData, 1);//0-increasing, 1-decreasing

	char work_dir[512];
	sprintf(work_dir, "sketch_sth");
	_mkdir(work_dir);
	sprintf(name, "%s\\all_strokes_sth.txt", work_dir);
	::saveArray(pData, num, name);

	sprintf(msg, " Total %d curve strengths saved into: %s\n",num, name);
	::skOutput(msg);

	return n;
}


