#ifndef _SK_PRIMPURSUIT_H_
#define _SK_PRIMPURSUIT_H_

#include <direct.h>
#include <stdio.h>

#include "skTemplate.h"
#include "skMatrix.h"
#include "skFilterCoef.h"
#include "skTemplate.h"

#include "skPrimitive.h"

extern const char g_pchSketchParamName[];
extern const char g_pchSketchWorkDir[];
extern const char g_pchProposalWorkDir[];

int skREBDetection(Matrix<float> &imgOrg, int reb, const char* pchProj);
int skNonmaximumSupressionFromMagAndDir(Matrix<int> &mag, Matrix<float> &dir, 
			Matrix<int> &sketch, Matrix<int> &mag1);

int skRidgeEdgeBlobDetection(const char *pchImgName);
int skComputeSketchStrength(const char *pchImgName);
int skComputeSketchProposal(const char *pchImgName);
int skPrimPursuit(FILE *pFile);
int skReadAndSaveProfileSetInOne(FILE *pFile);
int skSaveABlobFromProfile(Matrix<float> &mat, double *v, int ht0, int nXOffset, int nYOffset);
int skSaveAnEdgeFromProfile(Matrix<float> &mat, double *v, int ht0, double dOrt);
int skSaveAnEndFromProfile(Matrix<float> &mat, double *v, int ht0, double dOrt);
int skSaveARidgeFromProfile(Matrix<float> &mat, double *v, int ht0, double dOrt);

int skSortAllCodingLengthGain(FILE *pFile);


int skOutputPrimPursuitParam(SkPrimPursuitParam &param);

class SkPrimManager;
class SkLinelet
{
public:
	int x[2];
	int y[2];
};

class SkProfileSet
{
private:
public:
	oval_Array<SkRidgeModel> m_rmArray;//ridge models
	oval_Array<int> m_smpNumArray;//sample numver for each profile
	oval_Array<SkLinelet> m_lineletArray;
public:
	void clear() { m_rmArray.removeAll(); m_smpNumArray.removeAll(); m_lineletArray.removeAll(); }
	int add(SkRidgeModel &rm, int n, SkLinelet &line) 
		{ m_rmArray.add(rm); m_smpNumArray.add(n); m_lineletArray.add(line);return m_rmArray.getSize(); }
	int add(SkProfileSet &prflSet);
	int orderStepAndCenterProfiles();
	int sort();

	int save2Files(const char *type, double dImgMean0, double dImgStd0);
	int saveAsText(const char *name);
	int saveHistograms(const char *type, double dImgMean0, double dImgStd0);
	int saveLineletOrienataionHistogram(const char *type);

	int readFromText(const char *name);
};//
int skSaveSomeBlobEdgeRidgeSamplesFromPrflSet(SkProfileSet&prflSet,const char *type, 

											  double dImgMean0, double dImgStd0);
// define the connect to an end of a curve/branch
class SkEndConnection;

class SkPrimPursuit
{
public:
	SkPrimPursuitParam m_param;//parameters
	SkProfileSet m_prflSet;
	double m_dImgMean0, m_dImgStd0;
private:
	SkPrimManager m_primMan;//primitive mananger

	Matrix<float> m_imgOrg;
	char m_pchProjName[512];

	Matrix<int> m_mag;//original sketch magnitude
	Matrix<int> m_magRes;

	Matrix<float> m_blobMag;
	Matrix<int> m_magBlobRes;

	Matrix<int> m_filterType;//the filter type of each point, DOOG, DOOG1, DOG, etc
	Matrix<unsigned char> m_edge;//saves edges


	SkPrimManager m_primManTmp;
	int m_pnX[1024*256], m_pnY[1024*256];
	double m_pdX[1024*256], m_pdY[1024*256];

//	DWORD m_Time0;
	char m_pchDir0[512];//previous directory
public:
	int m_nProcStep;//process step

public:
	SkPrimPursuit();
	~SkPrimPursuit();
	Matrix<float> &getImgOrg() { return m_imgOrg; }
	double getImgStd0() { return m_dImgStd0; }
	double getImgMean0() { return m_dImgMean0; }
	void getImageDimension(int &ht, int&wd) {m_imgOrg.GetDimension(ht, wd);}


	int initializePrimPursuit(const char * pchImgName);
	int primPursuitWithoutPrior();
	int primPursuitByGraphOperations();
	int completePrimPursuit();
	int savePursuitResults(int step);


	int primPursuitWithFilterCoef();
	// below for hysteresis curve growing
	int initialize4HysteresisCurveGrowing();
	int primPursuitWithFilterCoefInOneBlockAndHysteresis(int x1, int y1, int x2, int y2);
	int initializePrimitiveManager();

	int getTotalSegmentNum();
	int buildProfileSet();
	int saveProfileSet(const char *type);

	int saveSketchImage(const char *type);

	int buildBranchNbrs();
	int buildBranchNbrs(SkBranchNode *pBhNode);
	int getPossibleNbrBranchesAndVirtualPrimitives(SkBranchNode *pBhNode, 
												 int *pnBranchIDs, int &nNbrBranch,
												 int *pnVpIDs, int &nNbrVp);
	int getPossibleNbrBranchesAndVirtualPrimitives(int *pnBranchIDs, int &nNbrBranch,
												 int *pnVpIDs, int &nNbrVp,
												 int *pnX, int *pnY, int m, int id0);
	int getExtendCrossBranchesOrVirtualPrimitives(int &nCrossBhID, int &nCrossVpID, 
												 SkIntPoint &p0, SkIntPoint &p1,
												 SkIntPoint &p_cross);


	int getPtArray2PtArrayDistance(SkIntPointArray& ptArray0, 
								  SkIntPointArray& ptArray, 
								  double *pdDis);


	int extendFreeBranchByLowerEdgeThresh(SkBranchNode *pBhNode, int e);
	int addPrimitivesFromPointMap(Matrix<int> &ptMat);

	// refine curve by curve
	int combineConnectedFreeEndPointsIntoOneBranch();
	int removeSmallImgCLGBranches();
	int diffuseBranchPoints();

	int refineCurvesByGraphOperations();
	int removeLowImageCLGBranches(double dMinImgCLG);
	int removeShortBranches(double dMinBranchLength);
	int refineSketchGraphByRemovingRedundentPoints();
	int combineConnectedVirtualPrimitives();

	int findCrossedVP(int num, int *pnX, int *pnY, Matrix<int> &vpOcp, 
					  oval_PointArray& ptArr, oval_IntArray &idArr);
	int findCrossedSk(int num, int *pnX, int *pnY, Matrix<oval_IntArray> &skOcp, 
					  oval_PointArray& ptArr, oval_Array<oval_IntArray> &idArr);
	int findCrossedExt(int num, int *pnX, int *pnY, Matrix<oval_IntArray> &extOcp,
	 				   Matrix<oval_IntArray> &extEndOcp,
					   oval_PointArray& ptArr, oval_Array<oval_IntArray> &idArr,
				       oval_Array<oval_IntArray> &endArr);
	int findAllPossibleConnections(oval_Array<SkEndConnection> &cnctArr, 
								  int id0, int end0, 
								  oval_PointArray &extVpPosArr, 
								  oval_IntArray &extVpIDArr, //VP
								  oval_PointArray &extSkPosArr,
								  oval_Array<oval_IntArray> &extSkIDArr, //sk
								  oval_PointArray &extExtPosArr, 
								  oval_Array<oval_IntArray> &extExtIDArr, 
								  oval_Array<oval_IntArray> &extExtEndArr);//ext
	int findAllPossibleConnections(oval_Array<SkEndConnection> &cnctArr, 
								  int id0, int end0, //
								  oval_PointArray &crossPtArr,//the points on the branch (id1)
								  int id1);
	int findOtherExtension2ThisSketch(int nSkPtNum, int *pnSkX, int *pnSkY, int id,//the point on the sketch
									 Matrix<oval_IntArray> &extOcp, 
									 Matrix<oval_IntArray> &extEndOcp,//the extension branch id and end ocps
									 oval_IntArray &bhIDArr, //found branch id array
									 oval_IntArray &bhEndArr,//according end array
									 oval_Array<oval_PointArray> &ptArrayArr);//the points for each id & end
	int removeDuplicateConnection(oval_Array<SkEndConnection> &cnctArray);
	bool wholellyInhibited(int nPtNum, int *pnX, int *pnY, Matrix<oval_IntArray> &ocp);
	int sortConnectionEnergy(oval_Array<SkEndConnection> &cnctArray);
	int changeOcp2Binary(Matrix<oval_IntArray> &ocp, Matrix<int> &mat);
	int saveRefinedCurves(int i);
	int saveRefinedSketchImage(Matrix<SkPixCandArray> &rec_cand, int i);

	int blobPursuitWithFilterCoef();

};

#endif //_SK_PRIMPURSUIT_H_