#ifndef _SK_PRIMITIVE_H_
#define _SK_PRIMITIVE_H_

#include <stdio.h>

#include "skTemplate.h"
#include "skMap.h"

#include "skSketchGraph.h"

extern const char g_pchSketchReconstructedDir[512];
extern int g_nHalfProfileWd;

/////////////////////////////////////////////////////////////////////
// class SkIntPoint -- integer point
class SkIntPoint
{
public:
	int x;
	int y;
public:
	SkIntPoint() { x=y=0; }
	SkIntPoint(int x0, int y0) { x = x0; y = y0; }
	SkIntPoint(SkIntPoint &p) { x = p.x; y = p.y; }

	double manhattanDistance(SkIntPoint p) { return fabs(double(p.x-x)) + fabs(double(p.y-y)); }

	bool operator == (SkIntPoint p)
	{
		if (x == p.x && y == p.y)
			return true;
		return false;
	}
	SkIntPoint operator + (SkIntPoint& p)
	{
		SkIntPoint tmp;
		tmp.x = x + p.x;
		tmp.y = y + p.y;
		return tmp;
	}
	SkIntPoint operator - (SkIntPoint& p)
	{
		SkIntPoint tmp;
		tmp.x = x - p.x;
		tmp.y = y - p.y;
		return tmp;
	}
};

/////////////////////////////////////////////////////////////////////
// class SkFloatPoint -- float point
class SkFloatPoint
{
public:
	float x;
	float y;
public:
	SkFloatPoint() { x=y=0; }
	SkFloatPoint(int x0, int y0) { x = x0; y = y0; }
	SkFloatPoint(SkFloatPoint &p) { x = p.x; y = p.y; }

	double manhattanDistance(SkFloatPoint p) { return fabs(p.x-x) + fabs(p.y-y); }

	bool operator == (SkFloatPoint p)
	{
		if (x == p.x && y == p.y)
			return true;
		return false;
	}
	SkFloatPoint operator + (SkFloatPoint& p)
	{
		SkFloatPoint tmp;
		tmp.x = x + p.x;
		tmp.y = y + p.y;
		return tmp;
	}
	SkFloatPoint operator - (SkFloatPoint& p)
	{
		SkFloatPoint tmp;
		tmp.x = x - p.x;
		tmp.y = y - p.y;
		return tmp;
	}
};

#ifndef SkIntPointArray
#define SkIntPointArray oval_Array<SkIntPoint>
#endif

#ifndef SkFloatPointArray
#define SkFloatPointArray oval_Array<SkFloatPoint>
#endif

#ifndef SkFloatArray
#define SkFloatArray oval_Array<float>
#endif

//class SkPrimProfile;

class SkRidgeModel
{
#define SK_MAX_PRFL_RIDGEMODEL 128

private:
public:
	int m_nDataLength;//data length
	int m_nStepNum;// the number of steps, ridgenumber=stepnum-2;
	int m_pnStepSt[SK_MAX_PRFL_RIDGEMODEL];//step start point, end point and mean value, 
	int m_pnStepEn[SK_MAX_PRFL_RIDGEMODEL];
	double m_pdStepMean[SK_MAX_PRFL_RIDGEMODEL];//very redundent representation
	bool m_bReverse;
	double m_dOffset;

public:
	double m_v[SK_MAX_PRFL_RIDGEMODEL];//orginal intenisty data
	double m_vm[SK_MAX_PRFL_RIDGEMODEL];//intensity by the model 
	double m_vmc[SK_MAX_PRFL_RIDGEMODEL];//intensity by the model, after centerred
	double m_vms[SK_MAX_PRFL_RIDGEMODEL];//intensity by the model, with sharpening 
public:
	SkRidgeModel() { m_bReverse = false; m_dOffset= 0; }
	int build(Matrix<double> &v);//v is a column vector use (i,0) to access each member
	int getDataLength() { return m_nDataLength; }
	double *getCenterredModelIntensity() { return m_vmc; }
	double *getModelIntensity() { return m_vm; }
	double *getModelIntensitySharp() { return m_vms; }
	double *getOrgIntensity() { return m_v; }
	int getStepNum() { return m_nStepNum; }
	int getStepStart(int i) { return m_pnStepSt[i]; }
	int getStepEnd(int i) { return m_pnStepEn[i]; }
	double getStepMean(int i) { return m_pdStepMean[i]; }

	bool operator > (SkRidgeModel &rm);
	int sortStepMean();
	int inverseTheCenterredModel();
	int computeModelIntensity();
	int centerModelIntensity();//make the ridge or edge at the center

	int getStepFollowingGap(int i) { return getStepStart(i+1) - getStepEnd(i); }
	int getStepWidth(int i) { return getStepEnd(i)-getStepStart(i) + 1; }
	int setOriginalIntensityAsCenterredModel() { memcpy(m_v, m_vmc, sizeof(double)*SK_MAX_PRFL_RIDGEMODEL); return 0; }
};

class SkBranchProfile
{
public:
//	SkIntPointArray m_ptArray;//saves all the points

	Matrix<double> m_sample0;//column vector for each sample, orignal sample
	Matrix<double> m_sample;//column vector for each sample, after translated
	SkRidgeModel m_ridgeModel;// ridge model
	Matrix<double> m_smpCenterOff;//sample center position offset
	Matrix<double> m_mu;//mu, mean of the samples (sometime, valid samples)
	static double m_dImgMean0, m_dImgStd0;

	double m_dImgCLG;//coding length gain
	double m_dImgCL0, m_dImgCL1;//coding length before the model and after

	// this is for coding. The difference between residue and coding length is that 
	//   CL subtracts the mean value before computing the squre sum
	//   while residue doesn't
	double m_dDeltaRes;//the residue change
	double m_dRes0, m_dRes1;//residue before and after

public:
	int setSampleByPointArrayProfile(SkIntPointArray &ptArray, SkFloatArray &ortArray, 
												  Matrix<float> *pImgOrg, int half_wd);
	int setSampleByLineProfile(int x0, int y0, int x1, int y1, Matrix<float> *pImgOrg, int half_wd);

	int buildEdgeRidgeModel();
	int addSample(Matrix<double> &mat);
	int setSample(Matrix<double> &mat);
	int computeMeanFromValidSamples();
	int findCenter4EachSample(Matrix<double> &mu);//for a given profile mu
	int updateSampleAndMeanByCenterOffset();
	int getSampleWidth() { return m_sample.rows(); }
	Matrix<double> &getSampleMean() { return m_mu;}//column vector, call by m_mu(i,0)
	int saveProfileAsBmp(const char *type);
	int getSampleNum() { return m_sample0.cols(); }
	Matrix<double> getRidgeModelPrfl();
	double computeImageCodingLengthGain();
//	SkIntPoint getPoint(int i){ return m_ptArray[i]; }
	double getFloatPointOffset(int i){ return m_smpCenterOff(i,0); }

};
/////////////////////////////////////////////////////////////////////
// class SkSegment
class SkSegment
{
private:
	SkIntPointArray m_ptArray;//saves all the points, use space to save compuation
	int m_nPtAxis;//the points in ptArray are along x or y axis: 0~x, 1~y
	int m_nPrflSign;//profile direction, -1: decreasing, 1: increasing

	bool m_bPtsComputed;
	SkIntPoint m_pEndPt[2];
	int m_nType;//0-line, 1-arc, 2-curve
	double m_pdParam[3];//param[0] - angle, 1- radius, 
	double m_dMeanDX, m_dMeanDY;
	double m_dStdDX, m_dStdDY;
public:
	SkBranchProfile m_prfl;
public:
	SkSegment() { m_nType = -1; m_bPtsComputed = false; }
	void setEndPoint(SkIntPoint &p, int e) { m_pEndPt[e] = p; m_bPtsComputed = false; }
	int getPtNum() { if (!m_bPtsComputed) computePoints(); return m_ptArray.getSize(); }
	SkIntPoint getEndPoint(int e) {return m_pEndPt[e];}
	SkIntPoint getPoint(int i) { if (!m_bPtsComputed) computePoints(); return m_ptArray[i]; }
	SkFloatPoint getFloatPoint(int i); 
	int computePoints();
	int  computeProfile(Matrix<float> *pImgOrg, int nHalfWd);
	double computeImageCodingLengthGain() 
	{ return m_prfl.computeImageCodingLengthGain(); }
	double* getModelIntensity(){ return m_prfl.m_ridgeModel.getModelIntensity();}
	int getPointAxis(){ return m_nPtAxis; }

};
#ifndef SkSegmentArray
#define SkSegmentArray oval_Array<SkSegment>
#endif


/////////////////////////////////////////////////////////////////////
// class SkPrimBranch
class SkPrimBranch
{
#define SK_MAX_BRANCH_LENGTH 1024*32
private:
	int m_pnPrimID[2];

	SkIntPointArray m_ptArray;//the points, absolute point of linelet
	SkFloatArray m_ortArray;//the ort for each point,[0,PI)
	SkSegmentArray	m_segArray;//segments array
	double m_dLinearity;//linearity, 0~1.0
	double m_dOrt;//[-PI, PI)
	double m_dR;//distance of the (0,0) to the line
	oval_FArray m_curv;//curvature
	double m_dRad;//radius for blob
	SkIntPoint m_refPt;//reference point for blob

	SkBranchProfile m_prfl;
	int m_pnLinesegEndIndex[1024];//the index of the end point of each segment

	double m_dImgCLG;//colding length gain
public:
	static int m_pnX[SK_MAX_BRANCH_LENGTH];
	static int m_pnY[SK_MAX_BRANCH_LENGTH];
	static int m_nHalfWd;
public:
	SkPrimBranch() {m_pnPrimID[0]=-1; m_pnPrimID[1]= -1;}// clear(); }
	~SkPrimBranch() {};
	//buildings
	int build(int x0, int y0, int x1, int y1, Matrix<float> &imgOrg);

	//settings
	void clear() { m_ptArray.removeAll(); m_pnPrimID[0]=-1; m_pnPrimID[1]= -1; }
	void setRefPoint(SkIntPoint &p){m_refPt = p;}
	void setRadius(double r) { m_dRad = r; }
	
	//gettings
	SkIntPoint getRefPoint() { return m_refPt; }
	double getRadius() { return m_dRad; }
	SkIntPoint& getPoint(int i) { return m_ptArray.getAt(i); }
	SkIntPoint getEndPoint(int e) { if (e == 1) return m_ptArray.getAt(m_ptArray.getSize()-1); else return m_ptArray.getAt(0); }
	int getPointNum() { return m_ptArray.getSize(); }
	int getLinesegEndIndex(int i) { return m_pnLinesegEndIndex[i]; }
	int getProfileWidth() { return m_nHalfWd*2+1; }

	int addPoint(SkIntPoint p, int e) 
		{ //if (m_ptArray.hasMember(p)) return -1; 
		if (e==1) return m_ptArray.add(p); else return m_ptArray.addAt(0,p);}
	void addPointAt(SkIntPoint p, int pos) { m_ptArray.addAt(pos, p); }

	void saveSketch2Matrix(Matrix<int> &mat, SkIntPoint &refPt); 
	void saveProfile2Matrix(Matrix<float> &mat, SkIntPoint &refPt); 
	void saveTransformedProfile2Matrix(Matrix<float> &mat, SkIntPoint &refPt); 
	void saveOrgImg2Matrix(Matrix<float> &mat, SkIntPoint &refPt, Matrix<float>* pImgOrg, SkIntPoint &refPtOrg); 
	void translate(SkIntPoint dp);

	//profile
	Matrix<double> &getProfile();
	int computeProfile(Matrix<float> *pImgOrg);
	int removeSamePointInPtArray();

	//segments
	int resetPointsFromSegments();
	int computeSegments(double pix_tol, double ang_tol, int link_rad );
	int recomputeSegmentsFromFloatPoints(double pix_tol, double ang_tol, int link_rad );
	int computeCurvature(int nHalfWindowSize);
	int addSeg(SkSegment &seg) { return m_segArray.add(seg); }
	int getSegNum() { return m_segArray.getSize(); }
	SkSegment *getSegment(int i) { return &(m_segArray[i]); }
	SkSegment *getEndSegment(int e) { if (e==0) return getSegment(0);
				else return getSegment(getSegNum()-1);}
	int setAllPointsAsOneSegment();

	int computeLinearity();
	double getOrt() { return m_dOrt; }
	double getR() { return m_dR; }
	double getLinearity() { return m_dLinearity; }

	void setPtArray(SkIntPointArray &ptArray) { m_ptArray = ptArray; }
	void setPrimID(int e, int id) { m_pnPrimID[e] = id; }
	void setOrt4EachPoint(Matrix<float>& ort);


	int getPrimID(int e) { return m_pnPrimID[e]; }
	double getEndOrt(int e);//[-PI,PI)

	int getEndPoints(int e, SkIntPointArray &ptArray, int nPtNum);
	SkIntPointArray& getPointArray() { return  m_ptArray; }
	double getCornerAngle(int i);//i: corner index
	SkIntPoint removeEndPoint(int e) ;

	double computeImageCodingLengthGain();
	double getImageCodingLengthGain() { return m_dImgCLG; }

};
/////////////////////////////////////////////////////////////////////
// class SkPrimBranch
class SkPrimBlob
{
private:
	double m_dRad;//radius for blob
	SkIntPoint m_refPt;//reference point for blob

	double m_dImgCLG;//colding length gain
public:
	SkBranchProfile m_prfl;
public:
	SkPrimBlob() {};// clear(); }
	~SkPrimBlob() {};
	//settings
	void setRefPoint(SkIntPoint &p){m_refPt = p;}
	void setRadius(double r) { m_dRad = r; }
	
	//gettings
	SkIntPoint getRefPoint() { return m_refPt; }
	double getRadius() { return m_dRad; }
	
	// computings
	int  computeProfile(Matrix<float> *pImgOrg, int nHalfWd)
	{
		int x0 = m_refPt.x + nHalfWd, y0 = m_refPt.y;
		int x1 = x0, y1 = y0-1;
		m_prfl.setSampleByLineProfile(x0, y0, x1, y1, pImgOrg, nHalfWd);
		m_prfl.computeMeanFromValidSamples();
		m_prfl.buildEdgeRidgeModel();
		return 0;
	}
	double computeImageCodingLengthGain() 
	{ return m_dImgCLG = m_prfl.computeImageCodingLengthGain(); }

};
/////////////////////////////////////////////////////////////////////
// class SkVirtualPrim
class SkVirtualPrim
{
private:
	oval_Array<int> m_branchIDArray;
	oval_Array<int> m_branchEndArray;
public:
	SkIntPoint m_refPt;//reference point
//	int m_nDegree;
public:
	SkVirtualPrim() { }//m_nDegree = 0; 
	~SkVirtualPrim() {}
	void setDegree(int d) {m_branchIDArray.setSize(d); m_branchEndArray.setSize(d);}//m_nDegree = d; 
	int getDegree() { return m_branchIDArray.getSize(); }
	void setRefPoint(SkIntPoint &p) { m_refPt = p; }
	void setRefPoint(int x, int y) { m_refPt.x = x; m_refPt.y = y; }

	int getBranchNum() { return m_branchIDArray.getSize(); }

	int getBranchID(int b) { return m_branchIDArray[b]; }
	int getBranchEnd(int b) { return m_branchEndArray[b]; }
	int addBranchIDAndEnd(int id, int e);
	int removeBranch(int id, int e);
};

#ifndef SkBranchNode
#define SkBranchNode SkMapNode<SkPrimBranch>
#endif

#ifndef SkBranchMap
#define SkBranchMap SkMap<SkPrimBranch>
#endif

#ifndef SkBlobNode
#define SkBlobNode SkMapNode<SkPrimBlob>
#endif

#ifndef SkBlobMap
#define SkBlobMap SkMap<SkPrimBlob>
#endif

#ifndef SkVPrimNode
#define SkVPrimNode SkMapNode<SkVirtualPrim>
#endif

#ifndef SkVPrimMap
#define SkVPrimMap SkMap<SkVirtualPrim>
#endif


#ifndef SkPrimNode
#define SkPrimNode SkMapNode<SkPrimitive>
#endif

#ifndef SkPrimMap
#define SkPrimMap SkMap<SkPrimitive>
#endif

/////////////////////////////////////////////////////////////////////
// class SkPrimOccupancy
class SkPrimOccupancy
{
public:
	Matrix<oval_IntArray> m_primID;//the primitive IDs which occupy this point
	Matrix<oval_IntArray> m_branchNo;// the branch id
	Matrix<int> m_primIDofRefPt;//the prim id 
public:
	void clearAll();
	void setDimension(int ht, int wd); 
	void addOccupy(SkIntPoint p, int prim_id, int branch_no, bool bRefPt);
	void removeOccupy(SkIntPoint p, int prim_id, int branch_no, bool bRefPt);
	
	int getOccupyPrimIDOfRefPt(int x, int y){ return m_primIDofRefPt(y,x)-1; }

	bool occupied(int x, int y) { if ( m_primID(y,x).getSize()> 0) return true; return false; }
	int occupyNum(int x, int y) { return m_primID(y,x).getSize(); }
	int occupyPrimID(int x, int y, int i) { return m_primID(y,x).getAt(i); }
	int occupyBranchNo(int x, int y, int i) { return m_branchNo(y,x).getAt(i); }

};

class SkPrimPursuit;

/////////////////////////////////////////////////////////////////////
// class SkSketchCLManager, skech coding length manage
class SkSketchCLManager
{//
public:
	static double m_dCL4VP;
	static double m_dCL4Pointer2VP;
	static double m_dImgCodingSigma2;//2*sigma^2 for the image coding
public:
	double getCL4Vp() { return m_dCL4VP; }
	double getCL4Pointer2Vp() { return m_dCL4Pointer2VP;}
	double computeCL4dA(SkBranchProfile &prfl0, SkBranchProfile &prfl1);
};


class SkEndConnection
{
public:
	int m_id0;//the branch id0
	int m_end0;
	int m_id1;//the vp or curve/branch ID
	int m_end1;//the curve end no, (possible)
	int m_x, m_y;//the point which the end connects to
	int m_type;//connection type, 0~VP, 1~curve, 2~curve extension
	double m_dEnergy;//the energy to build this connection
	double m_dImgCLG, m_dSkCLG;//image colding length gain, sketch coding length gain
	double m_dDeltaOrt0, m_dDist0;//the delta orientation and dist of the connection line to the first branch end
	double m_dDeltaOrt1, m_dDist1;//the delta orientation and dist of the connection line to the first branch end
public:
	bool operator ==(SkEndConnection &c)
	{ if (m_id0==c.m_id0&&m_id1==c.m_id1&&m_end0==c.m_end0&&m_end1==c.m_end1&&m_x==c.m_x&&m_y==c.m_y&&m_type==c.m_type)
		return true; else return false;}

};
class SkPrimPursuitParam
{
public:
	int m_nSaveIntermediateResults;

	double m_dPixTol;
	double m_dAngTol;
	int m_nLinkRad;

	int m_nSaveCurveByCurveStep;

	double m_dMagThresh1;//magnitude thresh for bases

	double m_dMinImgCLG;
	double m_dExtensionDist4Refine;
	double m_dMinTotalCLG;
	double m_dExtRegionHalfLeft;
	int m_nExtTrapezoidleRegion;

public:
	SkPrimPursuitParam();
	~SkPrimPursuitParam();
	int read(const char *name);
	int save(const char *name);

};
/////////////////////////////////////////////////////////////////////
// class SkPrimManager
class SkPrimManager
{
public:
	// parameters
	SkPrimPursuitParam m_param;//parameters

	SkSketchCLManager m_skCLMan;

	Matrix<int> m_bound;
	Matrix<float> m_imgOrg;
	double m_dImgMean, m_dImgStd;//original image mean and std
	char m_pchDir[512];//operation directory
	char m_pchProjName[512];//project name
	// virtual prim
	SkVPrimMap m_vpMap;
	SkBranchMap m_branchMap;
	SkBlobMap m_blobMap;

	bool m_bSegmentsComputed;
	bool m_bProfileComputed;

//	bool m_bUpdateBhVpIDs;
	Matrix<int> m_vpID;
	Matrix<oval_IntArray> m_branchIDs;//the primitive IDs which occupy this point
	
	oval_Array<int> m_sortedBhIDs;//sorted branch IDs by img CLG
	oval_Array<float> m_bhImgCLGs;//the coding length gains (CLG) sorted


	bool m_bUpdateExtCon;
	Matrix<oval_IntArray> m_conOcp;//the constraints (7 pixel width ) occupy
	Matrix<oval_IntArray> m_extOcp;//extension occupy branch id
	Matrix<oval_IntArray> m_extEndOcp;//the extension end of correspondent branch
	Matrix<SkPixCandArray> m_pixCandMat;


	//temporary 
	int m_pnX[1024*32], m_pnY[1024*32];
public:
	SkPrimManager();
	int initializeOcpIDMatrices();
	int	recomputeOcpIDMatrices();
	void removeAll() { m_vpMap.removeAll(); m_branchMap.removeAll(); m_blobMap.removeAll(); }
	void setBound(Matrix<int> &bound) { m_bound = bound; 
										int ht, wd; m_bound.GetDimension(ht, wd); }
	void setOrgImg(Matrix<float> *pImg) { m_imgOrg = *pImg; }
	void setOrgImgMeanStd(double dMean, double dStd) { m_dImgMean = dMean; m_dImgStd = dStd; }
	void setDir(const char *pDir) { strcpy(m_pchDir, pDir); }
	void setProjName(const char *pProj) { strcpy(m_pchProjName, pProj); }
	void getImageDimension(int &ht, int&wd) {m_imgOrg.GetDimension(ht, wd);}

	// primitive map
	int buildPrimitiveMap();
	//
	int combineConnectedFreeEndPointsIntoOneBranch();
	int combineConnectedFreeEndPointsIntoOneBranch(int id0, int e0);
	int combineConnectedFreeEndPointsByANewVP();
	int combineConnectedFreeEndPointsByANewVP(int id0, int e0);

	int extendBranchEnd2AJunction(int id, int e, int x1, int y1);
	int extendBranchEnd2ABranch(int id, int e, int x1, int y1);
	int extendBranchEnd2ABranchExtension(int id0, 
										int end0, 
										int id1, 
										int end1, 
										int x1, int y1);
	int addExtendedPoints2ABranch(SkBranchNode *pBhNode, int e, int x1, int y1);

	int checkBranchEndPoint(int id0, int x, int y);
	int removeDegree1And2VP();

	bool crossedConnection(int x1, int y1, int x2, int y2);
	bool crossedConnection(int x1_s, int y1_s, int x1_e, int y1_e,
										  int x2_s, int y2_s, int x2_e, int y2_e);
	bool crossedEndConnection(SkEndConnection &cnct);
	int removeCrossedConnection(oval_Array<SkEndConnection> &cnctArray);

	//profiles
	int buildBranchProfiles();
	int buildBranchSegments();
	int diffuseBranchPoints();
	// nbr relationship
//	int addOneBranchNbr(SkPrimNode* pPrimNode, int b);

	// virtual primitives
	int findAllVirtualPrimitives(Matrix<int> &bound);
	int findAllBranches(Matrix<int> &bound);
	int	addOneVirtualPrimitive(int x, int y);
	int addOneBranch(SkIntPointArray &ptArray, int vpID0, int vpID1);
	int removeShortFreeBranches(int nMinBranchLength);
	int removeOneBranch(int id);
	int removeShortBranches(double dMinBranchLength);
	int removeSmallImgCLGBranches(double dMinImgCLG);

	int removeOneBranchFromOneVirtualPrimitive(int bhID0, int e0, int vpID);
	int combineTwoBranches(int id0, int e0, int id1);
	int combineOneBranch(int id0, int e0);	
	int combineTwoContinuedBranches(int bhID[2], int end[2]);
	int connectTwoContinuedBranchesByANewVP(int bhID[2], int end[2]);
	int getVituralPrimitiveNum() {return m_vpMap.getSize(); }
	int getBranchNum() { return m_branchMap.getValidNodeNum(); }
	int getSegmentNum();

	int findPointsOfOneBranch(int x1, int y1, Matrix<int> &bound, Matrix<int> &vpID, 
										 SkIntPointArray &ptArray, int vpID0);
	int computePixCand(Matrix<SkPixCandArray> &rec_cand);//the candidate image

	int findSketchPixels(SkPrimBranch *pBranch, int &num, int *pnX, int *pnY);
	int findConPixels(SkPrimBranch *pBranch, int &num, int *pnX, int *pnY, int hf_prfl);
	int findNearbyPixels(SkPrimBranch *pBranch, int end, int &num, int *pnX, int *pnY);
	int findExtensionPixels(SkPrimBranch *pBranch, int end, int &num, int *pnX, int *pnY, double dExtDis);
	int recomputeExtConOcp(int hf_prfl, double extDis);
	int updateGeneralOcp(Matrix<oval_IntArray>&ocp, int id, 
									int x0, int y0, int x1, int y1, int flag);
	int updateGeneralOcp(Matrix<oval_IntArray>&ocp, int id, int num, int *pnX, int *pnY, int flag);
	int updateExtensionOcp(Matrix<oval_IntArray>&extOcp, 
									  Matrix<oval_IntArray>&extEndOcp,
									  int id, int end, 
									  int num, int *pnX, int *pnY, 
									  int flag);

	int updatePixCand4ABranch(SkPrimBranch *pBranch, 
							   Matrix<SkPixCandArray> &rec_cand,//the candidate image
								int add_remove, bool bSharp);
	int updatePixCand4ALinelet(int x0, int y0, int x1, int y1,//start and end absolute pt
						   int w,//the width of the profile
						   double *vm,//the intensity value of the profile
						   Matrix<SkPixCandArray> &rec_cand, int add_remove);//the candidate image

	int addPrimitivesFromOtherManager(SkPrimManager &primMan);

	//comtinve vps
	int combineConnectedVirtualPrimitives();
	int getConnectedPrimitiveRefPt2Matrix(Matrix<int> & mat);
	int combineVirtualPrimitivesWithConnectedRefPts(SkIntPointArray& ptArray);
	int combineVirtualPrimitives(SkIntPoint refPt,//new refence points
									 int *pnPrimIDs,//primitives needed to be combined
									 SkIntPointArray& ptArray);//the ref_pts of all primitives

	
	// energy, coding length gain
	int computeImageCodingLengthGain4EachBranch();
	int sortBrachesByCodeingLengthGain();
	//energy 
	int computeConnectionEnergy(oval_Array<SkEndConnection> &cnctArray);
	int computeConnectionEnergy_woimg(oval_Array<SkEndConnection> &cnctArray);
	double computeConnectionEnergy(SkEndConnection &cnct);
	double computeConnectionEnergy_woimg(SkEndConnection &cnct);
	double energyChangeByConnectingBranchEnd2AVp(SkEndConnection &cnct);
	double energyChangeByExtendingBranchEnd2AVp(SkEndConnection &cnct);
	double energyChangeByConnectingBranchEnd2ABranchInside(SkEndConnection &cnct);
	double energyChangeByExtendingBranchEnd2ABranchInside(SkEndConnection &cnct);
	double energyChangeByExtendingBranchEnd2ABranchExtension(SkEndConnection &cnct);
	double energyChangeByExtendingBranchEnd2ABranchEnd(SkEndConnection &cnct);
	
	double imageCodingLengthGainByAddingOneBranch(SkPrimBranch &branch);
	double imageCodingLengthGainByExtendingBranchEnd2APoint(int id0, int end0, SkIntPoint &p2);
	double imageCodingLengthGainByExtendingBranchEnd2ABranchExtension(int id[2], int end[2], SkIntPoint &p2);
	double sketchCodingLengthGainByConnectingBranchEnd2AVp(SkPrimBranch *pBranch0, int end0, SkPrimBranch &branch);
	double sketchCodingLengthGainByExtendingBranchEnd2AVp(SkPrimBranch *pBranch0, int end0);
	double sketchCodingLengthGainByExtendingBranchEnd2ABranchInside(SkPrimBranch *pBranch0, int end0);
	double sketchCodingLengthGainByExtendingBranchEnd2ABranchExtension();

	double codingLengthGainOfNewImage(Matrix<float> &org, Matrix<float> &rec, Matrix<int> &mask);

	bool isAtBranchExtension(SkPrimBranch *pBranch0, int end0, int x, int y, double m_dPixTol);
	int outputEndConnectionInfo(char *msg, SkEndConnection &cnct);


	//output
	int saveSketchableLattice(Matrix<int> &con);
	int saveSketchImageReconstructed(Matrix<float> &img, bool bSharp);
	int savePointsAndSketches(const char *type);
	int savePointsAndSketchesWithVertexDots(const char *type);
	int saveVerticesWithLegs(const char *type);
	int savePointsOfCurves(Matrix<int> &mat, int value);
	int saveStraightLinesOfCurves(Matrix<int> &mat, int value, int line_width);
	int markEndPointsAsDots(Matrix<int> &mat, int value);
	int markCornersAsDots(Matrix<int> &mat, int value);
	int markJunctionsAsDots(Matrix<int> &mat, int value);
	int saveCurveByCurve(const char *type, int step);
	int saveSingleJoints(const char *type);

	int saveImageAndSketch4ALinelet(int x0, int y0, int x1, int y1,//start and end absolute pt
						   int xe, int ye, //the absolute point which should be at the center of final image
						   int xd, int yd, //the position of (xe,ye) in the new image
						   int w,//the width of the profile
						   double *vm,//the intensity value of the profile
						   Matrix<float> &org,//original image to be saved
						   Matrix<SkPixCandArray> &rec_cand,//the candidate image
						   Matrix<int> &sk,//the sketch image
						   Matrix<float> &org0);//the whole original image
	int saveResidualImagesStokeByStroke(const char *type, int nStep);
	int saveAnExampleJoint();//just for a figure in the thesis
	int saveSketchGraphAsBmp(const char *type, int line_width, SkPrimPursuit *pPP);
	int saveAsText(const char *name);
	int readFromText(const char *name);
	int savePrimitiveGeometricStatistics(const char *type);



};
int skAddBoundary(const char *name, int bound);

int skLearnPrimitives(FILE *pFile);//learn the primives from a set of images whose names are saved in a file
int skBuildPrimitiveMap(const char *name, SkPrimManager *pPrimMan);
int skLearnProfilePrimitives(Matrix<double> &samples);


int skFindConnectedPoints(int x0, int y0,//the start point
						  SkIntPointArray &nbrPtArray,//nbr point array
						  Matrix<int> &imgBound);//the binary image of primitive positions, 255-valid, 0-not valid
int skFindAllConnectedPoints(int x, int y,//start point
						  SkIntPointArray &ptArray,//the array saves the connected points
						  Matrix<int> &mat);//the matrix indicates the points, 0-invalid

int	skThinnerBoundary(Matrix<int> &bound);//0-invalid,
bool skThinnerBoundary(int x1, int y1,//first point
					  int x2, int y2,//second point
					  Matrix<int> &bound);//0-invalid

bool skThinnerBoundaryIndirectly(int x1, int y1,//first point
								 int x2, int y2,//second point
								 Matrix<int> &bound);//0-invalid

//int skFindShortestPath(SkIntPoint p1, SkIntPoint p2, //path from p1-->p2,
//					   SkIntPointArray &pathPtArray, //saves the path, not include the last move, the first move (0,0) is included
//					   SkIntPointArray &ptArray);//the points on the path limited to the ptArray
int skComputeCurvature(SkIntPointArray& ptArray, //the point array
					   double *pCur,//the curvature for each point
					   int nHalfWindow);//half window size
int skComputeCornerness(SkIntPointArray& ptArray, //the point array
					   double *pCur,//the curvature for each point
					   int nHalfWindow);//half window size

//int skSaveLargePrimitiveIcons(SkPrimManager *pPrimMan,//the primitive managers
//							  int nPrimManNum,//the number of primitive manager
//							  Matrix<double>& prfl_samples);//all of the profiles
Matrix<double> skTCA1DPrimitive(Matrix<float> &org, //org image
								int w, int h,//the size of used image
								SkIntPoint &p0,//center of used patch
								double &dOrt, //orientation of the new center line
								double &dScale,//scale of perpendicular direction to the profile
								int &nType, //type of the profile 0-edge, 1-bar
								double &dError);//total reconstruction error
int findEndPointOfConnection(int x, int y, Matrix<int> &bound, SkIntPointArray &ptArray0);
int findSequentialPointOfConnection(int x, int y, Matrix<int> &bound, SkIntPointArray &ptArray0);

#endif