
#include "skPyramid.h"
#include "skMatrixIplOp.h"

/////////////////////////////////////////////////////////////////////
// class SkPyramid

SkPyramid::SkPyramid()
{
	setDefaultParameters();
}

SkPyramid::SkPyramid(int ht, int wd)
{
	setDefaultParameters();
	Matrix<float> M(ht, wd);
	M.Zero();
	importImage(M);
}
void SkPyramid::setDefaultParameters()
{
	m_nLevels=1; 
	m_dGauKerSize=3.0; 
	m_nZoomScale=1;
}
void SkPyramid::importImage(Matrix<float> &I)
{
	Matrix<float> B,C,D;
	float m;
	double v;
	double mean, std;
	C=I;
	for (int i=m_nLevels-1;i>=1;i--)
	{
		B=skGaussianBlur(C,m_dGauKerSize);
		D=skReduceMatrix(B,m_nZoomScale);
		B=skEnlargeMatrix(D,B.cols(), B.rows());
		m_pImScale[i]=C-B;
		
		skNormalizeMatrix2ZeroMean1Std(&m_pImScale[i], mean, std);
		m_pMean[i]=mean;
		m_pStd[i]=std;
		if (m_pStd[i]<0.001)
			m_pStd[i]=0.001;
//		v=m_pStd[i];
//		m_pImScale[i]=(m_pImScale[i]-skOnes(B.nx(),B.ny())*m)/v;
		
		// don't set zero mean and unit variance any more, this is due to base pursuit on edge strengh image
		//	07-23-2003
		// WE MUST set zero mean and unit variance for general base pursuit, otherwise the min_coef is changing 08-09-2002
//		m_pMean[i]=0;
//		m_pStd[i]=1;

		C=D;//skReduceMatrix(B,m_nZoomScale);
	}
	// May 21, 2002
	// I don't understand why not set m_pImScale[0] to zero m_pMean and 1 var
	// Set it. May lead to new problems unknown currently
//	m_pMean[0] = 0;
//	m_pStd[0] = 1;
//	m_pImScale[0]=C;
	// don't set zero mean and unit variance any more, this is due to base pursuit on edge strengh image
	//	07-23-2003
		// WE MUST set zero mean and unit variance for general base pursuit, otherwise the min_coef is changing 08-09-2002
	skNormalizeMatrix2ZeroMean1Std(&C, mean, std);
	m_pMean[0]=mean;
	m_pStd[0]=std;
	if (m_pStd[0]<0.001)
		m_pStd[0]=0.001;
	m_pImScale[0] = C;
//	m_pMean[0] = C.Mean();
//	m_pStd[0] = sqrt(C.Var());
//	m_pImScale[0]=(C-skOnes(C.nx(),C.ny())*m_pMean[0])/m_pStd[0];

	reconstruct();
}
void SkPyramid::zeroMeanStd()
{
	for (int i=m_nLevels-1;i>=1;i--)
	{
		m_pMean[i] = 0;
		m_pStd[i] = 1.0;
	}
}

Matrix<float> SkPyramid::reconstruct()
{
	Matrix<float> B,C;
	int nx,ny;
	//C=m_pImScale[0];
	C=m_pImScale[0] * m_pStd[0] + m_pMean[0];//May 21, 2002
	for(int i=1;i<m_nLevels;i++)
	{
		B=skEnlargeMatrix(C,m_pImScale[i].nx(),m_pImScale[i].ny());
		C=B+m_pImScale[i]*m_pStd[i]+skOnes(B.nx(),B.ny())*m_pMean[i];
	}
	nx=C.nx();ny=C.ny();
	SetDimension(ny,nx);
	for (int i=0;i<ny;i++)
		for (int j=0;j<nx;j++)
		{
			data[((long)i) * nx + j]=C.Data(i,j);
		}
	return C;
}
SkPyramid& SkPyramid::operator=(const SkPyramid& I )
{
	m_nLevels = I.m_nLevels;
	m_dGauKerSize = I.m_dGauKerSize;
	m_nZoomScale =  I.m_nZoomScale;
	for (int i=0;i<m_nLevels;i++)
	{
		m_pImScale[i]=I.m_pImScale[i];
		m_pMean[i]=I.m_pMean[i];
		m_pStd[i]=I.m_pStd[i];
	}
	reconstruct();
	return *this;
}

int SkPyramid::copyImage2LowestLevel()
{
	int i = m_nLevels-1;
	m_pImScale[i] = *this;
	m_pImScale[i] = (m_pImScale[i] - skOnes(nx(), ny())*m_pMean[i])/m_pStd[i];
	return 0;
}

/////////////
// copyMeanStd()
//	copy m_pMean and m_pStd from another pyramid
int SkPyramid::copyMeanStd(SkPyramid &pyr)
{
	int nLevel = pyr.m_nLevels;
	for (int i = 0; i < nLevel; i++)
	{
		m_pMean[i] = pyr.m_pMean[i];
		m_pStd[i] = pyr.m_pStd[i];
	}
	return 0;
}

// auxilary functions
Matrix<float> skOnes(int nx,int ny)
{
	Matrix<float> M;
	M.SetDimension(ny,nx);
	M.Set(1);
	return M;
}
Matrix<float> skZeros(int nx,int ny)
{
	Matrix<float> M;
	M.SetDimension(ny,nx);
	M.Zero();
	return M;
}
Matrix<float> skGaussianBlur(Matrix<float> &A, int kersize)
{
	Matrix<float> GaussKer,C;
	GaussKer.SetGaussian(1,kersize);
	C=A.ConvHor(GaussKer);
	return C.ConvVer(GaussKer.T());
}
Matrix<float> skReduceMatrix(Matrix<float> &A, int ntimes)
{
	int nx=A.nx(),ny=A.ny();
	int x,y,nnx=nx/ntimes,nny=ny/ntimes;
	if (nx%ntimes!=0)
		nnx++;
	if (ny%ntimes!=0)
		nny++;
	Matrix<float> B;
	B.SetDimension(nny,nnx);
	for(x=0;x<nnx;x++)
		for(y=0;y<nny;y++)
			B(y,x)=A.Data(y*ntimes,x*ntimes);
	return B;
}
Matrix<float> skEnlargeMatrix(Matrix<float> &A, int nx,int ny)
{
	int x,y,ntimes=nx/A.nx();
	if (ntimes<ny/A.ny())
		ntimes=ny/A.ny();
	if ((ntimes*A.nx()<nx)||(ntimes*A.ny()<ny))
		ntimes++;
	Matrix<float> B;
	B.SetDimension(ny,nx);
	for(x=0;x<nx;x++)
		for(y=0;y<ny;y++)
			B(y,x)=A.Data(y/ntimes,x/ntimes);

	return B;

}














