#ifndef _SK_PYRAMID_H_
#define _SK_PYRAMID_H_

#include "skMatrix.h"

#ifndef SK_MAX_PYRAMID_LEVEL
#define SK_MAX_PYRAMID_LEVEL 3
#endif

//auxiliary functions
Matrix<float> skGaussianBlur(Matrix<float> &A, int kersize);
Matrix<float> skReduceMatrix(Matrix<float> &A, int times);
Matrix<float> skEnlargeMatrix(Matrix<float> &A, int nx,int ny);
Matrix<float> skOnes(int nx,int ny);
Matrix<float> skZeros(int nx,int ny);;

/////////////////////////////////////////////////////////////////////
// class SkPyramid
class SkPyramid : public Matrix<float>
{
public:
	int m_nLevels;
	double m_dGauKerSize;
	double m_nZoomScale;

	Matrix<float>	m_pImScale[SK_MAX_PYRAMID_LEVEL];		//image at scale 0=Gaussian blur, 1...SK_MAX_PYRAMID_LEVEL-1=diference
	float m_pMean[SK_MAX_PYRAMID_LEVEL];
	double m_pStd[SK_MAX_PYRAMID_LEVEL];
public:
	SkPyramid();
	SkPyramid(int ht, int wd);

	void setDefaultParameters();
	void setLevels(int l) { m_nLevels = l; }
	void setGaussianKernalSize(double s) { m_dGauKerSize = s; }
	void setZoomScale(int s) { m_nZoomScale = s; }
	int getLevels() { return m_nLevels; }
	double getGaussianKernalSize() { return m_dGauKerSize; }
	int getZoomScale() { return m_nZoomScale; }

	void importImage(Matrix<float> &I);
	Matrix<float> reconstruct();
	SkPyramid&	operator=(Matrix<float> &I){ importImage(I);return *this;}
	SkPyramid&	operator=(const SkPyramid &I );

	void zero() { Matrix<float>::Zero(); for (int i=0; i<m_nLevels; i++) m_pImScale[i].Zero(); }
	void zeroMeanStd();
	int copyImage2LowestLevel();

	int copyMeanStd(SkPyramid &pyr);

};

#endif //_PYRAMID_H_