#include <memory.h>
#include <math.h>
#include <direct.h>
#include <malloc.h> 

#include "skMap.h"
#include "skSketchGraph.h"
#include "skOutput.h"
#include "skMatrixIplOp.h"

/////////////////////////////////////////////////////////////////////
// class SkPoint

bool SkPoint::operator == ( SkPoint p )
{
	if( x != p.x )
		return false;
	if( y != p.y )
		return false;
	return true;
}
SkPoint SkPoint::operator + ( SkPoint p )
{
	SkPoint p2;
	p2.x = x + p.x;
	p2.y = y + p.y;
	return p2;
}
bool SkPoint::operator != ( SkPoint p )
{
	if( x == p.x && y == p.y )
		return false;
	return true;
}
double SkPoint::Distance(SkPoint p)
{
	return sqrt(double((x-p.x)*(x-p.x) + (y-p.y)*(y-p.y)));
}
/////////////////////////////////////////////////////////////////////
// class SkNode
int SkNode::computeProfile(Matrix<float> *pImgOrg, int nMaxPfWd, int nScale, Matrix<SkPixCandArray>* pPixCandMat)
{
	SkNode *pNext = m_pNext;
	if (pNext == 0)
		return 0;
	SkPoint &p0 = m_data;
	SkPoint &p1 = pNext->m_data;
	assert(m_nMaxPrflWd > 0);
	int nPfWd = std::min<int>(nMaxPfWd, m_nMaxPrflWd);

	int nMaxPixelNum = int(p0.Distance(p1) * 2 * nScale);
	m_prfl.setScale(nScale);
	m_prfl.setWidthAndMaxSampleNum(nPfWd*nScale, nMaxPixelNum);
	m_prfl.addLineProfile(p0.x, p0.y, p1.x, p1.y, pImgOrg, *pPixCandMat);
//	m_prfl.computeGaussianModelFromSamples();//compute Gassian model and m_normMu
	m_prfl.computeMeanFromValidSamples();
	m_prfl.saveLineProfile(p0.x, p0.y, p1.x, p1.y, *pPixCandMat);
	
	m_prfl.normalize();

	return 0;
}
/////////////////////////////////////////////////
// shortenProfile()
//	shortenProfile the profile, set the best length (cut the line segment into two if needed)
//	return 0 if no cutting, return the new SkNode, if cut
SkNode* SkNode::shortenProfile(Matrix<float> *pImgOrg, int nMaxPfWd)
{
	assert(false);

	SkNode *pEnd = m_pNext;
	if (m_pNext == 0)
	{
		return 0;
	}
	SkPoint &p0 = m_data;
	SkPoint &p1 = m_pNext->m_data;

	int nOffset, nWhich;//the offset and which point, 0-center, 1-end, 2-both
	nOffset = m_prfl.maxOffsetOfSamples(nWhich);
	if (nOffset == 0)
	{
		return 0;
	}

	SkNode *pAdd = new SkNode;
	*pAdd = *this;
	//set linklist	
	pAdd->m_pPrev = this;
	pAdd->m_pNext = pEnd;
	this->m_pNext = pAdd;
	pEnd->m_pPrev = pAdd;

	SkPoint &p = pAdd->m_data;
	p.x = (p0.x+p1.x)/2;
	p.y = (p0.y+p1.y)/2;

	bool bProfileWdOnX;//the profile width is on X
	int s;
	if (fabs(double(p0.x-p1.x))>=fabs(double(p0.y-p1.y)))
	{
		bProfileWdOnX = false;
		s = sign(p1.x-p0.x);
	}
	else
	{
		bProfileWdOnX = true;
		s = sign(p1.y-p0.y);
	}

	if (nWhich == 0)
	{//change the center point
		if (bProfileWdOnX)
			p.x += nOffset*s;
		else
			p.y += nOffset*s;
	}
	else if (nWhich == 1)
	{//change the last point
		if (bProfileWdOnX)
			p1.x += nOffset*s;
		else
			p1.y += nOffset*s;
	}
	else
	{//change both
		if (bProfileWdOnX)
		{
			p.x += nOffset;
			p1.x += nOffset;
		}
		else
		{
			p.y += nOffset;
			p1.y += nOffset;
		}
	}

//	this->computeProfile(pImgOrg, nMaxPfWd);
//	pAdd->computeProfile(pImgOrg, nMaxPfWd);

	return pAdd;
}

/////////////////////////////////////////////////
// thinnerProfile()
//	thinner the profile, set the best width
//	return 0
int SkNode::thinnerProfile()
{
	SkNode *pNext = m_pNext;
	if (pNext == 0)
		return 0;
	SkPoint &p0 = m_data;
	SkPoint &p1 = pNext->m_data;

	int d = m_prfl.thinnerProfile();

	if (d != 0)
	{
		bool bProfileWdOnX;//the profile width is on X
		int s;
		if (fabs(double(p0.x-p1.x))>=fabs(double(p0.y-p1.y)))
		{
			bProfileWdOnX = false;
			s = sign(p1.x-p0.x);
		}
		else
		{
			bProfileWdOnX = true;
			s = sign(p1.y-p0.y);
		}

		if (bProfileWdOnX)
		{
			p0.x += d*s;
			p1.x += d*s;
		}
		else
		{
			p0.y += d*s;
			p1.y += d*s;
		}
	}
	return m_prfl.getValidWidth();
}

/////////////////////////////////////////////////////////////////////
// class SkNode
int SkNode::computeProfileDoG(Matrix<float> *pImgOrg, int nMaxPfWd, int nScale, Matrix<SkPixCandArray>* pPixCandMat)
{
	SkNode *pNext = m_pNext;
	if (pNext == 0)
		return 0;
	SkPoint p0 = m_data;
	SkPoint p1 = pNext->m_data;
	double dis = p0.Distance(p1);
	int nMaxPixelNum = 1;//no sample in fact
	m_prfl.setWidthAndMaxSampleNum(nMaxPfWd, nMaxPixelNum);
	m_prfl.addDoGProfile(p0.x, p0.y, p1.x, p1.y, pImgOrg, *pPixCandMat);
//	m_prfl.computeGaussianModelFromSamples();
	assert(false);
	m_prfl.saveDoGProfile(p0.x, p0.y, p1.x, p1.y, *pPixCandMat);

	return 0;
}
int SkNode::saveProfile2Matrix(Matrix<SkPixCandArray> &M)
{
	SkNode *pNext = m_pNext;
	if (pNext == 0)
		return 0;
	SkPoint p0 = m_data;
	SkPoint p1 = pNext->m_data;
	m_prfl.saveLineProfile(p0.x, p0.y, p1.x, p1.y, M);
	return 0;
}
int SkNode::saveProfileDoG2Matrix(Matrix<SkPixCandArray> &M)
{
	SkNode *pNext = m_pNext;
	if (pNext == 0)
		return 0;
	SkPoint p0 = m_data;
	SkPoint p1 = pNext->m_data;
	m_prfl.saveDoGProfile(p0.x, p0.y, p1.x, p1.y, M);
	return 0;
}

/////////////////////////////////////////////////////////////////////
// class SkPointList
SkPointList::SkPointList()
{
	m_pNodeHead = 0;
	m_pNodeTail = 0;
	m_nSize = 0;
}
SkPointList::~SkPointList()
{
	RemoveAll();
}
void SkPointList::RemoveAll()
{
	SkNode *pNode = m_pNodeHead;
	while ( pNode != 0 )
	{
		SkNode *pHead = pNode->m_pNext;
		delete pNode;
		pNode = pHead;
	}
	m_pNodeHead = 0;
	m_pNodeTail = 0;
	m_nSize = 0;
	m_dLength = 0;
}
SkNode * SkPointList::AddHead( SkPoint p )
{
	SkNode *pNode;
	if( m_pNodeHead == 0 && m_pNodeTail == 0 )
	{
		pNode = new SkNode;
		m_pNodeHead = pNode;
		m_pNodeTail = pNode;
		pNode->m_pNext = 0;
		pNode->m_pPrev = 0;
		pNode->m_data = p;
		m_dLength = 0;
	}
	else
	{
		SkNode *pOldHead = m_pNodeHead;
		pNode = new SkNode;
		m_pNodeHead = pNode;
		pOldHead->m_pPrev = m_pNodeHead;
		m_pNodeHead->m_pNext = pOldHead;
		m_pNodeHead->m_pPrev = 0;
		m_pNodeHead->m_data = p;

		SkPoint p0 = pOldHead->m_data;
		double dLength = p0.Distance(p);
		m_dLength += dLength;
	}
	m_nSize++;
	return pNode;
}


SkNode * SkPointList::AddTail( SkPoint p )
{
	SkNode *pNode;
	if( m_pNodeHead == 0 && m_pNodeTail == 0 )
	{
		pNode = new SkNode;
		m_pNodeHead = pNode;
		m_pNodeTail = pNode;
		pNode->m_pNext = 0;
		pNode->m_pPrev = 0;
		pNode->m_data = p;
		m_dLength = 0;
	}
	else
	{
		SkNode *pOldTail = m_pNodeTail;
		pNode = new SkNode;
		m_pNodeTail = pNode;
		pOldTail->m_pNext = m_pNodeTail;
		m_pNodeTail->m_pPrev = pOldTail;
		m_pNodeTail->m_pNext = 0;
		m_pNodeTail->m_data = p;
		SkPoint p0 = pOldTail->m_data;
		double dLength = p0.Distance(p);
		m_dLength += dLength;

	}
	m_nSize++;
	return pNode;
}
SkNode *SkPointList::Add( int nEnd, SkPoint p )
{
	if( nEnd == 0 )
		return AddHead( p );
	else if( nEnd == 1 )
		return AddTail( p );
	else
		return 0;
}


SkPoint SkPointList::GetNext(SKPOSITION& rPosition)  // return *Position++
{
	SkNode* pNode = (SkNode*) rPosition;
	//ASSERT(AfxIsValidAddress(pNode, sizeof(SkNode)));
	rPosition = (SKPOSITION) pNode->m_pNext;
	return pNode->m_data; 
}
SkPoint SkPointList::GetPrev(SKPOSITION& rPosition) // return *Position--
{
	SkNode* pNode = (SkNode*) rPosition;
	//ASSERT(AfxIsValidAddress(pNode, sizeof(SkNode)));
	rPosition = (SKPOSITION) pNode->m_pPrev;
	return pNode->m_data; 
}

SkPoint SkPointList::RemoveHead()
{
	SkPoint p(0,0);
	if( m_pNodeHead == 0 && m_pNodeTail == 0 )
		return p;
	else
	{
		p = m_pNodeHead->m_data;
		SkNode *pOldHead = m_pNodeHead;
		m_pNodeHead = pOldHead->m_pNext;
		delete pOldHead;
		if( m_pNodeHead == 0 )
			m_pNodeTail = 0;
		else
			m_pNodeHead->m_pPrev = 0;
		m_nSize--;
		SkPoint p0 = m_pNodeHead->m_data;
		double dLength = p0.Distance(p);
		m_dLength -= dLength;
		return p;
	}
}
SkPoint SkPointList::RemoveTail()
{
	SkPoint p(0,0);
	if( m_pNodeHead == 0 && m_pNodeTail == 0 )
		return p;
	else
	{
		p = m_pNodeTail->m_data;
		SkNode *pOldTail = m_pNodeTail;
		m_pNodeTail = pOldTail->m_pPrev;
		delete pOldTail;
		if( m_pNodeTail == 0 )
			m_pNodeHead = 0;
		else
			m_pNodeTail->m_pNext = 0;
		m_nSize--;
		SkPoint p0 = m_pNodeTail->m_data;
		double dLength = p0.Distance(p);
		m_dLength -= dLength;
		return p;
	}
}

/////////////////////////////////////////////////////////////////////
// class SkEdgeRidgeModel
///////////////////
int SkEdgeRidgeModel::build(Matrix<double> &v0)//v is a column vector use (i,0) to access each member
{
	char name[512];

	int wd, ht;
	v0.GetDimension(ht, wd);

	assert(wd == 1);
	assert(ht >= 1);
	assert(ht <= SK_MAX_PRFL_ERMODEL);

//	double dHalf = 3;
	double dHalf = ht/2;
	m_dStart = ht/2 - dHalf;
	m_dEnd = ht-1;

//	return 0;

	m_nWholeDataLength = ht;

	double dMin = v0.GetMin();
	double dMax = v0.GetMax();
	double dThresh = (dMax-dMin)/4;
	if (dMin == dMax)
	{
		m_nType = -1;//not valid, flat 
		return 1;
	}

	//do the segmentation
	int r;

	// copy the values
	for (r = 0; r < ht; r++)
		m_v[r] = v0(r,0);

	int s = 0;//current seg
	m_pnSegPos[s] = 0;//start position of each segment
	m_pdSegMean[s] = m_v[0];

	s = 1;
	int c_s = -1;//the seg no of the center point
	for (r = 1; r < ht; r++)
	{
		if ( fabs(m_v[r] - m_pdSegMean[s-1]) < dThresh)
		{//keep segment, update mean
			m_pdSegMean[s-1] = (m_pdSegMean[s-1] *(r-m_pnSegPos[s-1]) + m_v[r])/(r-m_pnSegPos[s-1]+1);
		}
		else
		{//new segment
			m_pnSegPos[s] = r;
			m_pdSegMean[s] = m_v[r];
			s++;
		}
		if (r == ht/2)
			c_s = s-1;
	}
	m_pnSegPos[s] = ht;//the last one with the out of range position, for computation convenience

	assert(s > 1);//s: total true segments
	// find the zero crossings

	int i;
	for (i = 0; i < s-1; i++)
		m_pdDeriv[i] = m_pdSegMean[i+1]-m_pdSegMean[i];
	m_pdDeriv[s-1] = m_pdDeriv[s-2];//first derivative [i+1] - [i]

	int nCross = 0;
	int pnCrossS[SK_MAX_PRFL_ERMODEL];
	for (i = 1; i < s; i++)
	{
		if (m_pdDeriv[i-1] * m_pdDeriv[i] < 0)//check second derivative
		{//[i] is the seg which is the extrema
			pnCrossS[nCross] = i;
			nCross++;
		}
	}

	int s0;
	if (nCross == 0)
	{//edge
		m_nType = 0;//edge
		m_pdEdges[0] = m_pnSegPos[1];
		m_pdInt[0] = m_pdSegMean[0];
		m_dStart = m_pnSegPos[0];
		m_pdEdges[1] = m_pnSegPos[s-1];
		m_pdInt[1] = m_pdSegMean[s-1];
		m_dEnd =  m_pnSegPos[s]-1;
		m_nCenterSeg = c_s;
		m_nSegNum = s;
	}
	else if (nCross == 1)
	{//ridge
		m_nType = 1;//ridge
		s0 = pnCrossS[0];
		assert(s0 >= 1);
		assert(s >= 3);//at least three segs
		m_pdEdges[1] = m_pnSegPos[s0];
		m_pdEdges[2] = m_pnSegPos[s0+1];//the center
		m_pdEdges[0] = m_pnSegPos[1];//the start of second seg
		m_pdEdges[3] = m_pnSegPos[s-1];//the start of the last seg
		m_pdInt[0] = m_pdSegMean[0];
		m_pdInt[1] = m_pdSegMean[s0];
		m_pdInt[2] = m_pdSegMean[s-1];
		m_dStart = 0;
		m_dEnd = ht-1;
		m_nCenterSeg = s0;
		m_nSegNum = s;
	}
	else
	{//more complicated, only consider center part
		s0 = c_s;
		if (s0 == 0)
		{//the first seg, then it should be an edge
			m_nType = 0;//edge
			// find previous seg
			int ss = 0;
			m_pdEdges[0] = m_pnSegPos[ss+1];
			m_pdInt[0] = m_pdSegMean[ss];
			m_dStart = m_pnSegPos[ss];
			for (ss = s0+1; ss < s; ss++)
			{
				if (m_pdDeriv[ss] * m_pdDeriv[ss-1] < 0)
				{//find the previous 
					break;
				}
			}
			if (ss >= s)
				ss = s-1;
			m_pdEdges[1] = m_pnSegPos[ss];
			m_pdInt[1] = m_pdSegMean[ss];
			m_dEnd =  m_pnSegPos[ss+1]-1;
			m_nCenterSeg = s0;
			m_nSegNum = s;
		}
		else if (s0 == s-1)
		{//the last seg, an edge too
			m_nType = 0;//edge
			// find previous seg
			int ss;
			for (ss = s0-1; ss >= 1; ss--)
			{
				if (m_pdDeriv[ss] * m_pdDeriv[ss-1] < 0)
				{//find the previous 
					break;
				}
			}
			m_pdEdges[0] = m_pnSegPos[ss+1];
			m_pdInt[0] = m_pdSegMean[ss];
			m_dStart = m_pnSegPos[ss];
			ss = s-1;
			m_pdEdges[1] = m_pnSegPos[ss];
			m_pdInt[1] = m_pdSegMean[ss];
			m_dEnd =  m_pnSegPos[ss+1]-1;
			m_nCenterSeg = s0;
			m_nSegNum = s;

		}
		else
		{
			assert(s >= 3);//at least three segs
			if (m_pdDeriv[s0] * m_pdDeriv[s0-1] < 0)
			{//ridge
				m_nType = 1;//ridge
				m_pdEdges[1] = m_pnSegPos[s0];
				m_pdEdges[2] = m_pnSegPos[s0+1];//the center
				m_pdInt[1] = m_pdSegMean[s0];
				// find previous seg
				int ss;
				for (ss = s0-1; ss >= 1; ss--)
				{
					if (m_pdDeriv[ss] * m_pdDeriv[ss-1] < 0)
					{//find the previous 
						break;
					}
				}
				m_pdEdges[0] = m_pnSegPos[ss+1];
				m_pdInt[0] = m_pdSegMean[ss];
				m_dStart = m_pnSegPos[ss];
				for (ss = s0+1; ss < s; ss++)
				{
					if (m_pdDeriv[ss] * m_pdDeriv[ss-1] < 0)
					{//find the previous 
						break;
					}
				}
				if (ss >= s)
					ss = s-1;
				m_pdEdges[3] = m_pnSegPos[ss];
				m_pdInt[2] = m_pdSegMean[ss];
				m_dEnd =  m_pnSegPos[ss+1]-1;
				m_nCenterSeg = s0;
				m_nSegNum = s;

			}//if this a ridge
			else
			{//edge
				m_nType = 0;//edge
				// find previous seg
				int ss;
				for (ss = s0-1; ss >= 1; ss--)
				{
					if (m_pdDeriv[ss] * m_pdDeriv[ss-1] < 0)
					{//find the previous 
						break;
					}
				}
				m_pdEdges[0] = m_pnSegPos[ss+1];
				m_pdInt[0] = m_pdSegMean[ss];
				m_dStart = m_pnSegPos[ss];
				for (ss = s0+1; ss < s; ss++)
				{
					if (m_pdDeriv[ss] * m_pdDeriv[ss-1] < 0)
					{//find the previous 
						break;
					}
				}
				if (ss >= s)
					ss = s-1;
				m_pdEdges[1] = m_pnSegPos[ss];
				m_pdInt[1] = m_pdSegMean[ss];
				m_dEnd =  m_pnSegPos[ss+1]-1;
				m_nCenterSeg = s0;
				m_nSegNum = s;
			}
		}
	}

//	checkRidge2Edge();

// set the width of first and last segments less than a bound

	int bound = 2;
	if (m_nType == 0)
	{//edge
		if (m_pdEdges[0] - m_dStart >= bound)
			m_dStart = m_pdEdges[0] - bound;
		if (m_dEnd - m_pdEdges[1] >= bound)
			m_dEnd = m_pdEdges[1] + bound;
	}
	else if (m_nType == 1)
	{
		if (m_pdEdges[0] - m_dStart >= bound)
			m_dStart = m_pdEdges[0] - bound;
		if (m_dEnd - m_pdEdges[3] >= bound)
			m_dEnd = m_pdEdges[3] + bound;
	}

//	assert(m_dEnd - m_dStart >= 3);
	return 0;
}


///////////////////
// check if ridge could be better modeled by an edge, if so change it
int SkEdgeRidgeModel::checkRidge2Edge()
{
	if (m_nType != 1)
		return 0;
	double dDeltaInt1 = fabs(m_pdInt[1] - m_pdInt[0]);
	double dDeltaInt2 = fabs(m_pdInt[1] - m_pdInt[2]);

	double dMaxDelta, dMinDelta;
	int nMaxSeg;
	if (dDeltaInt1 > dDeltaInt2)
	{
		dMaxDelta = dDeltaInt1;
		dMinDelta = dDeltaInt2;
		nMaxSeg = 0;
	}
	else
	{
		dMaxDelta = dDeltaInt2;
		dMinDelta = dDeltaInt1;
		nMaxSeg = 0;
	}

	assert(dMaxDelta > 0);
	double dRatioThresh = 1.0/3.0;
	double dPosThresh = m_nWholeDataLength / 6.0;
	if (dMinDelta/dMaxDelta > dRatioThresh // one edge is not sharp enough  
		&& m_pdEdges[0] > dPosThresh // too close to the boudary
		&& m_pdEdges[3] < m_nWholeDataLength - dPosThresh - 1)
		return 0;;//no change

	// change it to edge
	m_nType = 0;//edge
	m_pdEdges[0] = m_pdEdges[nMaxSeg*2];
	m_pdEdges[1] = m_pdEdges[nMaxSeg*2+1];
	m_pdInt[0] = m_pdInt[nMaxSeg];
	m_pdInt[1] = m_pdInt[nMaxSeg+1];

	if (nMaxSeg == 0)
	{
		m_dStart = m_dStart;
		m_dEnd =  m_pdEdges[2];
	}
	else
	{
		m_dStart = m_pdEdges[1];
		m_dEnd =  m_dEnd;
	}

	return 1;
}


/////////////////////////////////////////////////////////////////////
// class SkConProfile

///////////////////
int SkConProfile::setWidthAndMaxSampleNum(int w, int n)
{
	m_sample.SetDimension(n, w);
//	m_nSampleNum = 0;
	return 0;
}
///////////////////
int SkConProfile::clear()
{
//	assert(m_nWd > 0);
//	memset(m_pdDataSum, 0, sizeof(double)*m_nWd);
//	memset(m_pdData, 0, sizeof(double)*m_nWd);
//	memset(m_pnSampleNum, 0, sizeof(int)*m_nWd);
//	m_nSampleNum = 0;
	m_nValidSmpStart = -1;
	m_nValidSmpEnd = -1;

	return 0;
}
///////////////////
int SkConProfile::addSample(Matrix<double> &mat)
{
	int wd, ht;
	mat.GetDimension(ht, wd);

	int wd0, ht0;
	m_sample.GetDimension(ht0, wd0);

	if (wd0 != wd)
	{
		m_sample = mat;
//		m_nSampleNum = ht;
	}
	else
	{
//		int nSampleNum = m_nSampleNum + ht;
		int nSampleNum = ht0 + ht;

		Matrix<double> tmp;
		tmp.SetDimension(nSampleNum, wd);
		int r, c;
		for (r = 0; r < ht0; r++)
		{
			for (c = 0; c < wd; c++)
			{
				tmp(r,c) = m_sample(r,c);
			}
		}
		for (r = ht0; r < nSampleNum; r++)
		{
			for (c = 0; c < wd; c++)
			{
				tmp(r,c) = mat(r-ht0,c);
			}
		}
//			m_nSampleNum = nSampleNum;
		m_sample = tmp;
	}

	m_nValidSmpStart = 0;
	m_nValidSmpEnd = ht+ht0-1;

	m_smpCenterOff.SetDimension(ht+ht0, 1);
	m_smpCenterOff.Set(0);

	return ht+ht0;
}
///////////////////
int SkConProfile::buildEdgeRidgeModel()
{
	Matrix<double> m = 	getSampleMean();

	m_erModel.build(m);

	return 0;
}
void SkConProfile::setSampleCenterAsValidStartOrEnd(int e)
{	
	if (e == 0) 
		m_nValidSmpEnd = m_sample.rows()/2;
	else if (e == 1) 
		m_nValidSmpStart = m_sample.rows()/2; 
}
int SkConProfile::addOneJunction(SkJunction *p)
{ 
	assert(m_nCoverJunctNum<=SK_MAX_COVER_JNCT_NUM);
	m_pJunct[m_nCoverJunctNum] = p; 
	m_nCoverJunctNum++; 
	return m_nCoverJunctNum;
}
int SkConProfile::removeOneJunction(SkJunction *p)
{ 
	int i=0;
	for ( i=0; i<m_nCoverJunctNum;i++)
		if (m_pJunct[i] == p) 
			break; 
	for(int j=i;j<m_nCoverJunctNum-1;j++) 
		m_pJunct[j]=m_pJunct[j+1]; 
	if (i < m_nCoverJunctNum)
		m_nCoverJunctNum--; 
	return i;
}

//////////////////
Matrix<double> SkConProfile::getResidualImage(Matrix<double> *pMu) 
{
	int wd, ht;
	m_sample.GetDimension(ht, wd);
	Matrix<double> res(ht, wd);

	Matrix<double> mu;
	if (pMu == 0)
		mu= getSampleMean();
	else
		mu = *pMu;

	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			res(r,c) = m_sample(r,c) - mu(c,0);
		}
	}
	return res;
}
//////////////////
Matrix<double> SkConProfile::getPartResidualImage(Matrix<double> *pMu, int nPrimStart, int nPrimEnd, int nSmpStart, int nSmpEnd) 
{
	int wd, ht;
	m_sample.GetDimension(ht, wd);
	Matrix<double> res(nSmpEnd-nSmpStart+1, nPrimEnd-nPrimStart+1);

	Matrix<double> mu;
	if (pMu == 0)
		mu= getSampleMean();
	else
		mu = *pMu;

	int r, c;
	for (r = nSmpStart; r <= nSmpEnd; r++)
	{//each sample
		for (c = nPrimStart; c <= nPrimEnd; c++)
		{
			res(r-nSmpStart,c-nPrimStart) = m_sample(r,c) - mu(c,0);
		}
	}
	return res;
}
//////////////////
Matrix<double> SkConProfile::getPartSample(int nPrimStart, int nPrimEnd, int nSmpStart, int nSmpEnd) 
{
	int wd, ht;
	m_sample.GetDimension(ht, wd);
	Matrix<double> mat(nSmpEnd-nSmpStart+1, nPrimEnd-nPrimStart+1);

	int r, c;
	for (r = nSmpStart; r <= nSmpEnd; r++)
	{//each sample
		for (c = nPrimStart; c <= nPrimEnd; c++)
		{
			mat(r,c-nPrimStart) = m_sample(r,c);
		}
	}
	return mat;
}

/*
//////////////////
int SkConProfile::findCenter4EachSample(Matrix<double> &mu)//must after build EdgeRidge model
{
	double v[SK_MAX_PRFL_ERMODEL];
	int w, h;
	m_sample.GetDimension(h, w);

//	h = m_nSampleNum;

	int i, j;

	m_smpCenterOff.SetDimension(h, 1);
//	m_smpError.SetDimension(h, 1);

	for (i = 0; i < w; i++)
	{
		v[i] = mu(i,0);
	}

//	double dCenter0 = m_erModel.getCenterPosition();
	double dCenter0 = w/2;

	int nAccuracy = 10;

	double av[SK_MAX_PRFL_ERMODEL*10];

//	int s = m_erModel.m_dStart;
//	int e = m_erModel.m_dEnd;
	int s = 0;
	int e = w-1;

	for (i = s; i < e; i++)
	{
		for (j = 0; j < nAccuracy; j++)
		{//linear interpolation
			av[(i-s)*nAccuracy+j] = (v[i]*(nAccuracy - j) + v[i+1]*j) / nAccuracy;
		}
	}

	int wa = (e-s) * nAccuracy;

	double dError, dMinError, min_di;
	int nValidPt;
	int di;
	int dx, x;
	int sp;
	for (sp = 0; sp < h; sp++)
	{//for each sample
		dMinError = 1000000;
		for (di = -wa/2; di < wa/2; di++)
		{
			dError = 0;
			nValidPt = 0;
			dx = -s*nAccuracy - di;//?
			for (i = 0; i < w; i++)
			{
				x = dx + i*nAccuracy;
				if (x >= 0 && x < wa)
				{
					nValidPt++;
					dError += (av[x] - m_sample(sp,i))*(av[x] - m_sample(sp, i));
				}
			}
			if (nValidPt == 0)
				dError = 1000000;
			else 
				dError /= nValidPt;
			if (dMinError > dError)
			{
				dMinError = dError;
				min_di = di;
			}
		}
		double dOff = (double)min_di/nAccuracy;
		double dCenter = dCenter0 + dOff;
		if (dOff < -w/2) 
			dCenter = -w/2;
		if (dCenter > w/2) 
			dCenter = w/2;
		m_smpCenterOff(sp, 0) = dOff;
//		m_smpError(sp, 0) = dMinError;// update samples
	}

	// re-compute the Gaussian Model


	char name[512];

	// save the centers for debugging
	double dMax = m_sample.GetMax();
	Matrix<double> tmp = m_sample;
	for (int r = 0; r < h; r++)
	{
		int c = m_smpCenterOff(r, 0) + dCenter0;
		if (c >= 0 && c < w)
			tmp(r, c) = dMax*2;
	}
	tmp = tmp.T();

	sprintf(name, "center.bmp");
	::skSaveMatrixAsBitmap(name, tmp, 0, -1);


//	buildEdgeRidgeModel();//rebuild again
	return 0;

}
*/
//////////////////
int SkConProfile::findCenter4EachSample(Matrix<double> &mu)//must after build EdgeRidge model
{
	double v[SK_MAX_PRFL_ERMODEL];
	int w, h;
	m_sample.GetDimension(h, w);

	// w not always odd
	int i, j;

//	m_smpError.SetDimension(h, 1);

	for (i = 0; i < w; i++)
	{
		v[i] = mu(i,0);
	}

	double dCenter0 = w/2;
	int nAccuracy = 10;
	double av[SK_MAX_PRFL_ERMODEL*10];

	int pnOff[1024];
	assert(h <= 1024);

	for (i = 0; i < w-1; i++)
	{
		for (j = 0; j < nAccuracy; j++)
		{//linear interpolation
			av[i*nAccuracy+j] = (v[i]*(nAccuracy - j) + v[i+1]*j) / nAccuracy;
		}
	}

	int wa = (w-1) * nAccuracy;

	double dError, dMinError, min_di;
	int nValidPt;
	int di;
	int dx, x;
	int sp;
	int nPixelDiff = 1;
	for (int ii = 0; ii < h; ii++)
	{//for each sample
		int st, en;
		if (ii == 0)
		{//center one
			sp = m_nRotCenterSampleNo;//h/2;
//			st = -wa/2;
//			en = wa/2-1;
			st = 0;
			en = 0;//the center one is on the center
		}
		else if (ii <= m_nRotCenterSampleNo)//first half
		{
//			sp = h/2 - ii;
			sp = m_nRotCenterSampleNo - ii;
			st = pnOff[sp+1] - nPixelDiff*nAccuracy;//max diff two pixels
			en = pnOff[sp+1] + nPixelDiff*nAccuracy;
		}
		else
		{
			sp = ii;
			st = pnOff[sp-1] - nPixelDiff*nAccuracy;//max diff two pixels
			en = pnOff[sp-1] + nPixelDiff*nAccuracy;
		}
		dMinError = 1000000;
		for (di = st; di <= en; di++)
		{
			dError = 0;
			nValidPt = 0;
			dx = -di;//?
			for (i = 0; i < w; i++)
			{
				x = dx + i*nAccuracy;
				if (x >= 0 && x < wa)
				{
					nValidPt++;
					dError += (av[x] - m_sample(sp,i))*(av[x] - m_sample(sp, i));
				}
			}
			if (nValidPt == 0)
				dError = 1000000;
			else 
				dError /= nValidPt;
			if (dMinError > dError)
			{
				dMinError = dError;
				min_di = di;
			}
		}
		pnOff[sp] = min_di;
		double dOff = (double)min_di/nAccuracy;
		double dCenter = dCenter0 + dOff;
		if (dOff < -w/2) 
			dOff = -w/2;
		if (dOff > w/2) 
			dOff = w/2;
		m_smpCenterOff(sp, 0) += dOff;
//		m_smpError(sp, 0) = dMinError;// update samples
	}


	char name[512];

	// save the centers for debugging
	double dMax = m_sample.GetMax();
	Matrix<double> tmp = m_sample;
	for (int r = 0; r < h; r++)
	{
		int c = m_smpCenterOff(r, 0) + dCenter0;
		if (c >= 0 && c < w)
			tmp(r, c) = dMax*2;
	}
	tmp = tmp.T();

	sprintf(name, "center.bmp");
	::skSaveMatrixAsBitmap(name, tmp, 0, -1);


//	buildEdgeRidgeModel();//rebuild again
	return 0;

}

int SkConProfile::updateSampleAndMeanByCenterOffset()
{
	int w, h;
	m_sample.GetDimension(h, w);
	double pdTmpSample[SK_MAX_PRFL_ERMODEL];
	double pos, dOff;
	int sp, i;
	for (sp = 0; sp < h; sp++)
	{
		dOff = m_smpCenterOff(sp, 0);
		for (i = 0; i < w; i++)
		{
			pos = i + dOff;
			if (pos < 0)
				pdTmpSample[i] = m_sample(sp,0);
			else if (pos >= w-1)
				pdTmpSample[i] = m_sample(sp,w-1);
			else 
			{
				int p1 = (int)pos;
				int p2 = p1 + 1;
				pdTmpSample[i] = m_sample(sp,p1) *(p2-pos) + m_sample(sp,p2)*(pos-p1);
			}
		}
		for (i = 0; i < w; i++)
		{
			m_sample(sp, i) = pdTmpSample[i];
		}
	}
	computeMeanFromValidSamples();
	return 0;
}
/*
///////////////////
// computeGaussianModelFromSamples()
//	assume the m_sample is already created
//	1) compute m_gaussPDF, including mean and variance
int SkConProfile::computeGaussianModelFromSamples()
{
	int w = m_sample.cols();
	assert(w > 0 && w < MAX_CON_PROFILE_WD);
	int pnSampleNum[MAX_CON_PROFILE_WD];
	memset(pnSampleNum, 0, sizeof(int)*w);
	//compute mu and sigma
	
	//mu
	m_gaussPDF.mu.SetDimension(w, 1);//column verctor
	m_gaussPDF.mu.Zero();
	double data;
	int i, j;
	for (i = 0; i < w; i++)
	{
		for (j = 0; j < m_nSampleNum; j++)
		{
			data = m_sample(j, i);
			if (data < SK_INFINITE/2)
			{
				m_gaussPDF.mu(i,0) += data;
				pnSampleNum[i]++;
			}
		}
	}
	for (i = 0; i < w; i++)
	{
		if (pnSampleNum[i] > 0)
			m_gaussPDF.mu(i,0) /= pnSampleNum[i];
		else
			m_gaussPDF.mu(i,0) = SK_INFINITE;
	}
	//sigma
	m_gaussPDF.sigma.SetDimension(w, w);
	m_gaussPDF.sigma.Zero();

	Matrix<int> num(w,w);
	num.Zero();
	for (int n = 0; n < m_nSampleNum; n++)
	{
		for (i = 0; i < w; i++)
		{
			if (m_sample(n, i) > SK_INFINITE/2)
				continue;
			for (j = 0; j < w; j++)
			{
				if (m_sample(n, j) > SK_INFINITE/2)
					continue;
				m_gaussPDF.sigma(i,j) += (m_sample(n, i)-m_gaussPDF.mu(i, 0))*(m_sample(n, j)-m_gaussPDF.mu(j, 0));
				num(i,j)++;
			}
		}
	}
	for (i = 0; i < w; i++)
	{
		for (j = 0; j < w; j++)
		{
			if (num(i,j) != 0)
				m_gaussPDF.sigma(i,j) /= num(i,j);
			else
				m_gaussPDF.sigma(i,j) = 1.0/SK_INFINITE;

		}
	}

//	normalize();

	m_nValidWidth = w;
	return w;
}
*/
///////////////////
// computeMeanFromValidSamples()
//	assume the m_sample is already created
int SkConProfile::computeMeanFromValidSamples()
{
	int w, h;
	m_sample.GetDimension(h, w);
	assert(w > 0 && w < MAX_CON_PROFILE_WD);
	int pnSampleNum[MAX_CON_PROFILE_WD];
	memset(pnSampleNum, 0, sizeof(int)*w);
	//compute mu and sigma
	
	assert(m_nValidSmpStart >= 0 && m_nValidSmpStart < h
		&& m_nValidSmpEnd >= 0 && m_nValidSmpEnd < h);
	//mu
	m_mu.SetDimension(w, 1);//column verctor
	m_mu.Zero();
	double data;
	int i, j;
	for (i = 0; i < w; i++)
	{
		for (j = m_nValidSmpStart; j <= m_nValidSmpEnd; j++)
		{
			data = m_sample(j, i);
			if (data < SK_INFINITE/2)
			{
				m_mu(i,0) += data;
				pnSampleNum[i]++;
			}
		}
	}
	for (i = 0; i < w; i++)
	{
		if (pnSampleNum[i] > 0)
			m_mu(i,0) /= pnSampleNum[i];
		else
			m_mu(i,0) = SK_INFINITE;
	}
	return w;
}




///////////////////
// addProfile()
//	add a profile which is with the width of 'm' into the m_sample 	
//	more space will be allocated if needed
int SkConProfile::addProfile(int m, double *pdPrfl)
{
	int w = m_sample.cols();
	int h = m_sample.rows();
	double pData0[MAX_CON_PROFILE_WD], *pData;
	assert(w <= MAX_CON_PROFILE_WD);

	double pos;
	double data;
	int i, j, k;
	if (m != w)
	{
		for (i = 0; i < w; i++)
		{
			pos = (i-w/2)*m/w + m/2;
			if (pos < 0)
				data = pdPrfl[0];
			else if (pos >= m-1)
				data = pdPrfl[m-1];
			else 
			{
				int p1 = (int)pos;
				int p2 = p1 + 1;
				data = pdPrfl[p1] *(p2-pos) + pdPrfl[p2]*(pos-p1);
			}
			pData0[i] = data;
		}
		pData = pData0;
	}
	else
	{
		pData = pdPrfl;
	}
//	k = m_nSampleNum;
	k = h;
	assert(false);
	if (k == h)
	{
		Matrix<double> temp;
		temp = m_sample;
		m_sample.SetDimension(h+h/2, w);
		for (j = 0; j < h; j++)
		{
			for (i = 0; i < w; i++)
			{
				m_sample(j, i) = temp(j,i);
			}
		}

	}
	for (i = 0; i < w; i++)
	{
		m_sample(k, i) = pData[i];
	}
//	m_nSampleNum++;
	return h;//m_nSampleNum;
}
///////////////////
int  SkConProfile::getAbsoluteSampleCenter(double *pdX, double *pdY)
{
	int w, h;
	m_sample.GetDimension(h, w);

	double dx, dy;
	double dOrt = m_dOrt;
	int i;
	for (i = 0; i < h; i++)
	{// for each sample
		dx = i-m_nRotCenterSampleNo;
		dy = m_smpCenterOff(i,0);
		::rotate(dx, dy, dOrt);
		pdX[i] = m_x0 + dx;
		pdY[i] = m_y0 + dy;
	}
	return h;
}
void SkConProfile::getTwoValidEndPoints(double &x1, double &y1, double &x2, double &y2)
{
	getValidEndPoints(x1, y1, 0);
	getValidEndPoints(x2, y2, 1);
}
void SkConProfile::getValidEndPoints(double &x, double &y, int e)
{
	double dx, dy;
	double dOrt = m_dOrt;
	int i;

	if (e == 0)
		i = m_nValidSmpStart;
	else
		i = m_nValidSmpEnd;

	dx = i-m_nRotCenterSampleNo;
	dy = m_smpCenterOff(i,0);
	::rotate(dx, dy, dOrt);
	x = m_x0 + dx;
	y = m_y0 + dy;
}

int SkConProfile::getAbsoluteValidSamplePixels(double *pdX, double *pdY, int start, int end)
{
	int w, h;
	m_sample.GetDimension(h, w);

	double dx, dy;
	double dOrt = m_dOrt;
	int i, j;
	int n = 0;
	for (i = m_nValidSmpStart; i <= m_nValidSmpEnd; i++)
	{// for each sample
		for (j =  start; j <= end; j++)
		{
			dx = i-m_nRotCenterSampleNo;
			dy = j-w/2 + m_smpCenterOff(i,0);
			::rotate(dx, dy, dOrt);
			pdX[n] = m_x0 + dx;
			pdY[n] = m_y0 + dy;
			n++;
		}
	}
	return n;
}





///////////////////
// getProfile
//	save the file into *pdPrfl with the with 'd', 
//	interpolated if 'm_nValidWidth!=m'
int SkConProfile::getProfile(int m, double *pdPrfl)
{
	double pos;
	double data;
	int i;
	int w = getValidWidth();
	int ws = getSampleWidth();
	int off = (ws-w)/2;
	if (m != w)
	{
		for (i = 0; i < m; i++)
		{
			pos = (i-m/2)*w/m + w/2;
			if (pos < 0)
//				data = m_gaussPDF.mu(0,0);
				data = m_mu(0,0);
			else if (pos >= w-1)
//				data = m_gaussPDF.mu(w-1+off,0);
				data = m_mu(w-1+off,0);
			else 
			{
				int p1 = (int)pos;
				int p2 = p1 + 1;
//				data = m_gaussPDF.mu(p1+off,0) *(p2-pos) + m_gaussPDF.mu(p2+off,0)*(pos-p1);
				data = m_mu(p1+off,0) *(p2-pos) + m_mu(p2+off,0)*(pos-p1);
			}
			if (data < SK_INFINITE/2)
				pdPrfl[i] = data;
			else 
//				pdPrfl[i] = 0;
				pdPrfl[i] = SK_INFINITE;
		}
	}
	else
	{
		for (i = 0; i < m; i++)
		{
//			pdPrfl[i] = m_gaussPDF.mu(i+off,0);
			pdPrfl[i] = m_mu(i+off,0);
		}
	}

	return 0;
}
///////////////////
// addLineProfile()
//	add a line (x0,y0)->(x1,y0) into the m_sample
int SkConProfile::addLineProfile(int x0, int y0, int x1, int y1, Matrix<float> *pImgOrg)
{

	x0 *= m_nScale;
	x1 *= m_nScale;
	y0 *= m_nScale;
	y1 *= m_nScale;

	int w = m_sample.cols();

	int ht = pImgOrg->rows();
	int wd = pImgOrg->cols();

	int pnX_v[MAX_CON_PROFILE_WD], pnY_v[MAX_CON_PROFILE_WD];
	double pdPrfl[MAX_CON_PROFILE_WD];

	int pnX[4096];
	int pnY[4096];//axis point of the profile

	int pf_x0, pf_x1, pf_y0, pf_y1;
	int x, y, dx, dy, xc, yc;
	int n;
	n = ::rasterize_line_segment(x0, y0, x1, y1, pnX, pnY);
	
//	if(_heapchk()!=_HEAPOK)
//		DebugBreak();
	assert(n <= 4096);

	::verticalLineApproximately(x0, y0, x1, y1, pf_x0, pf_y0, pf_x1, pf_y1, w);
	int m = ::rasterize_line_segment_keeping_order(pf_x0, pf_y0, pf_x1, pf_y1, pnX_v, pnY_v);
	assert(m <= MAX_CON_PROFILE_WD);

	for (int i = 0; i < n; i++)
	{//there is an overlap on the join point
		xc = pnX[i];
		yc = pnY[i];//all the axis point

		for (int j = 0; j < m; j++)
		{
			dx = pnX_v[j];
			dy = pnY_v[j];//verticle piont of the 
			x = xc + dx;
			y = yc + dy;
			if (x < 0 || x >= wd || y < 0 || y >= ht)
				pdPrfl[j] = SK_INFINITE;
			else
				pdPrfl[j] = (*pImgOrg)(y,x);
		}
		addProfile(m, pdPrfl);
	}
	return 0;//m_nSampleNum;
}
///////////////////
//	addDoGProfile()
//	add a difference of Gaussian profile,
//	it is a circlar shape, not a line
int SkConProfile::addDoGProfile(int x0, int y0, int x1, int y1, Matrix<float> *pImgOrg)
{
	int w = m_sample.cols();

	int ht = pImgOrg->rows();
	int wd = pImgOrg->cols();

	double pdPrfl[MAX_CON_PROFILE_WD];
	int pnCount[MAX_CON_PROFILE_WD];

	memset(pdPrfl, 0, sizeof(double)*w);
	memset(pnCount, 0, sizeof(int)*w);

	int xc = (x0+x1)/2;
	int yc = (y0+y1)/2;
	int x, y;
	int d;
	for (int dy = -(w-1); dy <= (w-1); dy++)
	{
		y = yc + dy;
		if (y < 0 || y >= ht)
			continue;
		for (int dx = -(w-1); dx <= (w-1); dx++)
		{
			x = xc + dx;
			if (x < 0 || x >= wd)
				continue;
			d = (int)(sqrt(double(dx*dx + dy*dy)));
			if (d >= w)
				continue;
			pdPrfl[d] += (*pImgOrg)(y,x);
			pnCount[d]++;
		}
	}

	for (d = 0; d < w; d++)
	{
		if (pnCount[d] != 0)
			pdPrfl[d] /= pnCount[d];
	}

	addProfile(w, pdPrfl);
	return 1;
}
///////////////////
// addLineProfile()
//	add a line (x0,y0)->(x1,y0) into the m_sample
int SkConProfile::addLineProfile(int x0, int y0, int x1, int y1, Matrix<float> *pImgOrg, Matrix<SkPixCandArray> &M)
{

	x0 *= m_nScale;
	x1 *= m_nScale;
	y0 *= m_nScale;
	y1 *= m_nScale;

	int w = m_sample.cols();

	int ht = pImgOrg->rows();
	int wd = pImgOrg->cols();

	int pnX_v[MAX_CON_PROFILE_WD], pnY_v[MAX_CON_PROFILE_WD];
	double pdPrfl[MAX_CON_PROFILE_WD];

	int pnX[4096];
	int pnY[4096];//axis point of the profile

	int pf_x0, pf_x1, pf_y0, pf_y1;
	int x, y, dx, dy, xc, yc;
	int n;
	n = ::rasterize_line_segment(x0, y0, x1, y1, pnX, pnY);
	
//	if(_heapchk()!=_HEAPOK)
//		DebugBreak();
	assert(n <= 4096);

	::verticalLineApproximately(x0, y0, x1, y1, pf_x0, pf_y0, pf_x1, pf_y1, w);
	int m = ::rasterize_line_segment_keeping_order(pf_x0, pf_y0, pf_x1, pf_y1, pnX_v, pnY_v);
	assert(m <= MAX_CON_PROFILE_WD);

	for (int i = 0; i < n; i++)
	{//there is an overlap on the join point
		xc = pnX[i];
		yc = pnY[i];//all the axis point

		for (int j = 0; j < m; j++)
		{
			dx = pnX_v[j];
			dy = pnY_v[j];//verticle piont of the 
			x = xc + dx;
			y = yc + dy;

			if (x < 0 || x >= wd || y < 0 || y >= ht)
			{
				pdPrfl[j] = SK_INFINITE;
				continue;
			}
			double dDis = fabs(j-m/2.f);
			double dValue;
			double dMinDis = skGetMinDistanceOfPixelCandidate(M, x, y, dValue);
			
			if (dDis > dMinDis)
				pdPrfl[j] = SK_INFINITE;
			else
				pdPrfl[j] = (*pImgOrg)(y,x);
		}
		addProfile(m, pdPrfl);
	}
	return 0;//m_nSampleNum;
}
///////////////////
//	addDoGProfile()
//	add a difference of Gaussian profile,
//	it is a circlar shape, not a line
int SkConProfile::addDoGProfile(int x0, int y0, int x1, int y1, Matrix<float> *pImgOrg,  Matrix<SkPixCandArray> &M)
{
	int w = m_sample.cols();

	int ht = pImgOrg->rows();
	int wd = pImgOrg->cols();

	double pdPrfl[MAX_CON_PROFILE_WD];
	int pnCount[MAX_CON_PROFILE_WD];

	memset(pdPrfl, 0, sizeof(double)*w);
	memset(pnCount, 0, sizeof(int)*w);

	int xc = (x0+x1)/2;
	int yc = (y0+y1)/2;
	int x, y;
	int d;
	for (int dy = -(w-1); dy <= (w-1); dy++)
	{
		y = yc + dy;
		if (y < 0 || y >= ht)
			continue;
		for (int dx = -(w-1); dx <= (w-1); dx++)
		{
			x = xc + dx;
			if (x < 0 || x >= wd)
				continue;
			d = (int)(sqrt(double(dx*dx + dy*dy)));
			if (d >= w)
				continue;

			double dDis = fabs(double(d));
			double dValue;
			double dMinDis = skGetMinDistanceOfPixelCandidate(M, x, y, dValue);
			if (dDis >= dMinDis)
				continue;

			pdPrfl[d] += (*pImgOrg)(y,x);
			pnCount[d]++;
		}
	}

	for (d = 0; d < w; d++)
	{
		if (pnCount[d] != 0)
			pdPrfl[d] /= pnCount[d];
		else
			pdPrfl[d] = SK_INFINITE;
	}

	addProfile(w, pdPrfl);
	return 1;
}

///////////////////
// saveLineProfile()
//	save the line profile into a matrix, the line with ends (x0,y0)->(x1,y1)
int SkConProfile::saveLineProfile(int x0, int y0, int x1, int y1, Matrix<float> *M,  Matrix<SkPixCandArray>* pPixCandMat, int dx_2M, int dy_2M)
{
	int w = getValidWidth();//m_sample.cols();
	int ht = M->rows();
	int wd = M->cols();

	int pnX_v[MAX_CON_PROFILE_WD], pnY_v[MAX_CON_PROFILE_WD];
	double pdPrfl[MAX_CON_PROFILE_WD];

	int pnX[4096];
	int pnY[4096];

	int pf_x0, pf_x1, pf_y0, pf_y1;
	int x, y, dx, dy, xc, yc;
	int n = ::rasterize_line_segment(x0, y0, x1, y1, pnX, pnY);
	assert(n <= 4096);
	::verticalLineApproximately(x0, y0, x1, y1, pf_x0, pf_y0, pf_x1, pf_y1, w);
	int m = ::rasterize_line_segment_keeping_order(pf_x0, pf_y0, pf_x1, pf_y1, pnX_v, pnY_v);

	double dMinDis, dMinValue, dDis;
	int ht0=0, wd0=0;
	if (pPixCandMat != 0)
		pPixCandMat->GetDimension(ht0, wd0);
	for (int i = 0; i < n; i++)
	{//there is an overlap on the join point
		xc = pnX[i];
		yc = pnY[i];

		getProfile(m, pdPrfl);
		for (int j = 0; j < m; j++)
		{
			dx = pnX_v[j];
			dy = pnY_v[j];
			x = xc + dx;
			y = yc + dy;
			if (x < 0 || x >= wd || y < 0 || y >= ht)
				continue;
			if (pdPrfl[j] > SK_INFINITE/2)
				(*M)(y,x) = 0;
			else
				(*M)(y,x) = (float)pdPrfl[j];
			if (pPixCandMat != 0)
			{	
				int xx = x + dx_2M;
				int yy = y + dy_2M;
				if (xx < 0 || xx >= wd0 || yy < 0 || yy >= ht0)
					continue;
				dDis = fabs(j-m/2.f);
				dMinDis = skGetMinDistanceOfPixelCandidate(*pPixCandMat, xx, yy, dMinValue);
				if (dDis > dMinDis ||
					(dDis == dMinDis && dMinValue != pdPrfl[j]) )
				{
					(*M)(y,x) = 0;//not modeled by this profile
				}
			}
		}
	}
	return 0;//m_nSampleNum;
}
///////////////////
// saveDoGProfile()
//	save the Dog profile into a matrix, the Dog with center of (x0,y0)->(x1,y1)
int SkConProfile::saveDoGProfile(int x0, int y0, int x1, int y1, Matrix<float> *M,  Matrix<SkPixCandArray>* pPixCandMat, int dx_2M, int dy_2M)
{
	int w = m_sample.cols();

	double pdPrfl[MAX_CON_PROFILE_WD];
	getProfile(w, pdPrfl);

	int ht = M->rows();
	int wd = M->cols();

	double dMinDis, dMinValue, dDis;
	int ht0=0, wd0=0;
	if (pPixCandMat != 0)
		pPixCandMat->GetDimension(ht0, wd0);

	int xc = (x0+x1)/2;
	int yc = (y0+y1)/2;
	int x, y;
	int d;
	for (int dy = -(w-1); dy <= (w-1); dy++)
	{
		y = yc + dy;
		if (y < 0 || y >= ht)
			continue;
		for (int dx = -(w-1); dx <= (w-1); dx++)
		{
			x = xc + dx;
			if (x < 0 || x >= wd)
				continue;
			d = (int)(sqrt(double(dx*dx + dy*dy)));
			if (d >= w)
				continue;
			if (pdPrfl[d] > SK_INFINITE/2)
				(*M)(y,x) = 0;
			else
				(*M)(y,x) = pdPrfl[d];
			if (pPixCandMat != 0)
			{	
				int xx = x + dx_2M;
				int yy = y + dy_2M;
				if (xx < 0 || xx >= wd0 || yy < 0 || yy >= ht0)
					continue;
				dDis = fabs(double(d));
				dMinDis = skGetMinDistanceOfPixelCandidate(*pPixCandMat, xx, yy, dMinValue);
				if (dDis >= dMinDis ||
					(dDis == dMinDis && dMinValue != pdPrfl[d]) )
				{
					(*M)(y,x) = 0;//not modeled by this profile
				}
			}
		}
	}
	return 0;//m_nSampleNum;
}

///////////////////
// saveLineProfile()
//	save the line profile into a candidate matrix, the line with ends (x0,y0)->(x1,y1)
int SkConProfile::saveLineProfile(int x0, int y0, int x1, int y1, Matrix<SkPixCandArray> &M)
{
	int w = getValidWidth();//m_sample.cols();
	int ht = M.rows();
	int wd = M.cols();

	int pnX_v[MAX_CON_PROFILE_WD], pnY_v[MAX_CON_PROFILE_WD];
	double pdPrfl[MAX_CON_PROFILE_WD];

	int pnX[4096];
	int pnY[4096];

	int pf_x0, pf_x1, pf_y0, pf_y1;
	int x, y, dx, dy, xc, yc;
	int n = ::rasterize_line_segment(x0, y0, x1, y1, pnX, pnY);
	assert(n <= 4096);
	::verticalLineApproximately(x0, y0, x1, y1, pf_x0, pf_y0, pf_x1, pf_y1, w);
	int m = ::rasterize_line_segment_keeping_order(pf_x0, pf_y0, pf_x1, pf_y1, pnX_v, pnY_v);

	SkPixCand pc;
	for (int i = 0; i < n; i++)
	{//there is an overlap on the join point
		xc = pnX[i];
		yc = pnY[i];

		getProfile(m, pdPrfl);
		for (int j = 0; j < m; j++)
		{
			dx = pnX_v[j];
			dy = pnY_v[j];
			x = xc + dx;
			y = yc + dy;
			if (x < 0 || x >= wd || y < 0 || y >= ht)
				continue;
			pc.m_dValue = pdPrfl[j];
			pc.m_dDis = fabs(j-m/2.f);
			M(y,x).add(pc);
		}
	}
	return 0;//m_nSampleNum;
}


///////////////////
int SkConProfile::saveDoGProfile(int x0, int y0, int x1, int y1, Matrix<SkPixCandArray> &M)
{
	int w = m_sample.cols();
	int ht = M.rows();
	int wd = M.cols();
	SkPixCand pc;
	double pdPrfl[MAX_CON_PROFILE_WD];
	getProfile(w, pdPrfl);

	int xc = (x0+x1)/2;
	int yc = (y0+y1)/2;
	int x, y;
	int d;
	for (int dy = -(w-1); dy <= (w-1); dy++)
	{
		y = yc + dy;
		if (y < 0 || y >= ht)
			continue;
		for (int dx = -(w-1); dx <= (w-1); dx++)
		{
			x = xc + dx;
			if (x < 0 || x >= wd)
				continue;
			d = (int)(sqrt(double(dx*dx + dy*dy)));
			if (d >= w)
				continue;
			pc.m_dValue = pdPrfl[d];
			pc.m_dDis = fabs(double(d));
			M(y,x).add(pc);
		}
	}
	return 0;//m_nSampleNum;
}


double SkConProfile::projection(Matrix<double> data)
{
	assert(data.cols() == 1);
	Matrix<double> M;
	M = m_normMu.T() * data;
	return M(0,0)/m_dStd;
}
int SkConProfile::normalize()
{
//	Matrix<double> &mu = m_gaussPDF.mu;
	Matrix<double> &mu = m_mu;
	m_normMu = mu;
	int w = getSampleWidth();
	assert(w > 0);
	m_dMean = 0; 
	m_dStd = 0;
	int valid = 0;
	int i;
	double data;
	for (i = 0; i < w; i++)
	{
		data = mu(i,0); 
		if (data < SK_INFINITE/2)
		{
			m_dMean += data;
			valid++;
		}
	}
	if (valid > 0)
		m_dMean /= valid;
	else 
		m_dMean = 0;
	for (i = 0; i < w; i++)
	{
		data = mu(i,0); 
		if (data < SK_INFINITE/2)
		{
			data -= m_dMean;
			m_dStd += data*data;
			m_normMu(i,0) = data;
		}
	}
	m_dStd = sqrt(m_dStd);

	for (i = 0; i < w; i++)
	{
		data = mu(i,0); 
		if (data < SK_INFINITE/2)
		{
			m_normMu(i,0) /= m_dStd;
		}
	}
	return 0;
}
///////////////////
// relativePosOfLastSample()
//	return the relative positions of last sample
int	SkConProfile::maxOffsetOfSamples(int &nWhich)
{
	int nOffset = 0;

	int w = m_sample.cols();//profile collected width
	int n = m_sample.rows();//sample num
	if (n < 10)//only consider greater or equal to 7 pixel length profile
		return 0;
	if (w <= 3)// only consider profile width larger than 2
		return 0;
	int i, j, k;
	int nFirstCompare = 3;
	assert(w <= 1024);
	double sum[1024];
	memset(sum, 0, sizeof(double)*w);
	for (i = 0; i < nFirstCompare; i++)
	{
		for (j = 0; j < w; j++)
		{
			sum[j] += m_sample(i,j);
		}
	}
	for (j = 0; j < w; j++)
		sum[j] /= nFirstCompare;

	assert(n <= 1024);
	int offset[1024];
	for (i = 0; i < nFirstCompare; i++)
		offset[i] = 0;

	double pdDis[3], dMaxDis;
	int nMax;
	for (i = nFirstCompare; i < n; i++)
	{
		for (k = -1; k <= 1; k++)
		{
			pdDis[k+1] = 0;
			for (j = 1; j < w-1; j++)
			{
				pdDis[k+1] += sum[j] * m_sample(i,j+k);
			}
		}
		dMaxDis = pdDis[0];
		nMax = -1;
		for (k = 0; k < 1; k++)
		{
			if (pdDis[k+1] > dMaxDis)
			{
				dMaxDis = pdDis[k+1];
				nMax = k;
			}
		}
		offset[i] = nMax;
	}

	double dOff1 = 0, dOff2 = 0;
	for (i = n/2-1; i<= n/2+1; i++)
		dOff1 += offset[i];
	for (i = n-3; i<= n-1; i++)
		dOff2 += offset[i];
	dOff1 /= 3;
	dOff2 /= 3;

	double dTh = 0.6;
	if (fabs(dOff1) >=dTh)
	{
		if(fabs(dOff2) >=dTh)
		{
			if (dOff1*dOff2 > 0)
			{
				nWhich = 2;
				if (dOff1 > 0)
					nOffset = 1;
				else
					nOffset = -1;
			}
			else
			{
				if (fabs(dOff1) > fabs(dOff2))
				{
					nWhich = 0;
					if (dOff1 > 0)
						nOffset = 1;
					else
						nOffset = -1;
				}
				else
				{
					nWhich = 1;
					if (dOff2 > 0)
						nOffset = 1;
					else
						nOffset = -1;
				}
			}
		}
		else
		{
			nWhich = 0;
			if (dOff1 > 0)
				nOffset = 1;
			else
				nOffset = -1;
		}
	}
	else
	{
		if(fabs(dOff2) >=0.3)
		{
			nWhich = 1;
			if (dOff2 > 0)
				nOffset = 1;
			else
				nOffset = -1;
		}
		else
		{
			nWhich = 0;
			nOffset = 0;
		}
	}
	return nOffset;
}
///////////////////
// thinnerProfile()
//	assume the computeGaussianModelFromSamples() already executed
//	1) Consider the mean and variance of each pixel of the profile, 
//	2) Profile condition: profile should satisfy coding length of Gaussian l^g <= l^u (coding length of uniform)
//		which equals sigma <= M/(e^0.5*sqrt(2*pi))
//	3) From center to boundary, check profile condition, and decide profile
//	4) Thinner condition: l^g+l^u >= l^t, l^t is the coding length with thinner gaussian model
//	5) From boundary to center to, thinner the profile
//	6) call computeGaussianModelFromSamples()
//	7) return the center change, if too big variance profile, return 'w', sample width
int SkConProfile::thinnerProfile()
{
/*
	int w = getSampleWidth();//width of the profile
	int n = getSampleNum();
	int not_prof = 0;

	double M = 6.0;//uniform range of the pixel gray level (after normalization)
	double std_th = pow(M, (n-2.0)/n)/(exp(1)*sqrt(2*PI));

	double mu[512], std[512];
	assert(w <= 512);
	int i, j;

	for (i = 0; i < w; i++)
	{
		mu[i] = m_gaussPDF.mu(i,0);
		std[i] = sqrt(m_gaussPDF.sigma(i,i));
	}

	::saveArray(mu, w, "mu.txt");
	::saveArray(std, w, "std.txt");
	SaveResults::saveProfileAsBmp(*this, "profile.bmp", 123.11, 26.35);

	int s, t;
	//consider variance, profile condition
	for (i = 0; i < w; i++)
	{
		if (std[i] <= std_th)//profile should have small variance
			break;
	}
	s = i;
	if (s >= w)
		return not_prof;
	for (i = w-1; i >= 0; i--)
	{
		if (std[i] <= std_th)//profile should have small variance
			break;
	}
	t = i;
	if (t < 0)
		return not_prof;

	assert(t >= s);

	//cosider thinner condition
	double delta;
	int K = t-s+1;//# of pixels in profile
	double l_t, l_g, l_cb, l_pc;

	for (j = s+1; j <= t; j++)
	{//the pixel, which the others are combined to
		delta = 0;
		for (i = s; i < j; i++)
		{
			l_t = gaussianCodingLength(mu[i], std[i], mu[j], std[j]);//thinner gaussian
			l_g = gaussianCodingLength(mu[i], std[i]);//gaussian
			delta += ( l_t - l_g)*n;
		}
		l_cb = -(j-s)*2*uniformCodingLength(M);//code book change
		l_pc = K * n * log((K-(j-s))/(double)K);//all the pixel codes changes due to smaller code book
		delta +=  l_cb + l_pc ;
		if (delta > 0)
			break;
	}
	s = j-1;
//	if (s >= w)
//		return w;
	K=t-s+1;
	for (j = t-1; j >= s; j--)
	{//the pixel, which the others are combined to
		delta = 0;
		for (i = t; i > j; i--)
		{
			l_t = gaussianCodingLength(mu[i], std[i], mu[j], std[j]);//thinner gaussian
			l_g = gaussianCodingLength(mu[i], std[i]);//gaussian
			delta += ( l_t - l_g)*n;
		}
		l_cb = - (t-j)*2*uniformCodingLength(M);
		l_pc = K * n * log((K-(t-j))/(double)K);
		delta +=  l_cb + l_pc;
		if (delta > 0)
			break;
	}
	t = j+1;

	int c = (s+t)/2;//center
	int h = (t-s+1)/2;//half
	int ww = 2*h + 1;
	s = c - h;//keep odd width
	t = c + h;//
	if (s < 0)
	{
		s++;
		c++;
		t++;
		assert( s >= 0 && t < w);
	}
	if (t == w)
	{
		t--;
		c--;
		s--;
		assert(s >= 0 && t < w);
	}
	int d = c - w/2;//center displacement


	// reset samples
	Matrix<double> sample;
	sample = m_sample;
	m_sample.SetDimension(m_nSampleNum, ww);
	for (i = 0; i < m_nSampleNum; i++)
	{
		for (j = 0; j < ww; j++)
		{
			m_sample(i, j) = sample(i, j+s);
		}
	}
	computeGaussianModelFromSamples();
	return d;//center displacement
*/
	int w = m_normMu.rows();
	// no operation
	m_nValidWidth = w;
//	return w;

	assert(w <= 512);
	double pdVar[512];
	double pdMean[512];
	int i, j;
	for (i = 0; i < w/2; i++)
	{
		pdMean[i] = 0;
		for (j = i; j < w-i; j++)
		{
			pdMean[i] += m_normMu(j,0);
		}
		pdMean[i] /= (w-2*i);
		pdVar[i] = 0;
		for (j = i; j < w-i; j++)
		{
			pdVar[i] += (m_normMu(j,0)-pdMean[i])*(m_normMu(j,0)-pdMean[i]);
		}
		pdVar[i] /= (w-2*i);
	}

	double dMaxVar = -1;
	int nMax = -1;
	for (i = 0; i < w/2; i++)
	{
		if (dMaxVar < pdVar[i])
		{
			dMaxVar = pdVar[i];
			nMax = i;
		}
	}
	assert(nMax >= 0);
	m_nValidWidth = (w/2-nMax)*2 + 1;
	return 0;
	
}

/////////////////////////////////////////////////////////////////////
// class SkContourNeighbor

SkContourNeighbor::SkContourNeighbor()
{
	m_dContStrength = MIN_CNT_STRENGTH;
	m_nEnd = 0;// flag of the strength is to the head of m_pContour or the tail of m_pContour, 0-head
	m_pContour = 0;
}

///////////////////
double SkContourNeighbor::computeContStrength()
{
	m_dContStrength = m_dDeltaOrt*18/PI + m_dDeltaAngle*18/PI + m_dDis/2;//continuity strength
	return m_dContStrength;
}

/////////////////////////////////////////////////////////////////////
// class SkConNbrManager

///////////////////
// AddNeighbor()
// 
int SkConNbrManager::AddNeighbor( int end, SkContourNeighbor& conNbr )
{
	conNbr.computeContStrength();
	int n = GetNeighborNum(end);
	if (n == 0)
	{
		m_ppEndNbrArray[end][0] = conNbr;
		m_pnEndNbrNum[end] = 1;
		return 1;
	}
	int i=0;
	for ( i = 0; i < n; i++)
	{
		if (conNbr.m_dContStrength < m_ppEndNbrArray[end][i].m_dContStrength)
			break;
	}
	if (i >= MAX_CON_NBR_NUM)
		return n;

	int m = std::min<int>(n+1, MAX_CON_NBR_NUM);
	for (int j = m-1; j > i; j--)
	{
		m_ppEndNbrArray[end][j] = m_ppEndNbrArray[end][j-1];
	}
	m_ppEndNbrArray[end][i] = conNbr;
	m_pnEndNbrNum[end] = m;
	return m_pnEndNbrNum[end];
}
void SkConNbrManager::ClearNeighbor( int end )
{
	int n = GetNeighborNum(end);
	for (int i = end; i < n-1; i++)
		m_ppEndNbrArray[end][i] = m_ppEndNbrArray[end][i+1];
	m_pnEndNbrNum[end]--;
}
int SkConNbrManager::GetNeighborNum( int end )
{
	return m_pnEndNbrNum[end];
}
SkContourNeighbor SkConNbrManager::GetNeighbor( int end, int i )
{
	return m_ppEndNbrArray[end][i];
}
void SkConNbrManager::RemoveAllNeighbors()
{
	m_pnEndNbrNum[0] = m_pnEndNbrNum[1] = 0;
}
/*
int SkConNbrManager::SortNeighbors( int end )
{
	int nNeighborNum = m_pnEndNbrNum[end];
	SkContourNeighbor *pNbrArray = m_ppEndNbrArray[end];
	if( nNeighborNum <= 0 )
		return nNeighborNum;
	double dSumStrength = 0;
	for( int i = 0; i < nNeighborNum; i++ )
		dSumStrength += pNbrArray[i].m_dContStrength;
	if( dSumStrength != 0 )
	{
		for( i = 0; i < nNeighborNum; i++ )
		{
//				if( dSumStrength < 0 )
//					pNbrArray[i].m_dContStrength = -(float)(pNbrArray[i].m_dContStrength / dSumStrength);
//				else
				pNbrArray[i].m_dContStrength = (float)(pNbrArray[i].m_dContStrength / dSumStrength);
		}
	}
	float fStrength = pNbrArray[0].m_dContStrength;
	for( i = 0; i < nNeighborNum - 1; i++ )
	{
		for( int j = i + 1; j < nNeighborNum; j++ )
		{
			if( pNbrArray[i].m_dContStrength < pNbrArray[j].m_dContStrength ) 
			{
				SkContourNeighbor temp;
				temp = pNbrArray[i];
				pNbrArray[i] = pNbrArray[j];
				pNbrArray[j] = temp;
			}
		}
	}
	return nNeighborNum;
}
bool SkConNbrManager::NeighborJudge( int &end, SkContourNeighbor &cn )
{
	int nNeighborNum; 
	SkContourNeighbor *pNbrArray;
	for (int e = 0; e < 2; e++)
	{
		nNeighborNum = m_pnEndNbrNum[e];
		pNbrArray = m_ppEndNbrArray[e];
		for( int i = 0; i < nNeighborNum; i++ )
		{
			if( pNbrArray[i].m_pContour == cn.m_pContour )
			{
				end = e;
				cn.m_nEnd = pNbrArray[i].m_nEnd;
				return true;
			}
		}
	}
	return false;
}//bool NeighborJudge( int &end, SkContourNeighbor &cn )
*/
/////////////////////////////////////////////////////////////////////
// class SkContour
SkContour::SkContour()
{
	m_pNext = 0;
	m_pPrev = 0; 
	m_PointList.RemoveAll();// the list of the points in this contour
	m_contourType =  SKCURVE;//contour type: point, line, arc, curve
	m_pnEndProperty[0] = m_pnEndProperty[1] = -1;//[0]head,[1]tail, -1-free, 0-reach itself, 1-reach other contour, 2-reach bounary
}
SkContour::SkContour( SkPoint p1, SkPoint p2 )
{	
	m_pNext = 0;
	m_pPrev = 0; 
	m_PointList.RemoveAll();// the list of the points in this contour
	m_PointList.AddHead(p1);
	m_PointList.AddTail(p2);
	m_contourType =  SKLINE;//contour type: point, line, arc, curve

	m_pnEndProperty[0] = END_FREE;
	m_pnEndProperty[1] = END_FREE;
}
void SkContour::AddNeighbor( int end, SkContourNeighbor& conNbr )
{
	m_conNbrManager.AddNeighbor(end, conNbr);
}
void SkContour::ClearNeighbor( int end )
{
	m_conNbrManager.ClearNeighbor(end);
}
int SkContour::GetNeighborNum( int end )
{
	return m_conNbrManager.GetNeighborNum(end);
}
SkContourNeighbor SkContour::GetNeighbor( int end, int i )
{
	return m_conNbrManager.GetNeighbor(end, i);
}
void SkContour::RemoveAllNeighbors()
{
	m_conNbrManager.RemoveAllNeighbors();
}
/*
int SkContour::SortNeighbors( int end )
{
	int nNeighborNum = m_pnEndNbrNum[end];
	SkContourNeighbor *pNbrArray = m_ppEndNbrArray[end];
	if( nNeighborNum <= 0 )
		return nNeighborNum;
	double dSumStrength = 0;
	for( int i = 0; i < nNeighborNum; i++ )
		dSumStrength += pNbrArray[i].m_dContStrength;
	if( dSumStrength != 0 )
	{
		for( i = 0; i < nNeighborNum; i++ )
		{
//				if( dSumStrength < 0 )
//					pNbrArray[i].m_dContStrength = -(float)(pNbrArray[i].m_dContStrength / dSumStrength);
//				else
				pNbrArray[i].m_dContStrength = (float)(pNbrArray[i].m_dContStrength / dSumStrength);
		}
	}
	float fStrength = pNbrArray[0].m_dContStrength;
	for( i = 0; i < nNeighborNum - 1; i++ )
	{
		for( int j = i + 1; j < nNeighborNum; j++ )
		{
			if( pNbrArray[i].m_dContStrength < pNbrArray[j].m_dContStrength ) 
			{
				SkContourNeighbor temp;
				temp = pNbrArray[i];
				pNbrArray[i] = pNbrArray[j];
				pNbrArray[j] = temp;
			}
		}
	}
	return nNeighborNum;
}
bool SkContour::NeighborJudge( int &end, SkContourNeighbor &cn )
{
	int nNeighborNum; 
	SkContourNeighbor *pNbrArray;
	for (int e = 0; e < 2; e++)
	{
		nNeighborNum = m_pnEndNbrNum[e];
		pNbrArray = m_ppEndNbrArray[e];
		for( int i = 0; i < nNeighborNum; i++ )
		{
			if( pNbrArray[i].m_pContour == cn.m_pContour )
			{
				end = e;
				cn.m_nEnd = pNbrArray[i].m_nEnd;
				return true;
			}
		}
	}
	return false;
}//bool NeighborJudge( int &end, SkContourNeighbor &cn )
*/
SkPoint SkContour::GetEndPoint( int end )
{
	if( end == 0 )
		return m_PointList.GetHead();
	else
		return m_PointList.GetTail();
}
/*
int SkContour::GetLengthFromInsidePoint2OneEnd( int end, SkPoint p )
{
	int nLength = 0;
	SkPoint pCur;
	if( end == 0 )
	{
		SKPOSITION pos = m_PointList.GetHeadPosition();
		while( pos != 0 )
		{
			pCur = m_PointList.GetNext( pos );
			nLength++;
			if( pCur == p )
				break;
		}
	}
	else
	{
		SKPOSITION pos = m_PointList.GetTailPosition();
		while( pos != 0 )
		{
			pCur = m_PointList.GetPrev( pos );
			nLength++;
			if( pCur == p )
				break;
		}
	}
	return nLength;
}//int GetLengthFromInsidePoint2OneEnd( int end, SkPoint p )
*/
int SkContour::Sign( int n )
{ 
	if( n > 0 )
		return 1;
	else if( n < 0 )
		return -1;
	else
		return 0;
}
double SkContour::GetEndAngle(int e)
{//[-PI,PI)
	SkPoint p0, p1;//the angle from p1 to p0
	if (e == 0)//head
	{
		SKPOSITION pos = GetHeadPosition();
		p0 = GetNextPoint(pos);
		p1 = GetNextPoint(pos);
	}
	else
	{
		SKPOSITION pos = GetTailPosition();
		p0 = GetPrevPoint(pos);
		p1 = GetPrevPoint(pos);
	}
	double dx = p0.x - p1.x;
	double dy = p0.y - p1.y;
	double dAngle = atan2(dy, dx);
	return dAngle;//[-PI,PI)

}
int SkContour::computeProfile(Matrix<float> *pImgOrg, int nMaxPfWd, int nScale, Matrix<SkPixCandArray>* pPixCandMat)
{
	SkNode *pNodeAdd=0;
	SkNode *pNode = GetHeadNode();
	SkNode *pNodeNext;
	while (pNode != 0)
	{
		pNodeNext = pNode->m_pNext;
		if (m_nBaseType != DOG)
		{
			pNode->computeProfile(pImgOrg, nMaxPfWd, nScale, pPixCandMat);
//			pNodeAdd = pNode->shortenProfile(pImgOrg, nMaxPfWd);
//			pNode->thinnerProfile();
			if (pNodeAdd != 0)//one node added
			{
				m_PointList.m_nSize++;
//				pNodeAdd->thinnerProfile();
			}
		}
		else
		{
			nMaxPfWd = m_nBaseHalfSize;
			assert(nMaxPfWd > 0);
			pNode->computeProfileDoG(pImgOrg, nMaxPfWd, nScale, pPixCandMat);
		}
		pNode = pNodeNext;
	}
	return 0;
}
/*
int SkContour::refineProfile()
{
	SkNode *pNodeAdd;
	SkNode *pNode = GetHeadNode();
	while (pNode != 0)
	{
		if (m_nBaseType != DOG)
		{
			pNodeAdd = pNode->refineProfile();
			if (pNodeAdd != 0)
			{
			}
		}
		else
		{//no refinement for DOG base
		}
		pNode = pNode->m_pNext;
	}
	return 0;
}
*/
int SkContour::saveProfile2Matrix(Matrix<SkPixCandArray> &M)
{
	SkNode *pNode = GetHeadNode();
	while (pNode != 0)
	{
		if (m_nBaseType != DOG)
			pNode->saveProfile2Matrix(M);
		else
			pNode->saveProfileDoG2Matrix(M);
		pNode = pNode->m_pNext;
	}
	return 0;
}

/////////////////////////////////////////////////////////////////////
// class  SkContourList
SkContourList::SkContourList()
{
	m_pContourHead = 0;
	m_pContourTail = 0;
	m_nSize = 0;
}
SkContourList::~SkContourList()
{
	RemoveAll();
}
void SkContourList::RemoveAll()
{
	SkContour *pCnt = m_pContourHead;
	while ( pCnt != 0 )
	{
		SkContour *pHead = pCnt->m_pNext;
		pCnt->m_pNext = 0;
		pCnt->m_pPrev = 0;
		delete pCnt;
		pCnt = pHead;
	}
	m_pContourHead = 0;
	m_pContourTail = 0;
	m_nSize = 0;
}
void SkContourList::AddHead( SkContour *pCon )
{
	if( m_pContourHead == 0 && m_pContourTail == 0 )
	{
		m_pContourHead = pCon;
		m_pContourTail = pCon;
		pCon->m_pNext = 0;
		pCon->m_pPrev = 0;
	}
	else
	{
		SkContour *pOldHead = m_pContourHead;
		m_pContourHead = pCon;
		pOldHead->m_pPrev = m_pContourHead;
		m_pContourHead->m_pNext = pOldHead;
		m_pContourHead->m_pPrev = 0;
	}
	m_nSize++;
}
void SkContourList::AddTail( SkContour *pCon )
{
	if( m_pContourHead == 0 && m_pContourTail == 0 )
	{
		m_pContourHead = pCon;
		m_pContourTail = pCon;
		pCon->m_pNext = 0;
		pCon->m_pPrev = 0;
	}
	else
	{
		SkContour *pOldTail = m_pContourTail;
		m_pContourTail = pCon;
		pOldTail->m_pNext = m_pContourTail;
		m_pContourTail->m_pPrev = pOldTail;
		m_pContourTail->m_pNext = 0;
	}
	m_nSize++;
}
int SkContourList::GetSize()
{
	return m_nSize;
}
void SkContourList::Remove( SkContour* pC )
{
	if( pC == m_pContourHead )
	{
		m_pContourHead = m_pContourHead->m_pNext;
		if (m_pContourHead != 0)
			m_pContourHead->m_pPrev = 0;
	}
	else if( pC == m_pContourTail )
	{
		m_pContourTail = m_pContourTail->m_pPrev;
		if (m_pContourTail != 0)
			m_pContourTail->m_pNext = 0;
	}
	else
	{
		SkContour* p1 = pC->m_pPrev;
		SkContour* p2 = pC->m_pNext;
		p1->m_pNext = p2;
		p2->m_pPrev = p1;
	}
	delete pC;

	m_nSize--;
}

/////////////////////////////////////////////////////////////////////
// class SkVertex

///////////////////
// operator == 
bool SkVertex::operator == (SkVertex &v)
{
	if (m_ptNodeArray == v.m_ptNodeArray)
		return true;
	else 
		return false;
}

/////////////////////////////////////////////////////////////////////
// class SkVertexMap

///////////////////
// add()
int	SkVertexMap::add(SkVertex &a)
{
	// check if the vertex with the same points exists or not
	int nPos = getFirstPos();
	SkVertex *pVert;
	while (nPos >= 0)
	{
		pVert = getNext(nPos);
		if (*pVert == a)
			return -1;
	}

	int id=m_array.add(a); 
	m_array.getAt(id)->setID(id); 
	return id;
}


/////////////////////////////////////////////////////////////////////
// class SkOcpMatrix
//	the class for the occupy matrices
int SkOcpMatrix::add(int x, int y, int pPoint, int pCon, int pBase)
{
	if (!m_ocpPoint(y,x).HasMember(pPoint))
	{
		m_ocpPoint(y,x).add(pPoint);
		m_ocpCon(y,x).add(pCon);
		m_ocpBase(y,x).add(pBase);
	}

	return m_ocpPoint(y,x).getSize();
}
int SkOcpMatrix::remove(int x, int y, int pPoint, int pCon, int pBase)
{
	if (m_ocpPoint(y,x).HasMember(pPoint))
	{
		m_ocpPoint(y,x).remove(pPoint);
		m_ocpCon(y,x).remove(pCon);
		m_ocpBase(y,x).remove(pBase);
	}

	return m_ocpPoint(y,x).getSize();
}

/////////////////////////////////////////////////////////////////////
// class SkPixCand
//	the class for pixel candidate
SkPixCand &SkPixCand::operator = (SkPixCand &c)
{
	m_dDis = c.m_dDis;
	m_dValue = c.m_dValue;
	m_nBhID = c.m_nBhID;
	return *this;
}
SkPixCand &SkPixCand::operator = (double v)
{
	m_dValue = v;
	return *this;
}
bool SkPixCand::operator < (SkPixCand &c)
{
	if (m_dDis < c.m_dDis)
		return true; 
	else
		return false;
}
bool SkPixCand::operator > (SkPixCand &c)
{
	if (m_dDis > c.m_dDis)
		return true; 
	else
		return false;
}
SkPixCand SkPixCand::operator + (SkPixCand &c)
{
	if (m_dDis > c.m_dDis)
		return c;
	else
		return *this;
}
SkPixCand SkPixCand::operator + (double v)
{
	m_dValue += v;
	return *this;
}
SkPixCand &SkPixCand::operator += (SkPixCand &c)
{
	if (*this > c)
		*this = c;
	return *this;
}
SkPixCand SkPixCand::operator - (SkPixCand &c)
{
	return *this;
}
SkPixCand SkPixCand::operator * (double v)
{
	return *this;
}
SkPixCand SkPixCand::operator * (SkPixCand c)
{
	return *this;
}
SkPixCand SkPixCand::operator / (const SkPixCand c)
{
	return *this;
}
SkPixCand SkPixCand::operator / (double v)
{
	return *this;
}

bool SkPixCand::operator == (SkPixCand &c)
{
	if( m_dDis == c.m_dDis &&  m_dValue == c.m_dValue
		&& m_nBhID == c.m_nBhID)
		return true;
	else
		return false;
}
bool SkPixCand::operator != (SkPixCand &c)
{
	return !(*this == c);
}


/////////////////////////////////////////////////////////////////////
// class SkSketchGraph
//	the class for the graph on the sketches

///////////////////
// addNewContour()
SkContour* SkSketchGraph::addNewContour(int nBaseID, SkBaseMap *pBaseMap)
{
	assert(nBaseID >= 0);
	SkBaseNode *pBaseNode = pBaseMap->getNodeAt(nBaseID);
	pBaseNode->setValid(false);

	int nBaseWd = pBaseNode->m_data.getValidWidth();

	double dOrt = pBaseNode->m_data.getOrt();//[0,PI)

	int nBaseX0 = pBaseNode->getCol();
	int nBaseY0 = pBaseNode->getRow();

	int nBaseEndX = pBaseNode->m_data.getEndX();
	int nBaseEndY = pBaseNode->m_data.getEndY();

	SkContour *pCon = new SkContour;
	SkPoint p;

	p.x = nBaseX0 - nBaseEndX;
	p.y = nBaseY0 - nBaseEndY;
	p.m_pnBaseID[0] = -1;
	p.m_pnBaseID[1] = nBaseID;
	p.m_pCon = pCon;
	SkNode *pNode;
	pNode = pCon->AddHeadPoint(p);
//	pNode->m_nMaxPrflWd = nBaseWd;
	pNode->m_nMaxPrflWd = m_nMaxPrflWd;//not consider base width any more, 05-30-2003
//	pNode->m_nBaseType = pBaseNode->m_data.getType();
//	pNode->m_nBaseHalfSize = pBaseNode->m_data.getHalf();
	
	SkNode *pNodeNew = pNode;

	p.x = nBaseX0 + nBaseEndX;
	p.y = nBaseY0 + nBaseEndY;
	p.m_pnBaseID[0] = nBaseID;
	p.m_pnBaseID[1] = -1;
	p.m_pCon = pCon;
	pNode = pCon->AddTailPoint(p);
	pNode->m_nMaxPrflWd = -1;
//	pNode->m_nBaseType = -1;
//	pNode->m_nBaseHalfSize = -1


	pCon->m_nBaseType = pBaseNode->m_data.getType();
	pCon->m_nBaseHalfSize = pBaseNode->m_data.getHalf();

	Matrix<float> *pTmp;
	if (m_bUseRes4Prfl)//use residual image to build profile
		pTmp = &m_imgRes;
	else
		pTmp = m_pImgOrg;

	int nSketchType = pCon->m_nBaseType;
	//compute profile
	if (nSketchType != DOG)
	{
		pNodeNew->computeProfile(pTmp, m_nMaxPrflWd, 1, &m_pixCandMat);
	}
	else
	{
		int nMaxPfWd = pCon->m_nBaseHalfSize;
		assert(nMaxPfWd > 0);
		pNodeNew->computeProfileDoG(pTmp, nMaxPfWd, 1, &m_pixCandMat);
	}
	double dErrorChg = resErrorChangeByAddingNewProfile(pNodeNew, nSketchType);
	m_nSketchNum++;
	m_pdResError[m_nSketchNum] = m_pdResError[m_nSketchNum-1] + dErrorChg;

	int nShowSketchStep = 500;
	if (m_nSketchNum/nShowSketchStep * nShowSketchStep == m_nSketchNum )
	{//save intermediate results
		char dir[32] = "skps";
		char num[32];
		char name[512];
		::changeNumTo4Char(num, m_nSketchNum);
		
		m_imgRec = *m_pImgOrg - m_imgRes;
		sprintf(name, "%s\\%s_rec_prfl_%s.bmp", dir, m_pchProjName, num);
		skSaveMatrixAsBitmap(name, m_imgRec, m_dImgMean, m_dImgStd);

		sprintf(name, "%s\\%s_res_prfl_%s.bmp", dir, m_pchProjName, num);
		skSaveMatrixAsBitmap(name, m_imgRes, m_dImgMean, m_dImgStd);
	}
	if (m_nShowProcess)
	{
		char dir[512];
		sprintf(dir, "BupBsp\\rct");
		_mkdir(dir);

		char num[32];
		char name[512];
		::changeNumTo4Char(num, m_nSketchNum);
		
		m_imgRec = *m_pImgOrg - m_imgRes;
		sprintf(name, "%s\\%s_prfl_%s.bmp", dir, m_pchProjName, num);
		skSaveMatrixAsBitmap(name, m_imgRec, m_dImgMean, m_dImgStd);

	}


	m_conList.AddTail(pCon);

	return pCon;

}
////////////////////
// resErrorChangeByAddingNewProfile()
double SkSketchGraph::resErrorChangeByAddingNewProfile(SkNode *pNode,//the node of that sketch
													   int nSketchType)//the type of sketch, DOG or non-DOG
{
	int wd, ht;
	int xm, ym;//center of Matrix M
	wd = ht = 101;
	xm = ym = wd/2;//center of M
	Matrix<float> M(ht, wd);

	int x0, y0, x1, y1;//start and end center of the sketch
	int xc, yc;//center of the sketch
	int dx, dy;//displacement from xc to xm

	int n = 0;

	SkNode *pNext = pNode->m_pNext;
	if (pNext == 0)
		return 0;

	pNext = pNode->m_pNext;
	x0 = pNode->m_data.x;
	y0 = pNode->m_data.y;
	x1 = pNext->m_data.x;
	y1 = pNext->m_data.y;//real start and end point

	xc = (x0+x1)/2;
	yc = (y0+y1)/2;//center of the line of real points

	dx = xc - xm;
	dy = yc - ym;

	M.Set(0);
	if (nSketchType != DOG)
		pNode->m_prfl.saveLineProfile(x0-dx, y0-dy, x1-dx, y1-dy, &M, &m_pixCandMat, dx, dy);
	else
		pNode->m_prfl.saveDoGProfile(x0-dx, y0-dy, x1-dx, y1-dy, &M, &m_pixCandMat, dx, dy);

	int nBound = 0;
	double dResErrorChg = skSubtractImageAndReturnSquareSumChange(&m_imgRes, &M, dx, dy, nBound);
	return dResErrorChg;
}
///////////////////
// addNewPoint2Contour()
int SkSketchGraph::addNewPoint2Contour(int nBaseID, SkBaseMap *pBaseMap, SkContour* pCon, int a)
{
	assert(nBaseID >= 0);
	assert(a == 0 || a == 1);
	SkBaseNode *pBaseNode = pBaseMap->getNodeAt(nBaseID);
	double dOrt = pBaseNode->m_data.getOrt();//[0,PI)

	int nBaseWd = pBaseNode->m_data.getValidWidth();

	int nBaseX0 = pBaseNode->getCol();
	int nBaseY0 = pBaseNode->getRow();

	int nBaseEndX = pBaseNode->m_data.getEndX();
	int nBaseEndY = pBaseNode->m_data.getEndY();
	SkPoint p;

//set the next point which is furthur from the end of the current contour	
	SkPoint p0, p1, p2;
	p0 = pCon->GetEndPoint(a);

	
	p1.x = nBaseX0 - nBaseEndX;
	p1.y = nBaseY0 - nBaseEndY;
	p2.x = nBaseX0 + nBaseEndX;
	p2.y = nBaseY0 + nBaseEndY;
/* only for debugging
	int check_x = 119;
	int check_y = 47;
	int error = 5;

	if (fabs(p0.x-check_x) <=error && fabs(200-p0.y-check_y) <= error)
		p0.x += 0;
	
	if (fabs(p1.x-check_x) <=error && fabs(200-p1.y-check_y) <= 5
		|| fabs(p2.x-check_x) <=error && fabs(200-p2.y-check_y) <= error)
		check_x += 0;

	check_x = 66;
	check_y = 123;

	if (fabs(p0.x-check_x) <=error && fabs(200-p0.y-check_y) <= error)
		p0.x += 0;
	
	if (fabs(p1.x-check_x) <=error && fabs(200-p1.y-check_y) <= 5
		|| fabs(p2.x-check_x) <=error && fabs(200-p2.y-check_y) <= error)
		check_x += 0;
*/
	double d1 = p0.Distance(p1);
	double d2 = p0.Distance(p2);
	if (d1 > d2) p = p1;
	else p = p2;

//	assert(d1 < 7 || d2 < 7);//no reason to assume this, 05-29-2003

	SkNode *pNode, *pNodeNext, *pNodePrev, *pNodeNew;
	if (a == 0)
	{
//		p.x = nBaseX0 - nBaseEndX;
//		p.y = nBaseY0 - nBaseEndY;
		p.m_pnBaseID[0] = -1;
		p.m_pnBaseID[1] = nBaseID;
		p.m_pCon = pCon;
		pNode = pCon->AddHeadPoint(p);
//		pNode->m_nMaxPrflWd = nBaseWd;
		pNode->m_nMaxPrflWd = m_nMaxPrflWd;//not consider base width any more, 05-30-2003
		pNodeNext = pNode->m_pNext;
		assert(pNodeNext->m_data.m_pnBaseID[1] >= 0);
		assert(pNodeNext->m_data.m_pnBaseID[0] < 0);
		pNodeNext->m_data.m_pnBaseID[0] = nBaseID;
		pNodeNew = pNode;
	}
	else if (a == 1)
	{
//		p.x = nBaseX0 + nBaseEndX;
//		p.y = nBaseY0 + nBaseEndY;
		p.m_pnBaseID[0] = nBaseID;
		p.m_pnBaseID[1] = -1;
		p.m_pCon = pCon;
		pNode = pCon->AddTailPoint(p);
		pNode->m_nMaxPrflWd = -1;
		pNodePrev = pNode->m_pPrev;
		pNodePrev->m_nMaxPrflWd = m_nMaxPrflWd;
		assert(pNodePrev->m_data.m_pnBaseID[0] >= 0);
		assert(pNodePrev->m_data.m_pnBaseID[1] < 0);
		pNodePrev->m_data.m_pnBaseID[1] = nBaseID;
		pNodeNew = pNodePrev;
	}
	else
		assert(false);//not considered

	int nSketchType = pCon->m_nBaseType;
	
	Matrix<float> *pTmp;
	if (m_bUseRes4Prfl)//use residual image to build profile
		pTmp = &m_imgRes;
	else
		pTmp = m_pImgOrg;

	//compute profile
	if (nSketchType != DOG)
	{
		pNodeNew->computeProfile(pTmp, m_nMaxPrflWd, 1, &m_pixCandMat);
	}
	else
	{
		int nMaxPfWd = pCon->m_nBaseHalfSize;
		assert(nMaxPfWd > 0);
		pNodeNew->computeProfileDoG(pTmp, nMaxPfWd, 1, &m_pixCandMat);
	}
	double dErrorChg = resErrorChangeByAddingNewProfile(pNodeNew, nSketchType);
	m_nSketchNum++;
	m_pdResError[m_nSketchNum] = m_pdResError[m_nSketchNum-1] + dErrorChg;

	int nShowSketchStep = 500;
	if (m_nSketchNum/nShowSketchStep * nShowSketchStep == m_nSketchNum )
	{//save intermediate results
		char dir[32] = "skps";
		char num[32];
		char name[512];
		::changeNumTo4Char(num, m_nSketchNum);
		
		m_imgRec = *m_pImgOrg - m_imgRes;
		sprintf(name, "%s\\%s_rec_prfl_%s.bmp", dir, m_pchProjName, num);
		skSaveMatrixAsBitmap(name, m_imgRec, m_dImgMean, m_dImgStd);

		sprintf(name, "%s\\%s_res_prfl_%s.bmp", dir, m_pchProjName, num);
		skSaveMatrixAsBitmap(name, m_imgRes, m_dImgMean, m_dImgStd);
	}
	if (m_nShowProcess)
	{
		char dir[512];
		sprintf(dir, "BupBsp\\rct");
		_mkdir(dir);

		char num[32];
		char name[512];
		::changeNumTo4Char(num, m_nSketchNum);
		
		m_imgRec = *m_pImgOrg - m_imgRes;
		sprintf(name, "%s\\%s_prfl_%s.bmp", dir, m_pchProjName, num);
		skSaveMatrixAsBitmap(name, m_imgRec, m_dImgMean, m_dImgStd);

	}

	return 0;
}

///////////////////
// addNewContourByNbredBase()
//	1). create a new contour with this base
//	2). for all possible type of nbr, find the nbr base id, add it to the contour
//	3). repeat 2) until no nbr found with all possible nbr type
SkContour* SkSketchGraph::addNewContourByNbredBase(int nBaseID, SkBaseMap *pBaseMap)
{
	assert(nBaseID >= 0);
	SkContour *pCon;

	pCon = addNewContour(nBaseID, pBaseMap);

//	return pCon;//05-23-2003, build one contour for each base, in order to remove some accuracy error

	double dMaxConLength = 20;//max contour length, a thresh to only generate short contour
	double dConLength;

	int nbrType = ADR_NBR;
	assert(MAX_ADR_NUM == 2);//only consider 2 now
	SkBaseNode *pNode;
	for (int a = 0; a < MAX_ADR_NUM; a++)
	{
		pNode= pBaseMap->getNodeAt(nBaseID);
		pNode->setValid(false);
		int nbrID = pNode->getNbrIDAt(a, nbrType);
		dConLength = pCon->GetLength();
//		if (dConLength > dMaxConLength)
//			break;
		while (nbrID >= 0)
		{
			addNewPoint2Contour(nbrID, pBaseMap, pCon, a);
			pNode = pBaseMap->getNodeAt(nbrID);
			nbrID = pNode->getNbrIDAt(a, nbrType);
			pNode->setValid(false);
		}
	}

	addOccupy(pCon);

	findNewVerticesByContour(pCon);

	return pCon;
}

///////////////////
// buildSketchGraphFromNbredBaseMap()
//	Given a base map in which the ADR_NBRs have been found, build the sketch graph
//	return the num of contours in this graph
int SkSketchGraph::buildSketchGraphFromNbredBaseMap(SkBaseMap *pBaseMap, const char* err_name)
{
	m_nSketchNum = 0;
	m_imgRes = *m_pImgOrg;
	int nBound = 5;
	m_pdResError[m_nSketchNum] = skSquareSumOfMatrix(&m_imgRes, nBound);

	//get original image
	Matrix<float> *pImgOrg = getImgOrg();
	// get image dimension
	int wd, ht;
	pImgOrg->GetDimension(ht, wd);
	//clear pixel candiate matrix
	m_pixCandMat.SetDimension(ht, wd);
	clearPixelCandidateMatrix();
	// sketch occupancy matrix
	m_ocp.setDimension(wd, ht);

	RemoveAllContours();
	//set all the nodes valid first
	pBaseMap->setValid(true);//the flag for checked one node or not
	int pos = pBaseMap->getFirstPos(), oldPos;
	SkBaseNode *pNode;
	SkContour *pCon;
	while (pos >= 0)
	{
		oldPos = pos;
		pNode = pBaseMap->getNextNode(pos);
		if (!pNode->valid())
			continue;
		pCon = addNewContourByNbredBase(oldPos, pBaseMap);
	}
	pBaseMap->setValid(true);//set valid flag back

	m_imgRec = *m_pImgOrg - m_imgRes;

	//save resudual error
	::saveArray(m_pdResError, m_nSketchNum+1, err_name);
	return getContourNum();
}
///////////////////
// buildOccupyMatrix()
//	Build the matrix which indicates which contour and related points occupies each pixel in the image
int SkSketchGraph::buildOccupyMatrix(int wd, int ht)
{
	m_ocp.setDimension(wd, ht);


	SkContour *pCon = getContourHead();
	SKPOSITION pos, pos0, pos1;
	while (pCon != 0)
	{
		addOccupy(pCon);
		pCon = pCon->m_pNext;
	}
	
	return getContourNum();
}
///////////////////
// addOccupy()
int SkSketchGraph::addOccupy(SkContour *pCon)
{
	SKPOSITION pos = pCon->GetHeadPosition();
	if (pos == 0)
		return -1;
	SkBaseNode *pBaseNode = 0;

	SKPOSITION pos0 = pos, pos1;
	SkPoint p0, p1;
	p0 = pCon->GetNextPoint(pos);
	while (pos != 0)
	{
		pos1 = pos;
		p1 = pCon->GetNextPoint(pos);
		addOccupy(p0.x, p0.y, p1.x, p1.y, (int)pos0, (int)pos1, (int)pCon, (int)pBaseNode);

		p0 = p1;
		pos0 = pos1;
	}
	return 0;
}
///////////////////
// addOccupy()
int SkSketchGraph::addOccupy(int x0, int y0, //start point								
								int x1, int y1, //end pint
								int pt1, int pt2, //two nbr SkPoint
								int con, //contour 
								int base)//base
{
	int wd = m_nImgWd;
	int ht = m_nImgHt;
	int *pnX = new int[ht*wd];
	int *pnY = new int[ht*wd];

	int x, y;

	int n = ::rasterize_line_segment(x0, y0, x1, y1, pnX, pnY);
	for (int i = 0; i < n; i++)
	{
		x = pnX[i];
		y = pnY[i];
		if (x < 0 || x >= wd || y < 0 || y >= ht)
			continue;
		m_ocp.add(x, y, pt1, con, base);
		m_ocp.add(x, y, pt2, con, base);
		if (x < wd-1)
		{//add one more pixel in order to make sure all lines are overlapped at one pixel if they are cross each other
			m_ocp.add(x+1, y, pt1, con, base);
			m_ocp.add(x+1, y, pt2, con, base);
		}
	}
	delete []pnX;
	delete []pnY;
	return n;
}
///////////////////
// removeOccupy()
int SkSketchGraph::removeOccupy(int x0, int y0, //start point								
								int x1, int y1, //end pint
								int pt1, int pt2, //two nbr SkPoint
								int con, //contour 
								int base)//base
{
	int wd = m_nImgWd;
	int ht = m_nImgHt;
	int *pnX = new int[ht*wd];
	int *pnY = new int[ht*wd];

	int x, y;

	int n = ::rasterize_line_segment(x0, y0, x1, y1, pnX, pnY);
	for (int i = 0; i < n; i++)
	{
		x = pnX[i];
		y = pnY[i];
		if (x < 0 || x >= wd || y < 0 || y >= ht)
			continue;
		m_ocp.remove(x, y, pt1, con, base);
		m_ocp.remove(x, y, pt2, con, base);
		if (x < wd-1)
		{//add one more pixel in order to make sure all lines are overlapped at one pixel if they are cross each other
			m_ocp.remove(x+1, y, pt1, con, base);
			m_ocp.remove(x+1, y, pt2, con, base);
		}
	}
	delete []pnX;
	delete []pnY;
	return n;
}
///////////////////
// findVertices()
//	find vertices (cross points of curves)
//	return vertice num
int SkSketchGraph::findVertices()
{
	int wd = m_nImgWd;
	int ht = m_nImgHt;

	SkVertex vert;

	int r, c;
	int i, n;
	SkNode *pPtNode;
	SkContour *pCon;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			n = getOcpPointNum(r,c);
			if (n >= 4)
			{
				vert.removeAll();
				for (i = 0; i < n; i++)
				{
					pPtNode = getOcpPoint(r, c, i);
					pCon = getOcpContour(r, c, i);
					vert.addPoint(pPtNode, pCon);
				}
				vert.x = c;
				vert.y = r;
				m_vertMap.add(vert);
				c++;//no need to test next pixel
			}
		}
	}
	return 0;
}

///////////////////
// findNewVerticesByContour()
//	extend or trim all of the contours
int SkSketchGraph::findNewVerticesByContour(SkContour* pCon)
{
	SKPOSITION pos = pCon->GetHeadPosition();
	if (pos == 0)
		return -1;
	SkBaseNode *pBaseNode = 0;

	int wd = m_nImgWd;
	int ht = m_nImgHt;
	int *pnX = new int[ht*wd];
	int *pnY = new int[ht*wd];
	int r, c;
	SkVertex vert;
	SkNode *pPtNode;

	SKPOSITION pos0 = pos, pos1;
	SkPoint p0, p1;
	p0 = pCon->GetNextPoint(pos);
	int x0, y0, x1, y1;
	while (pos != 0)
	{
		p1 = pCon->GetNextPoint(pos);

		x0 = p0.x;
		y0 = p0.x;
		x1 = p1.x;
		y1 = p1.y;

		int n = ::rasterize_line_segment(x0, y0, x1, y1, pnX, pnY);
		for (int i = 0; i < n; i++)
		{
			c = pnX[i];
			r = pnY[i];
			if (c < 0 || c >= wd || r < 0 || r >= ht)
				continue;

			int m = getOcpPointNum(r,c);
			if (m >= 4)
			{
				vert.removeAll();
				for (int j = 0; j < m; j++)
				{
					pPtNode = getOcpPoint(r, c, j);
					pCon = getOcpContour(r, c, j);
					vert.addPoint(pPtNode, pCon);
				}
				vert.x = c;
				vert.y = r;
				m_vertMap.add(vert);
			}
		}
		p0 = p1;
	}


	delete []pnX;
	delete []pnY;
	return 0;
}

///////////////////
// extendOrTrimContours()
//	extend or trim all of the contours
int SkSketchGraph::extendOrTrimContours()
{
	int nConNum = getContourNum();
	SkContour **ppCons = new SkContour*[nConNum];
//	int nIncrease = 1;
//	int nIncrease = 0;
	int nIncrease = -1;
	
	getSortedContoursByLength(ppCons, nIncrease);

	for (int i = 0; i < nConNum; i++)
	{
		extendOrTrimContours(ppCons[i]);
	}
	delete []ppCons;

//	skSaveMatrixAsBitmap("res2.bmp", m_imgRes, m_dImgMean, m_dImgStd);

	return 0;
}


///////////////////
// extendOrTrimContours()
//	no consideration on DoG sketches
//	1) remove noisy contour first
//	2) find all neighbor contours
//	3) if the best neighbor candidate is satisfied some criteria then extend,
int SkSketchGraph::extendOrTrimContours(SkContour *pCon)
{
	if (pCon->m_nBaseType == DOG)
		return 1;

	if (removeNoisyContour(pCon) == 1)
		return 1;

	return 0;//only remove noisy sketches, no extension

	//for each end of the contours, find its neighbor
	int line_wd = 3;
	int nReturn;
	int nNbrNum;
	for (int e = 0; e < 2; e++)
	{
		findAllNbrContours(pCon, e);
		nNbrNum = pCon->GetNeighborNum(e);
		if (nNbrNum >= 1)
		{
			nReturn = extendContourEndByNeighbor(pCon, e);
			if (nReturn >= 0 && m_nShowProcess)
				skSaveSketchGraphEvolvingProcess(*this, line_wd);
		}
	}
	return 0;
}

///////////////////
// findAllNbrContours()
//	1) define the end search region
//	2) get the contours in the search region
//	3) for each contour, consider 
//		a) its two ends :
//		define  (i) distance 
//				(ii) delta ort of the new linelet and existing one
//				(iii) delta angle of the two ends of the two contours
//		b) the closest points to checking end
//		define  (i) distance 
//				(ii) delta ort of the new linelet and existing one
//	4) also consider just extend the free end 
//		define  (i) distance ONLY
//	5) all the possible nbrs are ordered by:
//		m_dContStrength = m_dDeltaOrt*18/PI + m_dDeltaAngle*18/PI + m_dDis/2;//continuity strength
int SkSketchGraph::findAllNbrContours(SkContour *pCon, int e)
{
	pCon->RemoveAllNeighbors();

	SkPoint p0 = pCon->GetEndPoint(e);
	int x0 = p0.x;
	int y0 = p0.y;

	if (fabs(x0 - 48.f) <= 3 && fabs(191.f-40-y0) <= 3)
		x0 += 0;

	double dAngle = pCon->GetEndAngle(e);
	double dLength = pCon->GetLength();

	int wd = m_nImgWd;
	int ht = m_nImgHt;

	oval_Array<oval_Point> ptArray, ocpPtArray;
	int k = getContourEndSearchRegion(x0, y0, dAngle, dLength, ptArray, wd, ht);
	int nConNum = getContourNum();

	SkContour **ppCons = new SkContour*[nConNum];

	int n = getContoursInRegion(ppCons, ptArray, ocpPtArray);
	// compute below as the nbrs to one end of the contour
	//	1) Nbr contours end
	//	2) nearest point to that nbr contour
	//	3) nearest extend cross point to some contour
	//	4) parallel contours

	SkContour *pConNbr;
	SkPoint p;
	double dDis;
	double dNbrEndAngle, dDeltaAngle;
	double dOrt, dDeltaOrt;//orientation between these two end points
	SkPoint point;
	SkContourNeighbor conNbr;
	for (int i = 0; i < n; i++)
	{
		pConNbr = ppCons[i];
		if (pConNbr == pCon)
			continue;
		conNbr.m_pContour = pConNbr;
		// two neighbors with the ends of nbr cons
		for (int ne = 0; ne < 2; ne++)
		{
			p = pConNbr->GetEndPoint(ne);
			dDis = p.Distance(p0);
			conNbr.m_dDis = dDis;
			point.x = p.x;
			point.y = p.y;
			conNbr.m_point = point;

			conNbr.m_nEnd = ne;
			
			dNbrEndAngle = pConNbr->GetEndAngle(ne);
			dDeltaAngle = ::angleDifference(-dNbrEndAngle, dAngle);
			conNbr.m_dDeltaAngle = dDeltaAngle;
			
			dOrt = atan2(double(p.y-p0.y), double(p.x-p0.x));
			dDeltaOrt = ::angleDifference(dAngle, dOrt);
			dDeltaOrt += ::angleDifference(dOrt, -dNbrEndAngle);				
			if (dDis <= 4.5)//don't compute the ort diff for too closed points
				dDeltaOrt = 0;
			conNbr.m_dDeltaOrt = dDeltaOrt;
			conNbr.computeContStrength();

			pCon->AddNeighbor(e, conNbr);
		}
		// nearest point
		p = nearestPointOnOneContour(pConNbr, p0, ocpPtArray, m_ocp.m_ocpCon);
		dDis = p.Distance(p0);
		conNbr.m_dDis = dDis;
		point.x = p.x;
		point.y = p.y;
		conNbr.m_point = point;

		dDeltaAngle = 0;
		conNbr.m_dDeltaAngle = dDeltaAngle;
		dOrt = atan2(double(p.y-p0.y), double(p.x-p0.x));
		dDeltaOrt = ::angleDifference(dAngle, dOrt);;				
		if (dDis <= 4.5)//don't compute the ort diff for too closed points
			dDeltaOrt = 0;
		conNbr.m_dDeltaOrt = dDeltaOrt;

		conNbr.m_nEnd = 2;
		
		conNbr.computeContStrength();
		pCon->AddNeighbor(e, conNbr);
	}
	//extend cross point
	p = nearestExtendPoint(p0, dAngle, pConNbr, ocpPtArray, m_ocp.m_ocpCon);

	if (p.x >= 0 && p.y >= 0)
	{
		dDis = p.Distance(p0);
		conNbr.m_dDis = dDis/2;//this distance is weighted less
		point.x = p.x;
		point.y = p.y;
		conNbr.m_point = point;

		dDeltaAngle = 0;
		conNbr.m_dDeltaAngle = dDeltaAngle;
		dDeltaOrt = 0;				
		conNbr.m_dDeltaOrt = dDeltaOrt;

		conNbr.computeContStrength();
		pCon->AddNeighbor(e, conNbr);
	}

	delete [] ppCons;
	return pCon->GetNeighborNum(e);
}

SkPoint SkSketchGraph::nearestPointOnOneContour(SkContour *pCon,//checked contour
											 SkPoint p0, //interested point
											 oval_Array<oval_Point> &ocpPtArray, //all possible point 
											 Matrix<oval_IntArray> &ocpCon)//the contour ocp matrix to check this con or not
{
	SkPoint p, p_min;
	p_min.x = -1;
	p_min.y = -1;
	double dDis, dDisMin = 100000;

	int n = ocpPtArray.GetSize();
	for (int i = 0; i < n; i++)
	{
		p.x = ocpPtArray.GetAt(i).x;
		p.y = ocpPtArray.GetAt(i).y;
		if (!ocpCon(p.y, p.x).HasMember((int&)(pCon)))
			continue;
		dDis = p.Distance(p0);
		if (dDis < dDisMin)
		{
			dDisMin = dDis;
			p_min = p;
		}
	}
	return p_min;
}
///////////////////
// nearestExtendPoint
SkPoint SkSketchGraph::nearestExtendPoint(SkPoint p0, // the point needs to be exended
									   double dAngle, //the contour angle at that point
									   SkContour* &pConNbr, //the contour on which the nearest point is
									   oval_Array<oval_Point> &ocpPtArray,//all possible pt 
									   Matrix<oval_IntArray> &ocpCon)//the contour ocp matrix to check this con or not
{
	double dTh = PI/180 * 15;
	SkPoint p, p_min;
	p_min.x = -1;
	p_min.y = -1;
	double dOrt, dDiff;
	double dDis, dDisMin = 100000;
	int n = ocpPtArray.GetSize();
	for (int i = 0; i < n; i++)
	{
		p.x = ocpPtArray.GetAt(i).x;
		p.y = ocpPtArray.GetAt(i).y;

		if (abs(p.x-56) <= 3 && abs(191-p.y-47) <= 3)
			p.x += 0;

		if ((p.x == p0.x || p.x == p0.x+1 || p.x == p0.x-1) && p.y == p0.y)
			continue;//there is one pixel overhead to deal with 45 degree non crossing problem

		dOrt = atan2(double(p.y-p0.y), double(p.x-p0.x));
		
		dDiff = ::angleDifference(dAngle, dOrt);

		if (dDiff < dTh)
		{
			dDis = p0.Distance(p);
			if (dDis < dDisMin)
			{
				dDisMin = dDis;
				p_min = p;
				assert(ocpCon(p.y, p.x).GetSize() >= 1);
				pConNbr = (SkContour*)(ocpCon(p.y, p.x).GetAt(0));
			}

		}
	}
	return p_min;

}
///////////////////
// save2Matrix()
//	Save the graph into a matrix
int SkSketchGraph::save2Matrix(Matrix<float> &M,//the matrix which saves the points on the graph, 0-on graph, 1-not
							int wd, int ht, //image width and height
							int line_wd)//the line-width used for the graph
{
	M.SetDimension(ht, wd);
	M.Set(1.0);
	int *pnX = new int[ht*wd];
	int *pnY = new int[ht*wd];

	int draw_wd;

	int x, y;

	int nDoGWd = MIN(2, line_wd);//DoG symbolic width
	int nPrflWd;

	SkContour *pCon = getContourHead();
	while (pCon != 0)
	{
		if (pCon->GetLength() < 20)
		{
//			pCon = pCon->m_pNext;
//			continue;
		}
		if (pCon->m_nBaseType != DOG)
		{
//			if (pCon->GetLength() < 100 || pCon->GetLength() > 170)
//			{
//				pCon = pCon->m_pNext;
//				continue;
//			}
			SKPOSITION pos = pCon->GetHeadPosition();
			if (pos == 0)
				assert(false);
			SkPoint p0, p1;
			p0 = pCon->GetNextPoint(pos);
			while (pos != 0)
			{
//				nPrflWd = ((SkNode*)pos)->m_pPrev->m_nPrflWd;
				nPrflWd = ((SkNode*)pos)->m_pPrev->getProfileValidWidth();
				if (line_wd > 3)//draw constraints
					draw_wd = nPrflWd;
				else
					draw_wd = line_wd;

				p1 = pCon->GetNextPoint(pos);
				int n = ::rasterize_line_segment(p0.x, p0.y, p1.x, p1.y, pnX, pnY, draw_wd);
				for (int i = 0; i < n; i++)
				{
					x = pnX[i];
					y = pnY[i];
					if (x < 0 || x >= wd || y < 0 || y >= ht)
						continue;
					M(y,x) = 0;
				}
				p0 = p1;
			}
		}
		else
		{
			SKPOSITION pos = pCon->GetHeadPosition();
			if (pos == 0)
				assert(false);
			SkPoint p0, p1;
			p0 = pCon->GetNextPoint(pos);
			p1 = pCon->GetNextPoint(pos);

			int w = pCon->m_nBaseHalfSize;
			int c = w/3;

			int xc = (p0.x+p1.x)/2;
			int yc = (p0.y+p1.y)/2;
			int x, y;
			int d;
			for (int dy = -(w-1); dy <= (w-1); dy++)
			{
				y = yc + dy;
				if (y < 0 || y >= ht)
					continue;
				for (int dx = -(w-1); dx <= (w-1); dx++)
				{
					x = xc + dx;
					if (x < 0 || x >= wd)
						continue;
					d = (int)(sqrt(double(dx*dx + dy*dy)));
					if (line_wd <= 3)
					{
						if (line_wd == 1)
						{
							if (d == c-1)
								M(y,x) = 0;

						}
						else
						{
							if(d >= c-1 && d <= c+1)
								M(y,x) = 0;
						}
					}
					else
					{
						if (d < line_wd)
							M(y,x) = 0;//thick graph keep all points
					}

				}
			}
		}
		pCon = pCon->m_pNext;
	}
	delete []pnX;
	delete []pnY;

	// mark boundary
/*
	int bound = 2;
	int r, c;
	//col boundary
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < bound; c++)
		{
			M(r,c) = 0;
			M(r,wd-c-1) = 0;
		}
	}
	//row boundary
	for (c = 0; c < wd; c++)
	{
		for (r = 0; r < bound; r++)
		{
			M(r,c) = 0;
			M(ht-r-1,c) = 0;
		}
	}
*/	return getContourNum();
}


///////////////////
// saveVertices2Matrix()
//	Save the save Vertics into a matrix
int SkSketchGraph::saveVertices2Matrix(Matrix<float> &M,//the matrix which saves the points on the graph, 0-on graph, 1-not
									int wd, int ht, //image width and height
									int line_wd)//the line-width used for the graph
{
	M.SetDimension(ht, wd);
	M.Set(1.0);
	int *pnX = new int[ht*wd];
	int *pnY = new int[ht*wd];

	int x, y, x0, y0, x1, y1;

	int n, i, j, k;
	SkNode *pPtNode;
	int pos = m_vertMap.getFirstPos();
	SkVertex *pVertex;
	while (pos >= 0)
	{
		pVertex = m_vertMap.getNext(pos);
		k = pVertex->getDegree();
		x0 = pVertex->x;
		y0 = pVertex->y;
/*		for (i = 0; i < k; i++)
		{
			pPtNode = pVertex->getPtNode(i);
			x1 = pPtNode->m_data.x;
			y1 = pPtNode->m_data.y;
			n = ::rasterize_line_segment(x0, y0, x1, y1, pnX, pnY, line_wd);
			for (j = 0; j < n; j++)
			{
				x = pnX[j];
				y = pnY[j];
				if (x < 0 || x >= wd || y < 0 || y >= ht)
					continue;
				M(y,x) = 0;
			}
		}
*/
 		M(y0,x0) = 0;
		for (int i = -1; i <= 1; i++)
		{
			y = y0 + i;
			if (y < 0 || y >= ht)
				continue;
			for (int j = -1; j <= 1; j++)
			{
				x = x0 + j;
				if (x < 0 || x >= wd)
					continue;
				M(y, x) = 0;
			}
		}
	}
	delete []pnX;
	delete []pnY;
	return getContourNum();
}

///////////////////
// getContourEndSearchRegion()
//	
int SkSketchGraph::getContourEndSearchRegion(int x0, int y0,//contour end point
										  double dAngle, //the angle of the end;
										  double dLength, //the length of the contour
										  oval_Array<oval_Point> &ptArray,//the points of the  search region
										  int wd, int ht)//image width and height
{
#define VER_NUM 4
	if (dLength > 40)
		dLength = 40;
	if (dLength < 10)
		dLength = 10;
	double dx[VER_NUM], dy[VER_NUM];
	dx[0] = -5; dy[0] = -10;
	dx[1] = dLength; dy[1] = -20;
	dx[2] = dx[1]; dy[2] = -dy[1];
	dx[3] = dx[0]; dy[3] = -dy[0];

	oval_Point p;
	oval_Array<oval_Point> vertArray;

	for (int i = 0; i < VER_NUM; i++)
	{
		::rotate(dx[i], dy[i], dAngle);
		dx[i] += x0;
		dy[i] += y0;
		if (dx[i] < 0) dx[i] = 0;
		if (dx[i] >= wd) dx[i] = wd-1;
		if (dy[i] < 0) dy[i] = 0;
		if (dy[i] >= ht) dy[i] = ht-1;
		p.x = (int)(dx[i]);
		p.y = (int)(dy[i]);
		vertArray.Add(p);
	}
	::polygonFilling(vertArray, ptArray);
	return ptArray.GetSize();

}
///////////////////
// getContoursInRegion()
int SkSketchGraph::getContoursInRegion(SkContour **ppCons, // the pointers to contours which are in the given region
									oval_Array<oval_Point> &ptArray,//the point array defining the region
									oval_Array<oval_Point> &ocpPtArray)//the point array which is occupied by contours
{
	int n = ptArray.GetSize();
	int m = 0;
	int x, y;
	oval_Point p;
	SkContour *pCon;
	for (int i = 0; i < n; i++)
	{
		p = ptArray.GetAt(i);
		x = p.x;
		y = p.y;
		int l = getOcpPointNum(y, x);
		if (l <= 0)
			continue;
		ocpPtArray.Add(p);
		for (int j = 0; j < l; j++)
		{
			pCon = getOcpContour(y, x, j);
			bool bAdded = false;
			for (int k = m-1; k >= 0; k--)
			{
				if (ppCons[k] == pCon)
				{
					bAdded = true;
					break;
				}
			}
			if (!bAdded)
			{
				ppCons[m] = pCon;
				m++;
			}
		}
	}
	return m;
}
///////////////////
// extendContourEndByNeighbor()
//	return the point num in pCon, -1: not extend
//	very greedy, just consider the nbr with lowest continuity energy
//	1) consider the ort diff between the new linelet ort and the contour end
//		if too big (>dThOrt2) 
//		or not too big (onl >dThOrt1) but distance very large > pCon->GetLength()/2
//			then not extend
//	2) if dis < 4.5, then just extend
//	3) for profile, if proj < projTh/4 or std < stdTh/4 then not extend
//	4) otherwise extend
int SkSketchGraph::extendContourEndByNeighbor(SkContour* pCon, // the contour
										   int e) // the end
{
	double dThOrt1 = PI/180 * 30;
	double dThOrt2 = PI/180 * 45;
	int nReturn = -1;
	double dMaxNoOrtDis = MAX(6, pCon->GetLength()/10);//max distance without consider orientation

	int n = pCon->GetNeighborNum(e);
//	int m = MIN(3, n);
	for (int i = 0; i < n; i++)
	{//consider all possible nbrs 
		SkContourNeighbor nbr = pCon->GetNeighbor(e, i);

		if ( (nbr.m_dDeltaOrt > dThOrt1 && nbr.m_dDis > MAX(pCon->GetLength()/2, 15) )
			|| nbr.m_dDeltaOrt > dThOrt2)
		{
			continue;
		}//

		SkNode *pPt = (SkNode*)(pCon->GetEndPosition(e));//end node

		int x0 = pPt->m_data.x;
		int y0 = pPt->m_data.y;

		int x = nbr.m_point.x;
		int y = nbr.m_point.y;

		if (x0 == x && y0 == y)
			continue;

		SkNode *pPt1;
		if  (nbr.m_dDis < dMaxNoOrtDis)
		{//the distance is too small, just change it, not adding new point
			int x1, y1;
			if (e==0) 
			{
				pPt1 = pPt->m_pNext;
				x1 = pPt1->m_data.x;
				y1 = pPt1->m_data.y;
				if (x1 == x && y1 == y)
					continue;
				updateResidual(x0, y0, x1,y1, &(pPt->m_prfl), REMOVE);
				removeOccupy(x0, y0, x1, y1, (int)pPt, (int)pPt1, (int)pCon, 0);
				pPt->m_data.setX(x);
				pPt->m_data.setY(y);
				//compute the profile
//				pPt->computeProfile(&m_imgRes, m_nPrflWd);
				pPt->computeProfile(m_pImgOrg, m_nMaxPrflWd, 1, &m_pixCandMat);//no need for profile width
				pPt->thinnerProfile();
				updateResidual(x, y, x1, y1, &(pPt->m_prfl), ADD);
				addOccupy(x, y, x1, y1, (int)pPt, (int)pPt1, (int)pCon, 0);
			}
			else 
			{
				pPt1 = pPt->m_pPrev;
				x1 = pPt1->m_data.x;
				y1 = pPt1->m_data.y;
				if (x1 == x && y1 == y)
					continue;
				updateResidual(x1, y1, x0, y0, &(pPt1->m_prfl), REMOVE);
				removeOccupy(x1, y1, x0, y0, (int)pPt, (int)pPt1, (int)pCon, 0);
				pPt->m_data.setX(x);
				pPt->m_data.setY(y);
				//compute the profile
//				pPt1->computeProfile(&m_imgRes, m_nPrflWd);
				pPt1->computeProfile(m_pImgOrg, m_nMaxPrflWd, 1, &m_pixCandMat);//no need for profile width
				pPt1->thinnerProfile();
				updateResidual(x1, y1, x, y, &(pPt1->m_prfl), ADD);
				addOccupy(x1, y1, x, y, (int)pPt, (int)pPt1, (int)pCon, 0);
			}

			return pCon->GetPointCount();
		}

		// consider the profile of promoted line
		SkContour *pConNbr = nbr.m_pContour;
		SkPoint p, p0;
		p.x = x;
		p.y = y;
		p.m_pCon = pCon;
		p.m_pnBaseID[0] = p.m_pnBaseID[1] = -1;

		p0 = pPt->m_data;
		int nSampleNum = (int)(p.Distance(p0)*2);

		SkConProfile *pPrfl, prfl;
		if (e == 0)
			pPrfl = &(pPt->m_prfl);
		else
			pPrfl = &(pPt->m_pPrev->m_prfl);
//		int nPrflWd = pPrfl->getValidWidth();
		int nPrflWd = pPrfl->getSampleWidth();

		prfl.setScale(m_nScaleImgOrg);
		prfl.setWidthAndMaxSampleNum(nPrflWd, nSampleNum);
		prfl.addLineProfile(p0.x, p0.y, p.x, p.y, &m_imgRes);
//		prfl.computeGaussianModelFromSamples();
		prfl.computeMeanFromValidSamples();
		prfl.normalize();

//		if (pPrfl->projection(prfl.m_gaussPDF.mu) < m_dProjTh/2)
//			continue;
		if (pPrfl->projection(prfl.getSampleMean()) < m_dProjTh/2)
			continue;

//		if (prfl.m_dStd < m_dStdTh/2)
//			continue;
		if (prfl.getProfileStd() < m_dStdTh/2)
			continue;

		pPt1 = pCon->AddPoint(e, p);
		//compute the profile
		if (e == 0)
		{
//			pPt1->computeProfile(&m_imgRes, m_nPrflWd);
			pPt1->m_nMaxPrflWd = nPrflWd;
			pPt1->computeProfile(m_pImgOrg, m_nMaxPrflWd, 1, &m_pixCandMat);
			pPt1->thinnerProfile();
			updateResidual(x, y, x0, y0, &(pPt1->m_prfl), ADD);
			addOccupy(x, y, x0, y0, (int)pPt, (int)pPt1, (int)pCon, 0);
		}
		else
		{
//			pPt->computeProfile(&m_imgRes, m_nPrflWd);
			pPt1->m_nMaxPrflWd = -1;
			pPt->m_nMaxPrflWd = nPrflWd;
			pPt->computeProfile(m_pImgOrg, m_nMaxPrflWd, 1, &m_pixCandMat);
			pPt->thinnerProfile();
			updateResidual(x0, y0, x, y, &(pPt->m_prfl), ADD);
			addOccupy(x0, y0, x, y, (int)pPt, (int)pPt1, (int)pCon, 0);
		}

		return pCon->GetPointCount();
	}//for i;
	return nReturn;
}
///////////////////
// getSortedContoursByLength
//	nIncrease - 1:increase, 0:decrease, -1:no-sort
int SkSketchGraph::getSortedContoursByLength(SkContour**ppCons, int nIncrease)
{
	int n = 0;
	SkContour *pCon;
	pCon = getContourHead();
	int i, j;

	if (nIncrease == 1)
	{
		while (pCon != 0)
		{
			for (i = 0; i < n; i++)
				if (pCon->GetLength() < ppCons[i]->GetLength())
					break;
			for (j = n; j > i; j--)
				ppCons[j] = ppCons[j-1];
			ppCons[i] = pCon;
			n++;
			pCon = pCon->m_pNext;
		}
	}
	else if (nIncrease == 0)
	{
		while (pCon != 0)
		{
			for (i = 0; i < n; i++)
				if (pCon->GetLength() > ppCons[i]->GetLength())
					break;
			for (j = n; j > i; j--)
				ppCons[j] = ppCons[j-1];
			ppCons[i] = pCon;
			n++;
			pCon = pCon->m_pNext;
		}
	}
	else if (nIncrease == -1)
	{
		while (pCon != 0)
		{
			ppCons[n] = pCon;
			n++;
			pCon = pCon->m_pNext;
		}
	}
	return n;

}
/*
///////////////////
// computeContourProfile()
int SkSketchGraph::computeContourProfile()
{
	Matrix<float> *pImgOrg = getImgOrg();

	int wd, ht;
	pImgOrg->GetDimension(ht, wd);

	m_pixCandMat.SetDimension(ht, wd);

	clearPixelCandidateMatrix();
	 
	int nScale = m_nScaleImgOrg;
	assert(nScale > 0);
	m_scaleImgOrg.SetDimension(nScale*ht, nScale*wd);
	skScaleMatrix(pImgOrg, &m_scaleImgOrg, nScale);

	SkContour *pCon;
	pCon = getContourHead();

	while (pCon != 0)
	{
//		pCon->computeProfile(pImgOrg, m_nMaxPrflWd);
		pCon->computeProfile(&m_scaleImgOrg, m_nMaxPrflWd, nScale, &m_pixCandMat);
		pCon = pCon->m_pNext;
	}
	return 0;
}
*/
/*
///////////////////
// refineContourProfile()
int SkSketchGraph::refineContourProfile()
{
	SkContour *pCon;
	pCon = getContourHead();

	while (pCon != 0)
	{
		pCon->refineProfile();
		pCon = pCon->m_pNext;
	}
	return 0;
}
*/
///////////////////
int SkSketchGraph::extendContoursByProfile()
{
//	skSaveMatrixAsBitmap("res0.bmp", m_imgRes, m_dImgMean, m_dImgStd);

	int nConNum = getContourNum();
	SkContour **ppCons = new SkContour*[nConNum];
//	int nIncrease = 1;
	int nIncrease = 0;//why not decreasing? 05-23-2003
	
	getSortedContoursByLength(ppCons, nIncrease);

	for (int i = 0; i < nConNum; i++)
	{
		extendContoursByProfile(ppCons[i]);
	}
	delete []ppCons;

//	skSaveMatrixAsBitmap("res1.bmp", m_imgRes, m_dImgMean, m_dImgStd);
	
	return getContourNum();
}
///////////////////
int SkSketchGraph::extendContoursByProfile(SkContour *pCon)
{
	if (pCon->m_nBaseType == DOG)
		return 0;

	int nExtend;
	for (int e = 0; e < 2; e++)
	{
		while (true)
		{
			nExtend = extendContoursByProfile(pCon, e);
			if (nExtend == 0)
				return 0;//the contour is removed
			if (nExtend != 1)
				break;
		}
	}
	return 0;
}
///////////////////
// extendContoursByProfile()
//  1) get all possible extension points, which is an arc around the free end, which has a distance the same as current sketchlet
//	2) for each point, connect it to the free end "e"
//	3) if parallel inhibited then next point, otherwise consider it as a candidate point
//	4) check the linelet to this candidate point, which could be 0~5 pixels short 
//	5) if the profile of the linelet has a small standard deviation, then not extend 
//	6) if the projection of the new linelet to the old one is smaller than a threshold, then not extend
//	7) consider the projection as the probabilty, weighted by log(length of linelet)
//	8) choose the highest prob in all candidates
//	return 1-extended, 0-removed, -1-keep the same
int SkSketchGraph::extendContoursByProfile(SkContour *pCon, int e)
{
	// the remove is not considered here any more
#ifndef MAX_EXTEND_CHOICE
#define MAX_EXTEND_CHOICE 2048
#endif
	int pnX[MAX_EXTEND_CHOICE];
	int pnY[MAX_EXTEND_CHOICE];
	double pdProb[MAX_EXTEND_CHOICE];

//	double pdEigValue[MAX_CON_PROFILE_WD];
	int wd = m_nImgWd;
	int ht = m_nImgHt;
	int line_wd = 3;

	SkNode *pNode0, *pNode1, *pNode, *pPrev, *pNext;
	SkPoint p0, p1, p;
	SkConProfile *pPrfl, prfl;
	double dOrtUnit = PI/180 * 10;
//	double dEigValueTh = 4.0;

	double dStdTh = m_dStdTh;

	// pNode0 is the free end node
	if (e == 0)
	{
		pNode1 = pCon->GetHeadNode();
		pNode0 = pNode1->m_pNext;
		pPrfl = &(pNode1->m_prfl);
	}
	else if (e == 1)
	{
		pNode1 = pCon->GetTailNode();
		pNode0 = pNode1->m_pPrev;
		pPrfl = &(pNode0->m_prfl);
	}
	else 
		assert(false);
//	int nPrflWd = pPrfl->getValidWidth();
	int nPrflWd = pPrfl->getSampleWidth();//we need consider wider regions

	assert(false);//changed 03-24-2004
	double dError = 0;//= pPrfl->getSampleVarDiagonalAverage();
	if (dError > m_dPrflVarTh)
		return -1;//keep the same

	double dStd = pPrfl->getProfileStd();
	if (dStd < m_dStdTh)
		return -1;//keep the same

//	return -1;

//	SaveResults::saveProfileAsBmp(*pPrfl, "tmpl_prfl", m_dImgMean, m_dImgStd);
	p0 = pNode0->m_data;
	p1 = pNode1->m_data;

	// the remove is not considered here any more
/*
	// remove or not by first eigen value
	if (pCon->GetPointCount() <= 2 && pCon->GetLength() < 20)
	{//short contour, consider to remove it first
//		pPrfl->getEigenValue(pdEigValue);
		if (pPrfl->m_dStd < dStdTh)
		{//remove this contour
			updateResidual(p0.x, p0.y, p1.x, p1.y, pPrfl, REMOVE);
			removeContour(pCon);
			SaveResults::saveSketchGraphEvolvingProcess(*this, line_wd);
			return 0;//removed
		}
	}
*/	//extending

	double dOrt = atan2(double(p1.y-p0.y), double(p1.x-p0.x));

	int l = (int)(p0.Distance(p1));
	int w = 5;
	int x0 = p1.x;
	int y0 = p1.y;
	int x1, y1;
	int nSampleNum;
	double xx, yy;

//	if (abs(x0-58) <= 2 && (abs(191-y0-114) <= 2) )
//		x0 += 0;

	int max_x = -1, max_y = -1;
	double dProb, dMaxProb = -1;
	double proj, max_proj, max_std;
	double projTh = m_dProjTh;//MAX(1.0, pPrfl->projection(pPrfl->m_gaussPDF.mu) / 2.0 );

	double dMaxOrt = PI/180 * 30;
	int n = possibleExtendPoints(l, dMaxOrt, pnX, pnY, pdProb);
	nSampleNum = l * 2;

	for (int i = 0; i < n; i++)
	{
		xx = pnX[i];
		yy = pnY[i];
		::rotate(xx, yy, dOrt);
		x1 = (int)(x0 + xx);
		y1 = (int)(y0 + yy);
		if (x1 < 0 || x1 >= wd || y1 <0 || y1 >= ht)
			continue;
		if (parallelInhibited(x0, y0, x1, y1))
			continue;
		
		prfl.setWidthAndMaxSampleNum(nPrflWd, nSampleNum);
		prfl.addLineProfile(x0, y0, x1, y1, &m_imgRes);
//		prfl.computeGaussianModelFromSamples();
		assert(false);
		prfl.normalize();
//		SaveResults::saveProfileAsBmp(prfl, "tmp_prfl", m_dImgMean, m_dImgStd);
		for (int j = l; j >= l-4; j--)
		{
//			prfl.setSampleNum(j);
//			prfl.computeGaussianModelFromSamples();
			assert(false);
			prfl.normalize();
			if (prfl.getProfileStd() < dStdTh)
				continue;
			proj = pPrfl->projection(prfl.getSampleMean());
			if (proj < projTh)
				continue;

			dProb = proj;//1.0;//pPrfl->m_gaussPDF.prob(prfl.m_gaussPDF.mu);
			dProb *= pdProb[i] * log(double(j));

			if (dProb > dMaxProb)
			{
				dMaxProb = dProb;
//				SaveResults::saveProfileAsBmp(prfl, "tmp_prfl", m_dImgMean, m_dImgStd);
				max_x =  (int)((double)pnX[i] * j/l);
				max_y =  (int)((double)pnY[i] * j/l);
				max_proj = proj;
				max_std = prfl.getProfileStd();
			}
		}
	}
	if (dMaxProb > 0)//&& dMaxProb > 0.0000001)
	{//add one point
//		return -1;
		xx = max_x;
		yy = max_y;
		::rotate(xx, yy, dOrt);
		x1 = (int)(x0 + xx);
		y1 = (int)(y0 + yy);
	
		p.x = x1;
		p.y = y1;
		p.m_pCon = pCon;
		if (e == 0)
		{
			pNode = pCon->AddHeadPoint(p);
			pNode->m_nMaxPrflWd = nPrflWd;
//			pNode->computeProfile(&m_imgRes, m_nPrflWd);
			pNode->computeProfile(m_pImgOrg, m_nMaxPrflWd, 1, &m_pixCandMat);
			pNode->thinnerProfile();
//			skSaveMatrixAsBitmap("pyramid//profile_res.bmp", m_imgRes, m_dImgMean, m_dImgStd);
//			SaveResults::saveProfileAsBmp(pNode->m_prfl, "tmpl_prfl", m_dImgMean, m_dImgStd);
		}
		else
		{
			pNode = pCon->AddTailPoint(p);
			pNode = pNode->m_pPrev; 
			pNode->m_nMaxPrflWd = nPrflWd;
//			pNode->computeProfile(&m_imgRes, m_nPrflWd);
			pNode->computeProfile(m_pImgOrg, m_nMaxPrflWd,1, &m_pixCandMat);
			pNode->thinnerProfile();
//			SaveResults::saveProfileAsBmp(pNode->m_pPrev->m_prfl, "tmpl_prfl", m_dImgMean, m_dImgStd);
		}
		if (m_nShowProcess) 
			skSaveSketchGraphEvolvingProcess(*this, line_wd);
		updateResidual(x0, y0, x1, y1, &(pNode->m_prfl), ADD);
		addOccupy(x0, y0, x1, y1, (int)pNode, (int)pNode0, (int)pCon, 0);		
		return 1;
	}
	return -1;
}

///////////////////
// possibleExtendPoints()
int SkSketchGraph::possibleExtendPoints(int l, double dMaxOrt, int *pnX, int *pnY, double* pdProb)
{
	int n = ::rasterize_circle_part(0, 0, l, pnX, pnY, -dMaxOrt, dMaxOrt);
	for (int i = 0; i < n; i++)
	{
		pdProb[i] = log10(fabs(double(pnX[i])));
	}
	return n;
}
/*
///////////////////
// computeReconstructedAndResidualImage()
int SkSketchGraph::computeReconstructedAndResidualImage()
{
	int wd = m_pImgOrg->cols();
	int ht = m_pImgOrg->rows();

	m_imgRec.SetDimension(ht, wd);
	m_imgRes.SetDimension(ht, wd);

	computeContourProfile();

	saveProfile2Matrix(m_imgRec, wd, ht);

	m_imgRes = *m_pImgOrg - m_imgRec;
	return 0;
}
*/
///////////////////
// saveProfile2Matrix()
//	Save the profile of the graph into a matrix
int SkSketchGraph::saveProfile2Matrix(Matrix<float> &M,//the matrix which saves the points on the graph, 0-on graph, 1-not
							int wd, int ht) //image width and height
{
//	assert(false);//new method applied

	M.SetDimension(ht, wd);
	M.Set(0);

//	m_pixCandMat.SetDimension(ht, wd);

//	clearPixelCandidateMatrix();
/*
	// get all candidates
	SkContour *pCon = getContourHead();
	while (pCon != 0)
	{
		pCon->saveProfile2Matrix(m_pixCandMat);
		pCon = pCon->m_pNext;
	}
*/
	// choose the best candidates
	int profile_pixels = 0;
	int n;
	double data;
	int r, c, i;
	double dMinDis, dMinValue;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			SkPixCandArray &array = m_pixCandMat(r,c);
			n = array.getSize();
			dMinDis = SK_INFINITE;
			dMinValue = SK_INFINITE;
			for (i = 0; i < n; i++)
			{
				SkPixCand &pc = array.getAt(i);
				if (dMinDis > pc.m_dDis)
				{
					dMinDis = pc.m_dDis;
					dMinValue = pc.m_dValue;
				}
			}
			if (dMinValue > SK_INFINITE / 2)
			{
				dMinValue = 0;
				profile_pixels++;
			}
			M(r,c) = dMinValue;

		}
	}

	// output information
	profile_pixels = ht*wd - profile_pixels;
	/////////////////////
	// output the pixel info
	int profile_w = m_nMaxPrflWd;
//	int profile_l = sketchGraph.getContourPixelLength();
//	int point_num = sketchGraph.getPointNum();
	int profile_num = getProfileNum();
	int bound = 3;
	int bound_pixels = 2*ht + 2*(wd-bound);

	char msg[512];
	sprintf(msg, "\xd\xa\xd\xa Synthesize with constraints: \xd\xa");
	skOutput2File(msg);
	sprintf(msg, "             image size:  w*h = %d * %d = %d\xd\xa", wd, ht, wd*ht);
	skOutput2File(msg);
	sprintf(msg, "             constrainted pixels:  \xd\xa");
	skOutput2File(msg);
	sprintf(msg, "                profile occupy pixels    = %d, %.3f%%  \xd\xa", profile_pixels, (double)profile_pixels/(wd*ht)*100);
	skOutput2File(msg);
	sprintf(msg, "                ridge function num     = %d,  width = %d  \xd\xa", profile_num, profile_w);
	skOutput2File(msg);
	sprintf(msg, "                total ridge func size  = %d, %.3f%%  \xd\xa", profile_num*profile_w, (double)(profile_num*profile_w)/(wd*ht)*100);
	skOutput2File(msg);
	sprintf(msg, "                boundary pixels   = %d, %.3f%%  \xd\xa", bound_pixels, (double)bound_pixels/(wd*ht)*100);
	skOutput2File(msg);

	return getContourNum();
}
///////////////////
// clearPixelCandidateMatrix()
int SkSketchGraph::clearPixelCandidateMatrix()
{
	int wd = m_pixCandMat.cols();
	int ht = m_pixCandMat.rows();

	for (int r = 0; r < ht; r++)
	{
		for (int c = 0; c < wd; c++)
		{
			m_pixCandMat(r,c).removeAll();
		}
	}
	return 0;
}

///////////////////
// updateResidual()
int SkSketchGraph::updateResidual(int x0, int y0, int x1, int y1, SkConProfile *pPrfl, int flag)
{
	int wd = m_imgRes.cols();
	int ht = m_imgRes.rows();

	Matrix<float> M(ht, wd);

	M.Zero();
	pPrfl->saveLineProfile(x0, y0, x1, y1, &M);

	if (flag == REMOVE)
		m_imgRes = m_imgRes + M;
	else
		m_imgRes = m_imgRes - M;

	return 0;
}
///////////////////
// getPointNum()
int SkSketchGraph::getPointNum()
{
	int n = 0;
	SkContour *pCon;
	pCon = getContourHead();

	while (pCon != 0)
	{
		n += pCon->getPointNum();
		pCon = pCon->m_pNext;
	}
	return n;
}
///////////////////
// getProfileNum()
int SkSketchGraph::getProfileNum()
{
	int n = 0;
	SkContour *pCon;
	pCon = getContourHead();

	while (pCon != 0)
	{
		n += pCon->getProfileNum();
		pCon = pCon->m_pNext;
	}
	return n;
}
///////////////////
// removeNoisyContour()
//	Consider the image match
//	If the profile standard deviation is less than the threshold, then remove it
//	return 1: removed, 0: not
int SkSketchGraph::removeNoisyContour(SkContour * pCon)
{
	
	if (pCon->GetPointCount() > 2 || pCon->GetLength() > 20)
		return 0;
	int line_wd = 3;
	double dStdTh = m_dStdTh;

	SkNode *pNode0 = pCon->GetHeadNode();
	SkNode *pNode1 = pNode0->m_pNext;
	SkConProfile *pPrfl = &(pNode0->m_prfl);
	SkPoint p0 = pNode0->m_data;
	SkPoint p1 = pNode1->m_data;

	if (pPrfl->getProfileStd() < dStdTh )
//		|| pPrfl->getSampleVarDiagonalAverage() > m_dPrflVarTh)
	{//remove this contour
		updateResidual(p0.x, p0.y, p1.x, p1.y, pPrfl, REMOVE);
		removeOccupy(p0.x,  p0.y, p1.x, p1.y, (int)pNode0, (int)pNode1, (int)pCon, 0);
		removeContour(pCon);
		if (m_nShowProcess)
			skSaveSketchGraphEvolvingProcess(*this, line_wd);
		return 1;//removed
	}

	/*
//	if (pCon->GetPointCount() > 2 || pCon->GetLength() > 20)
//		return 0;
	int line_wd = 3;
	double dStdTh = m_dStdTh;

	SkNode *pNode0 = pCon->GetHeadNode();
	SkNode *pNode1 = pNode0->m_pNext;
	SkConProfile *pPrfl = &(pNode0->m_prfl);
	SkPoint p0 = pNode0->m_data;
	SkPoint p1 = pNode1->m_data;

//	if (pPrfl->getProfileStd() < dStdTh)
	if (pCon->GetLength() < 50)
	{//remove this contour
		updateResidual(p0.x, p0.y, p1.x, p1.y, pPrfl, REMOVE);
		removeOccupy(p0.x,  p0.y, p1.x, p1.y, (int)pNode0, (int)pNode1, (int)pCon, 0);
		removeContour(pCon);
		if (m_nShowProcess)
			skSaveSketchGraphEvolvingProcess(*this, line_wd);
		return 1;//removed
	}
	*/
	return 0;
}
bool SkSketchGraph::parallelInhibited(int x0, int y0, int x1, int y1)
{
	bool inh = false;
	double ocp_th = 0.3;

	int pnX[4096], pnY[4096];
	int line_length = ::rasterize_line_segment(x0, y0, x1, y1, pnX, pnY);
	assert(line_length <= 4096);

	int v_x0, v_y0, v_x1, v_y1;
	int l = (int)m_dInhibitionDis;
	::verticalLine(x0, y0, x1, y1, v_x0, v_y0, v_x1, v_y1, l);

	int n = ::rasterize_line_segment(v_x0, v_y0, v_x1, v_y1, pnX, pnY);
	assert(n <= 4096);
	int dx, dy;
	int p_x0, p_y0, p_x1, p_y1;//the parallel line
	for (int i = 0; i < n; i++)
	{
		dx = pnX[i];
		dy = pnY[i];
		p_x0 = x0 + dx;
		p_y0 = y0 + dy;
		p_x1 = x1 + dx;
		p_y1 = y1 + dy;

		int num_ocp = numOfOccupyOnOneLine(p_x0, p_y0, p_x1, p_y1);

		if ((double)num_ocp / line_length > ocp_th)
			return true;
	}
	return inh;
}

int SkSketchGraph::numOfOccupyOnOneLine(int x0, int y0, int x1, int y1)
{
	int s = 0;
	int pnX[4096], pnY[4096];
	int n = ::rasterize_line_segment(x0, y0, x1, y1, pnX, pnY);
	assert(n <= 4096);

	int wd = m_pImgOrg->cols();
	int ht = m_pImgOrg->rows();

	int x, y;
	for (int i = 0; i < n; i++)
	{
		x = pnX[i];
		y = pnY[i];
		if (x < 0 || x >= wd || y < 0 || y >= ht)
			continue;
		if (getOcpPointNum(y, x) >= 1)
			s++;
	}
	return s;
}

/*
///////////////////
// saveResidualErrorBaseByBase()
int SkSketchGraph::saveResidualErrorSketchBySketch(const char *name)
{
	int nBound = 5;
	m_imgRes = *m_pImgOrg;
	double dResErrorChg=0;
	double pdResError[100000];
	pdResError[0] = skSquareSumOfMatrix(&m_imgRes, nBound);

	int wd, ht;
	int xm, ym;//center of Matrix M
	wd = ht = 101;
	xm = ym = wd/2;//center of M
	Matrix<float> M(ht, wd);

	int x0, y0, x1, y1;//start and end center of the sketch
	int xc, yc;//center of the sketch
	int dx, dy;//displacement from xc to xm

	int n = 0;

	SkContour *pCon = getContourHead();
	while (pCon != 0)
	{
		SkNode *pNode = pCon->GetHeadNode();
		SkNode *pNext;
		while (pNode != 0 && pNode->m_pNext != 0)
		{
			pNext = pNode->m_pNext;
			x0 = pNode->m_data.x;
			y0 = pNode->m_data.y;
			x1 = pNext->m_data.x;
			y1 = pNext->m_data.y;//real start and end point

			xc = (x0+x1)/2;
			yc = (y0+y1)/2;//center of the line of real points

			dx = xc - xm;
			dy = yc - ym;

			M.Set(0);
			if (pCon->m_nBaseType != DOG)
				pNode->m_prfl.saveLineProfile(x0-dx, y0-dy, x1-dx, y1-dy, &M, &m_pixCandMat, dx, dy);
			else
				pNode->m_prfl.saveDoGProfile(x0-dx, y0-dy, x1-dx, y1-dy, &M, &m_pixCandMat, dx, dy);
			pNode = pNode->m_pNext;

			dResErrorChg = skSubtractImageAndReturnSquareSumChange(&m_imgRes, &M, dx, dy);
			n++;
			pdResError[n] = pdResError[n-1] + dResErrorChg;

//			char msg[512];
//			sprintf(msg, "n=%d, res err change = %.2f\n", n, dResErrorChg);
//			skOutput(msg);

		}
		pCon = pCon->m_pNext;
	}
	::saveArray(pdResError, n, name);
	return 0;
}
*/
///////////////////
// skSaveSketchGraph()
//	save the graph of sketch
int skSaveSketchGraph(SkSketchGraph &sketchGraph, const char *pPreName, const char *pProjectName, const char *dir, int wd, int ht, int line_wd)
{
	char name[512];
	double dImgMean = 255;
	double dImgStd = -1;

//	char dir[512];
//	sprintf(dir, "pyramid");
//	_mkdir(dir);

	Matrix<float> M;
	sketchGraph.save2Matrix(M, wd, ht, line_wd);

	sprintf(name, "%s//%s_%s_sketch.bmp", dir, pProjectName, pPreName);
	skSaveMatrixAsBitmap(name, M, dImgMean, dImgStd);

	// save profile and constraints matrix

	if (line_wd ==  3)
	{
		line_wd = sketchGraph.m_nMaxPrflWd;
		sketchGraph.save2Matrix(M, wd, ht, line_wd);
		sprintf(name, "%s//%s_%s_con.bmp", dir, pProjectName, pPreName);
		skSaveMatrixAsBitmap(name, M, dImgMean, dImgStd);

		dImgMean = sketchGraph.m_dImgMean;
		dImgStd = sketchGraph.m_dImgStd;

		sketchGraph.saveProfile2Matrix(M, wd, ht);
		sprintf(name, "%s//%s_%s_pro.bmp", dir, pProjectName, pPreName);
		skSaveMatrixAsBitmap(name, M, dImgMean, dImgStd);
	}

	return 0;
}

///////////////////
int skSaveSketchGraphEvolvingProcess(SkSketchGraph &sketchGraph, int line_wd)
{
	sketchGraph.m_nProcessSteps++;

	int i = sketchGraph.m_nProcessSteps;

	char dir[512];
	sprintf(dir, "BupBsp");
	_mkdir(dir);

	int wd = sketchGraph.getImgWidth();
	int ht = sketchGraph.getImgHeight();

	Matrix<float> M;
	sketchGraph.save2Matrix(M, wd, ht, line_wd);

	char name[512], pPreName[512];
	::changeNumTo4Char(pPreName, i);

	sprintf(name, "%s\\%s_g_%s.bmp", dir, sketchGraph.m_pchProjName, pPreName);
	double dImgMean = 255;
	double dImgStd = -1;
	skSaveMatrixAsBitmap(name, M, dImgMean, dImgStd);
	return 0;
}

///////////////////
// skSaveVerticesInSketchGraph()
//	save vertices in the skecth graph
int skSaveVerticesInSketchGraph(SkSketchGraph &sketchGraph, const char *pPreName, int line_wd)
{
	char dir[512];
	sprintf(dir, "skps");
	_mkdir(dir);

	int wd = sketchGraph.getImgWidth();
	int ht = sketchGraph.getImgHeight();

	Matrix<float> M;
	sketchGraph.saveVertices2Matrix(M, wd, ht, line_wd);

	char name[512];
	sprintf(name, "%s\\%s.bmp", dir, pPreName);
	double dImgMean = 255;
	double dImgStd = -1;
	skSaveMatrixAsBitmap(name, M, dImgMean, dImgStd);

	return 0;
}
///////////////////
// skSaveProfileAsBmp()
int skSaveProfileAsBmp(SkConProfile &prfl, char *name, double dImgMean, double dImgStd)
{
	static int nn = 0;
	nn++;
	if (nn == 1000)
		nn = 1;
	char no[32];
	if (nn < 10)
		sprintf(no, "00%d", nn);
	else if (nn < 100)
		sprintf(no, "0%d", nn);
	else 
		sprintf(no, "%d", nn);

	int w = prfl.getSampleWidth();
	int nSampleNum = prfl.getSampleNum();

	int nSpace = 3;
	int wd = nSampleNum + 2*nSpace;
	int ht = 2*w + 3*nSpace;

	Matrix<float> M(ht, wd);
	M.Set(255.0);

	//save average profile
	int x0 = nSpace; 
	int y0 = nSpace + w/2;
	int x1 = nSpace + nSampleNum;
	int y1 = y0;
	prfl.saveLineProfile(x0, y0, x1, y1, &M);

	//save samples
	int r0 = w + 2*nSpace;
	int c0 = nSpace;
	for (int i = 0; i < nSampleNum; i++)
	{
		for (int j = 0; j < w; j++)
		{
			M(j+r0, i+c0)  = prfl.m_sample(i,w-1-j);
		}
	}

	char dir[512];
	sprintf(dir, "Profile");
	_mkdir(dir);

	char whole_name[512];
	sprintf(whole_name, "%s\\%s_%s.bmp", dir, name, no);
	::skSaveMatrixAsBitmap(whole_name, M, dImgMean, dImgStd);


	return 0;
}

///////////////////
// skSaveVerticesAsTexton()
int skSaveVerticesAsTexton(SkSketchGraph &sketchGraph, char* proj)
{
	char dir[512];
	sprintf(dir, "Texton");
	_mkdir(dir);

//	sketchGraph.computeReconstructedAndResidualImage();
	assert(false);//computeReconstructedAndResidualImage() CHANGed, 09-13-2003

	int wd = sketchGraph.getImgWidth();
	int ht = sketchGraph.getImgHeight();
	double dImgMean = sketchGraph.m_dImgMean;
	double dImgStd = sketchGraph.m_dImgStd;
//	int line_wd = sketchGraph.m_nPrflWd;
	Matrix<float> &rec = sketchGraph.m_imgRec;

	int w = 51;//g_nAtomSize;
	int h = w;

	Matrix<SkPixCandArray> M;
	M.SetDimension(ht, wd);

	Matrix<float> T;
	T.SetDimension(h, w);

	int *pnX = new int[ht*wd];
	int *pnY = new int[ht*wd];
	char name[512];
	char no[32];

	int x, y, x0, y0, x1, y1, x2, y2;

	int n, i, j, k;
	SkNode *pPtNode, *pNbr;
	int pos = sketchGraph.m_vertMap.getFirstPos();
	SkVertex *pVertex;
	SkContour *pCon;
	SkContour *pCons[10000];
	int nn = 0;
	double dAngle1, dAngle2;
	double dAngleTh = PI/180 * 15;
	while (pos >= 0)
	{
		pVertex = sketchGraph.m_vertMap.getNext(pos);
		k = pVertex->getDegree();
		x0 = pVertex->x;
		y0 = pVertex->y;

		for (i = 0; i < k; i++)
		{
			pPtNode = pVertex->getPtNode(i);
			assert(pPtNode != 0);
			pPtNode->saveProfile2Matrix(M);
			x1 = pPtNode->m_data.x;
			y1 = pPtNode->m_data.y;
			if (x1 - x0 == 0)
				dAngle1 = PI/2;
			else
				dAngle1 = atan((double)(y1-y0)/(x1-x0));
			//use several other nodes
			pNbr = pPtNode->m_pPrev;
			if (pNbr != 0)
			{
				x2 = pNbr->m_data.x;
				y2 = pNbr->m_data.y;
				if (x2 - x0 == 0)
					dAngle2 = PI/2;
				else
					dAngle2 = atan((double)(y2-y0)/(x2-x0));
				if (fabs(dAngle1-dAngle2) <= dAngleTh)
					pNbr->saveProfile2Matrix(M);
			}
			pNbr = pPtNode->m_pNext;
			if (pNbr != 0)
			{
				x2 = pNbr->m_data.x;
				y2 = pNbr->m_data.y;
				if (x2 - x0 == 0)
					dAngle2 = PI/2;
				else
					dAngle2 = atan((double)(y2-y0)/(x2-x0));
				if (fabs(dAngle1-dAngle2) <= dAngleTh)
					pNbr->saveProfile2Matrix(M);
			}
		}
/*
		k = sketchGraph.getOcpPointNum(y0, x0);
		int n = 0;
		for (i = 0; i < k; i++)
		{
			pCon = sketchGraph.getOcpContour(y0, x0, i);
			bool bExist = false;
			for (j = 0; j < n; j++)
			{
				if (pCon == pCons[j])
				{
					bExist = true;
					break;
				}
			}
			if (!bExist)
			{ 
				pCons[n] = pCon;
				n++;
			}
		}
		for (j = 0; j < n; j++)
		{
			pCon = pCons[j];
			pCon->saveProfile2Matrix(M);
		}
*/
		//copy 
		T.Zero();
		for (int dr = -h/2; dr <= h/2; dr++)
		{
			y = y0 + dr;
			if (y < 0 || y >= ht)
				continue;
			for (int dc = -w/2; dc <= w/2; dc++)
			{
				x = x0 + dc;
				if (x < 0 || x >= wd)
					continue;
				if (M(y,x).GetSize() != 0)
					T(dr+h/2, dc+w/2) = rec(y,x);
			}
		}
		nn++;
		if (nn < 10)
			sprintf(no, "00%d", nn);
		else if (nn < 100)
			sprintf(no, "0%d", nn);
		else 
			sprintf(no, "%d", nn);
		sprintf(name, "%s//%s_Texton_%s.bmp", dir, proj, no);
		::skSaveMatrixAsBitmap(name, T, dImgMean, dImgStd);

	}
	delete []pnX;
	delete []pnY;
	return sketchGraph.getContourNum();
}
///////////////////
// skSaveProfilesAsBases()
int skSaveProfilesAsBases(SkSketchGraph &sketchGraph, const char* proj)
{
	char dir[512];
	sprintf(dir, "profile");
	_mkdir(dir);

	int nPrflNum = sketchGraph.getProfileNum();
	int prfl_wd = sketchGraph.m_nMaxPrflWd;

	if (nPrflNum <= 0)
		return 1;

	Matrix<double> prfl_samples(nPrflNum, prfl_wd);
	Matrix<double> prfl_mean;

	double dImgMean = sketchGraph.m_dImgMean;
	double dImgStd = sketchGraph.m_dImgStd;

	char name[512], pchNCon[32], pchNPt[32];

	Matrix<float> S, P;

	int r0, c0, i, j;
	int x0, y0, x1, y1;
	int nSpace = 0;
	int wd, ht, nSampleNum, w;
	int n_con, n_pt;
	n_con = 0;
	SkContour *pCon = sketchGraph.getContourHead();
	int nPrflNo = 0;
	int nInvalidPrflNo = 0;
	while (pCon != 0)
	{
		n_con++;
		::changeNumTo3Char(pchNCon, n_con);
		n_pt = 0;
		SkNode *pNode = pCon->GetHeadNode();
		while (pNode != 0 && pNode->m_pNext != 0)
		{
			n_pt++;
			::changeNumTo3Char(pchNPt, n_pt);

			SkConProfile &prfl = pNode->m_prfl;
			w = prfl.getValidWidth();
			nSampleNum = prfl.getSampleNum();
			wd = nSampleNum + 2*nSpace;
			ht = w + 2*nSpace;

			S.SetDimension(ht, wd);
			S.Set(0);
			r0 = nSpace;
			c0 = nSpace;
			for (i = 0; i < nSampleNum; i++)
			{
				for (j = 0; j < w; j++)
				{
					S(j+r0, i+c0)  = prfl.m_sample(i,w-1-j);
				}
			}
			sprintf(name, "%s\\%s_prfl_org_%s_%s.bmp", dir, proj, pchNCon, pchNPt);
//			::skSaveMatrixAsBitmap(name, S, dImgMean, dImgStd);
			
			P.SetDimension(ht, wd);
			P.Set(0);
			x0 = nSpace; 
			y0 = nSpace + w/2;
			x1 = nSpace + nSampleNum;
			y1 = y0;
			prfl.saveLineProfile(x0, y0, x1, y1, &P);
			sprintf(name, "%s\\%s_prfl_mean_%s_%s.bmp", dir, proj, pchNCon, pchNPt);
//			::skSaveMatrixAsBitmap(name, P, dImgMean, dImgStd);

			prfl_mean = prfl.getSampleMean();
			prfl_mean = prfl_mean.T();
			if (prfl_mean.GetMax() < SK_INFINITE/2)
			{
				skCopyImage(&prfl_mean, &prfl_samples, 0, -nPrflNo);
				nPrflNo++;
			}
			else
			{
				nInvalidPrflNo++;
			}
			pNode = pNode->m_pNext;
		}
		pCon = pCon->m_pNext;
	}

	if (nPrflNo > 0)
	{
		// not all primitives are valid, 
		Matrix<double> temp = prfl_samples;
		prfl_samples.SetDimension(nPrflNo, prfl_wd);
		for (int y = 0; y < nPrflNo; y++)
			for (int x = 0; x < prfl_wd; x++)
				prfl_samples(y,x) = temp(y,x);

		sprintf(name, "prfl.txt", dir, proj);
		skSaveMatrix(name, prfl_samples);
	}
	return 0;
}
///////////////////
// skGetMinDistanceOfPixelCandidate()
double skGetMinDistanceOfPixelCandidate(Matrix<SkPixCandArray> &M, int x, int y, double &dValue)
{
	SkPixCandArray &array = M(y,x);
	int n = array.GetSize();
	double dMinDis = SK_INFINITE;
	dValue = SK_INFINITE;
	double dDis;
	for (int i = 0; i < n; i++)
	{
		dDis = array[i].m_dDis;
		if (dDis < dMinDis)
		{
			dMinDis = dDis;
			dValue = array[i].m_dValue;
		}
	}
	return dMinDis;
}
///////////////////
// skGetMinDistanceOfPixelCandidate()
double skGetMixingValueOfPixelCandidate(Matrix<SkPixCandArray> &M, int x, int y)
{
	SkPixCandArray &array = M(y,x);
	int n = array.GetSize();
	double dDis;
	double dWeight, dSumWeight = 0;
	double dValue, dSumValue = 0;
	for (int i = 0; i < n; i++)
	{
		dDis = array[i].m_dDis;
		dValue = array[i].m_dValue;
		dWeight = 1.0/(1.0 + dDis);
		dSumValue += dValue * dWeight;
		dSumWeight += dWeight;
	}
	if (dSumWeight != 0)
		dValue = dSumValue / dSumWeight;
	else
		dValue = 0;
	return dValue;
}
///////////////////
int	skComputeImageFromPixCandByMinDis(Matrix<SkPixCandArray> &cand,//pixel candidates
							  Matrix<float> &rec) //reconstructed image
{
	int ht, wd;
	cand.GetDimension(ht, wd);
	rec.SetDimension(ht, wd);
	rec.Set(0);

	double dMinDis, dValue;
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			dMinDis = skGetMinDistanceOfPixelCandidate(cand, c, r, dValue);
			rec(r,c) = dValue;
		}
	}
	return 0;
}

///////////////////
int	skComputeImageFromPixCandByMixing(Matrix<SkPixCandArray> &cand,//pixel candidates
							  Matrix<float> &rec) //reconstructed image
{
	int ht, wd;
	cand.GetDimension(ht, wd);
//	rec.SetDimension(ht, wd);
//	rec.Set(0);

	double dValue;
	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			dValue = skGetMixingValueOfPixelCandidate(cand, c, r);
			rec(r,c) = dValue;
		}
	}
	return 0;
}