#ifndef _SK_SKETCH_GRAPH_H_
#define _SK_SKETCH_GRAPH_H_

#include "skTemplate.h"
#include "skMatrix.h"
#include "skSparseArray.h"
#include "skMap.h"


struct __SKPOSITION { };
typedef __SKPOSITION* SKPOSITION;

#ifndef MAX_CON_NBR_NUM
#define MAX_CON_NBR_NUM 10
#endif

#ifndef MIN_CNT_STRENGTH
#define MIN_CNT_STRENGTH -1E10
#endif

#ifndef MAX_CNT_STRENGTH
#define MAX_CNT_STRENGTH 1E10
#endif


#ifndef PI
#define PI 3.14159265
#endif

enum SkContourType { SKPOINT, SKLINE, SKARC, SKCURVE };

class SkContour;

class SkPoint
{
public:
	int x, y;
	int m_pnBaseID[2];//one point connects two bases,
	int m_pnVertexID[2];//one point may connects two vertices
	SkContour *m_pCon;

//	SkPoint( SkPoint p ) { x = p.x; y = p.y; }
	SkPoint( int x0=0, int y0=0 ) { x = x0; y = y0;
				m_pnBaseID[0]=m_pnBaseID[1]=m_pnVertexID[0]=m_pnVertexID[1]=-1; m_pCon=0;}
	SkPoint operator + ( SkPoint p );
	bool operator == ( SkPoint p );
	bool operator != ( SkPoint p );
	double Distance(SkPoint p);
	void setX(int x0) { x = x0; }
	void setY(int y0) { y = y0; }
};
class SkContourNeighbor
{
public:
	double m_dContStrength;//continuity strength
	double m_dDeltaOrt;//oritentation with the end angle difference
	double m_dDeltaAngle;//end angle difference
	double m_dDis;//distance of the point
	SkPoint m_point;//the point which will be connected

	int m_nEnd;// flag of the strength is to the head of m_pContour or the tail of m_pContour, 0-head,1-tail,2-inside
	SkContour * m_pContour;
//	SkPoint m_Point;//the point connected to that contour
public:
	SkContourNeighbor();
	double computeContStrength();
};

class SkPixCand
{
public:
	double m_dDis;
	double m_dValue;
	int m_nBhID;
public:
	SkPixCand &operator = (SkPixCand &c);
	SkPixCand &operator = (double v);
	bool operator < (SkPixCand &c);
	bool operator > (SkPixCand &c);
	SkPixCand operator + (SkPixCand &c);
	SkPixCand operator + (double v);
	SkPixCand &operator += (SkPixCand &c);
	SkPixCand operator - (SkPixCand &c);
	SkPixCand operator * (double v);
	SkPixCand operator * (SkPixCand c);
	SkPixCand operator / (const SkPixCand c);
	SkPixCand operator / (double v);

	bool	operator == (SkPixCand &c);
	bool	operator != (SkPixCand &c);
};

#define SkPixCandArray oval_Array<SkPixCand>


#ifndef MAX_CON_PROFILE_WD
#define MAX_CON_PROFILE_WD 51
#endif

#ifndef PRFL_WD
#define PRFL_WD 13
#endif 

class SkEdgeRidgeModel
{
#define SK_MAX_PRFL_ERMODEL 128

private:
	// save intermediate values for fast access
	double m_pdDeriv[SK_MAX_PRFL_ERMODEL];
	int m_pnSegPos[SK_MAX_PRFL_ERMODEL];
	double m_pdSegMean[SK_MAX_PRFL_ERMODEL];
	int m_nSegNum;
	int m_nCenterSeg;
	int m_nWholeDataLength;
public:
	double m_v[SK_MAX_PRFL_ERMODEL];
	double m_dStart, m_dEnd;//start and end of the edge or ridge, [start,end], closed interval
	double m_pdEdges[4];//two edges points for edges, four for ridges
	double m_pdInt[3];//intensity levels, 2 for edges, 3 for ridges
	int m_nType;//0: edge, 1: ridge
public:
	int build(Matrix<double> &v);//v is a column vector use (i,0) to access each member
	int checkRidge2Edge();//check if ridge could be better modeled by an edge, if so change it
	double getCenterPosition() { if (m_nType == 1) return (m_pdEdges[1]+m_pdEdges[2])/2; 
		else if (m_nType == 0) return (m_pdEdges[1]+m_pdEdges[0])/2; 
		else return (m_dStart+m_dEnd)/2; }
};

class SkJunction;
//class SkProfileList;

class SkConProfile
{
#define SK_MAX_COVER_JNCT_NUM 10
public:
	SkConProfile *m_pNext, *m_pPrev;
	SkConProfile *m_pTmplPrfl;//the template profile

	void *m_pPrflList;//pointer to the prfl list which it is belonging to
	int m_x0, m_y0;
	int m_nRotCenterSampleNo;

	double m_dOrt;//[0,2*PI)
	Matrix<double> m_sample;//row vector for each sample
	SkEdgeRidgeModel m_erModel;//edge, ridge model
	Matrix<double> m_smpCenterOff;//sample center position offset

	int m_nValidSmpStart, m_nValidSmpEnd;
	Matrix<double> m_mu;//mu, mean of the samples (sometime, valid samples)
	int m_nType;//0 - strong, 1 - free
	SkJunction* m_pJunct[SK_MAX_COVER_JNCT_NUM];
	int m_nCoverJunctNum;
private:
	Matrix<double> m_normMu;//normalized mu, m_gaussPDF save un-normalized mu and variance
							//column vector, call by m_normMu(i,0)
	int m_nScale;
	double m_dMean, m_dStd;//mean and std of profile
	int m_nValidWidth;//a profile is cut out, but only a valid part (centerd) is used

public:
	// added or used on 03-22-2004
	SkConProfile() { m_nScale = 1; m_pNext = 0; m_pPrev = 0; m_nCoverJunctNum = 0; m_pPrflList=0;}
	void setRotCenter(int x, int y) { m_x0 = x; m_y0 = y; }
	void setRotCenterSampleNo(int n) { m_nRotCenterSampleNo = n; }
	void setOrt(double dOrt) { m_dOrt = dOrt; }//[0,2*PI)
	void setTemplateProfile(SkConProfile *p) { m_pTmplPrfl = p; }
	void setValidSampleStartAndEnd(int s, int e) { m_nValidSmpStart = s; m_nValidSmpEnd = e; }
	void setType(int t) { m_nType = t; }
	int addSample(Matrix<double> &mat);
	int computeMeanFromValidSamples();
	int buildEdgeRidgeModel();
	int findCenter4EachSample(Matrix<double> &mu);//for a given profile mu
	Matrix<double> &getSampleMean() { return m_mu;}//column vector, call by m_mu(i,0)
	Matrix<double> getResidualImage(Matrix<double> *pMu);
	Matrix<double> getPartResidualImage(Matrix<double> *pMu, int nPrimStart, int nPrimEnd, int nSmpStart, int nSmpEnd);
	int updateSampleAndMeanByCenterOffset();
	int getAbsoluteSampleCenter(double *pdX, double *pdY);
	int getAbsoluteValidSamplePixels(double *pdX, double *pdY, int start, int end);
	void setSampleCenterAsValidStartOrEnd(int e);
	Matrix<double> getPartSample(int nPrimStart, int nPrimEnd, int nSmpStart, int nSmpEnd);
	void getTwoValidEndPoints(double &x1, double &y1, double &x2, double &y2);
	void getValidEndPoints(double &x, double &y, int e);
	int getCoverJunctNum() { return m_nCoverJunctNum; }
	int addOneJunction(SkJunction *p);
	int removeOneJunction(SkJunction *p);
private:

	int addProfile(int m, double *pdPrfl);
public:
	int addLineProfile(int x0, int y0, int x1, int y1, Matrix<float> *pImgOrg);
	int addDoGProfile(int x0, int y0, int x1, int y1, Matrix<float> *pImgOrg);
	int addLineProfile(int x0, int y0, int x1, int y1, Matrix<float> *pImgOrg, Matrix<SkPixCandArray>&M);
	int addDoGProfile(int x0, int y0, int x1, int y1, Matrix<float> *pImgOrg, Matrix<SkPixCandArray>&M);
	int saveLineProfile(int x0, int y0, int x1, int y1, Matrix<float> *M,  Matrix<SkPixCandArray>* pPixCandMat=0, int dx_2M=0, int dy_2M=0);
	int saveDoGProfile(int x0, int y0, int x1, int y1, Matrix<float> *M,  Matrix<SkPixCandArray>* pPixCandMat=0, int dx_2M=0, int dy_2M=0);
	int saveLineProfile(int x0, int y0, int x1, int y1, Matrix<SkPixCandArray>&M);
	int saveDoGProfile(int x0, int y0, int x1, int y1, Matrix<SkPixCandArray>&M);
	int setWidthAndMaxSampleNum(int w, int n);
	int getProfile(int m, double *pdPrfl);
	int clear();

	int getSampleNum() { return m_sample.cols(); }

	double projection(Matrix<double> data); 
	int normalize();

	int getSampleWidth() { return m_sample.cols(); }
	int getValidWidth() { return m_nValidWidth; }
	int	maxOffsetOfSamples(int &nWhich);
	int thinnerProfile();

	double getProfileStd() { return m_dStd; }
	void setScale(int s) { m_nScale = s; }


};


class SkNode
{
public:
	SkPoint m_data;
	SkNode *m_pNext;
	SkNode *m_pPrev;
	SkNode(){ m_pNext = 0; m_pPrev = 0; m_nMaxPrflWd = -1;}
	SkConProfile m_prfl;
	int m_nMaxPrflWd;


public:
	int computeProfile(Matrix<float> *pImgOrg, int nMaxPfWd, int nScale,Matrix<SkPixCandArray>* pPixCandMat);
	SkNode* shortenProfile(Matrix<float> *pImgOrg, int nMaxPfWd);
	int computeProfileDoG(Matrix<float> *pImgOrg, int nMaxPfWd, int nScale, Matrix<SkPixCandArray>* pPixCandMat);
	int saveProfile2Matrix(Matrix<SkPixCandArray> &M);
	int saveProfileDoG2Matrix(Matrix<SkPixCandArray> &M);
	int thinnerProfile();
	int getProfileSampleWidth() { return m_prfl.getSampleWidth(); }
	int getProfileValidWidth() { return m_prfl.getValidWidth();}
};
class SkPointList
{
	// attributes
public:
	SkNode* m_pNodeHead;
	SkNode* m_pNodeTail;
	int m_nSize;
	double m_dLength;

	// operations
public:
	SkPointList();
	~SkPointList();
	void RemoveAll();
	SkNode * AddHead( SkPoint p );
	SkNode * AddTail( SkPoint p );
	SkNode * Add( int nEnd, SkPoint p );
	int GetCount(){return m_nSize;}
	int GetSize() {return m_nSize;}
	double GetLength(){return m_dLength;}
	bool IsEmpty(){return ( m_nSize == 0 );}
	SkPoint& GetHead(){return m_pNodeHead->m_data;}
	SkPoint GetHead() const{return m_pNodeHead->m_data;}
	SkPoint& GetTail(){return m_pNodeTail->m_data;}
	SkPoint GetTail() const{return m_pNodeTail->m_data;}
	SkPoint GetHeadOrTail(int end) const{if( end == 0 )return GetHead();
		else return GetTail();}
	SKPOSITION GetHeadPosition() const{return (SKPOSITION)m_pNodeHead;}
	SKPOSITION GetTailPosition() const {return (SKPOSITION)m_pNodeTail;}
	SkPoint GetNext(SKPOSITION& rPosition);
	SkPoint GetPrev(SKPOSITION& rPosition); // return *Position--
	SkPoint RemoveHead();
	SkPoint RemoveTail();

	SkNode * GetHeadNode() { return m_pNodeHead; }
	SkNode * GetTailNode() { return m_pNodeTail; }
};

class SkConNbrManager
{
public:
	SkContourNeighbor m_ppEndNbrArray[2][MAX_CON_NBR_NUM];// The array which is save its neighbor to the head point
	int m_pnEndNbrNum[2];
public:
	SkConNbrManager(){	m_pnEndNbrNum[0] = m_pnEndNbrNum[1] = 0;}
	int AddNeighbor( int end, SkContourNeighbor& conNbr );
	void ClearNeighbor( int end );
	int GetNeighborNum( int end );
	SkContourNeighbor GetNeighbor( int end, int i );
	void RemoveAllNeighbors();
};

#define END_FREE -1
#define END_CLOSEd 0
#define END_REACH_OTHER_CON 1
#define END_REACH_BOUND 2


class SkContour
{
private:
	SkPointList	m_PointList;// the list of the points in this contour
public:
	SkContour *m_pNext;
	SkContour *m_pPrev;// SkContour is a double linked list
	SkConNbrManager m_conNbrManager;
	SkContourType	m_contourType;//contour type: point, line, arc, curve
	double		m_pdParam[10];// parameters for this curve
	int			m_pnEndProperty[2];//[0]head,[1]tail, -1-free, 0-reach itself, 1-reach other contour, 2-reach bounary
	int			m_nBaseType;//DOOG, DOOG1, DOG, 
	int			m_nBaseHalfSize;
private:
	SkContour & operator = ( SkContour & c ){return *this;};//diable implictly copy
public:
	SkContour();
	SkContour( SkPoint p1, SkPoint p2 );
	//neighbor
	SkPoint GetEndPoint(int end);
	void AddNeighbor( int end, SkContourNeighbor &ConNeighbor );
	void ClearNeighbor(int end);
	int GetNeighborNum(int end);
	SkContourNeighbor GetNeighbor( int end, int i );
	void RemoveAllNeighbors();

	int Sign( int n );
	//points
	void RemoveAllPoint() { m_PointList.RemoveAll();};
	SkNode* AddHeadPoint( SkPoint p ) { return m_PointList.AddHead(p);}
	SkNode* AddTailPoint( SkPoint p ) { return m_PointList.AddTail(p);}
	SkNode* AddPoint( int nEnd, SkPoint p ){ return m_PointList.Add(nEnd,p);}
	inline int GetPointCount(){return m_PointList.GetCount();}
	inline bool IsEmpty(){return m_PointList.IsEmpty();}
	SkPoint& GetHeadPoint(){return m_PointList.GetHead();}
	SkPoint GetHeadPoint() const{return m_PointList.GetHead();}
	SkPoint& GetTailPoint(){return m_PointList.GetTail();}
	SkPoint GetTailPoint() const{return m_PointList.GetTail();}
	SkPoint GetEndPoint(int end) const{if( end == 0 )return GetHeadPoint();
		else return GetTailPoint();}
	SKPOSITION GetHeadPosition() const{return m_PointList.GetHeadPosition();}
	SKPOSITION GetTailPosition() const {return m_PointList.GetTailPosition();}
	SKPOSITION GetEndPosition(int e) const {if (e==0)return m_PointList.GetHeadPosition();else return m_PointList.GetTailPosition();}
	SkPoint GetNextPoint(SKPOSITION& rPos){return m_PointList.GetNext(rPos);}
	SkPoint GetPrevPoint(SKPOSITION& rPos){return m_PointList.GetPrev(rPos);} // return *Position--
	SkPoint RemoveHeadPoint() { return m_PointList.RemoveHead();};
	SkPoint RemoveTailPoint() { return m_PointList.RemoveTail();};

	SkNode* GetHeadNode() { return m_PointList.GetHeadNode(); }
	SkNode* GetTailNode() { return m_PointList.GetTailNode(); }

	double GetLength() { return m_PointList.GetLength(); }
	double GetEndAngle(int e);//[-PI,PI)
	// profile
	int computeProfile(Matrix<float> *pImgOrg, int nMaxPfWd, int nScale, Matrix<SkPixCandArray>* pPixCandMat);
	int	saveProfile2Matrix(Matrix<SkPixCandArray> &M);
	int getPointNum(){return m_PointList.GetSize();}
	int getProfileNum(){return m_PointList.GetSize()-1;}


};

class SkContourList
{
	// attributes
public:
	SkContour *m_pContourHead;
	SkContour *m_pContourTail;
	int m_nSize;
	// operations
public:
	SkContourList();
	~SkContourList();
	void RemoveAll();
	void AddHead( SkContour *pCon );
	void AddTail( SkContour *pCon );
	int GetSize();
	void Remove( SkContour* pC );
	SkContour *GetHead() { return m_pContourHead; }
	SkContour *GetTail() { return m_pContourTail; }
};


class SkVertex
{
public:
	oval_Array<SkNode *>		m_ptNodeArray;//the point nodes which connect to this vertex
	oval_Array<SkContour *>	m_conArray;//contours which connect to this vertex
	int						m_nID;
	int						x, y;
public:
	bool operator == (SkVertex &v);

	int getID() { return m_nID; }
	void setID(int id) { m_nID = id; }
	void removeAll() {m_ptNodeArray.RemoveAll(); m_conArray.RemoveAll();}
	int	 addPoint(SkNode* pPtNode, SkContour *pCon){m_ptNodeArray.add(pPtNode); m_conArray.add(pCon);return m_ptNodeArray.getSize();}
	int getDegree() { return m_ptNodeArray.getSize();}
	SkNode* getPtNode(int i) { return m_ptNodeArray.getAt(i); }

};

class SkVertexMap
{
public:
	CSparseArray<SkVertex> m_array;
public:
	int				getFirstPos() { return m_array.getFirstPos(); }
	SkVertex		*getNext(int& nPos) { return m_array.getNext(nPos); }
	SkVertex		*getAt(int nID) { return m_array.getAt(nID); }

	int				add(SkVertex &a);
	void			remove(SkVertex *p){m_array.removeAt(p->getID());}
	void			removeAll(){m_array.removeAll(); }
	int				getSize() {return m_array.getSize();}
};

class SkOcpMatrix
{
public:
	Matrix<oval_IntArray> m_ocpPoint;
	Matrix<oval_IntArray> m_ocpCon;
	Matrix<oval_IntArray> m_ocpBase;
public:
	void setDimension(int wd, int ht){m_ocpCon.SetDimension(ht,wd);m_ocpPoint.SetDimension(ht,wd);m_ocpBase.SetDimension(ht,wd);}
	int add(int x, int y, int pPoint, int pCon, int pBase);
	int remove(int x, int y, int pPoint, int pCon, int pBase);
};



class SkSketchGraph
{
public:
	int m_bUseRes4Prfl;//use residual image to build profile

	SkContourList	m_conList;//contour list, saves all of the contours
	SkOcpMatrix		m_ocp;//the occupy matrices
	int				m_nImgWd, m_nImgHt;
	int				m_nMaxPrflWd;
	double			m_dImgMean, m_dImgStd;	
	SkBaseMap		*m_pBaseMap;
	Matrix<float>   *m_pImgOrg;//orginal image
	Matrix<float>   m_imgRec;//reconstructed image
	Matrix<float>   m_imgRes;//residual image
	Matrix<SkPixCandArray> m_pixCandMat;
	double			m_dStdTh, m_dProjTh;
	double			m_dPrflVarTh; //profile variance threshold
	double			m_dInhibitionDis;
	char			m_pchProjName[512];
	int				m_nShowProcess;
	int				m_nProcessSteps;
	int				m_nScaleImgOrg;
	Matrix<float>	m_scaleImgOrg;
	double			m_pdResError[100000];//residual error for each sketch
	int				m_nSketchNum;//sketch number

	SkVertexMap		m_vertMap;//the vertex map, saves all of the vertices
public:
	SkSketchGraph() { m_nSketchNum = 0; m_dPrflVarTh = 0.3; m_bUseRes4Prfl = 0;}

	void			setImgDim(int wd, int ht) {m_nImgWd=wd; m_nImgHt=ht;}
	void			setImgMean(double m){ m_dImgMean = m; }
	void			setImgStd(double s) { m_dImgStd = s; }
	void			setBaseMap(SkBaseMap*pMap){m_pBaseMap=pMap;}
	void			setStdTh(double th) { m_dStdTh = th; }
	void			setProjTh(double th) { m_dProjTh = th; }
	void			setInhibitionDis(double dDis) { m_dInhibitionDis = dDis; }
	void			setMaxProfileWd(int w) { m_nMaxPrflWd = w; }
	int				getImgWidth(){return m_nImgWd;}
	int				getImgHeight(){return m_nImgHt;}
	SkBaseMap		*getBaseMap(){return m_pBaseMap;}
	Matrix<float>   *getImgOrg() { return m_pImgOrg; }
	Matrix<float>   *getImgRes() { return &m_imgRes; }
	void			setImgOrg(Matrix<float>* m) { m_pImgOrg = m; }
	void			setProjName(const char *name) { strcpy(m_pchProjName, name); }
	void			setShowProcess(int s) { m_nShowProcess = s; }
	void			setProcessSteps(int n){ m_nProcessSteps = n; }
	void			setScaleImgOrg(int s) { m_nScaleImgOrg = s; }
	void			setUseRes4Prfl(int b) { m_bUseRes4Prfl = b; }
	void			setPrflVarTh(double t) { m_dPrflVarTh = t; }

	//contours
	int				getContourNum() { return m_conList.GetSize(); }
	void			RemoveAllContours() { m_conList.RemoveAll(); }
	void			removeContour(SkContour *pCon) { m_conList.Remove(pCon); }
	SkContour		*getContourHead() { return m_conList.GetHead();}
	SkContour*		addNewContour(int nBaseID, SkBaseMap *pBaseMap);
	SkContour*		addNewContourByNbredBase(int nBaseID, SkBaseMap *pBaseMap);
	int				addNewPoint2Contour(int nBaseID, SkBaseMap *pBaseMap, SkContour* pCon, int a);
	int				buildSketchGraphFromNbredBaseMap(SkBaseMap *pBaseMap, const char *err_name);

	int				buildOccupyMatrix(int wd, int ht);
	int				findVertices();
	int				save2Matrix(Matrix<float> &M, int wd, int ht, int line_wd);
	int				saveProfile2Matrix(Matrix<float> &M, int wd, int ht);
	int				saveProfile2Matrix(Matrix<SkPixCandArray> &M, int wd, int ht);
	// occupation
	int				getOcpPointNum(int r, int c) { return m_ocp.m_ocpPoint(r,c).getSize();}
	SkNode*			getOcpPoint(int r, int c, int i){return (SkNode*)m_ocp.m_ocpPoint(r,c).getAt(i);}
	SkContour*		getOcpContour(int r, int c, int i) { return (SkContour*)m_ocp.m_ocpCon(r,c).getAt(i);}
	int				saveVertices2Matrix(Matrix<float> &M,//the matrix which saves the points on the graph, 0-on graph, 1-not
									int wd, int ht, //image width and height
									int line_wd);//the line-width used for the graph
	int				extendOrTrimContours();
	int				extendOrTrimContours(SkContour *pCon);
	int				findAllNbrContours(SkContour *pCon, int e);
	int				getContourEndSearchRegion(int x0, int y0,//contour end point
											  double dAngle, //the angle of the end;
											  double dLength, //the length of the contour
											  oval_Array<oval_Point> &ptArray,//the points of the  search region
											  int wd, int ht);//image width and height

	int				getContoursInRegion(SkContour **ppCons, // the pointers to contours which are in the given region
										oval_Array<oval_Point> &ptArray,//the point array defining the region
										oval_Array<oval_Point> &ocpPtArray);//the point array which is occupied by contours
	SkPoint			nearestPointOnOneContour(SkContour *pCon,//checked contour
											 SkPoint p0, //interested point
											 oval_Array<oval_Point> &ocpPtArray, //all possible point 
											 Matrix<oval_IntArray> &ocpCon);//the contour ocp matrix to check this con or not
	SkPoint			nearestExtendPoint(SkPoint p0, // the point needs to be exended
									   double dAngle, //the contour angle at that point
									   SkContour* &pConNbr, //the contour on which the nearest point is
									   oval_Array<oval_Point> &ocpPtArray,//all possible pt 
									   Matrix<oval_IntArray> &ocpCon);//the contour ocp matrix to check this con or not
	int				extendContourEndByNeighbor(SkContour* pCon, // the contour
											   int e); // the end
	int				getSortedContoursByLength(SkContour**ppCons, int nIncrease);
	int				addOccupy(SkContour *pCon);
	int				addOccupy(int x0, int y0, //start point								
								int x1, int y1, //end pint
								int pt1, int pt2, //two nbr SkPoint
								int con, //contour 
								int base);//base
	int				removeOccupy(int x0, int y0, //start point								
								int x1, int y1, //end pint
								int pt1, int pt2, //two nbr SkPoint
								int con, //contour 
								int base);//base

	int				extendContoursByProfile();
	int				extendContoursByProfile(SkContour *pCon);
	int 			extendContoursByProfile(SkContour *pCon, int e);
	int				possibleExtendPoints(int l, double dMaxOrt, int *pnX, int *pnY, double* pdProb);

	int				clearPixelCandidateMatrix();
	int				updateResidual(int x0, int y0, int x1, int y1, SkConProfile *pPrfl, int flag);

	int				getPointNum();
	int				getProfileNum();
	int				removeNoisyContour(SkContour * pCon);
	bool			parallelInhibited(int x0, int y0, int x1, int y1);
	int				numOfOccupyOnOneLine(int x0, int y0, int x1, int y1);

	int				findNewVerticesByContour(SkContour* pCon);

	double			resErrorChangeByAddingNewProfile(SkNode *pNode,//the node of that sketch
													   int nSketchType);//the type of sketch, DOG or non-DOG

};

int skSaveSketchGraph(SkSketchGraph &sketchGraph, const char *pPreName, const char *pProjectName,  const char *dir,
					  int wd, int ht, int line_wd);
int skSaveSketchGraphEvolvingProcess(SkSketchGraph &sketchGraph, int line_wd);
int skSaveVerticesInSketchGraph(SkSketchGraph &sketchGraph, const char *pPreName, int line_wd);
int skSaveProfileAsBmp(SkConProfile &prfl, char *name, double dImgMean, double dImgStd);
int skSaveVerticesAsTexton(SkSketchGraph &sketchGraph, char* proj);
int skSaveProfilesAsBases(SkSketchGraph &sketchGraph, const char* proj);


double skGetMinDistanceOfPixelCandidate(Matrix<SkPixCandArray> &M, int x, int y, double &dValue);
double skGetMixingValueOfPixelCandidate(Matrix<SkPixCandArray> &M, int x, int y);
int	skComputeImageFromPixCandByMinDis(Matrix<SkPixCandArray> &cand,//pixel candidates
							  Matrix<float> &rec); //reconstructed image
int	skComputeImageFromPixCandByMixing(Matrix<SkPixCandArray> &cand,//pixel candidates
							  Matrix<float> &rec); //reconstructed image
#endif//_SK_SKETCH_GRAPH_H_


