
#include <stdio.h>
#include <conio.h>
#include <direct.h>
#include <time.h>

#include "skSketchGraph.h"
#include "skSketchRefine.h"
#include "skBasePursuit.h"
#include "skMatrixIplOp.h"
#include "skOutput.h"


int sketch_refine_main( int argc, char** argv )
{
	char msg[512];

	FILE *pFile;
	// get current directory
	char pchDir[512];
	strcpy(pchDir, argv[0]);
	char *temp;
	temp = strrchr(pchDir, '\\');//find '\', return the pointer to it
	*temp = '\0';//path set correctly
	
	_chdir(pchDir);

	char pchImgNamesFile[512];
	char pchImgName[512];
	if ( argc < 2)
	{
		sprintf(pchImgNamesFile, "%s\\skrf_img_names.txt", pchDir);
		// read image names
		pFile = fopen(pchImgNamesFile, "r");
		if (pFile == 0)
		{
			sprintf(msg, "Can't find image file %s\n", pchImgNamesFile);
			skOutput(msg);
			return -1;
		}
		sprintf(msg, "Found image file %s\n", pchImgNamesFile);
		skOutput(msg);

		while (fscanf(pFile, "%s", pchImgName) != EOF)
		{
			skSketchRefine(pchImgName);
			break;
		}
		fclose(pFile);
	}
	else
	{
		assert( argc == 2);
		strcpy(pchImgName, argv[1]);
		skSketchRefine(pchImgName);
	}
	return -1;
}

/////////////////////////////////////////////////////////////////////
// class SkSketchRefineParam
SkSketchRefineParam::SkSketchRefineParam()
{
	m_dProfileStdTh = 1.0;//profile standard deviation threshold to be a sketch
	m_dProjTh = 0.8;//projection threshold set the second sketch as a true one
	m_dPrflVarTh = 0.4;
}
SkSketchRefineParam::~SkSketchRefineParam()
{
}
int SkSketchRefineParam::read(const char *name)
{
	FILE *pFile;
	pFile = fopen(name, "r");
	if (pFile == 0)
		return 1;
	fscanf(pFile, "ProfileStdTh:      %lf\n", &m_dProfileStdTh);//profile standard deviation threshold to be a sketch
	fscanf(pFile, "ProjectionTh:      %lf\n", &m_dProjTh);//projection threshold set the second sketch as a true one
	fclose(pFile);
	return 0;
}
int SkSketchRefineParam::save(const char *name)
{
	FILE *pFile;
	pFile = fopen(name, "w");
	if (pFile == 0)
		return 1;
	fprintf(pFile, "ProfileStdTh:      %.2lf\n", m_dProfileStdTh);//profile standard deviation threshold to be a sketch
	fprintf(pFile, "ProjectionTh:      %.2lf\n", m_dProjTh);//projection threshold set the second sketch as a true one
	fclose(pFile);
	return 0;
}

///////////////////
// skSketchRefine()
int skSketchRefine(const char *pchImgName)
{
	assert(false);
	return 0;

	char msg[512], name[512];

	int nReturn;
	//read original image
	Matrix<float> imgOrg;
	nReturn = skReadBitmap(pchImgName, &imgOrg);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", pchImgName);
		skOutput(msg);
		return 1;
	}
	int wd, ht;
	imgOrg.GetDimension(ht, wd);
	// normalize the image to 0 mean 1 variance
	double dImgMean = 0, dImgStd = -1;
	skNormalizeMatrix2ZeroMean1Std(&imgOrg, dImgMean, dImgStd);//-1 std will lead to find the mean and std

	// retreive the project name and directory name
	char pchProjName[512], pchDir[512];
	strcpy(name, pchImgName);
	char *temp;
	temp = strrchr(name, '.');//find '.', return the pointer to it
	*temp = '\0';
	temp = strrchr(name, '\\');//find '\', return the pointer to it
	strcpy(pchProjName, temp+1);
	*temp = '\0';
	strcpy(pchDir, name);

	_chdir(pchDir);//change current dir as the image dir

	// write or read paramters
	SkBasePursuitParam skps_param;
	sprintf(name, "%s\\%s_skps_param.txt", pchDir, pchProjName);
	skps_param.read(name);
	skps_param.save(name);//if read fails, then save the default

	SkSketchRefineParam skrf_param;
	sprintf(name, "%s\\%s_skrf_param.txt", pchDir, pchProjName);
	skrf_param.read(name);
	skrf_param.save(name);//if read fails, then save the default



	char pchType[32];
	sprintf(pchType, "skps");
	SkFilterBank filterBank, *pFB = &filterBank;
	sprintf(name, "%s\\%s_%s_filter_param.txt", pchDir, pchProjName, pchType);
	if (pFB->readParameters(name) != 0)
		pFB->m_param.initilaize4BasePursuit(0);

	pFB->saveParameters(name);//no such parameter file, save the default
	pFB->buildFilters();
	pFB->saveFiltersAsBmp(pchType);

	SkBaseMap baseMap;
	char skps_dir[32];
	sprintf(skps_dir, "skps");
	sprintf(name, "%s\\%s\\%s_skps.bsm", pchDir, skps_dir, pchProjName);
	baseMap.read(name);
	baseMap.setFilterBank4AllValidBases(pFB);

	Matrix<float> imgRes;
	imgRes = imgOrg;
	
	SkSketchGraph sketchGraph;

	sketchGraph.setImgDim(wd, ht);
	sketchGraph.setImgMean(dImgMean);
	sketchGraph.setImgStd(dImgStd);
	sketchGraph.setBaseMap(&baseMap);
	sketchGraph.setImgOrg(&imgOrg);

	sketchGraph.setInhibitionDis(skps_param.m_dInhibitionDis);
	sketchGraph.setMaxProfileWd(skps_param.m_nMaxProfileWd);

	int Time0, Time1, dTime;
	Time0 = ::GetTickCount();

//	sketchGraph.buildSketchGraphFromNbredBaseMap(&baseMap);
//	sketchGraph.computeContourProfile();
//	sketchGraph.computeReconstructedAndResidualImage();
	assert(false);
	
	Time1 = ::GetTickCount();
	dTime = (Time1 - Time0);

	skSketchRefine(sketchGraph, skrf_param, imgOrg, dImgMean, dImgStd, pchProjName, pchDir, skps_dir);
	return 0;
}
///////////////////
// skSketchRefine
int skSketchRefine(SkSketchGraph &sketchGraph, //the sketch graph
				   SkSketchRefineParam& param, //the parameters
				   Matrix<float> &imgOrg, //orignal image, normalized to zero mean, unit variance
				   double dImgMean, double dImgStd, //mean and std of original image
				   const char *pchProjName, //project name
				   const char *pchDir, //work directory
				   const char *skps_dir)//sketch pursuit directory
{	
	int Time0, Time1, dTime;
	int nMin;
	double dSec;
	char msg[512], name[512];

	sketchGraph.setStdTh(param.m_dProfileStdTh);
	sketchGraph.setProjTh(param.m_dProjTh);
	sketchGraph.setPrflVarTh(param.m_dPrflVarTh);
	
	int wd, ht;
	imgOrg.GetDimension(ht, wd);

	sketchGraph.buildOccupyMatrix(wd, ht);
	int line_wd;
/*	
	line_wd= 1;
	skSaveSketchGraph(sketchGraph, "w1_2", pchProjName, skps_dir, wd, ht, line_wd);
	line_wd= 3;
	skSaveSketchGraph(sketchGraph, "2", pchProjName, skps_dir, wd, ht, line_wd);
	
	sprintf(name, "%s\\%s\\%s_res_2.bmp", pchDir, skps_dir, pchProjName);
	skSaveMatrixAsBitmap(name, sketchGraph.m_imgRes, dImgMean, dImgStd);
*/
	int interpolateOrNot = 1;
	//2222222222222222222222222222222222
	Time0 = ::GetTickCount();

	sketchGraph.extendContoursByProfile();

	Time1 = ::GetTickCount();
	dTime = (Time1 - Time0);
	nMin = (int)(dTime/60000.0);
	dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "\n Sketch Pursuit With Profile Time: %d min %.3fs\n ", nMin, dSec);
	skOutput(msg);

	line_wd= 1;
	skSaveSketchGraph(sketchGraph, "w1_2", pchProjName, skps_dir, wd, ht, line_wd);
	line_wd= 3;
	skSaveSketchGraph(sketchGraph, "2", pchProjName, skps_dir, wd, ht, line_wd);


	sprintf(name, "%s\\%s\\%s_res_2.bmp", pchDir, skps_dir, pchProjName);
	skSaveMatrixAsBitmap(name, sketchGraph.m_imgRes, dImgMean, dImgStd);


	Time0 = ::GetTickCount();

	sketchGraph.buildOccupyMatrix(wd, ht);
	sketchGraph.extendOrTrimContours();

	Time1 = ::GetTickCount();
	dTime = (Time1 - Time0);
	nMin = (int)(dTime/60000.0);
	dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "\n Sketch Pursuit With Closure Time: %d min %.3fs\n ", nMin, dSec);
	skOutput(msg);


	line_wd = 1;
	skSaveSketchGraph(sketchGraph, "w1_3", pchProjName, skps_dir, wd, ht, line_wd);
	line_wd = 3;
	skSaveSketchGraph(sketchGraph, "3", pchProjName, skps_dir, wd, ht, line_wd);

	
	return 0;
}