#ifndef _SPARSEARRAY_H_
#define _SPARSEARRAY_H_

template<class TYPE>
class CSparseArray
{
private:
	int m_nGrowBy;
	int m_nSize;
	int m_nMaxSize;
	TYPE* m_pData;
	bool* m_pbUsed; //mark for valid data
	int m_nMinUnused;
public:
	CSparseArray(int nSize = -1);
	~CSparseArray();
	CSparseArray<TYPE> & operator = (CSparseArray<TYPE> & a);
	void setGrowBy(int nGrowBy);
	int getGrowBy();
	int add(TYPE & t);
	int removeAt(int n);
	void removeAll();
	bool remove(TYPE& t);
	int getSize();
	int getMaxSize() { return m_nMaxSize; }
//	TYPE& getAt(int& n);
	TYPE* getAt(int& n);
	int setAt(int& n, TYPE& t);
	bool hasMember( TYPE & t);
	int getFirstPos();
	TYPE* getNext(int& nPos);
};

// constructor
template<class TYPE>
CSparseArray<TYPE>::CSparseArray(int nSize)
{
	m_nGrowBy = 2;
	m_nSize = 0;
	m_nMaxSize = 0;
	m_pData = 0;
	m_pbUsed = 0; 
	m_nMinUnused = 0;
	if (nSize > 0)
	{
		m_nMaxSize = nSize;
		m_pData = new TYPE[m_nMaxSize];
		m_pbUsed = new bool[m_nMaxSize];
		for (int i = 0; i < m_nMaxSize; i++)
			m_pbUsed[i] = false;//init all to invalid
	}
}
// deconstructor
template<class TYPE>
CSparseArray<TYPE>::~CSparseArray()
{
	if (m_pData != 0)
		delete[] m_pData;
	if (m_pbUsed != 0)
		delete[] m_pbUsed;
}
// operator = 
template<class TYPE>
CSparseArray<TYPE> & CSparseArray<TYPE>::operator = (CSparseArray<TYPE> & a)
{// not clone, only copy used elements
	if (m_pData != 0)
		delete []m_pData;
	if (m_pbUsed != 0)
		delete []m_pbUsed;

	m_nSize = 0;
	m_nMaxSize = a.getSize();
	m_pData = new TYPE[m_nMaxSize];
	m_pbUsed = new bool[m_nMaxSize];
	for (int i = 0; i < m_nMaxSize; i++)
		m_pbUsed[i] = false;//init all to invalid
	m_nMinUnused = 0;

	int nPos = a.getFirstPos();
	while (nPos >= 0)
	{//both valid and invalid
		add(*(a.getNext(nPos)));
	}
	return *this;
}

// set grow by
template<class TYPE>
void CSparseArray<TYPE>::setGrowBy(int nGrowBy)
{
	m_nGrowBy = nGrowBy;
}
// get grow by
template<class TYPE>
int CSparseArray<TYPE>::getGrowBy()
{
	return m_nGrowBy;
}
// add one element
// return the position which the element was added to, it will be the ID of this element 
template<class TYPE>
int CSparseArray<TYPE>::add(TYPE &t)
{
	// search first unused element
	int nUnused = m_nMinUnused;
	if(nUnused < m_nMaxSize )
	{// there is still space left
		m_pData[nUnused] = t;
		m_pbUsed[nUnused] = true;
		m_nSize++;
		// set new min unused
		m_nMinUnused = -1;
		for (int i = nUnused+1; i < m_nMaxSize; i++)
			if (!m_pbUsed[i])
			{
				m_nMinUnused = i;
				break;
			}
		if (m_nMinUnused < 0)
			m_nMinUnused = m_nMaxSize;
	}
	else
	{// no more space, grow memory by new larger one
		m_nGrowBy = m_nMaxSize/2 + 1;
		TYPE* pTemp = new TYPE[m_nMaxSize+m_nGrowBy];
		bool* pTempUsed = new bool[m_nMaxSize+m_nGrowBy];
		for( int i = 0; i < m_nMaxSize; i++ )
		{
			pTemp[i] = m_pData[i];
			pTempUsed[i] = m_pbUsed[i];
		}
		nUnused = m_nMaxSize;
		m_nMaxSize += m_nGrowBy;
		for( int i = nUnused; i < m_nMaxSize; i++ )
			pTempUsed[i] = false;
		pTemp[nUnused] = t;
		pTempUsed[nUnused] = true;
		m_nSize++;
		if (m_pData != 0)
			delete[] m_pData;
		if (m_pbUsed != 0)
			delete[] m_pbUsed;
		m_pData = pTemp;
		m_pbUsed = pTempUsed;
		m_nMinUnused = nUnused+1;
	}
	return nUnused;
}
// remove one element at n
template<class TYPE>
int CSparseArray<TYPE>::removeAt( int n )
{
	if( n >= m_nMaxSize )
		return -1;
	m_pbUsed[n] = false;
	m_nSize--;
	if (m_nMinUnused > n)
		m_nMinUnused = n;
	return n;
}
// remove all element
template<class TYPE>
void CSparseArray<TYPE>::removeAll()
{
	m_nSize = 0;
	for (int i = 0; i < m_nMaxSize; i++)
		m_pbUsed[i] = false;
	m_nMinUnused = 0;
}
///////////////////
// remove one specific element
//	not good designing, generally not use this function
//	Please use removeAt(int n), n is the id
template<class TYPE>
bool CSparseArray<TYPE>::remove( TYPE& t )
{
	bool bRemove = false;
	for( int i = 0; i < m_nMaxSize; i++ )
	{
		if (m_pbUsed[i])
			if( t == m_pData[i] )
			{
				RemoveAt( i );
				bRemove = true;
				break;
			}
	}
	return bRemove;
}
// number of elements
template<class TYPE>
int CSparseArray<TYPE>::getSize()
{
	return m_nSize;
}
// return element at n
/*
template<class TYPE>
TYPE& CSparseArray<TYPE>::getAt( int& n )
{
	if (n < 0 || n >= m_nMaxSize)
		return m_pData[0];
	return m_pData[n];
}
*/
// return pointer to the element at n
template<class TYPE>
TYPE* CSparseArray<TYPE>::getAt( int& n )
{
	if (n < 0 || n >= m_nMaxSize)
		return 0;
	return m_pData+n;
}
// set element
template<class TYPE>
int CSparseArray<TYPE>::setAt( int& n, TYPE& t )
{
	if (n < 0 || n >= m_nMaxSize)
		return -1;
	m_pData[n] = t;
	return n;
}
// search a member
template<class TYPE>
bool CSparseArray<TYPE>::hasMember( TYPE & t )
{
	for (int i = 0; i < m_nMaxSize; i++)
		if (m_pbUsed[i])
			if(t == m_pData[i])
				return true;
	return false;
}
// get the first element in the array
template<class TYPE>
int CSparseArray<TYPE>::getFirstPos()
{
	int nReturn = -1;
	for (int i = 0; i < m_nMaxSize; i++)
		if (m_pbUsed[i])
		{
			nReturn = i;
			break;
		}
	return nReturn;
}
// get the first element in the array
template<class TYPE>
TYPE* CSparseArray<TYPE>::getNext(int& nPos)
{
	TYPE* pReturn = m_pData+nPos;
	int nNextPos = -1;
	for (int i = nPos+1; i < m_nMaxSize; i++)
	{
		if (m_pbUsed[i])
		{
			nNextPos = i;
			break;
		}
	}
	nPos = nNextPos;
	return pReturn;
}

#endif//_SPARSEARRAY_H_