#ifndef OVAL_TEMPLATE_H
#define OVAL_TEMPLATE_H

#include <assert.h>
#include <memory.h>
#include <math.h>
#include "inlineFunc.h"

#ifndef max
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#endif

#ifndef min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#endif

class oval_Point
{
public:
	int x;
	int y;
	oval_Point operator + ( oval_Point &p )
	{
		oval_Point tmp;
		tmp.x = x + p.x;
		tmp.y = y + p.y;
		return tmp;
	}
	oval_Point& operator += ( oval_Point &p )
	{
		x += p.x;
		y += p.y;
		return *this;
	}
	oval_Point& operator -= ( oval_Point &p )
	{
		x -= p.x;
		y -= p.y;
		return *this;
	}
};
template<class T>
class oval_TPoint
{
public:
	T x;
	T y;
	oval_TPoint<T> operator + ( oval_TPoint<T> &p )
	{
		x += p.x;
		y += p.y;
		return *this;
	}
	bool operator == ( oval_TPoint<T> &p )
	{
		if (x == p.x && y == p.y)
			return true;
		return false;
	}
	double distance(oval_TPoint<T> &p)
	{
		return sqrt((x-p.x)*(x-p.x) + (y-p.y)*(y-p.y));
	}
	void setX(int x0) { x = x0; }
	void setY(int y0) { y = y0; }

};

// define oval_IntArray
#ifndef oval_IntArray
#define oval_IntArray oval_Array<int>
#endif

#ifndef oval_PointArray
#define oval_PointArray oval_Array<oval_Point>
#endif
/*
// define CDoubleArray
#ifndef oval_DArray
#define oval_DArray oval_Array<double>
#endif
*/
// define CFloatArray
#ifndef oval_FArray
#define oval_FArray oval_Array<float>
#endif

///////////////////////////////////////////////////////////
// class for dynamic array
template<class TYPE>
class oval_Array
{
#define GROW_BY 2
private:
	int		m_nSize;
	int		m_nMaxSize;
	int		m_nGrowBy;
	TYPE*	m_pData;
public:
	oval_Array(int nSize = 0);
	~oval_Array();

	// applied when as a matrix 
	oval_Array &operator = (oval_Array<TYPE> &array);
	oval_Array &operator = (double v);
	bool operator < (oval_Array<TYPE> &array);
	bool operator > (oval_Array<TYPE> &array);
	oval_Array operator + (oval_Array<TYPE> &array);
	oval_Array &operator += (oval_Array<TYPE> &array);
	oval_Array operator - (oval_Array<TYPE> &array);
	oval_Array operator * (oval_Array<TYPE> &array);
	oval_Array operator * (double v);
	oval_Array operator * (TYPE v);
	oval_Array operator / (const oval_Array<TYPE> array);
	oval_Array operator / (double v);
	oval_Array operator / (TYPE v);

	bool	operator == (oval_Array<TYPE> &array);
	bool	operator != (oval_Array<TYPE> &array);
	TYPE	&operator [](int nIndex);
	
	int		add( TYPE  &t );//definately we need & to save time for big class
	void	removeAt( int n );
	void	removeAll();
	int		remove( TYPE &t );
	int		getSize() const;
	int		getMaxSize() const { return m_nMaxSize; }
	int		setMaxSize(int nMaxSize);
	int		setSize(int nSize);
	void	setAt( int n, TYPE &t );
	void	set(TYPE &t);
	TYPE	&getAt( int n ) const;
	TYPE	*getPointerAt( int n );
	int		addAt( int n, TYPE &t );
	void	replaceAt( int n, TYPE &t );
	bool	hasMember( TYPE &t );
	int		memberPos( TYPE &t );
	int		copyFrom(TYPE *p);
	int		copyTo(TYPE *p);
	void	reverse();

	int		Add( TYPE  &t ){return add(t);}//definately we need & to save time for big class
	void	RemoveAt( int n ){removeAt(n);}
	void	RemoveAll(){removeAll();}
	int		Remove( TYPE &t ){return remove(t);}
	int		GetSize(){return getSize();}
	int		SetSize(int nSize){return setSize(nSize);}
	void	SetAt( int n, TYPE &t ){setAt(n,t);}
	void	Set(TYPE &t){set(t);}
	TYPE	&GetAt( int n ){return getAt(n);}
	TYPE	*GetPointerAt( int n ){return getPointerAt(n);}
	int	AddAt( int n, TYPE &t ){ return addAt(n,t);}
	void	ReplaceAt( int n, TYPE &t ) {replaceAt(n,t);}
	bool	HasMember( TYPE &t ) {return hasMember(t); }
	int		CopyFrom(TYPE *p) {return copyFrom(p); }
	int		CopyTo(TYPE *p){return copyTo(p); }
	void	Reverse(){reverse();}

};



template<class TYPE>
oval_Array<TYPE>::oval_Array(int nSize)
{
	m_pData = 0;
	m_nSize = nSize;
	m_nGrowBy = GROW_BY;
//	if (nSize < m_nGrowBy)
//		m_nMaxSize = m_nGrowBy;
//	else
	m_nMaxSize = nSize;
	if (m_nMaxSize > 0)
		m_pData = new TYPE[m_nMaxSize];
}

template<class TYPE>
int oval_Array<TYPE>::setMaxSize(int nMaxSize)
{//all the data 
	m_nMaxSize = nMaxSize;
	TYPE* pTemp = new TYPE[m_nMaxSize];
	for( int i = 0; i < m_nSize; i++ )
		pTemp[i] = m_pData[i];
	if (m_pData != 0)
		delete[] m_pData;
	m_pData = pTemp;
	return m_nMaxSize;
}

template<class TYPE>
oval_Array<TYPE>::~oval_Array()
{
	if( m_pData != 0 )
		delete[] m_pData;
}

template<class TYPE>
oval_Array<TYPE> &oval_Array<TYPE>::operator = (oval_Array<TYPE> &array)
{
	RemoveAll();
	for (int i = 0; i < array.GetSize(); i++)
		Add(array.GetAt(i));
	return *this;
}
template<class TYPE>
oval_Array<TYPE> oval_Array<TYPE>::operator + (oval_Array<TYPE> &array)
{
	oval_Array<TYPE> tmp;
	int size0 = getSize();
	int size1 = array.getSize();
	int size = max(size0,size1 );
	tmp.setSize(size);

	TYPE t0, t1, t;
	for (int i = 0; i < size; i++)
	{
		if (i >= size0)
			t0 = 0;
		else
			t0 = getAt(i);
		if (i >= size1)
			t1 = 0;
		else
			t1 = array.getAt(i);
		t = t0+t1;
		tmp.setAt(i, t);
	}
	return tmp;
}
template<class TYPE>
oval_Array<TYPE>& oval_Array<TYPE>::operator += (oval_Array<TYPE> &array)
{
	int size0 = getSize();
	int size1 = array.getSize();

	TYPE t0, t1, t;
	for (int i = 0; i < size0; i++)
	{
		t0 = getAt(i);
		if (i >= size1)
			t1 = 0;
		else
			t1 = array.getAt(i);
		t = t0+t1;
		setAt(i, t);
	}
	return *this;
}
template<class TYPE>
oval_Array<TYPE> oval_Array<TYPE>::operator - (oval_Array<TYPE> &array)
{
	oval_Array<TYPE> tmp;
	int size0 = getSize();
	int size1 = array.getSize();
	int size = max(size0,size1 );
	tmp.setSize(size);

	TYPE t0, t1, t;
	for (int i = 0; i < size; i++)
	{
		if (i >= size0)
			t0 = 0;
		else
			t0 = getAt(i);
		if (i >= size1)
			t1 = 0;
		else
			t1 = array.getAt(i);
		t = t0-t1;
		tmp.setAt(i, t);
	}
	return tmp;
}
template<class TYPE>
oval_Array<TYPE> oval_Array<TYPE>::operator * (oval_Array<TYPE> &array)
{
	oval_Array<TYPE> tmp;
	int size0 = getSize();
	int size1 = array.getSize();
	int size = max(size0,size1 );
	tmp.setSize(size);

	TYPE t0, t1, t;
	for (int i = 0; i < size; i++)
	{
		if (i >= size0)
			t0 = 0;
		else
			t0 = getAt(i);
		if (i >= size1)
			t1 = 0;
		else
			t1 = array.getAt(i);
		t = t0*t1;
		tmp.setAt(i, t);
	}
	return tmp;
}
template<class TYPE>
oval_Array<TYPE> oval_Array<TYPE>::operator / (const oval_Array<TYPE> array)
{
	oval_Array<TYPE> tmp;
	int size0 = getSize();
	int size1 = array.getSize();
	int size = max(size0,size1 );
	tmp.setSize(size);

	TYPE t0, t1, t;
	for (int i = 0; i < size; i++)
	{
		if (i >= size0)
			t0 = 0;
		else
			t0 = getAt(i);
		if (i >= size1)
			t1 = 1;
		else
			t1 = array.getAt(i);
		t = t0/t1;
		tmp.setAt(i, t);
	}
	return tmp;
}
template<class TYPE>
oval_Array<TYPE> oval_Array<TYPE>::operator / (double v)
{
	oval_Array<TYPE> tmp;
	int size = getSize();
	tmp.setSize(size);

	TYPE t;
	for (int i = 0; i < size; i++)
	{
		t = getAt(i)/v;
		tmp.setAt(i, t);
	}
	return tmp;
}
template<class TYPE>
oval_Array<TYPE> oval_Array<TYPE>::operator / (TYPE v)
{
	oval_Array<TYPE> tmp;
	int size = getSize();
	tmp.setSize(size);

	TYPE t;
	for (int i = 0; i < size; i++)
	{
		t = getAt(i)/v;
		tmp.setAt(i, t);
	}
	return tmp;
}
template<class TYPE>
oval_Array<TYPE> oval_Array<TYPE>::operator * (double v)
{
	oval_Array<TYPE> tmp;
	int size = getSize();
	tmp.setSize(size);

	TYPE t;
	for (int i = 0; i < size; i++)
	{
		t = getAt(i)*v;
		tmp.setAt(i, t);
	}
	return tmp;
}
template<class TYPE>
oval_Array<TYPE> oval_Array<TYPE>::operator * (TYPE v)
{
	oval_Array<TYPE> tmp;
	int size = getSize();
	tmp.setSize(size);

	TYPE t;
	for (int i = 0; i < size; i++)
	{
		t = getAt(i)*v;
		tmp.setAt(i, t);
	}
	return tmp;
}
template<class TYPE>
bool oval_Array<TYPE>::operator < (oval_Array<TYPE> &array)
{
	if (getSize() < array.getSize())
		return true;
	else
		return false;
}
template<class TYPE>
bool oval_Array<TYPE>::operator > (oval_Array<TYPE> &array)
{
	if (getSize() > array.getSize())
		return true;
	else
		return false;
}
template<class TYPE>
oval_Array<TYPE> &oval_Array<TYPE>::operator = (double v)
{
	for (int i = 0; i < getSize(); i++)
		m_pData[i] = m_pData[i] + v;
	return *this;
}
template<class TYPE>
bool oval_Array<TYPE>::operator == (oval_Array<TYPE> &array)
{
	if (GetSize() != array.GetSize())
		return false;
	for (int i = 0; i < m_nSize; i++)
		if (!(m_pData[i] == array.GetAt(i)))
			return false;
	return true;
}
template<class TYPE>
bool oval_Array<TYPE>::operator != (oval_Array<TYPE> &array)
{
	return !(*this == array);
}
template<class TYPE>
TYPE & oval_Array<TYPE>::operator [](int nIndex)
{
	return GetAt(nIndex);
}

///////////////////
// Add()
//	return the position which the element is added to
template<class TYPE>
int oval_Array<TYPE>::add( TYPE &t )
{
	if( m_nSize < m_nMaxSize )
	{
		m_pData[m_nSize] = t;
		m_nSize++;
	}
	else
	{
		m_nGrowBy = m_nSize / 2 + 1;
		TYPE* pTemp = new TYPE[m_nMaxSize+m_nGrowBy];
		for( int i = 0; i < m_nMaxSize; i++ )
			pTemp[i] = m_pData[i];
		m_nMaxSize += m_nGrowBy;
		pTemp[m_nSize] = t;
		m_nSize++;
		if (m_pData != 0)
			delete[] m_pData;
		m_pData = pTemp;
	}
	return m_nSize-1;
}
template<class TYPE>
void oval_Array<TYPE>::removeAt( int n )
{
	assert(n >= 0 && n < m_nSize);
	for( int i = n; i < m_nSize - 1; i++ )
		m_pData[i] = m_pData[i+1];
	m_nSize--;
}
template<class TYPE>
void oval_Array<TYPE>::removeAll()
{
	m_nSize = 0;
}
template<class TYPE>
int oval_Array<TYPE>::remove( TYPE &t )
{
	bool bRemove = false;
	int i=0;
	for( i = 0; i < m_nSize; i++ )
	{
		if( t == m_pData[i] )
		{
			RemoveAt( i );
			bRemove = true;
			break;
		}
	}
	assert( bRemove );
	return i;
}
template<class TYPE>
int oval_Array<TYPE>::getSize() const
{
	return m_nSize;
}
template<class TYPE>
int oval_Array<TYPE>::setSize(int nSize)
{
	if (nSize <= m_nMaxSize)
		m_nSize = nSize;//ignore all exist elements
	else// more space are needed
	{
		TYPE* pTemp = new TYPE[nSize];
		for( int i = 0; i < m_nSize; i++ )
			pTemp[i] = m_pData[i];
		if (m_pData != 0)
			delete[] m_pData;
		m_pData = pTemp;
		m_nSize = m_nMaxSize = nSize;
	}
	return m_nSize;
}
template<class TYPE>
TYPE& oval_Array<TYPE>::getAt( int n ) const
{
	assert(n >= 0 && n < m_nSize);
	return m_pData[n];
}
template<class TYPE>
TYPE *oval_Array<TYPE>::getPointerAt( int n )
{
	assert(n >= 0 && n < m_nMaxSize);
	return m_pData+n;
}
template<class TYPE>
void oval_Array<TYPE>::setAt( int n, TYPE &t )
{
	assert(n >= 0 && n < m_nSize);
	m_pData[n] = t;
}
template<class TYPE>
void oval_Array<TYPE>::set(TYPE &t )
{
	for (int i = 0; i < m_nSize; i++)
		SetAt(i, t);
}
///////////////////
// AddAt()
//	add one element at position "n"
//	all the elements behind "n", will be moved one potision
//	the total size of the array will increase by one
template<class TYPE>
int oval_Array<TYPE>::addAt( int n, TYPE &t )
{
	Add(t);
	if (n != m_nSize-1)
	{// should not be at the end
		for (int i = m_nSize-1; i > n; i--)
			m_pData[i] = m_pData[i-1];
		m_pData[n] = t;
	}
	return n;
}
///////////////////
// ReplaceAt()
//	replace one element at position "n"
//	all the elements behind "n", will be kept
//	the total size of the array will keep the same
template<class TYPE>
void oval_Array<TYPE>::replaceAt( int n, TYPE &t )
{
	m_pData[n] = t;
}
template<class TYPE>
bool oval_Array<TYPE>::hasMember( TYPE &t )
{
	for( int i = 0; i < m_nSize; i++ )
	{
		if( t == m_pData[i] )
			return true;
	}
	return false;
}
template<class TYPE>
int oval_Array<TYPE>::memberPos( TYPE &t )
{
	for( int i = 0; i < m_nSize; i++ )
	{
		if( t == m_pData[i] )
			return i;
	}
	return -1;
}
template<class TYPE>
int oval_Array<TYPE>::copyFrom(TYPE *p)
{
	assert(p != 0);
	for (int i = 0; i < m_nSize; i++)
		m_pData[i] = p[i];
	return m_nSize;
}
template<class TYPE>
int oval_Array<TYPE>::copyTo(TYPE *p)
{
	assert(p != 0);
	for (int i = 0; i < m_nSize; i++)
		p[i] = m_pData[i];
	return m_nSize;
}
template<class TYPE>
void oval_Array<TYPE>::reverse()
{
	TYPE t;
	for (int i = 0; i < m_nSize/2; i++)
	{
		t = m_pData[i];
		m_pData[i] = m_pData[m_nSize-1-i];
		m_pData[m_nSize-1-i] = t;
	}
}

///////////////////
// polygonFilling()
//	For given polygon vertices, filling the polygon region
//	The boundary is included
inline int polygonFilling(oval_Array<oval_Point> &polyVetArray, oval_Array<oval_Point> &regionPtArray)
{
	int nEdgeNum = polyVetArray.getSize();
	if( nEdgeNum <= 2 )// less than 2 edges can't form a polygon
		return 1;

	int nMinY = 1000000, nMaxY = -1;
	int nMinX = 1000000, nMaxX = -1;
	for (int i = 0; i < nEdgeNum; i++)
	{
		if (nMinY > polyVetArray.getAt(i).y)
			nMinY = polyVetArray.getAt(i).y;
		if (nMaxY < polyVetArray.getAt(i).y)
			nMaxY = polyVetArray.getAt(i).y;
		if (nMinX > polyVetArray.getAt(i).x)
			nMinX = polyVetArray.getAt(i).x;
		if (nMaxX < polyVetArray.getAt(i).x)
			nMaxX = polyVetArray.getAt(i).x;

	}
	int *pIntersectionPointsX = new int[nEdgeNum];
	int nIntersectionNum;
	int x;
	oval_Point p0, p1, pt;
	for( int y = nMinY; y <= nMaxY; y++ )
	{
		pt.y = y;

		nIntersectionNum = 0;
		int n = 0;
		p0 = polyVetArray.getAt(n);
		n++;
		for( int l = 0; l < nEdgeNum; l++ )
		{//get all intersections of X
			if (n == nEdgeNum)
				n = 0;
			p1 = polyVetArray.getAt(n);
			n++;
			if( ( p0.y >= y && p1.y < y ) || ( p0.y < y && p1.y >= y ) )
			{// intersect
				x = (int)( (float)(p1.x-p0.x)*(y-p0.y)/(p1.y-p0.y) +p0.x);
				pIntersectionPointsX[nIntersectionNum] = x;
				nIntersectionNum++;
			}
			p0 = p1;
		}
		sort( nIntersectionNum, pIntersectionPointsX, 0 );// 0-increasing, 1-decreasing
		for( int i = 0; i < nIntersectionNum; i = i+2 )
		{
			int nStart = max( 0, pIntersectionPointsX[i] );
			int nEnd = min( pIntersectionPointsX[i+1], nMaxX );
			for( int xx = nStart; xx <= nEnd; xx++ )
			{
				pt.x = xx;
				regionPtArray.add(pt);
			}
		}
	}
	delete[] pIntersectionPointsX;
	return regionPtArray.getSize();

}

template<class TYPE>
class SkStack
{
public:
	oval_Array<TYPE> m_array;
	int push(TYPE t) { return m_array.add(t); }
	TYPE pop() { TYPE t = m_array.getAt(m_array.getSize()-1); 
					m_array.removeAt(m_array.getSize()-1); return t;}
	int getSize() { return m_array.getSize(); } 
	TYPE getTop() { return m_array.getAt(m_array.getSize()-1);}
	TYPE getAt(int i) { return m_array.getAt(i); }
};

#endif //OVAL_TEMPLATE_H
