#include "skSketchPursuit.h"
#include "cv.h"
#include "highgui.h"
#include "skJulesz.h"
#include "skImgSeg.h"

SkPrimPursuitParam param; 
SkSketchPursuit pursuit;

std::string strDir;
std::string strImgFile;
std::string strImgName;
std::string strImgBaseName;
std::string strShowFile;

cv::Mat img;

std::string strWindOrgImg("OrigImage");
std::string strWindStrength("Strength");
std::string strWindProposal("Proposal");
std::string strWindSketchGraph("SketchGraph");

std::string strWindSynInit("InitialSynthesizedImage");
std::string strWindSyn("SynthesizedImage");
std::string strWindSynOnly("SynthesizedImage(non-sketchable part)");

bool bDoJuleszSampling = false;

int nMaxClusterNum = 6;

std::string strMagThd1;

int origImgCLG = param.m_dMinImgCLG * 10;
int origTotalCLG = param.m_dMinTotalCLG * 10;
int origTh = param.m_dMagThresh1 * 100;

int nImgCLG=0;
int nTotalCLG = 0;
int nTh=0;

bool bDoJuleszSamplingNew = false;

void on_trackbar(int, void *)
{
	if ( origImgCLG!=nImgCLG || origTh!=nTh || origTotalCLG!=nTotalCLG )
	{
		param.m_dMinImgCLG = (float)nImgCLG /10.f;
		param.m_dMinTotalCLG = (float)nTotalCLG / 10.f;
		param.m_dMagThresh1 = (float)nTh / 100.f;

		pursuit.sketchingPursuitWithoutPrior(strImgName.c_str(), param);

		double dMTh = param.m_dMagThresh1;///255;
		int nMTh = (int)(dMTh*100);
		char mth_no[32];
		::changeNumTo2Char(mth_no, nMTh);	
		strMagThd1 = "_m" + std::string(mth_no);

		strShowFile = strDir + strImgBaseName +  "\\sketches\\" + strImgBaseName + "_1" + strMagThd1 +"_sketch_graph.bmp";
		img = cv::imread(strShowFile, 0);
		cv::imshow( strWindSketchGraph, img);
		cv::waitKey(1);			

		bDoJuleszSamplingNew = true;
	}	


	if ( bDoJuleszSampling && bDoJuleszSamplingNew )
	{
		skJuleszSamplingSketch(strImgFile.c_str() ); //may be time-consuming

		strShowFile = strDir + strImgBaseName + "\\" + strImgBaseName + "_syn_init.bmp";
		img = cv::imread(strShowFile, 0);
		cv::imshow( strWindSynInit, img);
		cv::waitKey(1);

		strShowFile = strDir + strImgBaseName + "\\" + strImgBaseName + "_syn.bmp";
		img = cv::imread(strShowFile, 0);
		cv::imshow( strWindSyn, img);
		cv::waitKey(1);


		strShowFile = strDir + strImgBaseName + "\\" + strImgBaseName + "_syn_only.bmp";
		img = cv::imread(strShowFile, 0);
		cv::imshow( strWindSynOnly, img);
		cv::waitKey(1);

		bDoJuleszSamplingNew = false;

	}	
}
 

int main(int argc, char* argv[])
{	
	//////////////////////////////////////////////////////////////////////////
	// check input arg
	FILE* f = 0;

	char _filename[1024];	

	if( argc == 1 )
	{
		printf("Usage: Sketching (<image_filename> | <image_list>.txt) [-syn 0 | 1]\n");
		return 0;
	}

	img = cv::imread(argv[1], -1);

	if( img.data )
	{
		strcpy(_filename, argv[1]);
	}
	else
	{
		f = fopen(argv[1], "rt");
		if(!f)
		{
			fprintf( stderr, "ERROR: the specified file could not be loaded\n");
			return -1;
		}
	}

	if ( argc>3 && !strcmp( argv[2], "-syn" ) )
	{
		bDoJuleszSampling = atoi( argv[3] );
	}

	std::cout << "Welcome to the primal sketch" << std::endl;

	cv::namedWindow(strWindOrgImg);
	cv::namedWindow(strWindStrength);
	cv::namedWindow(strWindProposal);
	cv::namedWindow(strWindSketchGraph);

	if ( bDoJuleszSampling )
	{
		cv::namedWindow(strWindSynInit);
		cv::namedWindow(strWindSyn);
		cv::namedWindow(strWindSynOnly);
	}

	//////////////////////////////////////////////////////////////////////////
	// start sketching 
	for(;;)
	{
		//check if the image file is valid
		char* filename = _filename;
		if(f)
		{
			if(!fgets(filename, (int)sizeof(_filename)-2, f))
				break;
		
			if(filename[0] == '#')
				continue;
			int l = strlen(filename);
			while(l > 0 && isspace(filename[l-1]))
				--l;
			filename[l] = '\0';
			img = cv::imread(filename, -1);
		}
		
		printf("\n%s:\n", filename);

		if(!img.data)
			continue;
		
		//////////////////////////////////////////////////////////////////////////
		//parse image file names
		strImgFile = std::string(filename);
		int idx = strImgFile.find_last_of('\\');
		int idx1 = strImgFile.find_last_of('.');
		
		strDir = strImgFile.substr(0, idx+1);
		strImgName =  strImgFile.substr(idx+1, strImgFile.length()-idx) ;
		strImgBaseName =  strImgFile.substr(idx+1, idx1-idx-1);

		cv::imshow( strWindOrgImg, img);
		cv::waitKey(1);		

		//////////////////////////////////////////////////////////////////////////
		//current working dir		
		_chdir( strDir.c_str() );

		//////////////////////////////////////////////////////////////////////////
		// strength map
		pursuit.computeSketchStrength(strImgName.c_str());
		
		strShowFile =strDir + strImgBaseName + "\\proposal\\" + strImgBaseName + "_sketch_mag0.bmp";
		img = cv::imread(strShowFile, -1);		
		cv::imshow( strWindStrength, img);
		cv::waitKey(1);

		//////////////////////////////////////////////////////////////////////////
		//initial proposal (edge map)
		pursuit.computeSketchProposal(strImgName.c_str());

		strShowFile = strDir + strImgBaseName +  "\\proposal\\" + strImgBaseName + "_sketch0.bmp";
		img = cv::imread(strShowFile, 0);		
		cv::imshow( strWindProposal, img);
		cv::waitKey(1);
		
		//////////////////////////////////////////////////////////////////////////
		// pursuit
		pursuit.sketchingPursuitWithoutPrior(strImgName.c_str(), param);

		double dMTh = param.m_dMagThresh1;///255;
		int nMTh = (int)(dMTh*100);
		char mth_no[32];
		::changeNumTo2Char(mth_no, nMTh);	
		strMagThd1 = "_m" + std::string(mth_no);

		strShowFile = strDir + strImgBaseName +  "\\sketches\\" + strImgBaseName + "_1" + strMagThd1 +"_sketch_graph.bmp";
		img = cv::imread(strShowFile, 0);
		img = cv::imread(strShowFile, 0);
		cv::imshow( strWindSketchGraph, img);
		cv::waitKey(1);

		skImageSegmentationSketch(strImgName.c_str(), nMaxClusterNum);

		bDoJuleszSamplingNew = true;

		//////////////////////////////////////////////////////////////////////////
		// update 
		nImgCLG = param.m_dMinImgCLG * 10;
		nTotalCLG = param.m_dMinTotalCLG * 10;
		nTh = param.m_dMagThresh1 * 100;

		cv::createTrackbar( "imgCLG", strWindSketchGraph, &nImgCLG, 400, on_trackbar );
		cv::createTrackbar( "minMagTh", strWindSketchGraph, &nTh, 100, on_trackbar );
		cv::createTrackbar( "totalCLG", strWindSketchGraph, &nTotalCLG, 400, on_trackbar );
				
		strShowFile = strDir + strImgBaseName + "\\" + strImgBaseName + "_sketching_param.txt";
		param.save(strShowFile.c_str());

		on_trackbar(0, 0);

		fflush(stdout);
		
		if( !f)
			break;
		
	}

	cv::waitKey(0);

	if(f)
		fclose(f);
	
	return 0;

}