#include <direct.h>

#include "skOutput.h"
#include "skMatrixIplOp.h"

#include "skIOImage.h"

#include "skClustering.h"

#include "skPrimitive.h"
#include "skPrimPursuit.h"
#include "skLineSeg.h"

const char g_pchSketchReconstructedDir[512] = "sk_rec";

/////////////////////////////////////////////////////////////////////
// class SkSketchCLManager
double SkSketchCLManager::m_dCL4VP = 9*2;
double SkSketchCLManager::m_dCL4Pointer2VP = 4.0;
double SkSketchCLManager::m_dImgCodingSigma2 = 2.0;


SkPrimManager::SkPrimManager()
{
	m_bUpdateExtCon = false; 
	m_bSegmentsComputed = false;
	m_bProfileComputed = false;



}


double SkSketchCLManager::computeCL4dA(SkBranchProfile &prfl0, 
									   SkBranchProfile &prfl1)
{
	int w0 = prfl0.getSampleWidth();
	int w1 = prfl1.getSampleWidth();

	double *v0 = prfl0.m_ridgeModel.getModelIntensity();
	double *v1 = prfl1.m_ridgeModel.getModelIntensity();

	int d;
	int i1, i0;

	int nTest = w1/2*2+1;
	double* pdSumErr = new double[nTest];
	int t = 0;
	for (d = -w1/2; d <= w1/2; d++)
	{
		pdSumErr[t] = 0;
		for (i1 = 0; i1 < w1; i1++)
		{
			i0 = (i1-w1/2) + w0/2 + d;
			if (i0 < 0)
				i0 = 0;
			else if (i0 >= w0)
				i0 = w0 - 1;
			pdSumErr[t] += (v0[i0] - v1[i1])*(v0[i0] - v1[i1]);
		}
		t++;
	}
	assert(t == nTest);
	double dMinSumErr = 10000000;
	for (t = 0; t < nTest; t++)
	{
		if (dMinSumErr > pdSumErr[t])
			dMinSumErr = pdSumErr[t];
	}
	delete []pdSumErr;

	double dCL = dMinSumErr/m_dImgCodingSigma2;//need to be normalized
	return dCL;
}

///////////////////
// skLearnPrimitves()
//	learn the primives from a set of images whose names are saved in a file
//	return the number of images
int skLearnPrimitives(FILE *pFile)
{
	char pchImgName[512];
	int n = 0;
	Matrix<double> prfl_samples;

	char name[512], msg[512];
	sprintf(name, "prfl.txt");
	
	if (skReadMatrix(name, prfl_samples) == 0)
	{
		sprintf(msg, "\n   The 1D primitives have been saved into a matrix. \n      Doing clustering from that.....\n");
		skOutput(msg);
		skLearnProfilePrimitives(prfl_samples);
		return 0;
	}

	{//just set locally to releasing memory before clustering
		SkPrimManager pPrimManager[1024];//max 1024 files
		while (fscanf(pFile, "%s", pchImgName) != EOF)
		{
			skBuildPrimitiveMap(pchImgName, &(pPrimManager[n]));
			n++;
			if (n >= 1024)
				break;
		}

//		skSaveLargePrimitiveIcons(pPrimManager, n, prfl_samples);
	}

	skSaveMatrix(name, prfl_samples);

//	skLearnProfilePrimitives(prfl_samples);
	return n;
}

///////////////////
// skBuildPrimitveMap()
int skBuildPrimitiveMap(const char *pchImgName,
					   SkPrimManager *pPrimMan)//find primitive map after finishing
{
	char pchProjName[512], name[512], pchDir[512], msg[512];

	sprintf(msg, "\n\nBuild Primitive map on image %s ..............................", pchImgName);
	skOutput(msg);

	DWORD Time0 = ::GetTickCount();

	char pchDir0[512];
	_getcwd(pchDir0, 512);

	strcpy(pchProjName, pchImgName);
	char *temp;
	temp = strrchr(pchProjName, '.');//find '.', return the pointer to it
	*temp = '\0';

	//load image
	//read original image and boundary image

	Matrix<float> imgOrg;
	int nReturn;
	/*
	nReturn = skReadBitmap(pchImgName, &imgOrg);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", pchImgName);
		skOutput(msg);
		return 1;
	}
*/
	Matrix<int> pImgChn[3];
	nReturn = ::skLoadImage2YCrCb(pchImgName, pImgChn);

	int ht, wd;
	pImgChn[0].GetDimension(ht, wd);
	if (ht <= 0 || wd <= 0)
		return 1;

	::skCopyMatrix(pImgChn[0], imgOrg);

	_mkdir(pchProjName);
	_chdir(pchProjName);

	Matrix<int> imgBound;//the binary image of primitive positions, 0-valid, 255-not valid
	sprintf(name, "%s_bound.bmp", pchProjName);
	nReturn = skReadBitmap(name, &imgBound);
	if (nReturn != 0)
	{
		sprintf(msg, "Couldn't read image %s\n", pchImgName);
		skOutput(msg);
		return 1;
	}

//	Matrix<int> img255(ht, wd);
//	img255.SetDimension(ht, wd);
//	img255.Set(255);
//	imgBound = img255 - imgBound;//0-valid, 255-not


	nReturn = skSaveMatrixAsBitmap(pchImgName, imgOrg, 0, 1);
//	nReturn = skSaveMatrixAsBitmap(name, imgBound, 0, 1);

	sprintf(pchDir, "%s\\%s", pchDir0, pchProjName);

	skThinnerBoundary(imgBound);

	sprintf(name, "%s_bound_thin.bmp", pchProjName);
	nReturn = skSaveMatrixAsBitmap(name, imgBound, 0, 1);


	double dImgMean0 = 0, dImgStd0 = -1;
	skNormalizeMatrix2ZeroMean1Std(&imgOrg, dImgMean0, dImgStd0);//-1 std will lead to find the mean and std

	pPrimMan->setBound(imgBound);
	pPrimMan->setOrgImg(&imgOrg);
	pPrimMan->setOrgImgMeanStd(dImgMean0, dImgStd0);
	pPrimMan->setDir(pchDir);
	pPrimMan->setProjName(pchProjName);
//	pPrimMan->setMaxBranchLength(7);
//	pPrimMan->setSaveLargeBranch(1);//only save large branch
//	pPrimMan->setSaveLargeBranch(0);//save all branch

	pPrimMan->buildPrimitiveMap();
	

	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	sprintf(msg, "\n Total build primitive time for image %s:   %d min %.3fs \n ", pchImgName, nMin, dSec);
	::skOutput(msg);

	_chdir(pchDir0);
	return 0;
}

///////////////////
// skTCA1DPrimitive()
//	compute the 1D primitive after TCA
Matrix<double> skTCA1DPrimitive(Matrix<float> &org, //org image
								int w, int h,//the size of used image
								SkIntPoint &p0,//center of used patch
								double &dOrt, //orientation of the new center line
								double &dScale,//scale of perpendicular direction to the profile
								int &nType, //type of the profile 0-edge, 1-bar
								double &dError)//total reconstruction error
{
	Matrix<double> prfl_mean;
	int wd, ht;
	org.GetDimension(ht, wd);
	prfl_mean.SetDimension(ht, 1);

	double dSum;
	dError = 0;
	int r, c;
	for (r = 0; r < ht; r++)
	{
		dSum = 0;
		for (c = 0; c < wd; c++)
		{
			dSum += org(r, c);
		}
		dSum /= wd;
		prfl_mean(r, 0) = dSum;
		for (c = 0; c < wd; c++)
		{
			dError += (org(r, c)-dSum)*(org(r, c)-dSum);
		}
	}
	dError /= wd*ht;
	return prfl_mean;

}


///////////////////
// skLearnProfilePrimitives()
int skLearnProfilePrimitives(Matrix<double> &samples)
{
	DWORD Time0 = ::GetTickCount();

	char pchDir0[512]; 
	_getcwd(pchDir0, 512);

	char pchDir[512], name[512];
	sprintf(pchDir, "primitives");
	_mkdir(pchDir);
	_chdir(pchDir);

	int nDataDim, nNum;
	samples.GetDimension(nNum, nDataDim);
	int nClusterDim = (nDataDim/2/2)*2 + 1;

	Matrix<double> weight(1, nClusterDim);//row vector
	weight.Set(1.0);//equal weight

	int nMaxClusterNum = 50;
	int nMinClusterSize = 10;
	double dMinClusterDisRate = 0.0;
	Matrix<double> cluster_center(nMaxClusterNum, nClusterDim);
	Matrix<int> cluster_id(1, nNum);
	Matrix<int> offset(1, nNum);

//	int nClusterNum = skRobustCluster(samples.data, nDataDim, nNum, weight.data, cluster_center.data, nMaxClusterNum, nMinClusterSize, dMinClusterDisRate, cluster_id.data);
	int nClusterNum = skRobustClusterTCA(samples.data, nDataDim, nNum, weight.data,  
		nMaxClusterNum, nMinClusterSize, dMinClusterDisRate, nClusterDim, cluster_center.data, cluster_id.data, offset.data);
	if (nClusterNum <= 0)
		return -1;
	// save the cluster centers
	int n = nClusterNum;
	int size = nClusterDim;
	int space = 3;

//	int nCol = 10;//sqrt(n);
	int nCol = sqrt(double(n));
	int nRow = n/nCol;
	if (n != nCol*nRow)
		nRow +=1;
	if (nRow == 1)
		nCol = n;

	int wd = nCol * (size+space) + space;
	int ht = nRow * (size+space) + space;

	Matrix<float> M(size, size);
	Matrix<float> cluster_icon(ht, wd);

	int r, c, x0, y0;
	for (int i = 0; i < n; i++)
	{
		r = i/nCol;
		c = i%nCol;
		x0 = c * (size+space) + size/2 + space;
		y0 = r * (size+space) + size/2 + space;//center of the icon

		for (int y = 0; y < nClusterDim; y++)
			for (int x = 0; x < nClusterDim; x++)
				M(y,x) = cluster_center(i, y);
		skCopyImage(&M, &cluster_icon, -(x0-size/2), -(y0-size/2));
	}

	float fNeg255 = (float)-255.0;
	float fPos255 = (float)255.0;
	skDrawSeperatedLinesAndNoOccupiedBlocks(cluster_icon, fPos255, fPos255, nRow, nCol, n, size, space);

	sprintf(name, "prim_1d_icon_cluster_center.bmp");

	double dImgMean = 128.0;
	double dImgStd = 64.0;

	::skSaveMatrixAsBitmap(name, cluster_icon, dImgMean, dImgStd);

	_chdir(pchDir0);


	DWORD Time1 = ::GetTickCount();
	DWORD dTime = (Time1 - Time0);
	int nMin = (int)(dTime/60000.0);
	double dSec = (dTime - nMin * 60000)/1000.0;
	char msg[512];
	sprintf(msg, "\n Total clustering time for %d samples:   %d min %.3fs \n ", nNum, nMin, dSec);
	::skOutput(msg);


	return 0;
}

///////////////////
// skFindConnectedPoints()
//	0-valid point
//	return connected points num
int skFindConnectedPoints(int x0, int y0,//the start point
						  SkIntPointArray &nbrPtArray,//nbr point array
						  Matrix<int> &imgBound)//the binary image of primitive positions, 0-valid
{
	nbrPtArray.removeAll();

	int ht, wd;
	imgBound.GetDimension(ht, wd);

	SkIntPoint p;
	int dx, dy, x, y;
	for (dy = -1; dy <= 1; dy++)
	{
		y = y0 + dy;
		if (y < 0 || y >= ht)
			continue;
		for (dx = -1; dx <= 1; dx++)
		{
			x = x0 + dx;
			if (x < 0 || x >= wd)
				continue;
			if (dx == 0 && dy == 0)
				continue;
			if (imgBound(y,x) == 0)
			{
				p.x = dx;
				p.y = dy;
				nbrPtArray.Add(p);
			}
		}
	}
	return nbrPtArray.GetSize();
}
///////////////////
// skThinnerBoundary()
//	0-valid point
//	thinner the boundary to only 1-connection in 8-connectness
int	skThinnerBoundary(Matrix<int> &bound)//0-valid,
{
	int ht, wd;
	bound.GetDimension(ht, wd);
	int x, y;
	bool bChange, bReturn;
	while (true)
	{
		bChange = false;
		for (y = 0; y < ht; y++)
		{
			for (x = 0; x < wd-2; x++)
			{
				if (bound(y,x) == 0 && bound(y, x+1) == 0)
				{
					bReturn = skThinnerBoundary(x, y, x+1, y, bound);
					if (bReturn)
						bChange = true;
				}
			}
		}
		for (x = 0; x < wd; x++)
		{
			for (y = 0; y < ht-2; y++)
			{
				if (bound(y,x) == 0 && bound(y+1, x) == 0)
				{
					bReturn = skThinnerBoundary(x, y, x, y+1, bound);
					if (bReturn)
						bChange = true;
				}
			}
		}
		for (y = 0; y < ht; y++)
		{
			for (x = wd-1; x > 1; x--)
			{
				if (bound(y,x) == 0 && bound(y, x-1) == 0)
				{
					bReturn = skThinnerBoundary(x, y, x-1, y, bound);
					if (bReturn)
						bChange = true;
				}
			}
		}
		for (x = 0; x < wd; x++)
		{
			for (y = ht-1; y > 1; y--)
			{
				if (bound(y,x) == 0 && bound(y-1, x) == 0)
				{
					bReturn = skThinnerBoundary(x, y, x, y-1, bound);
					if (bReturn)
						bChange = true;
				}
			}
		}
		if (!bChange)
			break;
	}
//	return 0;
	//thinner by indirect connectness
	while (true)
	{
		bool bChange = false;
		for (y = 0; y < ht; y++)
		{
			for (x = 0; x < wd-2; x++)
			{
				if (bound(y,x) == 0 && bound(y, x+1) == 0)
				{
					bReturn = skThinnerBoundaryIndirectly(x, y, x+1, y, bound);
					if (bReturn)
						bChange = true;
				}
			}
		}
		for (x = 0; x < wd; x++)
		{
			for (y = 0; y < ht-2; y++)
			{
				if (bound(y,x) == 0 && bound(y+1, x) == 0)
				{
					bReturn = skThinnerBoundaryIndirectly(x, y, x, y+1, bound);
					if (bReturn)
						bChange = true;
				}
			}
		}
		for (y = 0; y < ht; y++)
		{
			for (x = wd-1; x > 1; x--)
			{
				if (bound(y,x) == 0 && bound(y, x-1) == 0)
				{
					bReturn = skThinnerBoundaryIndirectly(x, y, x-1, y, bound);
					if (bReturn)
						bChange = true;
				}
			}
		}
		for (x = 0; x < wd; x++)
		{
			for (y = ht-1; y > 1; y--)
			{
				if (bound(y,x) == 0 && bound(y-1, x) == 0)
				{
					bReturn = skThinnerBoundaryIndirectly(x, y, x, y-1, bound);
					if (bReturn)
						bChange = true;
				}
			}
		}
		if (!bChange)
			break;
	}

	return 0;
}
///////////////////
// skThinnerBoundary()
//	0-valid point
//	thinner the boundary to only 1-connection in 8-connectness
//	(x1, y1) and (x2, y2) are two points
//	if (x1,y1) has all the neighbors which (x2,y2) has, then (x2,y2) is a redundent point, remove it
bool skThinnerBoundary(int x1, int y1,//first point
					  int x2, int y2,//second point
					  Matrix<int> &bound)//0-invalid
{
	SkIntPointArray nbrPtArray1, nbrPtArray2;
	int cp1 = skFindConnectedPoints(x1, y1, nbrPtArray1, bound);
	int cp2 = skFindConnectedPoints(x2, y2, nbrPtArray2, bound);

	if (cp2 == 1)
		return false;

	bool bRedundent = true;
	int i;
	SkIntPoint p0, p;
	for (i = 0; i < cp2; i++)
	{
		p0 = nbrPtArray2.getAt(i);
		p.x = x2 + p0.x;
		p.y = y2 + p0.y;//change to absolute 
		if (p.x == x1 && p.y == y1)
			continue;
		p.x -= x1;
		p.y -= y1;//change to relative position
		if (!nbrPtArray1.hasMember(p))
		{
			bRedundent = false;
			break;
		}
	}
	if (bRedundent)
	{
		bound(y2,x2) = 255;
		return true;
	}
	return false;
}
///////////////////
// skThinnerBoundaryIndirectly()
//	0-valid point
//	thinner the boundary to only 1-connection in 8-connectness
//	(x1, y1) and (x2, y2) are two points
//	if (x1,y1) or its neighbors has all the neighbors which (x2,y2) has, then (x2,y2) is a redundent point, remove it
bool skThinnerBoundaryIndirectly(int x1, int y1,//first point
								 int x2, int y2,//second point
								 Matrix<int> &bound)//0-invalid
{
	SkIntPointArray nbrPtArray1, nbrPtArray2, nbrPtArray;
	int cp1 = skFindConnectedPoints(x1, y1, nbrPtArray1, bound);
	int cp2 = skFindConnectedPoints(x2, y2, nbrPtArray2, bound);
	if (cp2 == 1)
		return false;

	bool bRedundent = true;
	int i, j;
	SkIntPoint dp, dp2, dp1, p1, p2, p, dpp;
	for (i = 0; i < cp2; i++)
	{//consider all of (x2,y2) neighbors
		dp = nbrPtArray2.getAt(i);
		p.x = x2 + dp.x;
		p.y = y2 + dp.y;//p: absolute value to be tested
		bool bConnected = false;
		if (p.x == x1 && p.y == y1)
		{
			bConnected = true;
			continue;//if (x1,y1) then don't consider this neighbor
		}
		for (j = 0; j < cp1; j++)
		{//consider all of (x1,y1) neighbors
			dp1 = nbrPtArray1.getAt(j);
			p1.x = x1 + dp1.x;
			p1.y = y1 + dp1.y;// p1: asolute value of (x1,y1)'s neighbor
			if (p1.x == p.x && p1.y == p.y)
			{
				bConnected = true;
				break;
			}
			if (p1.x == x2 && p1.y == y2)
				continue;//not consider the (x2,y2) as valid neighbor of (x1,y1) for testing
			nbrPtArray.removeAll();
			skFindConnectedPoints(p1.x, p1.y, nbrPtArray, bound);

			dpp.x = p.x - p1.x;
			dpp.y = p.y - p1.y;//change test point to relative position
			if (nbrPtArray.hasMember(dpp))
			{
				bConnected = true;
				break;
			}
		}
		if (!bConnected)
		{//if any of (x2,y2) neighbors, not connected
			bRedundent = false;
			break;
		}
	}//for all (x2,y2) neighbors
	if (bRedundent)
	{
		bound(y2,x2) = 255;
		return true;
	}
	return false;
}

///////////////////
// skFindAllConnectedPoints
//	0-valid point
int skFindAllConnectedPoints(int x, int y,//start point
						  SkIntPointArray &ptArray,//the array saves the connected points
						  Matrix<int> &mat)//the matrix indicates the points, 0-valid
{
	ptArray.removeAll();

	SkIntPoint p;
	p.x = x; 
	p.y = y;
	ptArray.add(p);

	SkIntPoint dp, p1;
	SkIntPointArray nbrPtArray;//the array saves the connected nbr points

	int i = 0;//current search point
	while (i < ptArray.getSize())
	{
		p = ptArray.getAt(i);
		int cp = skFindConnectedPoints(p.x, p.y, nbrPtArray, mat);
		for (int j = 0; j < cp; j++)
		{
			dp = nbrPtArray.getAt(j);
			p1 = dp + p;
			if (!ptArray.hasMember(p1))
				ptArray.add(p1);
		}
		i++;
	}
	return ptArray.getSize();
}

///////////////////
// skComputeCurvature()
//	k = (x'y'' - x''y')/((x'^2  +y'^2) ^ (3/2))
//	  =  (d^2 y / d x^2) / ((1+ (dy/dx)^2) ^(3/2))
int skComputeCurvature(SkIntPointArray& ptArrayD, //the relative point array
					   double *pCur,//the curvature for each point
					   int nHalfWindow)//half window size
{
	int n = ptArrayD.getSize();
	if (n <= 1)
		return 0;
	SkIntPoint ddp, dp;
	double k;

	for (int i = 0; i < n-1; i++)
	{
		dp = ptArrayD[i];
		ddp = ptArrayD[i+1] - ptArrayD[i];
		k = (dp.x * ddp.y - ddp.x * dp.y) / pow(dp.x*dp.x + dp.y*dp.y, 1.5);
		pCur[i] = k;
	}
	pCur[n-1] = pCur[n-2];
	return n;

}
///////////////////
// skComputeCornerness()
//	if the second derivative of the current point and previous point is not the same
//	then defien the current point is a corner pCur = 1; else pCur = 0
int skComputeCornerness(SkIntPointArray& ptArrayD, //the relative point array
					   double *pCur,//the curvature for each point
					   int nHalfWindow)//half window size
{
	int n = ptArrayD.getSize();
	if (n <= 1)
		return 0;

	SkIntPoint ddp;
	SkIntPointArray ptArrayDD;

	int i=0;
	for ( i = 0; i < n-1; i++)
	{
		ddp = ptArrayD[i+1] - ptArrayD[i];
		ptArrayDD.add(ddp);
	}
	ptArrayDD.add(ddp);//the last two are the same

	i = 0;
	pCur[i] = 0;
	double cor0, cor1;
	SkIntPoint ddp0, ddp1;
	for (int i = 1; i < n-1; i++)
	{
		ddp0 = ptArrayDD[i];//second derivative
		cor0 = fabs(double(ddp0.x)) + fabs(double(ddp0.y));
		ddp1 = ptArrayDD[i-1];
		cor1 = fabs(double(ddp1.x)) + fabs(double(ddp1.y));//second derivative of previous point
		if (cor0 == 0 || cor1 == 0)
			pCur[i] = 0;
		else
		{
			if (ptArrayD[i+1] == ptArrayD[i-1])
				pCur[i] = 0;
			else
			{
				pCur[i] = 1.0;
			}
		}
	}
	pCur[n-1] = 0;
	return n;
}



/////////////////////////////////////////////////////////////////////
// class SkRidgeModel
///////////////////
int SkRidgeModel::build(Matrix<double> &v0)//v is a column vector use (i,0) to access each member
{
	char name[512];

	int m_nSegNum;//segments num
	int m_pnSegPos[SK_MAX_PRFL_RIDGEMODEL];//start position of each segment
	double m_pdSegMean[SK_MAX_PRFL_RIDGEMODEL];//segment mean value
	int m_pnStepSegNo[SK_MAX_PRFL_RIDGEMODEL];//ridge correspondent seg no
	double m_pdDeriv[SK_MAX_PRFL_RIDGEMODEL];//the derivative of mean of each seg, 

	int wd, ht;
	v0.GetDimension(ht, wd);

	assert(wd == 1);
	assert(ht >= 1);
	assert(ht <= SK_MAX_PRFL_RIDGEMODEL);

	m_nDataLength = ht;

	// copy the values
	int r;
	for (r = 0; r < ht; r++)
	{
		m_v[r] = v0(r,0);
		m_vm[r] = 0;
		m_vms[r] = 0;
		m_vmc[r] = 0;
	}

	double dHalf = ht/2;


	double dMin = v0.GetMin();
	double dMax = v0.GetMax();
	double dThresh = (dMax-dMin)/4;
	if (dMin == dMax)
	{
		m_nSegNum = 1;
		m_nStepNum = 1;
		m_pnSegPos[0] = 0;
		m_pdSegMean[0] = dMin;
		m_pnStepSegNo[0] = 0;

		return m_nStepNum;
	}

	//do the segmentation

	int s = 0;//current seg
	m_pnSegPos[s] = 0;//start position of each segment
	m_pdSegMean[s] = m_v[0];

	s = 1;
//	int c_s = -1;//the seg no of the center point
	for (r = 1; r < ht; r++)
	{
		if ( fabs(m_v[r] - m_pdSegMean[s-1]) < dThresh)
		{//keep segment, update mean
			m_pdSegMean[s-1] = (m_pdSegMean[s-1] *(r-m_pnSegPos[s-1]) + m_v[r])/(r-m_pnSegPos[s-1]+1);
		}
		else
		{//new segment
			m_pnSegPos[s] = r;
			m_pdSegMean[s] = m_v[r];
			s++;
		}
//		if (r == ht/2)
//			c_s = s-1;
	}
	m_pnSegPos[s] = ht;//the last one with the out of range position, for computation convenience

	assert(s > 1);//s: total true segments
	// find the zero crossings

	int i;
	for (i = 0; i < s-1; i++)
		m_pdDeriv[i] = m_pdSegMean[i+1]-m_pdSegMean[i];
	m_pdDeriv[s-1] = m_pdDeriv[s-2];//first derivative [i+1] - [i]

	int nCross = 0;
	int pnCrossS[SK_MAX_PRFL_RIDGEMODEL];
	for (i = 1; i < s-1; i++)
	{//not consider the first and last segment
		if (m_pdDeriv[i-1] * m_pdDeriv[i] < 0)//check second derivative
		{//[i] is the seg which is the extrema
			pnCrossS[nCross] = i;
			nCross++;
		}
	}

	m_nSegNum = s;
	m_nStepNum = nCross + 2;
	m_pnStepSegNo[0] = 0;
	m_pnStepSegNo[m_nStepNum-1] = s-1;

	for (i = 1; i < m_nStepNum-1; i++)
		m_pnStepSegNo[i] = pnCrossS[i-1];

	// computeStepsAndModelIntensity
	int sn;
	// get start, end and mean for each step
	for (i = 0; i < m_nStepNum; i++)
	{
		sn = m_pnStepSegNo[i];//seg no
		m_pnStepSt[i] = m_pnSegPos[sn];
		m_pnStepEn[i] = m_pnSegPos[sn+1] - 1;
		m_pdStepMean[i] = m_pdSegMean[sn];
	}

	assert(m_nStepNum >= 2);
	
	computeModelIntensity();

	return m_nStepNum;
}
int SkRidgeModel::computeModelIntensity()
{
	int i, j;
	for (i = 0; i < m_nStepNum; i++)
	{
		for (j = m_pnStepSt[i]; j <= m_pnStepEn[i]; j++)
			m_vm[j] = m_pdStepMean[i];
		if (i == m_nStepNum - 1)
			break;//for the last step, no filling-in check needed
		int st = m_pnStepEn[i] + 1;
		double mean_st = m_pdStepMean[i];
		int en = m_pnStepSt[i+1] - 1;
		double mean_en = m_pdStepMean[i+1];
		if ( st <= en )
		{
			for (j = st; j <= en; j++)
			{//interpolation
				m_vm[j] = (mean_st*(en-j+1) + mean_en*(j-st+1))/(en-st+2);
			}
		}
	}

	for (i = 0; i < m_nStepNum; i++)
	{
		for (j = m_pnStepSt[i]; j <= m_pnStepEn[i]; j++)
			m_vms[j] = m_pdStepMean[i];
		if (i == m_nStepNum - 1)
			break;//for the last step, no filling-in check needed
		int st = m_pnStepEn[i] + 1;
		double mean_st = m_pdStepMean[i];
		int en = m_pnStepSt[i+1] - 1;
		double mean_en = m_pdStepMean[i+1];
		if ( st <= en )
		{
			for (j = st; j <= (en+st)/2; j++)
			{//interpolation
				m_vms[j] = mean_st;
			}
			for (j = (en+st)/2; j <=en; j++)
			{//interpolation
				m_vms[j] = mean_en;
			}
		}
	}

	for (i = 0; i < m_nDataLength; i++)
		m_vmc[i] = m_vm[i];//the centerred one equals the model

	return 0;
}


int SkRidgeModel::sortStepMean()
{
	int nStepNum = getStepNum();
	double dMean0 = getStepMean(0);
	double dMean1 = getStepMean(nStepNum-1);


	if (dMean0 > dMean1)
		inverseTheCenterredModel();	//inverse

	centerModelIntensity();//make the ridge or edge at the center

	return 1;
}
int SkRidgeModel::centerModelIntensity()//make the ridge or edge at the center
{
	int nStepNum = getStepNum();
	int nStepWd0 = getStepWidth(0);
	int nStepWd1 = getStepWidth(nStepNum-1);

	int nStepWd0_new = (nStepWd0+nStepWd1)/2;
	int dd = nStepWd0_new - nStepWd0;

	if (dd == 0)
		return dd;

	double pdVm[SK_MAX_PRFL_RIDGEMODEL];
	int nLn = getDataLength();
	int i;
	for (i = 0; i < nLn; i++)
		pdVm[i] = m_vmc[i];//back up of the centerred model

	double dMean0 = getStepMean(0);
	double dMean1 = getStepMean(nStepNum-1);

	if (dd > 0)
	{
		for (i = 0; i < dd; i++)
			m_vmc[i] = dMean0;
		for (i = dd; i < nLn; i++)
		{
			m_vmc[i] = pdVm[i-dd];
		}
		return dd;
	}
	else
	{//dd < 0
		dd = -dd;
		for (i = 0; i < nLn-dd; i++)
			m_vmc[i] = pdVm[i+dd];//back up
		for (i = nLn-dd; i < nLn; i++)
		{
			m_vmc[i] = dMean1;
		}
		return dd= -dd;
	}
	m_dOffset = dd;
	return dd;
}
int SkRidgeModel::inverseTheCenterredModel()
{
	assert(!m_bReverse);
	m_bReverse = true;
	SkRidgeModel rm;
	rm = *this;//back up
	int nStepNum = getStepNum();
	int nLn = getDataLength();
	for (int i = 0; i < nStepNum; i++)
	{
		m_pdStepMean[i] = rm.getStepMean(nStepNum-i-1);
		m_pnStepSt[i] = nLn - rm.getStepEnd(nStepNum-i-1) - 1;
		m_pnStepEn[i] = nLn - rm.getStepStart(nStepNum-i-1) - 1;
	}

	::inverseArray(m_vmc, m_nDataLength);

	return 0;
}

bool SkRidgeModel::operator > (SkRidgeModel &rm1)
{
	int nStepNum0 = getStepNum();
	int nStepNum1 = rm1.getStepNum();

	if (nStepNum0 > nStepNum1)
		return true;

	if (nStepNum0 < nStepNum1)
		return false;

	int i;
	for (i = 1; i < nStepNum0 - 1; i++)
	{//compare all of the step width
		int nStepWidth0 = getStepWidth(i);
		int nStepWidth1 = rm1.getStepWidth(i);

		if (nStepWidth0 > nStepWidth1)
			return true;
		else if (nStepWidth0 < nStepWidth1)
			return false;
	}
	// all step width are the same
	// then consider step gap
	for (i = 0; i < nStepNum0 -1; i++)
	{
		int nGap0 = getStepFollowingGap(i);
		int nGap1 = rm1.getStepFollowingGap(i);

		if (nGap0 > nGap1)
			return true;
		else if (nGap0 < nGap1)
			return false;
	}
	// all the gap are the same
	// then consider the step mean
	for (i = 0; i < nStepNum0; i++)
	{
		double dStepMean0 = getStepMean(i);
		double dStepMean1 = rm1.getStepMean(i);

		if (dStepMean0 > dStepMean1)
			return true;
		else if (dStepMean0 < dStepMean1)
			return false;
	}
	return false;//equal
}


/////////////////////////////////////////////////////////////////////
// class SkSegement

SkFloatPoint SkSegment::getFloatPoint(int i)
{
	SkIntPoint p = getPoint(i); 
	SkFloatPoint fp; 
	double dOff = m_prfl.getFloatPointOffset(i);
	if (m_nPtAxis) 
	{
		fp.x = p.x + dOff*m_nPrflSign; 
//		fp.x = p.x; 
		fp.y = p.y;
	} 
	else 
	{ 
		fp.y = p.y + dOff*m_nPrflSign;
//		fp.y = p.y;
		fp.x = p.x;
	} 
	return fp;
}
///////////////////
int  SkSegment::computePoints()
{
	int pnX[4096];
	int pnY[4096];//axis point of the profile

	int n;
	n = ::rasterize_line_segment_keeping_order(m_pEndPt[0].x, m_pEndPt[0].y,
		m_pEndPt[1].x, m_pEndPt[1].y, pnX, pnY);
	
	assert(n <= 4096);

	m_ptArray.removeAll();
	SkIntPoint p;

	for (int i = 0; i < n; i++)
	{//there is an overlap on the join point
		p.x = pnX[i];
		p.y = pnY[i];//all the axis point
		m_ptArray.add(p);
	}

	::raster_line_axis_profile_sign(m_pEndPt[0].x, m_pEndPt[0].y,
		m_pEndPt[1].x, m_pEndPt[1].y, m_nPtAxis, m_nPrflSign);
	m_bPtsComputed = true;
	return m_ptArray.getSize();
}

///////////////////
// computeProfile()
int  SkSegment::computeProfile(Matrix<float> *pImgOrg, int nHalfWd)
{
	SkIntPoint p0, p1;
	p0 = getEndPoint(0);
	p1 = getEndPoint(1);
	m_prfl.setSampleByLineProfile(p0.x, p0.y, p1.x, p1.y, pImgOrg, nHalfWd);

	int x0 = (p0.x+p1.x)/2;
	int y0 = (p0.y+p1.y)/2;

	char x0_no[32];
	char y0_no[32];

	::changeNumTo3Char(x0_no, x0);
	::changeNumTo3Char(y0_no, y0);

	char type[128];
	m_prfl.computeMeanFromValidSamples();

	// compute rige edge type and width
	m_prfl.findCenter4EachSample(m_prfl.getSampleMean());
	m_prfl.updateSampleAndMeanByCenterOffset();
	m_prfl.findCenter4EachSample(m_prfl.getSampleMean());//refine the prfl
	m_prfl.updateSampleAndMeanByCenterOffset();

	sprintf(type, "x%s_y%s", x0_no, y0_no);
//	m_prfl.saveProfileAsBmp(type);

	m_prfl.buildEdgeRidgeModel();

	sprintf(type, "x%s_y%s", x0_no, y0_no);
//	m_prfl.saveProfileAsBmp(type);

	return 0;
}


/////////////////////////////////////////////////////////////////////
// class SkBranchProfile
///////////////////
double SkBranchProfile::m_dImgMean0 = 128;
double SkBranchProfile::m_dImgStd0 = 64;

int SkBranchProfile::buildEdgeRidgeModel()
{
	Matrix<double> m = 	getSampleMean();

	m_ridgeModel.build(m);

	return 0;
}
//////////////////
int SkBranchProfile::findCenter4EachSample(Matrix<double> &mu)//must after build EdgeRidge model
{
	double v[SK_MAX_PRFL_RIDGEMODEL];
	int w, h;
	m_sample.GetDimension(h, w);

	// w not always odd
	int i, j;

//	m_smpError.SetDimension(h, 1);

	for (i = 0; i < h; i++)
	{
		v[i] = mu(i,0);
	}

	double dCenter0 = h/2;
	int nAccuracy = 10;
	double av[SK_MAX_PRFL_RIDGEMODEL*10];

	int pnOff[1024];
	assert(h <= 1024);

	for (i = 0; i < h-1; i++)
	{
		for (j = 0; j < nAccuracy; j++)
		{//linear interpolation
			av[i*nAccuracy+j] = (v[i]*(nAccuracy - j) + v[i+1]*j) / nAccuracy;
		}
	}

	int wa = (h-1) * nAccuracy;

	double dError, dMinError, min_di;
	int nValidPt;
	int di;
	int dy, y;
	int sp;
	int nPixelDiff = 1;
	for (int ii = 0; ii < w; ii++)
	{//for each sample
		int st, en;
		if (ii == 0)
		{//center one
			sp = 0;;
			st = 0;
			en = 0;//the center one is on the center
		}
		else if (ii <= 0)//first half
		{
			sp = 0 - ii;
			st = pnOff[sp+1] - nPixelDiff*nAccuracy;//max diff two pixels
			en = pnOff[sp+1] + nPixelDiff*nAccuracy;
		}
		else
		{
			sp = ii;
			st = pnOff[sp-1] - nPixelDiff*nAccuracy;//max diff two pixels
			en = pnOff[sp-1] + nPixelDiff*nAccuracy;
		}
		dMinError = 1000000;
		for (di = st; di <= en; di++)
		{
			dError = 0;
			nValidPt = 0;
			dy = -di;//?
			for (i = 0; i < h; i++)
			{
				y = dy + i*nAccuracy;
				if (y >= 0 && y < wa)
				{
					nValidPt++;
					dError += (av[y] - m_sample(i,sp))*(av[y] - m_sample(i,sp));
				}
			}
			if (nValidPt == 0)
				dError = 1000000;
			else 
				dError /= nValidPt;
			if (dMinError > dError)
			{
				dMinError = dError;
				min_di = di;
			}
		}
		pnOff[sp] = min_di;
		double dOff = (double)min_di/nAccuracy;
//		double dCenter = dCenter0 + dOff;
		if (dOff < -h/2) 
			dOff = -h/2;
		if (dOff > h/2) 
			dOff = h/2;
		m_smpCenterOff(sp, 0) += dOff;
//		m_smpError(sp, 0) = dMinError;// update samples
	}

	return 0;

}

int SkBranchProfile::updateSampleAndMeanByCenterOffset()
{
	int w, h;
	m_sample.GetDimension(h, w);
	double pdTmpSample[SK_MAX_PRFL_RIDGEMODEL];
	double pos, dOff;
	int sp, i;
	for (sp = 0; sp < w; sp++)
	{
		dOff = m_smpCenterOff(sp, 0);//this offset is related to orginal sample
		for (i = 0; i < h; i++)
		{
			pos = i + dOff;
			if (pos < 0)
				pdTmpSample[i] = m_sample0(0,sp);
			else if (pos >= h-1)
				pdTmpSample[i] = m_sample0(h-1,sp);
			else 
			{
				int p1 = (int)pos;
				int p2 = p1 + 1;
				pdTmpSample[i] = m_sample0(p1,sp) *(p2-pos) + m_sample0(p2,sp)*(pos-p1);
			}
		}
		for (i = 0; i < h; i++)
		{
			m_sample(i,sp) = pdTmpSample[i];//true sample used
		}
	}
	computeMeanFromValidSamples();
	return 0;
}

///////////////////
// computeMeanFromValidSamples()
//	assume the m_sample is already created
int SkBranchProfile::computeMeanFromValidSamples()
{
	int w, h;
	m_sample.GetDimension(h, w);
	assert(h > 0 && h < MAX_CON_PROFILE_WD);
	int pnSampleNum[MAX_CON_PROFILE_WD];
	memset(pnSampleNum, 0, sizeof(int)*h);
	//compute mu and sigma
	
//	assert(m_nValidSmpStart >= 0 && m_nValidSmpStart < w
//		&& m_nValidSmpEnd >= 0 && m_nValidSmpEnd < w);
	//mu
	m_mu.SetDimension(h, 1);//column verctor
	m_mu.Zero();
	double data;
	int i, j;
	for (i = 0; i < h; i++)
	{
		for (j = 0; j < w; j++)
		{
			data = m_sample(i, j);
			if (data < SK_INFINITE/2)
			{
				m_mu(i,0) += data;
				pnSampleNum[i]++;
			}
		}
	}
	for (i = 0; i < h; i++)
	{
		if (pnSampleNum[i] > 0)
			m_mu(i,0) /= pnSampleNum[i];
		else
			m_mu(i,0) = SK_INFINITE;
	}
	return w;
}
///////////////////
int SkBranchProfile::setSample(Matrix<double> &mat)
{//column verctor of each sample
	int wd, ht;
	mat.GetDimension(ht, wd);

	m_sample0 = mat;
	m_sample = mat;

	m_smpCenterOff.SetDimension(wd, 1);
	m_smpCenterOff.Set(0);

	return 0;
}
///////////////////
int SkBranchProfile::addSample(Matrix<double> &mat)
{//column verctor of each sample
	int wd, ht;
	mat.GetDimension(ht, wd);

	int wd0, ht0;
	m_sample.GetDimension(ht0, wd0);

	if (ht0 != ht)
	{
		m_sample = mat;
	}
	else
	{
		int nSampleNum = wd0 + wd;

		Matrix<double> tmp;
		tmp.SetDimension(ht, nSampleNum);
		int r, c;
		for (c = 0; c < wd0; c++)
		{
			for (r = 0; r < ht; r++)
			{
				tmp(r,c) = m_sample(r,c);
			}
		}
		for (c = wd0; c < nSampleNum; c++)
		{
			for (r = 0; r < ht; r++)
			{
				tmp(r,c) = mat(r,c-wd0);
			}
		}
//			m_nSampleNum = nSampleNum;
		m_sample = tmp;
	}

//	m_nValidSmpStart = 0;
//	m_nValidSmpEnd = wd+wd0-1;

	m_smpCenterOff.SetDimension(wd+wd0, 1);
	m_smpCenterOff.Set(0);

	return wd+wd0;
}

///////////////////
// setSampleByLineProfile()
//	add a line (x0,y0)->(x1,y0) into the m_sample
int SkBranchProfile::setSampleByLineProfile(int x0, int y0, int x1, int y1, Matrix<float> *pImgOrg, int half_wd)
{

	int ht = pImgOrg->rows();
	int wd = pImgOrg->cols();

	int pnX_v[MAX_CON_PROFILE_WD], pnY_v[MAX_CON_PROFILE_WD];
	double pdPrfl[MAX_CON_PROFILE_WD];

	int pnX[4096];
	int pnY[4096];//axis point of the profile

	int pf_x0, pf_x1, pf_y0, pf_y1;
	int x, y, dx, dy, xc, yc;
	int n;
	n = ::rasterize_line_segment_keeping_order(x0, y0, x1, y1, pnX, pnY);
	
	assert(n <= 4096);

	::verticalLineApproximately(x0, y0, x1, y1, pf_x0, pf_y0, pf_x1, pf_y1, half_wd*2+1);
	int m = ::rasterize_line_segment_keeping_order(pf_x0, pf_y0, pf_x1, pf_y1, pnX_v, pnY_v);

	Matrix<double> sample(m,n);
	assert(m <= MAX_CON_PROFILE_WD);

//	m_ptArray.removeAll();
	SkIntPoint p;

	for (int i = 0; i < n; i++)
	{//there is an overlap on the join point
		xc = pnX[i];
		yc = pnY[i];//all the axis point
		
		p.x = xc;
		p.y = yc;
//		m_ptArray.add(p);


		for (int j = 0; j < m; j++)
		{
			dx = pnX_v[j];
			dy = pnY_v[j];//verticle piont of the 
			x = xc + dx;
			y = yc + dy;
			if (x < 0 || x >= wd || y < 0 || y >= ht)
				sample(j,i) = 0;//SK_INFINITE;
			else
				sample(j,i) = (*pImgOrg)(y,x);
		}
	}
	setSample(sample);
	return 0;//m_nSampleNum;
}

///////////////////
// setSampleByPointArrayProfile()
//	add a profile at point in ptArray into the m_sample
int SkBranchProfile::setSampleByPointArrayProfile(SkIntPointArray &ptArray, SkFloatArray &ortArray, 
												  Matrix<float> *pImgOrg, int half_wd)
{

	int ht = pImgOrg->rows();
	int wd = pImgOrg->cols();

	int pnX_v[MAX_CON_PROFILE_WD], pnY_v[MAX_CON_PROFILE_WD];
	double pdPrfl[MAX_CON_PROFILE_WD];

	int pf_x0, pf_x1, pf_y0, pf_y1;
	int x, y, dx, dy, xc, yc;
	int n = ptArray.getSize();
	
	assert(n <= 4096);


	int m = half_wd*2+1;
	Matrix<double> sample(m,n);
	assert(m <= MAX_CON_PROFILE_WD);

//	m_ptArray.removeAll();
	SkIntPoint p;

	double dOrt;
	for (int i = 0; i < n; i++)
	{//there is an overlap on the join point
		xc = ptArray[i].x;
		yc = ptArray[i].y;//all the axis point
		dOrt = ortArray[i];

		p.x = xc;
		p.y = yc;
//		m_ptArray.add(p);

		::verticalLineApproximately(dOrt, pf_x0, pf_y0, pf_x1, pf_y1, half_wd*2+1);
		int mm = ::rasterize_line_segment_keeping_order(pf_x0, pf_y0, pf_x1, pf_y1, pnX_v, pnY_v);
		if (mm > m)//might be longer due to rotation error 
			mm = m;

		for (int j = 0; j < mm; j++)//change m to mm, Dec. 03
		{
			dx = pnX_v[j];
			dy = pnY_v[j];//verticle piont of the 
			x = xc + dx;
			y = yc + dy;
			if (x < 0 || x >= wd || y < 0 || y >= ht)
				sample(j,i) = 0;//SK_INFINITE;
			else
				sample(j,i) = (*pImgOrg)(y,x);
		}
	}
	setSample(sample);
	return 0;//m_nSampleNum;
}

///////////////////
// computeImageCodingLengthGain()
double SkBranchProfile::computeImageCodingLengthGain()
{
	m_dRes0 = skSquareSumOfMatrix(&m_sample, 0);

	Matrix<double> smp;
	smp = m_sample - m_sample.Mean();
	m_dImgCL0 = skSquareSumOfMatrix(&smp, 0);

	int h, w;//profile height and width
	m_sample.GetDimension(h, w);

	Matrix<double> M, pl;
	M = getRidgeModelPrfl();

	pl.SetDimension(h, w);
	::setSameValue4EachRow(pl, M, 0);

	Matrix<double> res;
	res = m_sample - pl;

	m_dRes1 = skSquareSumOfMatrix(&res, 0);

	res = res - res.Mean();
	
	m_dImgCL1 = skSquareSumOfMatrix(&res, 0);

	m_dImgCLG = m_dImgCL0 - m_dImgCL1;

	m_dDeltaRes = m_dRes0 - m_dRes1;

	return m_dImgCLG;

}

///////////////////
// saveProfileAsBmp()
int SkBranchProfile::saveProfileAsBmp(const char *type)
{
	char work_dir[32], name[512], no[32];
	sprintf(work_dir, "prfl");
	_mkdir(work_dir);

	int h, w;//profile height and width
	m_sample.GetDimension(h, w);

	sprintf(name, "%s\\org0_%s.bmp", work_dir, type);
	::skSaveMatrixAsBitmap(name, m_sample0, m_dImgMean0, m_dImgStd0);
	sprintf(name, "%s\\org_%s.bmp", work_dir, type);
	::skSaveMatrixAsBitmap(name, m_sample, m_dImgMean0, m_dImgStd0);

	Matrix<double> M, pl;

	M = getSampleMean();
	pl.SetDimension(h, w);
	::setSameValue4EachRow(pl, M, 0);

	sprintf(name, "%s\\prfl0_%s.bmp", work_dir, type);
	::skSaveMatrixAsBitmap(name, pl, m_dImgMean0, m_dImgStd0);

	M = getRidgeModelPrfl();

	pl.SetDimension(h, w);
	::setSameValue4EachRow(pl, M, 0);

	sprintf(name, "%s\\prfl_%s.bmp", work_dir, type);
	::skSaveMatrixAsBitmap(name, pl, m_dImgMean0, m_dImgStd0);

	Matrix<double> res;
	res = m_sample - pl;

	sprintf(name, "%s\\res_%s.bmp", work_dir, type);
	::skSaveMatrixAsBitmap(name, res, m_dImgMean0, m_dImgStd0);

	return 0;

}


///////////////////
// getRidgeModelPrfl()
Matrix<double> SkBranchProfile::getRidgeModelPrfl()
{
	int h, w;
	m_sample0.GetDimension(h, w);
	Matrix<double> M;
	M.SetDimension(h, 1);

	double *p = m_ridgeModel.getModelIntensity();
	int i;
	for (i = 0; i < h; i++)
		M(i, 0) = p[i];

	return M;
}



/////////////////////////////////////////////////////////////////////
// class SkPrimBranch
int SkPrimBranch::m_pnX[SK_MAX_BRANCH_LENGTH];
int SkPrimBranch::m_pnY[SK_MAX_BRANCH_LENGTH];
int SkPrimBranch::m_nHalfWd = 3;

int SkPrimBranch::build(int x0, int y0, int x1, int y1, Matrix<float> &imgOrg)
{
	int n = ::rasterize_line_segment_keeping_order(x0, y0, x1, y1, m_pnX, m_pnY);
	SkIntPoint p;
	for (int i = 0; i < n; i++)
	{
		p.x = m_pnX[i];
		p.y = m_pnY[i];
		addPoint(p, 1);
	}

	setAllPointsAsOneSegment();
	computeProfile(&imgOrg);
	return 0;
}
double SkPrimBranch::getEndOrt(int end)//[-PI,PI)
{
	int nSegNum = getSegNum();
	SkSegment *pSeg;
	int x_in, y_in, x_out, y_out;
	SkIntPoint p;
	if (end == 0)
	{
		pSeg = getSegment(0);
		p = pSeg->getEndPoint(0);
		x_out = p.x;
		y_out = p.y;
		p = pSeg->getEndPoint(1);
		x_in = p.x;
		y_in = p.y;
	}
	else
	{
		pSeg = getSegment(nSegNum - 1);
		p = pSeg->getEndPoint(1);
		x_out = p.x;
		y_out = p.y;
		p = pSeg->getEndPoint(0);
		x_in = p.x;
		y_in = p.y;
	}
	double dx = x_out - x_in;
	double dy = y_out - y_in;
	double dOrt;
	dOrt = atan2(dy, dx);

	return dOrt;
}
///////////////////
// saveSketch2Matrix()
void SkPrimBranch::setOrt4EachPoint(Matrix<float>& ort)
{
	m_ortArray.removeAll();
	int n = m_ptArray.getSize();
	if (n <= 0)
		return;
	SkIntPoint p0 = m_ptArray[0];

	float dOrt0 = ort(p0.y, p0.y);

	double dError = 0.00001;
	if (dOrt0 >= PI/2-dError)//rotate the smaller angle to horizontal
		dOrt0 = -(PI-dOrt0);//[-PI/2, PI/2)
	m_ortArray.add(dOrt0);


	int e = 0;
	float dOrt;
	for (int i = 1; i < n; i++) 
	{
		dOrt = ort(m_ptArray[i].y, m_ptArray[i].x);
		dOrt = getSearchOrt(dOrt0, dOrt, e);//[0,2*PI), after considering relative oritation and growing direction
		m_ortArray.add(dOrt);
	}
}

///////////////////
// saveSketch2Matrix()
void SkPrimBranch::saveSketch2Matrix(Matrix<int> &mat, SkIntPoint &refPt) 
{
	int ht, wd;
	mat.GetDimension(ht, wd);
	int n = m_ptArray.getSize(); 
	SkIntPoint p; 
	int x, y;
	for (int i = 0; i < n; i++) 
	{
		p = m_ptArray.getAt(i); 
		x = refPt.x+p.x;
		y = refPt.y+p.y;
		if (x >= 0 && x < wd && y >= 0 && y < ht)
			mat(y,x)=0;
	} 
}
///////////////////
// saveSketch2Matrix()
void SkPrimBranch::saveProfile2Matrix(Matrix<float> &mat, SkIntPoint &refPt) 
{
	SkIntPoint p0, p1, dp;
	p0 = refPt;
	dp = getEndPoint(1);
	p1 = p0 + dp;
	assert(false);

//	skSaveSketches(&m_prfl, mat, 3, refPt.x, refPt.y);//draw line(0) or point(1)or occupy region(2) or profile(3)
}
///////////////////
// saveSketch2Matrix()
void SkPrimBranch::saveTransformedProfile2Matrix(Matrix<float> &mat, SkIntPoint &refPt) 
{
	SkIntPoint p0, p1, dp;
	p0 = refPt;
	dp = getEndPoint(1);
	p1 = p0 + dp;

	assert(false);

//	skSaveSketches(&m_prfl, mat, 3, refPt.x, refPt.y);//draw line(0) or point(1)or occupy region(2) or profile(3)
}
///////////////////
// saveSketch2Matrix()
void SkPrimBranch::saveOrgImg2Matrix(Matrix<float> &mat, SkIntPoint &refPt, Matrix<float> *pImgOrg, SkIntPoint &refPtOrg) 
{
	SkIntPoint dp;
	dp = getEndPoint(1);

	int pnX[4096], pnY[4096];
	int line_wd = m_prfl.getSampleWidth();
	int n = ::rasterize_line_segment(0, 0, dp.x, dp.y, pnX, pnY, line_wd);
	
	int wd0, ht0, wd1, ht1;
	mat.GetDimension(ht0, wd0);
	pImgOrg->GetDimension(ht1, wd1);

	int x, y, x0, y0, x1, y1;
	for (int i = 0; i < n; i++)
	{
		x = pnX[i];
		y = pnY[i];
		x0 = x + refPt.x;
		y0 = y + refPt.y;
		x1 = x + refPtOrg.x;
		y1 = y + refPtOrg.y;
		if (x0 >= 0 && x0 < wd0 && y0 >= 0 && y0 < ht0
			&& x1 >= 0 && x1 < wd1 && y1 >= 0 && y1 < ht1)
			mat(y0,x0) = (*pImgOrg)(y1,x1);
	}
}
///////////////////
// translate()
void SkPrimBranch::translate(SkIntPoint dp) 
{
	int n = m_ptArray.getSize(); 
	for (int i = 0; i < n; i++) 
		m_ptArray[i] = m_ptArray[i]+dp; 
}


///////////////////
// computeProfile(), the profile for each segment is computed 
int  SkPrimBranch::computeProfile(Matrix<float> *pImgOrg)
{
	int n = getSegNum();
	for (int i = 0; i < n; i++)
	{
		getSegment(i)->computeProfile(pImgOrg, m_nHalfWd);
	}
	return 0;
}

///////////////////
// computeProfile(), the profile for each segment is computed 
double SkPrimBranch::computeImageCodingLengthGain()
{
	m_dImgCLG = 0;
	int n = getSegNum();
	for (int i = 0; i < n; i++)
	{
		 m_dImgCLG += getSegment(i)->computeImageCodingLengthGain();
	}

//	if (n > 0)
//		m_dCLG /= n;
	
	return m_dImgCLG;
}

int SkPrimBranch::removeSamePointInPtArray()
{
	int nRemoved = 0;
	SkIntPoint p1, p2;
	for (int i = 0; i < m_ptArray.getSize()-1; i++)
	{
		p1 = m_ptArray[i];
		for (int j = i+1; j < m_ptArray.getSize(); j++)
		{
			p2 = m_ptArray[j];
			if (p1 == p2)
			{
				m_ptArray.removeAt(j);
				j--;
				nRemoved++;
			}
		}
	}
	return nRemoved;
}
///////////////////
// setOneSegment()
//	set all points as one segment
int SkPrimBranch::setAllPointsAsOneSegment()
{
	m_segArray.removeAll();
	int nRemoved = removeSamePointInPtArray();

	SkSegment seg;
	SkIntPoint p;
	for (int e = 0; e < 2; e++)
	{
		p = getEndPoint(e);
		seg.setEndPoint(p, e);
	}
	int	id = addSeg(seg);
	return m_segArray.getSize();
}
///////////////////
// computeSegments()
//	assume:  the curvature computed
//	1. find the curvature larger than a threshold, consider as corners
//	2. cut the branch into segments
//	3. compute and decide the type and parameters of the segments
int SkPrimBranch::computeSegments(double tol, double angtol, int linkrad)
{
	m_segArray.removeAll();
	int nRemoved = removeSamePointInPtArray();

	SkIntPoint p;
	int n = m_ptArray.getSize();
	assert(n <= SK_MAX_BRANCH_LENGTH);
	for (int i = 0; i < n; i++)
	{
		p = m_ptArray[i];
		m_pnY[i] = p.y;
		m_pnX[i] = p.x;
	}

	//m_pnLinesegEndIndex: //the index of the end point of each segment
	int m = skLineSeg(m_pnY, m_pnX, n, m_pnLinesegEndIndex, tol, angtol, linkrad);//the number of segments

	SkSegment seg;
	int s, e;
	int id;
	for (int i = 0; i < m; i++)
	{ 
		if (i == 0)
			s = 0;
		else
			s = m_pnLinesegEndIndex[i-1];
		e = m_pnLinesegEndIndex[i];
		seg.setEndPoint(m_ptArray[s], 0);
		seg.setEndPoint(m_ptArray[e], 1);
		id = addSeg(seg);
	}

	return m_segArray.getSize();
}
///////////////////
// recomputeSegmentsFromFloatPoints()
//	assume:  the segments and profiles computed
//	1. get the float points, 
//	2. change them to integers
//	3. set the new points
//  4. compute the segments
int SkPrimBranch::recomputeSegmentsFromFloatPoints(double tol, double angtol, int linkrad)
{
	m_ptArray.removeAll();

	int nSegNum = getSegNum();
	SkIntPoint p;
	SkFloatPoint pf;
	for (int i = 0; i < nSegNum; i++)
	{
		SkSegment *pSeg = getSegment(i);
		int nPtNum = pSeg->getPtNum();
		for (int j = 0; j < nPtNum; j++)
		{
			if (i != 0 && j==0)
				continue;//points at ends are overlapped, exept the first
			pf = pSeg->getFloatPoint(j);
			p.x = (int)(pf.x + 0.5);
			p.y = (int)(pf.y + 0.5);

			m_ptArray.add(p);
		}
	}
	removeSamePointInPtArray();
	computeSegments(tol, angtol, linkrad);
//	resetPointsFromSegments();
	return m_segArray.getSize();
}
int SkPrimBranch::resetPointsFromSegments()
{
	m_ptArray.removeAll();
	int m = getSegNum();
	for (int i = 0; i < m; i++)
	{ 
		SkSegment *pSeg = getSegment(i);
		SkIntPoint p0 = pSeg->getEndPoint(0);
		SkIntPoint p1 = pSeg->getEndPoint(1);

		int ptNum = pSeg->getPtNum();
		for (int j = 0; j < ptNum; j++)
		{
			if (i != 0 && j == 0)
				continue;
			SkIntPoint p = pSeg->getPoint(j);
			m_ptArray.add(p);
		}
	}
	return m_ptArray.getSize();
}
///////////////////
// computeCurvature()
int SkPrimBranch::computeCurvature(int nHalfWindowSize)
{
	double pCur[1024];
	int n = m_ptArray.getSize();
	assert(n <= 1024);
	SkIntPointArray ptArrayD;//derivative
	SkIntPoint dp(0,0);
	for (int i = 0; i < n-1; i++)
	{
		dp = m_ptArray[i+1] - m_ptArray[i];
		ptArrayD.add(dp);
	}
	ptArrayD.add(dp);//the last one equals to the second last

	skComputeCornerness(ptArrayD, pCur, nHalfWindowSize);

	m_curv.removeAll();
	float fCurv;
	for (int i = 0; i < n; i++)
	{
		fCurv = pCur[i];
		m_curv.add(fCurv);
	}
	return 0;
}
///////////////////
//computeLinearity()
int SkPrimBranch::computeLinearity()
{
	int pnX[1024];
	int pnY[1024];
	int n = m_ptArray.GetSize();
	assert(n <= 1024);
	for (int i = 0; i < n; i++)
	{
		pnX[i] = m_ptArray[i].x;
		pnY[i] = m_ptArray[i].y;
	}
	double a, b;
	m_dLinearity = ::computeLinearity(pnX, pnY, n, m_dOrt, a, b);
	m_dR = b*sin(m_dOrt-PI/2);
	return 0;
}

Matrix<double> &SkPrimBranch::getProfile() 
{ 
	return m_prfl.getSampleMean(); 
}
SkIntPoint SkPrimBranch::removeEndPoint(int e)
{ 
	SkIntPoint p;
	if (e==0) 
	{
		p = m_ptArray[0];
		m_ptArray.removeAt(0);
	}
	else 
	{
		int k = getPointNum()-1;
		p = m_ptArray[k];
		m_ptArray.removeAt(k); 
	}
	return p;
}

int SkPrimBranch::getEndPoints(int e, SkIntPointArray &ptArray, int nPtNum)
{// from the end to the center
	ptArray.removeAll();

	int n = getPointNum();
	if (nPtNum > n)
		nPtNum = n;
	if (e == 0)
	{
		for (int i = 0; i < nPtNum; i++)
		{
			ptArray.add(m_ptArray.getAt(i));
		}
	}
	else
	{
		for (int i = n-1; i >= n-nPtNum; i--)
		{
			ptArray.add(m_ptArray.getAt(i));
		}
	}
	return nPtNum;
}
double SkPrimBranch::getCornerAngle(int i)//i: corner index
{
	SkIntPoint p0, p1, p2;
	int k0 = getLinesegEndIndex(i);
	p0 = getPoint(k0);
	int k1;
	if (i == 0)
		k1 = 0;
	else
		k1 = getLinesegEndIndex(i-1);
	p1 = getPoint(k1);
	int k2 = getLinesegEndIndex(i+1);
	p2 = getPoint(k2);
	double dCornerAngle = ::skGetCornerAngle(p0.x, p0.y, p1.x, p1.y, p2.x, p2.y);
	return dCornerAngle;
}


/////////////////////////////////////////////////////////////////////
// class SkVirtualPrim
int SkVirtualPrim::addBranchIDAndEnd(int id, int e) 
{ 
	m_branchIDArray.add(id); 
	m_branchEndArray.add(e); 
	return m_branchIDArray.getSize()-1;
}
int SkVirtualPrim::removeBranch(int id, int e)
{
	int n = m_branchIDArray.getSize();
	bool bFind = false;
	for (int i = 0; i < n; i++)
	{
		if (m_branchIDArray[i] == id && m_branchEndArray[i] == e)
		{
			m_branchIDArray.removeAt(i);
			m_branchEndArray.removeAt(i);
			bFind = true;
			break;
		}
	}
	assert(bFind);
	return m_branchIDArray.getSize();
}


/////////////////////////////////////////////////////////////////////
// class SkPrimOccupancy


void SkPrimOccupancy::clearAll() 
{
	int ht, wd; 
	m_primIDofRefPt.GetDimension(ht,wd);
	for (int y=0;y<ht;y++)
	{
		for(int x=0;x<wd;x++)
		{
			m_primID(y,x).removeAll();
			m_branchNo(y,x).removeAll();
		} 
	}
	m_primIDofRefPt.Set(0);
}
void SkPrimOccupancy::setDimension(int ht, int wd) 
{ 
	m_primID.SetDimension(ht, wd); 
	m_branchNo.SetDimension(ht, wd); 
	m_primIDofRefPt.SetDimension(ht, wd); 
	m_primIDofRefPt.Set(0);
}
	
void SkPrimOccupancy::addOccupy(SkIntPoint p, int prim_id, int branch_no, bool bRefPt)
{ 
	m_primID(p.y,p.x).add(prim_id); 
	m_branchNo(p.y,p.x).add(branch_no);
	if (bRefPt) 
	{
		assert(m_primIDofRefPt(p.y,p.x)==0 || m_primIDofRefPt(p.y,p.x)==prim_id+1); 
		m_primIDofRefPt(p.y,p.x) = prim_id+1; 
	}
}
void SkPrimOccupancy::removeOccupy(SkIntPoint p, int prim_id, int branch_no, bool bRefPt)
{ 
	m_primID(p.y,p.x).remove(prim_id); 
	m_branchNo(p.y,p.x).remove(branch_no);
	if (bRefPt) 
	{ 
		m_primIDofRefPt(p.y,p.x)=0; 
	}
}


/////////////////////////////////////////////////////////////////////
// class SkPrimManager

///////////////////
// initializeOcpIDMatrices()
int SkPrimManager::initializeOcpIDMatrices()
{
	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);

	m_vpID.SetDimension(ht, wd);
	m_vpID.Set(-1);
	m_branchIDs.SetDimension(1, 1);//memory reallocated
	m_branchIDs.SetDimension(ht, wd);

	return 0;
}
///////////////////
// initializeOcpIDMatrices()
int SkPrimManager::recomputeOcpIDMatrices()
{
	initializeOcpIDMatrices();

	SkVPrimMap &vpMap = m_vpMap;

	SkIntPoint p0;
	int nPos = vpMap.getFirstPos();
	SkVPrimNode *pVPNode;
	while (nPos >= 0)
	{
		pVPNode = vpMap.getNextNode(nPos);
		if (!pVPNode->valid())
			continue;
		int id0 = pVPNode->getID();
		p0 = pVPNode->m_data.m_refPt;
		m_vpID(p0.y, p0.x) = id0;
	}

	SkBranchMap& bhMap = m_branchMap;
	nPos = bhMap.getFirstPos();
	SkBranchNode *pBhNode;
	while (nPos >= 0)
	{
		pBhNode = bhMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		int id0 = pBhNode->getID();
		SkPrimBranch &bh = pBhNode->m_data;
		int n = bh.getPointNum();
		for (int i = 0; i < n; i++)
		{
			p0 = bh.getPoint(i);
			m_branchIDs(p0.y, p0.x).add(id0);
		}
	}
	return 0;
}

///////////////////
// buildPrimitiveMap()
int SkPrimManager::buildPrimitiveMap()
{
	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);

	initializeOcpIDMatrices();

	findAllVirtualPrimitives(m_bound);
	findAllBranches(m_bound);


	return 0;
}
///////////////////
//	value 0-valid point
//	find degree > 2 connected points as virtual primitives
int SkPrimManager::findAllVirtualPrimitives(Matrix<int> &bound)
{
	int wd, ht;
	bound.GetDimension(ht, wd);
	
	SkIntPointArray nbrPtArray;

	// deal with junctions first, which have more than one connected points
	int cp;//conneceted points
	int x, y;
	for (y = 0; y < ht; y++)
	{
		for (x = 0; x < wd; x++)
		{
			if (bound(y, x) != 0)
				continue;
			cp = skFindConnectedPoints(x, y, nbrPtArray, bound);
			if (cp <= 2)
				continue;
			addOneVirtualPrimitive(x, y);//only add joints first
		}
	}

	return getVituralPrimitiveNum();
}

///////////////////
// value 0-valid point
// 1) the branches from vitural primitives
// 2) free branches
int SkPrimManager::findAllBranches(Matrix<int> &bound)
{
	SkPrimBranch branch;
	SkIntPointArray ptArray0, ptArray1;

	int x0, y0;
	int x1, y1;
	int id0, id1;
	SkVPrimNode *pNode;
	int nPos0;
	int nPos = m_vpMap.getFirstPos();
	while (nPos >= 0)
	{
		pNode = m_vpMap.getNextNode(nPos);
		x0 = pNode->m_data.m_refPt.x;
		y0 = pNode->m_data.m_refPt.y;

		id0 = pNode->getID();//vp/joint id
		int cp = skFindConnectedPoints(x0, y0, ptArray0, bound);
		assert (cp > 2);

	
		for (int i = 0; i < ptArray0.getSize(); i++)
		{
			x1 = ptArray0[i].x + x0;
			y1 = ptArray0[i].y + y0;
			
			if (m_branchIDs(y1,x1).getSize() > 0)
				continue;//

			ptArray1.removeAll();//clear it first
			ptArray1.add(pNode->m_data.m_refPt);//add reference first

			id1 = findPointsOfOneBranch(x1, y1, bound, m_vpID, ptArray1, id0);//the othe joint ID,-1 free end
			addOneBranch(ptArray1, id0, id1);
		}
	}

	// free branches
	int wd, ht;
//	Matrix<int> &bound = m_bound;
	bound.GetDimension(ht, wd);
	int x, y;
	SkIntPoint p;
	SkIntPointArray ptArray;
	for (y = 0; y < ht; y++)
	{
		for (x = 0; x < wd; x++)
		{
			if (bound(y,x) != 0)
				continue;
			if (m_branchIDs(y,x).getSize() > 0)
				continue;//already added
			int n = findEndPointOfConnection(x, y, bound, ptArray);
//			assert(n <= 2);
			p = ptArray[0];
			n = findSequentialPointOfConnection(p.x, p.y, bound, ptArray);//find the start point
			int id0 = -1;
			int id1 = -1;
			//for a circle, the branches created here are not closed, the two end points are not same, but are nbrs
			addOneBranch(ptArray, id0, id1);
		}
	}
	return getBranchNum();
}

///////////////////
int SkPrimManager::extendBranchEnd2AJunction(int id0, int e, int x1, int y1)
{
	SkBranchNode *pBhNode = m_branchMap.getNodeAt(id0);
	SkPrimBranch *pBranch = &(pBhNode->m_data);
	if (pBranch->getPrimID(e) >= 0)//free end
		return -1;
	

	// add the extended pionts
	addExtendedPoints2ABranch(pBhNode, e, x1, y1);
	//build the connection with vp
	int vpID = m_vpID(y1,x1);
	assert(vpID >= 0);
	SkVirtualPrim *pVP = m_vpMap.getAt(vpID);
	pVP->addBranchIDAndEnd(id0, e);
	pBranch->setPrimID(e, vpID);

	pBranch->computeSegments(m_param.m_dPixTol, m_param.m_dAngTol, m_param.m_nLinkRad);
	pBranch->computeProfile(&m_imgOrg);

	
	return 0;
}
///////////////////
int SkPrimManager::extendBranchEnd2ABranch(int id0, int e, int x1, int y1)
{
	SkBranchNode *pBhNode = m_branchMap.getNodeAt(id0);
	SkPrimBranch *pBranch0 = &(pBhNode->m_data);
	if (pBranch0->getPrimID(e) >= 0)
		return -1;

	int nBranchNum = m_branchIDs(y1, x1).getSize();

	int id1;
	for(int i = 0; i < nBranchNum; i++)
	{
		 id1 = m_branchIDs(y1, x1).getAt(i);
		 if (id1 != id0)
			 break;
	}

	addExtendedPoints2ABranch(pBhNode, e, x1, y1);


	if (id0 == id1)
	{//reach itself
		combineOneBranch(id0, e);
	}
	else
		combineTwoBranches(id0, e, id1);

	return 0;
}

///////////////////
int SkPrimManager::extendBranchEnd2ABranchExtension(int id0, 
													int end0, 
													int id1, 
													int end1, 
													int x1, int y1)
{
	SkBranchNode *pBhNode0 = m_branchMap.getNodeAt(id0);
	SkBranchNode *pBhNode1 = m_branchMap.getNodeAt(id1);

	// assure free end
	if(pBhNode0->m_data.getPrimID(end0) >= 0)
		return -1;
	if(pBhNode1->m_data.getPrimID(end1) >= 0)
		return -1;
	//extend this two braches
	addExtendedPoints2ABranch(pBhNode0, end0, x1, y1);
	addExtendedPoints2ABranch(pBhNode1, end1, x1, y1);
	// add a new VP, and build the relationships
	SkVirtualPrim vp;
	int vpID = m_vpMap.add(vp);
	SkVirtualPrim *pVP = m_vpMap.getAt(vpID);
	pVP->setRefPoint(x1,y1);
	m_vpID(y1,x1) = vpID;//update vpID map

	//set branch vp relationship
//	int id0 = pBhNode0->getID();
//	int id1 = pBhNode1->getID();
	pVP->addBranchIDAndEnd(id0, end0);
	pVP->addBranchIDAndEnd(id1, end1);

	pBhNode0->m_data.setPrimID(end0, vpID);
	pBhNode1->m_data.setPrimID(end1, vpID);
	return 0;
}


///////////////////
int SkPrimManager::addExtendedPoints2ABranch(SkBranchNode *pBhNode, int e, int x1, int y1)
{
	SkPrimBranch *pBranch = &(pBhNode->m_data);
	int id0 = pBhNode->getID();
	// add the extended pionts
	SkIntPoint p0 = pBranch->getEndPoint(e);
	int x0 = p0.x;
	int y0 = p0.y;

	SkIntPoint p;
	int n = ::rasterize_line_segment_keeping_order(x0, y0, x1, y1, m_pnX, m_pnY);
	for (int i = 1; i < n; i++)
	{
		p.x = m_pnX[i];
		p.y = m_pnY[i];
		int nReturn = pBranch->addPoint(p, e);
		if (nReturn >= 0)//add sucessfully
			m_branchIDs(p.y,p.x).add(id0);
	}
	return 0;
}




int SkPrimManager::combineConnectedFreeEndPointsIntoOneBranch()
{
	//check each branch
	SkBranchMap& bhMap = m_branchMap;
	int nPos = bhMap.getFirstPos();
	SkBranchNode *pBhNode;
	while (nPos >= 0)
	{
		pBhNode = bhMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		int id0 = pBhNode->getID();
		SkPrimBranch &bh = pBhNode->m_data;
		for (int e = 0; e < 2; e++)
		{//for each end
			if (bh.getPrimID(e) < 0)
			{
				if (combineConnectedFreeEndPointsIntoOneBranch(id0, e))
					e--;//we might need to check that end again. (not debugged Nov. 18)
			}

		}
	}
	return 0;
}
int SkPrimManager::combineConnectedFreeEndPointsIntoOneBranch(int id0, int e0)
{
	int nCombined = 0;

	int ht, wd;
	m_imgOrg.GetDimension(ht,wd);

	SkBranchNode *pBhNode0 = m_branchMap.getNodeAt(id0);
	SkPrimBranch &branch0 = pBhNode0->m_data;
	SkIntPoint endPt0 = branch0.getEndPoint(e0);

	int pnPossX[256],pnPossY[256], pnPossID[256];
	int nPoss = 0;//possible x, y
	//check sk ocp
	int x0 = endPt0.x;
	int y0 = endPt0.y;
	int dx, dy, x, y;
	for (dx = -1; dx <= 1; dx++)
	{
		x = x0 + dx;
		if (x < 0 || x >= wd)
			continue;
		for (dy = -1; dy <= 1; dy++)
		{
			y = y0 + dy;
			if (y < 0 || y >= ht)
				continue;
			oval_IntArray &idArr = m_branchIDs(y,x);
			if (idArr.getSize() <= 0)
				continue;
			if (idArr.getSize() == 1 && idArr[0] == id0)
				continue;
			for (int j = 0; j < idArr.getSize(); j++)
			{
				if (idArr[j] == id0)
					continue;
				pnPossX[nPoss] = x;
				pnPossY[nPoss] = y;
				pnPossID[nPoss] = idArr[j];
				nPoss++;
				assert(nPoss <= 256);
			}
		}
	}

	SkIntPoint p0;
	int id1;
	for (int i = 0; i < nPoss; i++)
	{
		p0.x = pnPossX[i];
		p0.y = pnPossY[i];
		id1 = pnPossID[i];
		int e1 = checkBranchEndPoint(id1, p0.x, p0.y);
		if (e1 >= 0)
		{

			SkPrimBranch *pBranch1 = m_branchMap.getAt(id1);
			if (pBranch1->getPrimID(e1) < 0)
			{//free end
				branch0.addPoint(p0, e0);
				m_branchIDs(p0.y, p0.x).add(id0);
				int bhID[2], end[2];
				bhID[0] = id0;
				bhID[1] = id1;
				end[0] = e0;
				end[1] = e1;
				combineTwoContinuedBranches(bhID, end);
				nCombined = true;
				break;
			}
			else if (m_vpID(p0.y, p0.x) >= 0)
			{//vertex
				branch0.addPoint(p0, e0);
				m_branchIDs(p0.y, p0.x).add(id0);
				int vpID = m_vpID(p0.y, p0.x);
				branch0.setPrimID(e0, vpID);
				SkVirtualPrim *pVP = m_vpMap.getAt(vpID);
				pVP->addBranchIDAndEnd(id0, e0);
				break;
			}
		}
	}
	return nCombined;
}

int SkPrimManager::combineConnectedFreeEndPointsByANewVP()
{
	//check each branch
	SkBranchMap& bhMap = m_branchMap;
	int nPos = bhMap.getFirstPos();
	SkBranchNode *pBhNode;
	while (nPos >= 0)
	{
		pBhNode = bhMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		int id0 = pBhNode->getID();
		SkPrimBranch &bh = pBhNode->m_data;
		for (int e = 0; e < 2; e++)
		{//for each end
			if (bh.getPrimID(e) < 0)
				combineConnectedFreeEndPointsByANewVP(id0, e);

		}
	}
	return 0;
}
int SkPrimManager::combineConnectedFreeEndPointsByANewVP(int id0, int e0)
{

	int ht, wd;
	m_imgOrg.GetDimension(ht,wd);

	SkBranchNode *pBhNode0 = m_branchMap.getNodeAt(id0);
	SkPrimBranch &branch0 = pBhNode0->m_data;
	SkIntPoint endPt0 = branch0.getEndPoint(e0);

	int pnPossX[256],pnPossY[256], pnPossID[256];
	int nPoss = 0;//possible x, y
	//check sk ocp
	int x0 = endPt0.x;
	int y0 = endPt0.y;
	int dx, dy, x, y;
	for (dx = -1; dx <= 1; dx++)
	{
		x = x0 + dx;
		if (x < 0 || x >= wd)
			continue;
		for (dy = -1; dy <= 1; dy++)
		{
			y = y0 + dy;
			if (y < 0 || y >= ht)
				continue;
			oval_IntArray &idArr = m_branchIDs(y,x);
			if (idArr.getSize() <= 0)
				continue;
			if (idArr.getSize() == 1 && idArr[0] == id0)
				continue;
			for (int j = 0; j < idArr.getSize(); j++)
			{
				if (idArr[j] == id0)
					continue;
				pnPossX[nPoss] = x;
				pnPossY[nPoss] = y;
				pnPossID[nPoss] = idArr[j];
				nPoss++;
				assert(nPoss <= 256);
			}
		}
	}

	SkIntPoint p0;
	int id1;
	for (int i = 0; i < nPoss; i++)
	{
		p0.x = pnPossX[i];
		p0.y = pnPossY[i];
		id1 = pnPossID[i];
		int e1 = checkBranchEndPoint(id1, p0.x, p0.y);
		if (e1 >= 0)
		{//end point

			SkPrimBranch *pBranch1 = m_branchMap.getAt(id1);
			if (pBranch1->getPrimID(e1) < 0)
			{//free end
				branch0.addPoint(p0, e0);
				m_branchIDs(p0.y, p0.x).add(id0);
				int bhID[2], end[2];
				bhID[0] = id0;
				bhID[1] = id1;
				end[0] = e0;
				end[1] = e1;
				connectTwoContinuedBranchesByANewVP(bhID, end);
				break;
			}
			else if (m_vpID(p0.y, p0.x) >= 0)
			{//meet a VP
				branch0.addPoint(p0, e0);
				m_branchIDs(p0.y, p0.x).add(id0);
				int vpID = m_vpID(p0.y, p0.x);
				branch0.setPrimID(e0, vpID);
				SkVirtualPrim *pVP = m_vpMap.getAt(vpID);
				pVP->addBranchIDAndEnd(id0, e0);
				break;
			}
		}
	}
	return 0;
}
bool SkPrimManager::isAtBranchExtension(SkPrimBranch *pBranch0, int end0, int x, int y, double m_dPixTol)
{
	return true;
}

int SkPrimManager::checkBranchEndPoint(int id0, int x, int y)
{
	SkBranchNode *pBhNode0 = m_branchMap.getNodeAt(id0);
	SkPrimBranch &branch0 = pBhNode0->m_data;
	for (int e = 0; e < 2; e++)
	{
		SkIntPoint endPt0 = branch0.getEndPoint(e);
		if (endPt0.x == x && endPt0.y == y)
			return e;
	}
	return -1;
}

int SkPrimManager::removeDegree1And2VP()
{

	SkVPrimNode *pNode;
	int nPos = m_vpMap.getFirstPos();
	while (nPos >= 0)
	{
		pNode = m_vpMap.getNextNode(nPos);
		if (!pNode->valid())
			continue;
		SkVirtualPrim &vp = pNode->m_data;
		int nBranch = vp.getBranchNum();
		if (nBranch == 1)
		{
//			assert(false);
			int bhID = vp.getBranchID(0);
			int end = vp.getBranchEnd(0);
			SkPrimBranch *pBranch;
			pBranch = m_branchMap.getAt(bhID);
			pBranch->setPrimID(end, -1);
			pNode->setValid(false);

		}
		else if (nBranch == 2)
		{
			int bhID[2], end[2];
			for (int i = 0; i < 2; i++)
			{
				bhID[i] = vp.getBranchID(i);
				end[i] = vp.getBranchEnd(i);
			}
			combineTwoContinuedBranches(bhID, end);
			pNode->setValid(false);
		}
	}
	return 0;
}
///////////////////
// return the ID
int SkPrimManager::addOneVirtualPrimitive(int x, int y)
{
	SkVirtualPrim vp;
	vp.setRefPoint(x, y);

	int id = m_vpMap.add(vp);

	m_vpID(y, x) = id;

	return id;
}
///////////////////
int SkPrimManager::addOneBranch(SkIntPointArray &ptArray, int vpID0, int vpID1)
{
	SkPrimBranch branch;
	int id = m_branchMap.add(branch);
	SkPrimBranch *pBranch = m_branchMap.getAt(id);

	pBranch->setPrimID(0, vpID0);
	pBranch->setPrimID(1, vpID1);

	int n = ptArray.getSize();
	SkIntPoint p;
	for (int i = 0; i < n; i++)
	{
		p.x = ptArray[i].x;
		p.y = ptArray[i].y;

		if (i > 0 && i < n-1)
			assert(m_branchIDs(p.y,p.x).getSize() <= 0);

		pBranch->addPoint(p,1);
		m_branchIDs(p.y,p.x).add(id);
	}

	SkVirtualPrim *pVP;
	if (vpID0 >= 0)
	{
		pVP = m_vpMap.getAt(vpID0);
		pVP->addBranchIDAndEnd(id, 0);
	}
	if (vpID1 >= 0)
	{
		pVP = m_vpMap.getAt(vpID1);
		pVP->addBranchIDAndEnd(id, 1);
	}
	return m_branchMap.getSize();
	
}

///////////////////
// value 0-valid point
// 1) remove the points from m_branchIDs
// 2) remove from the nbr of vps if there is any
// 4) set branch node invalid
int SkPrimManager::removeOneBranch(int id)
{
	SkBranchNode *pBhNode = m_branchMap.getNodeAt(id);
	SkPrimBranch *pBranch = &(pBhNode->m_data);

	SkIntPointArray &ptArray = pBranch->getPointArray();
	int n = ptArray.getSize();
	SkIntPoint p;
	for (int i = 0; i < n; i++)
	{
		p.x = ptArray[i].x;
		p.y = ptArray[i].y;

		m_branchIDs(p.y,p.x).remove(id);
	}

	int vpID;
	SkVirtualPrim *pVP;
	for (int e = 0; e < 2; e++)
	{
		vpID = pBranch->getPrimID(e);
		if (vpID < 0)
			continue;
		pVP = m_vpMap.getAt(vpID);

		removeOneBranchFromOneVirtualPrimitive(id, e, vpID);
	}

	pBhNode->setValid(false);
	return m_branchMap.getSize();
	
}


///////////////////
// 1) remove branch from the vp
// 2) if vp has only two branches:
//    a) combine the two branches
//    b) remove the vp
int SkPrimManager::	removeOneBranchFromOneVirtualPrimitive(int bhID0, int e0, int vpID)
{
	if (vpID == 11)
		vpID += 0;

	SkVirtualPrim *pVP = m_vpMap.getAt(vpID);
	
	pVP->removeBranch(bhID0, e0);
	
	int nBhNum = pVP->getBranchNum();
	if (nBhNum == 1 || nBhNum > 2)//too many or only one, branches
		return m_vpMap.getSize();
	if (nBhNum == 0)
	{
		//remove vp from ocp matrix and vpmap
		SkIntPoint p0 = pVP->m_refPt;
		m_vpID(p0.y, p0.x) = -1;
		SkVPrimNode *pVpNode = m_vpMap.getNodeAt(vpID);
		pVpNode->setValid(false);

		return m_branchMap.getSize();
	}

	return m_vpMap.getSize();

}

///////////////////
// branch id0, meets branch id1 by the end of e0, at point (x0,y0), conbine them
int SkPrimManager::combineTwoBranches(int id0, int e0, int id1)
{
	SkPrimBranch *pBranch0 = m_branchMap.getAt(id0);
	// find branch1 length 
	SkPrimBranch *pBranch1 = m_branchMap.getAt(id1);
	SkIntPointArray &ptArray = pBranch1->getPointArray();
	SkIntPoint p0 = pBranch0->getEndPoint(e0);
	int pos = ptArray.memberPos(p0);
	assert(pos >= 0);
	int size = ptArray.getSize();

	if (pos != 0 && pos != size-1)
	{//not the end
		// get the vp of branch1
		int vpID1_bh1 = pBranch1->getPrimID(1);
		//new vp
		SkVirtualPrim vp;
		int vpID = m_vpMap.add(vp);
		SkVirtualPrim *pVP = m_vpMap.getAt(vpID);
		pVP->setRefPoint(p0);
		m_vpID(p0.y,p0.x) = vpID;//update vpID map

		//new branch
		SkPrimBranch bh2;
		int id2 = m_branchMap.add(bh2);
		// after add new branches, the memory might be reallocated
		pBranch0 = m_branchMap.getAt(id0);
		// find branch1 length 
		pBranch1 = m_branchMap.getAt(id1);
		SkIntPointArray &ptArray = pBranch1->getPointArray();
		SkPrimBranch *pBranch2 = m_branchMap.getAt(id2);
		for (int i = pos; i < size; i++)
		{
			SkIntPoint& pp = ptArray.getAt(i);
			pBranch2->addPoint(pp, 1);//add to the end
			m_branchIDs(pp.y,pp.x).remove(id1);
			m_branchIDs(pp.y,pp.x).add(id2);//update branchMap
		}
		SkIntPoint& pp = ptArray.getAt(pos);
		m_branchIDs(pp.y,pp.x).add(id1);//this point is shared

		// truncate branch1
		pBranch1->getPointArray().setSize(pos+1);

		//set branch vp relationship
		pVP->addBranchIDAndEnd(id0, e0);
		pVP->addBranchIDAndEnd(id1, 1);
		pVP->addBranchIDAndEnd(id2, 0);

		pBranch0->setPrimID(e0, vpID);
		pBranch1->setPrimID(1, vpID);
		pBranch2->setPrimID(0, vpID);
		pBranch2->setPrimID(1, vpID1_bh1);
		//related vpID1_bh1 also need to be updated
		if (vpID1_bh1 >= 0)
		{
			SkVirtualPrim *pVP1 = m_vpMap.getAt(vpID1_bh1);
			pVP1->removeBranch(id1, 1);
			pVP1->addBranchIDAndEnd(id2, 1);
		}
		pBranch2->computeSegments(m_param.m_dPixTol, m_param.m_dAngTol, m_param.m_nLinkRad);
	}
	else
	{//not needed for a new vp
		int bhID[2], end[2];
		if (pos == 0)
		{
			end[1] = 0;
//			assert(pBranch1->getPrimID(0) < 0);
		}
		if (pos == size-1)
		{
			end[1] = 1;
//			assert(pBranch1->getPrimID(1) < 0);
		}
		bhID[0] = id0;
		bhID[1] = id1;
		end[0] = e0;
		//keep branch0, remove branch1
		combineTwoContinuedBranches(bhID, end);
	}

	return 0;
}

///////////////////
// branch id0, meets itself by the end of e0, at point (x0,y0), conbine them
int SkPrimManager::combineOneBranch(int id0, int e0)
{
	SkPrimBranch *pBranch0 = m_branchMap.getAt(id0);
	SkIntPoint p0 = pBranch0->getEndPoint(e0);
	SkIntPointArray &ptArray0 = pBranch0->getPointArray();

	SkIntPointArray ptArray;//ptArray = ptArray0, 
	ptArray = ptArray0;//this is a copy

	//there should be two save points in the array, find the position of them
	int size = ptArray.getSize();
	int pos0 = ptArray.memberPos(p0);//end point position in the point array
	if (e0 == 0)
		assert(pos0 == 0);//end point is the first point
	ptArray.reverse();
	int pos1 = ptArray.memberPos(p0);
	pos1 = size - pos1 - 1;
	if (e0 == 1)
		assert(pos1 == size-1);//end point is the last point
	assert(pos0 >= 0 && pos1 >= 0);
	if (pos0 == pos1)
	{
		assert(false);
		return -1;//not possible, might be a bug if this happens
	}

	if (pos0 + pos1 == size-1)	
	{//since one of them must be the end, then if sum is size-1, 
		// then both are the end, form a circle

		//whenever two branches overlap, there should be a VP
		//new vp
		SkVirtualPrim vp;
		int vpID = m_vpMap.add(vp);
		SkVirtualPrim *pVP = m_vpMap.getAt(vpID);
		pVP->setRefPoint(p0);
		pVP->addBranchIDAndEnd(id0, 0);
		pVP->addBranchIDAndEnd(id0, 1);
		m_vpID(p0.y,p0.x) = vpID;//update vpID map

		pBranch0->setPrimID(0, vpID);
		pBranch0->setPrimID(1, vpID);
	}
	else 
	{//not the end
		// get the vp of the other end of the branch
		int vpID1 = pBranch0->getPrimID(1-e0);
		//new vp
		SkVirtualPrim vp;
		int vpID = m_vpMap.add(vp);
		SkVirtualPrim *pVP = m_vpMap.getAt(vpID);
		pVP->setRefPoint(p0);
		m_vpID(p0.y,p0.x) = vpID;//update vpID map

		//new branch
		SkPrimBranch bh2;
		int id2 = m_branchMap.add(bh2);
		// after add new branches, the memory might be reallocated
		pBranch0 = m_branchMap.getAt(id0);
		// find branch1 length 
		SkIntPointArray &ptArray0 = pBranch0->getPointArray();
		SkPrimBranch *pBranch2 = m_branchMap.getAt(id2);
		if (e0 == 0)
		{//end is head
			assert(pos0 == 0);
			for (int i = pos1; i < size; i++)
			{
				SkIntPoint& pp = ptArray0.getAt(i);
				if (pBranch2->addPoint(pp, 1) >= 0)//add to the end
				{
					m_branchIDs(pp.y,pp.x).remove(id0);
					m_branchIDs(pp.y,pp.x).add(id2);//update branchMap
				}
			}
			SkIntPoint& pp = ptArray0.getAt(pos1);
			m_branchIDs(pp.y,pp.x).add(id0);//this point is shared

			// truncate branch0
			ptArray0.setSize(pos1+1);

			//set branch vp relationship
			pVP->addBranchIDAndEnd(id0, 0);
			pVP->addBranchIDAndEnd(id0, 1);
			pVP->addBranchIDAndEnd(id2, 0);

			pBranch0->setPrimID(0, vpID);
			pBranch0->setPrimID(1, vpID);
			pBranch2->setPrimID(0, vpID);
			pBranch2->setPrimID(1, vpID1);

		}
		else
		{//end is the tail
			assert(pos1 == size-1);
			for (int i = pos1; i >=0; i--)
			{
				SkIntPoint& pp = ptArray0.getAt(i);
				if (pBranch2->addPoint(pp, 1) >= 0)//add to the end
				{
					m_branchIDs(pp.y,pp.x).remove(id0);
					m_branchIDs(pp.y,pp.x).add(id2);//update branchMap
				}
			}
			SkIntPoint& pp = ptArray0.getAt(pos1);
			m_branchIDs(pp.y,pp.x).add(id0);//this point is shared

			// truncate branch0
			for (int i = pos1; i < size; i++)
			{
				ptArray0[i-pos1] = ptArray0[i];
			}
			ptArray0.setSize(size-pos1+1);

			//set branch vp relationship
			pVP->addBranchIDAndEnd(id0, 0);
			pVP->addBranchIDAndEnd(id0, 1);
			pVP->addBranchIDAndEnd(id2, 0);

			pBranch0->setPrimID(0, vpID);
			pBranch0->setPrimID(1, vpID);
			pBranch2->setPrimID(0, vpID);
			pBranch2->setPrimID(1, vpID1);
		}
		pBranch2->computeSegments(m_param.m_dPixTol, m_param.m_dAngTol, m_param.m_nLinkRad);// not used
	}
	return 0;
}


///////////////////
// two branches with id0, id1 meet at (x0, y0) by ends e0, e1
// use the ppBranch[0] as new branch, the second one will be removed
int SkPrimManager::combineTwoContinuedBranches(int bhID[2], int end[2])
{

	SkPrimBranch *ppBranch[2];
	for (int e = 0; e < 2; e++)
		ppBranch[e] = m_branchMap.getAt(bhID[e]);
	
	SkIntPoint pp0[2];
	for (int i = 0; i < 2; i++)
		pp0[i] = ppBranch[i]->getEndPoint(end[i]);

	assert(pp0[0] == pp0[1]);

	SkIntPoint p0, p1, p;
	if (bhID[0] == bhID[1])// a closed curve, share this primitive
		return m_branchMap.getSize();

	{//join two branches
		int vpID0, vpID1;
		//use the ppBranch[0] as new branch

		// consider ppBranch[0] and ppBranch[1] 's VP 
		SkIntPointArray& ptArray0 = ppBranch[0]->getPointArray();
		vpID0 = ppBranch[0]->getPrimID(1-end[0]);
		if (vpID0 >= 0)
		{
			SkVirtualPrim *pVP0 = m_vpMap.getAt(vpID0);
			pVP0->removeBranch(bhID[0], 1-end[0]);
			pVP0->addBranchIDAndEnd(bhID[0], 0);
		}

		if (end[0] == 0)//inverse the point
			ptArray0.reverse();


		SkIntPointArray& ptArray1 = ppBranch[1]->getPointArray();
		if (end[1] == 1)//inverse the point
			ptArray1.reverse();
		vpID1 = ppBranch[1]->getPrimID(1-end[1]);
		
		if (vpID1 >= 0)
		{
			SkVirtualPrim *pVP1 = m_vpMap.getAt(vpID1);
			pVP1->removeBranch(bhID[1], 1-end[1]);
			pVP1->addBranchIDAndEnd(bhID[0], 1);
		}

		ppBranch[0]->setPrimID(0, vpID0);
		ppBranch[0]->setPrimID(1, vpID1);


		p = ptArray1.getAt(0);
		m_branchIDs(p.y,p.x).remove(bhID[1]);

		int n1 = ptArray1.getSize();
		for (int i = 1; i < n1; i++)
		{
			p = ptArray1.getAt(i);
			ptArray0.add(p);
			m_branchIDs(p.y,p.x).remove(bhID[1]);
			m_branchIDs(p.y,p.x).add(bhID[0]);
		}
		m_branchMap.getNodeAt(bhID[1])->setValid(false);
	}
	return m_branchMap.getSize();
}
///////////////////
// two branches with id0, id1 meet at (x0, y0) by ends e0, e1
// Add a new VP to connec them
int SkPrimManager::connectTwoContinuedBranchesByANewVP(int bhID[2], int end[2])
{

	SkPrimBranch *ppBranch[2];
	int i;
	for (i = 0; i < 2; i++)
		ppBranch[i] = m_branchMap.getAt(bhID[i]);
	
	SkIntPoint pp0[2];
	for (i = 0; i < 2; i++)
		pp0[i] = ppBranch[i]->getEndPoint(end[i]);

	assert(pp0[0] == pp0[1]);

	SkIntPoint p0, p1, p;
	if (bhID[0] == bhID[1])// a closed curve, share this primitive
		return m_branchMap.getSize();

	int vpID = addOneVirtualPrimitive(pp0[0].x, pp0[0].y);
	SkVirtualPrim *pVP = m_vpMap.getAt(vpID);

	for (i = 0; i < 2; i++)
	{
		assert(ppBranch[i]->getPrimID(end[i]) < 0);
		ppBranch[i]->setPrimID(end[i], vpID);
		pVP->addBranchIDAndEnd(bhID[i], end[i]);
	}
	return m_vpMap.getSize();
}

///////////////////
// return possible second vpID
int SkPrimManager::findPointsOfOneBranch(int x1, int y1, Matrix<int> &bound, Matrix<int> &vpID, 
										 SkIntPointArray &ptArray, int vpID0)
{
	SkIntPointArray nbrPtArray;
	SkIntPoint p0, p1, p;
	p1.x = x1; p1.y = y1;
	while (true)
	{
		int vpID1 = vpID(p1.y, p1.x);
		if ( vpID1 >= 0)
		{
			if (vpID1 == vpID0)
			{
				//if this happen, it is a circle
				ptArray.add(p1);
				return vpID1;//reach old one
			}
			else
			{//reach another vp
				ptArray.add(p1);
				return vpID1;
			}
		}//reach a vp
		int cp = skFindConnectedPoints(p1.x, p1.y, nbrPtArray, bound);//conneceted points
		assert(cp == 2 || cp == 1);
		if (cp == 1)
		{// reach end
			ptArray.add(p1);
			return -1;
		}
		p0 = ptArray[ptArray.getSize()-1];
		p.x = nbrPtArray[0].x + p1.x;
		p.y = nbrPtArray[0].y + p1.y;
		if (p == p0)
		{
			p.x = nbrPtArray[1].x + p1.x;
			p.y = nbrPtArray[1].y + p1.y;
		}		
		ptArray.add(p1);

		p1 = p;
	}
	assert(false);
	return -1;
}
int SkPrimManager::removeCrossedConnection(oval_Array<SkEndConnection> &cnctArray)
{
	int nRemoved = 0;
	for (int i = 0; i < cnctArray.getSize(); i++)
	{
		if (crossedEndConnection(cnctArray[i]))
		{
			cnctArray.removeAt(i);
			i--;
			nRemoved++;
		}
		
	}
	return nRemoved;
}
///////////////////
bool SkPrimManager::crossedEndConnection(SkEndConnection &cnct)
{
	SkPrimBranch *pBranch0 = m_branchMap.getAt(cnct.m_id0);
	int end0 = cnct.m_end0;
	SkIntPoint p0 = pBranch0->getEndPoint(end0);
	int x = cnct.m_x;
	int y = cnct.m_y;
	cnct.m_dDist0 = ::distanceBetweenPoints(p0.y, p0.x, y, x);

	if (x == 128 && y == 78)
		x += 0;

	double dOrt0 = pBranch0->getEndOrt(end0);//end ort
	double dx = x - p0.x;
	double dy = y - p0.y;
	double dOrt0_new = atan2(dy, dx);//new connection ort
	cnct.m_dDeltaOrt0 = skAbsoluteAngleDifferenceOfTwoVectors(dOrt0, dOrt0_new);

	if (cnct.m_type == 1 && cnct.m_id0 == cnct.m_id1)
	{
		SkIntPointArray &arr = pBranch0->getPointArray();
		int pos0 = arr.memberPos(p0);
		assert(pos0 >= 0);
		SkIntPoint p1;
		p1.x = x;
		p1.y = y;
		int pos1 = arr.memberPos(p1);
		assert(pos1 >= 0);
		double dMinDis = m_param.m_dExtRegionHalfLeft * 1.5;
		if (fabs(double(pos1-pos0)) <= dMinDis)
			return true;//same sketch and too close or wrong direction
	}

	int nPtNum;
	if (cnct.m_type == 0)
	{//connect to a VP
		if (crossedConnection(p0.x, p0.y, x, y))
			return true;
		return false;
	}
	else if (cnct.m_type == 1)
	{//connect to a sketch
		if (crossedConnection(p0.x, p0.y, x, y))
			return true;
		return false;
	}
	else if (cnct.m_type == 2)
	{//connect to a sketch extension
		if (cnct.m_id0 == cnct.m_id1 && cnct.m_end0 == cnct.m_end1)
			return true;
		if (crossedConnection(p0.x, p0.y, x, y))
			return true;

		SkPrimBranch *pBranch1 = m_branchMap.getAt(cnct.m_id1);
		int end1 = cnct.m_end1;

		SkIntPoint p1 = pBranch1->getEndPoint(end1);

		if (crossedConnection(p1.x, p1.y, x, y))
			return true;

		if (crossedConnection(p0.x, p0.y, x, y, p1.x, p1.y, x, y))
			return true;

		return false;

	}
	else
	{//not defined
		assert(false);
		return false;
	}
	return true;
}
bool SkPrimManager::crossedConnection(int x1, int y1, int x2, int y2)
{
	int wd, ht;
	m_branchIDs.GetDimension(ht, wd);
	int nPtNum = ::rasterize_line_segment_keeping_order(x1, y1, x2, y2, m_pnX, m_pnY);
	int nBound = 2;
//	if (nPtNum <= 2)
//		nBound = 0;
	for (int i = nBound; i < nPtNum-nBound; i++)
	{
		int x = m_pnX[i];
		int y = m_pnY[i];
		if (m_branchIDs(y,x).getSize() >= 1)
			return true;
		int x1 = x + 1;
		if (x1 >= wd)
			continue;
		if (m_branchIDs(y,x1).getSize() >= 1)
			return true;
	}
	return false;
}
bool SkPrimManager::crossedConnection(int x1_s, int y1_s, int x1_e, int y1_e,
									  int x2_s, int y2_s, int x2_e, int y2_e)
{
	int pnX1[1024*16], pnY1[1024*16];
	int pnX2[1024*16], pnY2[1024*16];

	int nPtNum1 = ::rasterize_line_segment_keeping_order(x1_s, y1_s, x1_e, y1_e, pnX1, pnY1);
	int nPtNum2 = ::rasterize_line_segment_keeping_order(x2_s, y2_s, x2_e, y2_e, pnX2, pnY2);
	int nBound = 1;
	for (int i = nBound; i < nPtNum1-nBound; i++)
	{
		for (int j = nBound; j < nPtNum2-nBound; j++)
		{
			if (pnX1[i] == pnX2[j] && pnY1[i] == pnY2[j])
				return true;
		}
	}
	return false;
}

///////////////////
// addPrimitivesFromOtherManager()
int SkPrimManager::addPrimitivesFromOtherManager(SkPrimManager &primMan)
{
	SkVPrimMap &vpMap = primMan.m_vpMap;

	int max_vp_id = vpMap.getValidMaxID();
	int *pnIDLUT = new int[max_vp_id+1];//id look up table
	memset(pnIDLUT, -1, sizeof(int)*(max_vp_id + 1));

	SkIntPoint p0;
	int nPos = vpMap.getFirstPos();
	SkVPrimNode *pVPNode;
	while (nPos >= 0)
	{
		pVPNode = vpMap.getNextNode(nPos);
		if (!pVPNode->valid())
			continue;
		int id0 = pVPNode->getID();
		p0 = pVPNode->m_data.m_refPt;
		int id1 = addOneVirtualPrimitive(p0.x, p0.y);
		pnIDLUT[id0] = id1;
	}

	SkBranchMap& bhMap = primMan.m_branchMap;
	nPos = bhMap.getFirstPos();
	SkBranchNode *pBhNode;
	while (nPos >= 0)
	{
		pBhNode = bhMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		SkPrimBranch &bh = pBhNode->m_data;
		int vpID0 = bh.getPrimID(0);
		if (vpID0 >= 0)
			vpID0 = pnIDLUT[vpID0];//id changed
		int vpID1 = bh.getPrimID(1);
		if (vpID1 >= 0)
			vpID1 = pnIDLUT[vpID1];//id changed
		addOneBranch(bh.getPointArray(), vpID0, vpID1);
	}
	delete []pnIDLUT;
	return 0;
}
int SkPrimManager::computeImageCodingLengthGain4EachBranch()
{
	SkBranchMap& bhMap = m_branchMap;

	int nPos = bhMap.getFirstPos();
	SkBranchNode *pBhNode;
	while (nPos >= 0)
	{
		pBhNode = bhMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		SkPrimBranch &bh = pBhNode->m_data;
		bh.computeImageCodingLengthGain();
	}
	return 0;
}

int SkPrimManager::sortBrachesByCodeingLengthGain()
{
	int n = getBranchNum();

	m_sortedBhIDs.setSize(n);//sorted branch IDs by image CLG
	m_bhImgCLGs.setSize(n);//the coding length gains (CLG) sorted

	int m = 0;
	SkBranchMap& bhMap = m_branchMap;
	int nPos = bhMap.getFirstPos();
	SkBranchNode *pBhNode;
	while (nPos >= 0)
	{
		pBhNode = bhMap.getNextNode(nPos);
		int id = pBhNode->getID();
		if (!pBhNode->valid())
			continue;
		SkPrimBranch &bh = pBhNode->m_data;
		m_sortedBhIDs[m] = id;
		m_bhImgCLGs[m] = bh.getImageCodingLengthGain();
		m++;
	}

	assert(m == n);

	oval_Array<int> id_bak;
	id_bak = m_sortedBhIDs;

	int *order = new int[m];
	float *pfData = m_bhImgCLGs.getPointerAt(0);
	::sortDecreaselyWithTrackingOrder(pfData, m, order);

	for (int i = 0; i < m; i++)
	{
		int j = order[i];
		m_sortedBhIDs[i] = id_bak[j];
	}

	delete []order;
	return 0;
}
//////////////////////////////////////////////
// energy --
///////////////////
int SkPrimManager::computeConnectionEnergy(oval_Array<SkEndConnection> &cnctArray)
{
	int n = cnctArray.getSize();
	for (int i = 0; i < n; i++)
		computeConnectionEnergy(cnctArray[i]);
	return 0;
}
int SkPrimManager::computeConnectionEnergy_woimg(oval_Array<SkEndConnection> &cnctArray)
{
	int n = cnctArray.getSize();
	for (int i = 0; i < n; i++)
		computeConnectionEnergy_woimg(cnctArray[i]);
	return 0;
}

///////////////////
double SkPrimManager::computeConnectionEnergy_woimg(SkEndConnection &cnct)
{
	double dEnergyScale = 1000000;
	double dEnergyScaleHf = 1000;
	double dDisNorm = PI/(180/5);//5 degree per pixel

	SkPrimBranch *pBranch0 = m_branchMap.getAt(cnct.m_id0);
	int end0 = cnct.m_end0;

	SkIntPoint p0 = pBranch0->getEndPoint(end0);
	double dOrt0 = pBranch0->getEndOrt(end0);//end ort

	double dOrt0_new;//end ort new

	int x = cnct.m_x;
	int y = cnct.m_y;

	double dx, dy;
	dx = x - p0.x;
	dy = y - p0.y;
	dOrt0_new = atan2(dy, dx);//new connection ort
	cnct.m_dDeltaOrt0 = skAbsoluteAngleDifferenceOfTwoVectors(dOrt0, dOrt0_new);

	cnct.m_dDist0 = ::distanceBetweenPoints(p0.y, p0.x, y, x);

	double dEnergy;

	if (cnct.m_type == 0)
	{//connect to a VP
		dEnergy = energyChangeByConnectingBranchEnd2AVp(cnct);
//		dEnergy = cnct.m_dDeltaOrt0 + cnct.m_dDist0 * dDisNorm;
//		if (cnct.m_dDist0 > 2*g_nHalfProfileWd)//only consider appearance energy after some distance
//			dEnergy += strokeAppreanceDifferenceEnergy(pBranch0, end0, x, y);
		cnct.m_dEnergy = dEnergy;
	}
	else if (cnct.m_type == 1)
	{//connect to a sketch
//		dEnergy = energyChangeByConnectingBranchEnd2ABranch(cnct);
		dEnergy = cnct.m_dDeltaOrt0 + cnct.m_dDist0 * dDisNorm;
		cnct.m_dEnergy = dEnergy * dEnergyScale;

		SkPrimBranch *pBranch1 = m_branchMap.getAt(cnct.m_id1);
		SkIntPoint endPt0 = pBranch1->getEndPoint(0);
		SkIntPoint endPt1 = pBranch1->getEndPoint(1);
		if ((endPt0.x == x && endPt0.y == y) || (endPt1.x == x && endPt1.y == y) )
			cnct.m_dEnergy /= dEnergyScaleHf;

	}
	else if (cnct.m_type == 2)
	{//connect to a sketch extension
		SkPrimBranch *pBranch1 = m_branchMap.getAt(cnct.m_id1);
		int end1 = cnct.m_end1;

		SkIntPoint p1 = pBranch1->getEndPoint(end1);
		double dOrt1 = pBranch1->getEndOrt(end0);//end ort

		double dOrt1_new;//end ort new

		dx = x - p1.x;
		dy = y - p1.y;
		dOrt1_new = atan2(dy, dx);//new connection ort
		cnct.m_dDeltaOrt1 = skAbsoluteAngleDifferenceOfTwoVectors(dOrt1, dOrt1_new);

		cnct.m_dDist1 = ::distanceBetweenPoints(p1.y, p1.x, y, x);

		dEnergy = cnct.m_dDeltaOrt0+cnct.m_dDeltaOrt1 + (cnct.m_dDist0 + cnct.m_dDist1)*dDisNorm;

		cnct.m_dEnergy = dEnergy * dEnergyScale * dEnergyScale;
	}
	else
	{//not defined
		assert(false);
	}
	return cnct.m_dEnergy;
}


///////////////////
int SkPrimManager::outputEndConnectionInfo(char *msg, SkEndConnection &cnct)
{
	SkPrimBranch *pBranch0 = m_branchMap.getAt(cnct.m_id0);
	int end0 = cnct.m_end0;
	SkIntPoint p0 = pBranch0->getEndPoint(end0);
	int x = cnct.m_x;
	int y = cnct.m_y;

	bool bAtExtension = isAtBranchExtension(pBranch0, end0, x, y, m_param.m_dPixTol);
	if (cnct.m_type == 0)
	{//connect to a VP
		if (bAtExtension)
			sprintf(msg, "  Ext Bh 2 VP: %d:%d:(%d,%d) to (%d,%d), s=%.2f, (%.2f,%.2f,%.2f)\n", 
				cnct.m_id0,end0,p0.x, p0.y, x, y, cnct.m_dDist0, cnct.m_dImgCLG, cnct.m_dSkCLG, cnct.m_dEnergy);
		else
			sprintf(msg, "  Cnt Bh 2 VP: %d:%d:(%d,%d) to (%d,%d), s=%.2f, (%.2f,%.2f,%.2f)\n", 
				cnct.m_id0,end0,p0.x, p0.y, x, y, cnct.m_dDist0, cnct.m_dImgCLG, cnct.m_dSkCLG, cnct.m_dEnergy);
	}
	else if (cnct.m_type == 1)
	{//connect to a sketch
		if (bAtExtension)
			sprintf(msg, "  Ext Bh 2 Bh In: %d:%d:(%d,%d) to %d:%d:(%d,%d), s=%.2f, (%.2f,%.2f,%.2f)\n", 
				cnct.m_id0,end0,p0.x, p0.y, cnct.m_id1, cnct.m_end1, x, y, cnct.m_dDist0,  cnct.m_dImgCLG, cnct.m_dSkCLG, cnct.m_dEnergy);
		else
			sprintf(msg, "  Cnt Bh 2 Bh In: %d:%d:(%d,%d) to %d:%d:(%d,%d), s=%.2f, (%.2f,%.2f,%.2f)\n", 
				cnct.m_id0,end0,p0.x, p0.y, cnct.m_id1, cnct.m_end1, x, y, cnct.m_dDist0, cnct.m_dImgCLG, cnct.m_dSkCLG, cnct.m_dEnergy);
	}
	else if (cnct.m_type == 2)
	{//connect to a sketch extension
		SkPrimBranch *pBranch1 = m_branchMap.getAt(cnct.m_id1);
		int end1 = cnct.m_end1;
		SkIntPoint p1 = pBranch1->getEndPoint(end1);
		sprintf(msg, "  Ext Bh 2 Bh Ex: %d:%d:(%d,%d), %d:%d:(%d,%d) to (%d,%d),  s0=%.2f,  s1=%.2f, (%.2f,%.2f,%.2f)\n", 
				cnct.m_id0,end0,p0.x, p0.y, cnct.m_id1, cnct.m_end1, p1.x, p1.y, x, y, cnct.m_dDist0, cnct.m_dDist1, cnct.m_dImgCLG, cnct.m_dSkCLG, cnct.m_dEnergy);
	}
	else
	{//not defined
		assert(false);
	}
	return 0;
}

///////////////////
double SkPrimManager::computeConnectionEnergy(SkEndConnection &cnct)
{
	SkPrimBranch *pBranch0 = m_branchMap.getAt(cnct.m_id0);
	int end0 = cnct.m_end0;
	int x = cnct.m_x;
	int y = cnct.m_y;

	bool bAtExtension = isAtBranchExtension(pBranch0, end0, x, y, m_param.m_dPixTol);

	SkIntPoint p0 = pBranch0->getEndPoint(end0);
	double dOrt0 = pBranch0->getEndOrt(end0);//end ort

	double dx, dy;
	dx = x - p0.x;
	dy = y - p0.y;
	double dOrt0_new = atan2(dy, dx);//new connection ort
	cnct.m_dDeltaOrt0 = skAbsoluteAngleDifferenceOfTwoVectors(dOrt0, dOrt0_new);

	cnct.m_dDist0 = ::distanceBetweenPoints(p0.y, p0.x, y, x);

	double dEnergy;

	if (cnct.m_type == 0)
	{//connect to a VP
		if (bAtExtension)
			dEnergy = energyChangeByExtendingBranchEnd2AVp(cnct);
		else
			dEnergy = energyChangeByConnectingBranchEnd2AVp(cnct);
	}
	else if (cnct.m_type == 1)
	{//connect to a sketch
		if (bAtExtension)
			dEnergy = energyChangeByExtendingBranchEnd2ABranchInside(cnct);
		else
			dEnergy = energyChangeByConnectingBranchEnd2ABranchInside(cnct);
	}
	else if (cnct.m_type == 2)
	{//connect to a sketch extension
		SkPrimBranch *pBranch1 = m_branchMap.getAt(cnct.m_id1);
		int end1 = cnct.m_end1;
		SkIntPoint p1 = pBranch1->getEndPoint(end1);
		cnct.m_dDist1 = ::distanceBetweenPoints(p1.y, p1.x, y, x);
		
		double dOrt1 = pBranch1->getEndOrt(end1);//end ort

		double dx, dy;
		dx = x - p1.x;
		dy = y - p1.y;
		double dOrt1_new = atan2(dy, dx);//new connection ort
		cnct.m_dDeltaOrt1 = skAbsoluteAngleDifferenceOfTwoVectors(dOrt1, dOrt1_new);

		dEnergy = energyChangeByExtendingBranchEnd2ABranchExtension(cnct);
	}
	else
	{//not defined
		assert(false);
	}
	return dEnergy;
}

///////////////////////////////////////////////////
// energy changes
double SkPrimManager::energyChangeByExtendingBranchEnd2AVp(SkEndConnection &cnct)
{
	int id0 = cnct.m_id0;
	int end0 = cnct.m_end0;
	SkPrimBranch *pBranch0 = m_branchMap.getAt(id0);

	SkIntPoint p2;
	p2.x = cnct.m_x;
	p2.y = cnct.m_y;

	cnct.m_dImgCLG = imageCodingLengthGainByExtendingBranchEnd2APoint(id0, end0, p2);
	
	cnct.m_dSkCLG = sketchCodingLengthGainByExtendingBranchEnd2AVp(pBranch0, end0);

	cnct.m_dEnergy = -(cnct.m_dImgCLG + cnct.m_dSkCLG);

	return cnct.m_dEnergy;
}
double SkPrimManager::energyChangeByConnectingBranchEnd2AVp(SkEndConnection &cnct)
{
	int id0 = cnct.m_id0;
	int end0 = cnct.m_end0;

	SkPrimBranch *pBranch0 = m_branchMap.getAt(id0);

	SkIntPoint p0 = pBranch0->getEndPoint(end0);

	int x0 = p0.x;
	int y0 = p0.y;

	int x1 = cnct.m_x;
	int y1 = cnct.m_y;

	SkPrimBranch branch;
	branch.build(x0, y0, x1, y1, m_imgOrg);
	
//	double dImgCLG0 = branch.computeImageCodingLengthGain();//only from the branch, just for debuging 

	cnct.m_dImgCLG = imageCodingLengthGainByAddingOneBranch(branch);

	cnct.m_dSkCLG = sketchCodingLengthGainByConnectingBranchEnd2AVp(pBranch0, end0, branch);

	cnct.m_dEnergy = -(cnct.m_dImgCLG + cnct.m_dSkCLG);

	return cnct.m_dEnergy;
}
double SkPrimManager::energyChangeByConnectingBranchEnd2ABranchInside(SkEndConnection &cnct)
{
	double dDeltaEnergy = 0;
	return dDeltaEnergy;
}
double SkPrimManager::energyChangeByExtendingBranchEnd2ABranchInside(SkEndConnection &cnct)
{
	int id0 = cnct.m_id0;
	int end0 = cnct.m_end0;
	SkPrimBranch *pBranch0 = m_branchMap.getAt(id0);

	SkIntPoint p2;
	p2.x = cnct.m_x;
	p2.y = cnct.m_y;

	cnct.m_dImgCLG = imageCodingLengthGainByExtendingBranchEnd2APoint(id0, end0, p2);
	
	cnct.m_dSkCLG = sketchCodingLengthGainByExtendingBranchEnd2ABranchInside(pBranch0, end0);

	cnct.m_dEnergy= -(cnct.m_dImgCLG + cnct.m_dSkCLG);

	return cnct.m_dEnergy;
}
double SkPrimManager::energyChangeByExtendingBranchEnd2ABranchExtension(SkEndConnection &cnct)
{
	int id[2], end[2];
	id[0] = cnct.m_id0;
	end[0] = cnct.m_end0;

	id[1] = cnct.m_id1;
	end[1] = cnct.m_end1;

	SkIntPoint p2;
	p2.x = cnct.m_x;
	p2.y = cnct.m_y;

	cnct.m_dImgCLG = imageCodingLengthGainByExtendingBranchEnd2ABranchExtension(id, end, p2);
	
	cnct.m_dSkCLG = sketchCodingLengthGainByExtendingBranchEnd2ABranchExtension();

	cnct.m_dEnergy = -(cnct.m_dImgCLG + cnct.m_dSkCLG);

	return cnct.m_dEnergy;
}


double SkPrimManager::imageCodingLengthGainByAddingOneBranch(SkPrimBranch &branch)
{
	int ht, wd;
	getImageDimension(ht, wd);
	Matrix<SkPixCandArray> rec_cand(ht, wd);
	
	updatePixCand4ABranch(&branch, rec_cand, ADD, false);

	Matrix<int> mask(ht, wd);
	mask.Set(255);

	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			SkPixCandArray & arr0 = rec_cand(r,c);
			if (arr0.getSize() <= 0)
				continue;
			SkPixCandArray & arr1 = m_pixCandMat(r,c);
			for (int i = 0; i < arr1.getSize(); i++)
				arr0.add(arr1[i]);
			mask(r,c) = 0;
		}
	}

	Matrix<float> rec(ht, wd);
	rec.Set(0);
	skComputeImageFromPixCandByMixing(rec_cand, rec);

	//compute dCL0 and dCL1

	double dImgCLG = codingLengthGainOfNewImage(m_imgOrg, rec, mask);

	return dImgCLG;

}

double SkPrimManager::sketchCodingLengthGainByConnectingBranchEnd2AVp(SkPrimBranch *pBranch0, int end0, SkPrimBranch &branch)
{
	double dCL4Pointer2Vp = m_skCLMan.getCL4Pointer2Vp();
	dCL4Pointer2Vp *= 2;//two vp

	SkSegment *pSeg0 = pBranch0->getEndSegment(end0);
	SkSegment *pSeg1 = branch.getEndSegment(0);
	double dCL4dA = m_skCLMan.computeCL4dA(pSeg0->m_prfl, pSeg1->m_prfl);


	double dSkCLG = -dCL4Pointer2Vp - dCL4dA;//negtive gain

	return 0;
}
double SkPrimManager::sketchCodingLengthGainByExtendingBranchEnd2AVp(SkPrimBranch *pBranch0, int end0)
{
	double dCL4Vp = m_skCLMan.getCL4Vp();

	double dSkCLG = dCL4Vp;//positive gain

	return 0;
}
double SkPrimManager::sketchCodingLengthGainByExtendingBranchEnd2ABranchInside(SkPrimBranch *pBranch0, int end0)
{
	double dSkCLG = 0;//no coding change

	return dSkCLG;
}
double SkPrimManager::sketchCodingLengthGainByExtendingBranchEnd2ABranchExtension()
{
	double dCL4Vp = m_skCLMan.getCL4Vp();

	double dSkCLG = -dCL4Vp;//negative gain

	return 0;
}
double SkPrimManager::codingLengthGainOfNewImage(Matrix<float> &org, Matrix<float> &rec, Matrix<int> &mask)
{
	int ht, wd;
	org.GetDimension(ht, wd);

	Matrix<float> res;
	res = org - rec;

	double nPtNum = 0;
	double dSumOrg = 0;
	double dSumRes = 0;

	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (mask(r,c) != 0)
				continue;
			dSumOrg += org(r,c);
			dSumRes += res(r,c);
			nPtNum++;
		}
	}
	if (nPtNum == 0)
		return 0;

	dSumOrg /= nPtNum;
	dSumRes /= nPtNum;

	double dCLOrg = 0;
	double dCLRes = 0;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			if (mask(r,c) != 0)
				continue;
			dCLOrg += (org(r,c)-dSumOrg)*(org(r,c)-dSumOrg);
			dCLRes += (res(r,c)-dSumRes)*(res(r,c)-dSumRes);
		}
	}

	return dCLOrg - dCLRes;
}
double SkPrimManager::imageCodingLengthGainByExtendingBranchEnd2APoint(int id, int end, SkIntPoint &p2)
{
	SkPrimBranch *pBranch = m_branchMap.getAt(id);

	SkSegment *pSeg = pBranch->getEndSegment(end);
	SkIntPoint p0 = pSeg->getEndPoint(end);

	int ht, wd;
	getImageDimension(ht, wd);
	Matrix<SkPixCandArray> rec_cand(ht, wd);
	
	int x0, y0, x1, y1;
	if (end == 1)
	{
		x0 = p0.x;
		y0 = p0.y;
		x1 = p2.x;
		y1 = p2.y;
	}
	else
	{
		x0 = p2.x;
		y0 = p2.y;
		x1 = p0.x;
		y1 = p0.y;
	}
	int w = pBranch->getProfileWidth();
	double *vm = pSeg->getModelIntensity();

	updatePixCand4ALinelet(x0, y0, x1, y1, w, vm, rec_cand, ADD);

	Matrix<int> mask(ht, wd);
	mask.Set(255);

	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			SkPixCandArray & arr0 = rec_cand(r,c);
			if (arr0.getSize() <= 0)
				continue;
			SkPixCandArray & arr1 = m_pixCandMat(r,c);
			for (int i = 0; i < arr1.getSize(); i++)
				arr0.add(arr1[i]);
			mask(r,c) = 0;
		}
	}

	Matrix<float> rec(ht, wd);
	rec.Set(0);
	skComputeImageFromPixCandByMixing(rec_cand, rec);

	//compute dCL0 and dCL1

	double dImgCLG = codingLengthGainOfNewImage(m_imgOrg, rec, mask);

	return dImgCLG;
}
double SkPrimManager::imageCodingLengthGainByExtendingBranchEnd2ABranchExtension(int id[2], int end[2], SkIntPoint &p2)
{
	int ht, wd;
	getImageDimension(ht, wd);
	Matrix<SkPixCandArray> rec_cand(ht, wd);
	for (int i = 0; i < 2; i++)
	{
		SkPrimBranch *pBranch = m_branchMap.getAt(id[i]);

		SkSegment *pSeg = pBranch->getEndSegment(end[i]);
		SkIntPoint p0 = pSeg->getEndPoint(end[i]);

	
		int x0, y0, x1, y1;
		if (end[i] == 1)
		{
			x0 = p0.x;
			y0 = p0.y;
			x1 = p2.x;
			y1 = p2.y;
		}
		else
		{
			x0 = p2.x;
			y0 = p2.y;
			x1 = p0.x;
			y1 = p0.y;
		}
		int w = pBranch->getProfileWidth();
		double *vm = pSeg->getModelIntensity();

		updatePixCand4ALinelet(x0, y0, x1, y1, w, vm, rec_cand, ADD);
	}


	Matrix<int> mask(ht, wd);
	mask.Set(255);

	int r, c;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			SkPixCandArray & arr0 = rec_cand(r,c);
			if (arr0.getSize() <= 0)
				continue;
			SkPixCandArray & arr1 = m_pixCandMat(r,c);
			for (int i = 0; i < arr1.getSize(); i++)
				arr0.add(arr1[i]);
			mask(r,c) = 0;
		}
	}

	Matrix<float> rec(ht, wd);
	rec.Set(0);
	skComputeImageFromPixCandByMixing(rec_cand, rec);

	//compute dCL0 and dCL1

	double dImgCLG = codingLengthGainOfNewImage(m_imgOrg, rec, mask);

	return dImgCLG;
}

///////////////////
// saveAsText()
int SkPrimManager::saveAsText(const char *name)
{
	FILE *pFile = fopen(name, "w");
	if (pFile == 0)
		return 1;


	int wd, ht;
	m_imgOrg.GetDimension(ht,wd);

	// save image size
	fprintf(pFile, "ImgWd = %d, ImgHt = %d \n\n", wd, ht);

	// save joints
	int vpNum = m_vpMap.getValidNodeNum();
	fprintf(pFile, "Junction Num: %d \n", vpNum);

	SkIntPoint p0, p1;
	SkVPrimNode *pNode;
	int nPos = m_vpMap.getFirstPos();
	int nMaxPtNum = 7;
	int n = 0;
	while (nPos >= 0)
	{
		pNode = m_vpMap.getNextNode(nPos);
		if (!pNode->valid())
			continue;
		SkVirtualPrim &vp = pNode->m_data;
		int nBranch = vp.getBranchNum();
		p0 = vp.m_refPt;

		fprintf(pFile, "    Junction %d: center (%d,%d); degree %d; connects to ((x,y), ort): ",  n, p0.x, p0.y, nBranch);
		for (int i = 0; i < nBranch; i++)
		{
			int id = vp.getBranchID(i);
			int e = vp.getBranchEnd(i);
			SkPrimBranch *pBranch = m_branchMap.getAt(id);
			int nSegNum = pBranch->getSegNum();
			SkSegment *pSeg;
			int nPtNum;
			if (e ==0)
			{
				pSeg = pBranch->getSegment(0);
				nPtNum = pSeg->getPtNum();
				p1 = pSeg->getPoint(nPtNum-1);
			}
			else
			{
				pSeg = pBranch->getSegment(nSegNum-1);
				p1 = pSeg->getPoint(0);
			}
			double dOrt = ::skGetOrientation(p0.x, p0.y, p1.x, p1.y);
			fprintf(pFile, "((%d,%d),%.2f) ", p1.x, p1.y, dOrt*180/PI);
		}
		fprintf(pFile, "\n\n");
		n++;
	}

	fprintf(pFile, "\n");

	int bhNum = m_branchMap.getValidNodeNum();
	fprintf(pFile, "Curve Num: %d \n", bhNum);

	SkIntPoint p;
	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;
	nPos = m_branchMap.getFirstPos();
	n = 0;
//	while (nPos >= 0)
	for (n = 0; n < bhNum; n++)
	{
		int bhID = m_sortedBhIDs[n];

		pBhNode = m_branchMap.getNodeAt(bhID);
		if (!pBhNode->valid())
			continue;
		pBranch = &(pBhNode->m_data);
		int nPtNum = pBranch->getPointNum();
		fprintf(pFile, "    Curve %d: point Num %d; end connectivity: ",  n, nPtNum);
		for (int e = 0; e < 2; e++)
		{
			int vpID = pBranch->getPrimID(e);
			int con;
			if (vpID >= 0)
				con = 1;
			else
				con = 0;
			fprintf(pFile, "%d ", con);
		}
		fprintf(pFile, "\n");

		for (int i = 0; i < nPtNum; i++)
		{
			p = pBranch->getPoint(i);
			fprintf(pFile, "(x%d,y%d,h%d) ", p.x, p.y, 5);
		}
		fprintf(pFile, "\n");

		int nCornerNum = pBranch->getSegNum() - 1;
		fprintf(pFile, "      Corner num: %d; Corner indices: ", nCornerNum);
		for (int i = 0; i < nCornerNum; i++)
		{
			fprintf(pFile, "%d ", pBranch->getLinesegEndIndex(i));
		}
		fprintf(pFile, "; Corner degrees: ");
		for (int i = 0; i < nCornerNum; i++)
		{
			double dCornerAngle = pBranch->getCornerAngle(i);

			fprintf(pFile, "%.2f ", dCornerAngle*180/PI);
		}

		fprintf(pFile, "\n\n");
//		n++;
	}
	fprintf(pFile, "\n");

	// save blobs
	bhNum = m_blobMap.getValidNodeNum();
	fprintf(pFile, "blob Num: %d \n", bhNum);

	nPos = m_blobMap.getFirstPos();
	SkBlobNode *pBlobNode;
	while (nPos >= 0)
	{
		pBlobNode = m_blobMap.getNextNode(nPos);
		if (!pBlobNode->valid())
			continue;
		SkPrimBlob *pBlob = &(pBlobNode->m_data);
		SkIntPoint p0 = pBlob->getRefPoint();
		double dRad = pBlob->getRadius();
		fprintf(pFile, "    Blob %d: center (%d,%d), radius %d\n",  n, p0.x, p0.y, dRad);
	}

	fclose(pFile);

	return 0;
}

///////////////////
// readFromText()
int SkPrimManager::readFromText(const char *name)
{
	FILE *pFile = fopen(name, "r");
	if (pFile == 0)
		return 1;


	int wd, ht;

	// save image size
	fscanf(pFile, "ImgWd = %d, ImgHt = %d \n\n", &wd, &ht);

	// save joints
	int vpNum;
	fscanf(pFile, "Junction Num: %d \n", &vpNum);

	int i;
	int x, y;
	int n, nBranch;
	for (i = 0; i < vpNum; i++)
	{
		fscanf(pFile, "    Junction %d: center (%d,%d); degree %d; connects to ((x,y), ort): ",  &n, &x, &y, &nBranch);
		for (int i = 0; i < nBranch; i++)
		{
			double dOrt;
			fscanf(pFile, "((%d,%d),%lf) ", &x, &y, &dOrt);
		}
		fscanf(pFile, "\n\n");	
	}

	fscanf(pFile, "\n");

	int bhNum, nPtNum;
	fscanf(pFile, "Curve Num: %d \n", &bhNum);

	int h;
	int ii;
	for (ii = 0; ii < bhNum; ii++)
	{
		fscanf(pFile, "    Curve %d: point Num %d; end connectivity: ",  &n, &nPtNum);
		for (int e = 0; e < 2; e++)
		{
			int con;
			fscanf(pFile, "%d ", &con);
		}
		fscanf(pFile, "\n");
		for (int j = 0; j < nPtNum; j++)
		{
			fscanf(pFile, "(x%d,y%d,h%d) ", &x, &y, &h);
		}
		int nCornerNum;
		fscanf(pFile, "      Corner num: %d; Corner indices: ", &nCornerNum);
		for (i = 0; i < nCornerNum; i++)
		{
			int ind;
			fscanf(pFile, "%d ", &ind);
		}
		fscanf(pFile, "; Corner degrees: ");
		for (i = 0; i < nCornerNum; i++)
		{
			double dCornerAngle;
			fscanf(pFile, "%f ", &dCornerAngle);
		}
		fscanf(pFile, "\n\n");
	}
	fscanf(pFile, "\n");

	// save blobs
	fscanf(pFile, "blob Num: %d \n", &bhNum);
	for (i = 0; i < bhNum; i++)
	{
		fscanf(pFile, "    Blob %d: center (%d,%d), radius %d\n",  &n, &x, &y, &h);
	}
	fclose(pFile);

	return 0;
}
int SkPrimManager::savePrimitiveGeometricStatistics(const char *type)
{
	char work_dir[32];
	sprintf(work_dir, "prim");
	_mkdir(work_dir);

	// for the junctions, all_angle doesn't include the biggest angle
	char name[512];
	int nAngleBinNum = 180;
	int nMaxPrimDegree = 8;
	int *pnPrimDegreeHist = new int[nMaxPrimDegree]; memset(pnPrimDegreeHist, 0, sizeof(int)*nMaxPrimDegree);
	int *pnAllAngleHist = new int[nAngleBinNum]; memset(pnAllAngleHist, 0, sizeof(int)*nAngleBinNum);
	int *pnCornerAngleHist = new int[nAngleBinNum]; memset(pnCornerAngleHist, 0, sizeof(int)*nAngleBinNum);
	int *pnJunctionAngleHist = new int[nAngleBinNum]; memset(pnJunctionAngleHist, 0, sizeof(int)*nAngleBinNum);
	int *pnJunctionSmallestAngleHist = new int[nAngleBinNum]; memset(pnJunctionSmallestAngleHist, 0, sizeof(int)*nAngleBinNum);
	int *pnJunctionDeltaAngleHist = new int[nAngleBinNum]; memset(pnJunctionDeltaAngleHist, 0, sizeof(int)*nAngleBinNum);
	int *pnJunctionTwoSmallestAngleSumHist = new int[2*nAngleBinNum]; memset(pnJunctionTwoSmallestAngleSumHist, 0, sizeof(int)*2*nAngleBinNum);
	// save joints
	int vpNum = m_vpMap.getValidNodeNum();

	SkIntPoint p0, p1;
	SkVPrimNode *pNode;
	int nPos = m_vpMap.getFirstPos();
	while (nPos >= 0)
	{
		pNode = m_vpMap.getNextNode(nPos);
		if (!pNode->valid())
			continue;
		SkVirtualPrim &vp = pNode->m_data;
		int nBranch = vp.getBranchNum();
		p0 = vp.m_refPt;

		assert(nBranch < nMaxPrimDegree);
		assert(nBranch > 0);


		double pdOrt[128], pdAngle[128];
		for (int i = 0; i < nBranch; i++)
		{
			int id = vp.getBranchID(i);
			int e = vp.getBranchEnd(i);
			SkPrimBranch *pBranch = m_branchMap.getAt(id);
			int nSegNum = pBranch->getSegNum();
			SkSegment *pSeg;
			int nPtNum;
			if (e ==0)
			{
				pSeg = pBranch->getSegment(0);
				nPtNum = pSeg->getPtNum();
				p1 = pSeg->getPoint(nPtNum-1);
			}
			else
			{
				pSeg = pBranch->getSegment(nSegNum-1);
				p1 = pSeg->getPoint(0);
			}
			pdOrt[i] = ::skGetOrientation(p0.x, p0.y, p1.x, p1.y);
		}
		::skComputeSortedAngleFromOrt(pdOrt, pdAngle, nBranch);
		if (nBranch == 1)
		{//free end point, no counting necessary
//			assert(false);
		}
		else if (nBranch == 2)
		{//corner
			pnPrimDegreeHist[nBranch]++;
			int nAngle = (int)(pdAngle[0]*180/PI+0.5);
			assert(nAngle >= 0);
			if (nAngle >= nAngleBinNum)
				nAngle = nAngleBinNum - 1;
			pnAllAngleHist[nAngle]++;
			pnCornerAngleHist[nAngle]++;
		}
		else
		{//nbranch >= 3
			pnPrimDegreeHist[nBranch]++;
			for (int i = 0; i < nBranch-1; i++)
			{
				int nAngle = (int)(pdAngle[i]*180/PI+0.5);
				assert(nAngle >= 0);
				if (nAngle >= nAngleBinNum)
					nAngle = nAngleBinNum - 1;
				pnAllAngleHist[nAngle]++;
				pnJunctionAngleHist[nAngle]++;

				int nAngle1 = (int)(pdAngle[i+1]*180/PI+0.5);
				assert(nAngle1 >= 0);
				if (nAngle1 >= nAngleBinNum)
					nAngle1 = nAngleBinNum - 1;
				int nDeltaAngle = nAngle1 - nAngle;
				assert(nDeltaAngle >= 0);
				pnJunctionDeltaAngleHist[nDeltaAngle]++;
				if (i == 0)
				{
					pnJunctionSmallestAngleHist[nAngle]++;
					int nSum = nAngle + nAngle1;
					assert(nSum >= 0);
					if (nSum >= 2*nAngleBinNum)
						nSum = 2*nAngleBinNum-1;
					pnJunctionTwoSmallestAngleSumHist[nSum]++;
				}
			}
			
		}
	}

	int bhNum = m_branchMap.getValidNodeNum();

	SkIntPoint p;
	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;
	nPos = m_branchMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = m_branchMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		pBranch = &(pBhNode->m_data);
		int nPtNum = pBranch->getPointNum();
		for (int e = 0; e < 2; e++)
		{
			int vpID = pBranch->getPrimID(e);
			int con;
			if (vpID >= 0)
				con = 1;
			else
				con = 0;
			if (con == 0)
				pnPrimDegreeHist[1]++;//end point
		}

		int nCornerNum = pBranch->getSegNum() - 1;
		int i;
		for (i = 0; i < nCornerNum; i++)
		{
			pnPrimDegreeHist[2]++;
			double dCornerAngle = pBranch->getCornerAngle(i);
			int nAngle = (int)(dCornerAngle * 180/PI + 0.5);
			assert (nAngle >= 0);
			if (nAngle >= nAngleBinNum)
				nAngle = nAngleBinNum - 1;
			pnAllAngleHist[nAngle]++;
			pnCornerAngleHist[nAngle]++;		
		}
	}

	// save blobs
	bhNum = m_blobMap.getValidNodeNum();
	pnPrimDegreeHist[0] += bhNum;

	sprintf(name, "%s\\%s_%s_prim_degree_hist.txt", work_dir, m_pchProjName, type);
	::saveArray(pnPrimDegreeHist, nMaxPrimDegree, name);
	sprintf(name, "%s\\%s_%s_all_angle_hist.txt", work_dir, m_pchProjName, type);
	::saveArray(pnAllAngleHist, nAngleBinNum, name);
	sprintf(name, "%s\\%s_%s_corner_angle_hist.txt", work_dir, m_pchProjName, type);
	::saveArray(pnCornerAngleHist, nAngleBinNum, name);
	sprintf(name, "%s\\%s_%s_junct_angle_hist.txt", work_dir, m_pchProjName, type);
	::saveArray(pnJunctionAngleHist, nAngleBinNum, name);
	sprintf(name, "%s\\%s_%s_junct_smallest_angle_hist.txt", work_dir, m_pchProjName, type);
	::saveArray(pnJunctionSmallestAngleHist, nAngleBinNum, name);
	sprintf(name, "%s\\%s_%s_junct_delta_angle_hist.txt", work_dir, m_pchProjName, type);
	::saveArray(pnJunctionDeltaAngleHist, nAngleBinNum, name);
	sprintf(name, "%s\\%s_%s_junct_2angle_sum_hist.txt", work_dir, m_pchProjName, type);
	::saveArray(pnJunctionTwoSmallestAngleSumHist, 2*nAngleBinNum, name);

	delete []pnPrimDegreeHist;
	delete []pnAllAngleHist;
	delete []pnCornerAngleHist;
	delete []pnJunctionAngleHist;
	delete []pnJunctionSmallestAngleHist;
	delete []pnJunctionDeltaAngleHist;
	delete []pnJunctionTwoSmallestAngleSumHist;

	return 0;
}

///////////////////
// 
int SkPrimManager::saveSketchableLattice(Matrix<int> &con)
{//0 - valid
	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);

	int value = 0;
	int hf = SkPrimBranch::m_nHalfWd;
	int w = 2*hf + 1;
	saveStraightLinesOfCurves(con, value, w);

	return 0;
}
///////////////////
// saveSketchGraphAsBmp()
int SkPrimManager::saveSketchGraphAsBmp(const char *type, int line_width, SkPrimPursuit *pPP)
{

	char work_dir[32];
	sprintf(work_dir, "sketches");
	_mkdir(work_dir);

	char name[512];
	// save as text first
//	sprintf(name, "%s\\%s_%s_graph.txt", work_dir, m_pchProjName, type);
	sprintf(name, "%s_%s_graph.txt", m_pchProjName, type);
	saveAsText(name);

//	savePrimitiveGeometricStatistics(type);

	savePointsAndSketches(type);
	savePointsAndSketchesWithVertexDots(type);
	saveVerticesWithLegs(type);

	return 0;
}
///////////////////
// savePrimitivesWithVerticesAsDots()
int SkPrimManager::savePointsAndSketches(const char *type)
{
	char work_dir[32];
	sprintf(work_dir, "sketches");
	_mkdir(work_dir);

	char name[512];
	int value;

	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);
	Matrix<int> mat(ht, wd);

	mat.Set(255);
	value = 0;
	savePointsOfCurves(mat, value);

	sprintf(name, "%s\\%s_%s_points.bmp", work_dir, m_pchProjName, type);
	::skSaveMatrixAsBitmap(name, mat, 0, 1);

	mat.Set(255);
	value = 0;
	saveStraightLinesOfCurves(mat, value, 1);
	sprintf(name, "%s\\%s_%s_sketch.bmp", work_dir, m_pchProjName, type);
	::skSaveMatrixAsBitmap(name, mat, 0, 1);
	sprintf(name, "%s_sketch.bmp", m_pchProjName);
	::skSaveMatrixAsBitmap(name, mat, 0, 1);

	/*char tmp_dir[512];
	sprintf(tmp_dir, "d:\\temp");
	_mkdir(tmp_dir);

	sprintf(name, "%s\\%s_%s_sketch.bmp", tmp_dir, m_pchProjName, type);
	::skSaveMatrixAsBitmap(name, mat, 0, 1);
	sprintf(name, "%s\\%s.bmp", tmp_dir, m_pchProjName);
	::skSaveMatrixAsBitmap(name, m_imgOrg, m_dImgMean, m_dImgStd);*/

	return 0;
}
///////////////////
// savePrimitivesWithVerticesAsDots()
int SkPrimManager::savePointsAndSketchesWithVertexDots(const char *type)
{
	char work_dir[32];
	sprintf(work_dir, "sketches");
	_mkdir(work_dir);

	char name[512];
	int value;

	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);
	Matrix<int> mat(ht, wd);

	mat.Set(255);
	value = 128;
	savePointsOfCurves(mat, value);
	value = 0;
	markEndPointsAsDots(mat, value);
	markCornersAsDots(mat, value);
	markJunctionsAsDots(mat, value);

	sprintf(name, "%s\\%s_%s_points_graph.bmp", work_dir, m_pchProjName, type);
	::skSaveMatrixAsBitmap(name, mat, 0, 1);

	mat.Set(255);
	value = 128;
	saveStraightLinesOfCurves(mat, value, 1);
	value = 0;
	markEndPointsAsDots(mat, value);
	markCornersAsDots(mat, value);
	markJunctionsAsDots(mat, value);
	sprintf(name, "%s\\%s_%s_sketch_graph.bmp", work_dir, m_pchProjName, type);
	::skSaveMatrixAsBitmap(name, mat, 0, 1);

	sprintf(name, "%s_%s_sk.bmp", m_pchProjName, type);
	::skSaveMatrixAsBitmap(name, mat, 0, 1);


	skAddBoundary(mat, 3, 0);
	sprintf(name, "%s_sketch_graph.bmp", m_pchProjName);
	::skSaveMatrixAsBitmap(name, mat, 0, 1);
	return 0;
}



///////////////////
// saveVerticesWithLegs()
int SkPrimManager::saveVerticesWithLegs(const char *type)
{
	char name[512];
	char work_dir[512];
	sprintf(work_dir, "sketches");
	_mkdir(work_dir);

	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);
	Matrix<int> primMat(ht, wd);
	primMat.Set(255);

	// save joints
	SkVPrimNode *pNode;
	int nPos = m_vpMap.getFirstPos();
	int nMaxPtNum = 7;
	int line_width = 1;
	while (nPos >= 0)
	{
		pNode = m_vpMap.getNextNode(nPos);
		if (!pNode->valid())
			continue;
		SkVirtualPrim &vp = pNode->m_data;
		int nBranch = vp.getBranchNum();
		for (int i = 0; i < nBranch; i++)
		{
			int bhID = vp.getBranchID(i);
			SkPrimBranch *pBh = m_branchMap.getAt(bhID);
			int e = vp.getBranchEnd(i);
			SkIntPointArray ptArray;
			int n = pBh->getEndPoints(e, ptArray, nMaxPtNum);
			for (int j = 0; j < n-1; j++)
			{
				int x0 = ptArray[j].x;
				int y0 = ptArray[j].y;
				int x1 = ptArray[j+1].x;
				int y1 = ptArray[j+1].y;
				int m = ::rasterize_line_segment(x0, y0, x1, y1, m_pnX, m_pnY, line_width);
				for (int k = 0; k < m; k++)
				{
					int x = m_pnX[k];
					int y = m_pnY[k];
					if (x < 0 || x >= wd || y < 0 || y >= ht)
						continue;
					primMat(y,x) = 0;
				}
			}
		}

	}
	sprintf(name, "%s\\%s_%s_joints.bmp", work_dir, m_pchProjName, type);
	::skSaveMatrixAsBitmap(name, primMat, 0, 1);

	// save the corners
	primMat.Set(255);

	nMaxPtNum = 7;
	int nPtNum;
	int nHalfWindowSize = 3;
	int nMinSegLength = 5;
	nPos = m_branchMap.getFirstPos();
	SkBranchNode *pBhNode;
	SkPrimBranch *pBranch;
	while (nPos >= 0)
	{
		pBhNode = m_branchMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		if (pBhNode->getID()== 131)
			nPos += 0;
		pBranch = &(pBhNode->m_data);
		int nSegNum = pBranch->getSegNum();
		SkSegment *pSeg1, *pSeg2;
		int st, nd;
		if (nSegNum >= 2)
		{
			for (int s = 0; s < nSegNum-1; s++)
			{
				pSeg1 = pBranch->getSegment(s);
				pSeg2 = pBranch->getSegment(s+1);

				//first part
				nPtNum = pSeg1->getPtNum();
				if (nPtNum > nMaxPtNum)
					st = nPtNum - nMaxPtNum;
				else
					st = 0;
				nd = nPtNum-1;
				SkIntPoint p;
				for (int i = st; i < nd; i++)
				{
					p = pSeg1->getPoint(i);
					int x0 = p.x;
					int y0 = p.y;
					p = pSeg1->getPoint(i+1);
					int x1 = p.x;
					int y1 = p.y;
					int m = ::rasterize_line_segment(x0, y0, x1, y1, m_pnX, m_pnY, line_width);
					for (int k = 0; k < m; k++)
					{
						int x = m_pnX[k];
						int y = m_pnY[k];
						if (x < 0 || x >= wd || y < 0 || y >= ht)
							continue;
						primMat(y,x) = 0;
					}
				}
				//second part
				nPtNum = pSeg2->getPtNum();
				st = 0;
				if (nPtNum > nMaxPtNum)
					nd = nMaxPtNum - 1;
				else
					nd = nPtNum-1;
				for (int i = st; i < nd; i++)
				{
					p = pSeg2->getPoint(i);
					int x0 = p.x;
					int y0 = p.y;
					p = pSeg2->getPoint(i+1);
					int x1 = p.x;
					int y1 = p.y;
					int m = ::rasterize_line_segment(x0, y0, x1, y1, m_pnX, m_pnY, line_width);
					for (int k = 0; k < m; k++)
					{
						int x = m_pnX[k];
						int y = m_pnY[k];
						if (x < 0 || x >= wd || y < 0 || y >= ht)
							continue;
						primMat(y,x) = 0;
					}
				}
			}// for each segment (corner)
		}//if there are corners
	}	//for all branches
	sprintf(name, "%s\\%s_%s_corner.bmp", work_dir, m_pchProjName, type);
	::skSaveMatrixAsBitmap(name, primMat, 0, 1);

	return 0;
}


int SkPrimManager::savePointsOfCurves(Matrix<int> &mat, int value)
{
	int wd, ht;
	mat.GetDimension(ht, wd);

	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;
	int nPos = m_branchMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = m_branchMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		pBranch = &(pBhNode->m_data);
		int nPtNum = pBranch->getPointNum();
		SkIntPoint p;
		for (int i = 0; i < nPtNum; i++)
		{
			p = pBranch->getPoint(i);
			if (p.x < 0 || p.x >= wd || p.y < 0 || p.y >= ht)
				continue;
			mat(p.y, p.x) = value;
		}
	}

		// save blobs
	nPos = m_blobMap.getFirstPos();
	SkBlobNode *pBlobNode;
	while (nPos >= 0)
	{
		pBlobNode = m_blobMap.getNextNode(nPos);
		if (!pBlobNode->valid())
			continue;
		SkPrimBlob *pBlob = &(pBlobNode->m_data);
		SkIntPoint p0 = pBlob->getRefPoint();

		double dRad = pBlob->getRadius();
		int n = ::rasterize_circle(p0.x, p0.y, int(dRad+0.5), m_pnX, m_pnY);

		for (int i = 0; i < n; i++)
		{
			int x = m_pnX[i];
			int y = m_pnY[i];
			if (x < 0 || x >= wd || y < 0 || y >= ht)
				continue;
			mat(y,x) = value;
		}
	}
	return 0;
}
int SkPrimManager::saveStraightLinesOfCurves(Matrix<int> &mat, int value, int line_width)
{
	// save cons
	int wd, ht;
	mat.GetDimension(ht, wd);

	int hf = line_width/2;

	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;

	int nPos = m_branchMap.getFirstPos();
	SkIntPoint p;
	while (nPos >= 0)
	{
		pBhNode = m_branchMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		pBranch = &(pBhNode->m_data);
		int nSegNum = pBranch->getSegNum();
		assert(nSegNum >= 1);

		SkSegment *pSeg;
		for (int s = 0; s < nSegNum; s++)
		{
			pSeg = pBranch->getSegment(s);

			int nPtNum = pSeg->getPtNum();
			int nPtAxis = pSeg->getPointAxis();//0~x, 1~y
			for (int k = 0; k < nPtNum; k++)
			{
				p = pSeg->getPoint(k);
				for (int d = -hf; d <= hf; d++)
				{
					int x = p.x;
					int y = p.y;
					if (nPtAxis == 0)
						y += d;
					else
						x += d;
					if (x < 0 || x >= wd || y < 0 || y >= ht)
						continue;
					mat(y,x) = value;
				}
			}
		}
	}	

		// save blobs
	nPos = m_blobMap.getFirstPos();
	SkBlobNode *pBlobNode;
	while (nPos >= 0)
	{
		pBlobNode = m_blobMap.getNextNode(nPos);
		if (!pBlobNode->valid())
			continue;
		SkPrimBlob *pBlob = &(pBlobNode->m_data);
		SkIntPoint p0 = pBlob->getRefPoint();
		double dRad = pBlob->getRadius();
		int n;
		if ( line_width == 1)
			n = ::rasterize_circle(p0.x, p0.y, int(dRad+0.5), m_pnX, m_pnY);
		else
			n = ::rasterize_pie(p0.x, p0.y, int(dRad+0.5), m_pnX, m_pnY);

		for (int i = 0; i < n; i++)
		{
			int x = m_pnX[i];
			int y = m_pnY[i];
			if (x < 0 || x >= wd || y < 0 || y >= ht)
				continue;
			mat(y,x) = value;
		}
	}

	return 0;
}

///////////////////
// //just for a figure in the thesis
int SkPrimManager::saveAnExampleJoint()
{
	char name[512];
	char work_dir[512];
	sprintf(work_dir, "joints");
	_mkdir(work_dir);


	int hf = 21;
	int wd1, ht1;
	wd1 = ht1 = hf*2 + 1;
	Matrix<float> org(ht1,wd1), rec(ht1,wd1), org0(ht1,wd1);
	Matrix<int> sk(ht1, wd1);
	Matrix<SkPixCandArray> rec_cand(ht1, wd1);

	rec.Set(128);
	org.Set(0);
	org0.Set(0);
	sk.Set(255);

	int m, n;

	int w = 3 * 2 + 1;


	int xe = hf;
	int ye = hf;//absolute point which should be in the center of the icon
	double *vm = new double[w];
	int i;
	int x0, y0, x1, y1;

	//stroke 1
	x0 = hf;
	y0 = hf;
	x1 = hf - 21;
	y1 = hf - 0;
	vm[0] = 50.0;
	vm[1] = 50.0;
	vm[2] = 100.0;
	vm[3] = 150.0;
	vm[4] = 200.0;
	vm[5] = 250.0;
	vm[6] = 250.0;

	for(i = 0; i < w; i++) 
		vm[i] -= 128;

	saveImageAndSketch4ALinelet(x0, y0, x1, y1, xe, ye, wd1/2, ht1/2, w, vm, org, rec_cand, sk, org0);

	//stroke 2
	x1 = hf + 0;
	y1 = hf - 21;
	vm[0] = 250.0;
	vm[1] = 150.0;
	vm[2] = 75.0;
	vm[3] = 0.0;
	vm[4] = 50.0;
	vm[5] = 100.0;
	vm[6] = 150.0;
	for(i = 0; i < w; i++) 
		vm[i] -= 128;

	saveImageAndSketch4ALinelet(x0, y0, x1, y1, xe, ye, wd1/2, ht1/2, w, vm, org, rec_cand, sk, org0);

	//stroke 3
	x1 = hf + 21;
	y1 = hf;
	vm[0] = 150.0;
	vm[1] = 150.0;
	vm[2] = 125.0;
	vm[3] = 100.0;
	vm[4] = 70.0;
	vm[5] = 50.0;
	vm[6] = 50.0;
	for(i = 0; i < w; i++) 
		vm[i] -= 128;

	saveImageAndSketch4ALinelet(x0, y0, x1, y1, xe, ye, wd1/2, ht1/2, w, vm, org, rec_cand, sk, org0);

	skComputeImageFromPixCandByMixing(rec_cand, rec);
	sprintf(name, "%s\\joint0_rec.bmp", work_dir);
	::skSaveMatrixAsBitmap(name, rec, 0, -1);
	sprintf(name, "%s\\joint0_sk.bmp", work_dir);
	::skSaveMatrixAsBitmap(name, sk, 0, 1);

	delete []vm;

	return 0;
}
///////////////////
// 
int SkPrimManager::saveSingleJoints(const char *type)
{
	char name[512];
	char work_dir[512];
	sprintf(work_dir, "joints");
	_mkdir(work_dir);

	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);

	int nMinPtNum = 11;
	int hf = nMinPtNum;
	int wd1, ht1;
	wd1 = ht1 = hf*2 + 1;
	Matrix<float> org(ht1,wd1), rec(ht1,wd1);
	Matrix<int> sk(ht1, wd1);
	Matrix<SkPixCandArray> rec_cand(ht1, wd1);

	int m, n;

	int w = SkPrimBranch::m_nHalfWd * 2 + 1;

	int pnJnctNum[10];
	memset(pnJnctNum, 0, sizeof(int)*10);
	char no[32];
	// save joints
	SkVPrimNode *pNode;
	int nPos = m_vpMap.getFirstPos();
	while (nPos >= 0)
	{
		pNode = m_vpMap.getNextNode(nPos);
		if (!pNode->valid())
			continue;
		SkVirtualPrim &vp = pNode->m_data;
		int nBranch = vp.getBranchNum();
		if (nBranch <= 2)
			continue;
		// check the linelets at the end of the branch are long
		bool bLong = true;
		int i;
		for (i = 0; i < nBranch; i++)
		{
			int bhID = vp.getBranchID(i);
			SkPrimBranch *pBh = m_branchMap.getAt(bhID);
			int e = vp.getBranchEnd(i);
			SkSegment *pSeg = pBh->getEndSegment(e);
			int nPtNum = pSeg->getPtNum();
			if (nPtNum < nMinPtNum)
			{
				bLong = false;
				break;
			}
		}
		if (!bLong) 
			continue;
		//clear the matrices
		rec.Set(0);
		org.Set(0);
		sk.Set(255);
		int r,c;
		for (r = 0; r < ht1; r++)
			for (c = 0; c < wd1; c++)
				rec_cand(r,c).removeAll();
		::changeNumTo4Char(no, pnJnctNum[nBranch]);
		pnJnctNum[nBranch]++;
		for (i = 0; i < nBranch; i++)
		{
			int bhID = vp.getBranchID(i);
			SkPrimBranch *pBh = m_branchMap.getAt(bhID);
			int e = vp.getBranchEnd(i);
			SkSegment *pSeg = pBh->getEndSegment(e);
			int nPtNum = pSeg->getPtNum();

			double *vm;
			vm = pSeg->m_prfl.m_ridgeModel.getModelIntensity();

			int xe, ye;//the end point connecting to the joint
			SkIntPoint p0, p1;
			if (e==0)
			{
				p0 = pSeg->getPoint(0);
				p1 = pSeg->getPoint(nMinPtNum-1);
				xe = p0.x;
				ye = p0.y;
			}
			else
			{
				p0 = pSeg->getPoint(nPtNum-nMinPtNum);
				p1 = pSeg->getPoint(nPtNum-1);
				xe = p1.x;
				ye = p1.y;
			}
			saveImageAndSketch4ALinelet(p0.x, p0.y, p1.x, p1.y, xe, ye, wd1/2, ht1/2, w, vm, org, rec_cand, sk, m_imgOrg);
		}//for each branch
		rec.Set(0);
		skComputeImageFromPixCandByMixing(rec_cand, rec);
		sprintf(name, "%s\\%s_%s_%dd_%s_rec.bmp", work_dir, m_pchProjName, type, nBranch, no);
		::skSaveMatrixAsBitmap(name, rec, m_dImgMean, m_dImgStd);
		sprintf(name, "%s\\%s_%s_%dd_%s_org.bmp", work_dir, m_pchProjName,  type, nBranch, no);
		::skSaveMatrixAsBitmap(name, org, m_dImgMean, m_dImgStd);
		sprintf(name, "%s\\%s_%s_%dd_%s_sk.bmp", work_dir, m_pchProjName, type, nBranch, no);
		::skSaveMatrixAsBitmap(name, sk, 0, 1);
	}//for all junctions

	// save the corners
	double dMaxCornerAngle = PI * 3/4;
	nPos = m_branchMap.getFirstPos();
	SkBranchNode *pBhNode;
	SkPrimBranch *pBranch;
	int nBranch = 2;
	while (nPos >= 0)
	{
		pBhNode = m_branchMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		pBranch = &(pBhNode->m_data);
		int nSegNum = pBranch->getSegNum();
		SkSegment *pSeg1, *pSeg2;
		int st, nd;
		if (nSegNum >= 2)
		{
			for (int s = 0; s < nSegNum-1; s++)
			{
				double dAngle = pBranch->getCornerAngle(s);
				if (dAngle > dMaxCornerAngle)
					continue;
				pSeg1 = pBranch->getSegment(s);
				pSeg2 = pBranch->getSegment(s+1);
				int nPtNum1 = pSeg1->getPtNum();
				if (nPtNum1 < nMinPtNum)
					continue;
				int nPtNum2 = pSeg2->getPtNum();
				if (nPtNum2 < nMinPtNum)
					continue;

				//clear the matrices
				rec.Set(0);
				org.Set(0);
				sk.Set(255);
				int r,c;
				for (r = 0; r < ht1; r++)
					for (c = 0; c < wd1; c++)
						rec_cand(r,c).removeAll();
				::changeNumTo4Char(no, pnJnctNum[nBranch]);
				pnJnctNum[nBranch]++;


				double *vm;
				SkIntPoint p0, p1;
				int xe, ye;//the end point connecting to the joint
				//first part
				vm = pSeg1->m_prfl.m_ridgeModel.getModelIntensity();

				p0 = pSeg1->getPoint(nPtNum1-nMinPtNum);
				p1 = pSeg1->getPoint(nPtNum1-1);
				xe = p1.x;
				ye = p1.y;
				saveImageAndSketch4ALinelet(p0.x, p0.y, p1.x, p1.y, xe, ye, wd1/2, ht1/2, w, vm, org, rec_cand, sk, m_imgOrg);
				//second part
				vm = pSeg2->m_prfl.m_ridgeModel.getModelIntensity();

				p0 = pSeg2->getPoint(0);
				p1 = pSeg2->getPoint(nMinPtNum-1);
				xe = p0.x;
				ye = p0.y;
				saveImageAndSketch4ALinelet(p0.x, p0.y, p1.x, p1.y, xe, ye, wd1/2, ht1/2, w, vm, org, rec_cand, sk, m_imgOrg);

				rec.Set(0);
				skComputeImageFromPixCandByMixing(rec_cand, rec);
				sprintf(name, "%s\\%s_%s_%dd_%s_rec.bmp", work_dir, m_pchProjName, type, nBranch, no);
				::skSaveMatrixAsBitmap(name, rec, m_dImgMean, m_dImgStd);
				sprintf(name, "%s\\%s_%s_%dd_%s_org.bmp", work_dir, m_pchProjName,  type, nBranch, no);
				::skSaveMatrixAsBitmap(name, org, m_dImgMean, m_dImgStd);
				sprintf(name, "%s\\%s_%s_%dd_%s_sk.bmp", work_dir, m_pchProjName, type, nBranch, no);
				::skSaveMatrixAsBitmap(name, sk, 0, 1);
			}// for each segment (corner)
		}//if there are corners
	}	//for all branches

	return 0;
}

int SkPrimManager::saveImageAndSketch4ALinelet(int x0, int y0, int x1, int y1,//start and end absolute pt
						   int xe, int ye, //the absolute point which should be at the center of final image
						   int xd, int yd, //the position of (xe,ye) in the new image
						   int w,//the width of the profile
						   double *vm,//the intensity value of the profile
						   Matrix<float> &org,//original image to be saved
						   Matrix<SkPixCandArray> &rec_cand,//the candidate image
						   Matrix<int> &sk,//the sketch image
						   Matrix<float> &org0)//the whole original image
{
	int wd, ht;
	org0.GetDimension(ht, wd);
	int wd1, ht1;
	org.GetDimension(ht1, wd1);

	int pnX_v[MAX_CON_PROFILE_WD], pnY_v[MAX_CON_PROFILE_WD];
	int pf_x0, pf_x1, pf_y0, pf_y1;
	int x, y, dx, dy;
	int xc, yc;//center position
	double xt, yt;//true position

	int m, n;
	n = ::rasterize_line_segment_keeping_order(x0, y0, x1, y1, m_pnX, m_pnY);
	::verticalLineApproximately(x0, y0, x1, y1, pf_x0, pf_y0, pf_x1, pf_y1, w);
	m = ::rasterize_line_segment_keeping_order(pf_x0, pf_y0, pf_x1, pf_y1, pnX_v, pnY_v);
	SkPixCand pc;
	for (int i = 0; i < n; i++)
	{//there is an overlap on the join point
		xc = m_pnX[i];
		yc = m_pnY[i];//all the axis point

		for (int j = 0; j < m; j++)
		{
			dx = pnX_v[j];
			dy = pnY_v[j];//verticle piont of the
			int xx = xc + dx;
			int yy = yc + dy;
			if (xx < 0 || xx >= wd || yy < 0 || yy >= ht)
				continue;
			x =  xx - xe + xd;
			y =  yy - ye + yd;
			if ( x < 0 || x >= wd1 || y < 0 || y >= ht1)
				continue;
			pc.m_dValue = vm[j];
			pc.m_dDis = fabs(j-w/2.f);
			rec_cand(y,x).add(pc);

			org(y,x) = org0(yy,xx);
		}//for each vertical pint
		x = xc - xe + xd;
		y = yc - ye + yd;
		if ( x < 0 || x >= wd1 || y < 0 || y >= ht1)
			continue;
		sk(y,x) = 0;
	}
	return n;
}
int SkPrimManager::updatePixCand4ALinelet(int x0, int y0, int x1, int y1,//start and end absolute pt
						   int w,//the width of the profile
						   double *vm,//the intensity value of the profile
						   Matrix<SkPixCandArray> &rec_cand, int add_remove)//the candidate image
{
	int wd, ht;
	rec_cand.GetDimension(ht, wd);

	int pnX_v[MAX_CON_PROFILE_WD], pnY_v[MAX_CON_PROFILE_WD];
	int pf_x0, pf_x1, pf_y0, pf_y1;
	int x, y, dx, dy;
	int xc, yc;//center position
	double xt, yt;//true position

	int m, n;
	n = ::rasterize_line_segment_keeping_order(x0, y0, x1, y1, m_pnX, m_pnY);
	::verticalLineApproximately(x0, y0, x1, y1, pf_x0, pf_y0, pf_x1, pf_y1, w);
	m = ::rasterize_line_segment_keeping_order(pf_x0, pf_y0, pf_x1, pf_y1, pnX_v, pnY_v);
	SkPixCand pc;
	for (int i = 0; i < n; i++)
	{//there is an overlap on the join point
		xc = m_pnX[i];
		yc = m_pnY[i];//all the axis point

		for (int j = 0; j < m; j++)
		{
			dx = pnX_v[j];
			dy = pnY_v[j];//verticle piont of the
			x = xc + dx;
			y = yc + dy;
			if (x < 0 || x >= wd || y < 0 || y >= ht)
				continue;
			pc.m_dValue = vm[j];
			pc.m_dDis = fabs(j-w/2.f);
			if (add_remove == ADD)
				rec_cand(y,x).add(pc);
			else
				rec_cand(y,x).remove(pc);
		}//for each vertical pint
	}
	return n;
}
int SkPrimManager::saveCurveByCurve(const char *type, int step)
{
	char work_dir[512];
	sprintf(work_dir, g_pchSketchWorkDir);
	_mkdir(work_dir);

	char name[512], no[32];

	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);
	Matrix<int> mat;
	mat.SetDimension(ht, wd);
	mat.Set(255);

	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;

	int value = 0;
	int n = m_branchMap.getValidNodeNum();
	if ( step > n)
		return -1;
	SkIntPoint p;

	// save a blank one
	int num = 0;
	::changeNumTo4Char(no, num);
	sprintf(name, "%s\\%s_%s_s%s.bmp", work_dir, m_pchProjName, type, no);
	::skSaveMatrixAsBitmap(name, mat, 0, -1);

	for (int i = 0; i < n; i++)
	{
		int id = m_sortedBhIDs[i];

		pBhNode = m_branchMap.getNodeAt(id);
		if (!pBhNode->valid())
			continue;
		pBranch = &(pBhNode->m_data);
		int nSegNum = pBranch->getSegNum();
		assert(nSegNum >= 1);

		SkSegment *pSeg;
		for (int s = 0; s < nSegNum; s++)
		{
			pSeg = pBranch->getSegment(s);

			int nPtNum = pSeg->getPtNum();
			for (int k = 0; k < nPtNum; k++)
			{
				p = pSeg->getPoint(k);
				int x = p.x;
				int y = p.y;
				if (x < 0 || x >= wd || y < 0 || y >= ht)
					continue;
				mat(y,x) = value;
			}
		}
		num = i+1; 
		if (num/step * step == num || num == n)
		{
			::changeNumTo4Char(no, num);
			sprintf(name, "%s\\%s_%s_s%s.bmp", work_dir, m_pchProjName, type, no);
			::skSaveMatrixAsBitmap(name, mat, 0, -1);
		}
	}
	
	sprintf(name, "%s_%s_curve_sth.txt", m_pchProjName, type);
	::saveArray(m_bhImgCLGs.getPointerAt(0), n, name);

	return 0;
}				

int SkPrimManager::saveSketchImageReconstructed(Matrix<float> &mat, bool bSharp)
{
	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);
	mat.SetDimension(ht, wd);
	mat.Set(0);

	Matrix<SkPixCandArray> rec_cand(ht,wd); 


	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;

	int pnX_v[MAX_CON_PROFILE_WD], pnY_v[MAX_CON_PROFILE_WD];
	int pf_x0, pf_x1, pf_y0, pf_y1;
	int x, y, dx, dy;
	int xc, yc;//center position
	double xt, yt;//true position
	int m, n;

	int w = SkPrimBranch::m_nHalfWd * 2 + 1;
	int nPos = m_branchMap.getFirstPos();
	SkIntPoint p;
	while (nPos >= 0)
	{
		pBhNode = m_branchMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		pBranch = &(pBhNode->m_data);
		updatePixCand4ABranch(pBranch, rec_cand, ADD, bSharp);
	}//for each curve/branch


	mat.Set(0);
	skComputeImageFromPixCandByMixing(rec_cand, mat);

		// save blobs
	nPos = m_blobMap.getFirstPos();
	SkBlobNode *pBlobNode;
	while (nPos >= 0)
	{
		pBlobNode = m_blobMap.getNextNode(nPos);
		if (!pBlobNode->valid())
			continue;
		SkPrimBlob *pBlob = &(pBlobNode->m_data);
		pBlob->computeProfile(&m_imgOrg, g_nHalfProfileWd);
		SkIntPoint p0 = pBlob->getRefPoint();
		double dRad = pBlob->getRadius();

		double *dvm = pBlob->m_prfl.m_ridgeModel.m_vm;
		int h = pBlob->m_prfl.getSampleWidth();
		skSaveABlobFromProfile(mat, dvm, h, p0.x, p0.y);

	}
	return 0;
}
int SkPrimManager::computePixCand(Matrix<SkPixCandArray> &rec_cand)//the candidate image
{
	int ht, wd;
	getImageDimension(ht, wd);
	rec_cand.SetDimension(1,1);
	rec_cand.SetDimension(ht,wd);

	SkBranchMap &bhMap = m_branchMap;
	SkBranchNode *pBhNode;
	SkPrimBranch *pBranch;

	// compute linearity and R0 and ort first
	int nPos = bhMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = bhMap.getNextNode(nPos);
		
		if (!pBhNode->valid())
			continue;
		pBranch = &(pBhNode->m_data);
		updatePixCand4ABranch(pBranch, rec_cand, ADD, false);
	}
	return 0;
}///////////////////
int SkPrimManager::findSketchPixels(SkPrimBranch *pBranch, int &num, int *pnX, int *pnY)

{
	// for sk, consider each point
	num = 0;
	int n = pBranch->getPointNum();
	SkIntPoint p;
	int i;
	for (i = 0; i < n; i++)
	{
		p = pBranch->getPoint(i);
		pnX[num] = p.x;
		pnY[num] = p.y;
		num++;
	}
	//for con, consider each stroke
	return num;
	
}

///////////////////
int SkPrimManager::findConPixels(SkPrimBranch *pBranch, int &num, int *pnX, int *pnY, int hf_prfl)
{
	int ht, wd;
	m_imgOrg.GetDimension(ht, wd);

	int nSegNum = pBranch->getSegNum();
//	assert(nSegNum >= 1);

	num = 0;

	SkSegment *pSeg;
	SkIntPoint p;
	for (int s = 0; s < nSegNum; s++)
	{
		pSeg = pBranch->getSegment(s);

		int nPtNum = pSeg->getPtNum();
		for (int k = 0; k < nPtNum; k++)
		{
			p = pSeg->getPoint(k);
			pnX[num] = p.x;
			pnY[num] = p.y;
			num++;
		}//for each point
	}//for each segment/stroke
	return num;
}


///////////////////
int SkPrimManager::findNearbyPixels(SkPrimBranch *pBranch, int end, int &num, int *pnX, int *pnY)
{
	int ht, wd;
	m_imgOrg.GetDimension(ht, wd);

	int dd = 1;
	SkIntPoint p0 = pBranch->getEndPoint(end);
	int x, y;
	int dx, dy;

	num = 0;
	for (dy = -dd; dy <= dd; dy++)
	{
		for (dx = -dd; dx <= dd; dx++)
		{
			x = dx + p0.x;
			y = dy + p0.y;
			if (x < 0 || x >= wd || y < 0 || y >= ht)
				continue;
			pnX[num] = x;
			pnY[num] = y;
			num++;
		}
	}
	return num;
}
///////////////////
int SkPrimManager::findExtensionPixels(SkPrimBranch *pBranch, int end, int &num, int *pnX, int *pnY, double dExtDis)
{
	num = 0;
	double left = -m_param.m_dExtRegionHalfLeft;
	double right = dExtDis;
	double top = left;
	double bottom = -top;

	int nSegNum = pBranch->getSegNum();
	SkSegment *pSeg;
	int x_in, y_in, x_out, y_out;
	SkIntPoint p;
	if (end == 0)
	{
		pSeg = pBranch->getSegment(0);
		p = pSeg->getEndPoint(0);
		x_out = p.x;
		y_out = p.y;
		p = pSeg->getEndPoint(1);
		x_in = p.x;
		y_in = p.y;
	}
	else
	{
		pSeg = pBranch->getSegment(nSegNum - 1);
		p = pSeg->getEndPoint(1);
		x_out = p.x;
		y_out = p.y;
		p = pSeg->getEndPoint(0);
		x_in = p.x;
		y_in = p.y;
	}
	double dx = x_out - x_in;
	double dy = y_out - y_in;
	double dOrt;
	dOrt = atan2(dy, dx);

	if (m_param.m_nExtTrapezoidleRegion)
		num = ::skGetRotatedTrapezoidleRegion(x_out, y_out, dOrt, left, right, top, bottom, pnX, pnY);
	else
		num = ::skGetRotatedRectangleRegion(x_out, y_out, dOrt, left, right, top, bottom, pnX, pnY);

	return num;
}
///////////////////
int SkPrimManager::recomputeExtConOcp(int hf_prfl, double extDis)
{
	int wd, ht;
	getImageDimension(ht, wd);
	m_conOcp.SetDimension(1,1);//remove all
	m_conOcp.SetDimension(ht, wd);
	m_extOcp.SetDimension(1,1);
	m_extOcp.SetDimension(ht, wd);
	m_extEndOcp.SetDimension(1,1);
	m_extEndOcp.SetDimension(ht, wd);

	SkBranchNode *pBhNode;
	int nPos = m_branchMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = m_branchMap.getNextNode(nPos);
		
		if (!pBhNode->valid())
			continue;
		int id = pBhNode->getID();

		SkPrimBranch *pBranch = &(pBhNode->m_data);
		int conPtNum;
		findConPixels(pBranch, conPtNum, m_pnX, m_pnY, hf_prfl);
		updateGeneralOcp(m_conOcp, id, conPtNum, m_pnX, m_pnY, ADD);

		int nExtPtNum;
		for (int e = 0; e < 2; e++)
		{
			if (pBranch->getPrimID(e) >= 0)
				continue;
			findExtensionPixels(pBranch, e, nExtPtNum, m_pnX, m_pnY, extDis);
			updateExtensionOcp(m_extOcp, m_extEndOcp, id, e, 
				nExtPtNum, m_pnX, m_pnY, ADD);

		}

	}
	return 0;
}
int SkPrimManager::updateGeneralOcp(Matrix<oval_IntArray>&ocp, int id, int num, int *pnX, int *pnY, int flag)
{
	int ht, wd;
	ocp.GetDimension(ht, wd);
	int i;
	for (i = 0; i < num; i++)
	{
		int x = pnX[i];
		int y = pnY[i];
		if (x < 0 || x >= wd || y < 0 || y >= ht)
			continue;
		if (flag == ADD)
			ocp(y,x).add(id);
		else
			ocp(y,x).remove(id);
	}
	return 0;
}
///////////////////
int SkPrimManager::updateGeneralOcp(Matrix<oval_IntArray>&ocp, int id, 
									int x0, int y0, int x1, int y1, int flag)
{
	int ht, wd;
	ocp.GetDimension(ht, wd);
	int num = ::rasterize_line_segment_keeping_order(x0, y0, x1, y1, m_pnX, m_pnY);
	int i;
	for (i = 1; i < num; i++)
	{//skip the first point
		int x = m_pnX[i];
		int y = m_pnY[i];
		if (x < 0 || x >= wd || y < 0 || y >= ht)
			continue;
		if (flag == ADD)
			ocp(y,x).add(id);
		else
			ocp(y,x).remove(id);
	}
	return num;
}
///////////////////
int SkPrimManager::updateExtensionOcp(Matrix<oval_IntArray>&extOcp, 
									  Matrix<oval_IntArray>&extEndOcp,
									  int id, int end, 
									  int num, int *pnX, int *pnY, 
									  int flag)
									  
{
	int ht, wd;
	extOcp.GetDimension(ht, wd);
	int i;
	for (i = 0; i < num; i++)
	{
		int x = pnX[i];
		int y = pnY[i];
		if (x < 0 || x >= wd || y < 0 || y >= ht)
			continue;
		if (flag == ADD)
		{
			extOcp(y,x).add(id);
			extEndOcp(y,x).add(end);
		}
		else
		{//remove
			oval_IntArray &idArr = extOcp(y,x);
			oval_IntArray &endArr = extEndOcp(y,x);
			int n = idArr.getSize();

			bool bFind = false;
			for (int k = 0; k < n; k++)
			{
				if (idArr[k] == id && endArr[k] == end)
				{
					idArr.removeAt(k);
					endArr.removeAt(k);
					bFind = true;
					break;
				}
			}
//			assert(bFind);//Here is a bug left, ext/con are not uptated consistantly
		}//remove
	}//for each point
	return 0;
}

int SkPrimManager::updatePixCand4ABranch(SkPrimBranch *pBranch, 
										 Matrix<SkPixCandArray> &rec_cand,//the candidate image
										int add_remove, bool bSharp)
{
	int w = SkPrimBranch::m_nHalfWd * 2 + 1; 
	int nSegNum = pBranch->getSegNum();
	assert(nSegNum >= 1);
	SkIntPoint p;
	SkSegment *pSeg;
	for (int s = 0; s < nSegNum; s++)
	{
		pSeg = pBranch->getSegment(s);

		int nPtNum = pSeg->getPtNum();

		double *vm;
		if (!bSharp)
			vm = pSeg->m_prfl.m_ridgeModel.getModelIntensity();
		else
			vm = pSeg->m_prfl.m_ridgeModel.getModelIntensitySharp();

		p = pSeg->getPoint(0);
		int x0 = p.x;
		int y0 = p.y;
		p = pSeg->getPoint(nPtNum-1);
		int x1 = p.x;
		int y1 = p.y;

		updatePixCand4ALinelet(x0, y0, x1, y1, w, vm, rec_cand, add_remove);
	}
	return 0;
}
int SkPrimManager::getSegmentNum()
{
	int n = 0;
	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;
	int nPos = m_branchMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = m_branchMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		pBranch = &(pBhNode->m_data);
		int nSegNum = pBranch->getSegNum();
		n += nSegNum;
	}

	return n;
}
int SkPrimManager::saveResidualImagesStokeByStroke(const char *type, int step)
{
	char name[512], work_dir[512], no[32];
	sprintf(work_dir, g_pchSketchReconstructedDir);
	_mkdir(work_dir);

	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);

	Matrix<float> rec(ht,wd), res(ht,wd);//reconstructed and residual image
	Matrix<SkPixCandArray> rec_cand(ht,wd); 

	res = m_imgOrg;
	int nBound = 0;

	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;

	int nSegNum = getSegmentNum();
	double *pdRes = new double[nSegNum+1];

	int w = SkPrimBranch::m_nHalfWd * 2 + 1;
	int nPos = m_branchMap.getFirstPos();
	SkIntPoint p;
	int n = 0;
	pdRes[n] = ::skSquareSumOfMatrix(&res, nBound);

	rec.Set(0);
	int num = 0;
	::changeNumTo4Char(no, num);
	sprintf(name, "%s\\%s_%s_rec_%s.bmp", work_dir, m_pchProjName, type, no);
	::skSaveMatrixAsBitmap(name, rec, m_dImgMean, m_dImgStd);

	res = m_imgOrg - rec;
	sprintf(name, "%s\\%s_%s_res_%s.bmp", work_dir, m_pchProjName, type, no);
	::skSaveMatrixAsBitmap(name, res, m_dImgMean, m_dImgStd);

	int nCurveNum = m_branchMap.getValidNodeNum();
	for (int i = 0; i < nCurveNum; i++)
	{
		int id = m_sortedBhIDs[i];

		pBhNode = m_branchMap.getNodeAt(id);
		if (!pBhNode->valid())
			continue;
		pBranch = &(pBhNode->m_data);
		int nSegNum = pBranch->getSegNum();
		assert(nSegNum >= 1);

		SkSegment *pSeg;
		for (int s = 0; s < nSegNum; s++)
		{
			pSeg = pBranch->getSegment(s);

			int nPtNum = pSeg->getPtNum();

			double *vm;
			vm = pSeg->m_prfl.m_ridgeModel.getModelIntensity();

			p = pSeg->getPoint(0);
			int x0 = p.x;
			int y0 = p.y;
			p = pSeg->getPoint(nPtNum-1);
			int x1 = p.x;
			int y1 = p.y;

			updatePixCand4ALinelet(x0, y0, x1, y1, w, vm, rec_cand, ADD);
		}//for each segment/stroke

		num = i+1; 
		if (num/step * step == num || num == nCurveNum)
		{
			rec.Set(0);
			skComputeImageFromPixCandByMixing(rec_cand, rec);
			::changeNumTo4Char(no, num);
			sprintf(name, "%s\\%s_%s_rec_%s.bmp", work_dir, m_pchProjName, type, no);
			::skSaveMatrixAsBitmap(name, rec, m_dImgMean, m_dImgStd);

			res = m_imgOrg - rec;
			sprintf(name, "%s\\%s_%s_res_%s.bmp", work_dir, m_pchProjName, type, no);
			::skSaveMatrixAsBitmap(name, res, m_dImgMean, m_dImgStd);

			n++;
			pdRes[n] = ::skSquareSumOfMatrix(&res, nBound);
		}
	}//for each branch/curve
	
	sprintf(name, "%s_%s_sk_res.txt", m_pchProjName, type);
	::saveArray(pdRes, n+1, name);

	delete []pdRes;
	return 0;
}

// return dots num
int SkPrimManager::markEndPointsAsDots(Matrix<int> &mat, int value)
{
	int wd, ht;
	mat.GetDimension(ht, wd);
	int sum_dots = 0;
		// save joints
	SkIntPoint p0;
	SkVPrimNode *pNode;
	int nPos = m_vpMap.getFirstPos();
	while (nPos >= 0)
	{
		pNode = m_vpMap.getNextNode(nPos);
		if (!pNode->valid())
			continue;
		SkVirtualPrim &vp = pNode->m_data;
		int nBranch = vp.getBranchNum();
		if (nBranch == 1)
		{
			sum_dots++;
			p0 = vp.m_refPt;
			int n = ::rasterize_pie(p0.x, p0.y, 1, m_pnX, m_pnY);
			for (int i = 0; i < n; i++)
			{
				int x = m_pnX[i];
				int y = m_pnY[i];
				if (x < 0 || x >= wd || y < 0 || y >= ht)
					continue;
				mat(y,x) = value;
			}
		}
	}
	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;
	nPos = m_branchMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = m_branchMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		pBranch = &(pBhNode->m_data);
		for (int e = 0; e < 2; e++)
		{
			int vpID = pBranch->getPrimID(e);
			if (vpID < 0)//end point
			{
				sum_dots++;
				p0 = pBranch->getEndPoint(e);
				int n = ::rasterize_pie(p0.x, p0.y, 1, m_pnX, m_pnY);
				for (int i = 0; i < n; i++)
				{
					int x = m_pnX[i];
					int y = m_pnY[i];
					if (x < 0 || x >= wd || y < 0 || y >= ht)
						continue;
					mat(y,x) = value;
				}
			}
		}
	}

	return sum_dots;
}
// return sum dots
int SkPrimManager::markCornersAsDots(Matrix<int> &mat, int value)
{
	int wd, ht;
	mat.GetDimension(ht, wd);

	int sum_dots = 0;
	SkIntPoint p0;
	SkVPrimNode *pNode;
	int nPos = m_vpMap.getFirstPos();
	while (nPos >= 0)
	{
		pNode = m_vpMap.getNextNode(nPos);
		if (!pNode->valid())
			continue;
		SkVirtualPrim &vp = pNode->m_data;
		int nBranch = vp.getBranchNum();
		if (nBranch == 2)
		{
			sum_dots++;
			p0 = vp.m_refPt;
			int n = ::rasterize_pie(p0.x, p0.y, 1, m_pnX, m_pnY);
			for (int i = 0; i < n; i++)
			{
				int x = m_pnX[i];
				int y = m_pnY[i];
				if (x < 0 || x >= wd || y < 0 || y >= ht)
					continue;
				mat(y,x) = value;
			}
		}
	}
	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;
	nPos = m_branchMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = m_branchMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		pBranch = &(pBhNode->m_data);
		int nSegNum = pBranch->getSegNum();
		SkSegment *pSeg;
		if (nSegNum >= 2)
		{
			for (int s = 0; s < nSegNum-1; s++)
			{
				sum_dots++;
				pSeg = pBranch->getSegment(s);
				int nPtNum = pSeg->getPtNum();
				p0 = pSeg->getPoint(nPtNum-1);
				int n = ::rasterize_pie(p0.x, p0.y, 1, m_pnX, m_pnY);
				for (int i = 0; i < n; i++)
				{
					int x = m_pnX[i];
					int y = m_pnY[i];
					if (x < 0 || x >= wd || y < 0 || y >= ht)
						continue;
					mat(y,x) = value;
				}			
			}// for each segment (corner)
		}//if there are corners
	}	//for all branches
	return sum_dots;
}
int SkPrimManager::markJunctionsAsDots(Matrix<int> &mat, int value)
{
	int wd, ht;
	mat.GetDimension(ht, wd);

	int sum_dots = 0;
	SkIntPoint p0;
	SkVPrimNode *pNode;
	int nPos = m_vpMap.getFirstPos();
	while (nPos >= 0)
	{
		pNode = m_vpMap.getNextNode(nPos);
		if (!pNode->valid())
			continue;
		SkVirtualPrim &vp = pNode->m_data;
		int nBranch = vp.getBranchNum();
		if (nBranch > 2)
		{
			sum_dots++;
			p0 = vp.m_refPt;
			int n = ::rasterize_pie(p0.x, p0.y, 1, m_pnX, m_pnY);
			for (int i = 0; i < n; i++)
			{
				int x = m_pnX[i];
				int y = m_pnY[i];
				if (x < 0 || x >= wd || y < 0 || y >= ht)
					continue;
				mat(y,x) = value;
			}
		}
	}	
	return sum_dots;
}

///////////////////
// removeShortFreeBranches()
// return the num of removed
int SkPrimManager::removeShortFreeBranches(int nMinBranchLength)
{
	int nRemoved = 0;

	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;
	int nPos = m_branchMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = m_branchMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		int id = pBhNode->getID();
		pBranch = &(pBhNode->m_data);
		if (pBranch->getPrimID(0) >= 0 && pBranch->getPrimID(1) >= 0)
			continue;
		int nPtNum = pBranch->getPointNum();
		if (nPtNum < nMinBranchLength)
		{
			 removeOneBranch(id);
			 nRemoved++;
		}
	}
	return nRemoved;
}

///////////////////
int SkPrimManager::removeShortBranches(double dMinBranchLength)
{	
	int nRemoved = 0;

	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;
	int nPos = m_branchMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = m_branchMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		int id = pBhNode->getID();
		pBranch = &(pBhNode->m_data);
		int nPtNum = pBranch->getPointNum();
		if (nPtNum < dMinBranchLength)
		{
			 removeOneBranch(id);
			 nRemoved++;
		}
	}
	return nRemoved;
}
///////////////////
int SkPrimManager::removeSmallImgCLGBranches(double dMinImgCLG)
{	
	int nRemoved = 0;

	SkPrimBranch *pBranch;
	SkBranchNode *pBhNode;
	int nPos = m_branchMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = m_branchMap.getNextNode(nPos);
		if (!pBhNode->valid())
			continue;
		int id = pBhNode->getID();
		pBranch = &(pBhNode->m_data);
		double dImgCLG = pBranch->getImageCodingLengthGain();
		if (dImgCLG < dMinImgCLG)
		{
			 removeOneBranch(id);
			 nRemoved++;
		}
	}
	return nRemoved;
}



///////////////////
// value 0-valid point
int SkPrimManager::diffuseBranchPoints()
{

	SkBranchMap &bhMap = m_branchMap;
	SkBranchNode *pBhNode;

	int nMinSegLength = 4;
	// compute linearity and R0 and ort first
	int nPos = bhMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = bhMap.getNextNode(nPos);
		
		if (!pBhNode->valid())
			continue;
		pBhNode->m_data.recomputeSegmentsFromFloatPoints(m_param.m_dPixTol, m_param.m_dAngTol, m_param.m_nLinkRad);
	}
	return 0;
}

///////////////////
// value 0-valid point
int SkPrimManager::buildBranchSegments()
{

	SkBranchMap &bhMap = m_branchMap;
	SkBranchNode *pBhNode;

	int nMinSegLength = 4;
	// compute linearity and R0 and ort first
	int nPos = bhMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = bhMap.getNextNode(nPos);
		
		if (!pBhNode->valid())
			continue;
		pBhNode->m_data.computeSegments(m_param.m_dPixTol, m_param.m_dAngTol, m_param.m_nLinkRad);
	}
	return 0;
}
///////////////////
// value 0-valid point
int SkPrimManager::buildBranchProfiles()
{

	SkBranchMap &bhMap = m_branchMap;
	SkBranchNode *pBhNode;

	// compute linearity and R0 and ort first
	int nPos = bhMap.getFirstPos();
	while (nPos >= 0)
	{
		pBhNode = bhMap.getNextNode(nPos);
		
		if (!pBhNode->valid())
			continue;
		pBhNode->m_data.computeProfile(&m_imgOrg);
	}
	return 0;
}


///////////////////
// combineConnectedVirtualPrimitives)
//	0-valid point
int SkPrimManager::combineConnectedVirtualPrimitives()
{
	Matrix<int> cnRefPt;
	getConnectedPrimitiveRefPt2Matrix(cnRefPt);
//	::skSaveMatrixAsBitmap("temp.bmp", cnRefPt, 0, 1);

	int ht, wd;
	cnRefPt.GetDimension(ht, wd);

	SkIntPointArray ptArray;
	SkIntPoint p;
	int x, y;
	for (y = 0; y < ht; y++)
	{
		for (x = 0; x < wd; x++)
		{
			if (cnRefPt(y,x) != 0)
				continue;//0  is valid
			int n = skFindAllConnectedPoints(x, y, ptArray, cnRefPt);
			combineVirtualPrimitivesWithConnectedRefPts(ptArray);//for those connected ref pts, conbine them
			for (int i = 0; i < ptArray.getSize(); i++)
			{
				p = ptArray[i];
				cnRefPt(p.y,p.x) = 255;
			}
		}
	}


	return 0;
}
///////////////////
// getConnectedPrimitiveRefPt2Matrix()
//	0-valid connected refPt
int SkPrimManager::getConnectedPrimitiveRefPt2Matrix(Matrix<int> & mat)
{
	int wd, ht;
	m_imgOrg.GetDimension(ht, wd);
	mat.SetDimension(ht, wd);
	mat.Set(255);

	// get all of the primitive center (ref) pts
	Matrix<int> refPtOcp(ht, wd);

	for (int y = 0; y < ht; y++)
	{
		for (int x = 0; x < wd; x++)
		{
			if (m_vpID(y,x) >= 0)
				refPtOcp(y,x) = 0;
			else
				refPtOcp(y,x) = 255;
		}
	}
	SkIntPointArray nbrPtArray;

	// check if two or more are connected to reach other, if so then they are connected primitive ref pts
	for (int y = 0; y < ht; y++)
	{
		for (int x = 0; x < wd; x++)
		{
			if (refPtOcp(y,x) != 0)
				continue;
			int cp = skFindConnectedPoints(x, y, nbrPtArray, refPtOcp);//conneceted points
			if (cp >= 1)
				mat(y, x) = 0;
		}
	}
	return 0;
}

///////////////////
// combineRawPrimitivesWithConnectedRefPts()
int SkPrimManager::combineVirtualPrimitivesWithConnectedRefPts(SkIntPointArray& ptArray)
{
	int n = ptArray.getSize();
	assert(n >= 2);

	int pnPrimIDs[256];
	assert(n <= 256);
	//find all the primitive nodes
	SkIntPoint p;
	int i, id;
	for (i = 0; i < n; i++)
	{
		p = ptArray[i];
		id = m_vpID(p.y, p.x);
		assert(id >= 0);
		pnPrimIDs[i] = id;
	}
	// decide the reference point
	SkIntPoint refPt;

	double sumX=0, sumY = 0;
	for (i = 0; i < n; i++)
	{
		sumX += ptArray[i].x;
		sumY += ptArray[i].y;
	}

	refPt.x = (int)(sumX/n + 0.5);
	refPt.y = (int)(sumY/n + 0.5);

	combineVirtualPrimitives(refPt, pnPrimIDs, ptArray);
	return 0;
}

///////////////////
int SkPrimManager::combineVirtualPrimitives(SkIntPoint refPt,//new refence points
									 int *pnPrimIDs,//primitives needed to be combined
									 SkIntPointArray& ptArray)//the ref_pts of all primitives
{
	SkVirtualPrim vp_tmp;
	vp_tmp.m_refPt = refPt;

	int new_vp_id = m_vpMap.add(vp_tmp);
	SkVirtualPrim *pVP_new = m_vpMap.getAt(new_vp_id);

	SkIntPoint p, p1;

	SkVPrimNode *pVPNode;
	int n = ptArray.getSize();
	for (int i = 0; i < n; i++)
	{
		pVPNode = m_vpMap.getNodeAt(pnPrimIDs[i]);

		SkVirtualPrim &vp = pVPNode->m_data;
		int nb = vp.getDegree();
		for (int b = 0; b < nb; b++)
		{//for each branch
			int bhID = vp.getBranchID(b);
			int end = vp.getBranchEnd(b);
			SkPrimBranch *pBranch = m_branchMap.getAt(bhID);
			int ptNum = pBranch->getPointNum();
			if (ptNum <= 1)
			{
				removeOneBranch(bhID);
				if (!pVPNode->valid())
					break;
				b--;
				nb--;
				continue;
			}
			if (end == 0)
				p1 = pBranch->getPoint(1);
			else
				p1 = pBranch->getPoint(ptNum - 2);

			SkIntPoint endPt = pBranch->removeEndPoint(end);
			m_branchIDs(endPt.y, endPt.x).remove(bhID);

			int nn = ::rasterize_line_segment_keeping_order(p1.x, p1.y, refPt.x, refPt.y, m_pnX, m_pnY);
			for (int j = 1; j < nn; j++)
			{
				p.x = m_pnX[j];
				p.y = m_pnY[j];
				pBranch->addPoint(p, end);
				m_branchIDs(p.y, p.x).add(bhID);
			}
			pBranch->setPrimID(end, new_vp_id);
			pVP_new->addBranchIDAndEnd(bhID, end);

		}
		SkIntPoint refPt_tmp = vp.m_refPt;
		m_vpID(refPt_tmp.y, refPt_tmp.x) = -1;
		pVPNode->setValid(false);
	}

	m_vpID(refPt.y, refPt.x) = new_vp_id;
	return 0;
}


///////////////////
// findEndPointOfConnection()
//	from a point (x,y), find any end point connected to (x,y) through a path
int findEndPointOfConnection(int x, int y, Matrix<int> &bound, SkIntPointArray &ptArray0)
{
	ptArray0.removeAll();

	SkIntPointArray nbrPtArray, ptArray;

	SkIntPoint p, dp1, p1;

	// find all of the points on the curve
	p.x = x;
	p.y = y;
	ptArray.add(p);
	int m = 0;
	while (m < ptArray.getSize())
	{
		p = ptArray.getAt(m);
		int cp = skFindConnectedPoints(p.x, p.y, nbrPtArray, bound);//conneceted points
		int n = 0;
		for (int i = 0; i < cp; i++)
		{
			dp1 = nbrPtArray.getAt(i);
			p1 = p + dp1;//branch end's nbr
			if (!ptArray.hasMember(p1))
			{
				ptArray.add(p1);
				n++;
			}
		}
		if (n == 0)
		{
			ptArray0.add(p);
		}
		m++;
	}
	return ptArray0.getSize();
}

///////////////////
// findSequentialPointOfConnection()
//	from a end point (x,y), all the points connected to (x,y) sequentially
int findSequentialPointOfConnection(int x, int y, Matrix<int> &bound, SkIntPointArray &ptArray0)
{
	ptArray0.removeAll();

	SkIntPointArray nbrPtArray;

	SkIntPoint p, dp1, p1;

	// find all of the points on the curve
	p.x = x;
	p.y = y;
	ptArray0.add(p);
	int m = 0;
	while (m < ptArray0.getSize())
	{
		p = ptArray0.getAt(m);
		int cp = skFindConnectedPoints(p.x, p.y, nbrPtArray, bound);//conneceted points
		for (int i = 0; i < cp; i++)
		{
			dp1 = nbrPtArray.getAt(i);
			p1 = p + dp1;//branch end's nbr
			if (!ptArray0.hasMember(p1))
			{
				ptArray0.add(p1);
				break;//since sequential is desired, find one road is enough
			}
		}
		m++;
	}
	return ptArray0.getSize();
}

///////////////////
int skAddBoundary(const char *name, int bound)
{
	Matrix<int> pChn[3];
	int nChannel = ::skLoadImage2YCrCb(name, pChn);

	int wd, ht;
	pChn[0].GetDimension(ht, wd);

	int r, c, k;
	skAddBoundary(pChn[0], bound, 0);
	for (k = 1; k < nChannel; k++)
		skAddBoundary(pChn[k], bound, 128);
	::skSaveImageFromYCrCb(name, pChn, nChannel);
	return 0;
}

	