//GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG
// skImageStat.h		implementation file
//
//
// Author:			Cheng-en Guo
//
// Created Time:	06-13-2003
// Revised Time:	
//
// Descriptions:	This file is for image statistics
//					
//
// Member variables:
//					
// Functions: 	
//
// Copyright (C) 2003, Cheng-en Guo, all rights reserved.
//
//GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG

#include "inlinefunc.h"

#include "skStruct.h"
#include "skImageStat.h"

///////////////////
// skComputeTextureness()
//	for a given image compute textureness
//	The textureness is defined as:
//	"Squared horizontal intensity/color gradient averaged over a squared window of a given size"
int skComputeTextureness(IplImage* pImg, //input image 
						 IplImage* &pTxtr,//textureness, float type, if not created, must initilized to be zero
						 GenParam& param)//pnInt[0] - the half window size to test the first derivative
						 // pdDouble[0] - the threshold to define texture or textureless
{
    CV_FUNCNAME( "skComputeTextureness" );
    __BEGIN__;

	CvMat  *img, *txtr;
    CvMat  img_stub, txtr_stub;
    int    coi;   
	
    CV_CALL( img = cvGetMat( pImg, &img_stub, &coi ));
    if( coi ) CV_ERROR( CV_BadCOI, "COI is not supported by the function" );

	CvSize sizeImg = cvGetSize( img );
	CvSize sizeTxtr; 

	if (pTxtr != 0)
	{
		CV_CALL( txtr = cvGetMat( pTxtr, &txtr_stub, &coi ));
		sizeTxtr= cvGetSize( txtr );
		if (coi || CV_MAT_TYPE( txtr->type ) == IPL_DEPTH_32F 
			|| sizeTxtr.height != sizeImg.height || sizeTxtr.width != sizeImg.width)
		{
			cvReleaseImage(&pTxtr);
			pTxtr = cvCreateImage(cvGetSize(pImg), IPL_DEPTH_32F , 1);//float type of textureness
		}
	}
	else
	{
		pTxtr = cvCreateImage(cvGetSize(pImg), IPL_DEPTH_32F , 1);//float type of textureness
		CV_CALL( txtr = cvGetMat( pTxtr, &txtr_stub, &coi ));
	}

	int nColorOrNot;
    if( CV_MAT_TYPE( img->type ) == CV_8UC1)
		nColorOrNot = 0;
	else
		nColorOrNot = 1;
	skComputeTextureness(img->data.ptr, txtr->data.ptr,
            sizeImg, img->step, txtr->step, param, nColorOrNot);

    __END__; 
	return 0;
}

///////////////////
// skComputeTextureness()
//	for a given image compute textureness
int	skComputeTextureness(uchar *img,//input image 
						 uchar *txtr,//txtureness image
                         CvSize size,//image size
						 int nImgWdStep,//image wd step
						 int nTxtrWdStep,//disp image wd step
                         GenParam &param,//parameters
						 int nColorOrNot)//1-color, 0-gray
{
	int wd = size.width;
	int ht = size.height;

	int r, c;
	double dTxtr;
	for (r = 0; r < ht; r++)
	{
		for (c = 0; c < wd; c++)
		{
			dTxtr = skComputeTextureness(r, c, img, size, nImgWdStep, param, nColorOrNot);
			((float*)(txtr+r*nTxtrWdStep))[c] = (float)dTxtr;
		}
	}
	return 0;
}

///////////////////
// skComputeTextureness()
//	for a given image compute textureness
double skComputeTextureness(int r0, int c0,//textureness at point (r0,c0)
						 uchar *img,//input image 
                         CvSize size,//image size
						 int nImgWdStep,//image wd step
                         GenParam &param,//parameters
						 int nColorOrNot)//1-color, 0-gray
{
	int nHalfWin = param.pnInt[0];

	int wd = size.width;
	int ht = size.height;

	double dTxtr = 0;
	double dDeriv;
	int dr, dc, r, c;
	int n = 0;
	int i;
	for (dr = -nHalfWin; dr <= nHalfWin; dr++)
	{
		r = r0 + dr;
		if (r < 0 || r >= ht)
			continue;
		for (dc = -nHalfWin; dc <= nHalfWin; dc++)
		{
			c = c0 + dc;
			if (c < 0 || c >= wd-1)
				continue;
			if ( !nColorOrNot )
			{
				dDeriv = (double)((img+r*nImgWdStep)[c+1]) - (img+r*nImgWdStep)[c];
//				dTxtr += dDeriv*dDeriv;
				dTxtr += fabs( dDeriv);
			}
			else
			{
				for (i = 0; i < 3; i++)
				{
					dDeriv = (double)((img+r*nImgWdStep)[(c+1)*3+i]) - (img+r*nImgWdStep)[c*3+i];
//					dTxtr += dDeriv*dDeriv;
					dTxtr += fabs( dDeriv);

				}
			}
			n++;
		}
	}
	if ( n == 0)
		dTxtr = 0;
	else
		dTxtr /= n;
	return dTxtr;
}







































